@@ -12,6 +12,7 @@ from ._common_imports import (
 
 from ._drain_type import DrainType
 from ._completion_type import CompletionType
+from ._history_src import HistorySrc
 
 from .htcondor2_impl import (
     _startd_drain_jobs,
@@ -142,8 +143,9 @@ class Startd():
 
         return _history_query(self._addr,
             str(constraint), projection_string, int(match), since,
-            # HRS_STARTD_JOB_HIST
-            1,
+            # Ad Type Filter
+            "",
+            HistorySrc.StartdJob,
             # GET_HISTORY
             429,
         )
