@@ -5,7 +5,6 @@
 #include "compat_classad.h"
 #include "classad_command_util.h"
 #include "classad_collection.h"
-#include "stat_wrapper.h"
 
 #include "annex.h"
 #include "annex-create.h"
@@ -81,6 +80,10 @@ createOneAnnex( ClassAd * command, Stream * replyStream, ClassAd * reply ) {
 		reply->Assign( "ExpectedDelay", expectedDelay );
 	}
 
+	std::string region;
+	command->LookupString( "Region", region );
+	reply->Assign( "Region", region );
+
 	//
 	// Construct the GAHPs.  We do this before anything else because we
 	// need pointers the GAHPs to hand off to the nonblocking sequence
@@ -154,9 +157,10 @@ createOneAnnex( ClassAd * command, Stream * replyStream, ClassAd * reply ) {
 	}
 
 	if( secretKeyFile != USE_INSTANCE_ROLE_MAGIC_STRING ) {
-		StatWrapper sw( secretKeyFile.c_str() );
-		mode_t mode = sw.GetBuf()->st_mode;
-		if( mode & S_IRWXG || mode & S_IRWXO || getuid() != sw.GetBuf()->st_uid ) {
+		struct stat sw = {};
+		stat( secretKeyFile.c_str(), &sw );
+		mode_t mode = sw.st_mode;
+		if( mode & S_IRWXG || mode & S_IRWXO || getuid() != sw.st_uid ) {
 			std::string errorString;
 			formatstr( errorString, "Secret key file must be accessible only by owner.  Please verify that your user owns the file and that the file permissons are restricted to the owner." );
 			dprintf( D_ALWAYS, "%s\n", errorString.c_str() );
@@ -324,12 +328,6 @@ createOneAnnex( ClassAd * command, Stream * replyStream, ClassAd * reply ) {
 		std::string instanceID, connectivityFunctionARN;
 		command->LookupString( "CollectorInstanceID", instanceID );
 		param( connectivityFunctionARN, "ANNEX_DEFAULT_CONNECTIVITY_FUNCTION_ARN" );
-		CheckConnectivity * cc = new CheckConnectivity(
-			connectivityFunctionARN, instanceID,
-			reply, lambdaGahp, scratchpad,
-			lambdaURL, publicKeyFile, secretKeyFile,
-			commandState, commandID );
-
 		PutRule * pr = new PutRule(
 			reply, eventsGahp, scratchpad,
 			eventsURL, publicKeyFile, secretKeyFile,
@@ -352,9 +350,9 @@ createOneAnnex( ClassAd * command, Stream * replyStream, ClassAd * reply ) {
 		// to restart a rollback, if that becomes necessary.
 		FunctorSequence * fs;
 		if( uf ) {
-			fs = new FunctorSequence( { cc, gf, uf, pr, pt, br }, last, commandState, commandID, scratchpad );
+			fs = new FunctorSequence( { gf, uf, pr, pt, br }, last, commandState, commandID, scratchpad );
 		} else {
-			fs = new FunctorSequence( { cc, gf, pr, pt, br }, last, commandState, commandID, scratchpad );
+			fs = new FunctorSequence( { gf, pr, pt, br }, last, commandState, commandID, scratchpad );
 		}
 
 		// Create a timer for the gahp to fire when it gets a result.  We must
@@ -375,11 +373,13 @@ createOneAnnex( ClassAd * command, Stream * replyStream, ClassAd * reply ) {
 		std::string instanceID, connectivityFunctionARN;
 		command->LookupString( "CollectorInstanceID", instanceID );
 		param( connectivityFunctionARN, "ANNEX_DEFAULT_CONNECTIVITY_FUNCTION_ARN" );
+/*
 		CheckConnectivity * cc = new CheckConnectivity(
 			connectivityFunctionARN, instanceID,
 			reply, lambdaGahp, scratchpad,
 			lambdaURL, publicKeyFile, secretKeyFile,
 			commandState, commandID );
+*/
 
 		OnDemandRequest * odr = new OnDemandRequest( reply, gahp, scratchpad,
 			serviceURL, publicKeyFile, secretKeyFile, commandState,
@@ -389,7 +389,6 @@ createOneAnnex( ClassAd * command, Stream * replyStream, ClassAd * reply ) {
 		command->LookupInteger( "EndOfLease", endOfLease );
 
 		if( (! odr->validateAndStore( command, validationError )) || (! validateLease( endOfLease, validationError )) ) {
-			delete cc;
 			delete odr;
 			goto cleanup;
 		}
@@ -430,9 +429,9 @@ createOneAnnex( ClassAd * command, Stream * replyStream, ClassAd * reply ) {
 
 		FunctorSequence * fs;
 		if( uf ) {
-			fs = new FunctorSequence( { cc, gf, uf, pr, pt, odr }, last, commandState, commandID, scratchpad );
+			fs = new FunctorSequence( { gf, uf, pr, pt, odr }, last, commandState, commandID, scratchpad );
 		} else {
-			fs = new FunctorSequence( { cc, gf, pr, pt, odr }, last, commandState, commandID, scratchpad );
+			fs = new FunctorSequence( { gf, pr, pt, odr }, last, commandState, commandID, scratchpad );
 		}
 
 		int functorSequenceTimer = daemonCore->Register_Timer( 0, TIMER_NEVER,
