@@ -13,7 +13,6 @@
 #include "match_prefix.h"
 #include "daemon.h"
 #include "dc_annexd.h"
-#include "stat_wrapper.h"
 #include "condor_base64.h"
 #include "my_username.h"
 #include "shortfile.h"
@@ -157,7 +156,8 @@ createConfigTarball(	const char * configDir,
 						long unclaimedTimeout,
 						std::string & tarballPath,
 						std::string & tarballError,
-						std::string & instanceID ) {
+						std::string & instanceID,
+						const std::string & requestID ) {
 	char * cwd = get_current_dir_name();
 
 	int rv = chdir( configDir );
@@ -168,6 +168,22 @@ createConfigTarball(	const char * configDir,
 		return false;
 	}
 
+	rv = mkdir( "config.d", 0755 );
+	if( rv != 0 ) {
+		formatstr( tarballError, "unable to make config.d (%d): '%s'",
+			errno, strerror( errno ) );
+		free(cwd);
+		return false;
+	}
+
+	rv = chdir( "config.d" );
+	if( rv != 0 ) {
+		formatstr( tarballError, "unable to change to config.d (%d): '%s'",
+			errno, strerror( errno ) );
+		free(cwd);
+		return false;
+	}
+
 	// Must be readable by the 'condor' user on the instance, but it
 	// will be owned by root.
 	int fd = safe_open_wrapper_follow( "00ec2-dynamic.config",
@@ -180,12 +196,10 @@ createConfigTarball(	const char * configDir,
 	}
 
 
-	std::string passwordFile = "password_file.pl";
-
 	std::string collectorHost;
-	param( collectorHost, "COLLECTOR_HOST" );
+	param( collectorHost, "ANNEX_COLLECTOR" );
 	if( collectorHost.empty() ) {
-		formatstr( tarballError, "COLLECTOR_HOST empty or undefined" );
+		formatstr( tarballError, "ANNEX_COLLECTOR empty or undefined" );
 		free(cwd);
 		close(fd);
 		return false;
@@ -203,10 +217,10 @@ createConfigTarball(	const char * configDir,
 		return false;
 	}
 
-	std::string startExpression = "START = MayUseAWS == TRUE\n";
+	std::string startExpression = "START = MY.AnnexName == TARGET.TargetAnnexName\n";
 	if( owner != NULL ) {
-		formatstr( startExpression,
-			"START = (MayUseAWS == TRUE) && stringListMember( Owner, \"%s\" )\n"
+		formatstr_cat( startExpression,
+			"START = $(START) && stringListMember( Owner, \"%s\" )\n"
 			"\n",
 			owner );
 	}
@@ -223,19 +237,67 @@ createConfigTarball(	const char * configDir,
 		"SEC_DEFAULT_INTEGRITY = REQUIRED\n"
 		"SEC_DEFAULT_ENCRYPTION = REQUIRED\n"
 		"\n"
-		"SEC_PASSWORD_FILE = /etc/condor/config.d/%s\n"
 		"ALLOW_WRITE = condor_pool@*/* $(LOCAL_HOSTS)\n"
 		"\n"
 		"AnnexName = \"%s\"\n"
 		"IsAnnex = TRUE\n"
-		"STARTD_ATTRS = $(STARTD_ATTRS) AnnexName IsAnnex\n"
-		"MASTER_ATTRS = $(MASTER_ATTRS) AnnexName IsAnnex\n"
+		"hpc_annex_request_id = \"%s\"\n"
+		"STARTD_ATTRS = $(STARTD_ATTRS) AnnexName IsAnnex hpc_annex_request_id\n"
+		"MASTER_ATTRS = $(MASTER_ATTRS) AnnexName IsAnnex hpc_annex_request_id.\n"
 		"\n"
 		"STARTD_NOCLAIM_SHUTDOWN = %ld\n"
 		"\n"
 		"%s",
-		collectorHost.c_str(), passwordFile.c_str(),
-		annexName, unclaimedTimeout, startExpression.c_str()
+		collectorHost.c_str(),
+		annexName,
+		requestID.c_str(),
+		unclaimedTimeout,
+		startExpression.c_str()
+	);
+
+	std::string scheddName;
+	CondorQuery q(SCHEDD_AD);
+	ClassAdList l;
+	auto r = q.fetchAds( l, collectorHost.c_str() );
+	if( r != Q_OK ) {
+		formatstr( tarballError, "unable to get schedd's ad" );
+		free(cwd);
+		return false;
+	}
+	if( l.Length() != 1 ) {
+		formatstr( tarballError, "Found more than one scheduler in the AP collector" );
+		free(cwd);
+		return false;
+	}
+	l.Rewind();
+	ClassAd * c = l.Next();
+	if(! c->LookupString( ATTR_NAME, scheddName )) {
+		formatstr( tarballError, "Scheduler ad in AP collector did not contain the scheduler's name." );
+		free(cwd);
+		return false;
+	}
+
+
+	formatstr_cat( contents,
+		"\n"
+		"\n"
+		"COLLECTOR_HOST = <%s>\n"
+		"\n"
+		"ALLOW_ADMINISTRATOR = $(ALLOW_ADMINISTRATOR) $(whoami)@$(hostname)\n"
+		"ALLOW_ADMINISTRATOR = $(ALLOW_ADMINISTRATOR) condor_pool@*\n"
+		"\n"
+		"SEC_DEFAULT_AUTHENTICATION_METHODS = FS IDTOKENS PASSWORD\n"
+		"SEC_PASSWORD_DIRECTORY = $(ETC)/passwords.d\n"
+		"SEC_TOKEN_SYSTEM_DIRECTORY = $(ETC)/tokens.d\n"
+		"SEC_TOKEN_DIRECTORY = $(ETC)/tokens.d\n"
+		"\n"
+		"UPDATE_INTERVAL = 137\n"
+		"\n"
+		"STARTD_DIRECT_ATTACH_SCHEDD_NAME = %s\n"
+		"STARTD_DIRECT_ATTACH_SCHEDD_POOL = %s\n"
+		"STARTD_DIRECT_ATTACH_INTERVAL = 5\n"
+		"UPDATE_INTERVAL = 5\n",
+		collectorHost.c_str(), scheddName.c_str(), collectorHost.c_str()
 	);
 
 	rv = write( fd, contents.c_str(), contents.size() );
@@ -253,31 +315,69 @@ createConfigTarball(	const char * configDir,
 	close( fd );
 
 
-	std::string localPasswordFile;
-	param( localPasswordFile, "SEC_PASSWORD_FILE" );
-	if( localPasswordFile.empty() ) {
-		formatstr( tarballError, "SEC_PASSWORD_FILE empty or undefined" );
-		free(cwd);
-		return false;
-	}
+	// Fetch a token for interacting with the AP collector and for directly
+	// attaching to the local AP.
 
-	fd = open( localPasswordFile.c_str(), O_RDONLY );
-	if( fd == -1 ) {
-		formatstr( tarballError, "Unable to open SEC_PASSWORD_FILE '%s': %s (%d)",
-			localPasswordFile.c_str(), strerror(errno), errno );
+	std::string annexTokenLifetime;
+	param( annexTokenLifetime, "ANNEX_TOKEN_LIFETIME", "7776000" );
+	std::string annexTokenKeyName;
+	param( annexTokenKeyName, "ANNEX_TOKEN_KEY_NAME", "hpcannex-key" );
+	std::string annexTokenDomain;
+	param( annexTokenDomain, "ANNEX_TOKEN_DOMAIN", "annex.osgdev.chtc.io" );
+
+	char * user_name = NULL;
+	if(! pcache()->get_user_name( geteuid(), user_name )) {
+		formatstr( tarballError, "failed to determine user name" );
 		free(cwd);
 		return false;
-	} else {
-		close( fd );
 	}
+	std::string annexUserName = user_name;
+	free(user_name);
 
 	// FIXME: Rewrite without system().
-	std::string cpCommand;
-	formatstr( cpCommand, "cp '%s' '%s'", localPasswordFile.c_str(), passwordFile.c_str() );
-	int status = system( cpCommand.c_str() );
+	rv = chdir( ".." );
+	ASSERT(rv == 0);
+	rv = mkdir( "tokens.d", 0755 );
+	ASSERT(rv == 0);
+	rv = chdir( "tokens.d" );
+	ASSERT(rv == 0);
+
+	std::string ctf;
+	formatstr( ctf,
+		"condor_token_fetch"
+		" -lifetime %s"
+		" -file %s.%s@%s"
+		" -key %s"
+		" -authz READ"
+		" -authz ADVERTISE_STARTD -authz ADVERTISE_MASTER"
+		" -authz WRITE",
+		annexTokenLifetime.c_str(),
+		annexName, annexUserName.c_str(), annexTokenDomain.c_str(),
+		annexTokenKeyName.c_str()
+	);
+	int status = system( ctf.c_str() );
 	if(! (WIFEXITED( status ) && (WEXITSTATUS( status ) == 0))) {
-		formatstr( tarballError, "failed to copy '%s' to '%s'",
-			localPasswordFile.c_str(), passwordFile.c_str() );
+		formatstr( tarballError, "failed to fetch IDTOKEN" );
+		free(cwd);
+		return false;
+	}
+	rv = chdir( ".." );
+	ASSERT(rv == 0);
+
+
+	// Copy ANNEX_PASSWORD_FILE to the annex for remote control.
+	rv = mkdir( "passwords.d", 0755 );
+	ASSERT(rv == 0);
+
+	std::string annexPasswordFile;
+	param( annexPasswordFile,
+		"ANNNEX_PASSWORD_FILE", "~/.condor/annex_password_file"
+	);
+	std::string cp;
+	formatstr( cp, "/bin/cp %s passwords.d/POOL", annexPasswordFile.c_str() );
+	status = system( cp.c_str() );
+	if(! (WIFEXITED( status ) && (WEXITSTATUS( status ) == 0))) {
+		formatstr( tarballError, "failed to copy ANNEX_PASSWORD_FILE (%s)", annexPasswordFile.c_str() );
 		free(cwd);
 		return false;
 	}
@@ -374,9 +474,9 @@ help( const char * argv0 ) {
 		"\t[-idle <idle duration in decimal hours>]\n"
 		"\n"
 		"To start the annex automatically without a yes/no approval prompt:\n"
-		"\t[-yes]"
+		"\t[-yes]\n"
 		"To customize the annex's HTCondor configuration:\n"
-		"\t[-config-dir </full/path/to/config.d>]\n"
+		"\t[-config-dir </full/path/to/etc>]\n"
 		"\n"
 		"To tag the instances (repeat -tag for each tag):\n"
 		"\t[-tag tag-name tag-value]\n"
@@ -466,7 +566,8 @@ void prepareTarballForUpload(	ClassAd & commandArguments,
 								const char * configDir,
 								char * owner, bool ownerSpecified,
 								bool noOwner,
-								long int unclaimedTimeout ) {
+								long int unclaimedTimeout,
+								const std::string & requestID ) {
 	std::string annexName;
 	commandArguments.LookupString( ATTR_ANNEX_NAME, annexName );
 	ASSERT(! annexName.empty());
@@ -515,7 +616,7 @@ void prepareTarballForUpload(	ClassAd & commandArguments,
 	if(! ownerSpecified) { owner = my_username(); }
 	bool createdTarball = createConfigTarball( tempDir, annexName.c_str(),
 		noOwner ? NULL : owner, unclaimedTimeout, tarballPath, tarballError,
-		instanceID );
+		instanceID, requestID );
 	if(! ownerSpecified) { free( owner ); }
 	commandArguments.Assign( "CollectorInstanceID", instanceID );
 
@@ -600,17 +701,16 @@ void getSFRApproval(	ClassAd & commandArguments, const char * sfrConfigFile,
 	}
 
 	std::string response;
-
-	fprintf( stdout,
-		"Will request %ld spot slot%s for %.2f hours.  "
-		"Each instance will terminate after being idle for %.2f hours.\n",
-		count,
-		count == 1 ? "" : "s",
-		leaseDuration / 3600.0,
-		unclaimedTimeout / 3600.0
-	);
-
 	if(! skipYesNoPrompt) {
+		fprintf( stdout,
+			"Will request %ld spot slot%s for %.2f hours.  "
+			"Each instance will terminate after being idle for %.2f hours.\n",
+			count,
+			count == 1 ? "" : "s",
+			leaseDuration / 3600.0,
+			unclaimedTimeout / 3600.0
+		);
+
 		fprintf( stdout, "Is that OK?  (Type 'yes' or 'no'): " );
 		std::getline( std::cin, response );
 	}
@@ -639,15 +739,15 @@ void getODIApproval(	ClassAd & commandArguments,
 						const char * odiInstanceType, bool odiInstanceTypeSpecified,
 						bool leaseDurationSpecified, bool unclaimedTimeoutSpecified,
 						long int count, long int leaseDuration, long int unclaimedTimeout, bool skipYesNoPrompt ) {
-	fprintf( stdout,
-		"Will request %ld %s on-demand instance%s for %.2f hours.  "
-		"Each instance will terminate after being idle for %.2f hours.\n",
-		count, odiInstanceType, count == 1 ? "" : "s",
-		leaseDuration / 3600.0, unclaimedTimeout / 3600.0
-	);
-
 	std::string response;
 	if(! skipYesNoPrompt) {
+		fprintf( stdout,
+			"Will request %ld %s on-demand instance%s for %.2f hours.  "
+			"Each instance will terminate after being idle for %.2f hours.\n",
+			count, odiInstanceType, count == 1 ? "" : "s",
+			leaseDuration / 3600.0, unclaimedTimeout / 3600.0
+		);
+
 		fprintf( stdout, "Is that OK?  (Type 'yes' or 'no'): " );
 		std::getline( std::cin, response );
 	}
@@ -737,6 +837,8 @@ annex_main( int argc, char ** argv ) {
 	std::vector< std::pair< std::string, std::string > > tags;
 	bool skipYesNoPrompt = false;
 
+	std::string requestID;
+
 	enum annex_t {
 		at_none = 0,
 		at_sfr = 1,
@@ -1130,6 +1232,14 @@ annex_main( int argc, char ** argv ) {
 			return 1;
 		} else if( is_dash_arg_prefix( argv[i], "check-setup", 11 ) ) {
 			theCommand = ct_check_setup;
+		} else if( is_dash_arg_prefix( argv[i], "request-id", 7 ) ) {
+			++i;
+			if( i < argc && argv[i] != NULL ) {
+				requestID = argv[i];
+			} else {
+				fprintf( stderr, "%s: -request-id requires an argument.\n", argv[0] );
+				return 1;
+			}
 		} else if( is_dash_arg_prefix( argv[i], "setup", 5 ) ) {
 			theCommand = ct_setup;
 			++i; if( i < argc ) { publicKeyFile = argv[i]; }
@@ -1335,7 +1445,7 @@ annex_main( int argc, char ** argv ) {
 			commandArguments.Assign( "UploadTo", tarballTarget ); }
 
 			// Deliberate fall-through to common code.
-            //@fallthrough@
+			//@fallthrough@
 
 		case ct_update_annex:
 			// Set AnnexType and LeaseFunctionARN.
@@ -1467,8 +1577,10 @@ annex_main( int argc, char ** argv ) {
 					ASSERT( false );
 			}
 
-			prepareTarballForUpload( commandArguments, configDir, owner, ownerSpecified, noOwner, unclaimedTimeout );
+			prepareTarballForUpload( commandArguments, configDir, owner, ownerSpecified, noOwner, unclaimedTimeout, requestID );
 			handleUserData( commandArguments, clUserDataWins, userData, userDataFileName );
+
+			commandArguments.Assign( "Region", region );
 			return create_annex( commandArguments );
 
 		case ct_update_annex:
