@@ -1,7 +1,6 @@
 #!/bin/bash
 
-blahconffile="${GLITE_LOCATION:-/}/etc/blah.config"
-binpath=`grep nqs_binpath $blahconffile|grep -v \#|awk -F"=" '{ print $2}'|sed -e 's/ //g'|sed -e 's/\"//g'`
+. `dirname $0`/blah_load_config.sh
 
 jnr=0
 jc=0
@@ -10,7 +9,7 @@ for job in  $@ ; do
 done
 for  job in  $@ ; do
         requested=`echo $job | sed 's/^.*\///'`
-        ${binpath}/qdel -k $requested >/dev/null 2>&1
+        ${nqs_binpath}qdel -k $requested >/dev/null 2>&1
         if [ "$?" == "0" ] ; then
                 if [ "$jnr" == "1" ]; then
                         echo " 0 No\\ error"
