@@ -415,20 +415,18 @@ def get_slurm_location(program):
     Locate the copy of the slurm bin the blahp configuration wants to use.
     """
     global _slurm_location_cache
-    if _slurm_location_cache is not None:
-        return os.path.join(_slurm_location_cache, program)
+    if _slurm_location_cache is None:
+        try:
+            slurm_bindir = env_expand(config.get('slurm_binpath'))
+        except:
+            slurm_bindir = ""
+        _slurm_location_cache = slurm_bindir
+    return os.path.join(_slurm_location_cache, program)
 
-    slurm_bindir = env_expand(config.get('slurm_binpath'))
-    slurm_bin_location = os.path.join(slurm_bindir, program)
-    if not os.path.exists(slurm_bin_location):
-        raise Exception("Could not find %s in slurm_binpath=%s" % (program, slurm_bindir))
-    _slurm_location_cache = slurm_bindir
-    return slurm_bin_location
-
-job_id_re = re.compile("JobId=([0-9]+) .*")
-exec_host_re = re.compile("\s*BatchHost=([\w\-.]+)")
-status_re = re.compile("\s*JobState=([\w]+) .*")
-exit_status_re = re.compile(".* ExitCode=(-?[0-9]+:[0-9]+)")
+job_id_re = re.compile(r"JobId=([0-9]+) .*")
+exec_host_re = re.compile(r"\s*BatchHost=([\w\-.]+)")
+status_re = re.compile(r"\s*JobState=([\w]+) .*")
+exit_status_re = re.compile(r".* ExitCode=(-?[0-9]+:[0-9]+)")
 status_mapping = {"BOOT_FAIL": 4, "CANCELLED": 3, "COMPLETED": 4, "CONFIGURING": 1, "COMPLETING": 2, "FAILED": 4, "NODE_FAIL": 4, "PENDING": 1, "PREEMPTED": 4, "RUNNING": 2, "SPECIAL_EXIT": 4, "STOPPED": 2, "SUSPENDED": 2, "TIMEOUT": 4}
 
 def parse_squeue(output):
@@ -485,7 +483,7 @@ def fill_cache(cache_location, cluster):
     global launchtime
     launchtime = time.time()
 
-cache_line_re = re.compile("([0-9]+[\.\w\-]+):\s+(.+)")
+cache_line_re = re.compile(r"([0-9]+[\.\w\-]+):\s+(.+)")
 def cache_to_status(jobid, fd):
     reader = csv.reader(fd, delimiter='\t')
     for row in reader:
@@ -545,7 +543,7 @@ def check_cache(jobid, cluster, recurse=True):
             return None
     return cache_to_status(jobid, fd)
 
-job_status_re = re.compile(".*JobStatus=(\d+);.*")
+job_status_re = re.compile(r".*JobStatus=(\d+);.*")
 
 def main():
     initLog()
@@ -566,7 +564,7 @@ def main():
         cluster = jobid_list[1]
 
     global config
-    config = blah.BlahConfigParser(defaults={'slurm_binpath': '/usr/bin'})
+    config = blah.BlahConfigParser()
 
     log("Checking cache for jobid %s" % jobid)
     if cluster != "":
