@@ -29,23 +29,15 @@
 #include "classad/classad_containers.h"
 #include "classad/exprTree.h"
 #include "classad/classad_flat_map.h"
+#include "classad/flat_set.h"
 
 namespace classad {
 
-typedef std::set<std::string, CaseIgnLTStr> References;
-typedef std::set<std::string, CaseIgnSizeLTStr> ReferencesBySize;
+typedef flat_set<std::string, CaseIgnLTStr> References;
+typedef flat_set<std::string, CaseIgnSizeLTStr> ReferencesBySize;
 typedef std::map<const ClassAd*, References> PortReferences;
 
-#if defined( EXPERIMENTAL )
-#include "classad/rectangle.h"
-#endif
-
-#define USE_CLASSAD_FLAT_MAP
-#ifdef USE_CLASSAD_FLAT_MAP
 using AttrList = ClassAdFlatMap;
-#else
-using AttrList = classad_unordered<std::string, ExprTree*, ClassadAttrNameHash, CaseIgnEqStr>;
-#endif
 
 typedef std::set<std::string, CaseIgnLTStr> DirtyAttrList;
 
@@ -55,7 +47,10 @@ void ClassAdLibraryVersion(std::string &version_string);
 // Should parsed expressions be cached and shared between multiple ads.
 // The default is false.
 void ClassAdSetExpressionCaching(bool do_caching);
+void ClassAdSetExpressionCaching(bool do_caching, int min_string_size);
 bool ClassAdGetExpressionCaching();
+bool ClassAdGetExpressionCaching(int & min_string_size);
+extern size_t  _expressionCacheMinStringSize;
 
 // This flag is only meant for use in Condor, which is transitioning
 // from an older version of ClassAds with slightly different evaluation
@@ -108,6 +103,12 @@ class ClassAd : public ExprTree
 		*/
 		bool Insert( const std::string& attrName, ExprTree* expr);   // (ignores cache)
 		bool InsertLiteral(const std::string& attrName, Literal* lit); // (ignores cache)
+		/*
+			Swap() is just like Insert() except it returns the old exprtree rather than
+			deleting it.  If there was no old exprtree, the old_expr is set to nullptr
+			i.e Swap is like Remove followed by Insert.
+		*/
+		bool Swap(const std::string& attrName, ExprTree* expr, ExprTree* & old_expr);
 
 		// insert through cache if cache is enabled, otherwise just parse and insert
 		// parsing of the rhs expression is done use old ClassAds syntax
@@ -118,13 +119,13 @@ class ClassAd : public ExprTree
 		 *  InsertViaCache() is used to do the actual insertion, so the
 		 *    value is parsed into an ExprTree in old ClassAds syntax.
 		 */
-	bool Insert(const char *str);
-	bool Insert(const std::string &str);
+		bool Insert(const char *str);
+		bool Insert(const std::string &str);
 
 		/* Insert an attribute/value into the Classad.
 		 * The value is parsed into an ExprTree using old ClassAds syntax.
 		 */
-	bool AssignExpr(const std::string &name, const char *value);
+		bool AssignExpr(const std::string &name, const char *value);
 
 		/** Inserts an attribute into a nested classAd.  The scope expression is
 		 		evaluated to obtain a nested classad, and the attribute is 
@@ -136,7 +137,7 @@ class ClassAd : public ExprTree
 			@return true if the operation succeeded, false otherwise.
 			@see ExprTree::setParentScope
 		*/
-		bool DeepInsert( ExprTree *scopeExpr, const std::string &attrName, 
+		bool DeepInsert( ExprTree *scopeExpr, const std::string &attrName,
 				ExprTree *expr );
 
 		/** Inserts an attribute into the ClassAd.  The integer value is
@@ -144,16 +145,11 @@ class ClassAd : public ExprTree
 				the classad.
 			@param attrName The name of the attribute.
 			@param value The integer value of the attribute.
-			@param f The multiplicative factor to be attached to value.
-			@see Value::NumberFactor
 		*/
-		bool InsertAttr( const std::string &attrName,int value, 
-				Value::NumberFactor f=Value::NO_FACTOR );
-		bool InsertAttr( const std::string &attrName,long value, 
-				Value::NumberFactor f=Value::NO_FACTOR );
-		bool InsertAttr( const std::string &attrName,long long value, 
-				Value::NumberFactor f );
-		bool InsertAttr( const std::string &attrName,long long value );
+		bool InsertAttr( const std::string &attrName, int value );
+		bool InsertAttr( const std::string &attrName, long value );
+		bool InsertAttr( const std::string &attrName, long long value );
+
 		bool Assign(const std::string &name, int value)
 		{ return InsertAttr(name, value); }
 		bool Assign(const std::string &name, unsigned int value)
@@ -167,7 +163,7 @@ class ClassAd : public ExprTree
 		bool Assign(const std::string &name, unsigned long long value)
 		{ return InsertAttr(name, (long long)value); }
 
-		/** Inserts an attribute into a nested classad.  The scope expression 
+		/** Inserts an attribute into a nested classad.  The scope expression
 		 		is evaluated to obtain a nested classad, and the attribute is
 		        inserted into this subclassad.  The integer value is
 				converted into a Literal expression, and then inserted into
@@ -175,28 +171,22 @@ class ClassAd : public ExprTree
 			@param scopeExpr The scope expression.
 			@param attrName The name of the attribute.
 			@param value The integer value of the attribute.
-			@param f The multiplicative factor to be attached to value.
-			@see Value::NumberFactor
 		*/
 		bool DeepInsertAttr( ExprTree *scopeExpr, const std::string &attrName,
-				int value, Value::NumberFactor f=Value::NO_FACTOR );
+				int value );
 		bool DeepInsertAttr( ExprTree *scopeExpr, const std::string &attrName,
-				long value, Value::NumberFactor f=Value::NO_FACTOR );
+				long value );
 		bool DeepInsertAttr( ExprTree *scopeExpr, const std::string &attrName,
-				long long value, Value::NumberFactor f=Value::NO_FACTOR );
+				long long value );
 
 		/** Inserts an attribute into the ClassAd.  The real value is
 				converted into a Literal expression, and then inserted into
 				the classad.
 			@param attrName The name of the attribute.
 			@param value The real value of the attribute.
-			@param f The multiplicative factor to be attached to value.
-			@see Value::NumberFactor
             @return true on success, false otherwise
 		*/
-		bool InsertAttr( const std::string &attrName,double value, 
-				Value::NumberFactor f);
-		bool InsertAttr( const std::string &attrName,double value);
+		bool InsertAttr( const std::string &attrName, double value);
 		bool Assign(const std::string &name, float value)
 		{ return InsertAttr(name, (double)value); }
 		bool Assign(const std::string &name, double value)
@@ -211,11 +201,10 @@ class ClassAd : public ExprTree
 			@param attrName The name of the attribute.
 			@param value The string attribute
             @param f A multipler for the number.
-			@see Value::NumberFactor
             @return true on success, false otherwise
 		*/
 		bool DeepInsertAttr( ExprTree *scopeExpr, const std::string &attrName,
-				double value, Value::NumberFactor f=Value::NO_FACTOR);
+				double value);
 
 		/** Inserts an attribute into the ClassAd.  The boolean value is
 				converted into a Literal expression, and then inserted into
@@ -282,10 +271,16 @@ class ClassAd : public ExprTree
 			@param attrName The name of the attribute.
 			@param value The string attribute
 		*/
-		bool DeepInsertAttr( ExprTree *scopeExpr, const std::string &attrName, 
+		bool DeepInsertAttr( ExprTree *scopeExpr, const std::string &attrName,
 				const std::string &value );
 		//@}
 
+
+		bool InsertAttr( const std::string & attrName, ExprTree * invalid ) = delete;
+		bool Assign( const std::string & name, ExprTree * invalid ) = delete;
+		bool DeepInsertAttr( ExprTree * scopeExpr, const std::string &attrName, ExprTree * invalid ) = delete;
+
+
 		/**@name Lookup Methods */
 		//@{
 		/** Finds the expression bound to an attribute name.  The lookup only
@@ -373,7 +368,7 @@ class ClassAd : public ExprTree
 				successfully removed, false otherwise.
 		*/
 		bool DeepDelete( ExprTree *scopeExpr, const std::string &attrName );
-	
+
 		/** Similar to Delete, but the expression is returned rather than 
 		  		deleted from the classad.
 			@param attrName The name of the attribute to be extricated.
@@ -381,6 +376,7 @@ class ClassAd : public ExprTree
 				the attribute could not be found.
 			@see Delete
 		*/
+		[[nodiscard]]
 		ExprTree *Remove( const std::string &attrName );
 
 		/** Similar to DeepDelete, but the expression is returned rather than 
@@ -592,13 +588,11 @@ class ClassAd : public ExprTree
         /** Return the number of attributes at the root level of this ClassAd.
          */
         int size(void) const { return (int)attrList.size(); }
+		bool empty() const { return attrList.empty(); }
 		//@}
 
 		void rehash(size_t s) { attrList.rehash(s);}
 		iterator erase(iterator i) { 
-#ifndef USE_CLASSAD_FLAT_MAP
-			delete i->second;
-#endif
 			return attrList.erase(i);
 		}
 		/** Deconstructor to get the components of a classad
@@ -631,20 +625,42 @@ class ClassAd : public ExprTree
          */
 		virtual ExprTree* Copy( ) const;
 
-        /** Make a deep copy of the ClassAd, via the == operator. 
+        /** Make a deep copy of the ClassAd, via the = operator.
          */
 		ClassAd &operator=(const ClassAd &rhs);
 
 		ClassAd &operator=(ClassAd &&rhs)  noexcept {
-			this->do_dirty_tracking = rhs.do_dirty_tracking;
-			this->chained_parent_ad = rhs.chained_parent_ad;
+			// In the order listed in the header, so that we don't miss any.
 			this->alternateScope = rhs.alternateScope;
 
-			this->dirtyAttrList = std::move(rhs.dirtyAttrList);
 			this->attrList = std::move(rhs.attrList);
+			for( auto & entry : attrList ) {
+				// I wonder what other invariants were forgotten?
+				entry.second->SetParentScope(this);
+			}
+			this->dirtyAttrList = std::move(rhs.dirtyAttrList);
+
+			this->do_dirty_tracking = rhs.do_dirty_tracking;
+			this->chained_parent_ad = rhs.chained_parent_ad;
+			this->parentScope = rhs.parentScope;
 
 			return *this;
 		}
+
+		ClassAd(ClassAd &&rhs) : 
+   				alternateScope(rhs.alternateScope),
+				attrList(std::move(rhs.attrList)),
+				dirtyAttrList(std::move(rhs.dirtyAttrList)),
+				do_dirty_tracking(rhs.do_dirty_tracking),
+				chained_parent_ad(rhs.chained_parent_ad),
+				parentScope(rhs.parentScope)
+		{
+				for(auto &entry: attrList ) {
+					// I wonder what other invariants were forgotten?
+					entry.second->SetParentScope(this);
+			}
+		}
+
         /** Fill in this ClassAd with the contents of the other ClassAd.
          *  This ClassAd is cleared of its contents before the copy happens.
          *  @return true if the copy succeeded, false otherwise.
@@ -711,11 +727,6 @@ class ClassAd : public ExprTree
          */
         bool GetInternalReferences( const ExprTree *tree, References &refs, bool fullNames) const;
 
-#if defined( EXPERIMENTAL )
-		bool AddRectangle( const ExprTree *tree, Rectangles &r, 
-					const std::string &allowed, const References &imported );
-#endif
-
 		/**@name Chaining functions */
         //@{
         /** Chain this ad to the parent ad.
@@ -840,10 +851,6 @@ class ClassAd : public ExprTree
 
         bool _GetInternalReferences(const ExprTree *expr, const ClassAd *ad,
             EvalState &state, References& refs, bool fullNames) const;
-#if defined( EXPERIMENTAL )
-		bool _MakeRectangles(const ExprTree*,const std::string&,Rectangles&, bool);
-		bool _CheckRef( ExprTree *, const std::string & );
-#endif
 
 		ClassAd *_GetDeepScope( const std::string& ) const;
 		ClassAd *_GetDeepScope( ExprTree * ) const;
