@@ -57,7 +57,6 @@ void ClassAdCollection::
 Setup(bool cacheOn)
 {
 	Cache = cacheOn;
-	test_checkpoint=0;
 	// create "root" view
 	viewTree.SetViewName( "root" );
 	RegisterView( "root", &viewTree );
@@ -142,11 +141,10 @@ InitializeFromLog( const string &logfile, const string storagefile, const string
 		CondorErrMsg = "";
 		return( false );
 	}
-	Value				val;
 	vector<ExprTree*>	vec;
 	ExprTree * pExpr=0;
 	if( !ad->InsertAttr( ATTR_REQUIREMENTS, true ) 						||
-			!ad->Insert( ATTR_RANK, (pExpr=Literal::MakeLiteral( val )) )		||
+			!ad->Insert( ATTR_RANK, (pExpr=Literal::MakeUndefined()) )		||
 			!ad->Insert( ATTR_PARTITION_EXPRS, (pExpr=ExprList::MakeExprList( vec )) )||
 			!viewTree.SetViewInfo( this, ad ) ) {
 		CondorErrMsg += "; failed to initialize from log";
@@ -550,7 +548,7 @@ PlayViewOp( int opType, ClassAd *logRec )
 				return( false );
 			}
 			viewInfo->Delete( "OpType" );
-			viewInfo->Remove( "Representative" );
+			std::ignore = viewInfo->Remove( "Representative" );
             return( parentView->InsertPartitionedView( this, viewInfo, rep ) );
         }
 
@@ -1121,7 +1119,7 @@ AddClassAd_NoTransaction(const string &key, ClassAd *newAd)
                 reinsert_old_ad = true;
                 success = false;
             }
-            rec->Remove(ATTR_AD);
+			std::ignore = rec->Remove(ATTR_AD);
             delete rec;
         }
     }
@@ -1632,12 +1630,12 @@ LogState( FILE *fp )
 		  
 		  if(!WriteLogEntry(fp, &logRec, true)) {
 			  CondorErrMsg += "; failed to log ad, could not log state";
-			  logRec.Remove( "Ad" );
+			  std::ignore = logRec.Remove( "Ad" );
 			  return( false );
 		  }
 		  buff = "";            
 		  local_unparser.Unparse(buff,&logRec);
-		  logRec.Remove( "Ad" );
+		  std::ignore = logRec.Remove( "Ad" );
 	  }
 	}
 	if( fsync( fileno( fp ) ) < 0 ) {
@@ -1814,7 +1812,7 @@ WriteCheckPoint(){
 	  string buffer;
 	  unparser.Unparse( buffer, &cla );
 	  buffer += "\n";
-	  int result = write(fd_check,(void*)(buffer.c_str()),(unsigned int)buffer.size());
+	  int result = write(fd_check,buffer.c_str(),(unsigned int)buffer.size());
 	  if (result < 0) {
 			CondorErrMsg = "failed to write to checkpoint file " + CheckFileName + " errno=" + std::to_string(errno);
 			fsync(fd_check);
@@ -1977,9 +1975,8 @@ TruncateStorageFile(){
 
 bool ClassAdCollection::
 dump_collection(){
-         ClassAdTable::iterator	itr;
-	 for (itr= classadTable.begin();itr!=classadTable.end();itr++){
-		 cout << "dump_collection key= " << itr->first << std::endl;
+	 for (const auto& [key, value] : classadTable){
+		 cout << "dump_collection key= " << key << std::endl;
 	 }
 	 return true;
 }
