@@ -102,14 +102,6 @@ FunctionCall( )
 		functionTable["anycompare"  ] = listCompare;
 		functionTable["allcompare"  ] = listCompare;
 
-			// basic apply-like functions
-		/*
-		functionTable["sumfrom"		sumAvgFrom );
-		functionTable["avgfrom"		sumAvgFrom );
-		functionTable["maxfrom"		boundFrom );
-		functionTable["minfrom"		boundFrom );
-		*/
-
 			// time management
 		functionTable["time"        ] = epochTime;
 		functionTable["currenttime"	] =	currentTime;
@@ -194,13 +186,13 @@ FunctionCall( )
 FunctionCall::
 ~FunctionCall ()
 {
-	for( ArgumentList::iterator i=arguments.begin(); i!=arguments.end(); i++ ) {
-		delete (*i);
+	for( auto arg : arguments ) {
+		delete arg;
 	}
 }
 
 FunctionCall::
-FunctionCall(FunctionCall &functioncall)
+FunctionCall(FunctionCall &functioncall) : ExprTree()
 {
     CopyFrom(functioncall);
     return;
@@ -242,10 +234,8 @@ CopyFrom(const FunctionCall &functioncall)
     functionName = functioncall.functionName;
 	function     = functioncall.function;
 
-	for (ArgumentList::const_iterator i = functioncall.arguments.begin(); 
-         i != functioncall.arguments.end();
-         i++ ) {
-		newArg = (*i)->Copy( );
+	for (const auto arg : functioncall.arguments) {
+		newArg = arg->Copy( );
 		if( newArg ) {
 			arguments.push_back( newArg );
 		} else {
@@ -413,8 +403,8 @@ _SetParentScope( const ClassAd* parent )
 {
 	parentScope = parent;
 
-	for( ArgumentList::iterator i=arguments.begin(); i!=arguments.end(); i++ ) {
-		(*i)->SetParentScope( parent );
+	for( auto arg : arguments ) {
+		arg->SetParentScope( parent );
 	}
 }
 	
@@ -445,9 +435,7 @@ MakeFunctionCall( const string &str, vector<ExprTree*> &args )
 
 	fc->functionName = str;
 
-	for( ArgumentList::iterator i = args.begin(); i != args.end( ); i++ ) {
-		fc->arguments.push_back( *i );
-	}
+	fc->arguments.insert(fc->arguments.end(), args.begin(), args.end());
 	return( fc );
 }
 
@@ -456,10 +444,7 @@ void FunctionCall::
 GetComponents( string &fn, vector<ExprTree*> &args ) const
 {
 	fn = functionName;
-	for( ArgumentList::const_iterator i = arguments.begin(); 
-			i != arguments.end( ); i++ ) {
-		args.push_back( *i );
-	}
+	args.insert(args.end(), arguments.begin(), arguments.end());
 }
 
 
@@ -495,9 +480,8 @@ _Evaluate( EvalState &state, Value &value, ExprTree *& tree ) const
 
 	tmpSig->functionName = functionName;
 	rval = true;
-	for(ArgumentList::const_iterator i=arguments.begin();i!=arguments.end();
-			i++) {
-		rval = (*i)->Evaluate( state, tmpVal, argSig );
+	for(const auto arg : arguments) {
+		rval = arg->Evaluate( state, tmpVal, argSig );
 		if( rval ) tmpSig->arguments.push_back( argSig );
 	}
 	tree = tmpSig;
@@ -533,9 +517,8 @@ _Flatten( EvalState &state, Value &value, ExprTree*&tree, int* ) const
 	newCall->function = function;
 
 	// flatten the arguments
-	for(ArgumentList::const_iterator i=arguments.begin();i!=arguments.end();
-			i++ ) {
-		if( (*i)->Flatten( state, argValue, argTree ) ) {
+	for(const auto arg : arguments) {
+		if( arg->Flatten( state, argValue, argTree ) ) {
 			if( argTree ) {
 				newCall->arguments.push_back( argTree );
 				fold = false;
@@ -689,139 +672,6 @@ testMember(const char *name,const ArgumentList &argList, EvalState &state,
     return true;
 }
 
-/*
-bool FunctionCall::
-sumAvgFrom (char *name, const ArgumentList &argList, EvalState &state, Value &val)
-{
-	Value		caVal, listVal;
-	ExprTree	*ca;
-	Value		tmp, result;
-	ExprList	*el;
-	ClassAd		*ad;
-	bool		first = true;
-	int			len;
-	bool		onlySum = ( strcasecmp( "sumfrom", name ) == 0 );
-
-	// need two arguments
-	if( argList.getlast() != 1 ) {
-		val.SetErrorValue();
-		return true;
-	}
-
-	// first argument must Evaluate to a list
-	if( !argList[0]->Evaluate( state, listVal ) ) {
-		val.SetErrorValue( );
-		return( false );
-	} else if( listVal.IsUndefinedValue() ) {
-		val.SetUndefinedValue( );
-		return( true );
-	} else if( !listVal.IsListValue( el ) ) {
-		val.SetErrorValue();
-		return( true );
-	}
-
-	el->Rewind();
-	result.SetUndefinedValue();
-	while( ( ca = el->Next() ) ) {
-		if( !ca->Evaluate( state, caVal ) ) {
-			val.SetErrorValue( );
-			return( false );
-		} else if( !caVal.IsClassAdValue( ad ) ) {
-			val.SetErrorValue();
-			return( true );
-		} else if( !ad->EvaluateExpr( argList[1], tmp ) ) {
-			val.SetErrorValue( );
-			return( false );
-		}
-		if( first ) {
-			result.CopyFrom( tmp );
-			first = false;
-		} else {
-			Operation::Operate( ADDITION_OP, result, tmp, result );
-		}
-		tmp.Clear();
-	}
-
-		// if the sumFrom( ) function was called, don't need to find average
-	if( onlySum ) {
-		val.CopyFrom( result );
-		return( true );
-	}
-
-
-	len = el->Number();
-	if( len > 0 ) {
-		tmp.SetRealValue( len );
-		Operation::Operate( DIVISION_OP, result, tmp, result );
-	} else {
-		val.SetUndefinedValue();
-	}
-
-	val.CopyFrom( result );
-	return true;
-}
-
-
-bool FunctionCall::
-boundFrom (char *fn, const ArgumentList &argList, EvalState &state, Value &val)
-{
-	Value		caVal, listVal, cmp;
-	ExprTree	*ca;
-	Value		tmp, result;
-	ExprList	*el;
-	ClassAd		*ad;
-	bool		first = true, b=false, min;
-
-	// need two arguments
-	if( argList.getlast() != 1 ) {
-		val.SetErrorValue();
-		return( true );
-	}
-
-	// first argument must Evaluate to a list
-	if( !argList[0]->Evaluate( state, listVal ) ) {
-		val.SetErrorValue( );
-		return( false );
-	} else if( listVal.IsUndefinedValue( ) ) {
-		val.SetUndefinedValue( );
-		return( true );
-	} else if( !listVal.IsListValue( el ) ) {
-		val.SetErrorValue();
-		return( true );
-	}
-
-	// fn is either "min..." or "max..."
-	min = ( tolower( fn[1] ) == 'i' );
-
-	el->Rewind();
-	result.SetUndefinedValue();
-	while( ( ca = el->Next() ) ) {
-		if( !ca->Evaluate( state, caVal ) ) {
-			val.SetErrorValue( );
-			return false;
-		} else if( !caVal.IsClassAdValue( ad ) ) {
-			val.SetErrorValue();
-			return( true );
-		} else if( !ad->EvaluateExpr( argList[1], tmp ) ) {
-			val.SetErrorValue( );
-			return( true );
-		}
-		if( first ) {
-			result.CopyFrom( tmp );
-			first = false;
-		} else {
-			Operation::Operate(min?LESS_THAN_OP:GREATER_THAN_OP,tmp,result,cmp);
-			if( cmp.IsBooleanValue( b ) && b ) {
-				result.CopyFrom( tmp );
-			}
-		}
-	}
-
-	val.CopyFrom( result );
-	return true;
-}
-*/
-
 // The size of a list, ClassAd, etc. 
 bool FunctionCall::
 size(const char *, const ArgumentList &argList, 
@@ -1456,8 +1306,7 @@ strCat( const char* name, const ArgumentList &argListIn, EvalState &state,
 				if (num_args > 1) {
 					argTemp.insert(argTemp.begin(), argListIn[0]);
 				} else {
-					Value sep; sep.SetStringValue(""); // cons up a default separator as the first argument
-					sepLit = Literal::MakeLiteral(sep);
+					sepLit = Literal::MakeString("");
 					argTemp.insert(argTemp.begin(), sepLit);
 				}
 				// now use this as the input arguments
@@ -1656,7 +1505,7 @@ subString( const char*, const ArgumentList &argList, EvalState &state,
 }
 
 bool FunctionCall::
-versionInRange( const char * name, const ArgumentList & argList,
+versionInRange( const char * /*name*/, const ArgumentList & argList,
   EvalState & state, Value & result ) {
 	if( argList.size() != 3 ) {
 		result.SetErrorValue();
@@ -1936,9 +1785,9 @@ bool FunctionCall::hasRefs(const char*, const ArgumentList& argList, EvalState&
 	if (expr && state.curAd) {
 		classad::References attrs;
 		if (ClassAd::_GetExternalReferences(expr, state.curAd, state, attrs, true)) {
-			for (auto it = attrs.begin(); it != attrs.end(); ++it) {
-				const char * attr = it->c_str();
-				size_t len = it->length();
+			for (const auto &attr_ref : attrs) {
+				const char * attr = attr_ref.c_str();
+				size_t len = attr_ref.length();
 				if (0 == strncasecmp(attr, "target.", 7)) {
 					attr += 7;
 					len -= 7;
@@ -2517,11 +2366,8 @@ eval( const char* /* name */,const ArgumentList &argList,EvalState &state,
 	}
 
 	ClassAdParser parser;
-	ExprTree *expr = NULL;
-	if( !parser.ParseExpression( s.c_str(), expr, true ) || !expr ) {
-		if( expr ) {
-			delete expr;
-		}
+	ExprTree *expr = parser.ParseExpression( s.c_str(), true );
+	if( !expr ) {
 		result.SetErrorValue();
 		return true;
 	}
@@ -2594,7 +2440,7 @@ interval( const char* /* name */,const ArgumentList &argList,EvalState &state,
 }
 
 bool FunctionCall::
-debug( const char* name,const ArgumentList &argList,EvalState &state,
+debug( const char* /*name*/,const ArgumentList &argList,EvalState &state,
 	Value &result )
 {
 	Value	arg;
@@ -2999,7 +2845,7 @@ static bool regexp_helper(
 			}
 
 #if PCRE2_MAJOR == 10 && PCRE2_MINOR < 43
-			pcre2_substring_list_free((PCRE2_SPTR *)groups_pcre2);
+			pcre2_substring_list_free(const_cast<PCRE2_SPTR *>(groups_pcre2));
 #else
 			pcre2_substring_list_free(groups_pcre2);
 #endif
@@ -3035,15 +2881,15 @@ static bool
 doSplitTime(const Value &time, ClassAd * &splitClassAd)
 {
     bool             did_conversion;
-    int              integer;
+    time_t           integral;
     double           real;
     abstime_t        asecs;
     double           rsecs;
     const ClassAd    *classad;
 
     did_conversion = true;
-    if (time.IsIntegerValue(integer)) {
-        asecs.secs = integer;
+    if (time.IsIntegerValue(integral)) {
+        asecs.secs = integral;
         asecs.offset = timezone_offset( asecs.secs, false );
         absTimeToClassAd(asecs, splitClassAd);
     } else if (time.IsRealValue(real)) {
