@@ -42,38 +42,39 @@ const char *ClassAdUnParser::opString[] =
 {
     "",     // no-op
 
-    " < ",  // comparisons
-    " <= ",
-    " != ",
-    " == ",
-    " >= ",
-    " > ",
-    " is ",
-    " isnt ",
+    " < ",  // comparisons LESS_THAN_OP
+    " <= ", // LESS_OR_EQUAL_OP
+	" != ", // NOT_EQUAL_OP
+	" == ", // EQUAL_OP
+	" >= ", // GREATER_OR_EQUAL_OP
+	" > ",  // GREATER_THAN_OP
+    " is ", // IS_OP
+    " isnt ", // ISNT_OP
 
-    " +",   // arithmetic
-    " -",
-    " + ",
-    " - ",
-    " * ",
-    " / ",
-    " % ",
+    " +",   // arithmetic UNARY_PLUS_OP
+    " -",   // UNARY_MINUS_OP
+	" + ",  // ADDITION_OP
+	" - ",  // SUBTRACTION_OP
+	" * ",  // MULTIPLICATION_OP
+	" / ",  // DIVISION_OP
+	" % ",  // MODULUS_OP
 
-    " !",   // logical
-    " || ",
-    " && ",
+    " !",   // logical LOGICAL_NOT_OP
+    " || ", // LOGICAL_OR_OP
+    " && ", // LOGICAL_AND_OP
 
-    " ~",   // bitwise
-    " | ",
-    " ^ ",
-    " & ",
-    " << ",
-    " >> ",
-    " >>> ",
+    " ~",   // BITWISE_NOT_OP = __BITWISE_START__,bitwise
+	" | ",  // BITWISE_OR_OP,          // (bitwise)
+	" ^ ",  // BITWISE_XOR_OP,         // (bitwise)
+	" & ",  // BITWISE_AND_OP,         // (bitwise)
+	" << ", // LEFT_SHIFT_OP,          // (bitwise)
+	" >> ", // RIGHT_SHIFT_OP,         // (bitwise)
+	" >>> ",// URIGHT_SHIFT_OP,        // (bitwise)
 
-    " () ", // misc mixfix --- no "single token" representation
-    " [] ",
-    " ?: "
+    " () ", // PARENTHESES_OP misc mixfix --- no "single token" representation
+    " [] ", // SUBSCRIPT_OP
+    " ?: ", // ELVIS_OP     A?:B
+	" ?: ", // TERNARY_OP   A?B:C
 };
 
 void ClassAdUnParser::
@@ -320,7 +321,7 @@ Unparse( string &buffer, const ExprTree *tree )
 			ExprTree *expr;
 			string	ref;
 			bool	absolute;
-			((AttributeReference*)tree)->GetComponents( expr, ref, absolute );
+			((const AttributeReference*)tree)->GetComponents( expr, ref, absolute );
 			UnparseAux( buffer, expr, ref, absolute );
 			return;
 		}
@@ -328,7 +329,7 @@ Unparse( string &buffer, const ExprTree *tree )
 		case ExprTree::OP_NODE: {
 			Operation::OpKind	op;
 			ExprTree			*t1, *t2, *t3;
-			((Operation*)tree)->GetComponents( op, t1, t2, t3 );
+			((const Operation*)tree)->GetComponents( op, t1, t2, t3 );
 			UnparseAux( buffer, op, t1, t2, t3 );
 			return;
 		}
@@ -336,21 +337,21 @@ Unparse( string &buffer, const ExprTree *tree )
 		case ExprTree::FN_CALL_NODE: {
 			string					fnName;
 			vector<ExprTree*> args;
-			((FunctionCall*)tree)->GetComponents( fnName, args );
+			((const FunctionCall*)tree)->GetComponents( fnName, args );
 			UnparseAux( buffer, fnName, args );
 			return;
 		}
 
 		case ExprTree::CLASSAD_NODE: {
 			vector< pair<string, ExprTree*> > attrs;
-			((ClassAd*)tree)->GetComponents( attrs );
+			((const ClassAd*)tree)->GetComponents( attrs );
 			UnparseAux( buffer, attrs );
 			return;
 		}
 
 		case ExprTree::EXPR_LIST_NODE: {
 			vector<ExprTree*> exprs;
-			((ExprList*)tree)->GetComponents( exprs );
+			((const ExprList*)tree)->GetComponents( exprs );
 			UnparseAux( buffer, exprs );
 			return;
 		}
@@ -358,7 +359,7 @@ Unparse( string &buffer, const ExprTree *tree )
 		case ExprTree::EXPR_ENVELOPE: {
 			{
 				// recurse b/c we indirect for this element.
-				Unparse( buffer, ((CachedExprEnvelope*)tree)->get());
+				Unparse( buffer, ((const CachedExprEnvelope*)tree)->get());
 			}
 			return;
 		}
