@@ -30,11 +30,6 @@
 #include <charconv>
 #include <system_error>
 
-using std::string;
-using std::vector;
-using std::pair;
-
-
 namespace classad {
 
 ClassAdXMLParser::
@@ -51,14 +46,14 @@ ClassAdXMLParser::
 }
 
 bool ClassAdXMLParser::
-ParseClassAd( const string &buffer, ClassAd &classad)
+ParseClassAd( const std::string &buffer, ClassAd &classad)
 {
 	int offset = 0;
 	return ParseClassAd(buffer, classad, offset);
 }
 
 bool ClassAdXMLParser::
-ParseClassAd( const string &buffer, ClassAd &classad, int &offset)
+ParseClassAd( const std::string &buffer, ClassAd &classad, int &offset)
 {
 	ClassAd          *classad_out;
 	StringLexerSource lexer_source(&buffer, offset);
@@ -81,8 +76,20 @@ ParseClassAd( FILE *file, ClassAd &ad )
 	return classad_out != NULL;
 }
 
+bool ClassAdXMLParser::
+ParseClassAd( LexerSource *lexer_source, ClassAd &ad)
+{
+	lexer.SetLexerSource(lexer_source);
+	bool success = ParseClassAd(&ad);
+	if (! success) {
+		ad.Clear();
+	}
+	return success;
+}
+
+
 ClassAd *ClassAdXMLParser::
-ParseClassAd( const string &buffer)
+ParseClassAd( const std::string &buffer)
 {
 	int offset = 0;
 
@@ -90,7 +97,7 @@ ParseClassAd( const string &buffer)
 }
 
 ClassAd *ClassAdXMLParser::
-ParseClassAd( const string &buffer, int &offset)
+ParseClassAd( const std::string &buffer, int &offset)
 {
 	ClassAd          *classad;
 	StringLexerSource lexer_source(&buffer, offset);
@@ -153,7 +160,7 @@ ParseClassAd(ClassAd *classad_in)
 				  delete local_ad;
 			      return NULL;
 			    } else if( token.tag_type == XMLLexer::tagType_Start) {
-					string attribute_name;
+					std::string attribute_name;
 					ExprTree *tree;
 					
 					tree = ParseAttribute(attribute_name);
@@ -195,7 +202,7 @@ ParseClassAd(ClassAd *classad_in)
 
 ExprTree *ClassAdXMLParser::
 ParseAttribute(
-	string    &attribute_name)
+	std::string    &attribute_name)
 {
 	ExprTree         *tree;
 	XMLLexer::Token  token;
@@ -267,7 +274,7 @@ ParseList(void)
 	ExprTree           *tree;
 	ExprTree           *subtree;
 	XMLLexer::Token    token;
-	vector<ExprTree*>  expressions;
+	std::vector<ExprTree*>  expressions;
 
 	tree = NULL;
 	lexer.ConsumeToken(&token);
@@ -304,19 +311,18 @@ ParseNumberOrString(XMLLexer::TagID tag_id)
 	have_token = lexer.ConsumeToken(&token);
 	assert(have_token && token.tag_id == tag_id);
 
-	// Get text of number or string
+	// Get text of number or std::string
 	have_token = lexer.PeekToken(&token);
 
 	if (have_token && token.token_type == XMLLexer::tokenType_Text) {
 		lexer.ConsumeToken(&token);
-		Value value;
 		if (tag_id == XMLLexer::tagID_Integer) {
 			int64_t number;
 			auto [pend, ec] = std::from_chars(token.text.data(), token.text.data() + token.text.size(), number, 10);
 			if ( ec != std::errc()) {
-				value.SetErrorValue();
+				tree = Literal::MakeError();
 			} else {
-				value.SetIntegerValue(number);
+				tree = Literal::MakeInteger(number);
 			}
 		}
 		else if (tag_id == XMLLexer::tagID_Real) {
@@ -325,29 +331,25 @@ ParseNumberOrString(XMLLexer::TagID tag_id)
 			const char * pnum = token.text.c_str();
 			real = strtod(pnum, &pend);
 			if (pend == pnum) {
-				value.SetErrorValue();
+				tree = Literal::MakeError();
 			} else {
-				value.SetRealValue(real);
+				tree = Literal::MakeReal(real);
 			}
 		}
-		else {        // its a string
+		else {        // its a std::string
 			bool validStr = true;
 			convert_escapes(token.text, validStr );
-			if(!validStr) {  // invalid string because it had /0 escape sequence
+			if(!validStr) {  // invalid std::string because it had /0 escape sequence
 				return NULL;
 			} else {
-				value.SetStringValue(token.text);
+				tree = Literal::MakeString(token.text);
 			}
 		}
 	
-		  tree = Literal::MakeLiteral(value);
-		
 	} else if (tag_id == XMLLexer::tagID_String) {
 		// We were expecting text and got none, so we had
-		// the empty string, which was skipped by the lexer.
-		Value  value;
-		value.SetStringValue("");
-		tree = Literal::MakeLiteral(value);
+		// the empty std::string, which was skipped by the lexer.
+		tree = Literal::MakeString("");
 	}
 
     SwallowEndTag(tag_id);
@@ -365,15 +367,15 @@ ParseBool(void)
 	lexer.ConsumeToken(&token);
 	assert(token.tag_id == XMLLexer::tagID_Bool);
 
-	Value  value;
-	string truth_value = token.attributes["v"];
+	bool   value;
+	std::string truth_value = token.attributes["v"];
 
 	if (truth_value == "t" || truth_value == "T") {
-		value.SetBooleanValue(true);
+		value = true;
 	} else {
-		value.SetBooleanValue(false);
+		value = false;
 	}
-	tree = Literal::MakeLiteral(value);
+	tree = Literal::MakeBool(value);
 
 	if (token.tag_type == XMLLexer::tagType_Start) {
         SwallowEndTag(XMLLexer::tagID_Bool);
@@ -392,13 +394,11 @@ ParseUndefinedOrError(XMLLexer::TagID tag_id)
 	lexer.ConsumeToken(&token);
 	assert(token.tag_id == tag_id);
 
-	Value value;
 	if (tag_id == XMLLexer::tagID_Undefined) {
-		value.SetUndefinedValue();
+		tree = Literal::MakeUndefined();
 	} else {
-		value.SetErrorValue();
+		tree = Literal::MakeError();
 	}
-	tree = Literal::MakeLiteral(value);
 
 	if (token.tag_type == XMLLexer::tagType_Start) {
         SwallowEndTag(tag_id);
