@@ -647,7 +647,7 @@ CollectorDaemon::pending_query_entry_t *  CollectorDaemon::make_query_entry(
 			attr = tag; attr += ATTR_REQUIREMENTS;
 			if (query->Lookup(attr)) {
 				bool tag_skip_absent = skip_absent;
-				query_entry->adt[ix].constraint = get_query_filter(query, attr.c_str(), tag_skip_absent);
+				query_entry->adt[ix].constraint = get_query_filter(query, attr, tag_skip_absent);
 				query_entry->adt[ix].skip_absent = tag_skip_absent;
 			}
 			query_entry->num_adtypes += 1;
@@ -709,6 +709,11 @@ int CollectorDaemon::receive_query_cedar(int command,
 
 	// Initial query handler
 	whichAds = receive_query_public( command );
+	if (whichAds < 0) {
+		// expect receive_query_public() to log the reason for the failure
+		return_status = FALSE;
+		goto END;
+	}
 	negotiate_auth = (command == QUERY_STARTD_PVT_ADS || command == QUERY_MULTIPLE_PVT_ADS);
 	query_entry = make_query_entry(whichAds, cad, negotiate_auth);
 	if ( ! query_entry) {
@@ -1037,14 +1042,13 @@ int CollectorDaemon::receive_query_cedar_worker_thread(void *in_query_entry, Str
 		// client requests them.
 	bool filter_private_attrs = true;
 	auto *verinfo = sock->get_peer_version();
-	if (verinfo && verinfo->built_since_version(8, 9, 3) &&
+	if (verinfo && verinfo->built_since_version(8, 9, 3) && !verinfo->built_since_version(10, 0, 0)) {
+		wants_pvt_attrs = true;
+	}
+	if (wants_pvt_attrs &&
 		(USER_AUTH_SUCCESS == daemonCore->Verify("send private attrs", NEGOTIATOR, *static_cast<ReliSock*>(sock), D_SECURITY|D_FULLDEBUG)))
 	{
-		if (verinfo->built_since_version(10, 0, 0)) {
-			filter_private_attrs = !wants_pvt_attrs;
-		} else {
-			filter_private_attrs = false;
-		}
+		filter_private_attrs = false;
 	}
 
 		// If our peer has ADMINISTRATOR authz and explicitly asks for
@@ -1906,7 +1910,7 @@ void CollectorDaemon::collect_op::process_invalidation (AdTypes whichAds, ClassA
 	if (whichAds == GENERIC_AD || whichAds == ANY_AD || whichAds == BOGUS_AD) {
 		if (query.LookupString(ATTR_TARGET_TYPE, target_type)) {
 			__mytype__ = target_type.c_str(); // in case we want to constrain an invalidate
-			hTable = collector.getGenericHashTable(target_type);
+			hTable = collector.getGenericHashTable(target_type.c_str());
 			if (hTable) { makeKey = makeGenericAdHashKey; }
 			else if (whichAds != GENERIC_AD) {
 				// maybe a standard table after all?
@@ -2478,7 +2482,7 @@ void CollectorDaemon::sendCollectorAd(int /* tid */)
 	}
 
 	// Send the ad
-	int num_updated = collectorsToUpdate->sendUpdates(UPDATE_COLLECTOR_AD, ad, NULL, false);
+	int num_updated = collectorsToUpdate->sendUpdates(UPDATE_COLLECTOR_AD, ad, nullptr, true);
 	if ( num_updated != (int)collectorsToUpdate->getList().size() ) {
 		dprintf( D_ALWAYS, "Unable to send UPDATE_COLLECTOR_AD to all configured collectors\n");
 	}
