@@ -60,7 +60,7 @@ makeAdTypeToWhichAdsTable() {
 		{ CLUSTER_ADTYPE,		CLUSTER_AD },
 		{ GENERIC_ADTYPE,		GENERIC_AD },
 		{ CREDD_ADTYPE,			CREDD_AD },
-		{ XFER_SERVICE_ADTYPE,	XFER_SERVICE_AD },
+		{ PLACEMENTD_ADTYPE,	PLACEMENTD_AD },
 		{ LEASE_MANAGER_ADTYPE,	LEASE_MANAGER_AD },
 		{ GRID_ADTYPE,			GRID_AD },
 		{ DEFRAG_ADTYPE,		DEFRAG_AD },    // Defrag ads go into the generic table in the collector
@@ -408,7 +408,7 @@ walkHashTable (AdTypes adType, int (*scanFunction)(CollectorRecord *))
 }
 #endif
 
-CollectorHashTable *CollectorEngine::findOrCreateTable(const std::string &type)
+CollectorHashTable *CollectorEngine::findOrCreateTable(const istring &type)
 {
 	CollectorHashTable *table=0;
 	if (GenericAds.lookup(type, table) == -1) {
@@ -464,7 +464,7 @@ collect (int command, Sock *sock, const condor_sockaddr& from, int &insert)
 
 	if( !getClassAdEx(sock, *clientAd, m_get_ad_options) )
 	{
-		dprintf (D_ALWAYS,"Command %d on Sock not followed by ClassAd (or timeout occured)\n",
+		dprintf (D_ALWAYS,"Command %d on Sock not followed by ClassAd (or timeout occurred)\n",
 				command);
 		delete clientAd;
 		sock->end_of_message();
@@ -686,17 +686,11 @@ collect (int command,ClassAd *clientAd,const condor_sockaddr& from,int &insert,S
 	AdNameHashKey		hk;
 	std::string hashString;
 	AdTypes realAdType = NO_AD;
-	static int repeatStartdAds = -1;		// for debugging
-	ClassAd		*clientAdToRepeat = NULL;
 #ifdef PROFILE_RECEIVE_UPDATE
 	_condor_auto_accum_runtime<collector_runtime_probe> rt(CollectorEngine_rucc_runtime);
 	double rt_last = rt.begin;
 #endif
 
-	if (repeatStartdAds == -1) {
-		repeatStartdAds = param_integer("COLLECTOR_REPEAT_STARTD_ADS",0);
-	}
-
 	if( !ValidateClassAd(command,clientAd,sock) ) {
 	    insert = -4;
 		return NULL;
@@ -711,9 +705,6 @@ collect (int command,ClassAd *clientAd,const condor_sockaddr& from,int &insert,S
 	{
 	  case UPDATE_STARTD_AD:
 	  case UPDATE_STARTD_AD_WITH_ACK:
-		if ( repeatStartdAds > 0 ) {
-			clientAdToRepeat = new ClassAd(*clientAd);
-		}
 		if (!makeStartdAdHashKey (hk, clientAd))
 		{
 			dprintf (D_ALWAYS, "Could not make hashkey --- ignoring ad\n");
@@ -799,35 +790,6 @@ collect (int command,ClassAd *clientAd,const condor_sockaddr& from,int &insert,S
 #endif
 		}
 
-		// create fake duplicates of this ad, each with a different name, if
-		// we are told to do so.  this feature exists for developer
-		// scalability testing.
-		if ( repeatStartdAds > 0 && clientAdToRepeat ) {
-			ClassAd *fakeAd;
-			int n;
-			char newname[150],oldname[130];
-			oldname[0] = '\0';
-			clientAdToRepeat->LookupString(ATTR_NAME,oldname,sizeof(oldname));
-			for (n=0;n<repeatStartdAds;n++) {
-				fakeAd = new ClassAd(*clientAdToRepeat);
-				snprintf(newname,sizeof(newname),
-						 "fake%d-%s",n,oldname);
-				fakeAd->Assign(ATTR_NAME, newname);
-				makeStartdAdHashKey (hk, fakeAd);
-				hk.sprint(hashString);
-				if (! updateClassAd (StartdSlotAds, "MachineSlotAd ", "Slot", true,
-							  fakeAd, hk, hashString, insert, from ) )
-				{
-					// don't leak memory if there is some failure
-					delete fakeAd;
-				}
-			}
-			delete clientAdToRepeat;
-			clientAdToRepeat = NULL;
-#ifdef PROFILE_RECEIVE_UPDATE
-			CollectorEngine_rucc_repeatAd_runtime.Add(rt.tick(rt_last));
-#endif
-		}
 		break;
 
 	  case MERGE_STARTD_AD:
@@ -1008,8 +970,7 @@ collect (int command,ClassAd *clientAd,const condor_sockaddr& from,int &insert,S
 			  retVal = 0;
 			  break;
 		  }
-		  std::string type(type_str);
-		  CollectorHashTable *cht = findOrCreateTable(type);
+		  CollectorHashTable *cht = findOrCreateTable(type_str);
 		  if (cht == NULL) {
 			  dprintf(D_ALWAYS, "collect: findOrCreateTable failed\n");
 			  insert = -3;
@@ -1214,21 +1175,6 @@ identifySelfAd(CollectorRecord * ad)
 
 extern bool   last_updateClassAd_was_insert;
 
-void
-movePrivateAttrs(ClassAd& dest, ClassAd& src)
-{
-	auto itr = src.begin();
-	while (itr != src.end()) {
-		if (ClassAdAttributeIsPrivateAny(itr->first)) {
-			const std::string &name = itr->first;
-			dest.Insert(name, itr->second->Copy());
-			itr = src.erase(itr);
-		} else {
-			itr++;
-		}
-	}
-}
-
 CollectorRecord * CollectorEngine::
 updateClassAd (CollectorHashTable &hashTable,
 			   const char *adType,
@@ -1256,7 +1202,7 @@ updateClassAd (CollectorHashTable &hashTable,
 		{
 			EXCEPT ("Error reading system time!");
 		}	
-		ad->Assign(ATTR_LAST_HEARD_FROM, (int)now);
+		ad->Assign(ATTR_LAST_HEARD_FROM, now);
 	}
 
 	// this time stamped ad is the new ad
@@ -1264,7 +1210,7 @@ updateClassAd (CollectorHashTable &hashTable,
 	last_updateClassAd_was_insert = false;
 
 	new_pvt_ad = new ClassAd();
-	movePrivateAttrs(*new_pvt_ad, *new_ad);
+	movePrivateAttrs(*new_ad, *new_pvt_ad);
 
 	// check if it already exists in the hash table ...
 	if ( hashTable.lookup (hk, record) == -1)
@@ -1307,7 +1253,7 @@ updateClassAd (CollectorHashTable &hashTable,
 
 		if ( m_forwardFilteringEnabled && filter_forward ) {
 			bool forward = false;
-			int last_forwarded = 0;
+			time_t last_forwarded = 0;
 			old_ad->LookupInteger( "LastForwarded", last_forwarded );
 			if ( last_forwarded + m_forwardInterval < time(NULL) ) {
 				forward = true;
@@ -1327,7 +1273,7 @@ updateClassAd (CollectorHashTable &hashTable,
 				}
 			}
 			new_ad->Assign( ATTR_SHOULD_FORWARD, forward );
-			new_ad->Assign( ATTR_LAST_FORWARDED, forward ? (int)time(NULL) : last_forwarded );
+			new_ad->Assign( ATTR_LAST_FORWARDED, forward ? time(nullptr) : last_forwarded );
 		}
 
 		// Now, finally, store the new ClassAd
@@ -1374,7 +1320,7 @@ mergeClassAd (CollectorHashTable &hashTable,
 		new_ad_copy.Delete(ATTR_TARGET_TYPE);
 
 		ClassAd new_pvt_ad;
-		movePrivateAttrs(new_pvt_ad, new_ad_copy);
+		movePrivateAttrs(new_ad_copy, new_pvt_ad);
 
 		// Now, finally, merge the new ClassAd into the old one
 		MergeClassAds(record->m_publicAd, &new_ad_copy, true);
