@@ -37,12 +37,12 @@
 int ViewServer::HistoryInterval;
 int ViewServer::MaxFileSize;
 DataSetInfo ViewServer::DataSet[DataSetCount][HistoryLevels];
-int ViewServer::TimeStamp;
+time_t ViewServer::TimeStamp;
 int ViewServer::HistoryTimer;
 std::string ViewServer::DataFormat[DataSetCount];
 AccHash* ViewServer::GroupHash;
 bool ViewServer::KeepHistory;
-HashTable< std::string, int >* ViewServer::FileHash;
+std::map< std::string, int >* ViewServer::FileHash;
 std::vector<ExtIntArray *> *ViewServer::TimesArray;
 std::vector<ExtOffArray *> *ViewServer::OffsetsArray;
 
@@ -117,11 +117,11 @@ void ViewServer::Init()
 
 	for (int i=0; i<DataSetCount; i++) {
 		for (int j=0; j<HistoryLevels; j++) {
-			DataSet[i][j].AccData=new AccHash(hashFunction);
+			DataSet[i][j].AccData=new AccHash;
 		}
 	}
-	GroupHash = new AccHash(hashFunction);
-	FileHash = new HashTable< std::string, int >(hashFunction);
+	GroupHash = new AccHash;
+	FileHash = new std::map<std::string, int>;
 	OffsetsArray = new std::vector< ExtOffArray*>;
 	TimesArray = new std::vector<ExtIntArray*>;
 
@@ -223,7 +223,9 @@ int ViewServer::ReceiveHistoryQuery(int command, Stream* sock)
 {
 	dprintf(D_ALWAYS,"Got history query %d\n",command);
 
-	int FromDate, ToDate, Options;
+	time_t FromDate;
+	time_t ToDate;
+	int Options;
 	char *Line = NULL;
 
 	// Read query parameters
@@ -239,7 +241,7 @@ int ViewServer::ReceiveHistoryQuery(int command, Stream* sock)
 		return 1;
 	}
 
-	dprintf(D_ALWAYS, "Got history query: FromDate=%d, ToDate=%d, Type=%d, Arg=%s\n",FromDate,ToDate,command,Line);
+	dprintf(D_ALWAYS, "Got history query: FromDate=%lld, ToDate=%lld, Type=%d, Arg=%s\n",(long long) FromDate,(long long) ToDate,command,Line);
 
 	// Reply to query
 
@@ -263,7 +265,7 @@ int ViewServer::ReceiveHistoryQuery(int command, Stream* sock)
 // appropriate function to send the data
 //-------------------------------------------------------------------
 
-int ViewServer::HandleQuery(Stream* sock, int command, int FromDate, int ToDate, int Options, std::string Arg)
+int ViewServer::HandleQuery(Stream* sock, int command, time_t FromDate, time_t ToDate, int Options, std::string Arg)
 {
 	int DataSetIdx=-1;
 	int ListFlag=0;
@@ -309,8 +311,8 @@ int ViewServer::HandleQuery(Stream* sock, int command, int FromDate, int ToDate,
 	}
 
 	if (HistoryLevel==-1) {
-		int LevelStartTime;
-		int MinStartTime=-1;
+		time_t LevelStartTime;
+		time_t MinStartTime=-1;
 		for (int j=0; j<HistoryLevels; j++) {
 			LevelStartTime=DataSet[DataSetIdx][j].OldStartTime;
 			if (LevelStartTime==-1) LevelStartTime=DataSet[DataSetIdx][j].NewStartTime;
@@ -325,7 +327,7 @@ int ViewServer::HandleQuery(Stream* sock, int command, int FromDate, int ToDate,
 
 	dprintf(D_ALWAYS,"DataSetIdx=%d, HistoryLevel=%d, OldFlag=%d, NewFlag=%d\n",DataSetIdx,HistoryLevel,OldFlag,NewFlag);
 	if (HistoryLevel==-1) return 0;
-	dprintf(D_ALWAYS,"OldStartTime=%d , NewStartTime=%d\n",DataSet[DataSetIdx][HistoryLevel].OldStartTime,DataSet[DataSetIdx][HistoryLevel].NewStartTime);
+	dprintf(D_ALWAYS,"OldStartTime=%lld , NewStartTime=%lld\n",(long long) DataSet[DataSetIdx][HistoryLevel].OldStartTime,(long long) DataSet[DataSetIdx][HistoryLevel].NewStartTime);
 
 	// Read file and send Data
 
@@ -346,16 +348,16 @@ int ViewServer::HandleQuery(Stream* sock, int command, int FromDate, int ToDate,
 // requested time range
 //---------------------------------------------------------------------
 
-int ViewServer::SendListReply(Stream* sock,const std::string& FileName, int FromDate, int ToDate, std::set<std::string>& Names)
+int ViewServer::SendListReply(Stream* sock,const std::string& FileName, time_t FromDate, time_t ToDate, std::set<std::string>& Names)
 {
-	int T = 0;
+	time_t T = 0;
 	int file_array_index;
 	ExtIntArray* times_array = NULL;
 	ExtOffArray* offsets = NULL;
-	// dprintf(D_ALWAYS, "Caches found=%d, looking for correct one...\n", TimesArray->length());
 
 		// first find out which vector to use, by checking the hash
-	if( FileHash->lookup( FileName, file_array_index ) == -1 ){
+	auto file_itr = FileHash->find(FileName);
+	if (file_itr == FileHash->end()) {
 
 			// FileName was not found in the FileHash
 			// Create the necessary arrays and set the index appropriately
@@ -363,18 +365,16 @@ int ViewServer::SendListReply(Stream* sock,const std::string& FileName, int From
 		times_array = new ExtIntArray(100);
 		offsets = new ExtOffArray(100);
 		file_array_index = OffsetsArray->size();
-		FileHash->insert( FileName, file_array_index );
+		FileHash->emplace(FileName, file_array_index);
 		TimesArray->push_back(times_array);
 		OffsetsArray->push_back(offsets);
 	} else {
-
 			// otherwise just get the appropriate array
+		file_array_index = file_itr->second;
 		times_array = (TimesArray->at( file_array_index ));
 		offsets = (OffsetsArray->at( file_array_index ));
-		// dprintf(D_ALWAYS, "Cache found for this file, %d indices\n", offsets->length());
 	}
 
-	// dprintf(D_ALWAYS,"Filename=%s\n",(const char*)FileName);
 	FILE* fp=safe_fopen_wrapper_follow(FileName.c_str(),"r");
 	if (!fp) return -1;
 
@@ -433,20 +433,21 @@ int ViewServer::SendListReply(Stream* sock,const std::string& FileName, int From
 	return 0;
 }
 
-int ViewServer::SendDataReply(Stream* sock,const std::string& FileName, int FromDate, int ToDate, int Options, const std::string& Arg)
+int ViewServer::SendDataReply(Stream* sock,const std::string& FileName, time_t FromDate, time_t ToDate, int Options, const std::string& Arg)
 {
 	std::string InpLine;
 	int Status=0;
-	int NewTime, OldTime;
+	time_t NewTime, OldTime;
 	float OutTime;
-	int T = 0;
+	time_t T = 0;
 	int file_array_index;
 	ExtIntArray* times_array = NULL;
 	ExtOffArray* offsets = NULL;
 	// dprintf(D_ALWAYS, "Caches found=%d, looking for correct one...\n", TimesArray->length());
 
 		// first find out which vector to use, by checking the hash
-	if( FileHash->lookup( FileName, file_array_index ) == -1 ){
+	auto file_itr = FileHash->find(FileName);
+	if (file_itr == FileHash->end()) {
 
 			// FileName was not found in the FileHash
 			// Create the necessary arrays and set the index appropriately
@@ -454,12 +455,13 @@ int ViewServer::SendDataReply(Stream* sock,const std::string& FileName, int From
 		times_array = new ExtIntArray(100);
 		offsets = new ExtOffArray(100);
 		file_array_index = OffsetsArray->size();
-		FileHash->insert( FileName, file_array_index );
+		FileHash->emplace(FileName, file_array_index);
 		TimesArray->push_back( times_array );
 		OffsetsArray->push_back( offsets );
 	} else {
 
 			// otherwise just get the appropriate array
+		file_array_index = file_itr->second;
 		times_array = (TimesArray->at( file_array_index ));
 		offsets = (OffsetsArray->at( file_array_index ));
 		// dprintf(D_ALWAYS, "Cache found for this file, %d indices\n", times_array->length());
@@ -483,12 +485,10 @@ int ViewServer::SendDataReply(Stream* sock,const std::string& FileName, int From
 	int new_offset_counter = 1;		// every fifty loops, mark an offset
 	while(readLine(InpLine,fp)) {
 
-		// dprintf(D_ALWAYS,"Line read: %s\n",InpLine);
 		T=ReadTimeChkName(InpLine,Arg);
 		if( times_array->size() < offsets->size() ) {
 				// a file offset was recorded before this line was read; now
 				// store the time that was on the marked line
-			// dprintf(D_ALWAYS, "Adding time=%d to the cache", T);
 			times_array->push_back( T );
 		}
 		// dprintf(D_ALWAYS,"T=%d\n",T);
@@ -540,10 +540,10 @@ int ViewServer::SendDataReply(Stream* sock,const std::string& FileName, int From
 //-------------------------------------------------------------------
 
 void
-ViewServer::addNewOffset(FILE* &fp, int &offset_ctr, int read_time, ExtIntArray* times_array, ExtOffArray* offsets) {
+ViewServer::addNewOffset(FILE* &fp, int &offset_ctr, time_t read_time, ExtIntArray* times_array, ExtOffArray* offsets) {
 	if( ++offset_ctr == 50) {
 		offset_ctr = 0;
-		if(times_array->size() == 0 || read_time > times_array->back()) {
+		if (times_array->empty() || read_time > times_array->back()) {
 				// mark the position in the file now, but wait to mark the time
 				// until after the line is read
 			// dprintf(D_ALWAYS, "Adding new offset to the cache\n");
@@ -562,7 +562,7 @@ ViewServer::addNewOffset(FILE* &fp, int &offset_ctr, int read_time, ExtIntArray*
 //-------------------------------------------------------------------
 
 fpos_t*
-ViewServer::findOffset(FILE* & /*fp*/, int FromDate, int ToDate, ExtIntArray* times_array, ExtOffArray* offsets) {
+ViewServer::findOffset(FILE* & /*fp*/, time_t FromDate, time_t ToDate, ExtIntArray* times_array, ExtOffArray* offsets) {
 	fpos_t* search_offset_ptr = NULL;
 	if( times_array->size() == 0 ) {
 
@@ -628,10 +628,10 @@ ViewServer::findOffset(FILE* & /*fp*/, int FromDate, int ToDate, ExtIntArray* ti
 // in the file
 //-------------------------------------------------------------------
 
-int ViewServer::FindFileStartTime(const char *Name)
+time_t ViewServer::FindFileStartTime(const char *Name)
 {
 	std::string line;
-	int T=-1;
+	time_t T=-1;
 	FILE* fp=safe_fopen_wrapper_follow(Name,"r");
 	if (fp) {
 		if (readLine(line,fp)) {
@@ -642,7 +642,7 @@ int ViewServer::FindFileStartTime(const char *Name)
 		}
 		fclose(fp);
 	}
-	dprintf(D_ALWAYS,"FileName=%s , StartTime=%d\n",Name,T);
+	dprintf(D_ALWAYS,"FileName=%s , StartTime=%lld\n",Name,(long long)T);
 	return T;
 }
 
@@ -650,10 +650,10 @@ int ViewServer::FindFileStartTime(const char *Name)
 // Parse the entry time from the line
 //-------------------------------------------------------------------
 
-int ViewServer::ReadTime(const char* Line)
+time_t ViewServer::ReadTime(const char* Line)
 {
-	int t=-1;
-	if (sscanf(Line,"%d",&t)!=1) return -1;
+	long long t=-1;
+	if (sscanf(Line,"%lld",&t)!=1) return -1;
 	return t;
 }
 
@@ -661,9 +661,9 @@ int ViewServer::ReadTime(const char* Line)
 // Parse the entry time and name from the line
 //-------------------------------------------------------------------
 
-int ViewServer::ReadTimeAndName(const std::string &line, std::string& Name)
+time_t ViewServer::ReadTimeAndName(const std::string &line, std::string& Name)
 {
-	int t=-1;
+	time_t t=-1;
 
 	// Line contains int, tab, a string, tab then junk
 	char *endOfInt = nullptr;
@@ -684,9 +684,9 @@ int ViewServer::ReadTimeAndName(const std::string &line, std::string& Name)
 // Parse the time and check for the specified name
 //-------------------------------------------------------------------
 
-int ViewServer::ReadTimeChkName(const std::string &line, const std::string& Name)
+time_t ViewServer::ReadTimeChkName(const std::string &line, const std::string& Name)
 {
-	int t=-1;
+	time_t t=-1;
 	//if (sscanf(Line,"%d %s",&t,tmp)!=2) return -1;
 	char *p = nullptr;
 	t = strtol(line.c_str(), &p, 10);
@@ -711,8 +711,6 @@ int ViewServer::ReadTimeChkName(const std::string &line, const std::string& Name
 
 void ViewServer::WriteHistory(int /* tid */)
 {
-	std::string Key;
-	GeneralRecord* GenRec;
 	FILE* DataFile;
 	struct stat statbuf;
 	std::string outline;
@@ -721,7 +719,7 @@ void ViewServer::WriteHistory(int /* tid */)
 
 	TimeStamp=time(0);
 
-	dprintf(D_ALWAYS,"Accumulating data: Time=%d\n",TimeStamp);
+	dprintf(D_ALWAYS,"Accumulating data: Time=%lld\n",(long long) TimeStamp);
 
 	if (!collector.walkHashTable(*collector.getHashTable(SUBMITTOR_AD), SubmittorScanFunc)) {
 		dprintf (D_ALWAYS, "Error accumulating data\n");
@@ -759,8 +757,7 @@ void ViewServer::WriteHistory(int /* tid */)
 
 			// Iterate through accumulated values
 
-			DataSet[i][j].AccData->startIterations();
-			while(DataSet[i][j].AccData->iterate(Key,GenRec)) {
+			for (auto& [Key, GenRec]: *DataSet[i][j].AccData) {
 				formatstr(outline,DataFormat[i].c_str(),TimeStamp,Key.c_str(),GenRec->Data[0],GenRec->Data[1],GenRec->Data[2],GenRec->Data[3],GenRec->Data[4],GenRec->Data[5],GenRec->Data[6], GenRec->Data[7], GenRec->Data[8]);
 				delete GenRec;
 				fputs(outline.c_str(), DataFile);
@@ -793,8 +790,10 @@ void ViewServer::WriteHistory(int /* tid */)
 				}
 				int newFileIndex = -1;
 				int oldFileIndex = -1;
-				if(FileHash->lookup(DataSet[i][j].OldFileName.c_str(),
-														oldFileIndex) != -1) {
+				auto new_file_itr = FileHash->end();
+				auto old_file_itr = FileHash->find(DataSet[i][j].OldFileName);
+				if (old_file_itr != FileHash->end()) {
+					oldFileIndex = old_file_itr->second;
 						// get rid of the old arrays and make new ones
 					delete (*TimesArray)[oldFileIndex];
 					for (auto p: *(*OffsetsArray)[oldFileIndex]) {
@@ -803,24 +802,20 @@ void ViewServer::WriteHistory(int /* tid */)
 					delete (*OffsetsArray)[oldFileIndex];
 					(*TimesArray)[oldFileIndex] = new ExtIntArray;
 					(*OffsetsArray)[oldFileIndex] = new ExtOffArray;
-					if(FileHash->lookup(DataSet[i][j].NewFileName.c_str(),
-														newFileIndex) != -1) {
+					new_file_itr = FileHash->find(DataSet[i][j].NewFileName);
+					if (new_file_itr != FileHash->end()) {
+						newFileIndex = new_file_itr->second;
 							// switch the indices to avoid copying data
-						FileHash->remove(DataSet[i][j].OldFileName.c_str());
-						FileHash->remove(DataSet[i][j].NewFileName.c_str());
-						FileHash->insert(DataSet[i][j].OldFileName.c_str(),
-															newFileIndex);
-						FileHash->insert(DataSet[i][j].NewFileName.c_str(),
-															oldFileIndex);
+						old_file_itr->second = newFileIndex;
+						new_file_itr->second = oldFileIndex;
 					}
 				}
-				else if(FileHash->lookup(DataSet[i][j].NewFileName.c_str(),
-													newFileIndex) != -1) {
+				else if((new_file_itr = FileHash->find(DataSet[i][j].NewFileName)) != FileHash->end()) {
+					newFileIndex = new_file_itr->second;
 						// if no file got overwritten, then just add to the
 						// hash and arrays
-					FileHash->remove(DataSet[i][j].NewFileName.c_str());
-					FileHash->insert(DataSet[i][j].OldFileName.c_str(),
-															newFileIndex);
+					FileHash->erase(new_file_itr);
+					(*FileHash)[DataSet[i][j].OldFileName] = newFileIndex;
 				}
 				DataSet[i][j].OldStartTime=DataSet[i][j].NewStartTime;
 				DataSet[i][j].NewStartTime=-1;
@@ -884,13 +879,10 @@ int ViewServer::SubmittorTotalFunc(void)
 {
 	// Add to Accumulated Data
 
-	std::string GroupName;
-	GeneralRecord* CurValue;
 	GeneralRecord* AccValue;
 	int NumSamples;
 
-	GroupHash->startIterations();
-	while(GroupHash->iterate(GroupName,CurValue)) {
+	for (const auto& [GroupName, CurValue]: *GroupHash) {
 		for (int j=0; j<HistoryLevels; j++) {
 			AccValue=GetAccData(DataSet[SubmittorGroupsData][j].AccData, GroupName);
 			NumSamples=DataSet[SubmittorGroupsData][j].NumSamples;
@@ -1013,13 +1005,10 @@ int ViewServer::StartdTotalFunc(void)
 {
 	// Add to Accumulated Data
 
-	std::string GroupName;
-	GeneralRecord* CurValue;
 	GeneralRecord* AccValue;
 	int NumSamples;
 
-	GroupHash->startIterations();
-	while(GroupHash->iterate(GroupName,CurValue)) {
+	for (const auto& [GroupName, CurValue]: *GroupHash) {
 		for (int j=0; j<HistoryLevels; j++) {
 			AccValue=GetAccData(DataSet[GroupsData][j].AccData, GroupName);
 			NumSamples=DataSet[GroupsData][j].NumSamples;
@@ -1081,23 +1070,22 @@ int ViewServer::CkptScanFunc(CollectorRecord* record)
 GeneralRecord* ViewServer::GetAccData(AccHash* AccData,const std::string& Key)
 {
 	GeneralRecord* GenRec = NULL;
-	int rval;
 
-	rval = AccData->lookup( Key, GenRec );
-	if ( ( rval < 0 ) || ( GenRec == NULL ) ) {
-		if ( rval >= 0 ) {
+	auto itr = AccData->find(Key);
+	if (itr == AccData->end() || itr->second == nullptr) {
+		if ( itr != AccData->end() ) {
 			dprintf( D_ALWAYS,
-					 "Hash %p lookup returned %d, but NULL record!\n",
-					 AccData, rval );
+					 "Hash lookup %s, but NULL record!\n",
+					 Key.c_str() );
 			EXCEPT( "Bad lookup" );
 		}
 		GenRec=new GeneralRecord;
 		if ( GenRec == NULL ) {
 			EXCEPT( "Failed to allocate a GeneralRecord" );
 		}
-		if ( AccData->insert(Key,GenRec) < 0 ) {
-			EXCEPT( "Insert failed: Key=%s", Key.c_str() );
-		}
+		AccData->emplace(Key, GenRec);
+	} else {
+		GenRec = itr->second;
 	}
 	return GenRec;
 }
