@@ -34,7 +34,6 @@
 #include "condor_daemon_core.h"
 #include "dc_collector.h"
 #include "time_offset.h"
-#include "condor_netdb.h"
 #include "subsystem_info.h"
 #include "condor_netaddr.h"
 #include "condor_sinful.h"
@@ -43,6 +42,8 @@
 
 #include "ipv6_hostname.h"
 
+int Daemon::m_next_sec_context_id = 1;
+
 void
 Daemon::common_init() {
 	_type = DT_NONE;
@@ -53,7 +54,6 @@ Daemon::common_init() {
 	_tried_init_version = false;
 	_is_configured = true;
 	_error_code = CA_SUCCESS;
-	m_daemon_ad_ptr = NULL;
 	char buf[200];
 	snprintf(buf,sizeof(buf),"%s_TIMEOUT_MULTIPLIER",get_mySubSystem()->getName() );
 	Sock::set_timeout_multiplier( param_integer(buf, param_integer("TIMEOUT_MULTIPLIER", 0)) );
@@ -133,12 +133,17 @@ Daemon::Daemon( const ClassAd* tAd, daemon_t tType, const char* tPool )
 	case DT_CREDD:
 		_subsys = "CREDD";
 		break;
+	case DT_PLACEMENTD:
+		_subsys = "PLACEMENTD";
+		break;
 	case DT_GENERIC:
 		_subsys = "GENERIC";
 		break;
 	case DT_HAD:
 		_subsys = "HAD";
 		break;
+	case DT_ANY:
+		break;
 	default:
 		EXCEPT( "Invalid daemon_type %d (%s) in ClassAd version of "
 				"Daemon object", (int)_type, daemonString(_type) );
@@ -155,7 +160,7 @@ Daemon::Daemon( const ClassAd* tAd, daemon_t tType, const char* tPool )
 			 _name.c_str(), _pool.c_str(), _addr.c_str() );
 
 	// let's have our own copy of the daemon's ad in this case.
-	m_daemon_ad_ptr = new ClassAd(*tAd);	
+	m_daemon_ad = *tAd;
 
 }
 
@@ -209,10 +214,10 @@ Daemon::deepCopy( const Daemon &copy )
 	_tried_init_hostname = copy._tried_init_hostname;
 	_tried_init_version = copy._tried_init_version;
 	_is_configured = copy._is_configured;
-	if(copy.m_daemon_ad_ptr) {
-		m_daemon_ad_ptr = new ClassAd(*copy.m_daemon_ad_ptr);
-	}
+	m_daemon_ad = copy.m_daemon_ad;
 
+	m_sec_context_id = copy.m_sec_context_id;
+	m_preferred_token = copy.m_preferred_token;
 	m_owner = copy.m_owner;
 	m_methods = copy.m_methods;
 
@@ -234,7 +239,6 @@ Daemon::~Daemon()
 		display( D_HOSTNAME );
 		dprintf( D_HOSTNAME, " --- End of Daemon object info ---\n" );
 	}
-	if( m_daemon_ad_ptr) { delete m_daemon_ad_ptr; }
 }
 
 
@@ -244,56 +248,54 @@ Daemon::~Daemon()
 
 ClassAd *
 Daemon::locationAd() {
-	if( m_daemon_ad_ptr ) {
+	if( !m_daemon_ad.empty() ) {
 		// dprintf( D_ALWAYS, "locationAd(): found daemon ad, returning it\n" );
-		return m_daemon_ad_ptr;
+		return &m_daemon_ad;
 	}
 
-	if( m_location_ad_ptr ) {
+	if( !m_location_ad.empty() ) {
 		// dprintf( D_ALWAYS, "locationAd(): found location ad, returning it\n" );
-		return m_location_ad_ptr;
+		return &m_location_ad;
 	}
 
-	ClassAd * locationAd = new ClassAd();
 	const char * buffer = NULL;
 
 	buffer = this->addr();
 	if(! buffer) { goto failure; }
-	if(! locationAd->InsertAttr(ATTR_MY_ADDRESS, buffer)) { goto failure; }
+	if(! m_location_ad.InsertAttr(ATTR_MY_ADDRESS, buffer)) { goto failure; }
 
 	buffer = this->name();
 	if(! buffer) { buffer = "Unknown"; }
-	if(! locationAd->InsertAttr(ATTR_NAME, buffer)) { goto failure; }
+	if(! m_location_ad.InsertAttr(ATTR_NAME, buffer)) { goto failure; }
 
 	buffer = this->fullHostname();
 	if(! buffer) { buffer = "Unknown"; }
-	if(! locationAd->InsertAttr(ATTR_MACHINE, buffer)) { goto failure; }
+	if(! m_location_ad.InsertAttr(ATTR_MACHINE, buffer)) { goto failure; }
 
 	/* This will inevitably be overwritten by CondorVersion(), below,
 	   so I don't know what the original was attempting accomplish here. */
 	buffer = this->version();
 	if(! buffer) { buffer = ""; }
-	if(! locationAd->InsertAttr(ATTR_VERSION, buffer)) { goto failure; }
+	if(! m_location_ad.InsertAttr(ATTR_VERSION, buffer)) { goto failure; }
 
 	AdTypes ad_type;
 	if(! convert_daemon_type_to_ad_type(this->type(), ad_type)) { goto failure; }
 	buffer = AdTypeToString(ad_type);
 	if(! buffer) { goto failure; }
-	if(! locationAd->InsertAttr(ATTR_MY_TYPE, buffer)) { goto failure; }
+	if(! m_location_ad.InsertAttr(ATTR_MY_TYPE, buffer)) { goto failure; }
 
 	buffer = CondorVersion();
-	if(! locationAd->InsertAttr(ATTR_VERSION, buffer)) { goto failure; }
+	if(! m_location_ad.InsertAttr(ATTR_VERSION, buffer)) { goto failure; }
 
 	buffer = CondorPlatform();
-	if(! locationAd->InsertAttr(ATTR_PLATFORM, buffer)) { goto failure; }
+	if(! m_location_ad.InsertAttr(ATTR_PLATFORM, buffer)) { goto failure; }
 
 	// dprintf( D_ALWAYS, "locationAd(): synthesized location ad, returning it.\n" );
-	m_location_ad_ptr = locationAd;
-	return m_location_ad_ptr;
+	return &m_location_ad;
 
   failure:;
 	// dprintf( D_ALWAYS, "Daemon::locationAd() failed.\n" );
-	delete locationAd;
+	m_location_ad.Clear();
 	return NULL;
 }
 
@@ -489,7 +491,7 @@ Daemon::rewindCmList()
 //////////////////////////////////////////////////////////////////////
 
 ReliSock*
-Daemon::reliSock( int timeout, time_t deadline, CondorError* errstack, bool non_blocking, bool ignore_timeout_multiplier )
+Daemon::reliSock( time_t timeout, time_t deadline, CondorError* errstack, bool non_blocking, bool ignore_timeout_multiplier )
 {
 	if( !checkAddr() ) {
 			// this already deals w/ _error for us...
@@ -510,7 +512,7 @@ Daemon::reliSock( int timeout, time_t deadline, CondorError* errstack, bool non_
 }
 
 SafeSock*
-Daemon::safeSock( int timeout, time_t deadline, CondorError* errstack, bool non_blocking )
+Daemon::safeSock( time_t timeout, time_t deadline, CondorError* errstack, bool non_blocking )
 {
 	if( !checkAddr() ) {
 			// this already deals w/ _error for us...
@@ -532,7 +534,7 @@ Daemon::safeSock( int timeout, time_t deadline, CondorError* errstack, bool non_
 
 
 bool
-Daemon::connectSock(Sock *sock, int sec, CondorError* errstack, bool non_blocking, bool ignore_timeout_multiplier )
+Daemon::connectSock(Sock *sock, time_t sec, CondorError* errstack, bool non_blocking, bool ignore_timeout_multiplier )
 {
 
 	sock->set_peer_description(idStr());
@@ -557,7 +559,7 @@ Daemon::connectSock(Sock *sock, int sec, CondorError* errstack, bool non_blockin
 
 
 StartCommandResult
-Daemon::startCommand_internal( const SecMan::StartCommandRequest &req, int timeout, SecMan *sec_man )
+Daemon::startCommand_internal( SecMan::StartCommandRequest &req, time_t timeout, SecMan *sec_man )
 {
 	// This function may be either blocking or non-blocking, depending
 	// on the flag that is passed in.  All versions of Daemon::startCommand()
@@ -566,6 +568,14 @@ Daemon::startCommand_internal( const SecMan::StartCommandRequest &req, int timeo
 	// NOTE: if there is a callback function, we _must_ guarantee that it is
 	// eventually called in all code paths.
 
+	if (m_use_new_sec_context_id) {
+		m_sec_context_id = m_next_sec_context_id++;
+		m_use_new_sec_context_id = false;
+	}
+	if (m_sec_context_id > 0) {
+		formatstr(req.m_sec_context_tag, "daemon:%d", m_sec_context_id);
+	}
+
 	ASSERT(req.m_sock);
 
 	// If caller wants non-blocking with no callback function,
@@ -585,7 +595,7 @@ Daemon::startCommand_internal( const SecMan::StartCommandRequest &req, int timeo
 
 Sock *
 Daemon::makeConnectedSocket( Stream::stream_type st,
-							 int timeout, time_t deadline,
+							 time_t timeout, time_t deadline,
 							 CondorError* errstack, bool non_blocking )
 {
 	switch( st ) {
@@ -602,7 +612,7 @@ Daemon::makeConnectedSocket( Stream::stream_type st,
 }
 
 StartCommandResult
-Daemon::startCommand( int cmd, Stream::stream_type st,Sock **sock,int timeout, CondorError *errstack, int subcmd, StartCommandCallbackType *callback_fn, void *misc_data, bool nonblocking, char const *cmd_description, bool raw_protocol, char const *sec_session_id, bool resume_response )
+Daemon::startCommand( int cmd, Stream::stream_type st,Sock **sock,time_t timeout, CondorError *errstack, int subcmd, StartCommandCallbackType *callback_fn, void *misc_data, bool nonblocking, char const *cmd_description, bool raw_protocol, char const *sec_session_id, bool resume_response )
 {
 	// This function may be either blocking or non-blocking, depending on
 	// the flag that was passed in.
@@ -641,15 +651,17 @@ Daemon::startCommand( int cmd, Stream::stream_type st,Sock **sock,int timeout, C
 	req.m_nonblocking = nonblocking;
 	req.m_cmd_description = cmd_description;
 	req.m_sec_session_id = sec_session_id ? sec_session_id : m_sec_session_id.c_str();
+	req.m_preferred_token = m_preferred_token;
 	req.m_owner = m_owner;
 	req.m_methods = m_methods;
+	req.m_force_auth = m_force_auth;
 
 	return startCommand_internal( req, timeout, &_sec_man );
 }
 
 
 bool
-Daemon::startSubCommand( int cmd, int subcmd, Sock* sock, int timeout, CondorError *errstack, char const *cmd_description,bool raw_protocol, char const *sec_session_id, bool resume_response )
+Daemon::startSubCommand( int cmd, int subcmd, Sock* sock, time_t timeout, CondorError *errstack, char const *cmd_description,bool raw_protocol, char const *sec_session_id, bool resume_response )
 {
 	SecMan::StartCommandRequest req;
 	req.m_cmd = cmd;
@@ -664,8 +676,10 @@ Daemon::startSubCommand( int cmd, int subcmd, Sock* sock, int timeout, CondorErr
 	req.m_nonblocking = false;
 	req.m_cmd_description = cmd_description;
 	req.m_sec_session_id = sec_session_id ? sec_session_id : m_sec_session_id.c_str();
+	req.m_preferred_token = m_preferred_token;
 	req.m_owner = m_owner;
 	req.m_methods = m_methods;
+	req.m_force_auth = m_force_auth;
 
 	auto rc = startCommand_internal(req, timeout, &_sec_man);
 
@@ -685,7 +699,7 @@ Daemon::startSubCommand( int cmd, int subcmd, Sock* sock, int timeout, CondorErr
 
 
 Sock*
-Daemon::startSubCommand( int cmd, int subcmd, Stream::stream_type st, int timeout, CondorError* errstack, char const *cmd_description, bool raw_protocol, char const *sec_session_id, bool resume_response )
+Daemon::startSubCommand( int cmd, int subcmd, Stream::stream_type st, time_t timeout, CondorError* errstack, char const *cmd_description, bool raw_protocol, char const *sec_session_id, bool resume_response )
 {
 	// This is a blocking version of startCommand.
 	const bool nonblocking = false;
@@ -710,7 +724,7 @@ Daemon::startSubCommand( int cmd, int subcmd, Stream::stream_type st, int timeou
 
 
 Sock*
-Daemon::startCommand( int cmd, Stream::stream_type st, int timeout, CondorError* errstack, char const *cmd_description, bool raw_protocol, char const *sec_session_id, bool resume_response )
+Daemon::startCommand( int cmd, Stream::stream_type st, time_t timeout, CondorError* errstack, char const *cmd_description, bool raw_protocol, char const *sec_session_id, bool resume_response )
 {
 	// This is a blocking version of startCommand.
 	const bool nonblocking = false;
@@ -734,7 +748,7 @@ Daemon::startCommand( int cmd, Stream::stream_type st, int timeout, CondorError*
 }
 
 StartCommandResult
-Daemon::startCommand_nonblocking( int cmd, Stream::stream_type st, int timeout, CondorError *errstack, StartCommandCallbackType *callback_fn, void *misc_data, char const *cmd_description, bool raw_protocol, char const *sec_session_id, bool resume_response )
+Daemon::startCommand_nonblocking( int cmd, Stream::stream_type st, time_t timeout, CondorError *errstack, StartCommandCallbackType *callback_fn, void *misc_data, char const *cmd_description, bool raw_protocol, char const *sec_session_id, bool resume_response )
 {
 	// This is a nonblocking version of startCommand.
 	const int nonblocking = true;
@@ -745,7 +759,7 @@ Daemon::startCommand_nonblocking( int cmd, Stream::stream_type st, int timeout,
 }
 
 StartCommandResult
-Daemon::startCommand_nonblocking( int cmd, Sock* sock, int timeout, CondorError *errstack, StartCommandCallbackType *callback_fn, void *misc_data, char const *cmd_description, bool raw_protocol, char const *sec_session_id, bool resume_response )
+Daemon::startCommand_nonblocking( int cmd, Sock* sock, time_t timeout, CondorError *errstack, StartCommandCallbackType *callback_fn, void *misc_data, char const *cmd_description, bool raw_protocol, char const *sec_session_id, bool resume_response )
 {
 	SecMan::StartCommandRequest req;
 	req.m_cmd = cmd;
@@ -760,14 +774,16 @@ Daemon::startCommand_nonblocking( int cmd, Sock* sock, int timeout, CondorError
 	req.m_nonblocking = true;
 	req.m_cmd_description = cmd_description;
 	req.m_sec_session_id = sec_session_id ? sec_session_id : m_sec_session_id.c_str();
+	req.m_preferred_token = m_preferred_token;
 	req.m_owner = m_owner;
 	req.m_methods = m_methods;
+	req.m_force_auth = m_force_auth;
 
 	return startCommand_internal(req, timeout, &_sec_man);
 }
 
 bool
-Daemon::startCommand( int cmd, Sock* sock, int timeout, CondorError *errstack, char const *cmd_description,bool raw_protocol, char const *sec_session_id, bool resume_response )
+Daemon::startCommand( int cmd, Sock* sock, time_t timeout, CondorError *errstack, char const *cmd_description,bool raw_protocol, char const *sec_session_id, bool resume_response )
 {
 	SecMan::StartCommandRequest req;
 	req.m_cmd = cmd;
@@ -782,8 +798,10 @@ Daemon::startCommand( int cmd, Sock* sock, int timeout, CondorError *errstack, c
 	req.m_nonblocking = false;
 	req.m_cmd_description = cmd_description;
 	req.m_sec_session_id = sec_session_id ? sec_session_id : m_sec_session_id.c_str();
+	req.m_preferred_token = m_preferred_token;
 	req.m_owner = m_owner;
 	req.m_methods = m_methods;
+	req.m_force_auth = m_force_auth;
 
 	StartCommandResult rc = startCommand_internal(req, timeout, &_sec_man);
 	switch(rc) {
@@ -801,7 +819,7 @@ Daemon::startCommand( int cmd, Sock* sock, int timeout, CondorError *errstack, c
 }
 
 bool
-Daemon::sendCommand( int cmd, Sock* sock, int sec, CondorError* errstack, char const *cmd_description )
+Daemon::sendCommand( int cmd, Sock* sock, time_t sec, CondorError* errstack, char const *cmd_description )
 {
 	
 	if( ! startCommand( cmd, sock, sec, errstack, cmd_description )) {
@@ -819,7 +837,7 @@ Daemon::sendCommand( int cmd, Sock* sock, int sec, CondorError* errstack, char c
 
 
 bool
-Daemon::sendCommand( int cmd, Stream::stream_type st, int sec, CondorError* errstack, char const *cmd_description )
+Daemon::sendCommand( int cmd, Stream::stream_type st, time_t sec, CondorError* errstack, char const *cmd_description )
 {
 	Sock* tmp = startCommand( cmd, st, sec, errstack, cmd_description );
 	if( ! tmp ) {
@@ -840,7 +858,7 @@ Daemon::sendCommand( int cmd, Stream::stream_type st, int sec, CondorError* errs
 
 bool
 Daemon::sendCACmd( ClassAd* req, ClassAd* reply, bool force_auth,
-				   int timeout, char const *sec_session_id )
+				   time_t timeout, char const *sec_session_id )
 {
 	ReliSock cmd_sock;
 	return sendCACmd( req, reply, &cmd_sock, force_auth, timeout, sec_session_id );
@@ -849,7 +867,7 @@ Daemon::sendCACmd( ClassAd* req, ClassAd* reply, bool force_auth,
 
 bool
 Daemon::sendCACmd( ClassAd* req, ClassAd* reply, ReliSock* cmd_sock,
-				   bool force_auth, int timeout, char const *sec_session_id )
+				   bool force_auth, time_t timeout, char const *sec_session_id )
 {
 	if( !req ) {
 		newError( CA_INVALID_REQUEST,
@@ -907,7 +925,7 @@ Daemon::sendCACmd( ClassAd* req, ClassAd* reply, ReliSock* cmd_sock,
 			err_msg += "CA_AUTH_CMD";
 		}
 		err_msg += "): ";
-		err_msg += errstack.getFullText().c_str();
+		err_msg += errstack.getFullText();
 		newError( CA_COMMUNICATION_ERROR, err_msg.c_str() );
 		return false;
 	}
@@ -940,7 +958,8 @@ Daemon::sendCACmd( ClassAd* req, ClassAd* reply, ReliSock* cmd_sock,
 		// Now, try to get the reply
 	cmd_sock->decode();
 	if( ! getClassAd(cmd_sock, *reply) ) {
-		newError( CA_COMMUNICATION_ERROR, "Failed to read reply ClassAd" );
+		CAResult caresult = cmd_sock->is_closed() ? CA_REPLY_COMMUNICATION_ERROR : CA_REPLY_TIMED_OUT;
+		newError( caresult, "Failed to read reply ClassAd" );
 		return false;
 	}
 	if( !cmd_sock->end_of_message() ) {
@@ -1025,6 +1044,7 @@ Daemon::locate( Daemon::LocateType method )
 		}
 	}
 	_tried_locate = true;
+	bool query_collector = _locate_local_only ? false : true;
 
 		// First call a subsystem-specific helper to get everything we
 		// have to.  What we do is mostly different between regular
@@ -1037,23 +1057,23 @@ Daemon::locate( Daemon::LocateType method )
 		rval = true;
 		break;
 	case DT_GENERIC:
-		rval = getDaemonInfo( GENERIC_AD, true, method );
+		rval = getDaemonInfo( GENERIC_AD, query_collector, method );
 		break;
 	case DT_CLUSTER:
 		setSubsystem( "CLUSTER" );
-		rval = getDaemonInfo( CLUSTER_AD, true, method );
+		rval = getDaemonInfo( CLUSTER_AD, query_collector, method );
 		break;
 	case DT_SCHEDD:
 		setSubsystem( "SCHEDD" );
-		rval = getDaemonInfo( SCHEDD_AD, true, method );
+		rval = getDaemonInfo( SCHEDD_AD, query_collector, method );
 		break;
 	case DT_STARTD:
 		setSubsystem( "STARTD" );
-		rval = getDaemonInfo( STARTD_AD, true, method );
+		rval = getDaemonInfo( STARTD_AD, query_collector, method );
 		break;
 	case DT_MASTER:
 		setSubsystem( "MASTER" );
-		rval = getDaemonInfo( MASTER_AD, true, method );
+		rval = getDaemonInfo( MASTER_AD, query_collector, method );
 		break;
 	case DT_COLLECTOR:
 		do {
@@ -1062,11 +1082,11 @@ Daemon::locate( Daemon::LocateType method )
 		break;
 	case DT_NEGOTIATOR:
 	  	setSubsystem( "NEGOTIATOR" );
-		rval = getDaemonInfo ( NEGOTIATOR_AD, true, method );
+		rval = getDaemonInfo ( NEGOTIATOR_AD, query_collector, method );
 		break;
 	case DT_CREDD:
 	  setSubsystem( "CREDD" );
-	  rval = getDaemonInfo( CREDD_AD, true, method );
+	  rval = getDaemonInfo( CREDD_AD, query_collector, method );
 	  break;
 	case DT_VIEW_COLLECTOR:
 		if( (rval = getCmInfo("CONDOR_VIEW")) ) {
@@ -1079,17 +1099,17 @@ Daemon::locate( Daemon::LocateType method )
 			rval = getCmInfo( "COLLECTOR" );
 		} while (rval == false && nextValidCm() == true);
 		break;
-	case DT_TRANSFERD:
-		setSubsystem( "TRANSFERD" );
-		rval = getDaemonInfo( ANY_AD, true, method );
+	case DT_PLACEMENTD:
+		setSubsystem( "PLACEMENTD" );
+		rval = getDaemonInfo( PLACEMENTD_AD, query_collector, method );
 		break;
 	case DT_HAD:
 		setSubsystem( "HAD" );
-		rval = getDaemonInfo( HAD_AD, true, method );
+		rval = getDaemonInfo( HAD_AD, query_collector, method );
 		break;
 	case DT_KBDD:
 		setSubsystem( "KBDD" );
-		rval = getDaemonInfo( NO_AD, true, method );
+		rval = getDaemonInfo( NO_AD, query_collector, method );
 		break;
 	default:
 		EXCEPT( "Unknown daemon type (%d) in Daemon::locate", (int)_type );
@@ -1139,14 +1159,10 @@ bool
 Daemon::getDaemonInfo( AdTypes adtype, bool query_collector, LocateType method )
 {
 	std::string			buf;
-	char				*tmp, *my_name;
+	char				*my_name;
 	char				*host = NULL;
 	bool				nameHasPort = false;
-
-	if ( _subsys.empty() ) {
-		dprintf( D_ALWAYS, "Unable to get daemon information because no subsystem specified\n");
-		return false;
-	}
+	bool				usedHostKnob = false;
 
 	if( ! _addr.empty() && is_valid_sinful(_addr.c_str()) ) {
 		dprintf( D_HOSTNAME, "Already have address, no info to locate\n" );
@@ -1154,6 +1170,11 @@ Daemon::getDaemonInfo( AdTypes adtype, bool query_collector, LocateType method )
 		return true;
 	}
 
+	if ( _subsys.empty() ) {
+		dprintf( D_ALWAYS, "Unable to get daemon information because no subsystem specified\n");
+		return false;
+	}
+
 		// If we were not passed a name or an addr, check the
 		// config file for a subsystem_HOST, e.g. SCHEDD_HOST=XXXX
 	if( _name.empty()  && _pool.empty() ) {
@@ -1163,6 +1184,7 @@ Daemon::getDaemonInfo( AdTypes adtype, bool query_collector, LocateType method )
 			dprintf( D_HOSTNAME, 
 					 "No name given, but %s defined to \"%s\"\n",
 					 buf.c_str(), _name.c_str() );
+			usedHostKnob = true;
 		}
 	}
 	if( ! _name.empty() ) {
@@ -1217,17 +1239,17 @@ Daemon::getDaemonInfo( AdTypes adtype, bool query_collector, LocateType method )
 			Set_addr(buf);
 		}
 
-		if (host) free( host );
+		free( host );
 		_is_local = false;
 		return true;
 
 		// Figure out if we want to find a local daemon or not, and
 		// fill in the various hostname fields.
-	} else if( ! _name.empty() ) {
+	} else if( ! _name.empty() && query_collector) {
 			// We were passed a name, so try to look it up in DNS to
 			// get the full hostname.
 
-		tmp = get_daemon_name( _name.c_str() );
+		auto_free_ptr tmp(get_daemon_name( _name.c_str() ));
 		if( ! tmp ) {
 				// we failed to contruct the daemon name.  the only
 				// possible reason for this is being given faulty
@@ -1241,10 +1263,9 @@ Daemon::getDaemonInfo( AdTypes adtype, bool query_collector, LocateType method )
 			// name (and the full hostname, since that's just the
 			// "host part" of the "name"...
 		_alias = get_host_part(_name.c_str());
-		_name = tmp;
-		dprintf( D_HOSTNAME, "Using \"%s\" for name in Daemon object\n",
-				 tmp );
-		free(tmp);
+		_name = tmp.ptr();
+		dprintf( D_HOSTNAME, "Using \"%s\" for name in Daemon object\n", _name.c_str());
+
 			// now, grab the fullhost from the name we just made...
 		_full_hostname = get_host_part(_name.c_str());
 		dprintf( D_HOSTNAME,
@@ -1254,8 +1275,12 @@ Daemon::getDaemonInfo( AdTypes adtype, bool query_collector, LocateType method )
 			// Now that we got this far and have the correct name, see
 			// if that matches the name for the local daemon.  
 			// If we were given a pool, never assume we're local --
-			// always try to query that pool...
-		if( ! _pool.empty() ) {
+			// always try to query that pool..., unless an address file was
+			// also specified, then first look in the address file.
+		if (specified_address_file()) {
+			_is_local = true; // try the address file first
+			dprintf( D_HOSTNAME, "Address file was specified, will try that first\n" );
+		} else if( ! _pool.empty() ) {
 			dprintf( D_HOSTNAME, "Pool was specified, "
 					 "forcing collector query\n" );
 		} else {
@@ -1276,25 +1301,33 @@ Daemon::getDaemonInfo( AdTypes adtype, bool query_collector, LocateType method )
 			// we'll still query the collector even if we don't have the 
             // name
 		_is_local = true;
-		tmp = localName();
-		_name = tmp;
-		free(tmp);
-		_full_hostname = get_local_fqdn();
-		dprintf( D_HOSTNAME, "Neither name nor addr specified, using local "
-				 "values - name: \"%s\", full host: \"%s\"\n", 
-				 _name.c_str(), _full_hostname.c_str() );
+		if ( ! usedHostKnob) {
+			auto_free_ptr tmp(localName());
+			if (tmp) {
+				_name = tmp.ptr();
+				_full_hostname = get_local_fqdn();
+				dprintf( D_HOSTNAME, "Neither name nor addr specified, using local "
+						 "values - name: \"%s\", full host: \"%s\"\n", 
+						 _name.c_str(), _full_hostname.c_str() );
+			}
+		}
 	}
 
 		// Now that we have the real, full names, actually find the
 		// address of the daemon in question.
 
 	if( _is_local ) {
-		bool foundLocalAd = readLocalClassAd( _subsys.c_str() );
-		// need to read the address file if we failed to
-		// find a local ad, or if we desire to use the super port
-		// (because the super port info is not included in the local ad)
-		if(!foundLocalAd || useSuperPort()) {
-			readAddressFile( _subsys.c_str() );
+		// need to read the address file if an address file was specified
+		// or if we desire to use the super port. (super port info is not included in the local ad)
+		if (specified_address_file() || useSuperPort()) {
+			readAddressFile(_subsys.c_str());
+		} else {
+			// we get more info from a local classad, so try that first
+			bool foundLocalAd = readLocalClassAd( _subsys.c_str() );
+			// no local ad, so go ahead an look for an address file.
+			if ( ! foundLocalAd) {
+				readAddressFile( _subsys.c_str() );
+			}
 		}
 	}
 
@@ -1356,7 +1389,7 @@ Daemon::getDaemonInfo( AdTypes adtype, bool query_collector, LocateType method )
 		}
 
 		if (method == LOCATE_FOR_ADMIN) {
-			query.addExtraAttribute(ATTR_SEND_PRIVATE_ATTRIBUTES, "true");
+			query.requestPrivateAttrs();
 		}
 
 			// We need to query the collector
@@ -1383,12 +1416,12 @@ Daemon::getDaemonInfo( AdTypes adtype, bool query_collector, LocateType method )
 		if ( ! getInfoFromAd( scan ) ) {
 			return false;
 		}
-		if( !m_daemon_ad_ptr) {
+		if( m_daemon_ad.empty()) {
 			// I don't think we can ever get into a case where we already
 			// have located the daemon and have a copy of its ad, but just
 			// in case, don't stash another copy of it if we can't find it.
 			// I hope this is a deep copy wiht no chaining bullshit
-			m_daemon_ad_ptr = new ClassAd(*scan);	
+			m_daemon_ad = *scan;
 		}
 			// The version and platfrom aren't critical, so don't
 			// return failure if we can't find them...
@@ -1728,7 +1761,7 @@ Daemon::initVersion( void )
 		// If we didn't find the version string via locate(), and
 		// we're a local daemon, try to ident the daemon's binary
 		// directly. 
-	if( _version.empty() && _is_local ) {
+	if( _version.empty() && _is_local && !_subsys.empty()) {
 		dprintf( D_HOSTNAME, "No version string in local address file, "
 				 "trying to find it in the daemon's binary\n" );
 		char* exe_file = param( _subsys.c_str() );
@@ -1819,7 +1852,7 @@ Daemon::useSuperPort()
 bool
 Daemon::readAddressFile( const char* subsys )
 {
-	char* addr_file = NULL;
+	auto_free_ptr addr_file;
 	FILE* addr_fp;
 	std::string param_name;
 	std::string buf;
@@ -1830,12 +1863,17 @@ Daemon::readAddressFile( const char* subsys )
 	{
 		formatstr( param_name, "%s_SUPER_ADDRESS_FILE", subsys );
 		use_superuser = true;
-		addr_file = param( param_name.c_str() );
+		addr_file.set(param(param_name.c_str()));
+	}
+	if ( ! _specified_address_file.empty()) {
+		use_superuser = false;
+		addr_file.set(strdup(_specified_address_file.c_str()));
+		param_name = "specified address file";
 	}
 	if ( ! addr_file ) {
 		formatstr( param_name, "%s_ADDRESS_FILE", subsys );
 		use_superuser = false;
-		addr_file = param( param_name.c_str() );
+		addr_file.set(param(param_name.c_str()));
 		if( ! addr_file ) {
 			return false;
 		}
@@ -1843,51 +1881,122 @@ Daemon::readAddressFile( const char* subsys )
 
 	dprintf( D_HOSTNAME, "Finding %s address for local daemon, "
 			 "%s is \"%s\"\n", use_superuser ? "superuser" : "local",
-			 param_name.c_str(), addr_file );
+			 param_name.c_str(), addr_file.ptr() );
 
 	if( ! (addr_fp = safe_fopen_wrapper_follow(addr_file, "r")) ) {
 		dprintf( D_HOSTNAME,
 				 "Failed to open address file %s: %s (errno %d)\n",
-				 addr_file, strerror(errno), errno );
-		free( addr_file );
+				 addr_file.ptr(), strerror(errno), errno );
 		return false;
 	}
-		// now that we've got a FILE*, we should free this so we don't
-		// leak it.
-	free( addr_file );
-	addr_file = NULL;
+
+	// how to interpret the first lines of the address file when it is
+	// not all key value pairs.
+	std::string addr;
+	struct _meaning { const char * attr; std::string & val; } line_meaning[] = {
+		{ ATTR_MY_ADDRESS, addr },
+		{ ATTR_VERSION, _version },
+		{ ATTR_PLATFORM, _platform },
+		{ nullptr, addr }, // nullptr is a sintinel value for end-of-meaning table
+	};
+	auto * meaning = &line_meaning[0];
+
+	int lines_of_data = 0;
+	bool file_has_keys = false;
+
+	// read lines from the file until we get one that isn't valid
+	// or we get to one we cannot interpret, if line is key=value
+	// just add it to the daemon ad, if not, interpret it as a string
+	// value for the next "meaning" line until we run out of meanings
+	while (readLine(buf, addr_fp)) {
+		trim(buf); // trim removes trailing \r\n along with other whitespace, so no need to chomp(buf);
+		if (buf.empty()) continue;
+		if (buf.front() == '#') continue;
+
+		// At the start of the file, we don't know for sure whether we are parsing key=val or raw lines
+		// but we do know that the first key=value must be a string value, so we can detect whether
+		// the first line is a raw sinful or a MyAddress="<sinful>" and then act appropriately
+		std::string attr;
+		const char * rhs = nullptr;
+		if (SplitLongFormAttrValue(buf.c_str(), attr, rhs)
+			&& (file_has_keys || (*rhs == '"' && IsValidAttrName(attr.c_str())))) {
+			if ( ! m_location_ad.AssignExpr(attr, rhs)) {
+				dprintf( D_HOSTNAME, "Could not parse %s=%s from address file %s\n", attr.c_str(), rhs, addr_file.ptr());
+				break;
+			}
+			file_has_keys = true; // require that the remainder of the file is classad key=value pairs
+		} else if (meaning->attr) {
+			if (0 == lines_of_data) {
+				// the address (first) field needs special handling, it is the only required field
+				// and it must be a sinful address. or the whole file is invalid
+				if ( ! is_valid_sinful(buf.c_str())) {
+					break;
+				}
+			}
+			meaning->val = buf;
+			m_location_ad.Assign(meaning->attr, meaning->val);
+			++meaning;
+		} else {
+			// don't know how to interpret this line so quit.
+			break;
+		}
+		++lines_of_data;
+	}
 
 		// Read out the sinful string.
-	if( ! readLine(buf, addr_fp) ) {
+	if ( ! lines_of_data) {
 		dprintf( D_HOSTNAME, "address file contained no data\n" );
 		fclose( addr_fp );
 		return false;
 	}
-	chomp(buf);
-	if( is_valid_sinful(buf.c_str()) ) {
-		dprintf( D_HOSTNAME, "Found valid address \"%s\" in "
-				 "%s address file\n", buf.c_str(), use_superuser ? "superuser" : "local" );
-		Set_addr(buf);
-		rval = true;
-	}
 
-		// Let's see if this is new enough to also have a
-		// version string and platform string...
-	if( readLine(buf, addr_fp) ) {
-			// chop off the newline
-		chomp(buf);
-		_version = buf;
-		dprintf( D_HOSTNAME,
-				 "Found version string \"%s\" in address file\n",
-				 buf.c_str() );
-		if( readLine(buf, addr_fp) ) {
-			chomp(buf);
-			_platform = buf;
-			dprintf( D_HOSTNAME,
-					 "Found platform string \"%s\" in address file\n",
-					 buf.c_str() );
+	// if the file had keys, populate some stuff from the location ad into this object
+	if (file_has_keys) {
+		if (addr.empty()) m_location_ad.LookupString(ATTR_MY_ADDRESS, addr);
+		if (_version.empty()) m_location_ad.LookupString(ATTR_VERSION, _version);
+		if (_platform.empty()) m_location_ad.LookupString(ATTR_PLATFORM, _platform);
+		if (_name.empty()) m_location_ad.LookupString(ATTR_NAME, _name);
+		if (m_location_ad.LookupString(ATTR_MACHINE, _full_hostname)) {
+			initHostnameFromFull();
+			_tried_init_hostname = false;
 		}
 	}
+
+	// make sure the location ad is fully populated for the case when the address file
+	// did not have a Name= or Machine= value
+	if ( ! _name.empty() && ! m_location_ad.Lookup(ATTR_NAME)) {
+		m_location_ad.InsertAttr(ATTR_NAME, _name);
+	}
+	if ( ! _full_hostname.empty() && ! m_location_ad.Lookup(ATTR_MACHINE)) {
+		m_location_ad.InsertAttr(ATTR_MACHINE, _full_hostname);
+	}
+	AdTypes ad_type;
+	if (convert_daemon_type_to_ad_type(_type, ad_type)) {
+		const char * mytype = AdTypeToString(ad_type);
+		if (mytype) m_location_ad.InsertAttr(ATTR_MY_TYPE, mytype);
+	}
+
+	if ( ! addr.empty()) {
+		dprintf( D_HOSTNAME, "Found valid address \"%s\" in %s address file for %s %s\n",
+			addr.c_str(), use_superuser ? "superuser" : "local",
+			subsys, _name.c_str());
+		Set_addr(addr);
+		rval = true;
+	}
+	if ( ! _version.empty()) {
+		dprintf( D_HOSTNAME,
+			"Found version string \"%s\" in address file\n",
+			_version.c_str() );
+	}
+
+	/* does anyone care?
+	if ( ! _platform.empty()) {
+		dprintf( D_HOSTNAME,
+			"Found platform string \"%s\" in address file\n",
+			buf.c_str() );
+	}
+	*/
+
 	fclose( addr_fp );
 	return rval;
 }
@@ -1925,8 +2034,8 @@ Daemon::readLocalClassAd( const char* subsys )
 	adFromFile = new ClassAd;
 	InsertFromFile(addr_fp, *adFromFile, "...", adIsEOF, errorReadingAd, adEmpty);
 	ASSERT(adFromFile);
-	if(!m_daemon_ad_ptr) {
-		m_daemon_ad_ptr = new ClassAd(*adFromFile);
+	if(m_daemon_ad.empty()) {
+		m_daemon_ad = *adFromFile;
 	}
 	std::unique_ptr<ClassAd> smart_ad_ptr(adFromFile);
 	
@@ -1964,13 +2073,18 @@ Daemon::getInfoFromAd( const ClassAd* ad )
 	initStringFromAd( ad, ATTR_NAME, _name );
 
 		// construct the IP_ADDR attribute
-	formatstr( buf, "%sIpAddr", _subsys.c_str() );
-	if ( ad->LookupString( buf, buf2 ) ) {
-		Set_addr(buf2);
-		found_addr = true;
-		addr_attr_name = buf;
+		// lookup the  <Daemon>IpAddr first, it will likely fail
+		// but when we are using the Schedd ad to get the address of the Credd
+		// we *must* look at CreddIpAddr before MyAddress to get the right answer
+	if (!_subsys.empty()) {
+		formatstr(buf, "%sIpAddr", _subsys.c_str());
+		if ( ad->LookupString(buf, buf2) ) {
+			Set_addr(buf2);
+			found_addr = true;
+			addr_attr_name = buf;
+		}
 	}
-	else if ( ad->LookupString( ATTR_MY_ADDRESS, buf2 ) ) {
+	if ( !found_addr && ad->LookupString(ATTR_MY_ADDRESS, buf2) ) {
 		Set_addr(buf2);
 		found_addr = true;
 		addr_attr_name = ATTR_MY_ADDRESS;
@@ -2478,9 +2592,9 @@ Daemon::getSessionToken( const std::vector<std::string> &authz_bounding_limit, i
 
 	classad::ClassAd result_ad;
 	if (!getClassAd(&rSock, result_ad)) {
-		if (err) err->pushf("DAEMON", 1, "Failed to recieve response from remote daemon at"
+		if (err) err->pushf("DAEMON", 1, "Failed to receive response from remote daemon at"
 			" at '%s'\n", _addr.c_str() );
-		dprintf(D_FULLDEBUG, "Daemon::getSessionToken() failed to recieve response from "
+		dprintf(D_FULLDEBUG, "Daemon::getSessionToken() failed to receive response from "
 			"remote daemon at '%s'\n", _addr.c_str());
 		return false;
 	}
@@ -2567,9 +2681,9 @@ Daemon::exchangeSciToken(const std::string &scitoken, std::string &token, Condor
 
 	classad::ClassAd result_ad;
 	if (!getClassAd(&rSock, result_ad)) {
-		err.pushf("DAEMON", 1, "Failed to recieve response from remote daemon at"
+		err.pushf("DAEMON", 1, "Failed to receive response from remote daemon at"
 			" at '%s'\n", _addr.c_str());
-		dprintf(D_FULLDEBUG, "Daemon::exchangeSciToken() failed to recieve response from "
+		dprintf(D_FULLDEBUG, "Daemon::exchangeSciToken() failed to receive response from "
 			"remote daemon at '%s'\n", _addr.c_str());
 		return false;
 	}
@@ -2709,9 +2823,9 @@ Daemon::startTokenRequest( const std::string &identity,
 
 	classad::ClassAd result_ad;
 	if (!getClassAd(&rSock, result_ad)) {
-		if (err) { err->pushf("DAEMON", 1, "Failed to recieve "
+		if (err) { err->pushf("DAEMON", 1, "Failed to receive "
 			"response from remote daemon at at '%s'", _addr.c_str()); }
-		dprintf(D_FULLDEBUG, "Daemon::startTokenRequest() failed to recieve "
+		dprintf(D_FULLDEBUG, "Daemon::startTokenRequest() failed to receive "
 			"response from remote daemon at '%s'\n", _addr.c_str());
 		return false;
 	}
@@ -2812,7 +2926,7 @@ Daemon::finishTokenRequest(const std::string &client_id, const std::string &requ
 		if (err) { err->pushf("DAEMON", 1, "Failed to recieve "
 			"response from remote daemon at '%s'",
 			_addr.c_str()); }
-		dprintf(D_FULLDEBUG, "Daemon::finishTokenRequest() failed to recieve "
+		dprintf(D_FULLDEBUG, "Daemon::finishTokenRequest() failed to receive "
 			"response from remote daemon at '%s'\n",
 			_addr.c_str());
 		return false;
@@ -3008,7 +3122,7 @@ Daemon::approveTokenRequest( const std::string &client_id, const std::string &re
 		if (err) { err->pushf("DAEMON", 1, "Failed to recieve "
 			"response from remote daemon at '%s'\n",
 			_addr.c_str()); }
-		dprintf(D_FULLDEBUG, "Daemon::approveTokenRequest() failed to recieve "
+		dprintf(D_FULLDEBUG, "Daemon::approveTokenRequest() failed to receive "
 			"response from remote daemon at '%s'\n",
 			_addr.c_str());
 		return false;
@@ -3115,9 +3229,9 @@ Daemon::autoApproveTokens( const std::string &netblock, time_t lifetime,
 
 	classad::ClassAd result_ad;
 	if (!getClassAd(&rSock, result_ad)) {
-		if (err) err->pushf("DAEMON", 1, "Failed to recieve response from remote daemon at"
+		if (err) err->pushf("DAEMON", 1, "Failed to receive response from remote daemon at"
 			" at '%s'\n", _addr.c_str());
-		dprintf(D_FULLDEBUG, "Daemon::autoApproveTokenRequest() failed to recieve response "
+		dprintf(D_FULLDEBUG, "Daemon::autoApproveTokenRequest() failed to receive response "
 			"from remote daemon at '%s'\n", _addr.c_str());
 		return false;
 	}
