@@ -133,6 +133,10 @@ public:
 		*/
 	enum LocateType {LOCATE_FULL, LOCATE_FOR_LOOKUP, LOCATE_FOR_ADMIN};
 	virtual bool locate( LocateType method=LOCATE_FOR_LOOKUP );
+	bool locate_local( LocateType method=LOCATE_FOR_LOOKUP ) {
+		_locate_local_only = true; // tell locate not to query the collector, better to fail instead.
+		return locate(method);
+	}
 
 		/** Return the error string.  If there's ever a problem
 		  enountered in the Daemon object, this will start returning a
@@ -246,6 +250,21 @@ public:
 		  */
 	bool isLocal( void ) const			{ return _is_local; }
 
+		/** Specify the path to the address file, which will be used
+		*   if locate decides to use an address file and if it exists
+		*/
+	void specify_address_file(std::string_view address_file) {
+		if (address_file.empty()) { _specified_address_file.clear(); }
+		else {_specified_address_file = address_file;}
+	}
+
+		/** return the specified address file, or nullptr if one was not specified.
+		*/
+	const char * specified_address_file() const {
+		if (_specified_address_file.empty()) return nullptr;
+		return _specified_address_file.c_str();
+	}
+
 		/** Returns a descriptive string for error messages.  This has
 		  all the logic about printing out an appropriate string to
 		  describe the daemon, it's type, and it's location.  For
@@ -275,7 +294,7 @@ public:
 		  @param deadline Time at which to give up (0 if never).
 		  @return A new ReliSock object connected to the daemon.  
 		  */
-	ReliSock* reliSock( int timeout = 0, time_t deadline = 0,
+	ReliSock* reliSock( time_t timeout = 0, time_t deadline = 0,
 						CondorError* errstack = 0, bool non_blocking = false,
 						bool ignore_timeout_multiplier = false );
 
@@ -287,7 +306,7 @@ public:
 		  @param deadline Time at which to give up (0 if never).
 		  @return A new SafeSock object connected to the daemon.  
 		  */
-	SafeSock* safeSock( int timeout = 0, time_t deadline = 0,
+	SafeSock* safeSock( time_t timeout = 0, time_t deadline = 0,
 						CondorError* errstack = 0, bool non_blocking = false );
 
 public:
@@ -300,7 +319,7 @@ public:
 		  @return A new Sock object connected to the daemon.  
 		  */
 	Sock *makeConnectedSocket( Stream::stream_type st = Stream::reli_sock,
-							   int timeout = 0, time_t deadline = 0,
+							   time_t timeout = 0, time_t deadline = 0,
 							   CondorError* errstack = NULL,
 							   bool non_blocking = false );
 
@@ -312,7 +331,7 @@ public:
 		             (If 0, then uses timeout already set on socket, if any.)
 		  @return true if connection attempt successful
 		  */
-	bool connectSock(Sock *sock, int sec=0, CondorError* errstack=NULL, bool non_blocking=false, bool ignore_timeout_multiplier=false );
+	bool connectSock(Sock *sock, time_t sec=0, CondorError* errstack=NULL, bool non_blocking=false, bool ignore_timeout_multiplier=false );
 
 		/** Send the given command to the daemon.  The caller gives
 		  the command they want to send, the type of Sock they
@@ -327,7 +346,7 @@ public:
 		  */
 	bool sendCommand( int cmd, 
 					   Stream::stream_type st = Stream::reli_sock,
-					   int sec = 0, CondorError* errstack = NULL,
+					   time_t sec = 0, CondorError* errstack = NULL,
 					  char const *cmd_description=NULL );
 	
 		/** Send the given command to the daemon.  The caller gives
@@ -342,7 +361,7 @@ public:
 		  @param sec The timeout you want to use on your Sock.
 		  @return Success or failure.
 		  */
-	bool sendCommand( int cmd, Sock* sock, int sec = 0, CondorError* errstack = NULL, char const *cmd_description=NULL );
+	bool sendCommand( int cmd, Sock* sock, time_t sec = 0, CondorError* errstack = NULL, char const *cmd_description=NULL );
 
 		/** Start sending the given command to the daemon.  The caller
 		  gives the command they want to send, and the type of Sock
@@ -362,7 +381,7 @@ public:
 		  */
 	Sock* startCommand( int cmd, 
 				Stream::stream_type st = Stream::reli_sock,
-				int sec = 0, CondorError* errstack = NULL,
+				time_t sec = 0, CondorError* errstack = NULL,
 				char const *cmd_description = NULL,
 				bool raw_protocol=false, char const *sec_session_id=NULL,
 				bool resume_response=true );
@@ -387,7 +406,7 @@ public:
 		  */
 	Sock* startSubCommand( int cmd, int subcmd,
 				Stream::stream_type st = Stream::reli_sock,
-				int sec = 0, CondorError* errstack = NULL,
+				time_t sec = 0, CondorError* errstack = NULL,
 				char const *cmd_description = NULL,
 				bool raw_protocol=false, char const *sec_session_id=NULL,
 				bool resume_response=true);
@@ -407,7 +426,7 @@ public:
 		  @return false on error, true on success.
 		*/
 	bool startCommand( int cmd, Sock* sock,
-			int sec = 0, CondorError* errstack = NULL,
+			time_t sec = 0, CondorError* errstack = NULL,
 			char const *cmd_description=NULL,
 			bool raw_protocol=false, char const *sec_session_id=NULL,
 			bool resume_response=true);
@@ -428,7 +447,7 @@ public:
 		  @return false on error, true on success.
 		*/
 	bool startSubCommand( int cmd, int subcmd, Sock* sock,
-			int sec = 0, CondorError* errstack = NULL,
+			time_t sec = 0, CondorError* errstack = NULL,
 			char const *cmd_description=NULL,
 			bool raw_protocol=false, char const *sec_session_id=NULL,
 			bool resume_response=true );
@@ -462,7 +481,7 @@ public:
 			@param sec_session_id use specified session if available
 			@return see definition of StartCommandResult enumeration.
 		  */
-	StartCommandResult startCommand_nonblocking( int cmd, Stream::stream_type st, int timeout, CondorError *errstack, StartCommandCallbackType *callback_fn, void *misc_data, char const *cmd_description=NULL, bool raw_protocol=false, char const *sec_session_id=NULL, bool resume_response=true );
+	StartCommandResult startCommand_nonblocking( int cmd, Stream::stream_type st, time_t timeout, CondorError *errstack, StartCommandCallbackType *callback_fn, void *misc_data, char const *cmd_description=NULL, bool raw_protocol=false, char const *sec_session_id=NULL, bool resume_response=true );
 
 		/** Start sending the given command to the daemon.  This
 			command claims to be nonblocking, but currently it only
@@ -493,7 +512,7 @@ public:
 			@param sec_session_id use specified session if available
 			@return see definition of StartCommandResult enumeration.
 		*/
-	StartCommandResult startCommand_nonblocking( int cmd, Sock* sock, int timeout, CondorError *errstack, StartCommandCallbackType *callback_fn, void *misc_data, char const *cmd_description=NULL, bool raw_protocol=false, char const *sec_session_id=NULL, bool resume_response=true );
+	StartCommandResult startCommand_nonblocking( int cmd, Sock* sock, time_t timeout, CondorError *errstack, StartCommandCallbackType *callback_fn, void *misc_data, char const *cmd_description=NULL, bool raw_protocol=false, char const *sec_session_id=NULL, bool resume_response=true );
 
 		/**
 		 * Asynchronously send a message (command + whatever) to the
@@ -661,17 +680,23 @@ public:
 
 		// Set the owner for this daemon; if possible, always
 		// authenticate with the remote daemon as this owner.
-	void setOwner(const std::string &owner) {m_owner = owner;}
+	void setOwner(const std::string &owner) {m_owner = owner; m_use_new_sec_context_id = true;}
 	const std::string &getOwner() const {return m_owner;}
 
 		// Set the authentication methods to use with this daemon object;
 		// overrides those built-in to the param table.
-	void setAuthenticationMethods(const std::vector<std::string> &methods) {m_methods = methods;}
+	void setAuthenticationMethods(const std::vector<std::string> &methods) {m_methods = methods; m_use_new_sec_context_id = true;}
 	const std::vector<std::string> &getAuthenticationMethods() const {return m_methods;}
 
 	void setSecSessionId(const std::string& sess_id) { m_sec_session_id = sess_id; }
 	const std::string& getSecSessionId() { return m_sec_session_id; }
 
+	void setPreferredToken(const std::string& token) { m_preferred_token = token; m_use_new_sec_context_id = true; }
+	const std::string& getPreferredToken() { return m_preferred_token; }
+
+	void setForceAuthentication(bool force) { m_force_auth = force; }
+	bool getForceAuthentication() { return m_force_auth; }
+
 protected:
 	// Data members
 	std::string _name;
@@ -684,6 +709,7 @@ protected:
 	std::string _platform;
 	std::string _pool;
 	std::string _error;
+	std::string _specified_address_file;
 	CAResult _error_code;
 	std::string _id_str;
 	std::string _subsys;
@@ -695,7 +721,9 @@ protected:
 	bool _tried_init_hostname;
 	bool _tried_init_version;
 	bool _is_configured;
+	bool _locate_local_only{false};
 	bool m_should_try_token_request{false};
+	bool m_force_auth{false};
 	SecMan _sec_man;
 	// If our target daemon is the default collector
 	// (i.e. param COLLECTOR_HOST) and it's a list of collectors,
@@ -877,7 +905,7 @@ protected:
 		   CA_SUCCESS.  Otherwise, true.   
 		*/
 	bool sendCACmd( ClassAd* req, ClassAd* reply, bool force_auth,
-					int timeout = -1, char const *sec_session_id=NULL );
+					time_t timeout = -1, char const *sec_session_id=NULL );
 
 		/** Same as above, except the socket for the command is passed
 			in as an argument.  This way, you can keep the ReliSock
@@ -886,7 +914,7 @@ protected:
 			call the above version.
 		*/
 	bool sendCACmd( ClassAd* req, ClassAd* reply, ReliSock* sock,
-					bool force_auth, int timeout = -1,
+					bool force_auth, time_t timeout = -1,
 					char const *sec_session_id=NULL );
 
 		/** 
@@ -915,7 +943,7 @@ protected:
 		   differentiate between the 6 different variants (besides the
 		   13 argument signature!).
 		 */
-	static StartCommandResult startCommand_internal( const SecMan::StartCommandRequest &req, int timeout, SecMan *sec_man );
+	StartCommandResult startCommand_internal( SecMan::StartCommandRequest &req, time_t timeout, SecMan *sec_man );
 
 		/**
 		   Internal function used by public versions of startCommand().
@@ -923,7 +951,7 @@ protected:
 		   nonblocking flag.  This version creates a socket of the
 		   specified type and connects it.
 		 */
-	StartCommandResult startCommand( int cmd, Stream::stream_type st,Sock **sock,int timeout, CondorError *errstack, int subcmd, StartCommandCallbackType *callback_fn, void *misc_data, bool nonblocking, char const *cmd_description=NULL, bool raw_protocol=false, char const *sec_session_id=NULL, bool resume_response=true );
+	StartCommandResult startCommand( int cmd, Stream::stream_type st,Sock **sock,time_t timeout, CondorError *errstack, int subcmd, StartCommandCallbackType *callback_fn, void *misc_data, bool nonblocking, char const *cmd_description=NULL, bool raw_protocol=false, char const *sec_session_id=NULL, bool resume_response=true );
 
 		/**
 		   Class used internally to handle non-blocking connects for
@@ -932,7 +960,6 @@ protected:
 	friend struct StartCommandConnectCallback;
 	friend class DCMessenger;
 
-	
 private:
 
 	// Note: we want to keep the m_daemon_ad_ptr data member private!
@@ -943,12 +970,16 @@ private:
 	// and folks who try to do something different will get a compile-time error
 	// unless they use DaemonAllowLocateFull::locate().
 
-	ClassAd *m_daemon_ad_ptr;
+	ClassAd m_daemon_ad;
 
-	ClassAd * m_location_ad_ptr = NULL;
+	ClassAd m_location_ad;
 
 	std::string m_trust_domain;
 
+	static int m_next_sec_context_id;
+	int m_sec_context_id{0};
+	bool m_use_new_sec_context_id{false};
+
 		// The virtual 'owner' of this collector object
 	std::string m_owner;
 
@@ -956,6 +987,10 @@ private:
 		// If empty, find/create a session automatically
 	std::string m_sec_session_id;
 
+		// When authenticating with IDTokens, use this token if possible
+		// (i.e. issuer and signing key match the server),
+	std::string m_preferred_token;
+
 		// Authentication method overrides
 	std::vector<std::string> m_methods;
 };
@@ -985,7 +1020,7 @@ public:
 			should be prepared for this method to return NULL.
 			The caller must copy the classad it gets back!
 		  */
-	ClassAd *daemonAd() { locate(LOCATE_FULL); return m_daemon_ad_ptr; }
+	ClassAd *daemonAd() { locate(LOCATE_FULL); return &m_daemon_ad; }
 
 };
 
