@@ -132,8 +132,7 @@ CollectorList::sendUpdates (int cmd, ClassAd * ad1, ClassAd* ad2, bool nonblocki
 	// advance the sequence numbers for these ads
 	//
 	time_t now = time(NULL);
-	DCCollectorAdSeq * seqgen = adSeq->getAdSeq(*ad1);
-	if (seqgen) { seqgen->advance(now); }
+	adSeq->getAdSeq(*ad1).advance(now);
 
 	size_t num_collectors = m_list.size();
 	for (auto& daemon : m_list) {
@@ -143,7 +142,7 @@ CollectorList::sendUpdates (int cmd, ClassAd * ad1, ClassAd* ad2, bool nonblocki
 			continue;
 		}
 
-		if ((num_collectors > 1) && daemon->isBlacklisted()) {
+		if (!nonblocking && (num_collectors > 1) && daemon->isBlacklisted()) {
 			dprintf(D_ALWAYS, "Skipping update to collector %s which has timed out in the past\n", daemon->addr());
 			continue;
 		}
@@ -156,14 +155,14 @@ CollectorList::sendUpdates (int cmd, ClassAd * ad1, ClassAd* ad2, bool nonblocki
 				identity, authz_name);
 		}
 
-		if( num_collectors > 1 ) {
+		if( !nonblocking && num_collectors > 1 ) {
 			daemon->blacklistMonitorQueryStarted();
 		}
 
 		bool success = daemon->sendUpdate(cmd, ad1, *adSeq, ad2, nonblocking,
 			DCTokenRequester::daemonUpdateCallback, data);
 
-		if( num_collectors > 1 ) {
+		if( !nonblocking && num_collectors > 1 ) {
 			daemon->blacklistMonitorQueryFinished(success);
 		}
 
@@ -176,6 +175,18 @@ CollectorList::sendUpdates (int cmd, ClassAd * ad1, ClassAd* ad2, bool nonblocki
 	return success_count;
 }
 
+// pass flag down to the individual DCCollector objects
+void
+CollectorList::allowNewTcpConnections(bool allow)
+{
+	for (auto * dcc : m_list) { if (dcc) dcc->allowNewTcpConnections(allow); }
+}
+
+// pass flag down to the individual DCCollector objects
+void CollectorList::checkVersionBeforeSendingUpdates(bool check) {
+	for (auto * dcc : m_list) { if (dcc) dcc->checkVersionBeforeSendingUpdate(check); }
+}
+
 QueryResult
 CollectorList::query (CondorQuery & cQuery, bool (*callback)(void*, ClassAd *), void* pv, CondorError * errstack) {
 
@@ -219,7 +230,7 @@ CollectorList::query (CondorQuery & cQuery, bool (*callback)(void*, ClassAd *),
 				daemon->blacklistMonitorQueryStarted();
 			}
 
-			result = cQuery.processAds (callback, pv, daemon->addr(), errstack);
+			result = cQuery.processAds (callback, pv, *daemon, errstack);
 
 			if( num_collectors > 1 ) {
 				daemon->blacklistMonitorQueryFinished( result == Q_OK );
