@@ -35,9 +35,11 @@ std::map< std::string, Timeslice > DCCollector::blacklist;
 
 // Instantiate things
 
-DCCollector::DCCollector( const char* dcName, UpdateType uType ) 
+DCCollector::DCCollector( const char* dcName, UpdateType uType )
 	: Daemon( DT_COLLECTOR, dcName, NULL )
 {
+	this->constructorName = dcName;
+
 	up_type = uType;
 	init( true );
 }
@@ -77,11 +79,11 @@ DCCollector&
 DCCollector::operator = ( const DCCollector& copy )
 {
 		// don't copy ourself!
-    if (&copy != this) {
+	if (&copy != this) {
 		deepCopy( copy );
 	}
 
-    return *this;
+	return *this;
 }
 
 
@@ -111,8 +113,8 @@ DCCollector::deepCopy( const DCCollector& copy )
 	up_type = copy.up_type;
 
 	if( update_destination ) {
-        free(update_destination);
-    }
+		free(update_destination);
+	}
 	update_destination = copy.update_destination ? strdup( copy.update_destination ) : NULL;
 
 	startTime = copy.startTime;
@@ -185,7 +187,7 @@ DCCollector::requestScheddToken(const std::string &schedd_name,
 		err.pushf("DAEMON", 1, "Failed to recieve "
 			"response from remote collector at '%s'",
 			_addr.c_str());
-		dprintf(D_FULLDEBUG, "DCCollector::requestScheddToken() failed to recieve "
+		dprintf(D_FULLDEBUG, "DCCollector::requestScheddToken() failed to receive "
 			"response from remote daemon at '%s'\n",
 			_addr.c_str());
 		return false;
@@ -255,7 +257,7 @@ DCCollector::parseTCPInfo( void )
 			free( tmp );
 			if( ! _name.empty() &&
 				contains_anycase_withwildcard(tcp_collectors, _name) )
-			{	
+			{
 				use_tcp = true;
 				break;
 			}
@@ -272,7 +274,7 @@ DCCollector::parseTCPInfo( void )
 	}
 }
 
-// do a >= version check 
+// do a >= version check
 bool DCCollector::checkCachedVersion(int major, int minor, int subminor, bool default_value) {
 	if (_version.empty()) return default_value;
 	return CondorVersionInfo(_version.c_str()).built_since_version(major, minor, subminor);
@@ -561,11 +563,8 @@ public:
 			}
 			dprintf(D_ALWAYS,"Failed to start non-blocking update to %s.\n",who);
 			if (dc_collector) {
-				while (!dc_collector->pending_update_list.empty()) {
-					// UpdateData's dtor removes this from the pending update list
-					delete(dc_collector->pending_update_list.front());
-				}
-				ud = 0;	
+				dc_collector->clearAndNotifyUpdateQueue(sock, trust_domain, should_try_token_request, ud);
+				dc_collector->relocate();
 			}
 		}
 		else if(sock && !DCCollector::finishUpdate(ud->dc_collector,sock,ud->ad1,ud->ad2, ud->m_callback_fn, ud->m_miscdata)) {
@@ -573,11 +572,8 @@ public:
 			if(sock) who = sock->get_sinful_peer();
 			dprintf(D_ALWAYS,"Failed to send non-blocking update to %s.\n",who);
 			if (dc_collector) {
-				while (!dc_collector->pending_update_list.empty()) {
-					// UpdateData's dtor removes this from the pending update list
-					delete(dc_collector->pending_update_list.front());
-				}
-				ud = 0;	
+				dc_collector->clearAndNotifyUpdateQueue(sock, trust_domain, should_try_token_request, ud);
+				dc_collector->relocate();
 			}
 		}
 		else if(sock && sock->type() == Sock::reli_sock) {
@@ -624,6 +620,8 @@ public:
 					dprintf(D_ALWAYS,"Failed to send update to %s.\n",who);
 					delete dc_collector->update_rsock;
 					dc_collector->update_rsock = NULL;
+					dc_collector->relocate();
+
 					// Notice we remove the element from the list of pending updates
 					// even on failure.
 				}
@@ -647,6 +645,26 @@ public:
 	}
 };
 
+
+void
+DCCollector::clearAndNotifyUpdateQueue(Sock* sock, const std::string &trust_domain, bool should_try_token_request, UpdateData*& failed) {
+	for (auto& ud : pending_update_list) {
+		// Run callback function for all except for the failed (handled outside prior)
+		if (ud != failed && ud->m_callback_fn) {
+			(*(ud->m_callback_fn))(false, sock, nullptr, trust_domain, should_try_token_request, ud->m_miscdata);
+		}
+
+		delete ud;
+		ud = nullptr;
+	}
+
+	// The failed UpdateData point will have been deleted so remove dangling pointer
+	failed = nullptr;
+
+	pending_update_list.clear();
+}
+
+
 bool
 DCCollector::sendUDPUpdate( int cmd, ClassAd* ad1, ClassAd* ad2, bool nonblocking, StartCommandCallbackType callback_fn, void *miscdata )
 {
@@ -660,23 +678,16 @@ DCCollector::sendUDPUpdate( int cmd, ClassAd* ad1, ClassAd* ad2, bool nonblockin
 			 "Attempting to send update via UDP to collector %s\n",
 			 update_destination );
 
-	bool raw_protocol = false;
-	if( cmd == UPDATE_COLLECTOR_AD || cmd == INVALIDATE_COLLECTOR_ADS ) {
-			// we *never* want to do security negotiation with the
-			// developer collector.
-		raw_protocol = true;
-	}
-
 	if(nonblocking) {
 		UpdateData *ud = new UpdateData(cmd, Sock::safe_sock, ad1, ad2, this, callback_fn, miscdata);
 		if (this->pending_update_list.size() == 1)
 		{
-			startCommand_nonblocking(cmd, Sock::safe_sock, 20, NULL, UpdateData::startUpdateCallback, ud, NULL, raw_protocol );
+			startCommand_nonblocking(cmd, Sock::safe_sock, 20, NULL, UpdateData::startUpdateCallback, ud);
 		}
 		return true;
 	}
 
-	Sock *ssock = startCommand(cmd, Sock::safe_sock, 20, NULL, NULL, raw_protocol);
+	Sock *ssock = startCommand(cmd, Sock::safe_sock, 20);
 
 	if(!ssock) {
 		newError( CA_COMMUNICATION_ERROR,
@@ -736,11 +747,13 @@ DCCollector::sendTCPUpdate( int cmd, ClassAd* ad1, ClassAd* ad2, bool nonblockin
 		}
 		return true;
 	}
-	dprintf( D_FULLDEBUG, 
+	dprintf( D_FULLDEBUG,
 			 "Couldn't reuse TCP socket to update collector, "
 			 "starting new connection\n" );
 	delete update_rsock;
 	update_rsock = NULL;
+	relocate();
+
 	return initiateTCPUpdate( cmd, ad1, ad2, nonblocking, callback_fn, miscdata );
 }
 
@@ -753,6 +766,10 @@ DCCollector::initiateTCPUpdate( int cmd, ClassAd* ad1, ClassAd* ad2, bool nonblo
 		delete update_rsock;
 		update_rsock = NULL;
 	}
+	if (!new_tcp_connections) {
+		dprintf(D_FULLDEBUG, "Not allowing new TCP connection to collector %s\n", update_destination);
+		return false;
+	}
 	if(nonblocking) {
 		UpdateData *ud = new UpdateData(cmd, Sock::reli_sock, ad1, ad2, this, callback_fn, miscdata);
 			// Note that UpdateData automatically adds itself to the pending_update_list.
@@ -875,10 +892,11 @@ DCCollector::~DCCollector( void )
 Timeslice &DCCollector::getBlacklistTimeslice()
 {
 	std::map< std::string, Timeslice >::iterator itr;
-	itr = blacklist.find(addr());
+	std::string address = (addr() == nullptr) ? "" : addr();
+	itr = blacklist.find(address);
 	if( itr == blacklist.end() ) {
 		Timeslice ts;
-		
+
 			// Blacklist this collector if last failed contact took more
 			// than 1% of the time that has passed since that operation
 			// started.  (i.e. if contact fails quickly, don't worry, but
@@ -889,7 +907,7 @@ Timeslice &DCCollector::getBlacklistTimeslice()
 		ts.setMaxInterval(avoid_time);
 		ts.setInitialInterval(0);
 
-		itr = blacklist.insert( std::map< std::string, Timeslice >::value_type(addr(),ts) ).first;
+		itr = blacklist.emplace(address,ts).first;
 	}
 	return itr->second;
 }
@@ -926,3 +944,24 @@ DCCollector::blacklistMonitorQueryFinished( bool success ) {
 		}
 	}
 }
+
+void
+DCCollector::relocate() {
+	dprintf( D_HOSTNAME, "DCCollector::relocate(%s)\n", this->constructorName.c_str() );
+
+	// This is awful, but easier than trying to unknot the logic that
+	// EXCEPT()s when trying to call locate() a second time.
+	DCCollector self( this->constructorName.c_str(), up_type );
+	self.locate();
+
+	// The assignment operator for DCCollector doesn't call the assignment
+	// operator for Daemon[Client], its superclass, so it doesn't actually
+	// do anything useful.
+	this->theRealDeepCopy(self);
+}
+
+void
+DCCollector::theRealDeepCopy(const DCCollector & other) {
+	Daemon::deepCopy(other);
+	deepCopy(other);
+}
