@@ -33,8 +33,6 @@
 DCStartd::DCStartd( const char* tName, const char* tPool ) 
 	: Daemon( DT_STARTD, tName, tPool )
 {
-	claim_id = NULL;
-	extra_ids = NULL;
 }
 
 
@@ -47,45 +45,27 @@ DCStartd::DCStartd( const char* tName, const char* tPool, const char* tAddr,
 	}
 		// claim_id isn't initialized by Daemon's constructor, so we
 		// have to treat it slightly differently 
-	claim_id = NULL;
 	if( tId ) {
-		claim_id = strdup( tId );
+		claim_id = tId;
 	}
 
-	extra_ids = NULL;
-	if( ids && (strlen(ids) > 0)) {
-		extra_ids = strdup( ids );
+	if( ids ) {
+		extra_ids = ids;
 	}
 }
 
 DCStartd::DCStartd( const ClassAd *ad, const char *tPool )
-	: Daemon(ad,DT_STARTD,tPool),
-	  claim_id(NULL), extra_ids(NULL)
+	: Daemon(ad,DT_STARTD,tPool)
 {
 }
 
-DCStartd::~DCStartd( void )
-{
-	if( claim_id ) {
-		free(claim_id);
-	}
-	if( extra_ids ) {
-		free(extra_ids);
-	}
-}
-
-
 bool
 DCStartd::setClaimId( const char* id ) 
 {
 	if( ! id ) {
 		return false;
 	}
-	if( claim_id ) {
-		free(claim_id);
-		claim_id = NULL;
-	}
-	claim_id = strdup( id );
+	claim_id = id;
 	return true;
 }
 
@@ -314,7 +294,13 @@ ClaimStartdMsg::readMsg( DCMessenger * /*messenger*/, Sock *sock ) {
 
 
 void
-DCStartd::asyncRequestOpportunisticClaim( ClassAd const *req_ad, char const *description, char const *scheduler_addr, int alive_interval, bool claim_pslot, int timeout, int deadline_timeout, classy_counted_ptr<DCMsgCallback> cb )
+DCStartd::asyncRequestOpportunisticClaim(
+	ClassAd const *req_ad,
+	char const *description,
+	char const *scheduler_addr,
+	int alive_interval,
+	requestClaimOptions & opts,
+	int timeout, int deadline_timeout, classy_counted_ptr<DCMsgCallback> cb )
 {
 	dprintf(D_FULLDEBUG|D_PROTOCOL,"Requesting claim %s\n",description);
 
@@ -322,12 +308,12 @@ DCStartd::asyncRequestOpportunisticClaim( ClassAd const *req_ad, char const *des
 	ASSERT( checkClaimId() );
 	ASSERT( checkAddr() );
 
-	classy_counted_ptr<ClaimStartdMsg> msg = new ClaimStartdMsg( claim_id, extra_ids, req_ad, description, scheduler_addr, alive_interval );
+	classy_counted_ptr<ClaimStartdMsg> msg = new ClaimStartdMsg( claim_id.c_str(), extra_ids.c_str(), req_ad, description, scheduler_addr, alive_interval );
 
 	ASSERT( msg.get() );
 	msg->setCallback(cb);
 
-	if (claim_pslot) {
+	if (opts.claim_pslot) {
 		// TODO Currently, we always request the pslot's max lease time
 		//   (msg->m_pslot_claim_lease=0).
 		//   Consider adding option to let client request shorter lease time.
@@ -340,12 +326,14 @@ DCStartd::asyncRequestOpportunisticClaim( ClassAd const *req_ad, char const *des
 	req_ad->LookupString("WorkingCM", working_cm);
 	if (!working_cm.empty()) {
 		msg->m_num_dslots = 0;
+	} else {
+		msg->m_num_dslots = opts.num_dslots;
 	}
 
 	msg->setSuccessDebugLevel(D_ALWAYS|D_PROTOCOL);
 
 		// if this claim is associated with a security session
-	ClaimIdParser cid(claim_id);
+	ClaimIdParser cid(claim_id.c_str());
 	if (param_boolean("SEC_ENABLE_MATCH_PASSWORD_AUTHENTICATION", true) &&
 		cid.secSessionInfo()[0] != '\0')
 	{
@@ -357,103 +345,153 @@ DCStartd::asyncRequestOpportunisticClaim( ClassAd const *req_ad, char const *des
 	sendMsg(msg.get());
 }
 
-
-bool 
-DCStartd::deactivateClaim( bool graceful, bool *claim_is_closing )
+bool
+DCStartd::reactivateClaimCheck(bool & claim_is_closing)
 {
-	dprintf( D_FULLDEBUG, "Entering DCStartd::deactivateClaim(%s)\n",
-			 graceful ? "graceful" : "forceful" );
+	claim_is_closing = false;
+	setCmdStr( "reactivateClaimCheck" );
 
+	return deactivateClaim(REACTIVATE_CLAIM_CHECK, false, &claim_is_closing, nullptr);
+}
+
+bool
+DCStartd::deactivateClaim( bool graceful, bool got_job_done, bool *claim_is_closing, bool *still_cleaning )
+{
 	if( claim_is_closing ) {
 		*claim_is_closing = false;
 	}
+	if( still_cleaning ) {
+		*still_cleaning = false;
+	}
 
 	setCmdStr( "deactivateClaim" );
+
+	int cmd = graceful ? DEACTIVATE_CLAIM : DEACTIVATE_CLAIM_FORCIBLY;
+
+	return deactivateClaim(cmd, got_job_done, claim_is_closing, still_cleaning);
+}
+
+// protected helper for deactivateClaim and reactivateClaimCheck
+bool
+DCStartd::deactivateClaim(int cmd, bool got_job_done, bool *claim_is_closing, bool *still_cleaning)
+{
+	dprintf( D_FULLDEBUG, "Entering DCStartd::%s(%s)\n", _cmd_str.c_str(), got_job_done ? "job_done" : "");
+
 	if( ! checkClaimId() ) {
+		// checkClaimId sets newError
 		return false;
 	}
 	if( ! checkAddr() ) {
+		// checkAddr set newError
 		return false;
 	}
 
-		// if this claim is associated with a security session
-	ClaimIdParser cidp(claim_id);
+	ClaimIdParser cidp(claim_id.c_str());
+
+	if (got_job_done && (cmd != REACTIVATE_CLAIM_CHECK)) {
+		CondorVersionInfo cvi = cidp.secSessionInfoVersion();
+		// we need a newish Startd in order to send DEACTIVATE_CLAIM_JOB_DONE
+		if (cvi.getMajorVer() <= 0) {
+			// dprintf(D_ZKM, "Startd version is not known, will use %s\n", getCommandStringSafe(cmd));
+		} else {
+			if (cvi.built_since_version(24,7,0)) {
+				cmd = DEACTIVATE_CLAIM_JOB_DONE;
+				// dprintf(D_ZKM, "Startd version is known and job_has_exited, will use JOB_DONE\n");
+			}
+		}
+	}
+
+	// if this claim is associated with a security session
 	char const *sec_session = cidp.secSessionId();
 
 	if (IsDebugLevel(D_COMMAND)) {
-		int cmd = graceful ? DEACTIVATE_CLAIM : DEACTIVATE_CLAIM_FORCIBLY;
-		dprintf (D_COMMAND, "DCStartd::deactivateClaim(%s,...) making connection to %s\n", getCommandStringSafe(cmd), _addr.c_str());
+		dprintf (D_COMMAND, "DCStartd::%s(%s,...) making connection to %s\n", _cmd_str.c_str(), getCommandStringSafe(cmd), _addr.c_str());
 	}
 
 	bool  result;
 	ReliSock reli_sock;
 	reli_sock.timeout(20);   // years of research... :)
 	if( ! reli_sock.connect(_addr.c_str()) ) {
-		std::string err = "DCStartd::deactivateClaim: ";
+		std::string err = "DCStartd::" + _cmd_str + ": ";
 		err += "Failed to connect to startd (";
 		err += _addr;
 		err += ')';
 		newError( CA_CONNECT_FAILED, err.c_str() );
 		return false;
 	}
-	int cmd;
-	if( graceful ) {
-		cmd = DEACTIVATE_CLAIM;
-	} else {
-		cmd = DEACTIVATE_CLAIM_FORCIBLY;
-	}
 	result = startCommand( cmd, (Sock*)&reli_sock, 20, NULL, NULL, false, sec_session ); 
 	if( ! result ) {
-		std::string err = "DCStartd::deactivateClaim: ";
+		std::string err = "DCStartd::" + _cmd_str + ": ";
 		err += "Failed to send command ";
-		if( graceful ) {
-			err += "DEACTIVATE_CLAIM";
-		} else {
-			err += "DEACTIVATE_CLAIM_FORCIBLY";
-		}
+		err += getCommandStringSafe(cmd);
 		err += " to the startd";
 		newError( CA_COMMUNICATION_ERROR, err.c_str() );
 		return false;
 	}
 		// Now, send the ClaimId
 	if( ! reli_sock.put_secret(claim_id) ) {
-		newError( CA_COMMUNICATION_ERROR,
-				  "DCStartd::deactivateClaim: Failed to send ClaimId to the startd" );
+		std::string err = "DCStartd::" + _cmd_str + ": Failed to send ClaimId to the startd";
+		newError( CA_COMMUNICATION_ERROR, err.c_str() );
 		return false;
 	}
 	if( ! reli_sock.end_of_message() ) {
-		newError( CA_COMMUNICATION_ERROR,
-				  "DCStartd::deactivateClaim: Failed to send EOM to the startd" );
+		std::string err = "DCStartd::" + _cmd_str + ": Failed to send EOM to the startd";
+		newError( CA_COMMUNICATION_ERROR, err.c_str() );
 		return false;
 	}
 
 	reli_sock.decode();
 	ClassAd response_ad;
 	if( !getClassAd(&reli_sock, response_ad) || !reli_sock.end_of_message() ) {
-		newError(CA_COMMUNICATION_ERROR,
-		         "DCStartd::deactivateClaim: failed to read response ad.");
+		// for now, we assume that the failure must be a timeout if the socket is still open.
+		std::string err = "DCStartd::" + _cmd_str + ": ";
+		if (reli_sock.is_closed()) {
+			err += "failed to read response ad";
+			newError(CA_REPLY_COMMUNICATION_ERROR, err.c_str());
+		} else {
+			err += "timed out waiting for response ad";
+			newError(CA_REPLY_TIMED_OUT, err.c_str());
+		}
+		dprintf(D_ZKM, "failed to get deactivateClaim response (%s)\n", err.c_str());
 		return false;
 	}
 	else {
+		std::string adbuf;
+		dprintf(D_ZKM, "got deactivateClaim response:\n%s\n", formatAd(adbuf, response_ad, "\t"));
 		bool start = true;
 		response_ad.LookupBool(ATTR_START,start);
 		if( claim_is_closing ) {
 			*claim_is_closing = !start;
 		}
+		bool cleaning = false;
+		if (response_ad.LookupBool("StillCleaning", cleaning) && cleaning) {
+			int starter_pid = 0;
+			response_ad.LookupInteger("StarterPID", starter_pid);
+			dprintf(D_COMMAND, "DCStartd::%s(%s) succeeded, %sStarter is still cleaning up (pid=%d)\n",
+				_cmd_str.c_str(), getCommandStringSafe(cmd), start?"":"claim is closing and ", starter_pid);
+		}
+		if( still_cleaning ) {
+			*still_cleaning = cleaning;
+		}
 	}
 
 		// we're done
-	dprintf( D_FULLDEBUG, "DCStartd::deactivateClaim: "
-			 "successfully sent command\n" );
+	dprintf( D_FULLDEBUG, "DCStartd::%s: successfully sent command\n", _cmd_str.c_str() );
 	return true;
 }
 
 
 int
-DCStartd::activateClaim( ClassAd* job_ad, int starter_version,
-						 ReliSock** claim_sock_ptr ) 
+DCStartd::activateClaim( ClassAd* job_ad, ReliSock** claim_sock_ptr, ClassAd * replyAd )
 {
 	int reply;
+	ClassAd dummyAd;
+	bool want_failure_ad = false;
+	// 2 is the vanilla starter, but the Startd ignores this now
+	// Prior to 25.x the STARTD would refuse if the value was >= 10, but otherwise ignored it.
+	int starter_version = 2;
+	const char * ATTR_send_failure_ad = "_condor_send_activation_failure_ad";
+
 	dprintf( D_FULLDEBUG, "Entering DCStartd::activateClaim()\n" );
 
 	setCmdStr( "activateClaim" );
@@ -465,15 +503,20 @@ DCStartd::activateClaim( ClassAd* job_ad, int starter_version,
 			// we'll give them a pointer to the real object.
 		*claim_sock_ptr = NULL;
 	}
+	if (replyAd) {
+		want_failure_ad = true;
+		replyAd->Clear();
+	}
+	else { replyAd = &dummyAd; }
 
-	if( ! claim_id ) {
+	if( claim_id.empty() ) {
 		newError( CA_INVALID_REQUEST,
-				  "DCStartd::activateClaim: called with NULL claim_id, failing" );
+				  "DCStartd::activateClaim: called with empty claim_id, failing" );
 		return CONDOR_ERROR;
 	}
 
 		// if this claim is associated with a security session
-	ClaimIdParser cidp(claim_id);
+	ClaimIdParser cidp(claim_id.c_str());
 	char const *sec_session = cidp.secSessionId();
 
 	Sock* tmp;
@@ -495,12 +538,17 @@ DCStartd::activateClaim( ClassAd* job_ad, int starter_version,
 		delete tmp;
 		return CONDOR_ERROR;
 	}
+
+	if (want_failure_ad) { job_ad->Assign(ATTR_send_failure_ad, true); }
 	if( ! putClassAd(tmp, *job_ad) ) {
+		if (want_failure_ad) { job_ad->Delete(ATTR_send_failure_ad); }
 		newError( CA_COMMUNICATION_ERROR,
 				  "DCStartd::activateClaim: Failed to send job ClassAd to the startd" );
 		delete tmp;
 		return CONDOR_ERROR;
 	}
+	if (want_failure_ad) { job_ad->Delete(ATTR_send_failure_ad); }
+
 	if( ! tmp->end_of_message() ) {
 		newError( CA_COMMUNICATION_ERROR,
 				  "DCStartd::activateClaim: Failed to send EOM to the startd" );
@@ -508,20 +556,26 @@ DCStartd::activateClaim( ClassAd* job_ad, int starter_version,
 		return CONDOR_ERROR;
 	}
 
+
 		// Now, try to get the reply
 	tmp->decode();
-	if( !tmp->code(reply) || !tmp->end_of_message()) {
+	if (tmp->code(reply) &&
+		(tmp->peek_end_of_message() || getClassAd(tmp, *replyAd)) &&
+		tmp->end_of_message())
+	{
+		dprintf( D_FULLDEBUG, "DCStartd::activateClaim: successfully sent command, reply is: %d%s\n",
+			reply, replyAd->size() ? " (with ad)" : "" );
+	} else {
+		// for now, we assume that the failure must be a timeout if the socket is still open.
+		CAResult caresult = ((ReliSock*)tmp)->is_closed() ? CA_REPLY_COMMUNICATION_ERROR : CA_REPLY_TIMED_OUT;
+
 		std::string err = "DCStartd::activateClaim: ";
 		err += "Failed to receive reply from ";
-		err += _addr.c_str();
-		newError( CA_COMMUNICATION_ERROR, err.c_str() );
-		delete tmp;
-		return CONDOR_ERROR;
+		err += _addr;
+		newError( caresult, err.c_str() );
+		reply = CONDOR_ERROR;
 	}
 
-	dprintf( D_FULLDEBUG, "DCStartd::activateClaim: "
-			 "successfully sent command, reply is: %d\n", reply ); 
-
 	if( reply == OK && claim_sock_ptr ) {
 		*claim_sock_ptr = (ReliSock*)tmp;
 	} else {
@@ -631,8 +685,7 @@ DCStartd::resumeClaim( ClassAd* reply, int timeout )
 
 
 bool
-DCStartd::deactivateClaim( VacateType vType, ClassAd* reply,
-						   int timeout )
+DCStartd::deactivateClaimCA( VacateType vType, ClassAd* reply, int timeout )
 {
 	setCmdStr( "deactivateClaim" );
 	if( ! checkClaimId() ) {
@@ -753,14 +806,14 @@ DCStartd::delegateX509Proxy( const char* proxy, time_t expiration_time, time_t *
 
 	setCmdStr( "delegateX509Proxy" );
 
-	if( ! claim_id ) {
+	if( claim_id.empty() ) {
 		newError( CA_INVALID_REQUEST,
-				  "DCStartd::delegateX509Proxy: Called with NULL claim_id" );
+				  "DCStartd::delegateX509Proxy: Called with empty claim_id" );
 		return CONDOR_ERROR;
 	}
 
 		// if this claim is associated with a security session
-	ClaimIdParser cidp(claim_id);
+	ClaimIdParser cidp(claim_id.c_str());
 
 	//
 	// 1) begin the DELEGATE_GSI_CRED_STARTD command
@@ -896,7 +949,7 @@ DCStartd::vacateClaim( const char* name_vacate )
 	result = startCommand( cmd, (Sock*)&reli_sock ); 
 	if( ! result ) {
 		newError( CA_COMMUNICATION_ERROR,
-				  "DCStartd::vacateClaim: Failed to send command PCKPT_JOB to the startd" );
+				  "DCStartd::vacateClaim: Failed to send command VACATE_CLAIM to the startd" );
 		return false;
 	}
 
@@ -927,7 +980,7 @@ DCStartd::_suspendClaim( )
 	}
 
 	// if this claim is associated with a security session
-	ClaimIdParser cidp(claim_id);
+	ClaimIdParser cidp(claim_id.c_str());
 	char const *sec_session = cidp.secSessionId();
 	
 	if (IsDebugLevel(D_COMMAND)) {
@@ -985,7 +1038,7 @@ DCStartd::_continueClaim( )
 	}
 
 	// if this claim is associated with a security session
-	ClaimIdParser cidp(claim_id);
+	ClaimIdParser cidp(claim_id.c_str());
 	char const *sec_session = cidp.secSessionId();
 	
 	if (IsDebugLevel(D_COMMAND)) {
@@ -1031,58 +1084,6 @@ DCStartd::_continueClaim( )
 }
 
 
-bool 
-DCStartd::checkpointJob( const char* name_ckpt )
-{
-	dprintf( D_FULLDEBUG, "Entering DCStartd::checkpointJob(%s)\n",
-			 name_ckpt );
-
-	setCmdStr( "checkpointJob" );
-
-	if (IsDebugLevel(D_COMMAND)) {
-		int cmd = PCKPT_JOB;
-		dprintf (D_COMMAND, "DCStartd::checkpointJob(%s,...) making connection to %s\n", getCommandStringSafe(cmd), _addr.c_str());
-	}
-
-	bool  result;
-	ReliSock reli_sock;
-	reli_sock.timeout(20);   // years of research... :)
-	if( ! reli_sock.connect(_addr.c_str()) ) {
-		std::string err = "DCStartd::checkpointJob: ";
-		err += "Failed to connect to startd (";
-		err += _addr;
-		err += ')';
-		newError( CA_CONNECT_FAILED, err.c_str() );
-		return false;
-	}
-
-	int cmd = PCKPT_JOB;
-
-	result = startCommand( cmd, (Sock*)&reli_sock ); 
-	if( ! result ) {
-		newError( CA_COMMUNICATION_ERROR,
-				  "DCStartd::checkpointJob: Failed to send command PCKPT_JOB to the startd" );
-		return false;
-	}
-
-		// Now, send the name
-	if( ! reli_sock.put(name_ckpt) ) {
-		newError( CA_COMMUNICATION_ERROR,
-				  "DCStartd::checkpointJob: Failed to send Name to the startd" );
-		return false;
-	}
-	if( ! reli_sock.end_of_message() ) {
-		newError( CA_COMMUNICATION_ERROR,
-				  "DCStartd::checkpointJob: Failed to send EOM to the startd" );
-		return false;
-	}
-		// we're done
-	dprintf( D_FULLDEBUG, "DCStartd::checkpointJob: "
-			 "successfully sent command\n" );
-	return true;
-}
-
-
 bool 
 DCStartd::getAds( ClassAdList &adsList )
 {
@@ -1124,7 +1125,7 @@ DCStartd::getAds( ClassAdList &adsList )
 bool
 DCStartd::checkClaimId( void )
 {
-	if( claim_id ) {
+	if( !claim_id.empty() ) {
 		return true;
 	}
 	std::string err_msg;
