@@ -53,9 +53,6 @@ public:
 
 	DCStartd( const ClassAd *ad, const char *pool = NULL );
 
-		/// Destructor.
-	~DCStartd();
-
 		/** Set the ClaimId to use when talking to this startd. 
 			@param id The ClaimID string
 			@return true on success, false on invalid input (NULL)
@@ -66,7 +63,7 @@ public:
 		/** @return the ClaimId string for this startd, NULL if we
 			don't have a value yet.
 		*/
-	const char* getClaimId( void ) { return claim_id; };
+	const char* getClaimId( void ) { return claim_id.c_str(); };
 
 		/** This is the old-style way of requesting a claim, not the
 			"generic ClassAd" way, which currently only supports COD
@@ -87,14 +84,36 @@ public:
 			in the DCStartdMsg object, which may be obtained from the callback
 			object at callback time.
 		*/
-	void asyncRequestOpportunisticClaim( ClassAd const *req_ad, char const *description, char const *scheduler_addr, int alive_interval, bool claim_pslot, int timeout, int deadline_timeout, classy_counted_ptr<DCMsgCallback> cb );
+		struct requestClaimOptions {
+			int num_dslots{1};
+			bool claim_pslot{false};
+			bool send_leftovers{false};
+			// TODO: add resource sharing info here??
+		};
+
+	void asyncRequestOpportunisticClaim( ClassAd const *req_ad, char const *description, char const *scheduler_addr, int alive_interval, requestClaimOptions & opts, int timeout, int deadline_timeout, classy_counted_ptr<DCMsgCallback> cb );
 
 		/** Send the command to this startd to deactivate the claim 
 			@param graceful Should we be graceful or forcful?
+			@param got_job_done do we believe that the job is done? (i.e. got the job_exit syscall)
+			@param claim_is_closing startd indicates if not accepting more jobs
+			@param job_is_restarting  Query-only.  Used by starter to check if
+			       it should restart a job after uploading its checkpoint.
+			@return true on success, false on failure
+		 */
+	bool deactivateClaim( bool graceful, bool got_job_done, bool *claim_is_closing, bool *still_cleaning);
+
+		/** Send the command to this startd to ask if claim_is_closing would be returned if we deactivated.
+			Used by starter to check if it should restart a job after uploading its checkpoint.
 			@param claim_is_closing startd indicates if not accepting more jobs
 			@return true on success, false on failure
 		 */
-	bool deactivateClaim( bool graceful = true, bool *claim_is_closing=NULL );
+	bool reactivateClaimCheck(bool & claim_is_closing);
+
+protected:
+		// helper for deactivateClaim and reactivateClaimCheck
+	bool deactivateClaim(int cmd, bool got_job_done, bool *claim_is_closing, bool *still_cleaning);
+public:
 
 		/** Try to activate the claim on this started with the given
 			job ClassAd and version of the starter we want to use. 
@@ -108,8 +127,7 @@ public:
 			@return OK on success, NOT_OK on failure, CONDOR_TRY_AGAIN
 		        if the startd is busy and wants us to try back later.
 		*/
-	int activateClaim( ClassAd* job_ad, int starter_version, 
-					   ReliSock** claim_sock_ptr );
+	int activateClaim( ClassAd* job_ad, ReliSock** claim_sock_ptr, ClassAd * replyAd /*= nullptr*/);
 
 		/** Before activating a claim, attempt to delegate the user proxy
 			(if there is one). We used do this from the shadow if
@@ -139,7 +157,7 @@ public:
 	
 	bool resumeClaim( ClassAd* reply, int timeout = -1 );
 
-	bool deactivateClaim( VacateType type, ClassAd* reply,
+	bool deactivateClaimCA( VacateType type, ClassAd* reply,
 						  int timeout = -1 );
 
 	bool releaseClaim( VacateType type, ClassAd* reply,
@@ -165,8 +183,6 @@ public:
 	
 	bool vacateClaim( const char* name );
 
-	bool checkpointJob( const char* name );
-
 	bool getAds( ClassAdList &adsList );
 
 		// request_id: set to the request id (can be used to cancel request)
@@ -185,8 +201,8 @@ public:
 	bool updateMachineAd( const ClassAd * update, ClassAd * reply, int timeout = -1 );
 
  private:
-	char* claim_id;
-	char* extra_ids;
+	std::string claim_id;
+	std::string extra_ids;
 
 		// Helper methods
 	bool checkClaimId( void );
