@@ -58,6 +58,7 @@
 #include "generic_stats.h"
 #include "filesystem_remap.h"
 #include "daemon_keep_alive.h"
+#include "classadHistory.h"
 
 #include <vector>
 #include <memory>
@@ -103,6 +104,12 @@ bool dc_args_default_to_background(bool background);
 // Disable default log setup and ignore -l log directory. Must be called before dc_main()
 void DC_Disable_Default_Log();
 
+// functions to temporarily disable the address file
+// and then to re-enable it and drop a fresh one.
+// Used by the Schedd to prevent early dropping of incomplete address files
+void DC_Disable_Addr_File();
+void DC_Enable_And_Drop_Addr_File();
+
 #ifndef WIN32
 // call in the forked child of a HTCondor daemon that is started in backgroun mode when it is ok for the fork parent to exit
 bool dc_release_background_parent(int status);
@@ -118,43 +125,29 @@ extern void (*dc_main_shutdown_graceful)();
 extern void (*dc_main_pre_dc_init)(int argc, char *argv[]);
 extern void (*dc_main_pre_command_sock_init)();
 
-/** @name Typedefs for Callback Procedures
- */
-//@{
-///
 typedef int     (*CommandHandler)(int,Stream*);
-
-///
 typedef int     (Service::*CommandHandlercpp)(int,Stream*);
+using StdCommandHandler = std::function<int(int, Stream*)>;
 
-///
 typedef int     (*SignalHandler)(int);
-
-///
 typedef int     (Service::*SignalHandlercpp)(int);
+using StdSignalHandler = std::function<int(int)>;
 
-///
 typedef int     (*SocketHandler)(Stream*);
-
-///
 typedef int     (Service::*SocketHandlercpp)(Stream*);
+using StdSocketHandler = std::function<int (Stream *)>;
 
-///
 typedef int     (*PipeHandler)(int);
-
-///
 typedef int     (Service::*PipeHandlercpp)(int);
+using StdPipeHandler = std::function<int (int)>;
 
-///
 typedef int     (*ReaperHandler)(int pid,int exit_status);
-
-///
 typedef int     (Service::*ReaperHandlercpp)(int pid,int exit_status);
+using StdReaperHandler = std::function<int (int, int )>;
+
 
-///
 typedef int		(*ThreadStartFunc)(void *,Stream*);
 
-///
 typedef int     (*PumpWorkCallback)(void* cls, void* data);
 
 /// Register with RegisterTimeSkipCallback. Call when clock skips.  First
@@ -216,6 +209,7 @@ struct FamilyInfo {
 	gid_t* group_ptr{nullptr};
 #endif
 	bool want_pid_namespace{false};
+	bool want_net_namespace{false};
 	const char* cgroup{nullptr};
 	uint64_t cgroup_memory_limit{0};
 	uint64_t cgroup_memory_limit_low{0};      // limit after which kernel aggressively evicts memory
@@ -465,8 +459,7 @@ class DaemonCore : public Service
 	 * Typically these methods are invoked from functions inside 
 	 * of daemon_core_main.C.
 	 */
-    DaemonCore (int ComSize = 0, int SigSize = 0,
-                int SocSize = 0, int ReapSize = 0);
+    DaemonCore ();
     ~DaemonCore();
     void Driver();
 
@@ -521,15 +514,28 @@ class DaemonCore : public Service
                                 command handler.
         @return Not_Yet_Documented
     */
-    int Register_Command (int             command,
-                          const char *    com_descrip,
-                          CommandHandler  handler, 
-                          const char *    handler_descrip,
-                          DCpermission    perm,
-                          bool            force_authentication = false,
-                          int             wait_for_payload = 0,
-                          std::vector<DCpermission> *alternate_perms = nullptr);
-    
+    int Register_Command (int               command,
+                          const char *      com_descrip,
+                          CommandHandler    handler,
+                          const char *      handler_descrip,
+                          DCpermission      perm,
+                          bool              force_authentication = false,
+                          int               wait_for_payload = 0,
+                          std::vector<DCpermission> *
+                                            alternate_perms = nullptr
+    );
+
+    int Register_Command( int               command,
+                          const char *      command_description,
+                          StdCommandHandler
+                                            handler,
+                          const char *      handler_description,
+                          DCpermission      permission,
+                          bool              force_authentication = false,
+                          int               wait_for_payload = 0,
+                          std::vector<DCpermission> *
+                                            alternate_permissions = nullptr );
+
     /** Not_Yet_Documented
         @param command         Not_Yet_Documented
         @param com_descrip     Not_Yet_Documented
@@ -731,7 +737,7 @@ class DaemonCore : public Service
     */
     int Register_Signal (int             sig,
                          const char *    sig_descrip,
-                         SignalHandler   handler, 
+                         SignalHandler   handler,
                          const char *    handler_descrip);
     
     /** Not_Yet_Documented
@@ -747,12 +753,27 @@ class DaemonCore : public Service
                          SignalHandlercpp    handlercpp, 
                          const char *        handler_descrip,
                          Service*            s);
-    
+
+    // If except_if_duplicate is true:
+    //      EXCEPT() if a handler has already been registered for `signal`;
+    //      otherwise, return `signal` on success or -1 on failue.
+    // Otherwise,
+    //      return which handler for the given signal was registered,
+    //      or -1 on failure.
+    int Register_Signal( int signal,
+        const char * sig_description,
+        StdSignalHandler handler,
+        const char * handler_description,
+        std::function<void(void)> destroyer,
+        bool except_if_duplicate = false
+    );
+
     /** Not_Yet_Documented
         @param sig Not_Yet_Documented
         @return Not_Yet_Documented
     */
     int Cancel_Signal (int sig);
+    bool Cancel_Signal( int sig, int which );
 
 
     /** Not_Yet_Documented
@@ -823,10 +844,14 @@ class DaemonCore : public Service
         @return Not_Yet_Documented
     */
      int Register_Reaper (const char *      reap_descript,
-                          ReaperHandlercpp  handlercpp, 
+                          ReaperHandlercpp  handlercpp,
                           const char *      handler_descrip,
                           Service*          s);
 
+    int Register_Reaper (const char *     reap_descrip,
+                         StdReaperHandler handler,
+                         const char *     handler_descrip);
+
     /** Not_Yet_Documented
         @param rid The Reaper ID
         @param reap_descrip     Not_Yet_Documented
@@ -919,6 +944,24 @@ class DaemonCore : public Service
                          HandlerType          handler_type = HANDLE_READ,
                          void **              prev_entry = NULL);
 
+    int Register_Socket (Stream*           iosock,
+                         const char *      iosock_descrip,
+                         StdSocketHandler  handler,
+                         const char *      handler_descrip,
+                         HandlerType       handler_type = HANDLE_READ,
+                         void **           prev_entry = NULL) {
+        return Register_Socket (iosock,
+                                iosock_descrip,
+                                (SocketHandler)nullptr,
+                                (SocketHandlercpp)nullptr,
+                                handler_descrip,
+                                nullptr, // Service
+                                handler_type,
+								false,
+                                prev_entry,
+								&handler);
+	}
+
     /** Not_Yet_Documented
         @param iosock           Not_Yet_Documented
         @param descrip          Not_Yet_Documented
@@ -934,7 +977,7 @@ class DaemonCore : public Service
                                 "DC Command Handler",
                                 NULL,
 				HANDLE_READ,
-                                0); 
+                                0);
     }
 
     /** Not_Yet_Documented
@@ -943,6 +986,13 @@ class DaemonCore : public Service
     */
     int Cancel_Socket ( Stream * insock, void *prev_entry = NULL );
 
+    //
+    // Calls the registered "destructor" for each signal handler
+    // matching the description.  The registered destructor should
+    // almost certainly cancel the corresponding signal handler.
+    //
+    void Destroy_Signals_By_Description( const std::string & d );
+
 		// Returns true if the given socket is already registered.
 	bool SocketIsRegistered( Stream *sock );
 
@@ -1034,6 +1084,12 @@ class DaemonCore : public Service
                          Service*             s,
                          HandlerType          handler_type = HANDLE_READ);
 
+    int Register_Pipe (int		           pipe_end,
+                         const char *      pipe_descrip,
+                         StdPipeHandler    handler,
+                         const char *      handler_descrip,
+                         HandlerType       handler_type     = HANDLE_READ);
+
     /** Not_Yet_Documented
         @param pipe_end           Not_Yet_Documented
         @return Not_Yet_Documented
@@ -1125,30 +1181,7 @@ class DaemonCore : public Service
         @param event_descrip   Not_Yet_Documented
         @return Not_Yet_Documented
     */
-    int Register_Timer (unsigned     deltawhen,
-                        TimerHandler handler,
-                        const char * event_descrip);
-
-	/** Not_Yet_Documented
-        @param deltawhen       Not_Yet_Documented
-        @param event           Not_Yet_Documented
-        @param event_descrip   Not_Yet_Documented
-        @return Not_Yet_Documented
-    */
-    int Register_Timer (unsigned     deltawhen,
-                        TimerHandler handler,
-						Release      release,
-                        const char * event_descrip);
-    
-    /** Not_Yet_Documented
-        @param deltawhen       Not_Yet_Documented
-        @param period          Not_Yet_Documented
-        @param event           Not_Yet_Documented
-        @param event_descrip   Not_Yet_Documented
-        @return Not_Yet_Documented
-    */
-    int Register_Timer (unsigned     deltawhen,
-                        unsigned     period,
+    int Register_Timer (time_t     deltawhen,
                         TimerHandler handler,
                         const char * event_descrip);
 
@@ -1159,11 +1192,16 @@ class DaemonCore : public Service
         @param s               Not_Yet_Documented
         @return Not_Yet_Documented
     */
-    int Register_Timer (unsigned     deltawhen,
+    int Register_Timer (time_t     deltawhen,
                         TimerHandlercpp handler,
                         const char * event_descrip,
                         Service*     s);
 
+    int Register_Timer( time_t        deltawhen,
+                        time_t        period,
+                        StdTimerHandler f,
+                        const char *    event_description );
+
     // register a callback from any thread that will be called on the main thread
     // as soon as we are back in the daemon core pump (just before timers are handled).
     // In effect, this is a thread safe register of a zero length one-shot timer
@@ -1180,8 +1218,8 @@ class DaemonCore : public Service
         @param s               Not_Yet_Documented
         @return Not_Yet_Documented
     */
-    int Register_Timer (unsigned     deltawhen,
-                        unsigned     period,
+    int Register_Timer (time_t     deltawhen,
+                        time_t       period,
                         TimerHandlercpp handler,
                         const char * event_descrip,
                         Service *    s);
@@ -1216,11 +1254,11 @@ class DaemonCore : public Service
 
     /** Not_Yet_Documented
         @param id The timer's ID
-        @param when   Not_Yet_Documented
+        @param deltawhen   Not_Yet_Documented
         @param period Not_Yet_Documented
         @return 0 if successful, -1 on failure (timer not found)
     */
-    int Reset_Timer ( int id, unsigned when, unsigned period = 0 );
+    int Reset_Timer ( int id, time_t deltawhen, time_t  period = 0 );
 
     /** Change a timer's period.  Recompute time to fire next based on this
 		new period and how long this timer has been waiting.
@@ -1228,7 +1266,7 @@ class DaemonCore : public Service
         @param period New period for this timer.
         @return 0 on success
     */
-    int Reset_Timer_Period ( int id, unsigned period );
+    int Reset_Timer_Period ( int id, time_t period );
 
     /** Change a timer's timeslice settings.
         @param id The timer's ID
@@ -1410,8 +1448,11 @@ class DaemonCore : public Service
 	static const int ERRNO_REGISTRATION_FAILED;
 	static const int ERRNO_EXIT;
 
+	const static char* DEFAULT_INDENT;
+
     /** Methods for operating on a process family
     */
+	int Snapshot();
     int Get_Family_Usage(pid_t, ProcFamilyUsage&, bool full = false);
     int Suspend_Family(pid_t);
     int Continue_Family(pid_t);
@@ -1559,6 +1600,8 @@ class DaemonCore : public Service
 	*/
 	void Forked_Child_Wants_Fast_Exit( bool exit_by_exec );
 
+	// If you call this, you own the inherited sockets, and are
+	// responsible for deleting them
     Stream **GetInheritedSocks() { return (Stream **)inheritedSocks; }
 
 
@@ -1597,6 +1640,10 @@ class DaemonCore : public Service
 	bool GetPeacefulShutdown() const;
 	void SetPeacefulShutdown(bool value);
 
+	static void TimerHandler_main_shutdown_fast(int tid);
+	static int handle_dc_sigterm(int sig);
+	static int handle_dc_sigquit(int sig);
+
 	/** Called when it looks like the clock jumped unexpectedly.
 
 	data is opaquely passed to the TimeSkipFunc.
@@ -1615,7 +1662,9 @@ class DaemonCore : public Service
     SelfMonitorData monitor_data;
 
 	char 	*localAdFile;
-	void	UpdateLocalAd(ClassAd *daemonAd,char const *fname=NULL); 
+	void	UpdateLocalAd(ClassAd *daemonAd,char const *fname=NULL);
+
+	ClassAd & ContactInfoExtra() { return m_contact_info_extra; }
 
 		/**
 		   Publish all DC-specific attributes into the given ClassAd.
@@ -1627,6 +1676,10 @@ class DaemonCore : public Service
 		*/
 	void	publish(ClassAd *ad);
 
+
+		// Append provided Daemon ClassAd to the daemon history file.
+	void AppendDaemonHistory(ClassAd* ad);
+
 		/**
 		   @return A pointer to the CollectorList object that holds
 		   all of the collectors this daemons is configured to
@@ -1737,6 +1790,8 @@ class DaemonCore : public Service
 	   stats_entry_recent<double> TimerRuntime;   //  total time spent handling timers
 	   stats_entry_recent<double> SocketRuntime;  //  total time spent handling socket messages
 	   stats_entry_recent<double> PipeRuntime;    //  total time spent handling pipe messages
+       std::map<std::string, stats_entry_probe<double>> UserRuntimes; // map for user command runtimes (keyed by user_cmd)
+       std::map<std::string, stats_entry_probe<double>> ReaperRuntimes; // map for reaper runtimes
 
 	   stats_entry_recent<int> Signals;        //  number of signals handlers called
 	   stats_entry_abs<int> TimersFired;    //  number of timer handlers called
@@ -1844,10 +1899,9 @@ class DaemonCore : public Service
 		std::shared_ptr<SafeSock> ssock() { return m_ssock; }
 
 		// Associate a ReliSock or SafeSock with this SockPair. Does nothing
-		// if one is already associated. b must always be true and always
-		// returns true.
-		bool has_relisock(bool b);
-		bool has_safesock(bool b);
+		// if one is already associated.
+		void add_relisock();
+		void add_safesock();
 	private:
 		std::shared_ptr<ReliSock> m_rsock;	// tcp command socket
 		std::shared_ptr<SafeSock> m_ssock;	// udp command socket
@@ -1889,52 +1943,50 @@ class DaemonCore : public Service
     int HandleProcessExit(pid_t pid, int exit_status);
     int HandleDC_SIGCHLD(int sig);
 	int HandleDC_SERVICEWAITPIDS(int sig);
- 
+
+    void callSignalHandlers(double & runtime);
+
     int Register_Command(int command, const char *com_descip,
-                         CommandHandler handler, 
+                         CommandHandler handler,
                          CommandHandlercpp handlercpp,
                          const char *handler_descrip,
-                         Service* s, 
+                         Service* s,
                          DCpermission perm,
                          int is_cpp,
                          bool force_authentication,
                          int wait_for_payload,
-                         std::vector<DCpermission> *alternate_perm);
-
-    int Register_Signal(int sig,
-                        const char *sig_descip,
-                        SignalHandler handler, 
-                        SignalHandlercpp handlercpp,
-                        const char *handler_descrip,
-                        Service* s, 
-                        int is_cpp);
+                         std::vector<DCpermission> *alternate_perm,
+                         StdCommandHandler * handler_f = nullptr );
 
     int Register_Socket(Stream* iosock,
                         const char *iosock_descrip,
-                        SocketHandler handler, 
+                        SocketHandler handler,
                         SocketHandlercpp handlercpp,
                         const char *handler_descrip,
-                        Service* s, 
-			HandlerType handler_type,
+                        Service* s,
+                        HandlerType handler_type,
                         int is_cpp,
-                        void **prev_entry = NULL);
+                        void **prev_entry = nullptr,
+                        StdSocketHandler *handler_f = nullptr);
 
     int Register_Pipe(int pipefd,
                         const char *pipefd_descrip,
-                        PipeHandler handler, 
+                        PipeHandler handler,
                         PipeHandlercpp handlercpp,
                         const char *handler_descrip,
-                        Service* s, 
-					    HandlerType handler_type, 
-                        int is_cpp);
+                        Service* s,
+					    HandlerType handler_type,
+                        int is_cpp,
+                        StdPipeHandler *handler_f = nullptr);
 
     int Register_Reaper(int rid,
                         const char *reap_descip,
-                        ReaperHandler handler, 
+                        ReaperHandler handler,
                         ReaperHandlercpp handlercpp,
                         const char *handler_descrip,
-                        Service* s, 
-                        int is_cpp);
+                        Service* s,
+                        int is_cpp,
+                        StdReaperHandler *handle_f = nullptr);
 
 	bool Register_Family(pid_t child_pid,
 	                     pid_t parent_pid,
@@ -1951,6 +2003,7 @@ class DaemonCore : public Service
 	bool m_use_clone_to_create_processes;
 	bool UseCloneToCreateProcesses() const { return m_use_clone_to_create_processes; }
 #else
+	// If we don't have clone(), this is always false no matter what knobs might be set
 	bool UseCloneToCreateProcesses() { return false; }
 #endif
 
@@ -1971,6 +2024,7 @@ class DaemonCore : public Service
 		bool            force_authentication{false};
 		CommandHandler  handler{nullptr};
 		CommandHandlercpp   handlercpp{nullptr};
+		StdCommandHandler   std_handler;
 		DCpermission    perm{ALLOW};
 		Service*        service{nullptr};
 		char*           command_descrip{nullptr};
@@ -1986,20 +2040,24 @@ class DaemonCore : public Service
 	std::vector<CommandEnt>         comTable;       // command table
     CommandEnt          m_unregisteredCommand;
 
-    struct SignalEnt 
+    struct SignalEnt
     {
         int             num;
-        bool            is_cpp;
         bool            is_blocked;
         // Note: is_pending must be volatile because it could be set inside
         // of a Unix asynchronous signal handler (such as SIGCHLD).
         volatile bool   is_pending;
-        SignalHandler   handler;
-        SignalHandlercpp    handlercpp;
-        Service*        service; 
-        char*           sig_descrip;
-        char*           handler_descrip;
-        void*           data_ptr;
+
+        struct HandlerEntry {
+            bool valid;
+            StdSignalHandler handler;
+            std::function<void(void)> destroyer;
+            std::string sig_descrip;
+            std::string handler_descrip;
+        };
+        std::vector<HandlerEntry> handlers;
+
+        void * data_ptr;
     };
     void                DumpSigTable(int, const char* = NULL);
 	std::vector<SignalEnt> sigTable;    // signal table
@@ -2010,6 +2068,7 @@ class DaemonCore : public Service
         Sock*           iosock;
         SocketHandler   handler;
         SocketHandlercpp    handlercpp;
+		StdSocketHandler std_handler;
         Service*        service; 
         char*           iosock_descrip;
         char*           handler_descrip;
@@ -2053,7 +2112,8 @@ class DaemonCore : public Service
     {
         PipeHandler		handler;
         PipeHandlercpp  handlercpp;
-        Service*        service; 
+        StdPipeHandler  std_handler;
+        Service*        service;
         char*           pipe_descrip;
         char*           handler_descrip;
         void*           data_ptr;
@@ -2072,6 +2132,7 @@ class DaemonCore : public Service
         bool            is_cpp;
         ReaperHandler   handler;
         ReaperHandlercpp    handlercpp;
+        StdReaperHandler    std_handler;
         Service*        service; 
         char*           reap_descrip;
         char*           handler_descrip;
@@ -2303,9 +2364,10 @@ class DaemonCore : public Service
 	void send_invalidate_session ( const char* sinful, const char* sessid,
 	                               const ClassAd* info_ad = NULL ) const;
 
-	bool m_wants_restart;
-	bool m_in_daemon_shutdown;
-	bool m_in_daemon_shutdown_fast;
+	bool m_wants_restart{true};
+	bool m_in_shutdown_peaceful{false};
+	bool m_in_shutdown_graceful{false};
+	bool m_in_shutdown_fast{false};
 
 	char* m_private_network_name;
 
@@ -2322,6 +2384,7 @@ class DaemonCore : public Service
 	bool m_dirty_sinful; // true if m_sinful needs to be reinitialized
 	std::vector<Sinful> m_command_sock_sinfuls; // Cached copy of our command sockets' sinful strings.
 	bool m_dirty_command_sock_sinfuls; // true if m_command_sock_sinfuls needs to be reinitialized.
+	ClassAd m_contact_info_extra; // has extra attributes that should be included in our daemon address files
 
 	//
 	// For compabitility with existing configurations and code, when we
@@ -2348,6 +2411,9 @@ class DaemonCore : public Service
 	bool m_enable_remote_admin{false};
 	time_t m_remote_admin_last_time{0};
 	std::string m_remote_admin_last;
+
+	std::string m_daemon_history{};
+	HistoryFileRotationInfo m_hist_rotation_info{};
 };
 
 /**
