@@ -295,8 +295,8 @@ DaemonCommandProtocol::CommandProtocolResult DaemonCommandProtocol::AcceptUDPReq
 
 		if (sess_id) {
 			KeyCacheEntry *session = NULL;
-			auto sess_itr = m_sec_man->session_cache->find(sess_id);
-			if (sess_itr == m_sec_man->session_cache->end()) {
+			auto sess_itr = m_sec_man->session_cache.find(sess_id);
+			if (sess_itr == m_sec_man->session_cache.end()) {
 				dprintf ( D_ERROR, "DC_AUTHENTICATE: session %s NOT FOUND; this session was requested by %s with return address %s\n", sess_id, m_sock->peer_description(), return_address_ss ? return_address_ss : "(none)");
 				// no session... we outta here!
 
@@ -390,8 +390,8 @@ DaemonCommandProtocol::CommandProtocolResult DaemonCommandProtocol::AcceptUDPReq
 
 		if (sess_id) {
 			KeyCacheEntry *session = NULL;
-			auto sess_itr = m_sec_man->session_cache->find(sess_id);
-			if (sess_itr == m_sec_man->session_cache->end()) {
+			auto sess_itr = m_sec_man->session_cache.find(sess_id);
+			if (sess_itr == m_sec_man->session_cache.end()) {
 				dprintf ( D_ERROR, "DC_AUTHENTICATE: session %s NOT FOUND; this session was requested by %s with return address %s\n", sess_id, m_sock->peer_description(), return_address_ss ? return_address_ss : "(none)");
 				// no session... we outta here!
 
@@ -515,45 +515,38 @@ DaemonCommandProtocol::CommandProtocolResult DaemonCommandProtocol::ReadHeader()
 {
 	m_sock->decode();
 
-		// Determine if incoming socket is HTTP over TCP, or if it is CEDAR.
-		// For better or worse, we figure this out by seeing if the socket
-		// starts w/ a GET or POST.  Hopefully this does not correspond to
-		// a daemoncore command int!  [not ever likely, since CEDAR ints are
-		// exapanded out to 8 bytes]  Still, in a perfect world we would replace
-		// with a more foolproof method.
-		// Note: We no longer support soap, but this peek is part of the
-		//   code below that checks if this is a command that shared port
-		//   should transparently hand off to the collector.
-	char tmpbuf[6];
-	memset(tmpbuf,0,sizeof(tmpbuf));
-	if ( m_is_tcp && daemonCore->HandleUnregistered() ) {
-			// TODO Should we be ignoring the return value of condor_read?
-		condor_read(m_sock->peer_description(), m_sock->get_file_desc(),
-			tmpbuf, sizeof(tmpbuf) - 1, 1, MSG_PEEK);
-	}
-
-		// This was not a soap request; next, see if we have a special command
-		// handler for unknown command integers.
-		//
-		// We do manual CEDAR parsing here to look at the command int directly
-		// without consuming data from the socket.  The first few bytes are the
-		// size of the message, followed by the command int itself.
-	int tmp_req; memcpy(static_cast<void*>(&tmp_req), tmpbuf+1, sizeof(int));
-	tmp_req = ntohl(tmp_req);
-	if (daemonCore->HandleUnregistered() && (tmp_req >= 8)) {
+	// See if we have a special command handler for unknown command integers.
+	//
+	// We do manual CEDAR parsing here to look at the command int directly
+	// without consuming data from the socket.  The first five bytes are the
+	// CEDAR packet header. The next eight bytes are the command int itself.
+	// Leaving the data unconsumed is necessary for the shared port daemon
+	// to transparently hand connections to the collector for commands it
+	// doesn't handle.
+	if (m_is_tcp && daemonCore->HandleUnregistered()) {
 			// Peek at the command integer if one exists.
-		char tmpbuf2[8+5]; memset(tmpbuf2, 0, sizeof(tmpbuf2));
-		condor_read(m_sock->peer_description(), m_sock->get_file_desc(),
-			tmpbuf2, 8+5, 1, MSG_PEEK);
-		char *tmpbuf3 = tmpbuf2 + 5;
-		if (8-sizeof(int) > 0) { tmpbuf3 += 8-sizeof(int); } // Skip padding
-		memcpy(static_cast<void*>(&tmp_req), tmpbuf3, sizeof(int));
-		tmp_req = ntohl(tmp_req);
+		long long tmp_req;
+		char tmpbuf[8+5]; memset(tmpbuf, 0, sizeof(tmpbuf));
+		int sz = sizeof(tmpbuf);
+		int rc;
+		rc = condor_read(m_sock->peer_description(), m_sock->get_file_desc(),
+			tmpbuf, sz, 1, CondorRWFlags::Peek);
+		if (rc != sz) {
+			char const *ip = m_sock->peer_ip_str();
+			if(!ip) {
+				ip = "unknown address";
+			}
+			dprintf(D_ERROR, "DaemonCore: Can't receive command request from %s (perhaps a timeout?)\n", ip);
+			m_result = FALSE;
+			return CommandProtocolFinished;
+		}
+		memcpy(static_cast<void*>(&tmp_req), tmpbuf + 5, sizeof(tmp_req));
+		tmp_req = ntohLL(tmp_req);
 
 			// Lookup the command integer in our command table to see if it is unregistered
 		int tmp_cmd_index;
 		if(	   (!m_isSharedPortLoopback)
-			&& (! daemonCore->CommandNumToTableIndex( tmp_req, &tmp_cmd_index ))
+			&& (! daemonCore->CommandNumToTableIndex( (int)tmp_req, &tmp_cmd_index ))
 			&& ( daemonCore->HandleUnregisteredDCAuth()
 				|| (tmp_req != DC_AUTHENTICATE) ) ) {
 			ScopedEnableParallel(false);
@@ -563,7 +556,7 @@ DaemonCommandProtocol::CommandProtocolResult DaemonCommandProtocol::ReadHeader()
 				m_sock->set_deadline(0);
 			}
 
-			m_result = daemonCore->CallUnregisteredCommandHandler(tmp_req, m_sock);
+			m_result = daemonCore->CallUnregisteredCommandHandler((int)tmp_req, m_sock);
 			return CommandProtocolFinished;
 		}
 	}
@@ -723,8 +716,8 @@ DaemonCommandProtocol::CommandProtocolResult DaemonCommandProtocol::ReadCommand(
 				m_auth_info.Delete(ATTR_SEC_NONCE);
 
 				// lookup the suggested key
-				auto sess_itr = m_sec_man->session_cache->find(m_sid);
-				if (sess_itr == m_sec_man->session_cache->end()) {
+				auto sess_itr = m_sec_man->session_cache.find(m_sid);
+				if (sess_itr == m_sec_man->session_cache.end()) {
 
 					// the key id they sent was not in our cache.  this is a
 					// problem.
@@ -881,6 +874,7 @@ DaemonCommandProtocol::CommandProtocolResult DaemonCommandProtocol::ReadCommand(
 					m_policy->LookupBool(ATTR_SEC_TRIED_AUTHENTICATION,tried_authentication);
 					m_sock->setTriedAuthentication(tried_authentication);
 					m_sock->setSessionID(session->id());
+					m_sock->setPolicyAd(*m_policy);
 				}
 
 				// If the cached policy doesn't have a version, then
@@ -952,7 +946,10 @@ DaemonCommandProtocol::CommandProtocolResult DaemonCommandProtocol::ReadCommand(
 									get_local_hostname().c_str(), daemonCore->mypid,
 							   (long long)time(0), ZZZ_always_increase() );
 
-					if (will_authenticate == SecMan::SEC_FEAT_ACT_YES) {
+					std::string peer_ec;
+					m_auth_info.EvaluateAttrString(ATTR_SEC_ECDH_PUBLIC_KEY, peer_ec);
+
+					if (will_authenticate == SecMan::SEC_FEAT_ACT_YES || !peer_ec.empty()) {
 						std::string crypto_method;
 						if (!m_policy->LookupString(ATTR_SEC_CRYPTO_METHODS, crypto_method)) {
 							dprintf ( D_ERROR, "DC_AUTHENTICATE: tried to enable encryption for request from %s, but we have none!\n", m_sock->peer_description() );
@@ -962,8 +959,7 @@ DaemonCommandProtocol::CommandProtocolResult DaemonCommandProtocol::ReadCommand(
 						Protocol method = SecMan::getCryptProtocolNameToEnum(crypto_method.c_str());
 
 							// If the client provided a EC key, we'll respond in kind.
-						std::string peer_ec;
-						if (m_auth_info.EvaluateAttrString(ATTR_SEC_ECDH_PUBLIC_KEY, peer_ec)) {
+						if (!peer_ec.empty()) {
 							m_peer_pubkey_encoded = peer_ec;
 							if (!(m_keyexchange = SecMan::GenerateKeyExchange(m_errstack))) {
 								dprintf(D_ERROR, "DC_AUTHENTICATE: Error in generating key: %s\n", m_errstack->getFullText().c_str());
@@ -1266,29 +1262,6 @@ DaemonCommandProtocol::CommandProtocolResult DaemonCommandProtocol::Authenticate
 	if( auth_success ) {
 		dprintf (D_SECURITY, "DC_AUTHENTICATE: authentication of %s complete.\n", m_sock->peer_ip_str());
 		m_sock->getPolicyAd(*m_policy);
-
-		if (m_keyexchange) {
-			std::string crypto_method;
-			if (!m_policy->LookupString(ATTR_SEC_CRYPTO_METHODS, crypto_method)) {
-				dprintf ( D_ERROR, "DC_AUTHENTICATE: No crypto methods enabled for request from %s.\n", m_sock->peer_description() );
-				m_result = false;
-				return CommandProtocolFinished;
-			}
-			Protocol method = SecMan::getCryptProtocolNameToEnum(crypto_method.c_str());
-
-			size_t keylen = method == CONDOR_AESGCM ? SEC_SESSION_KEY_LENGTH_V9 : SEC_SESSION_KEY_LENGTH_OLD;
-			std::unique_ptr<unsigned char, decltype(&free)> rbuf(
-				static_cast<unsigned char *>(malloc(keylen)),
-				&free);
-			if (!SecMan::FinishKeyExchange(std::move(m_keyexchange), m_peer_pubkey_encoded.c_str(), rbuf.get(), keylen, m_errstack)) {
-				dprintf(D_ERROR, "DC_AUTHENTICATE: Failed to generate a symmetric key for session with %s: %s.\n", m_sock->peer_description(), m_errstack->getFullText().c_str());
-				m_result = false;
-				return CommandProtocolFinished;
-			}
-
-			dprintf (D_SECURITY, "DC_AUTHENTICATE: generating %s key for session %s...\n", crypto_method.c_str(), m_sid.c_str());
-			m_key = new KeyInfo(rbuf.get(), keylen, method, 0);
-		}
 	}
 	else {
 		bool auth_required = true;
@@ -1322,6 +1295,29 @@ DaemonCommandProtocol::CommandProtocolResult DaemonCommandProtocol::EnableCrypto
 {
 	dprintf( D_DAEMONCORE, "DAEMONCORE: EnableCrypto()\n");
 
+	// If we're doing ECDH, derive the session key now
+	if (m_keyexchange) {
+		std::string crypto_method;
+		if (!m_policy->LookupString(ATTR_SEC_CRYPTO_METHODS, crypto_method)) {
+			dprintf ( D_ERROR, "DC_AUTHENTICATE: No crypto methods enabled for request from %s.\n", m_sock->peer_description() );
+			m_result = false;
+			return CommandProtocolFinished;
+		}
+		Protocol method = SecMan::getCryptProtocolNameToEnum(crypto_method.c_str());
+		size_t keylen = method == CONDOR_AESGCM ? SEC_SESSION_KEY_LENGTH_V9 : SEC_SESSION_KEY_LENGTH_OLD;
+		std::unique_ptr<unsigned char, decltype(&free)> rbuf(
+			static_cast<unsigned char *>(malloc(keylen)),
+			&free);
+		if (!SecMan::FinishKeyExchange(std::move(m_keyexchange), m_peer_pubkey_encoded.c_str(), rbuf.get(), keylen, m_errstack)) {
+			dprintf(D_ERROR, "DC_AUTHENTICATE: Failed to generate a symmetric key for session with %s: %s.\n", m_sock->peer_description(), m_errstack->getFullText().c_str());
+			m_result = false;
+			return CommandProtocolFinished;
+		}
+
+		dprintf (D_SECURITY, "DC_AUTHENTICATE: generating %s key for session %s...\n", crypto_method.c_str(), m_sid.c_str());
+		m_key = new KeyInfo(rbuf.get(), keylen, method, 0);
+	}
+
 	// We must configure encryption before integrity on the socket
 	// when using AES over TCP. If we do integrity first, then ReliSock
 	// will initialize an MD5 context and then tear it down when it
@@ -1470,20 +1466,17 @@ DaemonCommandProtocol::CommandProtocolResult DaemonCommandProtocol::VerifyComman
 					return CommandProtocolFinished;
 				}
 
-				// well, they didn't authenticate, turn on encryption,
-				// or turn on integrity.  check to see if any of those
-				// were required.
+				// Well, they didn't authenticate. See if that was required.
+				// Also, if security negotiation is required, see if they
+				// have a security session.
 
 				if (  (m_sec_man->sec_lookup_req(*our_policy, ATTR_SEC_NEGOTIATION)
-					   == SecMan::SEC_REQ_REQUIRED)
-				   || (m_sec_man->sec_lookup_req(*our_policy, ATTR_SEC_AUTHENTICATION)
-					   == SecMan::SEC_REQ_REQUIRED)
-				   || (m_sec_man->sec_lookup_req(*our_policy, ATTR_SEC_ENCRYPTION)
-					   == SecMan::SEC_REQ_REQUIRED)
-				   || (m_sec_man->sec_lookup_req(*our_policy, ATTR_SEC_INTEGRITY)
+					   == SecMan::SEC_REQ_REQUIRED && m_sock->getSessionID().empty())
+				   || (m_sec_man->sec_lookup_req(*our_policy, ATTR_SEC_AUTHENTICATION_NEW)
 					   == SecMan::SEC_REQ_REQUIRED) ) {
 
-					// yep, they were.  deny.
+					// yep, negotiation or authentication was required and
+					// they didn't do it.  deny.
 
 					dprintf(D_ALWAYS,
 						"DaemonCore: PERMISSION DENIED for %d (%s) via %s%s%s from host %s (access level %s)\n",
@@ -1542,7 +1535,16 @@ DaemonCommandProtocol::CommandProtocolResult DaemonCommandProtocol::VerifyComman
 				// these limits if present.
 			std::string authz_policy;
 			bool can_attempt = true;
-			if (m_policy && m_policy->EvaluateAttrString(ATTR_SEC_LIMIT_AUTHORIZATION, authz_policy)) {
+			bool has_capability = false;
+			const ClassAd* policy_ad = m_policy ? m_policy : m_sock->getPolicyAd();
+			if (policy_ad) {
+				if (policy_ad->EvaluateAttrString("TokenCapabilities", authz_policy)) {
+					has_capability = true;
+				} else {
+					policy_ad->EvaluateAttrString(ATTR_SEC_LIMIT_AUTHORIZATION, authz_policy);
+				}
+			}
+			if (!authz_policy.empty()) {
 				std::set<DCpermission> authz_limits;
 				for (const auto& limit_str: StringTokenIterator(authz_policy)) {
 					DCpermission limit_perm = getPermissionFromString(limit_str.c_str());
@@ -1569,7 +1571,18 @@ DaemonCommandProtocol::CommandProtocolResult DaemonCommandProtocol::VerifyComman
 					can_attempt = false;
 				}
 			}
-			if (can_attempt) {
+			if (has_capability && can_attempt) {
+				// Client has capability that authorizes this command
+				dprintf(D_STATUS,
+					"PERMISSION GRANTED to %s from host %s for %s, "
+					"access level %s: reason: %s\n",
+					m_user.c_str(),
+					m_sock->peer_addr().to_ip_string_ex().c_str(),
+					command_desc.c_str(),
+					PermString(m_comTable[m_cmd_index].perm),
+					"client has capability that allows this command");
+				m_perm = USER_AUTH_SUCCESS;
+			} else if (can_attempt) {
 					// A bit redundant to have the outer conditional,
 					// but this gets the log verbosity right and has
 					// zero cost in the "normal" case with no alternate
@@ -1795,7 +1808,7 @@ DaemonCommandProtocol::CommandProtocolResult DaemonCommandProtocol::SendResponse
 		// because then this key would get confused for an
 		// outgoing session to a daemon with that IP and
 		// port as its command socket.
-		m_sec_man->session_cache->emplace(m_sid, KeyCacheEntry(m_sid, "", keyvec, *m_policy, expiration_time, session_lease));
+		m_sec_man->session_cache.emplace(m_sid, KeyCacheEntry(m_sid, "", keyvec, *m_policy, expiration_time, session_lease));
 		dprintf (D_SECURITY, "DC_AUTHENTICATE: added incoming session id %s to cache for %i seconds (lease is %ds, return address is %s).\n", m_sid.c_str(), durint, session_lease, return_addr.c_str());
 		if (IsDebugVerbose(D_SECURITY)) {
 			dPrintAd(D_SECURITY, *m_policy);
@@ -1863,6 +1876,18 @@ DaemonCommandProtocol::CommandProtocolResult DaemonCommandProtocol::ExecCommand(
 		ClassAd q_response;
 		q_response.Assign( ATTR_SEC_AUTHORIZATION_SUCCEEDED, (m_perm == USER_AUTH_SUCCESS) );
 
+		// Include token-related attributes
+		// Can m_policy be NULL in practice?
+		if (m_policy) {
+			size_t prefix_len = strlen(ATTR_TOKEN_prefix);
+			for (const auto& token_attr: *m_policy) {
+				if (strncasecmp(token_attr.first.c_str(), ATTR_TOKEN_prefix, prefix_len) != 0) {
+					continue;
+				}
+				q_response.Insert(token_attr.first, token_attr.second->Copy());
+			}
+		}
+
 		if (!putClassAd(m_sock, q_response) ||
 			!m_sock->end_of_message()) {
 			dprintf (D_ERROR, "SECMAN: Error sending DC_SEC_QUERY reply to %s!\n", m_sock->peer_description());
