@@ -13,12 +13,17 @@
 #include <fstream>
 #include <cstdio>
 #include <stdexcept>
-#include <rapidjson/document.h>
+
+#define PLUGIN_VERSION "0.2"
 
 #define MAX_RETRY_ATTEMPTS 20
 int max_retry_attempts = MAX_RETRY_ATTEMPTS;
 
-// Setup a transfer progress callback. We'll use this to manually timeout 
+// This is the CURL default after version 8.3.0 (released September, 2023).
+#define MAX_REDIRS 30
+int max_redirs = MAX_REDIRS;
+
+// Setup a transfer progress callback. We'll use this to manually timeout
 // any transfers that are not making forward progress.
 
 struct xferProgress {
@@ -151,24 +156,20 @@ GetToken(const std::string & cred_name, std::string & token) {
 			token = line;
 			break;
 		}
-		rapidjson::Document doc;
-		if (doc.Parse(line.c_str()).HasParseError()) {
+
+		classad::ClassAdJsonParser parser;
+		ClassAd *ad = parser.ParseClassAd(line);
+		if (!ad) {
 			// DO NOT include the error message as part of the exception; the error
 			// message may include private information in the credential file itself,
 			// which we don't want to go into the public hold message.
 			throw std::runtime_error("Unable to parse token as JSON");
-                }
-		if (!doc.IsObject()) {
-			throw std::runtime_error("Token is not a JSON object");
 		}
-		if (!doc.HasMember("access_token")) {
+		token.clear();
+		ad->LookupString("access_token", token);
+		if (token.empty()) {
 			throw std::runtime_error("No 'access_token' key in JSON object");
 		}
-		auto &access_obj = doc["access_token"];
-		if (!access_obj.IsString()) {
-			throw std::runtime_error("'access_token' value is not a string");
-		}
-		token = access_obj.GetString();
 	}
 }
 
@@ -264,6 +265,14 @@ MultiFileCurlPlugin::InitializeCurlHandle(const std::string &url, const std::str
 			fprintf(stderr, "Can't setopt HEADERFUNCTION\n");
 		}
 
+        // We MUST successfully set the USERAGENT, because some URLs
+        // will 403 without one. (!!)  [https://repo.anaconda.com]
+        // ... blowing as ASSERT() here is probably not OK, though.
+        r = curl_easy_setopt( _handle, CURLOPT_USERAGENT, "condor_curl_plugin/" "PLUGIN_VERSION" );
+        if (r != CURLE_OK) {
+            fprintf(stderr, "Can't setopt USERAGENT\n");
+        }
+
         GetToken(cred, token);
     }
     // Libcurl options for FTP
@@ -326,10 +335,15 @@ MultiFileCurlPlugin::InitializeCurlHandle(const std::string &url, const std::str
 	if (r != CURLE_OK) {
 		fprintf(stderr, "Can't setopt NOPROGRESS\n");
 	}
+
+	r = curl_easy_setopt(_handle, CURLOPT_MAXREDIRS, (long)max_redirs);
+	if (r != CURLE_OK) {
+		fprintf(stderr, "Can't setopt MAXREDIRS\n");
+	}
 }
 
 FILE *
-MultiFileCurlPlugin::OpenLocalFile(const std::string &local_file, const char *mode) const {
+MultiFileCurlPlugin::OpenLocalFile(const std::string &local_file, const char *mode, std::string & errorString) const {
     FILE *file = nullptr;
     if ( !strcmp( local_file.c_str(), "-" ) ) {
         int fd = dup(1);
@@ -344,7 +358,8 @@ MultiFileCurlPlugin::OpenLocalFile(const std::string &local_file, const char *mo
     }
 
     if( !file ) {
-        fprintf( stderr, "ERROR: could not open local file %s, error %d (%s)\n", local_file.c_str(), errno, strerror(errno) );
+        formatstr( errorString, "could not open local file %s, error %d (%s)", local_file.c_str(), errno, strerror(errno) );
+        fprintf( stderr, "ERROR: %s\n", errorString.c_str() );
     }
 
     return file;
@@ -384,7 +399,7 @@ MultiFileCurlPlugin::FinishCurlTransfer( int rval, FILE *file ) {
         _this_file_stats->TransferTotalBytes += (int64_t) bytes_uploaded;
     }
 
-    _this_file_stats->ConnectionTimeSeconds +=  ( transfer_total_time - transfer_connection_time );
+    _this_file_stats->ConnectionTimeSeconds += ( transfer_total_time - transfer_connection_time );
     _this_file_stats->TransferHTTPStatusCode = return_code;
     _this_file_stats->LibcurlReturnCode = rval;
 
@@ -393,17 +408,120 @@ MultiFileCurlPlugin::FinishCurlTransfer( int rval, FILE *file ) {
         _this_file_stats->TransferSuccess = true;
         _this_file_stats->TransferError = "";
         _this_file_stats->TransferFileBytes = ftell( file );
-    }
-    else if ( rval == CURLE_ABORTED_BY_CALLBACK ) {
-            // Transfer failed because our xferInfo callback above returned abort.
-            // The error string returned by libcurl just says "Callback aborted",
-            // so lets give something more meaningful.
-        _this_file_stats->TransferSuccess = false;
-        _this_file_stats->TransferError = "Aborted due to lack of progress";
-    }
-    else {
+    } else {
         _this_file_stats->TransferSuccess = false;
         _this_file_stats->TransferError = _error_buffer;
+
+
+        ClassAd transferErrorAd;
+        transferErrorAd.InsertAttr( "ErrorCode", rval );
+        transferErrorAd.InsertAttr( "ErrorString", _error_buffer );
+
+        switch(rval) {
+            /* case CURLE_CONSTANT: // <value> -- <count>,
+               as of 2025-02-04, from the OSPool? epoch logs
+               via ElasticSearch and JasonP. */
+            case CURLE_URL_MALFORMAT: // 3 -- 513
+                transferErrorAd.InsertAttr( "ErrorType", "Parameter" );
+                transferErrorAd.InsertAttr( "PluginLaunched", true );
+                transferErrorAd.InsertAttr( "PluginVersion", PLUGIN_VERSION );
+                break;
+            case CURLE_COULDNT_RESOLVE_PROXY: // 5 -- 5
+                transferErrorAd.InsertAttr( "ErrorType", "Resolution" );
+                // This is probably the <schema>_proxy environment variable, we're not sure.
+                transferErrorAd.InsertAttr( "FailedName", "Unknown" );
+                transferErrorAd.InsertAttr( "FailureType", "Unknown" );
+                break;
+            case CURLE_COULDNT_RESOLVE_HOST: // 6 -- 5162
+                transferErrorAd.InsertAttr( "ErrorType", "Resolution" );
+                // This is an assumption.
+                transferErrorAd.InsertAttr( "FailedName", _this_file_stats->TransferHostName );
+                transferErrorAd.InsertAttr( "FailureType", "Unknown" );
+                break;
+            case CURLE_COULDNT_CONNECT: // 7 -- 259390
+            case CURLE_SSL_CONNECT_ERROR: // 35 -- 83
+                transferErrorAd.InsertAttr( "ErrorType", "Contact" );
+                // This error is explicitly for both host and proxy.
+                transferErrorAd.InsertAttr( "FailedServer", "Unknown" );
+                break;
+            case CURLE_REMOTE_ACCESS_DENIED:
+                transferErrorAd.InsertAttr( "ErrorType", "Authorization" );
+                // This is an assumption.
+                transferErrorAd.InsertAttr( "FailedServer", _this_file_stats->TransferHostName );
+                transferErrorAd.InsertAttr( "FailureType", "Unknown" );
+                break;
+            case CURLE_HTTP_RETURNED_ERROR: // 22 -- 780535
+                switch (_this_file_stats->TransferHTTPStatusCode) {
+                    case 400:
+                        // Bad Request.
+                        transferErrorAd.InsertAttr( "ErrorType", "Specification" );
+                        transferErrorAd.InsertAttr( "FailedServer", _this_file_stats->TransferHostName );
+                        break;
+                    case 403:
+                        // Forbidden.
+                        // This is NOT 401: Unauthorized.
+                        transferErrorAd.InsertAttr( "ErrorType", "Authorization" );
+                        transferErrorAd.InsertAttr( "FailureType", "Authorization" );
+                        transferErrorAd.InsertAttr( "FailedServer", _this_file_stats->TransferHostName );
+                        break;
+                    case 404:
+                        // Not Found.
+                        transferErrorAd.InsertAttr( "ErrorType", "Specification" );
+                        transferErrorAd.InsertAttr( "FailedServer", _this_file_stats->TransferHostName );
+                        break;
+                    case 500:
+                        // Internal Server Error.
+                        // There's not really a category for this.
+                        break;
+                    case 503:
+                        // Service Unavailable.
+                        // There's not really a category for this.
+                        break;
+                    default:
+                        break;
+                }
+                break;
+            case CURLE_WRITE_ERROR: // 23 -- 4090
+                // Only seen with file:// URLs so far.
+                break;
+            case CURLE_OPERATION_TIMEDOUT: // 28 -- 6
+                // How did we get here?  Did a DNS query time out?
+                break;
+            case CURLE_FILE_COULDNT_READ_FILE: // 37 -- 2645180
+                // We've only see this happen with file:// URLs used for downloads.
+                transferErrorAd.InsertAttr( "ErrorType", "Specification" );
+                break;
+            case CURLE_ABORTED_BY_CALLBACK: // 42 -- 1308562
+                // Transfer failed because our xferInfo callback above returned abort.
+                // The error string returned by libcurl just says "Callback aborted",
+                // so lets give something more meaningful.
+                _this_file_stats->TransferSuccess = false;
+                _this_file_stats->TransferError = "Aborted due to lack of progress";
+
+                transferErrorAd.InsertAttr( "ErrorType", "Transfer" );
+                transferErrorAd.InsertAttr( "FailedServer", "Unknown" );
+                transferErrorAd.InsertAttr( "FailureType", "TooSlow" );
+                break;
+            case CURLE_TOO_MANY_REDIRECTS:
+                transferErrorAd.InsertAttr( "ErrorType", "Specification" );
+                transferErrorAd.InsertAttr( "FailedServer", "Unknown" );
+                break;
+            case CURLE_SEND_ERROR: // 55 -- 1089
+                // We've only seen this for file:// URLs, but I don't know what it means.
+                break;
+            case CURLE_RECV_ERROR: // 56 -- 51
+                // We've only seen this for S3 URLs, so far but I don't know what it means.
+                break;
+            case CURLE_PEER_FAILED_VERIFICATION: // 60 -- 7
+                transferErrorAd.InsertAttr( "ErrorType", "Authorization" );
+                // This could presumably happen with an https proxy.
+                transferErrorAd.InsertAttr( "FailedServer", "Unknown" );
+                transferErrorAd.InsertAttr( "FailureType", "Authentication" );
+                break;
+        }
+
+
+        _this_file_stats->TransferErrorData.push_back(transferErrorAd);
     }
 }
 
@@ -414,7 +532,13 @@ MultiFileCurlPlugin::UploadFile( const std::string &url, const std::string &loca
     FILE *file = nullptr;
     int rval = -1;
 
-    if( !(file=OpenLocalFile(local_file_name, "r")) ) {
+    std::string errorString;
+    if( !(file=OpenLocalFile(local_file_name, "r", errorString)) ) {
+        _this_file_stats->TransferSuccess = false;
+        _this_file_stats->TransferType = "upload";
+        _this_file_stats->TransferTries += 1;
+        _this_file_stats->TransferError = errorString;
+
         return rval;
     }
     struct curl_slist *header_list = NULL;
@@ -425,7 +549,7 @@ MultiFileCurlPlugin::UploadFile( const std::string &url, const std::string &loca
         _this_file_stats->TransferError = exc.what();
         fprintf( stderr, "Error: %s.\n", exc.what() );
         return rval;
-    }   
+    }
 
     int fd = fileno(file);
     struct stat stat_buf;
@@ -489,14 +613,20 @@ MultiFileCurlPlugin::UploadFile( const std::string &url, const std::string &loca
 }
 
 
-int 
+int
 MultiFileCurlPlugin::DownloadFile( const std::string &url, const std::string &local_file_name, const std::string &cred, long &partial_bytes ) {
 
     char partial_range[20];
     FILE *file = NULL;
     int rval = -1;
 
-    if ( !(file=OpenLocalFile(local_file_name, partial_bytes ? "a+" : "w")) ) {
+    std::string errorString;
+    if ( !(file=OpenLocalFile(local_file_name, partial_bytes ? "a+" : "w", errorString)) ) {
+        _this_file_stats->TransferSuccess = false;
+        _this_file_stats->TransferType = "download";
+        _this_file_stats->TransferTries += 1;
+        _this_file_stats->TransferError = errorString;
+
         return rval;
     }
     struct curl_slist *header_list = NULL;
@@ -507,7 +637,7 @@ MultiFileCurlPlugin::DownloadFile( const std::string &url, const std::string &lo
         _this_file_stats->TransferError = exc.what();
         fprintf( stderr, "Error: %s.\n", exc.what() );
         return rval;
-    }   
+    }
 
     // Libcurl options that apply to all transfer protocols
 	CURLcode r;
@@ -572,11 +702,11 @@ MultiFileCurlPlugin::DownloadFile( const std::string &url, const std::string &lo
 
         // Error handling and cleanup
     if( _diagnostic && rval ) {
-        fprintf(stderr, "curl_easy_perform returned CURLcode %d: %s\n", 
-                rval, curl_easy_strerror( ( CURLcode ) rval ) ); 
+        fprintf(stderr, "curl_easy_perform returned CURLcode %d: %s\n",
+                rval, curl_easy_strerror( ( CURLcode ) rval ) );
     }
 
-    fclose( file ); 
+    fclose( file );
 
     return rval;
 }
@@ -593,7 +723,7 @@ MultiFileCurlPlugin::BuildTransferRequests(const std::string &input_filename, st
     // of inputs.
     input_file = safe_fopen_wrapper( input_filename.c_str(), "r" );
     if( input_file == NULL ) {
-        fprintf( stderr, "Unable to open curl_plugin input file %s.\n", 
+        fprintf( stderr, "Unable to open curl_plugin input file %s.\n",
             input_filename.c_str() );
         return 1;
     }
@@ -604,7 +734,7 @@ MultiFileCurlPlugin::BuildTransferRequests(const std::string &input_filename, st
     }
     else {
         // Iterate over the classads in the file, and insert each one into our
-        // requested_files map, with the key: url, value: additional details 
+        // requested_files map, with the key: url, value: additional details
         // about the transfer.
         ClassAd transfer_file_ad;
         std::string local_file_name;
@@ -614,8 +744,8 @@ MultiFileCurlPlugin::BuildTransferRequests(const std::string &input_filename, st
 
         int count = 0;
         while ( adFileIter.next( transfer_file_ad ) > 0 ) {
-            transfer_file_ad.EvaluateAttrString( "LocalFileName", local_file_name );
-            transfer_file_ad.EvaluateAttrString( "Url", url );
+            if(! transfer_file_ad.EvaluateAttrString( "LocalFileName", local_file_name )) { continue; }
+            if(! transfer_file_ad.EvaluateAttrString( "Url", url )) { continue; }
             request_details.local_file_name = local_file_name;
 
             if (url.substr(0, 7) == "davs://") {
@@ -652,6 +782,7 @@ MultiFileCurlPlugin::UploadMultipleFiles( const std::string &input_filename ) {
     classad::ClassAdUnParser unparser;
     if ( _diagnostic ) { fprintf( stderr, "Uploading multiple files.\n" ); }
 
+    bool rval_already_set = false;
     for (const auto &file_pair : requested_files) {
 
         const auto &local_file_name = file_pair.second.local_file_name;
@@ -664,7 +795,7 @@ MultiFileCurlPlugin::UploadMultipleFiles( const std::string &input_filename ) {
         _this_file_stats.reset(new FileTransferStats());
         InitializeStats( url );
         _this_file_stats->TransferStartTime = time(NULL);
-	_this_file_stats->TransferFileName = local_file_name;
+        _this_file_stats->TransferFileName = local_file_name;
 
         // Enter the loop that will attempt/retry the curl request
         for ( ;; ) {
@@ -673,8 +804,8 @@ MultiFileCurlPlugin::UploadMultipleFiles( const std::string &input_filename ) {
 
             // Everything prior to the first '+' is the credential name.
             std::string full_scheme = getURLType(url.c_str(), false);
-            auto offset = full_scheme.find_last_of("+");
-            auto cred = (offset == std::string::npos) ? "" : full_scheme.substr(0, offset);
+            auto offset = full_scheme.find_last_of('+');
+            const auto &cred = (offset == std::string::npos) ? "" : full_scheme.substr(0, offset);
 
             // The actual transfer should only be everything after the last '+'
             std::string full_url = url;
@@ -711,7 +842,8 @@ MultiFileCurlPlugin::UploadMultipleFiles( const std::string &input_filename ) {
 
         // Note that we attempt to upload all files, even if one fails!
         // The upload protocol demands that all attempted files have a corresponding ad.
-        if ( ( file_rval != CURLE_OK ) && ( rval != -1 ) ) {
+        if ( ( file_rval != CURLE_OK ) && (! rval_already_set) ) {
+            rval_already_set = true;
             rval = file_rval;
         }
     }
@@ -735,6 +867,7 @@ MultiFileCurlPlugin::DownloadMultipleFiles( const std::string &input_filename )
     }
     classad::ClassAdUnParser unparser;
 
+    bool any_error_ever = false;
     // Iterate over the map of files to transfer.
     for ( const auto &file_pair : requested_files ) {
 
@@ -749,7 +882,7 @@ MultiFileCurlPlugin::DownloadMultipleFiles( const std::string &input_filename )
         _this_file_stats.reset( new FileTransferStats() );
         InitializeStats( url );
         _this_file_stats->TransferStartTime = time(NULL);
-	_this_file_stats->TransferFileName = local_file_name;
+        _this_file_stats->TransferFileName = local_file_name;
 
         long partial_bytes = 0;
         // Enter the loop that will attempt/retry the curl request
@@ -760,8 +893,8 @@ MultiFileCurlPlugin::DownloadMultipleFiles( const std::string &input_filename )
 
             // Everything prior to the first '+' is the credential name.
             std::string full_scheme = getURLType(url.c_str(), false);
-            auto offset = full_scheme.find_last_of("+");
-            auto cred = (offset == std::string::npos) ? "" : full_scheme.substr(0, offset);
+            auto offset = full_scheme.find_last_of('+');
+            const auto &cred = (offset == std::string::npos) ? "" : full_scheme.substr(0, offset);
 
             // The actual transfer should only be everything after the last '+'
             std::string full_url = url;
@@ -778,7 +911,7 @@ MultiFileCurlPlugin::DownloadMultipleFiles( const std::string &input_filename )
             }
             // If we have not exceeded the maximum number of retries, and we encounter
             // a non-fatal error, stay in the loop and try again
-            else if( retry_count <= max_retry_attempts && 
+            else if( retry_count <= max_retry_attempts &&
                      ShouldRetryTransfer(rval) ) {
                 continue;
             }
@@ -793,16 +926,17 @@ MultiFileCurlPlugin::DownloadMultipleFiles( const std::string &input_filename )
         // Regardless of success/failure, update the stats
         classad::ClassAd stats_ad;
         _this_file_stats->Publish( stats_ad );
-	std::string stats_string;
+        std::string stats_string;
         unparser.Unparse( stats_string, &stats_ad );
         _all_files_stats += stats_string;
         stats_ad.Clear();
 
-        // If the transfer did fail, break out of the loop immediately
-        if ( rval > 0 ) break;
+        // Try each URL so that the user doesn't have to re-run
+        // the job one output typo at a time.
+        if( rval != 0 ) { any_error_ever = true; }
     }
 
-    if ( rval != 0 ) return TransferPluginResult::Error;
+    if ( any_error_ever ) return TransferPluginResult::Error;
 
     return TransferPluginResult::Success;
 }
@@ -810,10 +944,10 @@ MultiFileCurlPlugin::DownloadMultipleFiles( const std::string &input_filename )
 /*
     Check if this server supports resume requests using the HTTP "Range" header
     by sending a Range request and checking the return code. Code 206 means
-    resume is supported, code 200 means not supported. 
+    resume is supported, code 200 means not supported.
     Return: 1 if resume is supported, 0 if not.
 */
-int 
+int
 MultiFileCurlPlugin::ServerSupportsResume( const std::string &url ) {
 
     int rval = -1;
@@ -919,7 +1053,7 @@ MultiFileCurlPlugin::InitializeStats( std::string request_url ) {
 
 size_t
 MultiFileCurlPlugin::HeaderCallback( char* buffer, size_t size, size_t nitems, void *userdata ) {
-    fprintf(stderr, "[MultiFileCurlPlugin::HeaderCallback] called\n");
+    // fprintf(stderr, "[MultiFileCurlPlugin::HeaderCallback] called\n");
     auto ft_stats = static_cast<FileTransferStats*>(userdata);
 
     // Work around a bug in libcurl; see HTCONDOR-1426.
@@ -965,7 +1099,7 @@ MultiFileCurlPlugin::HeaderCallback( char* buffer, size_t size, size_t nitems, v
 size_t
 MultiFileCurlPlugin::FtpWriteCallback( void* buffer, size_t size, size_t nmemb, void* stream ) {
     FILE* outfile = ( FILE* ) stream;
-    return fwrite( buffer, size, nmemb, outfile); 
+    return fwrite( buffer, size, nmemb, outfile);
 }
 
 
@@ -1036,13 +1170,13 @@ main( int argc, char **argv ) {
 
             printf( "%s",
                 "MultipleFileSupport = true\n"
-                "PluginVersion = \"0.2\"\n"
+                "PluginVersion = \"" PLUGIN_VERSION "\"\n"
                 "PluginType = \"FileTransfer\"\n"
-                "ProtocolVersion = 2\n"
+                "ProtocolVersion = 4\n"
             );
             printf( "SupportedMethods = \"%s\"\n", SupportedMethods );
 
-            for( auto method : StringTokenIterator(SupportedMethods) ) {
+            for( const auto& method : StringTokenIterator(SupportedMethods) ) {
                 std::string envVarName = method + "_proxy";
                 char * proxy = getenv(envVarName.c_str());
                 if( proxy != NULL ) {
@@ -1092,12 +1226,30 @@ main( int argc, char **argv ) {
 
 	// Mainly for testing to not wait forever to see errors
 	const char *attempts_str = getenv("CONDOR_CURL_MAX_RETRY_ATTEMPTS");
+	if( attempts_str == NULL ) {
+		attempts_str = getenv("CURL_MAX_RETRY_ATTEMPTS");
+	}
 	if (attempts_str) {
 		if (atoi(attempts_str) > 0) {
 			max_retry_attempts = atoi(attempts_str);
 		}
 	}
 
+    const char *redirs_str = getenv("CURL_MAX_REDIRS");
+    if (redirs_str) {
+        if (atoi(redirs_str) > 0) {
+            max_redirs = atoi(redirs_str);
+        }
+    }
+
+    const char *ignore_str = getenv("CURL_IGNORE_HTTP_PROXY");
+    if (ignore_str) {
+        if (atoi(ignore_str) > 0) {
+            UnsetEnv("http_proxy");
+        }
+    }
+
+
     // Instantiate a MultiFileCurlPlugin object and handle the request
     MultiFileCurlPlugin curl_plugin( diagnostic );
     if( curl_plugin.InitializeCurl() != 0 ) {
