@@ -42,7 +42,7 @@ std::string& ArcResource::HashName( const char *resource_name,
 
 	formatstr( hash_name, "arc %s#%d#%s#%s", resource_name,
 	           gahp_id,
-	           proxy_subject ? proxy_subject : "NULL",
+	           proxy_subject ? proxy_subject : "",
 	           token_file.c_str() );
 
 	return hash_name;
@@ -156,7 +156,7 @@ void ArcResource::PublishResourceAd( ClassAd *resource_ad )
 	gahp->PublishStats( resource_ad );
 }
 
-void ArcResource::DoPing( unsigned& ping_delay, bool& ping_complete,
+void ArcResource::DoPing(time_t& ping_delay, bool& ping_complete,
 							bool& ping_succeeded )
 {
 	int rc;
