@@ -27,6 +27,7 @@
 #include "gahp-client.h"
 
 #include <algorithm>
+#include <limits>
 
 #define DEFAULT_MAX_SUBMITTED_JOBS_PER_RESOURCE		1000
 #define DEFAULT_JOB_POLL_RATE 5
@@ -221,13 +222,13 @@ bool BaseResource::Invalidate () {
         "((TARGET.%s =?= \"%s\") && (TARGET.%s =?= \"%s\") && "
 		 "(TARGET.%s =?= \"%s\") && (TARGET.%s =?= \"%s\"))",
         ATTR_HASH_NAME, GetHashName (),
-        ATTR_SCHEDD_NAME, ScheddObj->name (),
+        ATTR_SCHEDD_NAME, ScheddName,
 		ATTR_SCHEDD_IP_ADDR, ScheddObj->addr (),
 		ATTR_OWNER, myUserName );
     ad.AssignExpr ( ATTR_REQUIREMENTS, line.c_str() );
 
 	ad.Assign( ATTR_HASH_NAME, GetHashName() );
-	ad.Assign( ATTR_SCHEDD_NAME, ScheddObj->name() );
+	ad.Assign( ATTR_SCHEDD_NAME, ScheddName );
 	ad.Assign( ATTR_SCHEDD_IP_ADDR, ScheddObj->addr() );
 	ad.Assign( ATTR_OWNER, myUserName );
 
@@ -266,7 +267,7 @@ void BaseResource::UpdateCollector( int /* timerID */ ) {
 	/* avoid updating the collector too often, except on the 
 	first update */
 	if ( !_firstCollectorUpdate ) {
-		int delay = ( _lastCollectorUpdate + 
+		time_t delay = ( _lastCollectorUpdate +
 			_collectorUpdateInterval ) - time ( NULL );
 		if ( delay > 0 ) {
 			daemonCore->Reset_Timer ( _updateCollectorTimerId, delay );
@@ -350,7 +351,7 @@ void BaseResource::RegisterJob( BaseJob *job )
 		}
 	}
 
-	int lease_expiration = -1;
+	time_t lease_expiration = -1;
 	job->jobAd->LookupInteger( ATTR_JOB_LEASE_EXPIRATION, lease_expiration );
 	if ( lease_expiration > 0 ) {
 		RequestUpdateLeases();
@@ -419,10 +420,8 @@ bool BaseResource::RequestSubmit( BaseJob *job )
 		return true;
 	}
 
-	if ( submitsAllowed.size() < jobLimit &&
-		 submitsWanted.size() > 0 ) {
-		EXCEPT("In BaseResource for %s, SubmitsWanted is not empty and SubmitsAllowed is not full",resourceName);
-	}
+	// If submitsAllowed is under jobLimit, then submitsWanted must be empty
+	ASSERT(submitsAllowed.size() >= jobLimit || submitsWanted.size() == 0);
 	if ( submitsAllowed.size() < jobLimit ) {
 		submitsAllowed.insert(job);
 		SetJobPollInterval();
@@ -474,7 +473,7 @@ void BaseResource::Ping( int /* timerID */ )
 	// Don't start a new ping too soon after the previous one. If the
 	// resource is up, the minimum time between pings is probeDelay. If the
 	// resource is down, the minimum time between pings is probeInterval.
-	int delay;
+	time_t delay;
 	if ( resourceDown == false ) {
 		delay = (lastPing + probeDelay) - time(NULL);
 	} else {
@@ -487,7 +486,7 @@ void BaseResource::Ping( int /* timerID */ )
 
 	daemonCore->Reset_Timer( pingTimerId, TIMER_NEVER );
 
-	unsigned ping_delay;
+	time_t ping_delay;
 	bool ping_complete;
 	bool ping_succeeded;
 	DoPing( ping_delay, ping_complete, ping_succeeded );
@@ -549,7 +548,7 @@ void BaseResource::Ping( int /* timerID */ )
 	}
 }
 
-void BaseResource::DoPing( unsigned& ping_delay, bool& ping_complete,
+void BaseResource::DoPing(time_t& ping_delay, bool& ping_complete,
 						   bool& ping_succeeded )
 {
 	ping_delay = 0;
@@ -569,11 +568,11 @@ time_t BaseResource::GetLeaseExpiration( const BaseJob *job ) const
 	// Otherwise, if we haven't established a shared lease yet,
 	// calculate one for this job only.
 	time_t new_expiration = 0;
-	int job_lease_duration = m_defaultLeaseDuration;
+	time_t job_lease_duration = m_defaultLeaseDuration;
 	job->jobAd->LookupInteger( ATTR_JOB_LEASE_DURATION, job_lease_duration );
 	if ( job_lease_duration > 0 ) {
 		new_expiration = m_sharedLeaseExpiration > 0 ?
-			m_sharedLeaseExpiration : time(NULL) + job_lease_duration;
+			m_sharedLeaseExpiration : time(nullptr) + job_lease_duration;
 	}
 	return new_expiration;
 }
@@ -587,39 +586,39 @@ void BaseResource::RequestUpdateLeases() const
 
 void BaseResource::UpdateLeases( int /* timerID */ )
 {
-dprintf(D_FULLDEBUG,"*** UpdateLeases called\n");
+	dprintf(D_FULLDEBUG,"*** UpdateLeases called\n");
 	if ( hasLeases == false ) {
-dprintf(D_FULLDEBUG,"    Leases not supported, cancelling timer\n" );
+		dprintf(D_FULLDEBUG,"    Leases not supported, cancelling timer\n" );
 		daemonCore->Cancel_Timer( updateLeasesTimerId );
 		updateLeasesTimerId = TIMER_UNSET;
 		return;
 	}
 
 	// Don't start a new lease update too soon after the previous one.
-	int delay;
-	delay = (lastUpdateLeases + UPDATE_LEASE_DELAY) - time(NULL);
+	time_t delay;
+	delay = (lastUpdateLeases + UPDATE_LEASE_DELAY) - time(nullptr);
 	if ( delay > 0 ) {
 		daemonCore->Reset_Timer( updateLeasesTimerId, delay );
-dprintf(D_FULLDEBUG,"    UpdateLeases: last update too recent, delaying %d secs\n",delay);
+		dprintf(D_FULLDEBUG,"    UpdateLeases: last update too recent, delaying %lld secs\n",(long long)delay);
 		return;
 	}
 
 	daemonCore->Reset_Timer( updateLeasesTimerId, TIMER_NEVER );
 
     if ( updateLeasesActive == false ) {
-		int new_lease_duration = INT_MAX;
-dprintf(D_FULLDEBUG,"    UpdateLeases: calc'ing new leases\n");
+		time_t new_lease_duration = std::numeric_limits<time_t>::max();
+		dprintf(D_FULLDEBUG,"    UpdateLeases: calc'ing new leases\n");
 		for (auto curr_job: registeredJobs) {
-			int job_lease_duration = m_defaultLeaseDuration;
+			time_t job_lease_duration = m_defaultLeaseDuration;
 			curr_job->jobAd->LookupInteger( ATTR_JOB_LEASE_DURATION, job_lease_duration );
 			if ( job_lease_duration > 0 && job_lease_duration < new_lease_duration ) {
 				new_lease_duration = job_lease_duration;
 			}
 		}
-dprintf(D_FULLDEBUG,"    UpdateLeases: new shared lease duration: %d\n", new_lease_duration );
+		dprintf(D_FULLDEBUG,"    UpdateLeases: new shared lease duration: %lld\n", (long long) new_lease_duration);
 		// This is how close to the lease expiration time we want to be
 		// when we try a renewal.
-		int renew_threshold = ( new_lease_duration * 2 / 3 ) + 10;
+		time_t renew_threshold = ( new_lease_duration * 2 / 3 ) + 10;
 		if ( new_lease_duration == INT_MAX ||
 			 m_sharedLeaseExpiration > time(NULL) + renew_threshold ) {
 			// Lease doesn't need renewal, yet.
@@ -628,7 +627,7 @@ dprintf(D_FULLDEBUG,"    UpdateLeases: new shared lease duration: %d\n", new_lea
 				 next_renew_time > time(NULL) + 3600 ) {
 				next_renew_time = time(NULL) + 3600;
 			}
-dprintf(D_FULLDEBUG,"    UpdateLeases: nothing to renew, resetting timer for %ld secs\n",next_renew_time - time(NULL));
+			dprintf(D_FULLDEBUG,"    UpdateLeases: nothing to renew, resetting timer for %lld secs\n",(long long) next_renew_time - time(nullptr));
 			lastUpdateLeases = time(NULL);
 			daemonCore->Reset_Timer( updateLeasesTimerId,
 									 next_renew_time - time(NULL) );
@@ -647,7 +646,7 @@ dprintf(D_FULLDEBUG,"    UpdateLeases: nothing to renew, resetting timer for %ld
 					}
 				}
 			}
-dprintf(D_FULLDEBUG,"    new shared lease expiration at %ld, performing renewal...\n",m_sharedLeaseExpiration);
+			dprintf(D_FULLDEBUG,"    new shared lease expiration at %ld, performing renewal...\n",m_sharedLeaseExpiration);
 			requestScheddUpdateNotification( updateLeasesTimerId );
 			updateLeasesActive = true;
 		}
@@ -657,7 +656,7 @@ dprintf(D_FULLDEBUG,"    new shared lease expiration at %ld, performing renewal.
 	bool update_complete;
 	std::vector<PROC_ID> update_succeeded;
 	bool update_success;
-dprintf(D_FULLDEBUG,"    UpdateLeases: calling DoUpdateLeases\n");
+	dprintf(D_FULLDEBUG,"    UpdateLeases: calling DoUpdateLeases\n");
 	if ( m_hasSharedLeases ) {
 		DoUpdateSharedLease( update_delay, update_complete, update_success );
 	} else {
@@ -666,17 +665,17 @@ dprintf(D_FULLDEBUG,"    UpdateLeases: calling DoUpdateLeases\n");
 
 	if ( update_delay ) {
 		daemonCore->Reset_Timer( updateLeasesTimerId, update_delay );
-dprintf(D_FULLDEBUG,"    UpdateLeases: DoUpdateLeases wants delay of %u secs\n",update_delay);
+		dprintf(D_FULLDEBUG,"    UpdateLeases: DoUpdateLeases wants delay of %u secs\n",update_delay);
 		return;
 	}
 
 	if ( !update_complete ) {
 		updateLeasesCmdActive = true;
-dprintf(D_FULLDEBUG,"    UpdateLeases: DoUpdateLeases in progress\n");
+		dprintf(D_FULLDEBUG,"    UpdateLeases: DoUpdateLeases in progress\n");
 		return;
 	}
 
-dprintf(D_FULLDEBUG,"    UpdateLeases: DoUpdateLeases complete, processing results\n");
+	dprintf(D_FULLDEBUG,"    UpdateLeases: DoUpdateLeases complete, processing results\n");
 	bool first_update = lastUpdateLeases == 0;
 
 	updateLeasesCmdActive = false;
@@ -703,13 +702,13 @@ formatstr_cat(msg, " %d.%d", curr_id.cluster, curr_id.proc);
 				itr->second->UpdateJobLeaseSent(m_sharedLeaseExpiration);
 			}
 		}
-dprintf(D_FULLDEBUG,"%s\n",msg.c_str());
+		dprintf(D_FULLDEBUG,"%s\n",msg.c_str());
 		leaseUpdates.clear();
 	}
 
 	updateLeasesActive = false;
 
-dprintf(D_FULLDEBUG,"    UpdateLeases: lease update complete, resetting timer for 30 secs\n");
+	dprintf(D_FULLDEBUG,"    UpdateLeases: lease update complete, resetting timer for 30 secs\n");
 	daemonCore->Reset_Timer( updateLeasesTimerId, UPDATE_LEASE_DELAY );
 }
 
@@ -717,7 +716,7 @@ void BaseResource::DoUpdateLeases( unsigned& update_delay,
                                    bool& update_complete,
                                    std::vector<PROC_ID>& /* update_succeeded */ )
 {
-dprintf(D_FULLDEBUG,"*** BaseResource::DoUpdateLeases called\n");
+	dprintf(D_FULLDEBUG,"*** BaseResource::DoUpdateLeases called\n");
 	update_delay = 0;
 	update_complete = true;
 }
@@ -726,7 +725,7 @@ void BaseResource::DoUpdateSharedLease( unsigned& update_delay,
 										bool& update_complete,
 										bool& update_succeeded )
 {
-dprintf(D_FULLDEBUG,"*** BaseResource::DoUpdateSharedLease called\n");
+	dprintf(D_FULLDEBUG,"*** BaseResource::DoUpdateSharedLease called\n");
 	update_delay = 0;
 	update_complete = true;
 	update_succeeded = false;
@@ -734,9 +733,7 @@ dprintf(D_FULLDEBUG,"*** BaseResource::DoUpdateSharedLease called\n");
 
 void BaseResource::StartBatchStatusTimer()
 {
-	if(m_batchPollTid != TIMER_UNSET) {
-		EXCEPT("BaseResource::StartBatchStatusTimer called more than once!");
-	}
+	ASSERT(m_batchPollTid == TIMER_UNSET);
 	dprintf(D_FULLDEBUG, "Grid type for %s will use batch status requests (DoBatchStatus).\n", ResourceName());
 	m_batchPollTid = daemonCore->Register_Timer( 0,
 		(TimerHandlercpp)&BaseResource::DoBatchStatus,
