@@ -178,7 +178,7 @@ CondorJob::CondorJob( ClassAd *classad )
 	newRemoteStatusServerTime = 0;
 	doActivePoll = false;
 	m_remoteJobFinished = false;
-	int int_value;
+	time_t int_value;
 
 	lastRemoteStatusServerTime = 0;
 
@@ -200,7 +200,7 @@ CondorJob::CondorJob( ClassAd *classad )
 
 	int_value = 0;
 	if ( jobAd->LookupInteger( ATTR_DELEGATED_PROXY_EXPIRATION, int_value ) ) {
-		delegatedProxyExpireTime = (time_t)int_value;
+		delegatedProxyExpireTime = int_value;
 		delegatedProxyRenewTime = GetDelegatedProxyRenewalTime(delegatedProxyExpireTime);
 	}
 
@@ -364,7 +364,7 @@ void CondorJob::doEvaluateState( int /* timerID */ )
 	int old_gm_state;
 	int old_remote_state;
 	bool reevaluate_state = true;
-	time_t now = time(NULL);
+	time_t now = time(nullptr);
 
 	int rc;
 
@@ -448,7 +448,7 @@ void CondorJob::doEvaluateState( int /* timerID */ )
 			} break;
 		case GM_RECOVER_POLL: {
 			int num_ads = 0;
-			int tmp_int = 0;
+			time_t tmp_int = 0;
 			ClassAd **status_ads = NULL;
 			std::string constraint;
 			formatstr( constraint, "%s==%d&&%s==%d", ATTR_CLUSTER_ID,
@@ -496,13 +496,13 @@ void CondorJob::doEvaluateState( int /* timerID */ )
 					// Copy any attributes we care about from the remote
 					// ad to our local one before doing anything else
 				ProcessRemoteAd( status_ads[0] );
-				int server_time;
+				time_t server_time;
 				if ( status_ads[0]->LookupInteger( ATTR_SERVER_TIME,
 												   server_time ) == 0 ) {
 					dprintf( D_ALWAYS, "(%d.%d) Ad from remote schedd has "
 							 "no %s, faking with current local time\n",
 							 procID.cluster, procID.proc, ATTR_SERVER_TIME );
-					server_time = time(NULL);
+					server_time = time(nullptr);
 				}
 				lastRemoteStatusServerTime = server_time;
 
@@ -558,9 +558,9 @@ void CondorJob::doEvaluateState( int /* timerID */ )
 				if ( myResource->RequestSubmit(this) == false ) {
 					break;
 				}
-				char *job_id_string = NULL;
-				if ( gahpAd == NULL ) {
-					int new_expiration = myResource->GetLeaseExpiration( this );
+				char *job_id_string = nullptr;
+				if ( gahpAd == nullptr ) {
+					time_t new_expiration = myResource->GetLeaseExpiration( this );
 					if ( new_expiration > 0 ) {
 							// This will set the job lease sent attrs,
 							// which get referenced in buildSubmitAd()
@@ -623,7 +623,7 @@ void CondorJob::doEvaluateState( int /* timerID */ )
 			} else if ( condorState == REMOVED || condorState == HELD ) {
 				gmState = GM_UNSUBMITTED;
 			} else {
-				unsigned int delay = 0;
+				time_t delay = 0;
 				if ( (lastSubmitAttempt + submitInterval) > now ) {
 					delay = (lastSubmitAttempt + submitInterval) - now;
 				}				
@@ -675,8 +675,7 @@ void CondorJob::doEvaluateState( int /* timerID */ )
 					if( actual_expiration == 0 || actual_expiration > jobProxy->expiration_time ) {
 						actual_expiration = jobProxy->expiration_time;
 					}
-					jobAd->Assign( ATTR_DELEGATED_PROXY_EXPIRATION,
-								   (int)actual_expiration );
+					jobAd->Assign(ATTR_DELEGATED_PROXY_EXPIRATION, actual_expiration);
 				}
 				gmState = GM_POLL_ACTIVE;
 			} else {
@@ -736,7 +735,7 @@ void CondorJob::doEvaluateState( int /* timerID */ )
 				} else {
 					dprintf( D_ALWAYS, "(%d.%d) Delaying refresh of proxy\n",
 							 procID.cluster, procID.proc );
-					unsigned int delay = 0;
+					time_t delay = 0;
 					delay = (lastProxyRefreshAttempt + interval) - now;
 					daemonCore->Reset_Timer( evaluateStateTid, delay );
 				}
@@ -787,10 +786,9 @@ void CondorJob::doEvaluateState( int /* timerID */ )
 					if( actual_expiration == 0 || actual_expiration > jobProxy->expiration_time ) {
 						actual_expiration = jobProxy->expiration_time;
 					}
-					jobAd->Assign( ATTR_DELEGATED_PROXY_EXPIRATION,
-								   (int)actual_expiration );
+					jobAd->Assign(ATTR_DELEGATED_PROXY_EXPIRATION, actual_expiration );
 				}
-				lastProxyRefreshAttempt = time(NULL);
+				lastProxyRefreshAttempt = time(nullptr);
 				gmState = GM_SUBMITTED;
 			}
 		} break;
@@ -916,13 +914,13 @@ void CondorJob::doEvaluateState( int /* timerID */ )
 				break;
 			}
 			ProcessRemoteAd( status_ads[0] );
-			int server_time;
+			time_t server_time;
 			if ( status_ads[0]->LookupInteger( ATTR_SERVER_TIME,
 											   server_time ) == 0 ) {
 				dprintf( D_ALWAYS, "(%d.%d) Ad from remote schedd has no %s, "
 						 "faking with current local time\n",
 						 procID.cluster, procID.proc, ATTR_SERVER_TIME );
-				server_time = time(NULL);
+				server_time = time(nullptr);
 			}
 			lastRemoteStatusServerTime = server_time;
 			delete status_ads[0];
@@ -1221,7 +1219,7 @@ void CondorJob::SetRemoteJobId( const char *job_id )
 
 void CondorJob::NotifyNewRemoteStatus( ClassAd *update_ad )
 {
-	int tmp_int;
+	time_t tmp_int;
 	if ( update_ad == NULL ) {
 			// This job was missing from a collective status query. Trigger
 			// a specific query to see what's wrong.
@@ -1354,7 +1352,7 @@ void CondorJob::ProcessRemoteAd( ClassAd *remote_ad )
 		}
 	}
 	if ( new_remote_state == COMPLETED ) {
-		int tmp_val;
+		time_t tmp_val;
 		if ( remote_ad->LookupInteger( ATTR_JOB_FINISHED_HOOK_DONE, tmp_val ) ) {
 			m_remoteJobFinished = true;
 		}
@@ -1422,10 +1420,10 @@ BaseResource *CondorJob::GetResource()
 // New black-list version
 ClassAd *CondorJob::buildSubmitAd()
 {
-	int now = time(NULL);
+	time_t now = time(nullptr);
 	std::string expr;
 	ClassAd *submit_ad;
-	int tmp_int;
+	time_t tmp_int;
 
 		// Base the submit ad on our own job ad
 	submit_ad = new ClassAd( *jobAd );
@@ -1434,6 +1432,7 @@ ClassAd *CondorJob::buildSubmitAd()
 	submit_ad->Delete( ATTR_PROC_ID );
 	submit_ad->Delete( ATTR_USER );
 	submit_ad->Delete( ATTR_OWNER );
+	submit_ad->Delete( ATTR_OS_USER );
 	submit_ad->Delete( ATTR_GRID_RESOURCE );
 	submit_ad->Delete( ATTR_JOB_MANAGED );
 	submit_ad->Delete( ATTR_STAGE_IN_FINISH );
@@ -1634,10 +1633,10 @@ ClassAd *CondorJob::buildSubmitAd()
 	// JobLeaseDuration. Otherwise, starter-shadow reconnect is disabled.
 	// We need to be careful to respect the user's setting of the
 	// attribute, hence why we do this check last.
-	tmp_int = CONDOR_UNIVERSE_VANILLA;
-	submit_ad->LookupInteger( ATTR_JOB_UNIVERSE, tmp_int );
-	if ( universeCanReconnect( tmp_int ) &&
-		 submit_ad->Lookup( ATTR_JOB_LEASE_DURATION ) == NULL ) {
+	int universe_id = CONDOR_UNIVERSE_VANILLA;
+	submit_ad->LookupInteger( ATTR_JOB_UNIVERSE, universe_id );
+	if ( universeCanReconnect( universe_id ) &&
+		 submit_ad->Lookup( ATTR_JOB_LEASE_DURATION ) == nullptr ) {
 
 		submit_ad->Assign( ATTR_JOB_LEASE_DURATION, 40 * 60 );
 	}
