@@ -428,7 +428,7 @@ void CondorResource::DoScheddPoll( int /* timerID */ )
 	}
 }
 
-void CondorResource::DoPing( unsigned& ping_delay, bool& ping_complete,
+void CondorResource::DoPing(time_t& ping_delay, bool& ping_complete,
 							 bool& ping_succeeded )
 {
 	int rc;
