@@ -48,6 +48,7 @@ usage( char *name )
 void
 main_init( int argc, char ** const argv )
 {
+	const char* os_name = nullptr;
 
 	// Setup dprintf to display pid
 	DebugId = display_dprintf_header;
@@ -100,16 +101,33 @@ main_init( int argc, char ** const argv )
 			i++;
 			break;
 		case 'u':
-			if ( argc <= i + 1 )
+			// What job owner are we managing jobs for?
+			if ( argc <= i + 1 ) {
 				usage( argv[0] );
+			}
+			if (myUserName) free(myUserName); // If multiple -u's
 			myUserName = strdup(argv[i + 1]);
 			i++;
 			break;
 		case 'o':
-			// We handled this in main_pre_dc_init(), so just verify that
-			// it has an argument.
-			if ( argc <= i + 1 )
+			// Say what OS account we're running jobs under.
+			// If the schedd starts us as root, we need to switch to
+			// this uid for most of our life.
+			if ( argc <= i + 1 ) {
 				usage( argv[0] );
+			}
+			os_name = argv[i + 1];
+			i++;
+			break;
+		case 'n':
+			if (argc <= i + 1) {
+				usage(argv[0]);
+			}
+			if (ScheddName) {
+				free(ScheddName);
+				ScheddName = nullptr;
+			}
+			ScheddName = strdup(argv[i + 1]);
 			i++;
 			break;
 		default:
@@ -126,7 +144,22 @@ main_init( int argc, char ** const argv )
 
 	// Tell DaemonCore that we want to spend all our time as the job owner,
 	// not as user condor.
-	daemonCore->Register_Priv_State( PRIV_USER );
+	if (os_name) {
+		std::string buf;
+		const char *owner = name_of_user(os_name, buf);
+		const char *domain = domain_of_user(os_name, nullptr);
+		if ( !init_user_ids(owner, domain)) {
+			dprintf(D_ALWAYS, "init_user_ids() failed!\n");
+			// uids.cpp will EXCEPT when we set_user_priv() now
+			// so there's not much we can do at this point
+		}
+		set_user_priv();
+		daemonCore->Register_Priv_State( PRIV_USER );
+	} else if ( is_root() ) {
+		dprintf( D_ALWAYS, "Don't know what user to run as!\n" );
+		DC_Exit( 1 );
+	}
+
 	set_user_priv();
 
 	Init();
@@ -151,45 +184,6 @@ main_shutdown_graceful()
 	DC_Exit(0);
 }
 
-void
-main_pre_dc_init( int argc, char* argv[] )
-{
-	// handle -o, so that we can switch euid to the user before
-	// daemoncore does most of its initialization work.
-	const char *os_name = nullptr;
-	int i = 1;
-	while ( i < argc ) {
-		if ( !strcmp( argv[i], "-o" ) ) {
-			// Say what user we're running jobs on behave of.
-			// If the schedd starts us as root, we need to switch to
-			// this uid for most of our life.
-			if ( argc <= i + 1 ) {
-				usage( argv[0] );
-			}
-			os_name = argv[i + 1];
-			break;
-		}
-		i++;
-	}
-
-	if ( os_name ) {
-		std::string buf;
-		const char *owner = name_of_user(os_name, buf);
-		const char *domain = domain_of_user(os_name, nullptr);
-		if ( !init_user_ids(owner, domain)) {
-			dprintf(D_ALWAYS, "init_user_ids() failed!\n");
-			// uids.C will EXCEPT when we set_user_priv() now
-			// so there's not much we can do at this point
-		}
-		set_user_priv();
-		// We can't call daemonCore->Register_Priv_State() here because
-		// there's no daemonCore object yet. We'll call it in main_init().
-	} else if ( is_root() ) {
-		dprintf( D_ALWAYS, "Don't know what user to run as!\n" );
-		DC_Exit( 1 );
-	}
-}
-
 int
 main( int argc, char **argv )
 {
@@ -199,7 +193,6 @@ main( int argc, char **argv )
 	dc_main_config = main_config;
 	dc_main_shutdown_fast = main_shutdown_fast;
 	dc_main_shutdown_graceful = main_shutdown_graceful;
-	dc_main_pre_dc_init = main_pre_dc_init;
 	return dc_main( argc, argv );
 }
 
