@@ -867,7 +867,7 @@ dprintf( D_ALWAYS, "... found myself in list: %s\n", s.getSinful() );
     win in checkList election process
 */
 bool
-HADStateMachine::checkList( std::set<int> list ) const
+HADStateMachine::checkList( const std::set<int>& list ) const
 {
 	for (auto id : list) {
 		if (id > m_selfId) {
