@@ -34,6 +34,7 @@
 #define JOB_ADTYPE			"Job"
 #define JOB_SET_ADTYPE			"JobSet"
 #define OWNER_ADTYPE		"Owner"	 /* schedd's owner-of-jobs ad */
+#define PROJECT_ADTYPE		"Project" /* schedd's project ad */
 #define QUERY_ADTYPE			"Query"
 #define COLLECTOR_ADTYPE		"Collector"
 #define CKPT_FILE_ADTYPE		"CkptFile"
@@ -44,7 +45,7 @@
 #define NEGOTIATOR_ADTYPE		"Negotiator"
 #define HAD_ADTYPE			"HAD"
 #define REPLICATION_ADTYPE		"Replication"
-#define XFER_SERVICE_ADTYPE		"XferService"		/* No longer used */
+#define PLACEMENTD_ADTYPE		"PlacementD"
 #define LEASE_MANAGER_ADTYPE		"LeaseManager"	/* No longer used */
 #define CREDD_ADTYPE			"CredD"
 #define JOB_ROUTER_ADTYPE		"Job_Router"
@@ -55,6 +56,7 @@
 #define REPLY_ADTYPE			"Reply"
 #define GATEWAY_ADTYPE			"Gateway"
 #define CLUSTER_ADTYPE	 		"Cluster"
+#define CLUSTERPVT_ADTYPE		"ClusterPvt"
 #define GRID_ADTYPE			"Grid"
 #define BOGUS_ADTYPE		"Bogus"
 
@@ -82,7 +84,7 @@ typedef enum : long
 	DATABASE_AD,
 	TT_AD,
 	GRID_AD,
-	XFER_SERVICE_AD,	// placeholder: this type no longer used
+	PLACEMENTD_AD,
 	LEASE_MANAGER_AD,	// placeholder: this type no longer used
 	DEFRAG_AD,
 	ACCOUNTING_AD,
