@@ -76,21 +76,27 @@
 #define ATTR_CAPABILITY  "Capability"
 #define ATTR_CGROUP_ENFORCED  "CgroupEnforced"
 #define ATTR_CE_REQUIREMENTS  "CERequirements"
+#define ATTR_CHIRP_DELAYED_ATTRS  "ChirpDelayedAttrs"
 #define ATTR_CLAIM_STARTD  "ClaimStartd"
 #define ATTR_COD_CLAIMS  "CODClaims"
 #define ATTR_COLLECTOR_HOST  "CollectorHost"
 #define ATTR_COMMAND  "Command"
+#define ATTR_COMMON_INPUT_FILES "CommonInputFiles"
 #define ATTR_COMPRESS_FILES  "CompressFiles"
 #define ATTR_CONTAINER_SERVICE_NAMES "ContainerServiceNames"
 #define ATTR_CONTAINER_DURATION "ContainerDuration"
 #define ATTR_CONTAINER_PORT_SUFFIX "_ContainerPort"
 #define ATTR_CONTAINER_TARGET_DIR "ContainerTargetDir"
+#define ATTR_CONTEXT_AD "ContextAd"
+#define ATTR_REQUEST_RESULT_CODE "RequestResultCode"
+#define ATTR_REQUEST_RESULT_SUBCODE "RequestResultSubCode"
 #define ATTR_REQUESTED_CAPACITY  "RequestedCapacity"
 #define ATTR_CHECKPOINT_SIG  "CheckpointSig"
 #define ATTR_CHILD_CLAIM_IDS "ChildClaimIds"
 #define ATTR_CLAIM_END_TIME "ClaimEndTime"
 #define ATTR_CLAIM_ID  "ClaimId"
 #define ATTR_CLAIM_IDS  "ClaimIds"
+#define ATTR_SPLIT_CLAIM_ID "SplitClaimId"
 #define ATTR_PUBLIC_CLAIM_ID  "PublicClaimId"
 #define ATTR_PUBLIC_CLAIM_IDS  "PublicClaimIds"
 //extern const char ATTR_CHILD_REMOVE_CONSTRAINT [];
@@ -108,6 +114,7 @@
 #define ATTR_MATCHED_PSG  "Matched_PSG"
 #define ATTR_CONCURRENCY_LIMITS  "ConcurrencyLimits"
 #define ATTR_PREEMPTING_CONCURRENCY_LIMITS  "PreemptingConcurrencyLimits"
+#define ATTR_CLEANUP_CATEGORY_COUNTS  "CleanupCategoryCounts"
 #define ATTR_CONDOR_LOAD_AVG			"CondorLoadAvg"
 #define ATTR_CONDOR_SCRATCH_DIR			"CondorScratchDir"
 #define ATTR_CONDOR_ADMIN				"CondorAdmin"
@@ -146,9 +153,11 @@
 #define ATTR_DAGMAN_JOB_ID  "DAGManJobId"
 #define ATTR_DAGMAN_WORKFLOW_LOG  "DAGManNodesLog"
 #define ATTR_DAGMAN_WORKFLOW_MASK  "DAGManNodesMask"
+#define ATTR_DAG_LIFETIME_JOB "DAGLifetimeJob"
 #define ATTR_DAG_IN_RECOVERY "DAG_InRecovery"
 #define ATTR_DAG_NODE_NAME  "DAGNodeName"
 #define ATTR_DAG_NODE_NAME_ALT  "dag_node_name"
+#define ATTR_DAG_ADDRESS "DAG_Address"
 #define ATTR_DAG_NODES_DONE "DAG_NodesDone"
 #define ATTR_DAG_NODES_FAILED "DAG_NodesFailed"
 #define ATTR_DAG_NODES_POSTRUN "DAG_NodesPostrun"
@@ -180,13 +189,16 @@
 #define ATTR_DISABLE_REASON "DisableReason"
 #define ATTR_DISK  "Disk"
 #define ATTR_DISK_USAGE  "DiskUsage"
+#define ATTR_DISK_USAGE_ENFORCED "IsEnforcingDiskUsage"
 #define ATTR_SCRATCH_DIR_FILE_COUNT "ScratchDirFileCount"
+#define ATTR_IS_SPLITTABLE "IsSplittable"
 #define ATTR_DOCKER_CACHED_IMAGE_SIZE "DockerCachedImageSizeMb"
 #define ATTR_DOCKER_IMAGE "DockerImage"
 #define ATTR_DOCKER_NETWORKS "DockerNetworks"
 #define ATTR_DOCKER_NETWORK_TYPE "DockerNetworkType"
 #define ATTR_DOCKER_OVERRIDE_ENTRYPOINT  "DockerOverrideEntrypoint"
 #define ATTR_DOCKER_PULL_POLICY "DockerPullPolicy"
+#define ATTR_DOCKER_SEND_CREDENTIALS "DockerSendCredentials"
 #define ATTR_DOCKER_VOLUMES "DockerVolumes"
 #define ATTR_DOCKER_VERSION  "DockerVersion"
 #define ATTR_EMAIL_ATTRIBUTES  "EmailAttributes"
@@ -194,6 +206,7 @@
 #define ATTR_ENTERED_CURRENT_ACTIVITY  "EnteredCurrentActivity"
 #define ATTR_ENTERED_CURRENT_STATE  "EnteredCurrentState"
 #define ATTR_ENTERED_CURRENT_STATUS  "EnteredCurrentStatus"
+#define ATTR_EPOCH_AD_TYPE "EpochAdType"
 #define ATTR_ERROR_STRING  "ErrorString"
 #define ATTR_ERROR_CODE  "ErrorCode"
 #define ATTR_WARNING "Warning"
@@ -224,6 +237,7 @@
 #define ATTR_FLOCKED_JOBS  "FlockedJobs"
 #define ATTR_FLOCK_TO "FlockTo"
 #define ATTR_EFFECTIVE_FLOCK_LIST "EffectiveFlockList"
+#define ATTR_FIRST_JOB_MATCH_DATE "FirstJobMatchDate"
 #define ATTR_FLAVOR  "Flavor"
 #define ATTR_FORCE  "Force"
 #define ATTR_GAHP_PID "GahpPid"
@@ -240,6 +254,7 @@
 #define ATTR_GID  "Gid"
 #define ATTR_GLOBAL_JOB_ID  "GlobalJobId"
 #define ATTR_GZIP  "GZIP"
+#define ATTR_X509_USER_prefix  "x509User"
 #define ATTR_X509_USER_PROXY  "x509userproxy"
 #define ATTR_X509_USER_PROXY_EXPIRATION  "x509UserProxyExpiration"
 #define ATTR_X509_USER_PROXY_SUBJECT  "x509userproxysubject"
@@ -247,11 +262,13 @@
 #define ATTR_X509_USER_PROXY_VONAME  "x509UserProxyVOName"
 #define ATTR_X509_USER_PROXY_FIRST_FQAN  "x509UserProxyFirstFQAN"
 #define ATTR_X509_USER_PROXY_FQAN  "x509UserProxyFQAN"
+#define ATTR_TOKEN_prefix "AuthToken"
 #define ATTR_TOKEN_SUBJECT "AuthTokenSubject"
 #define ATTR_TOKEN_ISSUER "AuthTokenIssuer"
 #define ATTR_TOKEN_GROUPS "AuthTokenGroups"
 #define ATTR_TOKEN_SCOPES "AuthTokenScopes"
 #define ATTR_TOKEN_ID "AuthTokenId"
+#define ATTR_TOKEN_PROJECT "AuthTokenProject"
 #define ATTR_DELEGATED_PROXY_EXPIRATION  "DelegatedProxyExpiration"
 #define ATTR_GRID_RESOURCE  "GridResource"
 #define ATTR_GRID_RESOURCE_UNAVAILABLE_REASON  "GridResourceUnavailableReason"
@@ -263,9 +280,11 @@
 #define ATTR_MYPROXY_PASSWORD  "MyProxyPassword"
 #define ATTR_HARDWARE_ADDRESS  "HardwareAddress"
 #define ATTR_HAS_CHECKPOINTING  "HasCheckpointing"
+#define ATTR_HAS_COMMON_FILES_TRANSFER  "HasCommonFilesTransfer"
 #define ATTR_HAS_CONTAINER  "HasContainer"
 #define ATTR_HAS_DOCKER  "HasDocker"
 #define ATTR_HAS_DOCKER_URL  "HasDockerURL"
+#define ATTR_DOCKER_OFFLINE_REASON "DockerOfflineReason"
 #define ATTR_HAS_START_DAEMON_AD  "HasStartDaemonAd"
 #define ATTR_HAS_SANDBOX_IMAGE  "HasSandboxImage"
 #define ATTR_HAS_SIF  "HasSIF"
@@ -275,6 +294,7 @@
 #define ATTR_HAS_FILE_TRANSFER_PLUGIN_METHODS  "HasFileTransferPluginMethods"
 #define ATTR_HAS_JOB_TRANSFER_PLUGINS  "HasJobTransferPlugins"
 #define ATTR_HAS_PER_FILE_ENCRYPTION  "HasPerFileEncryption"
+#define ATTR_HAS_JOB_NETWORKING  "HasJobNetworking"
 #define ATTR_HAS_IO_PROXY  "HasIOProxy"
 #define ATTR_HAS_JAVA  "HasJava"
 #define ATTR_HAS_JIC_LOCAL_CONFIG  "HasJICLocalConfig"
@@ -282,6 +302,7 @@
 #define ATTR_HAS_JOB_AD  "HasJobAd"
 #define ATTR_HAS_JOB_AD_FROM_FILE  "HasJobAdFromFile"
 #define ATTR_HAS_JOB_DEFERRAL  "HasJobDeferral"
+#define ATTR_HAS_OS_USER  "HasOsUser"
 #define ATTR_HAS_MPI  "HasMPI"
 #define ATTR_HAS_OLD_VANILLA  "HasOldVanilla"
 #define ATTR_HAS_PVM  "HasPVM"
@@ -294,6 +315,7 @@
 #define ATTR_HAS_USER_NAMESPACES "HasUserNamespaces"
 #define ATTR_HAS_WIN_RUN_AS_OWNER  "HasWindowsRunAsOwner"
 #define ATTR_HAS_VM  "HasVM"
+#define ATTR_VM_OFFLINE_REASON  "VMOfflineReason"
 #define ATTR_HELD_JOBS  "HeldJobs"
 #define ATTR_HIBERNATION_LEVEL  "HibernationLevel"
 #define ATTR_HIBERNATION_STATE  "HibernationState"
@@ -301,9 +323,11 @@
 #define ATTR_HIBERNATION_RAW_MASK  "HibernationRawMask"
 #define ATTR_HIBERNATION_METHOD  "HibernationMethod"
 #define ATTR_UNHIBERNATE  "Unhibernate"
+#define ATTR_CONTAINER_IS_COMMON "ContainerIsCommon"
 #define ATTR_HOLD_KILL_SIG  "HoldKillSig"
 #define ATTR_HOOK_KEYWORD  "HookKeyword"
 #define ATTR_HISTORY_RECORD_SOURCE  "HistoryRecordSource"
+#define ATTR_DAEMON_HISTORY_SUBSYS  "DaemonHistorySubsys"
 #define ATTR_HISTORY_AD_TYPE_FILTER  "HistoryAdTypeFilter"
 #define ATTR_IDLE_JOBS  "IdleJobs"
 #define ATTR_IMAGE_SIZE  "ImageSize"
@@ -335,8 +359,10 @@
 #define ATTR_JOB_ALLOWED_EXECUTE_DURATION "AllowedExecuteDuration"
 #define ATTR_JOB_EPOCH_WRITE_DATE "EpochWriteDate"
 #define ATTR_JOB_LAST_SHADOW_EXCEPTION    "LastShadowException"
+#define ATTR_JOB_LAST_FILE_TRANSFER_ERROR_PROTOCOL "LastFileTransferErrorProtocol"
 #define ATTR_JOB_COOL_DOWN_EXPIRATION "JobCoolDownExpiration"
 #define ATTR_NUM_JOB_COOL_DOWNS "NumJobCoolDowns"
+#define ATTR_JOB_MATERIALIZE_DATE         "JobMaterializeDate"
 #define ATTR_JOB_MATERIALIZE_DIGEST_FILE  "JobMaterializeDigestFile"
 #define ATTR_JOB_MATERIALIZE_ITEMS_FILE   "JobMaterializeItemsFile"
 #define ATTR_JOB_MATERIALIZE_LIMIT        "JobMaterializeLimit"
@@ -355,12 +381,19 @@
 #define ATTR_JOB_ORIG_CMD  "OrigCmd"
 #define ATTR_JOB_DESCRIPTION "JobDescription"
 #define ATTR_STACK_SIZE  "StackSize"
-#define ATTR_JOB_CMD_HASH  "CmdHash"
-#ifdef FIPS_MODE
-#define ATTR_JOB_CMD_CHECKSUM  "CmdSHA"
-#else
-#define ATTR_JOB_CMD_CHECKSUM  "CmdMD5"
-#endif
+#define ATTR_OCU  "OCU"
+#define ATTR_OCU_BORROWER_ACTIVATIONS  "OCUBorrowerActivations"
+#define ATTR_OCU_BORROWER_ACTIVATION_TIME  "OCUBorrowerActivationTime"
+#define ATTR_OCU_CLAIM      "OCUClaim"
+#define ATTR_OCU_CLAIMED_TIME      "OCUClaimedTime"
+#define ATTR_OCU_ACTIVATION_START_TIME      "OCUActivationStartTime"
+#define ATTR_OCU_OWNER_ACTIVATIONS     "OCUOwnerActivations"
+#define ATTR_OCU_OWNER_ACTIVATION_TIME "OCUOwnerActivationTime"
+#define ATTR_OCU_HOLDER  "IsOCUHolder"
+#define ATTR_OCU_ID  "OCUId"
+#define ATTR_OCU_NAME  "OCUName"
+#define ATTR_OCU_WANTED  "OCUWanted"
+#define ATTR_OCU_STATE  "OCUState"
 #define ATTR_ORIG_JOB_CMD  "OrigCmd" 
 #define ATTR_JOB_CORE_DUMPED  "JobCoreDumped"
 #define ATTR_JOB_CORE_FILENAME  "JobCoreFileName"
@@ -373,18 +406,21 @@
 #define ATTR_CUMULATIVE_TRANSFER_TIME  "CumulativeTransferTime"
 #define ATTR_JOB_DURATION  "JobDuration"
 #define ATTR_ENCRYPT_EXECUTE_DIRECTORY "EncryptExecuteDirectory"
+#define ATTR_EXECUTE_DIRECTORY_ENCRYPTED "ExecuteDirWasEncrypted"
 #define ATTR_JOB_ENV_V1  "Env"
 #define ATTR_JOB_ENV_V1_DELIM  "EnvDelim"
 #define ATTR_JOB_ENVIRONMENT  "Environment"
 #define ATTR_JOB_ERROR  "Err"
 #define ATTR_JOB_ORIGINAL_ERROR  "OriginalErr"
 #define ATTR_JOB_ERROR_SIZE  "ErrSize"
+#define ATTR_JOB_INITIAL_WAIT_DURATION "InitialWaitDuration"
 #define ATTR_JOB_CPU_INSTRUCTIONS  "CPUInstructions"
 #define ATTR_JOB_KEEP_CLAIM_IDLE  "KeepClaimIdle"
 #define ATTR_JOB_KEYWORD  "JobKeyword"
 #define ATTR_JOB_LEASE_DURATION  "JobLeaseDuration"
 #define ATTR_JOB_LEASE_EXPIRATION  "JobLeaseExpiration"
 #define ATTR_JOB_LIMIT_DIRECTORY_ACCESS "LimitDirectoryAccess"
+#define ATTR_JOB_MOUNT_UNDER_SCRATCH "MountUnderScratch"
 #define ATTR_JOB_SPOOL_EXECUTABLE  "SpoolExecutable"
 #define ATTR_JOB_EXIT_STATUS  "ExitStatus"
 #define ATTR_JOB_EXIT_REQUIREMENTS  "ExitRequirements"
@@ -413,8 +449,10 @@
 #define ATTR_JOB_NOTIFICATION  "JobNotification"
 #define ATTR_JOB_OUTPUT  "Out"
 #define ATTR_JOB_ORIGINAL_OUTPUT  "OriginalOut"
+#define ATTR_JOB_OS_HOME_DIR  "OSHomeDir"
 #define ATTR_JOB_OUTPUT_SIZE  "OutSize"
 #define ATTR_JOB_PID  "JobPid"
+#define ATTR_JOB_PRIMARY_UNIX_GROUP  "PrimaryUnixGroup"
 #define ATTR_PRE_JOB_PRIO1  "PreJobPrio1"
 #define ATTR_PRE_JOB_PRIO2  "PreJobPrio2"
 #define ATTR_POST_JOB_PRIO1  "PostJobPrio1"
@@ -449,6 +487,8 @@
 #define ATTR_JOB_START_DATE  "JobStartDate"
 #define ATTR_JOB_STATE  "JobState"
 #define ATTR_JOB_STATUS  "JobStatus"
+#define ATTR_JOB_STDOUT_MTIME "StdoutMtime"
+#define ATTR_JOB_STDERR_MTIME "StderrMtime"
 #define ATTR_LAST_JOB_STATUS  "LastJobStatus"
 #define ATTR_JOB_STARTER_DEBUG "JobStarterDebug"
 #define ATTR_JOB_STARTER_LOG "JobStarterLog"
@@ -500,6 +540,7 @@
 #define ATTR_LAST_MATCH_LIST_LENGTH  "LastMatchListLength"
 #define ATTR_LAST_REJ_MATCH_TIME  "LastRejMatchTime"
 #define ATTR_LAST_REJ_MATCH_REASON  "LastRejMatchReason"
+#define ATTR_LAST_REJ_MATCH_NEGOTIATOR "LastRejMatchNegotiator"
 #define ATTR_LAST_PERIODIC_CHECKPOINT  "LastPeriodicCheckpoint"
 #define ATTR_LAST_RELEASE_REASON  "LastReleaseReason"
 #define ATTR_LAST_REMOTE_HOST  "LastRemoteHost"
@@ -511,6 +552,11 @@
 #define ATTR_LOCAL_FILES  "LocalFiles"
 #define ATTR_LOCATION_QUERY  "LocationQuery"
 #define ATTR_LOAD_AVG  "LoadAvg"
+#define ATTR_LVM_USE_LOOPBACK "LvmUsingLoopback"
+#define ATTR_LVM_USE_THIN_PROVISION "LvmIsThinProvisioning"
+#define ATTR_LVM_BACKING_STORE "LvmBackingStore"
+#define ATTR_LVM_DETECTED_DISK "LvmDetectedDisk"
+#define ATTR_LVM_NON_CONDOR_USAGE "LvmNonCondorDiskUsage"
 #define ATTR_MACHINE  "Machine"
 #define ATTR_MACHINE_COUNT  "MachineCount"
 #define ATTR_MICROARCH  "Microarch"
@@ -524,11 +570,13 @@
 #define ATTR_MAX_JOBS_RUNNING  "MaxJobsRunning"
 #define ATTR_CURB_MATCHMAKING "CurbMatchmaking"
 #define ATTR_MATCH_CLAIMED_PSLOTS "MatchClaimedPslots"
+#define ATTR_MATCH_CAPS "MatchCaps"
 #define ATTR_MAX_CLAIM_TIME "MaxClaimTime"
 #define ATTR_MEMORY  "Memory"
 #define ATTR_MEMORY_USAGE  "MemoryUsage"
 #define ATTR_DETECTED_MEMORY  "DetectedMemory"
 #define ATTR_DETECTED_CPUS  "DetectedCpus"
+#define ATTR_DETECTED_DISK  "DetectedDisk"
 #define ATTR_MIN_HOSTS  "MinHosts"
 #define ATTR_MIPS  "Mips"
 #define ATTR_MPI_IS_MASTER  "MPIIsMaster"
@@ -539,10 +587,8 @@
 #define ATTR_MY_TYPE  "MyType"
 #define ATTR_NAME  "Name"
 #define ATTR_NICE_USER_deprecated  "NiceUser"
-#ifdef NO_DEPRECATE_NICE_USER
-#define ATTR_NICE_USER ATTR_NICE_USER_deprecated
-#endif
 
+#define ATTR_NEED_REFRESH  "NeedRefresh"
 #define ATTR_NEGOTIATOR_JOB_CONSTRAINT  "NegotiatorJobConstraint"
 #define ATTR_NEGOTIATOR_NAME  "NegotiatorName"
 #define ATTR_NEGOTIATOR_REQUIREMENTS  "NegotiatorRequirements"
@@ -554,6 +600,7 @@
 #define ATTR_NEXT_JOB_START_DELAY  "NextJobStartDelay"
 #define ATTR_NODE  "Node"
 #define ATTR_ARC_APPLICATION  "ArcApplication"
+#define ATTR_ARC_DATA_STAGING  "ArcDataStaging"
 #define ATTR_ARC_RTE  "ArcRte"
 #define ATTR_ARC_RESOURCES  "ArcResources"
 #define ATTR_NOTIFY_USER  "NotifyUser"
@@ -575,7 +622,9 @@
 #define ATTR_LIMIT_RESULTS "LimitResults"
 #define ATTR_NUM_HOLDS "NumHolds"
 #define ATTR_NUM_HOLDS_BY_REASON "NumHoldsByReason"
+#define ATTR_NUM_INPUT_XFER_STARTS "NumInputTransferStarts"
 #define ATTR_NUM_JOB_STARTS  "NumJobStarts"
+#define ATTR_NUM_OUTPUT_XFER_STARTS "NumOutputTransferStarts"
 #define ATTR_NUM_JOB_COMPLETIONS "NumJobCompletions"
 #define ATTR_NUM_JOB_RECONNECTS  "NumJobReconnects"
 #define ATTR_NUM_PIDS  "NumPids"
@@ -584,6 +633,10 @@
 #define ATTR_NUM_SHADOW_STARTS  "NumShadowStarts"
 #define ATTR_NUM_SYSTEM_HOLDS  "NumSystemHolds"
 #define ATTR_NUM_USERS  "NumUsers"
+#define ATTR_NUM_VACATES  "NumVacates"
+#define ATTR_NUM_VACATES_PRE_EXECUTION  "NumVacatesPreExecution"
+#define ATTR_NUM_VACATES_BY_REASON "NumVacatesByReason"
+#define ATTR_NUM_VACATES_BY_REASON_PRE_EXECUTION "NumVacatesByReasonPreExecution"
 #define ATTR_NUM_OWNERS "NumOwners"
 #define ATTR_OFFLINE  "Offline"
 #define ATTR_OPSYS  "OpSys"
@@ -595,8 +648,10 @@
 #define ATTR_OPSYS_SHORT_NAME  "OpSysShortName"
 #define ATTR_OPSYS_LEGACY  "OpSysLegacy"
 #define ATTR_ORIG_MAX_HOSTS  "OrigMaxHosts"
+#define ATTR_OS_USER  "OsUser"
 #define ATTR_OTHER_JOB_REMOVE_REQUIREMENTS  "OtherJobRemoveRequirements" 
 #define ATTR_OUTPUT_DESTINATION  "OutputDestination"
+#define ATTR_OUTPUT_DIRECTORY "OutputDirectory"
 #define ATTR_OWNER  "Owner" 
 #define ATTR_PARALLEL_SCHEDULING_GROUP  "ParallelSchedulingGroup"
 #define ATTR_PARALLEL_SCRIPT_SHADOW  "ParallelScriptShadow"  
@@ -633,6 +688,7 @@
 #define ATTR_REMOTE_NEGOTIATING_GROUP  "RemoteNegotiatingGroup"
 #define ATTR_REMOTE_OWNER  "RemoteOwner"
 #define ATTR_REMOTE_POOL  "RemotePool"
+#define ATTR_REMOTE_PROJECT  "RemoteProject"
 #define ATTR_REMOTE_SCHEDD_NAME  "RemoteScheddName"
 #define ATTR_REMOTE_SLOT_ID  "RemoteSlotID"
 #define ATTR_REMOTE_SPOOL_DIR  "RemoteSpoolDir"
@@ -667,6 +723,8 @@
 #define ATTR_ADDRESS_V1 "AddressV1"
 #define ATTR_NUM_JOB_STARTS_DELAYED  "NumJobStartsDelayed"
 #define ATTR_NUM_PENDING_CLAIMS  "NumPendingClaims"
+#define ATTR_NUM_PENDING_CLAIMS_IN_Q "NumPendingClaimsInQueue"
+#define ATTR_NUM_PENDING_CLAIMS_IN_CONTACT "NumPendingClaimsInContact"
 #define ATTR_SCHEDD_SWAP_EXHAUSTED  "ScheddSwapExhausted"
 #define ATTR_SCHEDD_INTERVAL  "ScheddInterval"
 #define ATTR_SCHEDD_IP_ADDR  "ScheddIpAddr"
@@ -683,6 +741,9 @@
 #define ATTR_SLOT_PARTITIONABLE  "PartitionableSlot"
 #define ATTR_SLOT_BACKFILL  "BackfillSlot"
 #define ATTR_SLOT_DYNAMIC  "DynamicSlot"
+#define ATTR_SLOT_BROKEN_CODE "SlotBrokenCode"
+#define ATTR_SLOT_BROKEN_REASON "SlotBrokenReason"
+#define ATTR_SLOT_BROKEN_REFID "BrokenItemId"
 #define ATTR_SOURCE  "Source"
 #define ATTR_STAGE_IN_START  "StageInStart"
 #define ATTR_STAGE_IN_FINISH  "StageInFinish"
@@ -696,8 +757,27 @@
 #define ATTR_STARTD_SENDS_ALIVES  "StartdSendsAlives"
 #define ATTR_STARTER_EXIT_STATUS "StarterExitStatus"
 #define ATTR_STARTER_HANDLES_ALIVES "_condor_StartdHandlesAlives"
+#define ATTR_STARTUP_LIMIT_UUID          "StartupLimitUUID"
+#define ATTR_STARTUP_LIMIT_TAG           "StartupLimitTag"
+#define ATTR_STARTUP_LIMIT_NAME          "StartupLimitName"
+#define ATTR_STARTUP_LIMIT_EXPR          "StartupLimitExpression"
+#define ATTR_STARTUP_LIMIT_RATE_COUNT    "StartupLimitRateCount"
+#define ATTR_STARTUP_LIMIT_RATE_WINDOW   "StartupLimitRateWindow"
+#define ATTR_STARTUP_LIMIT_BURST         "StartupLimitRateBurst"
+#define ATTR_STARTUP_LIMIT_MAX_BURST_COST "StartupLimitRateBurstMaxCost"
+#define ATTR_STARTUP_LIMIT_COST_EXPR     "StartupLimitRateCost"
+#define ATTR_STARTUP_LIMIT_EXPIRATION    "StartupLimitExpiration"
+#define ATTR_STARTUP_LIMIT_STATUS        "StartupLimitStatus"
+#define ATTR_STARTUP_LIMIT_ERROR         "StartupLimitError"
+#define ATTR_STARTUP_LIMIT_JOBS_SKIPPED  "StartupLimitJobsSkipped"
+#define ATTR_STARTUP_LIMIT_JOBS_ALLOWED  "StartupLimitJobsAllowed"
+#define ATTR_STARTUP_LIMIT_COST_ALLOWED  "StartupLimitCostAllowed"
+#define ATTR_STARTUP_LIMIT_MATCHES_IGNORED "StartupLimitMatchesIgnored"
+#define ATTR_STARTUP_LIMIT_LAST_IGNORED  "StartupLimitLastIgnoredTime"
+#define ATTR_STARTUP_LIMIT_IGNORED_USERS "StartupLimitIgnoredSources"
 #define ATTR_STATE  "State"
 #define ATTR_STARTER_IP_ADDR  "StarterIpAddr"
+#define ATTR_STARTER_PID "StarterPid"
 #define ATTR_STARTER_ABILITY_LIST  "StarterAbilityList"
 #define ATTR_STARTER_ULOG_FILE  "StarterUserLog"
 #define ATTR_STARTER_ULOG_USE_XML  "StarterUserLogUseXML"
@@ -812,6 +892,7 @@
 #define ATTR_USER  "User"
 #define ATTR_USERREC_OPT_prefix "_userrec_opt_"
 #define ATTR_USERREC_OPT_CREATE "_userrec_opt_create"
+#define ATTR_USERREC_OPT_CREATE_PROJECT "_userrec_opt_create_project"
 #define ATTR_USERREC_OPT_UPDATE "_userrec_opt_update"
 #define ATTR_USERREC_OPT_ME "_userrec_opt_me"
 #define ATTR_USERREC_OPT_CREATE_DEPRECATED "create"
@@ -844,6 +925,7 @@
 #define ATTR_WANT_DOCKER  "WantDocker"
 #define ATTR_WANT_DOCKER_IMAGE  "WantDockerImage"
 #define ATTR_WANT_SANDBOX_IMAGE  "WantSandboxImage"
+#define ATTR_WANT_JOB_NETWORKING  "WantJobNetworking"
 #define ATTR_WANT_SIF  "WantSIF"
 #define ATTR_COLLECTOR_IP_ADDR  "CollectorIpAddr"
 #define ATTR_NEGOTIATOR_IP_ADDR  "NegotiatorIpAddr"
@@ -867,6 +949,7 @@
 #define ATTR_TRANSFER_INPUT_FILES  "TransferInput"
 #define ATTR_TRANSFER_INPUT_REMAPS  "TransferInputRemaps"
 #define ATTR_TRANSFER_INPUT_STATS  "TransferInputStats"
+#define ATTR_TRANSFER_INPUT_OBJECT_COUNTS  "TransferInputFileCounts"
 #define ATTR_TRANSFER_Q_URL_IN_LIST "TransferQueueInputList"
 #define ATTR_TRANSFER_PLUGINS  "TransferPlugins"
 #define ATTR_WANT_TRANSFER_PLUGIN_METHODS  "WantTransferPluginMethods"
@@ -929,6 +1012,8 @@
 #define ATTR_PERIODIC_RELEASE_CHECK  "PeriodicRelease"
 #define ATTR_PERIODIC_REMOVE_CHECK  "PeriodicRemove"
 #define ATTR_PERIODIC_VACATE_CHECK  "PeriodicVacate"
+#define ATTR_TRANSFORM_ON_EVICT  "TransformOnEvict"
+#define ATTR_TRANSFORM_BODY_PREFIX "TransformBody_"
 #define ATTR_TIMER_REMOVE_CHECK  "TimerRemove"
 #define ATTR_TIMER_REMOVE_CHECK_SENT  "TimerRemoveSent"
 
@@ -938,6 +1023,7 @@
 #define ATTR_SEC_CRYPTO_METHODS  "CryptoMethods"
 #define ATTR_SEC_CRYPTO_METHODS_LIST  "CryptoMethodsList"
 #define ATTR_SEC_AUTHENTICATION  "Authentication"
+#define ATTR_SEC_AUTHENTICATION_NEW  "AuthenticationNew"
 #define ATTR_SEC_AUTH_REQUIRED  "AuthRequired"
 #define ATTR_SEC_ENCRYPTION  "Encryption"
 #define ATTR_SEC_INTEGRITY  "Integrity"
@@ -1075,6 +1161,8 @@ extern const char ATTR_SEC_AUTHENTICATED_USER [];
 #define ATTR_REQUIRE_GPUS  "RequireGPUs"
 #define ATTR_REQUIRE_PREFIX "Require"
 #define ATTR_REQUEST_VIRTUAL_MEMORY  "RequestVirtualMemory"
+#define ATTR_MAX_REQUEST_MEMORY "MaxRequestMemory"
+#define ATTR_RETRY_REQUEST_MEMORY "RetryRequestMemory"
 // GPUs property constraints
 #define ATTR_GPUS_MIN_MEMORY "GPUsMinMemory"
 #define ATTR_GPUS_MIN_CAPABILITY "GPUsMinCapability"
@@ -1194,6 +1282,7 @@ extern const char ATTR_SEC_AUTHENTICATED_USER [];
 #define ATTR_HOW_FAST  "HowFast"
 #define ATTR_DRAIN_REASON "DrainReason"
 #define ATTR_RESUME_ON_COMPLETION  "ResumeOnCompletion"
+#define ATTR_RUN_INSTANCE_ID "RunInstanceID"
 #define ATTR_DRAINING  "Draining"
 #define ATTR_DRAINING_REQUEST_ID  "DrainingRequestId"
 #define ATTR_DRAINING_DEADLINE "DrainingDeadline"
@@ -1206,6 +1295,7 @@ extern const char ATTR_SEC_AUTHENTICATED_USER [];
 #define ATTR_CHECK_EXPR  "CheckExpr"
 #define ATTR_START_EXPR  "StartExpr"
 #define ATTR_PROJECTION  "Projection"
+#define ATTR_PROJECT_NAME "ProjectName"
 #define ATTR_LAST_DRAIN_START_TIME  "LastDrainStartTime"
 #define ATTR_LAST_DRAIN_STOP_TIME  "LastDrainStopTime"
 
