@@ -55,7 +55,7 @@ const int REQUEST_CLAIM_SLOT_AD          = 7;
 // 		And we're willing to binary search at runtime to do the lookup
 //
 // 	So ...
-// 	We define a constexpr function in this header file that returns a 
+// 	We define a constexpr function in this header file that returns a
 // 	constexpr std::array that can be constexpr sorted by either field
 // 	in the Get..Num or Get..String functions, so that they can binary
 // 	search that sorted table.  This constexpr table-generating funciton
@@ -67,7 +67,7 @@ const int REQUEST_CLAIM_SLOT_AD          = 7;
 
 
 constexpr const
-std::array<std::pair<int, const char *>, 198> makeCommandTable() {
+std::array<std::pair<int, const char *>, 206> makeCommandTable() {
 	return {{ // Yes, we need two...
 
 /****
@@ -121,8 +121,8 @@ std::array<std::pair<int, const char *>, 198> makeCommandTable() {
 
 //#define AVAILABILITY		(SCHED_VERS+12)		/* Not used */
 //		{AVAILABILITY, "AVAILABILITY"},
-//#define NUM_FRGN_JOBS		(SCHED_VERS+13)		/* Not used */
-//		{NUM_FRGN_JOBS, "NUM_FRGN_JOBS"},
+#define DEACTIVATE_CLAIM_JOB_DONE		(SCHED_VERS+13)		/* formerly NUM_FRGN_JOBS */
+		{DEACTIVATE_CLAIM_JOB_DONE, "DEACTIVATE_CLAIM_JOB_DONE"},
 //#define STARTD_INFO			(SCHED_VERS+14)		/* Not used */
 //		{STARTD_INFO, "STARTD_INFO"},
 //#define SCHEDD_INFO			(SCHED_VERS+15)		/* Not used */
@@ -165,10 +165,10 @@ std::array<std::pair<int, const char *>, 198> makeCommandTable() {
 //		{SEND_ALL_JOBS, "SEND_ALL_JOBS"},
 //#define SEND_ALL_JOBS_PRIO	(SCHED_VERS+34)			/* Not used */
 //		{SEND_ALL_JOBS_PRIO, "SEND_ALL_JOBS_PRIO"},
-#define REQ_NEW_PROC		(SCHED_VERS+35)
-		{REQ_NEW_PROC, "REQ_NEW_PROC"},
-#define PCKPT_FRGN_JOB		(SCHED_VERS+36)
-		{PCKPT_FRGN_JOB, "PCKPT_FRGN_JOB"},
+//#define REQ_NEW_PROC		(SCHED_VERS+35)			/* Not used */
+//		{REQ_NEW_PROC, "REQ_NEW_PROC"},
+//#define PCKPT_FRGN_JOB		(SCHED_VERS+36)		/* Not used */
+//		{PCKPT_FRGN_JOB, "PCKPT_FRGN_JOB"},
 //#define SEND_RUNNING_JOBS	(SCHED_VERS+37)			/* Not used */
 //		{SEND_RUNNING_JOBS, "SEND_RUNNING_JOBS"},
 //#define CHECK_CAPABILITY    (SCHED_VERS+38)		/* Not used */
@@ -187,8 +187,8 @@ std::array<std::pair<int, const char *>, 198> makeCommandTable() {
 		{ACTIVATE_CLAIM, "ACTIVATE_CLAIM"},
 //#define PRIORITY_INFO       (SCHED_VERS+45)     /* negotiator to accountant, Not used */
 //		{PRIORITY_INFO, "PRIORITY_INFO"},
-#define PCKPT_ALL_JOBS		(SCHED_VERS+46)
-		{PCKPT_ALL_JOBS, "PCKPT_ALL_JOBS"},
+//#define PCKPT_ALL_JOBS		(SCHED_VERS+46)		/* Not used */
+//		{PCKPT_ALL_JOBS, "PCKPT_ALL_JOBS"},
 #define VACATE_ALL_CLAIMS	(SCHED_VERS+47)
 		{VACATE_ALL_CLAIMS, "VACATE_ALL_CLAIMS"},
 #define GIVE_STATE			(SCHED_VERS+48)
@@ -227,8 +227,8 @@ std::array<std::pair<int, const char *>, 198> makeCommandTable() {
 		{ATTEMPT_ACCESS, "ATTEMPT_ACCESS"},
 #define VACATE_CLAIM		(SCHED_VERS+65)     // vacate a given claim
 		{VACATE_CLAIM, "VACATE_CLAIM"},
-#define PCKPT_JOB			(SCHED_VERS+66)     // periodic ckpt a given slot
-		{PCKPT_JOB, "PCKPT_JOB"},
+//#define PCKPT_JOB			(SCHED_VERS+66)     // Not used
+//		{PCKPT_JOB, "PCKPT_JOB"},
 #define DAEMON_OFF			(SCHED_VERS+67)		// specific daemon, subsys follows
 		{DAEMON_OFF, "DAEMON_OFF"},
 #define DAEMON_OFF_FAST		(SCHED_VERS+68)		// specific daemon, subsys follows
@@ -322,13 +322,8 @@ std::array<std::pair<int, const char *>, 198> makeCommandTable() {
 		{RECYCLE_SHADOW, "RECYCLE_SHADOW"},
 #define CLEAR_DIRTY_JOB_ATTRS (SCHED_VERS+111) // schedd: clear dirty attributes for a job
 		{CLEAR_DIRTY_JOB_ATTRS, "CLEAR_DIRTY_JOB_ATTRS"},
-		// These two commands originally used the same command int by mistake.
-		// In 7.9.6, GET_PRIORITY_ROLLUP was assigned a new command int.
-		// List DRAIN_JOBS first, so that lower_bound finds it first
 #define DRAIN_JOBS (SCHED_VERS+112)
 		{DRAIN_JOBS, "DRAIN_JOBS"},
-#define GET_PRIORITY_ROLLUP_OLD (SCHED_VERS+112) // negotiator
-		{GET_PRIORITY_ROLLUP_OLD, "GET_PRIORITY_ROLLUP_OLD"},
 #define CANCEL_DRAIN_JOBS (SCHED_VERS+113)
 		{CANCEL_DRAIN_JOBS, "CANCEL_DRAIN_JOBS"},
 #define GET_PRIORITY_ROLLUP (SCHED_VERS+114) // negotiator
@@ -386,6 +381,29 @@ std::array<std::pair<int, const char *>, 198> makeCommandTable() {
 		{RESET_USERREC, "RESET_USERREC"},
 #define DELETE_USERREC    (SCHED_VERS+149)
 		{DELETE_USERREC, "DELETE_USERREC"},
+#define GET_CONTACT_INFO  (SCHED_VERS+150) // Ask Schedd for child daemons contact information (addr and secret) Note: Used for DAGMan
+		{GET_CONTACT_INFO, "GET_CONTACT_INFO"},
+#define CREATE_OCU_FOR_USERREC (SCHED_VERS + 151) // Create an OCU(s) for a given UserRec
+		{CREATE_OCU_FOR_USERREC, "CREATE_OCU_FOR_USERREC"},
+#define REMOVE_OCU_FROM_USERREC (SCHED_VERS + 152) // Remove an OCU(s) from a given UserRec
+		{REMOVE_OCU_FROM_USERREC, "REMOVE_OCU_FROM_USERREC"},
+#define QUERY_OCU_FROM_USERREC (SCHED_VERS + 153) // Query the OCU(s) associated with a given UserRec
+		{QUERY_OCU_FROM_USERREC, "QUERY_OCU_FROM_USERREC"},
+
+#define	 REACTIVATE_CLAIM_CHECK (SCHED_VERS+154)
+		{REACTIVATE_CLAIM_CHECK, "REACTIVATE_CLAIM_CHECK"},
+#define PLACEMENT_USER_LOGIN    (SCHED_VERS+155)
+		{PLACEMENT_USER_LOGIN, "PLACEMENT_USER_LOGIN"},
+#define PLACEMENT_QUERY_USERS    (SCHED_VERS+156)
+		{PLACEMENT_QUERY_USERS, "PLACEMENT_QUERY_USERS"},
+#define PLACEMENT_QUERY_TOKENS    (SCHED_VERS+157)
+		{PLACEMENT_QUERY_TOKENS, "PLACEMENT_QUERY_TOKENS"},
+#define PLACEMENT_QUERY_AUTHORIZATIONS    (SCHED_VERS+158)
+		{PLACEMENT_QUERY_AUTHORIZATIONS, "PLACEMENT_QUERY_AUTHORIZATIONS"},
+#define CREATE_STARTUP_LIMIT      (SCHED_VERS+159)
+		{CREATE_STARTUP_LIMIT, "CREATE_STARTUP_LIMIT"},
+#define QUERY_STARTUP_LIMITS      (SCHED_VERS+160)
+		{QUERY_STARTUP_LIMITS, "QUERY_STARTUP_LIMITS"},
 
 #define HAD_ALIVE_CMD                   (HAD_COMMANDS_BASE + 0)
 		{HAD_ALIVE_CMD, "HAD_ALIVE_CMD"},
@@ -468,8 +486,8 @@ std::array<std::pair<int, const char *>, 198> makeCommandTable() {
 */
 
 #define STARTER_COMMANDS_BASE 1500
-#define STARTER_HOLD_JOB    (STARTER_COMMANDS_BASE+0)
-		{STARTER_HOLD_JOB, "STARTER_HOLD_JOB"},
+#define STARTER_VACATE_JOB    (STARTER_COMMANDS_BASE+0)
+		{STARTER_VACATE_JOB, "STARTER_VACATE_JOB"},
 #define CREATE_JOB_OWNER_SEC_SESSION (STARTER_COMMANDS_BASE+1)
 		{CREATE_JOB_OWNER_SEC_SESSION, "CREATE_JOB_OWNER_SEC_SESSION"},
 #define START_SSHD (STARTER_COMMANDS_BASE+2)
@@ -609,6 +627,11 @@ std::array<std::pair<int, const char *>, 198> makeCommandTable() {
 #define FILETRANS_DOWNLOAD (FILETRANSFER_BASE+1)
 		{FILETRANS_DOWNLOAD,"FILETRANS_DOWNLOAD"},
 
+// Commands for DAGMan
+#define DAGMAN_BASE 61500
+#define DAGMAN_GENERIC (DAGMAN_BASE+0)
+		{DAGMAN_GENERIC, "DAGMAN_GENERIC"}, // Generic send payload ad with more details and return result ad
+
 
 /*
 *** Condor Password Daemon Commands
@@ -622,8 +645,8 @@ std::array<std::pair<int, const char *>, 198> makeCommandTable() {
 *** Commands used by the daemon core Shadow
 */
 #define DCSHADOW_BASE 71000
-#define SHADOW_UPDATEINFO	   (DCSHADOW_BASE+0)
-		{SHADOW_UPDATEINFO, "SHADOW_UPDATEINFO"},
+//#define SHADOW_UPDATEINFO	   (DCSHADOW_BASE+0)
+//		{SHADOW_UPDATEINFO, "SHADOW_UPDATEINFO"},
 //#define TAKE_MATCH             (DCSHADOW_BASE+1)  // for MPI & parallel shadow, Not used
 #define MPI_START_COMRADE      (DCSHADOW_BASE+2)  // for MPI & parallel shadow
 		{MPI_START_COMRADE, "MPI_START_COMRADE"},
@@ -677,7 +700,9 @@ std::array<std::pair<int, const char *>, 198> makeCommandTable() {
 		{CREDD_REMOVE_CRED, "CREDD_REMOVE_CRED"},
 #define CREDD_QUERY_CRED (CREDD_BASE+3)
 		{CREDD_QUERY_CRED, "CREDD_QUERY_CRED"},
-#define CREDD_REFRESH_ALL (CREDD_BASE+20)
+#define CREDD_GET_TOKEN (CREDD_BASE+4)
+		{CREDD_GET_TOKEN, "CREDD_GET_TOKEN"},
+#define CREDD_REFRESH_ALL (CREDD_BASE+20)	// Not used
 		{CREDD_REFRESH_ALL, "CREDD_REFRESH_ALL"},
 #define CREDD_CHECK_CREDS (CREDD_BASE+30)	// check to see if the desired oauth tokens are stored
 		{CREDD_CHECK_CREDS, "CREDD_CHECK_CREDS"},
