@@ -110,7 +110,7 @@ int Authentication::authenticate( char *hostAddr, const char* auth_methods,
 		CondorError* errstack, int timeout, bool non_blocking)
 {
 	int retval;
-	int old_timeout=0;
+	time_t old_timeout=0;
 	if (timeout>=0) {
 		old_timeout=mySock->timeout(timeout);
 	}
@@ -626,7 +626,7 @@ void Authentication::load_map_file() {
 	}
 }
 
-void Authentication::map_authentication_name_to_canonical_name(int authentication_type, const char* method_string, const char* authentication_name, std::string& canonical_user) {
+void Authentication::map_authentication_name_to_canonical_name(int /*authentication_type*/, const char* method_string, const char* authentication_name, std::string& canonical_user) {
     load_map_file();
 
 	dprintf (D_SECURITY|D_VERBOSE, "AUTHENTICATION: attempting to map '%s'\n", authentication_name);
@@ -637,33 +637,9 @@ void Authentication::map_authentication_name_to_canonical_name(int authenticatio
 	if (global_map_file) {
 
 		dprintf (D_SECURITY|D_VERBOSE, "AUTHENTICATION: 1: attempting to map '%s'\n", auth_name_to_map.c_str());
-		bool mapret = global_map_file->GetCanonicalization(method_string, auth_name_to_map.c_str(), canonical_user);
+		bool mapret = global_map_file->GetCanonicalization(method_string, auth_name_to_map, canonical_user);
 		dprintf (D_SECURITY|D_VERBOSE, "AUTHENTICATION: 2: mapret: %i canonical_user: %s\n", mapret, canonical_user.c_str());
 
-		// if the method is SCITOKENS and mapping failed, try again
-		// with a trailing '/'.  this is to assist admins who have
-		// misconfigured their mapfile.
-		//
-		// depending on the setting of SEC_SCITOKENS_ALLOW_EXTRA_SLASH
-		// (default false) we will either let it slide or print a loud
-		// warning to make it easier to identify the problem.
-		// 
-		// reminder: GetCanonicalization returns "true" on failure.
-		if (mapret && authentication_type == CAUTH_SCITOKENS) {
-			auth_name_to_map += "/";
-			bool withslash_result = global_map_file->GetCanonicalization(method_string, auth_name_to_map.c_str(), canonical_user);
-			if (param_boolean("SEC_SCITOKENS_ALLOW_EXTRA_SLASH", false)) {
-				// just continue as if everything is fine.  we've now
-				// already updated canonical_user with the result. complain
-				// a little bit though so the admin can notice and fix it.
-				dprintf(D_SECURITY, "MAPFILE: WARNING: The CERTIFICATE_MAPFILE entry for SCITOKENS \"%s\" contains a trailing '/'. This was allowed because SEC_SCITOKENS_ALLOW_EXTRA_SLASH is set to TRUE.\n", authentication_name);
-				mapret = withslash_result;
-			} else {
-				// complain loudly
-				dprintf(D_ALWAYS, "MAPFILE: ERROR: The CERTIFICATE_MAPFILE entry for SCITOKENS \"%s\" contains a trailing '/'. Either correct the mapfile or set SEC_SCITOKENS_ALLOW_EXTRA_SLASH in the configuration.\n", authentication_name);
-			}
-		}
-
 		if (!mapret) {
 			// returns true on failure?
 			dprintf (D_FULLDEBUG|D_VERBOSE, "AUTHENTICATION: successful mapping to %s\n", canonical_user.c_str());
