@@ -22,7 +22,6 @@
 #include "internet.h"
 #include "condor_config.h"
 #include "condor_perms.h"
-#include "condor_netdb.h"
 #include "subsystem_info.h"
 
 #include "sock.h"
@@ -216,12 +215,18 @@ IpVerify::add_hash_entry(const struct in6_addr & sin6_addr, const char * user, p
 }
 
 perm_mask_t 
-IpVerify::allow_mask(DCpermission perm) { 
+IpVerify::allow_mask(DCpermission perm) {
+	if (perm < 0) {
+		return 0;
+	}
 	return ((perm_mask_t)1 << (1+2*perm));
 }
 
 perm_mask_t 
-IpVerify::deny_mask(DCpermission perm) { 
+IpVerify::deny_mask(DCpermission perm) {
+	if (perm < 0) {
+		return 0;
+	}
 	return ((perm_mask_t)1 << (2+2*perm));
 }
  
@@ -865,7 +870,7 @@ IpVerify::lookup_user(UserHash_t& users, netgroup_list_t& netgroups, char const
 	for (auto& [host_key, userlist] : users) {
 		bool host_matches = false;
 		if (ip) {
-			host_matches = matches_withnetwork(host_key.c_str(), ip);
+			host_matches = matches_withnetwork(host_key, ip);
 		} else {
 			host_matches = matches_anycase_withwildcard(host_key.c_str(), hostname);
 		}
