@@ -18,8 +18,6 @@
  ***************************************************************/
 
 
-#define _POSIX_SOURCE
-
 #include "condor_common.h"
 #include "condor_io.h"
 #include "condor_debug.h"
@@ -27,6 +25,7 @@
 #include "selector.h"
 #include "condor_threads.h"
 #include "condor_sockfunc.h"
+#include "condor_rw.h"
 
 /*
  * Returns true if the given error number indicates
@@ -102,21 +101,25 @@ static char const *not_null_peer_description(char const *peer_description,SOCKET
  * Returns < 0 on failure.  -1 = general error, -2 = peer closed socket
  */
 int
-condor_read( char const *peer_description, SOCKET fd, char *buf, int sz, int timeout, int flags, bool non_blocking )
+condor_read( char const *peer_description, SOCKET fd, char *buf, int sz, time_t timeout, CondorRWFlags flags, bool non_blocking )
 {
 	Selector selector;
 	int nr = 0, nro;
 	time_t start_time=0, cur_time=0;
 	char sinbuf[SINFUL_STRING_BUF_SIZE];
+	int recv_flags = 0;
+	if (flags == CondorRWFlags::Peek) {
+		recv_flags = MSG_PEEK;
+	}
 
 	if( IsDebugLevel(D_NETWORK) ) {
 		dprintf(D_NETWORK,
-				"condor_read(fd=%d %s,,size=%d,timeout=%d,flags=%d,non_blocking=%d)\n",
+				"condor_read(fd=%d %s,,size=%d,timeout=%lld,flags=%d,non_blocking=%d)\n",
 				fd,
 				not_null_peer_description(peer_description,fd,sinbuf),
 				sz,
-				timeout,
-				flags,
+				(long long)timeout,
+				recv_flags,
 				non_blocking);
 	}
 
@@ -140,7 +143,7 @@ condor_read( char const *peer_description, SOCKET fd, char *buf, int sz, int tim
 #endif
 		nr = -2;
 		while (nr == -2 || (nr == -1 && errno == EINTR)) {
-			nr = recv(fd, buf, sz, flags);
+			nr = recv(fd, buf, sz, recv_flags);
 		}
 
 		if ( nr <= 0 ) {
@@ -153,7 +156,7 @@ condor_read( char const *peer_description, SOCKET fd, char *buf, int sz, int tim
 			the_error = errno;
 			the_errorstr = strerror(the_error);
 #endif
-			if ( nr == 0 && !(flags & MSG_PEEK)) {
+			if ( nr == 0 && !(recv_flags & MSG_PEEK)) {
 				nr = -2;
 				dprintf( D_FULLDEBUG, "condor_read(): "
 					"Socket closed when trying to read %d bytes from %s in non-blocking mode\n",
@@ -169,10 +172,10 @@ condor_read( char const *peer_description, SOCKET fd, char *buf, int sz, int tim
 			} else if ( !errno_is_temporary(the_error) ) {
 				dprintf( D_ALWAYS, "condor_read() failed: recv() %d bytes from %s "
 					"returned %d, "     
-					"timeout=%d, errno=%d %s.\n",
+					"timeout=%lld, errno=%d %s.\n",
 					sz,                 
 					not_null_peer_description(peer_description,fd,sinbuf),
-					nr, timeout, the_error, the_errorstr );
+					nr, (long long)timeout, the_error, the_errorstr );
 			}
 			else
 			{
@@ -195,7 +198,7 @@ condor_read( char const *peer_description, SOCKET fd, char *buf, int sz, int tim
 	selector.add_fd( fd, Selector::IO_READ );
 	
 	if ( timeout > 0 ) {
-		start_time = time(NULL);
+		start_time = time(nullptr);
 		cur_time = start_time;
 	}
 
@@ -261,7 +264,7 @@ condor_read( char const *peer_description, SOCKET fd, char *buf, int sz, int tim
 		
 		start_thread_safe("recv");
 
-		nro = recv(fd, &buf[nr], sz - nr, flags);
+		nro = recv(fd, &buf[nr], sz - nr, recv_flags);
 		// Save the error value before stop_thread_safe(), as that may
 		// overwrite it.
 		int the_error;
@@ -314,10 +317,10 @@ condor_read( char const *peer_description, SOCKET fd, char *buf, int sz, int tim
 				else {
 					int lapse = (int)(time(NULL)-start_time);
 					dprintf( D_ALWAYS,
-							 "condor_read(): UNEXPECTED read timeout after %ds during non-blocking read from %s (desired timeout=%ds)\n",
+							 "condor_read(): UNEXPECTED read timeout after %ds during non-blocking read from %s (desired timeout=%llds)\n",
 							 lapse,
 							 not_null_peer_description(peer_description,fd,sinbuf),
-							 timeout);
+							 (long long)timeout);
 				}
 			}
 			if ( errno_is_temporary(the_error) ) {
@@ -342,7 +345,11 @@ condor_read( char const *peer_description, SOCKET fd, char *buf, int sz, int tim
 
 			return -1;
 		}
-		nr += nro;
+		// Increment the number of bytes read,
+		// but not if we got a short read when using MSG_PEEK.
+		if ( !(recv_flags & MSG_PEEK) || nro == (sz - nr) ) {
+			nr += nro;
+		}
 	}
 	
 /* Post Conditions */
@@ -352,7 +359,7 @@ condor_read( char const *peer_description, SOCKET fd, char *buf, int sz, int tim
 
 
 int
-condor_write( char const *peer_description, SOCKET fd, const char *buf, int sz, int timeout, int flags, bool non_blocking )
+condor_write( char const *peer_description, SOCKET fd, const char *buf, int sz, time_t timeout, int flags, bool non_blocking )
 {
 	int nw = 0, nwo = 0;
 	time_t start_time = 0, cur_time = 0;
@@ -364,11 +371,11 @@ condor_write( char const *peer_description, SOCKET fd, const char *buf, int sz,
 
 	if( IsDebugLevel( D_NETWORK ) ) {
 		dprintf(D_NETWORK,
-				"condor_write(fd=%d %s,,size=%d,timeout=%d,flags=%d,non_blocking=%d)\n",
+				"condor_write(fd=%d %s,,size=%d,timeout=%lld,flags=%d,non_blocking=%d)\n",
 				fd,
 				not_null_peer_description(peer_description,fd,sinbuf),
 				sz,
-				timeout,
+				(long long)timeout,
 				flags,
 				non_blocking);
 	}
@@ -409,10 +416,10 @@ condor_write( char const *peer_description, SOCKET fd, const char *buf, int sz,
 			if ( !errno_is_temporary(the_error) ) {
 				dprintf( D_ALWAYS, "condor_write() failed: send() %d bytes to %s "
 					"returned %d, "     
-					"timeout=%d, errno=%d %s.\n",
+					"timeout=%lld, errno=%d %s.\n",
 					sz,                 
 					not_null_peer_description(peer_description,fd,sinbuf),
-					nw, timeout, the_error, the_errorstr );
+					nw, (long long)timeout, the_error, the_errorstr );
 			}
 			else
 			{
@@ -586,10 +593,10 @@ condor_write( char const *peer_description, SOCKET fd, const char *buf, int sz,
 
 			dprintf( D_ALWAYS, "condor_write() failed: send() %d bytes to %s "
 					 "returned %d, "
-					 "timeout=%d, errno=%d %s.\n",
+					 "timeout=%lld, errno=%d %s.\n",
 					 sz,
 					 not_null_peer_description(peer_description,fd,sinbuf),
-					 nwo, timeout, the_error, the_errorstr );
+					 nwo, (long long)timeout, the_error, the_errorstr );
 
 			return -1;
 		}
