@@ -24,10 +24,15 @@
 
 #include "sock.h"
 
+enum class CondorRWFlags {
+	Read = 0,
+	Peek = 1
+};
+
  // Returns < 0 upon an error.
-int condor_read(char const *peer_description,SOCKET fd, char *buf, int sz, int timeout, int flags=0, bool non_blocking=false);
+int condor_read(char const *peer_description,SOCKET fd, char *buf, int sz, time_t timeout, CondorRWFlags flags = CondorRWFlags::Read, bool non_blocking=false);
 
 
-int condor_write(char const *peer_description,SOCKET fd, const char *buf, int sz, int timeout, int flags=0, bool non_blocking=false);
+int condor_write(char const *peer_description,SOCKET fd, const char *buf, int sz, time_t timeout, int flags=0, bool non_blocking=false);
 
 #endif
