@@ -35,6 +35,7 @@
 #include "globus_utils.h"
 #include "directory.h"
 #include "shared_port_client.h"
+#include "basename.h"
 #include "ipv6_hostname.h"
 
 #ifdef WIN32
@@ -814,7 +815,7 @@ void ReliSock::RcvMsg::reset()
 }
 
 
-int ReliSock::RcvMsg::rcv_packet( char const *peer_description, SOCKET _sock, int _timeout)
+int ReliSock::RcvMsg::rcv_packet( char const *peer_description, SOCKET _sock, time_t _timeout)
 {
 	char	        hdr[MAX_HEADER_SIZE];
 	char *cksum_ptr = &hdr[5];
@@ -837,7 +838,7 @@ int ReliSock::RcvMsg::rcv_packet( char const *peer_description, SOCKET _sock, in
 
 	header_filled = 0;
 
-	retval = condor_read(peer_description,_sock,hdr,header_size,_timeout, 0, p_sock->is_non_blocking());
+	retval = condor_read(peer_description,_sock,hdr,header_size,_timeout, CondorRWFlags::Read, p_sock->is_non_blocking());
 	if ( retval == 0 ) {   // 0 means that the read would have blocked; unlike a normal read(), condor_read
 	                       // returns -2 if the socket has been closed.
 		dprintf(D_NETWORK, "Reading header would have blocked.\n");
@@ -1096,7 +1097,7 @@ void ReliSock::SndMsg::reset()
 	m_out_buf = NULL;
 }
 
-int ReliSock::SndMsg::finish_packet(const char *peer_description, int sock, int timeout)
+int ReliSock::SndMsg::finish_packet(const char *peer_description, int sock, time_t timeout)
 {
 	if (m_out_buf == NULL) {
 		return true;
@@ -1136,7 +1137,7 @@ void ReliSock::SndMsg::stash_packet()
 	// partially send it to the wire.  If this happens, we return 2.  In such a case,
 	// we leave the SndMsg buffer in a valid state -- if the caller cannot buffer the
 	// data itself, it may use the 'put_force' method to grow the underlying buffer.
-int ReliSock::SndMsg::snd_packet( char const *peer_description, int _sock, int end, int _timeout )
+int ReliSock::SndMsg::snd_packet( char const *peer_description, int _sock, int end, time_t _timeout )
 {
 		// First, see if we have an incomplete packet.
 	int retval = finish_packet(peer_description, _sock, _timeout);
@@ -1883,6 +1884,11 @@ ReliSock::get_file( filesize_t *size, const char *destination,
 		// Open the file
 		errno = 0;
 		fd = ::safe_open_wrapper_follow(destination, flags, 0600);
+		if ((fd == -1) && (errno == ENOENT)) {
+			std::string directory = condor_dirname(destination);
+			mkdir_and_parents_if_needed(directory.c_str(), 0700);
+			fd = ::safe_open_wrapper_follow(destination, flags, 0600);
+		}
 	}
 	else {
 		fd = -1;
@@ -2211,15 +2217,15 @@ ReliSock::put_file( filesize_t *size, int fd, filesize_t offset, filesize_t max_
 	bool buffered = get_encryption() && get_crypto_state()->m_keyInfo.getProtocol() == CONDOR_AESGCM;
 	const size_t buf_sz = buffered ? AES_FILE_BUF_SZ : OLD_FILE_BUF_SZ;
 
-	StatInfo filestat( fd );
-	if ( filestat.Error() ) {
-		int		staterr = filestat.Errno( );
+	struct stat filestat = {};
+	if (fstat(fd, &filestat) != 0) {
+		int		staterr = errno;
 		dprintf(D_ALWAYS, "ReliSock: put_file: StatBuf failed: %d %s\n",
 				staterr, strerror( staterr ) );
 		return -1;
 	}
 
-	if ( filestat.IsDirectory() ) {
+	if ( filestat.st_mode & S_IFDIR ) {
 		dprintf(D_ALWAYS,
 				"ReliSock: put_file: Failed because directories are not supported.\n" );
 			// Give the receiver an empty file so that this message is
@@ -2240,7 +2246,7 @@ ReliSock::put_file( filesize_t *size, int fd, filesize_t offset, filesize_t max_
 		return PUT_FILE_OPEN_FAILED;
 	}
 
-	filesize = filestat.GetFileSize( );
+	filesize = filestat.st_size;
 	dprintf( D_FULLDEBUG,
 			 "put_file: Found file size " FILESIZE_T_FORMAT "\n",
 			 filesize );
@@ -2267,7 +2273,12 @@ ReliSock::put_file( filesize_t *size, int fd, filesize_t offset, filesize_t max_
 	}
 
 	if ( offset ) {
-		lseek( fd, offset, SEEK_SET );
+		int r = lseek( fd, offset, SEEK_SET );
+		if (r < 0) {
+			dprintf(D_ALWAYS, "ReliSock: put_file: Seek failed: %s\n", strerror(errno));
+			// Well, not really open failed, but let's reuse this error
+			return PUT_FILE_OPEN_FAILED;
+		}
 	}
 
 	// Log what's going on
@@ -2491,13 +2502,12 @@ ReliSock::put_file_with_permissions( filesize_t *size, const char *source, files
 
 #ifndef WIN32
 	// Stat the file
-	StatInfo stat_info( source );
+	struct stat stat_info = {};
 
-	if ( stat_info.Error() ) {
+	if (stat(source, &stat_info) != 0) {
 		dprintf( D_ALWAYS, "ReliSock::put_file_with_permissions(): "
-				 "Failed to stat file '%s': %s (errno: %d, si_error: %d)\n",
-				 source, strerror(stat_info.Errno()), stat_info.Errno(),
-				 stat_info.Error() );
+				 "Failed to stat file '%s': %s (errno: %d)\n",
+				 source, strerror(errno), errno );
 
 		// Now send an empty file in order to recover sanity on this
 		// stream.
@@ -2515,7 +2525,7 @@ ReliSock::put_file_with_permissions( filesize_t *size, const char *source, files
 		}
 		return PUT_FILE_OPEN_FAILED;
 	}
-	file_mode = (condor_mode_t)stat_info.GetMode();
+	file_mode = (condor_mode_t)stat_info.st_mode;
 #else
 		// We don't know what unix permissions a windows file should have,
 		// so tell the other side to ignore permissions from us (act like
