@@ -27,7 +27,6 @@
 #include "condor_debug.h"
 #include "internet.h"
 #include "condor_config.h"
-#include "condor_netdb.h"
 #include "selector.h"
 #include "condor_sockfunc.h"
 #include "condor_sinful.h"
@@ -676,9 +675,7 @@ int SafeSock::handle_incoming_packet()
                                              _shortMsg.isDataHashed(),
                                              _shortMsg.md(), 
                                              _shortMsg.isDataEncrypted(), prev);
-            if(!prev->nextMsg) {    
-                EXCEPT("Error:handle_incomming_packet: Out of Memory");
-            }
+            ASSERT(prev->nextMsg);
             //dprintf(D_NETWORK, "new msg created\n");
             //prev->nextMsg->dumpMsg();
         } else { // first message in the bucket
@@ -686,9 +683,7 @@ int SafeSock::handle_incoming_packet()
                                               _shortMsg.isDataHashed(),
                                               _shortMsg.md(), 
                                               _shortMsg.isDataEncrypted(), NULL);
-            if(!_inMsgs[index]) {
-                EXCEPT("Error:handle_incomming_packet: Out of Memory");
-            }
+            ASSERT(_inMsgs[index]);
             //dprintf(D_NETWORK, "new msg created\n");
             //_inMsgs[index]->dumpMsg();
         }
