@@ -617,7 +617,7 @@ SharedPortState::HandleUnbound(Stream *&s)
 		if( has_socket && has_alt_socket ) {
 			// If using an abstract domain socket, don't print the
 			// secret cookie portion of the name.
-			dprintf( D_ALWAYS, "SharedPortServer:%s failed to connect %s%s: "
+			dprintf( D_ALWAYS, "SharedPortServer:%s failed to connect to %s%s: "
 				"primary (%s%s): %s (%d); alt (%s): %s (%d)\n",
 				server_busy ? " server was busy," : "",
 				m_sock_name.c_str(),
