@@ -1158,7 +1158,7 @@ SharedPortEndpoint::ReceiveSocket( ReliSock *named_sock, ReliSock *return_remote
 
 	if( recvmsg(named_sock->get_file_desc(),&msg,0) != 1 ) {
 		dprintf(D_ALWAYS,
-				"SharedPortEndpoint: failed to receive message containing forwarded socket: errno=%d: %s",
+				"SharedPortEndpoint: failed to receive message containing forwarded socket: errno=%d: %s\n",
 				errno,strerror(errno));
 		free(buf);
 		return;
@@ -1350,8 +1350,8 @@ SharedPortEndpoint::UseSharedPort(std::string *why_not,bool already_open)
 	static bool cached_result = false;
 	static time_t cached_time = 0;
 
-	time_t now = time(NULL);
-	if( abs((int)now-(int)cached_time) > 10 || cached_time==0 || why_not ) {
+	time_t now = time(nullptr);
+	if( abs(now-cached_time) > 10 || cached_time==0 || why_not ) {
 		cached_time = now;
 
 		std::string socket_dir;
