@@ -39,7 +39,7 @@
 #endif
 
 
-//JobRouter *job_router = NULL;
+JobRouter *job_router = NULL;
 
 //-------------------------------------------------------------
 const char * MyName = NULL;
@@ -162,8 +162,6 @@ int main(int argc, const char *argv[])
 	//bool dash_d_fulldebug = false;
 	const char * route_jobs_filename = NULL;
 
-	g_jobs = new classad::ClassAdCollection();
-
 	for (int i = 1; i < argc; ++i) {
 
 		const char * pcolon = NULL;
@@ -223,6 +221,7 @@ int main(int argc, const char *argv[])
 	// before we call init() for the router, we need to install a pseudo-schedd object
 	// so that init() doesn't install a real schedd object.
 	Scheduler* schedd = new Scheduler(0);
+	g_jobs = schedd->GetClassAds();
 
 	g_silence_dprintf = dash_diagnostic ? false : true;
 	g_save_dprintfs = true;
@@ -320,9 +319,8 @@ int main(int argc, const char *argv[])
 	}
 	if (! found.empty()) {
 		fprintf(stdout,
-			"\nWARNING: %s are deprecated and will be removed for V24 of HTCondor. New configuration"
-			" syntax for the job router is defined using JOB_ROUTER_ROUTE_NAMES and JOB_ROUTER_ROUTE_<name>."
-			" Note: The removal will occur during the lifetime of the HTCondor V23 feature series\n",
+			"\nWARNING: %s are no longer supported. New configuration"
+			" syntax for the job router is defined using JOB_ROUTER_ROUTE_NAMES and JOB_ROUTER_ROUTE_<name>.\n",
 			found.c_str());
 	}
 
@@ -332,17 +330,17 @@ int main(int argc, const char *argv[])
 class CondorQClassAdFileParseHelper : public ClassAdFileParseHelper
 {
  public:
-	virtual int PreParse(std::string & line, classad::ClassAd & ad, FILE* file);
-	virtual int OnParseError(std::string & line, classad::ClassAd & ad, FILE* file);
+	virtual int PreParse(std::string & line, classad::ClassAd & ad, classad::LexerSource & lexsrc);
+	virtual int OnParseError(std::string & line, classad::ClassAd & ad, classad::LexerSource & lexsrc);
 	// return non-zero if new parser, o if old (line oriented) parser, non-zero is returned the above functions will never be called.
-	virtual int NewParser(classad::ClassAd & /*ad*/, FILE* /*file*/, bool & detected_long, std::string & /*errmsg*/) { detected_long = false; return 0; }
+	virtual int NewParser(classad::ClassAd & /*ad*/,classad::LexerSource & lexsrc /*file*/, bool & detected_long, std::string & /*errmsg*/) { detected_long = false; return 0; }
 	std::string schedd_name;
 	std::string schedd_addr;
 };
 
 // this method is called before each line is parsed. 
 // return 0 to skip (is_comment), 1 to parse line, 2 for end-of-classad, -1 for abort
-int CondorQClassAdFileParseHelper::PreParse(std::string & line, classad::ClassAd & /*ad*/, FILE* /*file*/)
+int CondorQClassAdFileParseHelper::PreParse(std::string & line, classad::ClassAd & /*ad*/, classad::LexerSource & lexsrc /*file*/)
 {
 	// treat blank lines as delimiters.
 	if (line.size() <= 0) {
@@ -390,16 +388,16 @@ int CondorQClassAdFileParseHelper::PreParse(std::string & line, classad::ClassAd
 
 // this method is called when the parser encounters an error
 // return 0 to skip and continue, 1 to re-parse line, 2 to quit parsing with success, -1 to abort parsing.
-int CondorQClassAdFileParseHelper::OnParseError(std::string & line, classad::ClassAd & ad, FILE* file)
+int CondorQClassAdFileParseHelper::OnParseError(std::string & line, classad::ClassAd & ad, classad::LexerSource & lexsrc)
 {
 	// when we get a parse error, skip ahead to the start of the next classad.
-	int ee = this->PreParse(line, ad, file);
+	int ee = this->PreParse(line, ad, lexsrc);
 	while (1 == ee) {
-		if ( ! readLine(line, file, false) || feof(file)) {
+		if ( ! readLine(line, lexsrc, false) || lexsrc.AtEnd()) {
 			ee = 2;
 			break;
 		}
-		ee = this->PreParse(line, ad, file);
+		ee = this->PreParse(line, ad, lexsrc);
 	}
 	return ee;
 }
@@ -409,18 +407,18 @@ static bool read_classad_file(const char *filename, classad::ClassAdCollection &
 {
 	bool success = false;
 
-	FILE* file = NULL;
-	bool  read_from_stdin = false;
+	CompatFileLexerSource lexsrc;
 	if (MATCH == strcmp(filename, "-")) {
-		read_from_stdin = true;
-		file = stdin;
+		lexsrc.SetSource(stdin, false);
 	} else {
-		file = safe_fopen_wrapper_follow(filename, "r");
+		FILE* file = safe_fopen_wrapper_follow(filename, "rb");
+		if ( ! file) {
+			fprintf(stderr, "Can't open file of job ads: %s\n", filename);
+			return false;
+		}
+		lexsrc.SetSource(file, true);
 	}
-	if (file == NULL) {
-		fprintf(stderr, "Can't open file of job ads: %s\n", filename);
-		return false;
-	} else {
+
 		// this helps us parse the output of condor_q -long
 		CondorQClassAdFileParseHelper parse_helper;
 
@@ -429,7 +427,7 @@ static bool read_classad_file(const char *filename, classad::ClassAdCollection &
 
 			int error;
 			bool is_eof;
-			int cAttrs = InsertFromFile(file, *classad, is_eof, error, &parse_helper);
+			int cAttrs = InsertFromStream(lexsrc, *classad, is_eof, error, &parse_helper);
 
 			bool include_classad = cAttrs > 0 && error >= 0;
 			if (include_classad && constr) {
@@ -466,8 +464,6 @@ static bool read_classad_file(const char *filename, classad::ClassAdCollection &
 			}
 		}
 
-		if ( ! read_from_stdin) { fclose(file); }
-	}
 	return success;
 }
 
@@ -479,13 +475,14 @@ public:
 	JobLogMirror(char const *job_queue=NULL) {}
 	~JobLogMirror() {}
 
-	void init() {}
 	void config() {}
 	void stop() {}
 
 private:
 };
 
+class UserRecord;
+
 Scheduler::Scheduler(int id /*=0*/)
 	: m_consumer(NULL)
 	, m_mirror(NULL)
@@ -501,19 +498,11 @@ Scheduler::~Scheduler()
 	m_consumer = NULL;
 }
 
-classad::ClassAdCollection *Scheduler::GetClassAds() const
-{
-	if (m_id == 0) {
-		return g_jobs;
-	}
-	return NULL;
-}
-
-void Scheduler::init() {  if (m_mirror) m_mirror->init(); }
+void Scheduler::init() {  config(); }
 void Scheduler::config() { if (m_mirror) m_mirror->config(); }
 void Scheduler::stop()  { if (m_mirror) m_mirror->stop(); }
 void Scheduler::poll()  { }
-
+UserRecord * Scheduler::GetJobUser(const ClassAd * jobad) { return nullptr; }
 
 // 
 JobRouterHookMgr::JobRouterHookMgr() : HookClientMgr(), m_warn_cleanup(false), m_warn_update(false), m_warn_translate(false), m_warn_exit(false),NUM_HOOKS(0), UNDEFINED("UNDEFINED"), m_default_hook_keyword(NULL) {}
@@ -543,14 +532,7 @@ JobRouterHookMgr::getHookKeyword(const classad::ClassAd &ad)
 
 
 
-ClaimJobResult claim_job(int cluster, int proc, std::string * error_details, const char * my_identity)
-{
-	return CJR_OK;
-}
-
-
-
-ClaimJobResult claim_job(classad::ClassAd const &ad, const char * pool_name, const char * schedd_name, int cluster, int proc, std::string * error_details, const char * my_identity, bool target_is_sandboxed)
+ClaimJobResult claim_job(classad::ClassAd const &ad, const ScheddContactInfo & scci, int cluster, int proc, std::string& error_details, const char * my_identity)
 {
 	classad::ClassAd * job = const_cast<classad::ClassAd*>(&ad);
 	job->InsertAttr(ATTR_JOB_MANAGED, MANAGED_EXTERNAL);
@@ -560,24 +542,19 @@ ClaimJobResult claim_job(classad::ClassAd const &ad, const char * pool_name, con
 	return CJR_OK;
 }
 
-bool yield_job(bool done, int cluster, int proc, classad::ClassAd const &job_ad, std::string * error_details, const char * my_identity, bool target_is_sandboxed, bool release_on_hold, bool *keep_trying) {
-	return true;
-}
 
-
-bool yield_job(classad::ClassAd const &ad,const char * pool_name,
-	const char * schedd_name, bool done, int cluster, int proc,
-	std::string * error_details, const char * my_identity, bool target_is_sandboxed,
+bool yield_job(classad::ClassAd const &ad, const ScheddContactInfo & scci,
+	bool done, int cluster, int proc,
+	std::string& error_details, const char * my_identity,
         bool release_on_hold, bool *keep_trying)
 {
 	return true;
 }
 
-bool submit_job( const std::string & owner, const std::string & domain, ClassAd & src, const char * schedd_name, const char * pool_name, bool is_sandboxed, int * cluster_out /*= 0*/, int * proc_out /*= 0 */)
+bool submit_job( const std::string & owner, const std::string & domain, ClassAd & src, const ScheddContactInfo & scci, bool is_sandboxed,int * cluster_out /*= 0*/, int * proc_out /*= 0 */)
 {
-	fprintf(stdout, "submit_job as %s@%s to %s pool:%s%s:\n", owner.c_str(), domain.c_str(),
-		schedd_name ? schedd_name : "local",
-		pool_name ? pool_name : "local",
+	fprintf(stdout, "submit_job as %s@%s to %s%s:\n", owner.c_str(), domain.c_str(),
+		scci.label(),
 		is_sandboxed ? " (sandboxed)" : "");
 	if (submitted_jobs_fh) {
 		fPrintAd(submitted_jobs_fh, src);
@@ -601,7 +578,7 @@ bool push_dirty_attributes(classad::ClassAd & src)
 	the dirty attributes.
 	Establishes (and tears down) a qmgr connection.
 */
-bool push_dirty_attributes(classad::ClassAd & src, const char * schedd_name, const char * pool_name)
+bool push_dirty_attributes(classad::ClassAd & src, const ScheddContactInfo & scci)
 {
 	return true;
 }
@@ -621,17 +598,17 @@ bool push_classad_diff(classad::ClassAd & src,classad::ClassAd & dest)
     This handles attribute deletion as well as change of value.
 	Establishes (and tears down) a qmgr connection.
 */
-bool push_classad_diff(classad::ClassAd & src, classad::ClassAd & dest, const char * schedd_name, const char * pool_name)
+bool push_classad_diff(classad::ClassAd & src, classad::ClassAd & dest, const ScheddContactInfo & scci)
 {
 	return true;
 }
 
-bool finalize_job(const std::string &owner, const std::string &domain, classad::ClassAd const &ad,int cluster, int proc, const char * schedd_name, const char * pool_name, bool is_sandboxed)
+bool finalize_job(const std::string &owner, const std::string &domain, classad::ClassAd const &ad,int cluster, int proc, const ScheddContactInfo & scci, bool is_sandboxed)
 {
 	return true;
 }
 
-bool remove_job(classad::ClassAd const &ad, int cluster, int proc, char const *reason, const char * schedd_name, const char * pool_name, std::string &error_desc)
+bool remove_job(int cluster, int proc, char const *reason, const ScheddContactInfo & scci, std::string &error_desc)
 {
 	return true;
 }
@@ -651,32 +628,32 @@ bool InitializeHoldEvent( JobHeldEvent *event, classad::ClassAd const &job_ad )
 	return true;
 }
 
-bool WriteEventToUserLog( ULogEvent const &event, classad::ClassAd const &ad )
+bool WriteEventToUserLog( ULogEvent const &event, classad::ClassAd const &ad, UserRecord* )
 {
 	return true;
 }
 
-bool WriteTerminateEventToUserLog( classad::ClassAd const &ad )
+bool WriteTerminateEventToUserLog( classad::ClassAd const &ad, const UserRecord* )
 {
 	return true;
 }
 
-bool WriteAbortEventToUserLog( classad::ClassAd const &ad )
+bool WriteAbortEventToUserLog( classad::ClassAd const &ad, const UserRecord* )
 {
 	return true;
 }
 
-bool WriteHoldEventToUserLog( classad::ClassAd const &ad )
+bool WriteHoldEventToUserLog( classad::ClassAd const &ad, const UserRecord* )
 {
 	return true;
 }
 
-bool WriteExecuteEventToUserLog( classad::ClassAd const &ad )
+bool WriteExecuteEventToUserLog( classad::ClassAd const &ad, const UserRecord* )
 {
 	return true;
 }
 
-bool WriteEvictEventToUserLog( classad::ClassAd const &ad )
+bool WriteEvictEventToUserLog( classad::ClassAd const &ad, const UserRecord* )
 {
 	return true;
 }
