@@ -288,8 +288,8 @@ void Accountant::Initialize(GroupEntry* root_group)
       grpq.pop_front();
       // This creates entries if they don't already exist:
       GetPriority(group->name);
-      for (std::vector<GroupEntry*>::iterator j(group->children.begin());  j != group->children.end();  ++j) {
-          grpq.push_back(*j);
+      for (GroupEntry* child : group->children) {
+          grpq.push_back(child);
       }
   }
 
@@ -587,7 +587,7 @@ void Accountant::AddMatch(const std::string& CustomerName, ClassAd* ResourceAd)
   GetAttributeInt(CustomerRecord+CustomerName,ResourcesUsedAttr,ResourcesUsed);
   double WeightedResourcesUsed=0.0;
   GetAttributeFloat(CustomerRecord+CustomerName,WeightedResourcesUsedAttr,WeightedResourcesUsed);
-  int UnchargedTime=0;
+  time_t UnchargedTime=0;
   GetAttributeInt(CustomerRecord+CustomerName,UnchargedTimeAttr,UnchargedTime);
   double WeightedUnchargedTime=0.0;
   GetAttributeFloat(CustomerRecord+CustomerName,WeightedUnchargedTimeAttr,WeightedUnchargedTime);
@@ -613,7 +613,7 @@ void Accountant::AddMatch(const std::string& CustomerName, ClassAd* ResourceAd)
 
   int GroupResourcesUsed=0;
   double GroupWeightedResourcesUsed = 0.0;
-  int GroupUnchargedTime=0;
+  time_t GroupUnchargedTime=0;
   double WeightedGroupUnchargedTime=0.0;
 
   dprintf(D_ACCOUNTANT, "Customername %s GroupName is: %s\n",CustomerName.c_str(), GroupName.c_str());
@@ -709,7 +709,7 @@ void Accountant::RemoveMatch(const std::string& ResourceName, time_t T)
   double WeightedResourcesUsed=0;
   GetAttributeFloat(CustomerRecord+CustomerName,WeightedResourcesUsedAttr,WeightedResourcesUsed);
   
-  int UnchargedTime=0;
+  time_t UnchargedTime=0;
   GetAttributeInt(CustomerRecord+CustomerName,UnchargedTimeAttr,UnchargedTime);
   double WeightedUnchargedTime=0.0;
   GetAttributeFloat(CustomerRecord+CustomerName,WeightedUnchargedTimeAttr,WeightedUnchargedTime);
@@ -719,7 +719,7 @@ void Accountant::RemoveMatch(const std::string& ResourceName, time_t T)
   
   int GroupResourcesUsed=0;
   double GroupWeightedResourcesUsed=0.0;
-  int GroupUnchargedTime=0;
+  time_t GroupUnchargedTime=0;
   double WeightedGroupUnchargedTime=0.0;
   double HierWeightedResourcesUsed = 0.0;
   
@@ -836,7 +836,7 @@ void Accountant::DisplayMatches()
 void Accountant::UpdatePriorities() 
 {
   time_t T=time(0);
-  int TimePassed=T-LastUpdateTime;
+  time_t TimePassed=T-LastUpdateTime;
   if (TimePassed==0) return;
 
   // TimePassed might be less than zero for a number of reasons, including
@@ -850,7 +850,7 @@ void Accountant::UpdatePriorities()
   LastUpdateTime=T;
   SetAttributeInt(AcctRecord,LastUpdateTimeAttr,LastUpdateTime);
 
-  dprintf(D_ACCOUNTANT,"(ACCOUNTANT) Updating priorities - AgingFactor=%8.3f , TimePassed=%d\n",AgingFactor,TimePassed);
+  dprintf(D_ACCOUNTANT,"(ACCOUNTANT) Updating priorities - AgingFactor=%8.3f , TimePassed=%lld\n",AgingFactor,(long long)TimePassed);
 
   std::string HK;
   ClassAd* ad;
@@ -896,7 +896,7 @@ void Accountant::UpdatePriorities()
 }
 
 void
-Accountant::UpdateOnePriority(int T, int TimePassed, double AgingFactor, const char *key, ClassAd *ad) {
+Accountant::UpdateOnePriority(time_t T, time_t TimePassed, double AgingFactor, const char *key, ClassAd *ad) {
 
 	double Priority, OldPrio, PriorityFactor;
 	int UnchargedTime;
@@ -1004,28 +1004,25 @@ Accountant::UpdateOnePriority(int T, int TimePassed, double AgingFactor, const c
 // that were broken (by checkibg the claimed state)
 //------------------------------------------------------------------
 
-void Accountant::CheckMatches(ClassAdListDoesNotDeleteAds& ResourceList) 
+void Accountant::CheckMatches(std::vector<ClassAd *> &ResourceList) 
 {
   dprintf(D_ACCOUNTANT,"(Accountant) Checking Matches\n");
 
-  ClassAd* ResourceAd;
   std::string HK;
   ClassAd* ad;
   std::string ResourceName;
   std::string CustomerName;
 
 	  // Create a hash table for speedier lookups of Resource ads.
-  HashTable<std::string,ClassAd *> resource_hash(hashFunction);
-  ResourceList.Open();
-  while ((ResourceAd=ResourceList.Next())!=NULL) {
+  std::map<std::string,ClassAd *> resource_hash;
+  for (ClassAd *ResourceAd: ResourceList) {
     ResourceName = GetResourceName(ResourceAd);
-    bool success = ( resource_hash.insert( ResourceName, ResourceAd ) == 0 );
+    auto [it, success] = resource_hash.emplace(ResourceName, ResourceAd);
     if (!success) {
       dprintf(D_ALWAYS, "WARNING: found duplicate key: %s\n", ResourceName.c_str());
       dPrintAd(D_FULLDEBUG, *ResourceAd);
     }
   }
-  ResourceList.Close();
 
   // Remove matches that were broken
   AcctLog->table.startIterations();
@@ -1033,12 +1030,14 @@ void Accountant::CheckMatches(ClassAdListDoesNotDeleteAds& ResourceList)
     char const *key = HK.c_str();
     if (strncmp(ResourceRecord.c_str(),key,ResourceRecord.length())) continue;
     ResourceName=key+ResourceRecord.length();
-    if( resource_hash.lookup(ResourceName,ResourceAd) < 0 ) {
+    auto itr = resource_hash.find(ResourceName);
+    if (itr == resource_hash.end()) {
       dprintf(D_ACCOUNTANT,"Resource %s class-ad wasn't found in the resource list.\n",ResourceName.c_str());
       RemoveMatch(ResourceName);
     }
 	else {
 		// Here we need to figure out the CustomerName.
+      ClassAd* ResourceAd = itr->second;
       ad->LookupString(RemoteUserAttr,CustomerName);
       if (!CheckClaimedOrMatched(ResourceAd, CustomerName)) {
         dprintf(D_ACCOUNTANT,"Resource %s was not claimed by %s - removing match\n",ResourceName.c_str(),CustomerName.c_str());
@@ -1048,12 +1047,10 @@ void Accountant::CheckMatches(ClassAdListDoesNotDeleteAds& ResourceList)
   }
 
   // Scan startd ads and add matches that are not registered
-  ResourceList.Open();
-  while ((ResourceAd=ResourceList.Next())!=NULL) {
+  for (ClassAd *ResourceAd: ResourceList) {
 	  std::string cust_name;
     if (IsClaimed(ResourceAd, cust_name)) AddMatch(cust_name, ResourceAd);
   }
-  ResourceList.Close();
 
 	  // Recalculate limits from the set of resources that are reporting
   LoadLimits(ResourceList);
@@ -1116,7 +1113,7 @@ void Accountant::CheckResources(const std::string& CustomerName, int& NumResourc
     NumResourcesRW = 0;
 
     bool isGroup=false;
-	std::string cgrp = GroupEntry::GetAssignedGroup(hgq_root_group, CustomerName.c_str(), isGroup)->name;
+	std::string cgrp = GroupEntry::GetAssignedGroup(hgq_root_group, CustomerName, isGroup)->name;
     // This is a defunct group:
     if (isGroup && (cgrp != CustomerName)) return;
 
@@ -1162,8 +1159,8 @@ ClassAd* Accountant::ReportState(bool rollup) {
         GroupEntry* group = grpq.front();
         grpq.pop_front();
         gnmap[group->name] = EntryNum++;
-        for (std::vector<GroupEntry*>::iterator j(group->children.begin());  j != group->children.end();  ++j) {
-            grpq.push_back(*j);
+        for (GroupEntry* child : group->children) {
+            grpq.push_back(child);
         }
     }
 
@@ -1370,8 +1367,8 @@ void Accountant::ReportGroups(GroupEntry* group, ClassAd* ad, bool rollup, std::
     
     // Populate group's children recursively, if it has any
     // Recursion is to allow for proper rollup from children to parents
-    for (std::vector<GroupEntry*>::iterator j(group->children.begin());  j != group->children.end();  ++j) {
-        ReportGroups(*j, ad, rollup, gnmap);
+    for (GroupEntry* child : group->children) {
+        ReportGroups(child, ad, rollup, gnmap);
     }
 
     // if we aren't doing rollup, finish now
@@ -1778,24 +1775,6 @@ bool Accountant::GetAttributeString(const std::string& Key, const std::string& A
   return true;
 }
 
-//------------------------------------------------------------------
-// Find a resource ad in class ad list (by name)
-//------------------------------------------------------------------
-
-#if 0
-ClassAd* Accountant::FindResourceAd(const string& ResourceName, ClassAdListDoesNotDeleteAds& ResourceList)
-{
-  ClassAd* ResourceAd;
-  
-  ResourceList.Open();
-  while ((ResourceAd=ResourceList.Next())!=NULL) {
-	if (ResourceName==GetResourceName(ResourceAd)) break;
-  }
-  ResourceList.Close();
-  return ResourceAd;
-}
-#endif
-
 //------------------------------------------------------------------
 // Get the users domain
 //------------------------------------------------------------------
@@ -1813,17 +1792,14 @@ std::string Accountant::GetDomain(const std::string& CustomerName)
 // Functions for accessing and changing Concurrency Limits
 //------------------------------------------------------------------
 
-void Accountant::LoadLimits(ClassAdListDoesNotDeleteAds &resourceList)
+void Accountant::LoadLimits(std::vector<ClassAd *> &resourceList)
 {
-	ClassAd *resourceAd;
-
 		// Wipe out all the knowledge of limits we think we know
 	dprintf(D_ACCOUNTANT, "Previous Limits --\n");
 	ClearLimits();
 
 		// Record all the limits that are actually in use in the pool
-	resourceList.Open();
-	while (NULL != (resourceAd = resourceList.Next())) {
+	for (ClassAd *resourceAd: resourceList) {
 		std::string limits;
 
 		if (resourceAd->LookupString(ATTR_CONCURRENCY_LIMITS, limits)) {
@@ -1848,7 +1824,6 @@ void Accountant::LoadLimits(ClassAdListDoesNotDeleteAds &resourceList)
 			IncrementLimits(str);
 		}
 	}
-	resourceList.Close();
 
 		// Print out the new limits, at D_ACCOUNTANT. This is useful
 		// because the list printed from ClearLimits can be compared
