@@ -113,8 +113,8 @@ class Matchmaker : public Service
 		void updateCollector( int timerID = -1 );
 		
 		// auxillary functions
-		bool obtainAdsFromCollector(ClassAdList &allAds, ClassAdListDoesNotDeleteAds &startdAds, std::vector<ClassAd *> &submitterAds, std::set<std::string> &submitterNames, ClaimIdHash &claimIds );	
-		char * compute_significant_attrs(ClassAdListDoesNotDeleteAds & startdAds);
+		bool obtainAdsFromCollector(ClassAdList &allAds, std::vector<ClassAd *> &startdAds, std::vector<ClassAd *> &submitterAds, std::set<std::string> &submitterNames, ClaimIdHash &claimIds );	
+		void compute_significant_attrs(std::vector<ClassAd *> & startdAds, std::string & attr_refs);
 		bool consolidate_globaljobprio_submitter_ads(std::vector<ClassAd *>& submitterAds) const;
 
 		void SetupMatchSecurity(std::vector<ClassAd *> &submitterAds);
@@ -169,7 +169,7 @@ class Matchmaker : public Service
 		int negotiate(char const* groupName, char const *submitterName, const ClassAd *submitterAd,
 		   double priority,
            double submitterLimit, double submitterLimitUnclaimed, int submitterCeiling,
-		   ClassAdListDoesNotDeleteAds &startdAds, ClaimIdHash &claimIds, 
+		   std::vector<ClassAd *> &startdAds, ClaimIdHash &claimIds, 
 		   bool ignore_schedd_limit, time_t deadline,
            int& numMatched, double &pieLeft);
 
@@ -177,13 +177,13 @@ class Matchmaker : public Service
 								 int untrimmed_num_startds,
 								 double untrimmedSlotWeightTotal,
 								 double minSlotWeight,
-								 ClassAdListDoesNotDeleteAds& startdAds, 
+								 std::vector<ClassAd *>& startdAds, 
 								 ClaimIdHash& claimIds,
 								 std::vector<ClassAd *>& submitterAds, 
 								 double groupQuota=INT_MAX, const char* groupName=NULL);
 
 		
-		ClassAd *matchmakingAlgorithm(const char* submitterName, const char* scheddAddr, const char* scheddName, ClassAd& request, ClassAdListDoesNotDeleteAds& startdAds,
+		ClassAd *matchmakingAlgorithm(const char* submitterName, const char* scheddAddr, const char* scheddName, ClassAd& request, std::vector<ClassAd *>& startdAds,
 									  double preemptPrio, 
                                       double limitUsed, double limitUsedUnclaimed,
                                       double submitterLimit, double submitterLimitUnclaimed, 
@@ -273,9 +273,9 @@ class Matchmaker : public Service
 
 		void MakeClaimIdHash(ClassAdList &startdPvtAdList, ClaimIdHash &claimIds);
 		void addRemoteUserPrios( ClassAd* ad );
-		void addRemoteUserPrios( ClassAdListDoesNotDeleteAds &cal );
+		void addRemoteUserPrios( std::vector<ClassAd *> &cal );
 		void insertNegotiatorMatchExprs(ClassAd *ad);
-		void insertNegotiatorMatchExprs( ClassAdListDoesNotDeleteAds &cal );
+		void insertNegotiatorMatchExprs(std::vector<ClassAd *> &cal );
 		void reeval( ClassAd *ad );
 		void updateNegCycleEndTime(time_t startTime, ClassAd *submitter);
 		friend int comparisonFunction (ClassAd *, ClassAd *,
@@ -298,14 +298,14 @@ class Matchmaker : public Service
 			@param startdAds List of startd ads to trim
 			@return the number of ads removed from the startdAds list 
 		**/
-		int trimStartdAds(ClassAdListDoesNotDeleteAds &startdAds);
+		int trimStartdAds(std::vector<ClassAd *> &startdAds);
 		// Note: these are called by trimStartdAds as required
-		int trimStartdAds_PreemptionLogic(ClassAdListDoesNotDeleteAds &startdAds) const;
-		int trimStartdAds_ShutdownLogic(ClassAdListDoesNotDeleteAds &startdAds);
-		int trimStartdAds_ClaimedPslotLogic(ClassAdListDoesNotDeleteAds &startdAds);
+		int trimStartdAds_PreemptionLogic(std::vector<ClassAd *> &startdAds) const;
+		int trimStartdAds_ShutdownLogic(std::vector<ClassAd *> &startdAds);
+		int trimStartdAds_ClaimedPslotLogic(std::vector<ClassAd *> &startdAds);
 
 		bool SubmitterLimitPermits(ClassAd* request, ClassAd* candidate, double used, double allowed, double pieLeft);
-		double sumSlotWeights(ClassAdListDoesNotDeleteAds &startdAds,double *minSlotWeight, ExprTree* constraint);
+		double sumSlotWeights(std::vector<ClassAd *> &startdAds,double *minSlotWeight, ExprTree* constraint);
 
 			// Returns a pslot to the match list (after consumption policies have been applied).
 			// Recalculates ranks and re-sorts match list.
@@ -317,13 +317,13 @@ class Matchmaker : public Service
 
 		// configuration information
 		char *AccountantHost;		// who (if at all) is the accountant?
-		int  NegotiatorInterval;	// max interval between negotiation cycles
-		int  NegotiatorMinInterval;	// min interval between negotiation cycles
+		time_t  NegotiatorInterval;	// max interval between negotiation cycles
+		time_t  NegotiatorMinInterval;	// min interval between negotiation cycles
 		int  NegotiatorTimeout;		// timeouts for communication
 		int  MaxTimePerCycle;		// how long for total negotiation cycle
 		int  MaxTimePerSubmitter;   // how long to talk to any one submitter
 		int  MaxTimePerSpin;        // How long per pie spin
-		int  MaxTimePerSchedd;		// How long to talk to any one schedd
+		time_t  MaxTimePerSchedd;		// How long to talk to any one schedd
 		ExprTree *PreemptionReq;	// only preempt if true
 		ExprTree *PreemptionRank; 	// rank preemption candidates
 		bool preemption_req_unstable;
@@ -340,6 +340,7 @@ class Matchmaker : public Service
 		bool want_inform_startd;	
 		/// Should the negotiator use non-blocking connect to contact startds?
 		bool want_nonblocking_startd_contact;
+		bool MatchPasswordEnabled{false};
         char * strSlotConstraint; // query collector for machineAds with this constraint
         ExprTree *SlotPoolsizeConstraint;   // Filter machineAds by this
                                          // constraint before calculating quotas
@@ -352,14 +353,13 @@ class Matchmaker : public Service
 
 		std::map<std::string, std::string> NegotiatorMatchExprs;
 
-		std::map<std::string, int> ScheddsTimeInCycle;
+		std::map<std::string, time_t> ScheddsTimeInCycle;
 
 		CollectorList* Collectors;
 
-		typedef HashTable<std::string, MapEntry*> AdHash;
-		AdHash *stashedAds;			
+		std::map<std::string, MapEntry*> stashedAds;
 
-		groupQuotasHashType *groupQuotasHash;
+		groupQuotasHashType groupQuotasHash;
 
 		// rank condition on matches
 		ExprTree *rankCondStd;// no preemption or machine rank-preemption 
@@ -377,7 +377,7 @@ class Matchmaker : public Service
 		bool GotRescheduleCmd;
 
 		// external references in startd ads ... used for autoclustering
-		char * job_attr_references;
+		std::string job_attr_references;
 
 		// Epoch time when we started/finished most rescent negotiation cycle
 		time_t startedLastCycleTime;
@@ -535,6 +535,8 @@ class Matchmaker : public Service
         bool accept_surplus;
         bool autoregroup;
 
+		bool force_my_slot_weight{false};
+
         // true if resource ads with consumption policies are present
         // for the current negotiation cycle
         bool cp_resources;
