@@ -34,8 +34,7 @@
 
 size_t pidHashFunc( const pid_t& pid );
 
-HashTable <pid_t, procHashNode *> * ProcAPI::procHash = 
-    new HashTable <pid_t, procHashNode *> ( pidHashFunc );
+std::map<pid_t, procHashNode> ProcAPI::procHash;
 
 piPTR ProcAPI::allProcInfos = NULL;
 
@@ -106,19 +105,6 @@ double ProcAPI::TIME_UNITS_PER_SEC = (double) sysconf(_SC_CLK_TCK);
 double ProcAPI::TIME_UNITS_PER_SEC = 1.0;
 #endif
 
-procHashNode::procHashNode()
-{
-	lasttime = 0.0;
-	oldtime = 0.0;
-	oldusage = 0.0;
-	oldminf = 0L;
-	oldmajf = 0L;
-	majfaultrate = 0L;
-	minfaultrate = 0L;
-	creation_time = 0L;
-	garbage = false;
-}
-
 ProcAPI::~ProcAPI() {
         // deallocate stuff like crazy.
 #ifndef WIN32
@@ -126,13 +112,6 @@ ProcAPI::~ProcAPI() {
 #endif
     deallocAllProcInfos();
 
-    struct procHashNode * phn;
-    procHash->startIterations();
-    while ( procHash->iterate( phn ) )
-        delete phn;
-    
-    delete procHash;
-
 #ifdef WIN32
     if ( offsets )
         free ( offsets );
@@ -617,14 +596,10 @@ ProcAPI::fillProcInfoEnv(piPTR pi)
 		do {
 			if (env_buffer == NULL) {
 				env_buffer = (char*)malloc(sizeof(char) * read_size);
-				if ( env_buffer == NULL ) {
-					EXCEPT( "Procapi::getProcInfo: Out of memory!");
-				}
+				ASSERT(env_buffer);
 			} else {
 				env_buffer = (char*)realloc(env_buffer, read_size * multiplier);
-				if ( env_buffer == NULL ) {
-					EXCEPT( "Procapi::getProcInfo: Out of memory!");
-				}
+				ASSERT(env_buffer);
 				multiplier++;
 			}
 
@@ -664,9 +639,7 @@ ProcAPI::fillProcInfoEnv(piPTR pi)
 		// it mimics and environ variable. The addition +1 is for the end NULL;
 		char **env_environ;
 		env_environ = (char**)malloc(sizeof(char *) * (entries + 1));
-		if (env_environ == NULL) {
-			EXCEPT( "Procapi::getProcInfo: Out of memory!");
-		}
+		ASSERT(env_environ);
 
 		// set up the pointers from the env_environ into the env_buffer
 		index = 0;
@@ -1198,9 +1171,7 @@ ProcAPI::getProcInfoRaw( pid_t pid, procInfoRaw& procRaw, int &status )
 	}
 
 	kprocbuf = kp = (struct kinfo_proc *)malloc(bufSize);
-	if (kp == NULL) {
-		EXCEPT("ProcAPI: getProcInfo() Out of memory!");
-	}
+	ASSERT(kp);
 
 	if (sysctl(mib, 4, kp, &bufSize, NULL, 0) < 0) {
 		if (errno == ESRCH) {
@@ -1501,19 +1472,17 @@ ProcAPI::getProcInfoRaw( pid_t pid, procInfoRaw& procRaw, int &status )
         *((LARGE_INTEGER*)(ctrblk + offsets->utime));
     LARGE_INTEGER st = (LARGE_INTEGER) 
         *((LARGE_INTEGER*)(ctrblk + offsets->stime));
-    LARGE_INTEGER imgsz = (LARGE_INTEGER) 
-        *((LARGE_INTEGER*)(ctrblk + offsets->imgsize));
 
     procRaw.pid       = (pid_t) *((long*)(ctrblk + offsets->procid  ));
     procRaw.ppid      = ntSysInfo.GetParentPID(pid);
-    procRaw.imgsize   = imgsz.QuadPart;
 
 	if (offsets->rssize_width == 4) {
 		procRaw.rssize = *(long*)(ctrblk + offsets->rssize);
+		procRaw.imgsize = *(long*)(ctrblk + offsets->workset);
 	}
 	else {
-		procRaw.rssize =
-			((LARGE_INTEGER*)(ctrblk + offsets->rssize))->QuadPart;
+		procRaw.rssize = ((LARGE_INTEGER*)(ctrblk + offsets->rssize))->QuadPart;
+		procRaw.imgsize = ((LARGE_INTEGER*)(ctrblk + offsets->workset))->QuadPart;
 	}
 
 	procRaw.majfault  = (long) *((long*)(ctrblk + offsets->faults  ));
@@ -1587,15 +1556,16 @@ ProcAPI::buildProcInfoList(pid_t /*BOLOpid*/)
         ++cGetProcInfoListPid;
         sGetProcInfoListPid += qpcDeltaSec(iter_start);
 
-		LARGE_INTEGER* liptr;
-		liptr = (LARGE_INTEGER*)(ctrblk + offsets->imgsize);
-		pi->imgsize = (unsigned long)(liptr->QuadPart / 1024);
+		LARGE_INTEGER* liptr = nullptr;
 		if (offsets->rssize_width == 4) {
 			pi->rssize = *(long*)(ctrblk + offsets->rssize) / 1024;
+			pi->imgsize = *(long*)(ctrblk + offsets->workset) / 1024;
 		}
 		else {
 			liptr = (LARGE_INTEGER*)(ctrblk + offsets->rssize);
 			pi->rssize = (unsigned long)(liptr->QuadPart / 1024);
+			liptr = (LARGE_INTEGER*)(ctrblk + offsets->workset);
+			pi->imgsize = (unsigned long)(liptr->QuadPart / 1024);
 		}
 
 		pi->user_time = (long)(ut.QuadPart / objectFrequency);
@@ -1692,23 +1662,21 @@ ProcAPI::do_usage_sampling( piPTR& pi,
 		procHashNode which we have not performed a lookup on in
 		over an hour.  -Todd <tannenba@cs.wisc.edu> */
 	static double last_garbage_collection_time = 0.0;	
-	pid_t garbage_pid;
 	if ( now - last_garbage_collection_time > 3600 ) {
 		last_garbage_collection_time = now;
 			// first delete anything still flagged as garbage
-		procHash->startIterations();
-		while ( procHash->iterate( garbage_pid, phn ) ) {
-			if ( phn->garbage == true ) {
+		auto it = procHash.begin();
+		while (it != procHash.end()) {
+			if ( it->second.garbage == true ) {
 				// it is still flagged as garbage; delete it
-				procHash->remove(garbage_pid);
-				delete phn;
-				phn = NULL;
+				it = procHash.erase(it);
 			} else {
 				// it is not still flagged as garbarge; so do 
 				// not delete it, but instead reset the garbage
 				// flag to true.  It will be cleared when/if we
 				// perform a lookup on this node.
-				phn->garbage = true;
+				it->second.garbage = true;
+				it++;
 			}
 		}	// end of while loop to iterate through the hash table
 	}	// end of if it is garbage collection time
@@ -1722,7 +1690,10 @@ ProcAPI::do_usage_sampling( piPTR& pi,
 		   sure we've got the right one. Mike & Derek 3/24/99 */
 
 	phn = NULL;	// clear to NULL before attempting the lookup
-	if (procHash->lookup( pi->pid, phn ) == 0) {
+	auto it = procHash.find(pi->pid);
+	if (it != procHash.end()) {
+
+		phn = &it->second;
 
 		/* Allow 2 seconds "slack" on creation time, like we do in
 		   ProcFamily, since (at least on Linux) the value can
@@ -1731,8 +1702,7 @@ ProcAPI::do_usage_sampling( piPTR& pi,
 		long birth = phn->creation_time - pi->creation_time;
 		if (-2 > birth || birth > 2) {
 			/* must be a different process associated with this, remove it. */
-			procHash->remove(pi->pid);
-			delete phn;
+			procHash.erase(it);
 			phn = NULL;
 		}
 	}
@@ -1786,24 +1756,21 @@ ProcAPI::do_usage_sampling( piPTR& pi,
             pi->minfault = (long unsigned)(nowminf / (double) pi->age);
             pi->majfault = (long unsigned)(nowmajf / (double) pi->age);
         }
-    } 
 
-		// if we got that phn from the hashtable, remove it now.
-	if ( phn ) {
-		procHash->remove(pi->pid);
-	}
+        auto [it, success] = procHash.emplace(pi->pid, procHashNode());
+        ASSERT(success);
+        phn = &it->second;
+    }
 
 		// put new vals back into hashtable
-	struct procHashNode * new_phn = new procHashNode;
-	new_phn->lasttime = now;
-	new_phn->oldtime  = ustime;   // store raw data for next call...
-	new_phn->oldminf  = nowminf;  //  ""
-	new_phn->oldmajf  = nowmajf;  //  ""
-	new_phn->oldusage = pi->cpuusage;  // Also store results in case the
-	new_phn->minfaultrate = pi->minfault;   // next sample is < 1 sec
-	new_phn->majfaultrate = pi->majfault;   // from now.
-	new_phn->creation_time = pi->creation_time;
-	procHash->insert( pi->pid, new_phn );
+	phn->lasttime = now;
+	phn->oldtime  = ustime;   // store raw data for next call...
+	phn->oldminf  = nowminf;  //  ""
+	phn->oldmajf  = nowmajf;  //  ""
+	phn->oldusage = pi->cpuusage;  // Also store results in case the
+	phn->minfaultrate = pi->minfault;   // next sample is < 1 sec
+	phn->majfaultrate = pi->majfault;   // from now.
+	phn->creation_time = pi->creation_time;
 
 		// due to some funky problems, do some sanity checking here for
 		// strange numbers:
@@ -1832,9 +1799,6 @@ ProcAPI::do_usage_sampling( piPTR& pi,
 		pi->age = 0;
 	}
 
-		// now we can delete this.
-	if ( phn ) delete phn;
-	
 }
 
 procInfo*
@@ -1887,6 +1851,10 @@ ProcAPI::initpi ( piPTR& pi ) {
 #if !defined(WIN32)
 	pi->owner    = 0;
 #endif
+#if HAVE_PSS
+    pi->pssize = 0;
+	pi->pssize_available = false;
+#endif
 
 	pidenvid_init(&pi->penvid);
 }
@@ -2274,15 +2242,8 @@ ProcAPI::printProcInfo(FILE* fp, piPTR pi){
 uid_t 
 ProcAPI::getFileOwner(int fd) {
 	
-#if defined(HAVE_FSTAT64) && !defined(DARWIN)
-	// If we do not use fstat64(), fstat() fails if the inode number
-	// is too big and possibly for a few other reasons as well.
-	struct stat64 si;
-	if ( fstat64(fd, &si) != 0 ) {
-#else
 	struct stat si;
 	if ( fstat(fd, &si) != 0 ) {
-#endif
 		dprintf(D_ALWAYS, 
 			"ProcAPI: fstat failed in /proc! (errno=%d)\n", errno);
 		return 0; 	// 0 is probably wrong, but this should never
@@ -2407,11 +2368,6 @@ void ProcAPI::grabOffsets ( PPERF_OBJECT_TYPE pThisObject ) {
   
     pThisCounter = nextCounter(pThisCounter);  // "Virtual Bytes Peak"
     pThisCounter = nextCounter(pThisCounter);  // "Virtual Bytes"
-//    printcounter ( stdout, pThisCounter );
-    offsets->imgsize = pThisCounter->CounterOffset;  // image size
-	if (pThisCounter->CounterSize != 8) {
-		unexpected_counter_size("image size", pThisCounter->CounterSize, "8");
-	}
   
     pThisCounter = nextCounter(pThisCounter);  // "Page Faults/Sec"
 //    printcounter ( stdout, pThisCounter );
@@ -2424,13 +2380,20 @@ void ProcAPI::grabOffsets ( PPERF_OBJECT_TYPE pThisObject ) {
     offsets->rssize = pThisCounter->CounterOffset;   // working set peak 
 	offsets->rssize_width = pThisCounter->CounterSize;
 	if ((offsets->rssize_width != 4) && (offsets->rssize_width != 8)) {
-		unexpected_counter_size("working set",
+		unexpected_counter_size("working set peak",
 		                        pThisCounter->CounterSize,
 		                        "4 or 8");
 	}
 
 //    printcounter ( stdout, pThisCounter );
 	pThisCounter = nextCounter(pThisCounter); // "Working Set"
+	offsets->workset = pThisCounter->CounterOffset;   // working set
+	if (offsets->rssize_width != pThisCounter->CounterSize) {
+		unexpected_counter_size("working set",
+			pThisCounter->CounterSize,
+			"equal in size to working set peak");
+	}
+
 	pThisCounter = nextCounter(pThisCounter); // "Page File Bytes Peak"
 
     
