@@ -24,6 +24,7 @@
 #include "condor_attributes.h"
 #include "classad/classadCache.h" // for CachedExprEnvelope
 #include "qmgmt.h"
+#include "qmgmt_startup_limits.h"
 #include "schedd_stats.h" // for schedd_runtime_probe
 
 // this is a placeholder for a future class that will compactly hold a set of jobs
@@ -40,6 +41,7 @@ public:
 	bool last(JOB_ID_KEY &jid)  { if (jobs.empty()) return false; jid = *(jobs.rbegin()); return true; }
 	void insert(JOB_ID_KEY &jid) { jobs.insert(jid); }
 	void erase(JOB_ID_KEY &jid) { jobs.erase(jid); }
+	const std::set<JOB_ID_KEY> & joblist() const { return jobs; }
 private:
 	std::set<JOB_ID_KEY> jobs;
 	std::set<JOB_ID_KEY>::const_iterator it;
@@ -240,8 +242,11 @@ int JobCluster::getClusterid(JobQueueJob & job, bool expand_refs, std::string *
 			// (Also we can't tolerate dotted attribute names in the significant attrs list)
 			job.GetExternalReferences(tree, exattrs, true);
 			for (auto it = exattrs.begin(); it != exattrs.end();) { // c++ 20 has erase_if, but we can't use it
-				auto tmp = it++;
-				if (tmp->find_first_of('.') != std::string::npos) { exattrs.erase(tmp); }
+				if (it->find_first_of('.') != std::string::npos) {
+					it = exattrs.erase(it);
+				} else {
+					it++;
+				}
 			}
 			// now add in the internal refs
 			job.GetInternalReferences(tree, exattrs, false);
@@ -452,6 +457,11 @@ bool AutoCluster::config(const classad::References &basic_attrs, const char* sig
 			required_attrs.insert(basic_attr);
 		}
 
+		// Ensure job-only startup limit attributes participate in autocluster signatures.
+		classad::References limit_refs;
+		StartupLimitsCollectJobRefs(limit_refs);
+		required_attrs.insert(limit_refs.begin(), limit_refs.end());
+
 		// If the configuration specifies a whitelist of attributes, add them to the required attrs list
 		std::string other_attrs;
 		if (param(other_attrs, "ADD_SIGNIFICANT_ATTRIBUTES")) {
@@ -504,13 +514,13 @@ bool AutoCluster::config(const classad::References &basic_attrs, const char* sig
 						continue; // skip this one
 					}
 				}
-				attrs.append(attr->c_str());
+				attrs.append(*attr);
 				attrs.append(" ");
 			}
 
 			// now append the required attrs that were not banned and not already in the input list.
 			for (const auto & required_attr : required_attrs) {
-				attrs.append(required_attr.c_str());
+				attrs.append(required_attr);
 				attrs.append(" ");
 			}
 			trim(attrs);
@@ -669,6 +679,17 @@ void AutoCluster::removeFromAutocluster(JobQueueJob &job)
 	}
 }
 
+#ifdef USE_AUTOCLUSTER_TO_JOBID_MAP
+const std::set<JOB_ID_KEY> & AutoCluster::joblist(JobQueueJob & job) {
+	static std::set<JOB_ID_KEY> empty;
+	auto jit = find_job_id_set(job);
+	if (jit != cluster_use.end()) {
+		return jit->second.joblist();
+	}
+	return empty;
+}
+#endif
+
 #ifdef ALLOW_ON_THE_FLY_AGGREGATION
 
 class aggregate_jobs_args {
