@@ -17,6 +17,9 @@
  *
  ***************************************************************/
 
+#ifndef __DEDICATED_SCHEDULER_H_
+#define __DEDICATED_SCHEDULER_H_
+
 #include <string>
 #include <map>
 
@@ -209,7 +212,7 @@ class DedicatedScheduler : public Service {
 		// Used for claiming/releasing startds we control
 	bool releaseClaim( match_rec* m_rec );
 	bool deactivateClaim( match_rec* m_rec );
-	void sendAlives( void );
+	void checkClaimLeases( void );
 
 		// Reaper for the MPI shadow
 	int reaper( int pid, int status );
@@ -377,8 +380,6 @@ class DedicatedScheduler : public Service {
 		*/
 	bool isPossibleToSatisfy( CAList* jobs, int max_hosts );
 
-	void holdAllDedicatedJobs( void );
-
 	bool satisfyJobWithGroups(CAList *jobs, int cluster, int nprocs);
 
 		// // // // // // 
@@ -431,21 +432,21 @@ class DedicatedScheduler : public Service {
 		// All resources that are busy (and claimed)
 	ResList*		busy_resources;
 
-        // hashed on cluster, all our allocations
-    HashTable <int, AllocationNode*>* allocations;
+		// keyed on cluster, all our allocations
+	std::map <int, AllocationNode*> allocations;
 
 		// List of resources to preempt
 	CAList *pending_preemptions;
 
 		// hashed on resource name, each claim we have
-	HashTable <std::string, match_rec*>* all_matches;
+	std::map<std::string, match_rec*> all_matches;
 
 		// hashed on ClaimId, each claim we have.  only store
 		// pointers in here into the real match records we store in
 		// all_matches.  This is needed for some functions that only
 		// know the ClaimId (like DelMrec(), since vacate_service()
 		// is only given a ClaimId to identify the lost claim).
-	HashTable <std::string, match_rec*>* all_matches_by_id;
+	std::map<std::string, match_rec*> all_matches_by_id;
 
 		// Queue for resource requests we need to negotiate for. 
 	std::list<PROC_ID> resource_requests;
@@ -503,3 +504,4 @@ void displayRequest( ClassAd* ad, char* str, int debug_level );
 // do with the mrec being allocated to a certain MPI job.
 void deallocMatchRec( match_rec* mrec );
 
+#endif
