@@ -28,10 +28,13 @@
 #include "condor_ver_info.h"
 #include "condor_attributes.h"
 #include "exit.h"
-#include "HashTable.h"
 #include "condor_uid.h"
 #include "condor_email.h"
 #include "shared_port_endpoint.h"
+#include "scheduler.h"
+#include "qmgmt.h"
+
+extern char *Name;
 
 // Initialize static data members
 const int GridUniverseLogic::job_added_delay = 3;
@@ -50,7 +53,7 @@ GridUniverseLogic::GridUniverseLogic()
 	ASSERT( gman_pid_table == nullptr );
 
 	// Make our hashtable
-	gman_pid_table = new GmanPidTable_t(hashFunction);
+	gman_pid_table = new GmanPidTable_t;
 
 	// Register a reaper for this grid managers
 	rid = daemonCore->Register_Reaper("GManager",
@@ -65,9 +68,7 @@ GridUniverseLogic::GridUniverseLogic()
 GridUniverseLogic::~GridUniverseLogic()
 {
 	if ( gman_pid_table) {
-		gman_node_t * node = nullptr;
-		gman_pid_table->startIterations();
-		while (gman_pid_table->iterate( node ) == 1 ) {
+		for (const auto &[name, node]: *gman_pid_table) {
 			if (daemonCore) {
 				if ( node->add_timer_id >= 0 ) {
 					daemonCore->Cancel_Timer(node->add_timer_id);
@@ -89,8 +90,8 @@ GridUniverseLogic::~GridUniverseLogic()
 }
 
 void 
-GridUniverseLogic::JobCountUpdate(const char* user, const char* osname,
-	   	const char* attr_value, const char* attr_name, int cluster, int proc, 
+GridUniverseLogic::JobCountUpdate(const GridUserIdentity& userident,
+		const char* attr_name,
 		int num_globus_jobs, int num_globus_unmanaged_jobs)
 {
 	// Quick sanity checks - this should never be...
@@ -102,7 +103,7 @@ GridUniverseLogic::JobCountUpdate(const char* user, const char* osname,
 	// does not know they are in the queue. so tell it some jobs
 	// were added.
 	if ( num_globus_unmanaged_jobs > 0 ) {
-		JobAdded(user, osname, attr_value, attr_name, cluster, proc);
+		JobAdded(userident, attr_name);
 		return;
 	}
 
@@ -110,7 +111,7 @@ GridUniverseLogic::JobCountUpdate(const char* user, const char* osname,
 	// are any globus jobs at all.  if there are, make certain that there
 	// is a grid manager watching over the jobs and start one if there isn't.
 	if ( num_globus_jobs > 0 ) {
-		StartOrFindGManager(user, osname, attr_value, attr_name, cluster, proc);
+		StartOrFindGManager(userident, attr_name);
 		return;
 	}
 
@@ -120,12 +121,12 @@ GridUniverseLogic::JobCountUpdate(const char* user, const char* osname,
 
 
 void 
-GridUniverseLogic::JobAdded(const char* user, const char* osname,
-	   	const char* attr_value, const char* attr_name, int cluster, int proc)
+GridUniverseLogic::JobAdded(const GridUserIdentity& userident,
+		const char* attr_name)
 {
 	gman_node_t* node = nullptr;
 
-	node = StartOrFindGManager(user, osname, attr_value, attr_name, cluster, proc);
+	node = StartOrFindGManager(userident, attr_name);
 
 	if (!node) {
 		// if we cannot find nor start a gridmanager, there's
@@ -146,12 +147,12 @@ GridUniverseLogic::JobAdded(const char* user, const char* osname,
 }
 
 void 
-GridUniverseLogic::JobRemoved(const char* user, const char* osname,
-	   	const char* attr_value, const char* attr_name,int cluster, int proc)
+GridUniverseLogic::JobRemoved(const GridUserIdentity& userident,
+		const char* attr_name)
 {
 	gman_node_t* node = nullptr;
 
-	node = StartOrFindGManager(user, osname, attr_value, attr_name, cluster, proc);
+	node = StartOrFindGManager(userident, attr_name);
 
 	if (!node) {
 		// if we cannot find nor start a gridmanager, there's
@@ -212,9 +213,7 @@ GridUniverseLogic::signal_all(int sig)
 	// Iterate through our entire table and send the desired sig
 
 	if (gman_pid_table) {
-		gman_node_t* tmpnode = nullptr;
-		gman_pid_table->startIterations();
-		while ( gman_pid_table->iterate(tmpnode) ) {
+		for (const auto &[name, tmpnode]: *gman_pid_table) {
 			if (tmpnode->pid) {
 				daemonCore->Send_Signal(tmpnode->pid,sig);
 			}
@@ -248,11 +247,10 @@ GridUniverseLogic::GManagerReaper(int pid, int exit_status)
 	// are not that many of them.
 
 	if (gman_pid_table) {
-		gman_node_t* tmpnode = nullptr;
-		gman_pid_table->startIterations();
-		while ( gman_pid_table->iterate(user_key, tmpnode) ) {
+		for (const auto &[key, tmpnode]: *gman_pid_table) {
 			if (tmpnode->pid == pid ) {
 				// found it!
+				user_key = key;
 				gman_node = tmpnode;
 				break;
 			}
@@ -270,12 +268,12 @@ GridUniverseLogic::GManagerReaper(int pid, int exit_status)
 		formatstr( exit_reason, "due to %s",
 							 daemonCore->GetExceptionString( exit_status ) );
 	}
-	dprintf(D_ALWAYS, "condor_gridmanager (PID %d, user %s) exited %s.\n",
+	dprintf(D_STATUS, "condor_gridmanager (PID %d, user %s) exited %s.\n",
 			pid, user_safe.c_str(), exit_reason.c_str() );
 	if(WIFEXITED(exit_status) && WEXITSTATUS(exit_status) == DPRINTF_ERROR) {
 		const char *condorUserName = get_condor_username();
 
-		dprintf(D_ALWAYS, 
+		dprintf(D_ERROR, 
 			"The gridmanager had a problem writing its log. "
 			"Check the permissions of the file specified by GRIDMANAGER_LOG; "
 			"it needs to be writable by Condor.\n");
@@ -300,7 +298,7 @@ GridUniverseLogic::GManagerReaper(int pid, int exit_status)
 				email_close(email);
 			} else {
 					// Error sending an email message
-				dprintf(D_ALWAYS,"ERROR: Cannot send email to the admin\n");
+				dprintf(D_ERROR,"ERROR: Cannot send email to the admin\n");
 			}
 		}	
 	}	// end if(WIFEXITED(exit_status) && WEXITSTATUS(exit_status) == DPRINTF_ERROR)
@@ -318,14 +316,13 @@ GridUniverseLogic::GManagerReaper(int pid, int exit_status)
 		daemonCore->Cancel_Timer(gman_node->remove_timer_id);
 	}
 	// Remove node from our hash table
-	gman_pid_table->remove(user_key);
+	gman_pid_table->erase(user_key);
 	// Remove any scratch directory used by this gridmanager
 	std::string scratchdirbuf;
 	const char *scratchdir = scratchFilePath(gman_node, scratchdirbuf);
 	ASSERT(scratchdir);
-	std::string tmp;
 	if ( IsDirectory(scratchdir) && 
-		 init_user_ids(name_of_user(gman_node->user, tmp), domain_of_user(gman_node->user, "")) )
+		 init_user_ids(gman_node->ownerinfo) )
 	{
 		priv_state saved_priv = set_user_priv();
 			// Must put this in braces so the Directory object
@@ -354,17 +351,14 @@ GridUniverseLogic::GManagerReaper(int pid, int exit_status)
 
 
 GridUniverseLogic::gman_node_t *
-GridUniverseLogic::lookupGmanByOwner(const char* user, const char* attr_value,
-					int cluster, int proc)
+GridUniverseLogic::lookupGmanByOwner(const char* user, const char* attr_value)
 {
 	gman_node_t* result = nullptr;
 	std::string user_key(user);
-	if(attr_value){
+	if(attr_value && *attr_value){
+		user_key += '#';
 		user_key += attr_value;
 	}
-	if (cluster) {
-		formatstr_cat( user_key, "-%d.%d", cluster, proc );
-	}
 
 	if (!gman_pid_table) {
 		// destructor has already been called; we are probably
@@ -372,23 +366,19 @@ GridUniverseLogic::lookupGmanByOwner(const char* user, const char* attr_value,
 		return nullptr;
 	}
 
-	gman_pid_table->lookup(user_key,result);
+	if (gman_pid_table->contains(user_key)) {
+		result = (*gman_pid_table)[user_key];
+	}
 
 	return result;
 }
 
 int
-GridUniverseLogic::FindGManagerPid(const char* user,
-					const char* attr_value,	
-					int cluster, int proc)
+GridUniverseLogic::FindGManagerPid(const GridUserIdentity& userident)
 {
 	gman_node_t* gman_node = nullptr;
 
-	if ( attr_value && strlen(attr_value)==0 ) {
-		attr_value = nullptr;
-	}
-
-	if ( (gman_node=lookupGmanByOwner(user, attr_value, cluster, proc)) ) {
+	if ( (gman_node=lookupGmanByOwner(userident.m_ownerinfo->Name(), userident.m_auxid.c_str())) ) {
 		return gman_node->pid;
 	}
 	else {
@@ -397,25 +387,27 @@ GridUniverseLogic::FindGManagerPid(const char* user,
 }
 
 GridUniverseLogic::gman_node_t *
-GridUniverseLogic::StartOrFindGManager(const char* user, const char* osname,
-	   	const char* attr_value, const char* attr_name, int cluster, int proc)
+GridUniverseLogic::StartOrFindGManager(const GridUserIdentity& userident, const char* attr_name)
 {
+	const char* user = userident.m_ownerinfo->Name();
+	const char* osname = userident.m_ownerinfo->OsUser();
+	const char* attr_value = userident.m_auxid.c_str();
 	gman_node_t* gman_node = nullptr;
 	int pid = 0;
 
-		// If attr_value is an empty string, convert to NULL since code
+		// If attr_name is an empty string, convert to NULL since code
 		// after this point expects that.
-	if ( attr_value && strlen(attr_value)==0 ) {
+	if ( attr_name && strlen(attr_name)==0 ) {
 		attr_value = nullptr;
 		attr_name = nullptr;
 	}
 
 	if ( !user || !osname ) {
-		dprintf(D_ALWAYS,"ERROR - missing user/osname field\n");
+		dprintf(D_ERROR,"ERROR - missing user/osname field\n");
 		return nullptr;
 	}
 
-	if ( (gman_node=lookupGmanByOwner(user, attr_value, cluster, proc)) ) {
+	if ( (gman_node=lookupGmanByOwner(user, attr_value)) ) {
 		// found it
 		return gman_node;
 	}
@@ -432,18 +424,17 @@ GridUniverseLogic::StartOrFindGManager(const char* user, const char* osname,
 
 #ifndef WIN32
 	if (strcasecmp(user, "root") == 0 ) {
-		dprintf(D_ALWAYS, "Tried to start condor_gmanager as root.\n");
+		dprintf(D_ERROR, "Tried to start condor_gmanager as root.\n");
 		return nullptr;
 	}
 #endif
 
-	dprintf( D_FULLDEBUG, "Starting condor_gmanager for user %s (%d.%d)\n",
-			user, cluster, proc);
+	dprintf( D_FULLDEBUG, "Starting condor_gmanager for user %s\n", user);
 
 	char *gman_binary = nullptr;
 	gman_binary = param("GRIDMANAGER");
 	if ( !gman_binary ) {
-		dprintf(D_ALWAYS,"ERROR - GRIDMANAGER not defined in config file\n");
+		dprintf(D_ERROR, "ERROR - GRIDMANAGER not defined in config file\n");
 		return nullptr;
 	}
 
@@ -453,21 +444,25 @@ GridUniverseLogic::StartOrFindGManager(const char* user, const char* osname,
 	args.AppendArg("condor_gridmanager");
 	args.AppendArg("-f");
 
-	if ( attr_value && *attr_value && param_boolean( "GRIDMANAGER_LOG_APPEND_SELECTION_EXPR", false ) ) {
+	std::string log_suffix = osname;
+
+	if ( attr_name && param_boolean( "GRIDMANAGER_LOG_APPEND_SELECTION_EXPR", false ) ) {
 		const std::string filename_filter = "abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789.-_";
-		std::string log_suffix = attr_value;
+		log_suffix += '.';
+		log_suffix += attr_value;
 		size_t pos = 0;
 		while( (pos = log_suffix.find_first_not_of( filename_filter, pos )) != std::string::npos ) {
 			log_suffix[pos] = '_';
 		}
-		args.AppendArg("-a");
-		args.AppendArg(log_suffix.c_str());
 	}
 
+	args.AppendArg("-a");
+	args.AppendArg(log_suffix);
+
 	char *gman_args = param("GRIDMANAGER_ARGS");
 
 	if(!args.AppendArgsV1RawOrV2Quoted(gman_args,error_msg)) {
-		dprintf( D_ALWAYS, "ERROR: failed to parse gridmanager args: %s\n",
+		dprintf( D_ERROR, "ERROR: failed to parse gridmanager args: %s\n",
 				 error_msg.c_str());
 		free(gman_binary);
 		free(gman_args);
@@ -481,7 +476,7 @@ GridUniverseLogic::StartOrFindGManager(const char* user, const char* osname,
 						   ATTR_USER, user,
 						   ATTR_JOB_UNIVERSE, CONDOR_UNIVERSE_GRID);
 	} else {
-		formatstr(constraint, R"((%s=?="%s"&&%s=?="%s"&&%s==%d))",
+		formatstr(constraint, R"((%s=?="%s"&&(%s?:"")=?="%s"&&%s==%d))",
 						   ATTR_USER, user,
 						   attr_name, attr_value,
 						   ATTR_JOB_UNIVERSE, CONDOR_UNIVERSE_GRID);
@@ -492,16 +487,18 @@ GridUniverseLogic::StartOrFindGManager(const char* user, const char* osname,
 	args.AppendArg("-C");
 	args.AppendArg(constraint);
 
-	dprintf(D_ALWAYS, "Launching gridmanager with -o %s -u %s\n", osname, user);
+	dprintf(D_STATUS, "Launching gridmanager with -o %s -u %s\n", osname, user);
 
 	args.AppendArg("-o");
 	args.AppendArg(osname);
 	args.AppendArg("-u");
 	args.AppendArg(user);
+	args.AppendArg("-n");
+	args.AppendArg(Name);
 
 	std::string tmp;
-	if (!init_user_ids(name_of_user(osname, tmp), domain_of_user(osname, ""))) {
-		dprintf(D_ALWAYS,"ERROR - init_user_ids(%s) failed in GRIDMANAGER\n", osname);
+	if (!init_user_ids(userident.m_ownerinfo)) {
+		dprintf(D_ERROR,"ERROR - init_user_ids(%s) failed in GRIDMANAGER\n", osname);
 		free(gman_binary);
 		return nullptr;
 	}
@@ -544,7 +541,7 @@ GridUniverseLogic::StartOrFindGManager(const char* user, const char* osname,
 			}
 				// if we made it here, blow away this subdir
 			if ( tmp.Remove_Current_File() ) {
-				dprintf(D_ALWAYS,"Removed old scratch dir %s\n",
+				dprintf(D_FULLDEBUG, "Removed old scratch dir %s\n",
 				tmp.GetFullPath());
 			}
 		}	// end of while for cleanup of old scratch dirs
@@ -564,7 +561,7 @@ GridUniverseLogic::StartOrFindGManager(const char* user, const char* osname,
 	priv_state saved_priv = set_user_priv();
 	if ( (mkdir(finalpath,0700)) < 0 ) {
 		// mkdir failed.  
-		dprintf(D_ALWAYS,"ERROR - mkdir(%s,0700) failed in GRIDMANAGER, errno=%d (%s)\n",
+		dprintf(D_ERROR, "ERROR - mkdir(%s,0700) failed in GRIDMANAGER, errno=%d (%s)\n",
 				finalpath, errno, strerror(errno));
 		failed = true;
 	}
@@ -600,34 +597,33 @@ GridUniverseLogic::StartOrFindGManager(const char* user, const char* osname,
 	free(gman_binary);
 
 	if ( pid <= 0 ) {
-		dprintf ( D_ALWAYS, "StartOrFindGManager: Create_Process problems!\n" );
+		dprintf ( D_ERROR, "StartOrFindGManager: Create_Process problems!\n" );
 		if (gman_node) delete gman_node;
 		return nullptr;
 	}
 
 	// If we made it here, we happily started up a new gridmanager process
 
-	dprintf( D_ALWAYS, "Started condor_gmanager for user %s pid=%d\n",
+	dprintf( D_STATUS, "Started condor_gmanager for user %s pid=%d\n",
 			user,pid);
 
 	// Make a new gman_node entry for our hashtable & insert it
 	if ( !gman_node ) {
 		gman_node = new gman_node_t;
 	}
+	gman_node->ownerinfo = userident.m_ownerinfo;
 	gman_node->pid = pid;
 	gman_node->user[0] = '\0';
 	if ( user ) {
 		strcpy(gman_node->user, user);
 	}
 	std::string user_key(user);
-	if(attr_value){
+	if(attr_value && *attr_value){
+		user_key += '#';
 		user_key += attr_value;
 	}
-	if (cluster) {
-		formatstr_cat( user_key, "-%d.%d", cluster, proc );
-	}
 
-	ASSERT( gman_pid_table->insert(user_key,gman_node) == 0 );
+	gman_pid_table->emplace(user_key,gman_node);
 
 	// start timer to signal gridmanager if we haven't already
 	if ( gman_node->add_timer_id == -1 ) {  // == -1 means no timer set
