@@ -21,6 +21,12 @@
 #ifndef _CONDOR_GRID_UNIVERSE
 #define _CONDOR_GRID_UNIVERSE
 
+#include <map>
+#include <string>
+
+struct GridUserIdentity;
+class JobQueueUserRec;
+
 class GridUniverseLogic : public Service
 {
 	public:
@@ -28,21 +34,17 @@ class GridUniverseLogic : public Service
 		GridUniverseLogic();
 		~GridUniverseLogic();
 
-		static void JobCountUpdate(const char* user, const char* osname,
-				const char* attr_value, const char* attr_name, int cluster, 
-				int proc, int num_globus_jobs, int num_globus_unmanaged_jobs);
+		static void JobCountUpdate(const GridUserIdentity& userident,
+				const char* attr_name,
+				int num_globus_jobs, int num_globus_unmanaged_jobs);
 
-		static void JobRemoved(const char* user, const char* osname,
-			   	const char* attr_value, const char* attr_name, int cluster, 
-				int proc);
+		static void JobRemoved(const GridUserIdentity& userident,
+				const char* attr_name);
 
-		static void JobAdded(const char* user, const char* osname,
-			   	const char* attr_value, const char* attr_name, int cluster, 
-				int proc);
+		static void JobAdded(const GridUserIdentity& userident,
+				const char* attr_name);
 
-		static int FindGManagerPid(const char* user,
-							const char* attr_value,
-							int cluster, int proc);
+		static int FindGManagerPid(const GridUserIdentity& userident);
 		static void reconfig() { signal_all(SIGHUP); }
 		static void shutdown_graceful() { signal_all(SIGTERM); }
 		static void shutdown_fast() { signal_all(SIGQUIT); }
@@ -54,21 +56,19 @@ class GridUniverseLogic : public Service
 		static const int job_removed_delay;
 
 		struct gman_node_t {
-			int pid;
-			int add_timer_id;
-			int remove_timer_id;
-			char user[200];
-			gman_node_t() : pid(0),add_timer_id(-1),remove_timer_id(-1) {user[0] = '\0';};
+			const JobQueueUserRec* ownerinfo{nullptr};
+			int pid{0};
+			int add_timer_id{-1};
+			int remove_timer_id{-1};
+			char user[200]{""};
 		};
 
-		static gman_node_t* lookupGmanByOwner(const char* user,
-							const char* attr_name, int cluster, int proc);
+		static gman_node_t* lookupGmanByOwner(const char* user, const char* attr_value);
 
 		static int GManagerReaper(int pid, int exit_status);
 
-		static gman_node_t* StartOrFindGManager(const char* user,
-				const char* osname, const char* attr_value,  const char* attr_name,
-				int cluster, int proc);
+		static gman_node_t* StartOrFindGManager(const GridUserIdentity& userident,
+				const char* attr_name);
 
 		// SendAddSignal and SendRemoveSignal are DC Timer Event handlers
 		static void SendAddSignal(int tid);
@@ -78,7 +78,7 @@ class GridUniverseLogic : public Service
 		// write the scratch dirctory into the path argument and return path.Value()
 		static const char *scratchFilePath(gman_node_t *, std::string & path);
 
-		typedef HashTable<std::string,gman_node_t *> GmanPidTable_t;
+		typedef std::map<std::string,gman_node_t *> GmanPidTable_t;
 		static GmanPidTable_t * gman_pid_table;
 
 		static int rid;
