@@ -26,36 +26,64 @@
 #ifndef _PRIO_REC_H_
 #define _PRIO_REC_H_
 
-const 	int		INITIAL_MAX_PRIO_REC = 2048;
+#include <deque>
 
 /* this record contains all the parameters required for
  * assigning priorities to all jobs */
 
 class prio_rec {
 public:
-    PROC_ID     id;
-    int         pre_job_prio1;
-    int         pre_job_prio2;
-    int         post_job_prio1;
-    int         post_job_prio2;
-    int         job_prio;
-	int			auto_cluster_id;
+	PROC_ID     id{0,0};
+	int         pre_job_prio1{0};
+	int         pre_job_prio2{0};
+	int         post_job_prio1{0};
+	int         post_job_prio2{0};
+	int         job_prio{0};
+	int         auto_cluster_id{0};
 	std::string submitter;
-	bool		not_runnable;
-	bool		matched;
+	bool        not_runnable{false};
+	bool        matched{false};
+};
 
-	prio_rec() {
-		id.cluster = 0;
-		id.proc = 0;
-		job_prio = 0;
-		auto_cluster_id = 0;
-		pre_job_prio1 = 0;
-		pre_job_prio2 = 0;
-		post_job_prio1 = 0;
-		post_job_prio2 = 0;
-		not_runnable = false;
-		matched = false;
+// prio rec lower bound by submitter
+// This allows us to binary search the PrioRecArray by submitter
+// Note this must match the primary key comparison in struct prio_compar (in qmgmt.cpp)
+struct prio_rec_submitter_lb {
+	bool operator()(const prio_rec& a, const std::string &user) const {
+		if (a.submitter.length() < user.length()) {
+			return false;
+		}
+
+		if (a.submitter.length() > user.length()) {
+			return true;
+		}
+
+		if (a.submitter > user) {
+			return true;
+		}
+
+		return false;
 	}
 };
 
+// The corresponding upper bound function, the negation of the above
+struct prio_rec_submitter_ub {
+	bool operator()(const std::string &user, const prio_rec &a) const {
+		if (a.submitter.length() < user.length()) {
+			return true;
+		}
+
+		if (a.submitter.length() > user.length()) {
+			return false;
+		}
+
+		if (a.submitter < user) {
+			return true;
+		}
+
+		return false;
+	}
+};
+
+
 #endif
