@@ -27,6 +27,9 @@
 #include "condor_sockaddr.h"
 #include "classad_log.h"
 #include "live_job_counters.h"
+#include "condor_attributes.h"
+#include <generic_stats.h>
+#include <vector>
 
 // the pedantic idiots at gcc generate this warning whenever you use offsetof on a struct or class that has a constructor....
 GCC_DIAG_OFF(invalid-offsetof)
@@ -38,8 +41,6 @@ GCC_DIAG_OFF(invalid-offsetof)
 #error This header must be included before condor_qmgr.h for code internal to the SCHEDD, and not at all for external code
 #endif
 
-#define JOB_QUEUE_PAYLOAD_IS_BASE 1
-#define USE_JOB_QUEUE_USERREC 1 // replace ephemeral OwnerInfo struct with a persistent JobQueueUserRec struct
 
 // until we can remove targettype from the classad log entirely
 // use the legacy "Machine" value as the target adtype
@@ -62,13 +63,9 @@ class QmgmtPeer {
 		void unset();
 
 		bool initAuthOwner(bool read_only);
-	#ifdef USE_JOB_QUEUE_USERREC
 		bool setEffectiveOwner(const class JobQueueUserRec * urec, bool not_super_effective);
 		// used during submit when a UserRec is created as a side effect of submit
 		void attachNewUserRec(const class JobQueueUserRec * urec) { jquser = urec; }
-	#else
-		bool setEffectiveOwner(char const *o);
-	#endif
 		bool setAllowProtectedAttrChanges(bool val);
 		bool getAllowProtectedAttrChanges() const { return allow_protected_attr_changes_by_superuser; }
 		bool getReadOnly() const { return readonly; }
@@ -84,30 +81,25 @@ class QmgmtPeer {
 		const char* getRealOwner() const { return sock ? sock->getOwner() : owner; }
 		const char* getRealUser() const { return sock ? sock->getFullyQualifiedUser() : fquser; }
 		const char* getUser() const  { return fquser ? fquser : (sock ? sock->getFullyQualifiedUser() : nullptr); }
-	#ifdef USE_JOB_QUEUE_USERREC
 		const class JobQueueUserRec * UserRec() const { return jquser; } // EffectiveUser as a JobQueueUserRec
-	#endif
 		int isAuthenticated() const;
 		bool isAuthorizationInBoundingSet(const char *authz) const {return sock->isAuthorizationInBoundingSet(authz);}
 
-	#ifdef USE_JOB_QUEUE_USERREC
 		friend inline const char * EffectiveUserName(QmgmtPeer * qsock);
 		friend inline const class JobQueueUserRec * EffectiveUserRec(QmgmtPeer * qsock);
-	#else
-		friend inline const char * EffectiveUser(QmgmtPeer * qsock);
-	#endif
+
+		CondorError& getErrStack() { return errstack; }
 
 	protected:
 
 		char *owner;  
 		char *fquser;  // owner@domain
-	#ifdef USE_JOB_QUEUE_USERREC
 		const class JobQueueUserRec * jquser = nullptr; // same as 
-	#endif
 		char *myendpoint; 
 		condor_sockaddr addr;
 		ReliSock *sock; 
 
+		CondorError errstack;
 		Transaction *transaction;
 		int next_proc_num{}, active_cluster_num{};
 		time_t xact_start_time{};
@@ -115,10 +107,8 @@ class QmgmtPeer {
 		bool readonly{false};
 		bool write_ok = false;
 		bool allow_protected_attr_changes_by_superuser{true};
-	#ifdef USE_JOB_QUEUE_USERREC
 		bool real_auth_is_super = false;	// real auth identifier is a super user
 		bool not_super_effective = false;	// Ignore EffectiveOwner Superuser status
-	#endif
 
 
 	private:
@@ -130,7 +120,7 @@ class QmgmtPeer {
 
 extern bool user_is_the_new_owner; // set in schedd.cpp at startup
 extern bool ignore_domain_mismatch_when_setting_owner;
-#ifdef USE_JOB_QUEUE_USERREC
+
 inline const char * EffectiveUserName(QmgmtPeer * peer) {
 	if (peer) {
 		// with JobQueueUserRec we always want to know the full username
@@ -140,26 +130,6 @@ inline const char * EffectiveUserName(QmgmtPeer * peer) {
 	}
 	return "";
 }
-#else
-inline const char * EffectiveUser(QmgmtPeer * peer) {
-	if (peer) {
-	#ifdef USE_JOB_QUEUE_USERREC
-		// with JobQueueUserRec we always want to know the full username
-		// so regardless of the setting for user_is_the_new_owner, we want
-		// return a fully qualified user here
-		return peer->getUser();
-	#else
-		if (user_is_the_new_owner) {
-			if (peer->sock) return peer->sock->getFullyQualifiedUser();
-			if (peer->fquser && peer->fquser[0]) return peer->fquser;
-		} else {
-			return peer->getOwner();
-		}
-	#endif
-	}
-	return "";
-}
-#endif
 
 #define JQJ_CACHE_DIRTY_JOBOBJ        0x00001 // set when an attribute cached in the JobQueueJob that doesn't have it's own flag has changed
 #define JQJ_CACHE_DIRTY_SUBMITTERDATA 0x00002 // set when an attribute that affects the submitter name is changed
@@ -194,17 +164,22 @@ private:
 	qelm *prv;
 };
 
-#ifdef USE_JOB_QUEUE_USERREC
 const int USERRECID_qkey1 = 0;
 inline int USERRECID_to_qkey2(unsigned int userrec_id) {
 	if ((int)userrec_id <= 0) dprintf(D_ALWAYS | D_BACKTRACE, "USERRECID_to_qkey2 called with id=%d", userrec_id);
 	ASSERT(userrec_id > 0);
 	return (int)(userrec_id);
 }
-#endif
 
 const int JOBSETID_qkey2 = -100;
 const int CLUSTERID_qkey2 = -1;
+const int CLUSTERPRIVATE_qkey2 = -2;
+
+// The magic procid value for OCUs.  OCUs are not jobs, but almost
+// have the job nature, as they are sent to the negotiator.
+// OCU_qkey2 identifies an OCU record.
+const int OCU_qkey2 = -99;
+
 // jobset ids are id.-100
 inline int JOBSETID_to_qkey1(unsigned int jobset_id) {
     if (jobset_id <= 0) dprintf(D_ALWAYS | D_BACKTRACE, "JOBSETID_to_qkey1 called with id=%ud", jobset_id);
@@ -236,7 +211,9 @@ public:
 		entry_type_unknown = 0,
 		entry_type_header,
 		entry_type_userrec,
+		entry_type_project,
 		entry_type_jobset,
+		entry_type_cluster_private,
 		entry_type_cluster,
 		entry_type_job,
 	};
@@ -245,31 +222,114 @@ public:
 			if (key.proc >= 0) { return entry_type_job; }
 			if (key.proc == JOBSETID_qkey2) { return entry_type_jobset; }
 			if (key.proc == CLUSTERID_qkey2) { return entry_type_cluster; }
+			if (key.proc == CLUSTERPRIVATE_qkey2) { return entry_type_cluster_private; }
 		} else if ( ! key.cluster) {
 			if ( ! key.proc) { return entry_type_header; }
-			else if (key.proc > 0) { return entry_type_userrec; }
+			else if (key.proc > 0) { return entry_type_userrec; } // note: could be project rec can't tell just from the jid.
 		}
 		return entry_type_unknown;
 	}
+	static bool IsJobId(const JOB_ID_KEY &key) { return TypeOfJid(key) == entry_type_job; }
+	static bool IsClusterId(const JOB_ID_KEY &key) { return TypeOfJid(key) == entry_type_cluster; }
+	static bool IsJobSetId(const JOB_ID_KEY &key) { return TypeOfJid(key) == entry_type_jobset; }
+	static bool IsUserRecId(const JOB_ID_KEY &key) { return TypeOfJid(key) == entry_type_userrec; }
+	static bool IsClusterPrivateId(const JOB_ID_KEY &key) { return TypeOfJid(key) == entry_type_cluster_private; }
+
 	void CheckJidAndType(const JOB_ID_KEY &key); // called when reloading the job queue
-#ifdef JOB_QUEUE_PAYLOAD_IS_BASE
 	bool IsType(char _type) const { return entry_type == _type; }
-#else
-	bool IsType(char _type) { if (!entry_type) this->PopulateFromAd(); return entry_type == _type; }
-#endif
 	bool IsJob() const { return IsType(entry_type_job); }
 	bool IsHeader() const { return IsType(entry_type_header); }
 	bool IsUserRec() const { return IsType(entry_type_userrec); }
+	bool IsProject() const { return IsType(entry_type_project); }
 	bool IsJobSet() const { return IsType(entry_type_jobset); }
 	bool IsCluster() const { return IsType(entry_type_cluster); }
+	bool IsClusterPrivate() const { return IsType(entry_type_cluster_private); }
+
+	// write a value from the current record into the job queue log
+	// note that this evaluates expressions, so it is only suitable for working with literals
+	bool UpdateSecureAttribute(const char * attr);
+};
+
+int get_next_cluster_num();
+
+class match_rec;
+// Owned Capacity Unit
+class OCU {
+	public:
+		OCU() = default;
+		OCU(const ClassAd &ad, int ocu_id = -1) : ad(ad), mrec(nullptr), ocu_id(ocu_id) {}
+		ClassAd ad; // the request ad
+		match_rec *mrec{nullptr}; // null if no current match
+		int ocu_id;
+		char state{'U'}; // U=Unclaimed I=Idle, R=Running
+};
+
+// class used to hold stats counters for User and Project records
+//
+class schedd_daily_usage_stats {
+public:
+	stats_entry_daily<long long> slot_time;
+	stats_entry_daily<double> weighted_idle;
+	stats_entry_daily<int> launched;
+	stats_entry_daily<int> not_started;
+	stats_entry_daily<int> requeued;
+	stats_entry_daily<int> held;
+
+	static const int hour_bucket_sec = 4*3600; // 4 hour buckets
+	schedd_daily_usage_stats(int hour_buckets=(24*3600)/hour_bucket_sec, int days=10)
+		: slot_time(hour_buckets,days)
+		, weighted_idle(hour_buckets,days)
+		, launched(hour_buckets,days)
+		, not_started(hour_buckets,days)
+		, requeued(hour_buckets,days)
+		, held(hour_buckets,days)
+	{}
+
+private:
+	static const char * ajoin(std::string & buf, const char * tag, const char * attr) {
+		if (tag) { buf = tag; buf += attr; }
+		else { buf = attr; }
+		return buf.c_str();
+	}
+
+public:
+	void publish(ClassAd & ad, const char * tag) const {
+		std::string buf;
+		int pub_flags = stats_entry_daily<int>::PubDefault | IF_NONZERO;
+		slot_time.Publish(ad, ajoin(buf,tag,"SlotTime"), pub_flags);
+		weighted_idle.Publish(ad, ajoin(buf,tag,"WeightedIdle"), pub_flags);
+		launched.Publish(ad, ajoin(buf,tag,"Launched"), pub_flags);
+		not_started.Publish(ad, ajoin(buf,tag,"FailedToStart"), pub_flags);
+		requeued.Publish(ad, ajoin(buf,tag,"Requeued"), pub_flags);
+		held.Publish(ad, ajoin(buf,tag,"Held"), pub_flags);
+	}
+
+	// add to Hourly/Daily stats (called from count jobs)
+	void accum(int status, long long sec, int idle_weight) {
+		if (status == IDLE) {
+			weighted_idle += (double)(idle_weight * sec) / (double)hour_bucket_sec;
+		} else {
+			slot_time += sec;
+		}
+	}
+
+	void AdvanceBy(int cBuckets, int cDays) {
+		slot_time.AdvanceBy(cBuckets,cDays);
+		weighted_idle.AdvanceBy(cBuckets,cDays);
+		launched.AdvanceBy(cBuckets,cDays);
+		not_started.AdvanceBy(cBuckets,cDays);
+		requeued.AdvanceBy(cBuckets,cDays);
+		held.AdvanceBy(cBuckets,cDays);
+	}
 };
 
-#ifdef USE_JOB_QUEUE_USERREC
 
 // flag values for JobQueueUserRec
 #define JQU_F_DIRTY    0x01   // PopulateFromAd needed 
 #define JQU_F_PENDING  0x80   // JobQueueUserRec is in the pendingOwners collection
 
+#define GENERIC_AP_USER_PLACEHOLDER "<tmp_ap_user>"
+
 class JobQueueUserRec : public JobQueueBase {
 public:
 	struct CountJobsCounters {
@@ -285,25 +345,35 @@ public:
 		int SchedulerJobsIdle=0;
 		int SchedulerJobsRunning=0;
 		int SchedulerJobsHeld=0;
+		int OCUJobsRunning = 0;
 		void clear_counters() { memset(this, 0, sizeof(*this)); }
 	};
 
-	//JOB_ID_KEY jid;
+	//JOB_ID_KEY jid    - declared in JobQueueBase
+	//char entry_type   - declared in JobQueueBase
 	unsigned char flags=JQU_F_DIRTY;   // dirty on creation
 protected:
 	bool        enabled=true;
 	bool        local=false;
+	bool        os_user_differs=false;
 	unsigned char super=JQU_F_PENDING; // config stale on creation
 	std::string name;   // the name used in the schedd's map of OwnerInfo records
-	std::string domain; // holds windows NTDomain, this is future use on *nix
+	std::string os_user; // the os account to use for this user
+	// used by the JobQueueProjectRec constructor
+	JobQueueUserRec(int userrec_id, char _etype, const char *_name=nullptr)
+		: JobQueueBase(JOB_ID_KEY(USERRECID_qkey1,userrec_id), (_etype == entry_type_project) ? entry_type_project : entry_type_userrec)
+		, super(false), name(_name?_name:"")
+	{}
 public:
 	CountJobsCounters num; // job counts by OWNER rather than by submitter
 	LiveJobCounters live; // job counts that are always up-to-date with the committed job state
 	time_t LastHitTime=0; // records the last time we incremented num.Hit, use to expire OwnerInfo
 
-	JobQueueUserRec(int userrec_id, const char* _name=nullptr, const char * _domain=nullptr, unsigned char is_super=0)
+	schedd_daily_usage_stats daily_stats;
+
+	JobQueueUserRec(int userrec_id, const char* _name=nullptr, const char * _os_user=nullptr, unsigned char is_super=0)
 		: JobQueueBase(JOB_ID_KEY(USERRECID_qkey1,userrec_id), entry_type_userrec)
-		, super(is_super), name(_name?_name:""), domain(_domain?_domain:"")
+		, super(is_super), name(_name?_name:""), os_user(_os_user?_os_user:"")
 	{}
 	virtual ~JobQueueUserRec() {};
 
@@ -311,7 +381,7 @@ public:
 
 	bool empty() const { return name.empty(); }
 	const char * Name() const { return name.empty() ? "" : name.c_str(); }
-	const char * NTDomain() const { return domain.empty() ? nullptr : domain.c_str(); }
+	const char * OsUser() const { return os_user.empty() ? nullptr : os_user.c_str(); }
 	bool isDirty() const { return (flags & JQU_F_DIRTY) != 0; }
 	void setDirty() { flags |= JQU_F_DIRTY; }
 	bool isPending() const { return (flags & JQU_F_PENDING) != 0; }
@@ -319,6 +389,11 @@ public:
 	void setPending() { flags |= JQU_F_PENDING; }
 	bool IsEnabled() const { return enabled; }
 	bool IsLocalUser() const { return local; }
+	bool OsUserDiffers() const { return os_user_differs; }
+
+	// used to assign a specific OS user to a userrec when the Owner is not a local user
+	// or when you don't want to use the local user account that matches the Owner
+	bool assignOsUser(const std::string & os_user);
 
 	// The super member has 4 possible values
 	// super == 0 is not super
@@ -335,6 +410,58 @@ public:
 	void setStaleConfigSuper() { if (super != 1) super = JQU_F_PENDING; } // intrinsic super cant be stale
 	bool IsInherentlySuper() const { return super == 1; }
 	bool IsConfigSuper() const { return super == 2; }
+
+	std::vector<OCU> ocus; // vector of OCU ClassAds held by this user record
+
+	void syncOCUs() {
+		std::vector<ExprTree *> l;
+		for (auto &ocu: ocus) {
+			l.push_back(new ClassAd(ocu.ad)); // What is the ownership model here?
+		}
+		classad::ExprList *exprList = classad::ExprList::MakeExprList(l);
+		this->Insert("ocus", exprList);
+		UpdateSecureAttribute("ocus");
+	}
+
+	ClassAd addOCU(const ClassAd &ocu_ad) {
+		ocus.emplace_back();
+		ocus.back().ad = ocu_ad;
+		int ocu_id = get_next_cluster_num();
+		ocus.back().ad.Assign(ATTR_OCU_ID, ocu_id);
+		ocus.back().ocu_id = ocu_id;
+
+		// Give the OCU a name if it doesn't have one
+		std::string ocu_name;
+		ocus.back().ad.LookupString(ATTR_OCU_NAME, ocu_name);
+		if (ocu_name.empty()) {
+			formatstr(ocu_name, "OCU-%d", ocu_id);
+		} 
+		ocus.back().ad.Assign(ATTR_OCU_NAME, ocu_name);
+		ocus.back().ad.Assign("OCUClaim", true);
+		ocus.back().ad.Assign(ATTR_OCU_HOLDER, true);
+		syncOCUs();
+
+		ClassAd result;
+		result.Assign(ATTR_OCU_ID, ocu_id);
+		result.Assign(ATTR_RESULT, 0);
+		result.Assign(ATTR_OCU_NAME, ocu_name);
+		return result;
+	}
+
+	ClassAd removeOCU(const ClassAd &ocu); 
+
+	// add a numeric value to a numeric value in the user record, and optionally
+	// write the result to the job queue log
+	template <typename T> bool AccumToRecordAndLog(const char * attr, T addval, bool log_set_attr=true);
+};
+
+// used to store a Project ClassAd in a condor hashtable.
+class JobQueueProjectRec : public JobQueueUserRec {
+public:
+	JobQueueProjectRec(int userrec_id, const char * _name=nullptr)
+		: JobQueueUserRec(userrec_id, entry_type_project, _name)
+	{}
+	virtual ~JobQueueProjectRec() {};
 };
 
 typedef JobQueueUserRec OwnerInfo;
@@ -351,8 +478,9 @@ inline int SetUserAttributeString(JobQueueUserRec & urec, const char * attr_name
 	return SetUserAttributeValue(urec, attr_name, tmp);
 }
 int DeleteUserAttribute(JobQueueUserRec & urec, const char * attr_name);
-struct UpdateUserAttributesInfo { int valid{0}; int invalid{0}; int special{0}; };
-int UpdateUserAttributes(JobQueueKey & key, const ClassAd & cmdAd, bool enabled, struct UpdateUserAttributesInfo& info );
+// these can be used on UserRec or ProjectRec ads
+struct UpdateUserRecAttributesInfo { int valid{0}; int invalid{0}; int special{0}; };
+int UpdateUserRecAttributes(JobQueueKey & key, bool is_project, const ClassAd & cmdAd, bool enabled, struct UpdateUserRecAttributesInfo& info);
 
 // get the Effect User record from the peer
 // returns NULL if no peer or the peer has not yet had an userrec set.
@@ -365,43 +493,62 @@ inline const class JobQueueUserRec * EffectiveUserRec(QmgmtPeer * peer)
 	return nullptr;
 }
 
-#else
-typedef struct OwnerInfo OwnerInfo;
-#endif
+class JobQueueClusterPrivate : public JobQueueBase
+{
+public:
+
+	//JOB_ID_KEY jid    - declared in JobQueueBase
+	//char entry_type   - declared in JobQueueBase
+	unsigned char flags=JQU_F_DIRTY;   // dirty on creation
+public:
+	JobQueueClusterPrivate(const JOB_ID_KEY & key) : JobQueueBase(key, entry_type_cluster_private) {}
+	virtual ~JobQueueClusterPrivate() {};
+	virtual void PopulateFromAd() { flags &= ~JQU_F_DIRTY; }
+
+	bool isDirty() const { return (flags & JQU_F_DIRTY) != 0; }
+	void setDirty() { flags |= JQU_F_DIRTY; }
+};
+
+
+// state of JobQueueJob run variable, used along with the prio-rec array to track which jobs
+// need matches and which have already been given matches.
+// Also used to cache ATTR_JOB_NOOP since that is also checked in the prio-rec
+enum class JobRunnableState : char {
+	Unset       = 0,
+	Runnable    = 1,
+	NotRunnable = 2,
+	Matched     = 3,
+	Cooldown    = 4,
+	DirtyNoop   = 5,  // when ATTR_JOB_NOOP has been set, but not yet checked to see if it is true
+	Noop        = 6,  // when ATTR_JOB_NOOP is true for the job
+};
 
 class JobQueueJob : public JobQueueBase {
 public:
-	//JOB_ID_KEY jid;
+	//JOB_ID_KEY jid (defined in JobQueueBase)
 protected:
-	char universe;      // this is in sync with ATTR_JOB_UNIVERSE
-	char has_noop_attr; // 1 if job has ATTR_JOB_NOOP
-	char status;        // this is in sync with committed job status and used when tracking job counts by state
+	//char entry_type (defined in JobQueueBase)
+	char universe{0};      // this is in sync with ATTR_JOB_UNIVERSE
+	char status{0};        // this is in sync with committed job status and used when tracking job counts by state
 public:
-	int dirty_flags;	// one or more of JQJ_CHACHE_DIRTY_ flags indicating that the job ad differs from the JobQueueJob 
-	int set_id;
-	int autocluster_id;
-	// cached pointer into schedulers's SubmitterDataMap and OwnerInfoMap
+	JobRunnableState run{JobRunnableState::Unset};
+	int dirty_flags{0};	// one or more of JQJ_CHACHE_DIRTY_ flags indicating that the job ad differs from the JobQueueJob 
+	int set_id{0};
+	int autocluster_id{0};
+	// cached pointer into schedulers's SubmitterDataMap and OwnerInfoMap and ProjectInfoMap
 	// it is set by count_jobs() or by scheduler::get_submitter_and_owner()
 	// DO NOT FREE FROM HERE!
-	OwnerInfo * ownerinfo;
-	struct SubmitterData * submitterdata;
+	OwnerInfo * ownerinfo{nullptr};
+	JobQueueProjectRec * project{nullptr};
+	struct SubmitterData * submitterdata{nullptr};
 protected:
-	JobQueueCluster * parent; // job pointer back to the 
+	JobQueueCluster * parent{nullptr}; // job pointer back to the cluster ad
 	qelm qe;
 
 public:
 	JobQueueJob(const JOB_ID_KEY & key)
 		: JobQueueBase(key, (key.proc < 0) ? entry_type_cluster : entry_type_job)
-		//TT , jid(0,0)
-		, universe(0)
-		, has_noop_attr(2) // value of 2 forces IsNoopJob() to populate this field
-		, status(0) // JOB_STATUS_MIN
-		, dirty_flags(0)
-		, set_id(0)
-		, autocluster_id(0)
-		, ownerinfo(NULL)
-		, submitterdata(NULL)
-		, parent(NULL)
+		//base class initializes jid to 0,0
 	{}
 	virtual ~JobQueueJob() {};
 
@@ -413,7 +560,8 @@ public:
 	void SetUniverse(int uni) { universe = uni; }
 	void SetStatus(int st) { status = st; }
 	bool IsNoopJob();
-	void DirtyNoopAttr() { has_noop_attr = 2; }
+	bool IsOCUClaimer() const; // True if this job holds the ocu claim. It is not a real job
+	void DirtyNoopAttr() { run = JobRunnableState::DirtyNoop; }
 #if 0
 	// FUTURE:
 	int NumProcs() { if (entry_type == entry_type_cluster) return future_num_procs_or_hosts; return 0; }
@@ -433,23 +581,17 @@ public:
 // structure of job_queue hashtable entries for clusters.
 class JobQueueCluster : public JobQueueJob {
 public:
-	JobFactory * factory; // this will be non-null only for cluster ads, and only when the cluster is doing late materialization
+	JobFactory * factory{nullptr}; // this will be non-null only for cluster ads, and only when the cluster is doing late materialization
 protected:
-	int cluster_size; // number of materialized jobs in this cluster that the schedd is currently tracking.
-	int num_attached; // number of procs attached to this cluster.
-	int num_idle;
-	int num_running;
-	int num_held;
+	int cluster_size{0}; // number of materialized jobs in this cluster that the schedd is currently tracking.
+	int num_attached{0}; // number of procs attached to this cluster.
+	int num_idle{0};
+	int num_running{0};
+	int num_held{0};
 
 public:
 	JobQueueCluster(JOB_ID_KEY & job_id)
 		: JobQueueJob(job_id)
-		, factory(NULL)
-		, cluster_size(0)
-		, num_attached(0)
-		, num_idle(0)
-		, num_running(0)
-		, num_held(0)
 		{
 		}
 	virtual ~JobQueueCluster();
@@ -472,56 +614,36 @@ public:
 	void PopulateInfoAd(ClassAd & iad, int num_pending, bool include_factory_info); // fill out an info ad from fields in this structure and from the factory
 };
 
-// There are some bits of the qmgmt code that iterate the job queue
-// and assume that they are looking at a JobQueueJob without checking the type
-// so (until we can refactor out this behavior). 
-//
-#ifdef JOB_QUEUE_PAYLOAD_IS_BASE // JobQueueJobSet from JobQueueJob
 class JobQueueJobSet : public JobQueueBase {
-#else
-class JobQueueJobSet : public JobQueueJob {
-#endif
 public:
 	//inherited from JobQueueBase JOB_ID_KEY jid;
 	//inherited from JobQueueBase char entry_type;
 	enum class garbagePolicyEnum { immediateAfterEmpty, delayedAferEmpty };
 
-#ifdef JOB_QUEUE_PAYLOAD_IS_BASE // JobQueueJobSet from JobQueueJob
 protected:
 	// 3 bytes needed to align the next int
-	char spareA = 0;
-	char spareB = 0;
-	bool dirty = false;
+	char spareA{0};
+	char spareB{0};
+	bool dirty{false};
 public:
-	garbagePolicyEnum garbagePolicy = garbagePolicyEnum::immediateAfterEmpty;
-	unsigned int member_count = 0;
-	OwnerInfo * ownerinfo = nullptr;
+	garbagePolicyEnum garbagePolicy{garbagePolicyEnum::immediateAfterEmpty};
+	unsigned int member_count{0};
+	unsigned int pending_remove_count{0};
+	OwnerInfo * ownerinfo{nullptr};
 	LiveJobCounters jobStatusAggregates;
 	unsigned int Jobset() const { return (unsigned int)jid.cluster; }
-#else
-public:
-	garbagePolicyEnum garbagePolicy = garbagePolicyEnum::immediateAfterEmpty;
-	unsigned int member_count = 0;
-	LiveJobCounters jobStatusAggregates;
-#endif
 
 public:
-#ifdef JOB_QUEUE_PAYLOAD_IS_BASE // JobQueueJobSet from JobQueueJob
 	JobQueueJobSet(unsigned int jobset_id)
 		: JobQueueBase(JOB_ID_KEY(jobset_id,JOBSETID_qkey2), entry_type_jobset)
 	{
 	}
-#else
-	JobQueueJobSet(unsigned int jobset_id)
-		: JobQueueBase(entry_type_jobset)
-		, id(jobset_id)
-	{
-		jid.cluster = JOBSETID_to_qkey1(jobset_id);
-		jid.proc = JOBSETID_qkey2;
-	}
-#endif
 	virtual ~JobQueueJobSet() = default;
 	virtual void PopulateFromAd(); // populate this structure from contained ClassAd state
+
+	// add a numeric value to a numeric value in the user record, and optionally
+	// write the result to the job queue log
+	template <typename T> bool AccumToRecordAndLog(const char * attr, T addval, bool log_set_attr=true);
 };
 
 // until we can remove targettype from the classad log entirely
@@ -580,7 +702,7 @@ typedef enum {
 int PauseJobFactory(JobFactory * factory, MaterializeMode pause_code);
 int ResumeJobFactory(JobFactory * factory, MaterializeMode pause_code);
 bool CheckJobFactoryPause(JobFactory * factory, int want_pause); // Make sure factory mode matches the persist mode
-bool GetJobFactoryMaterializeMode(JobQueueCluster * cluster, int & pause_code);
+bool GetJobFactoryMaterializeMode(const JobQueueCluster * cluster, int & pause_code);
 void PopulateFactoryInfoAd(JobFactory * factory, ClassAd & iad);
 bool JobFactoryIsSubmitOnHold(JobFactory * factory, int & hold_code);
 void ScheduleClusterForDeferredCleanup(int cluster_id);
@@ -599,9 +721,9 @@ void DestroyJobQueue( void );
 int handle_q(int, Stream *sock);
 void dirtyJobQueue( void );
 bool SendDirtyJobAdNotification(const PROC_ID& job_id);
+bool JobQueueInitDone();
 
-#ifdef USE_JOB_QUEUE_USERREC
-
+bool ContainsUserName(const std::vector<std::string>& list, const char* user);
 bool isQueueSuperUser(const JobQueueUserRec * user);
 
 // Verify that the user issuing a command (test_owner) is authorized
@@ -615,24 +737,12 @@ bool UserCheck(const JobQueueBase *ad, const JobQueueUserRec * test_owner);
 // to modify the given queue object (job, jobset, userrec, etc).
 // when not_super is true, behave as if test_owner is not a superuser even if it is one.
 bool UserCheck2(const JobQueueBase *ad, const JobQueueUserRec * test_owner, bool not_super=false);
-#else
 
-bool isQueueSuperUser( const char* user );
-
-// Verify that the user issuing a command (test_owner) is authorized
-// to modify the given job.  In addition to everything UserCheck2()
-// does, this also calls IPVerify to check for WRITE authorization.
-// This call assumes Q_SOCK is set to a valid QmgmtPeer object.
-bool UserCheck( const ClassAd *ad, const char *test_owner );
-
-// Verify that the user issuing a command (test_owner) is authorized
-// to modify the given job.  Either ad or job_owner should be given
-// but not both.  If job_owner is NULL, the owner is looked up in the ad.
-bool UserCheck2( const ClassAd *ad, const char *test_owner, char const *job_owner=NULL );
-#endif
+int  MaxRunningPer(const JobQueueUserRec * urec);
+int  CurRunningPer(const JobQueueUserRec * urec);
 
 bool BuildPrioRecArray(bool no_match_found=false);
-void DirtyPrioRecArray();
+void DirtyPrioRecArray(int tid=-1);
 extern ClassAd *dollarDollarExpand(int cid, int pid, ClassAd *job, ClassAd *res, bool persist_expansions);
 bool rewriteSpooledJobAd(ClassAd *job_ad, int cluster, int proc, bool modify_ad);
 
@@ -660,6 +770,7 @@ typedef unsigned int SetAttributeFlags_t;
 const SetAttributeFlags_t SetAttribute_SubmitTransform     = (1 << 16);
 const SetAttributeFlags_t SetAttribute_LateMaterialization = (1 << 17);
 const SetAttributeFlags_t SetAttribute_Delete              = (1 << 18);
+const SetAttributeFlags_t SetAttribute_UserTransform       = (1 << 19); // a post-submit transform defined by the user
 
 JobQueueJob* GetNextJob(int initScan);
 JobQueueJob* GetNextJobByConstraint(const char *constraint, int initScan);
@@ -713,11 +824,7 @@ public:
 	JOB_ID_KEY_BUF(const JOB_ID_KEY& rhs)     : JOB_ID_KEY(rhs.cluster, rhs.proc) { job_id_str[0] = 0; }
 };
 
-#ifdef JOB_QUEUE_PAYLOAD_IS_BASE
 typedef JobQueueBase* JobQueuePayload;
-#else
-typedef JobQueueJob* JobQueuePayload;
-#endif
 // new for 8.3, use a non-string type as the key for the JobQueue
 // and a type derived from ClassAd for the payload.
 typedef ClassAdLog<JOB_ID_KEY, JobQueuePayload> JobQueueLogType;
@@ -752,36 +859,9 @@ public:
 	}
 	virtual bool insert(const char * key, ClassAd * ad) {
 		JOB_ID_KEY k(key);
-	#ifdef JOB_QUEUE_PAYLOAD_IS_BASE
 		JobQueuePayload payload = dynamic_cast<JobQueuePayload>(ad);
 		ASSERT(payload);
 		return table.insert(k, payload) >= 0;
-	#else
-		bool new_ad = false;
-		JobQueuePayload Ad = dynamic_cast<JobQueuePayload>(ad);
-		// if the incoming ad is really a ClassAd and not a JobQueue object, then make a new object.
-		// note this hack is just in case we have old code that is still treating jobs as classad
-		// eventually we should be able to get rid of this hack.  We can assume here that we will
-		// never be asked to make cluster or jobset objects
-		if ( ! Ad) {
-			ASSERT((k.cluster > 0 && k.proc >= 0) || (k.cluster == 0 && k.proc == 0));
-			Ad = new JobQueueJob(); Ad->Update(*ad); new_ad = true;
-		}
-		Ad->SetDirtyTracking(true);
-		JobQueueJob* payload = reinterpret_cast<JobQueueJob*>(Ad);
-		int iret = table.insert(k, payload);
-		// If we made a new ad, we must now delete one of them.
-		// On success, delete the original ad.
-		// On failure, delete the new ad (our caller will delete the original one).
-		if ( new_ad ) {
-			if ( iret >= 0 ) {
-				delete ad;
-			} else {
-				delete Ad;
-			}
-		}
-		return iret >= 0;
-	#endif
 	}
 	virtual void startIterations() { table.startIterations(); } // begin iterations
 	virtual bool nextIteration(const char*& key, ClassAd*&ad) {
@@ -847,9 +927,11 @@ JobQueueLogType::filter_iterator GetJobQueueIteratorEnd();
 
 
 class schedd_runtime_probe;
-#define WJQ_WITH_CLUSTERS 1  // include cluster ads when walking the job queue
-#define WJQ_WITH_JOBSETS  2  // include jobset ads when walking the job queue
-#define WJQ_WITH_NO_JOBS  4  // do not include job (proc) ads when walking the job queue
+#define WJQ_WITH_CLUSTERS 0x01  // include cluster ads when walking the job queue
+#define WJQ_WITH_JOBSETS  0x02  // include jobset ads when walking the job queue
+#define WJQ_WITH_NO_JOBS  0x04  // do not include job (proc) ads when walking the job queue
+#define WJQ_WITH_USERS    0x10  // include user records
+#define WJQ_WITH_PROJECTS 0x20  // include project records
 typedef int (*queue_scan_func)(JobQueuePayload ad, const JobQueueKey& key, void* user);
 void WalkJobQueueEntries(int with, queue_scan_func fn, void* pv, schedd_runtime_probe & ftm);
 #define WalkJobQueueWith(with,fn,pv) WalkJobQueueEntries(with, (fn), pv, WalkJobQ_ ## fn ## _runtime )
@@ -885,21 +967,36 @@ QmgmtPeer* getQmgmtConnectionInfo();
 bool JobSetDestroy(int setid);
 bool JobSetCreate(int setId, const char * setName, const char * ownerinfoName);
 
-#ifdef USE_JOB_QUEUE_USERREC
 bool UserRecDestroy(int userrec_id);
-bool UserRecCreate(int userrec_id, const char * ownerinfoName, const ClassAd & cmdAd, const ClassAd & defaultsAd, bool enabled);
+bool UserRecCreate(int userrec_id, bool is_project, const char * name, const ClassAd & cmdAd, bool enabled);
 void UserRecFixupDefaultsAd(ClassAd & defaultsAd);
-#endif
 
 // priority records
-extern prio_rec *PrioRec;
-extern int N_PrioRecs;
-extern HashTable<int,int> *PrioRecAutoClusterRejected;
-extern int grow_prio_recs(int);
+//#define PRIO_REC_IS_VECTOR 1
+#ifdef PRIO_REC_IS_VECTOR
+  extern std::vector<prio_rec> PrioRec;
+#else
+  extern std::deque<prio_rec> PrioRec;
+#endif
 
-extern void	FindRunnableJob(PROC_ID & jobid, ClassAd* my_match_ad, char const * user);
-extern bool Runnable(PROC_ID*);
-extern bool Runnable(JobQueueJob *job, const char *& reason);
+extern void	FindRunnableJob(PROC_ID & jobid, ClassAd* my_match_ad, char const * user, const char* pool=nullptr, bool is_ocu=false, bool is_new_match=true);
+//extern bool Runnable(PROC_ID*);
+enum class runnable_reason_code : int {
+	IsRunnable=0,
+	NotFound,
+	IsNoopJob,
+	NotIdle,
+	UniverseNotInService,
+	IsOCUClaimer, // job is an OCU claimer, not a real job
+	InLongCooldown,  // cooldown > 5min
+	InShortCooldown, // cooldown <= 5min
+	AlreadyMatched,
+	MaxRunningAlready,
+};
+//extern bool Runnable(JobQueueJob *job, const char *& reason);
+extern bool Runnable(JobQueueJob *job, runnable_reason_code & code);
+extern const char * getRunnableReason(runnable_reason_code code);
+extern prio_rec * findInPrioRec(const PROC_ID & jobid);
 
 extern class ForkWork schedd_forker;
 
