@@ -17,8 +17,6 @@
  *
  ***************************************************************/
 
-#define _POSIX_SOURCE
-
 #include "condor_common.h"
 #include "condor_io.h"
 #include "condor_classad.h"
@@ -456,7 +454,7 @@ SetAttribute( int cluster_id, int proc_id, char const *attr_name, char const *at
 }
 
 int
-SetTimerAttribute( int cluster_id, int proc_id, char const *attr_name, int duration )
+SetTimerAttribute( int cluster_id, int proc_id, char const *attr_name, time_t duration )
 {
 	int	rval = -1;
 
@@ -641,7 +639,7 @@ GetAttributeFloat( int cluster_id, int proc_id, char *attr_name, float *value )
 
 
 int
-GetAttributeInt( int cluster_id, int proc_id, char const *attr_name, int *value )
+GetAttributeInt( int cluster_id, int proc_id, char const *attr_name, long long *value )
 {
 	int	rval = -1;
 
@@ -668,6 +666,23 @@ GetAttributeInt( int cluster_id, int proc_id, char const *attr_name, int *value
 	return rval;
 }
 
+int
+GetAttributeInt( int cluster_id, int proc_id, char const *attr_name, int *value )
+{
+	long long full_value = *value;
+	int rval = GetAttributeInt(cluster_id, proc_id, attr_name, &full_value);
+	*value = (int)full_value;
+	return rval;
+}
+
+int
+GetAttributeInt( int cluster_id, int proc_id, char const *attr_name, long *value )
+{
+	long long full_value = *value;
+	int rval = GetAttributeInt(cluster_id, proc_id, attr_name, &full_value);
+	*value = (long)full_value;
+	return rval;
+}
 
 int
 GetAttributeStringNew( int cluster_id, int proc_id, char const *attr_name, char **val )
