@@ -35,9 +35,6 @@
 #include <unordered_map>
 #include <queue>
 
-// use a persistent JobQueueUserRec instead of ephemeral OwnerInfo class
-#define USE_JOB_QUEUE_USERREC 1
-
 #include "dc_collector.h"
 #include "daemon.h"
 #include "daemon_list.h"
@@ -119,6 +116,8 @@ class LocalJobRec {
 
 bool jobLeaseIsValid( ClassAd* job, int cluster, int proc );
 
+int init_user_ids(const JobQueueUserRec * user);
+
 class match_rec;
 
 struct shadow_rec
@@ -144,6 +143,7 @@ struct shadow_rec
 	PROC_ID			prev_job_id;
 	Stream*			recycle_shadow_stream;
 	bool			exit_already_handled;
+	char*			secret; // Secret provided to spawned daemon for authorization of commands with tools
 
 	shadow_rec();
 	~shadow_rec();
@@ -169,6 +169,10 @@ struct SubmitterCounters {
   int SchedulerJobsIdle;    // Scheduler Universe (i.e dags)
   int LocalJobsRunning; // Local universe
   int LocalJobsIdle;    // Local universe
+  int OCUClaims; // OCU Claims held indefinitely by this submitter
+  int OCUClaimsBorrowed; // OCU Claims in use by someone other than the owner
+  int OCUWantedJobs; // Total number of idle+running jobs that want an OCU claim
+  int OCURunningJobs; // Running number of jobs that want an OCU Claim
   int Hits;  // used in the mark/sweep algorithm of count_jobs to detect Owners that no longer have any jobs in the queue.
   int JobsCounted; // smaller than Hits by the number of match recs for this Owner.
 //  int JobsRecentlyAdded; // zeroed on each sweep, incremented on submission.
@@ -182,6 +186,10 @@ struct SubmitterCounters {
 	, JobsFlocked(0)
 	, SchedulerJobsRunning(0), SchedulerJobsIdle(0)
 	, LocalJobsRunning(0), LocalJobsIdle(0)
+	, OCUClaims(0)
+	, OCUClaimsBorrowed(0)
+	, OCUWantedJobs(0)
+	, OCURunningJobs(0)
 	, Hits(0)
 	, JobsCounted(0)
 //	, JobsRecentlyAdded(0)
@@ -215,168 +223,87 @@ struct SubmitterData {
 
 typedef std::map<std::string, SubmitterData> SubmitterDataMap;
 
-#ifdef USE_JOB_QUEUE_USERREC
 class JobQueueUserRec;
+class JobQueueProjectRec;
 typedef JobQueueUserRec OwnerInfo;
 typedef std::map<std::string, JobQueueUserRec*> OwnerInfoMap;
+typedef std::map<std::string, JobQueueProjectRec*, classad::CaseIgnLTStr> ProjectInfoMap;
 // attribute of the JobQueueUserRec to use as the Name() and key value of the OwnerInfo struct
 constexpr int  CONDOR_USERREC_ID = 1;
 constexpr int  LAST_RESERVED_USERREC_ID = CONDOR_USERREC_ID;
-#if 1
- constexpr bool USERREC_NAME_IS_FULLY_QUALIFIED = true;
- #define ATTR_USERREC_NAME ATTR_USER
-#else
- constexpr bool USERREC_NAME_IS_FULLY_QUALIFIED = false;
- #define ATTR_USERREC_NAME ATTR_OWNER
-#endif
-#else
-
-struct RealOwnerCounters {
-  int Hits;                 // counts (possibly overcounts) references to this class, used for mark/sweep expiration code
-  int JobsCounted;          // ALL jobs in the queue owned by this owner at the time count_jobs() was run
-  int JobsRecentlyAdded;    // ALL jobs owned by this owner that were added since count_jobs() was run
-  int JobsIdle;             // does not count Local or Scheduler universe jobs, or Grid jobs that are externally managed.
-  int JobsRunning;
-  int JobsHeld;
-  int LocalJobsIdle;
-  int LocalJobsRunning;
-  int LocalJobsHeld;
-  int SchedulerJobsIdle;
-  int SchedulerJobsRunning;
-  int SchedulerJobsHeld;
-  void clear_counters() { memset(this, 0, sizeof(*this)); }
-  RealOwnerCounters()
-	: Hits(0)
-	, JobsCounted(0)
-	, JobsRecentlyAdded(0)
-	, JobsIdle(0)
-	, JobsRunning(0)
-	, JobsHeld(0)
-	, LocalJobsIdle(0)
-	, LocalJobsRunning(0)
-	, LocalJobsHeld(0)
-	, SchedulerJobsIdle(0)
-	, SchedulerJobsRunning(0)
-	, SchedulerJobsHeld(0)
-  {}
-};
-
-// The schedd will have one of these records for each unique owner, it counts jobs that
-// have that Owner attribute even if the jobs also have an AccountingGroup or NiceUser
-// attribute and thus have a different SUBMITTER name than their OWNER name
-// The counts in this structure are used to enforce MAX_JOBS_PER_OWNER and other PER_OWNER
-// limits, they are NOT sent to the collector and are never seen by the accountant - the SubmitterData is used for accounting
-//
-struct OwnerInfo {
-  std::string name;
-  const char * Name() const { return name.empty() ? "" : name.c_str(); }
-  bool empty() const { return name.empty(); }
-  RealOwnerCounters num; // job counts by OWNER rather than by submitter
-  LiveJobCounters live; // job counts that are always up-to-date with the committed job state
-  time_t LastHitTime; // records the last time we incremented num.Hit, use to expire OwnerInfo
-  OwnerInfo() : LastHitTime(0) { }
-};
-
-typedef std::map<std::string, OwnerInfo> OwnerInfoMap;
-#endif
 
 class match_rec
 {
  public:
-    match_rec(char const*, char const*, PROC_ID*, const ClassAd*, char const*, char const* pool,bool is_dedicated);
+    match_rec(char const*, char const*, const JOB_ID_KEY & jid, const ClassAd*, char const*, char const* pool,bool is_dedicated);
 	~match_rec();
 
-    char*   		peer; //sinful address of startd
-	std::string        m_description;
+	char * peer{nullptr}; //sinful address of startd
+	char * user{nullptr};
+	char * pool{nullptr}; // negotiator hostname if flocking; else empty
+	shadow_rec * shadowRec{nullptr};
 
 		// cluster of the job we used to obtain the match
-	int				origcluster; 
+	int origcluster{0};
 
 		// if match is currently active, cluster and proc of the
 		// running job associated with this match; otherwise,
 		// cluster==origcluster and proc==-1
 		// NOTE: use SetMrecJobID() to change cluster and proc,
 		// because this updates the index of matches by job id.
-    int     		cluster;
-    int     		proc;
+	int cluster{0};
+	int proc{0};
+
+	int status{0};
+	int num_exceptions{0};
+	int keep_while_idle{0}; // number of seconds to hold onto an idle claim
+	time_t idle_timer_deadline{0}; // if the above is nonzero, abstime to hold claim
+	time_t entered_current_status{0};
+	time_t last_alive{time(0)};
+	PROC_ID m_now_job{0,0};
+
+	ClassAd * my_match_ad{nullptr};
+	ClassAd m_added_attrs;
+
+	bool is_dedicated{false}; // true if this match belongs to ded. sched.
+	bool allocated{false}; // For use by the DedicatedScheduler
+	bool scheduled{false}; // For use by the DedicatedScheduler
+	bool needs_release_claim{false};
+	bool use_sec_session{false};
+	bool			is_ocu {false}; // when true, hold forever, hand out to others
+    PROC_ID         ocu_originator;  // procid of the ocu claimer job
+									
+	bool m_claim_pslot{false};
+	int  m_multi_slot{0}; // when > 1, this is a multi-slot claim request
 
-    int     		status;
-	shadow_rec*		shadowRec;
-	int				num_exceptions;
-	time_t			entered_current_status;
-	ClassAd*		my_match_ad;
-	ClassAd         m_added_attrs;
-	char*			user;
-	bool            is_dedicated; // true if this match belongs to ded. sched.
-	bool			allocated;	// For use by the DedicatedScheduler
-	bool			scheduled;	// For use by the DedicatedScheduler
-	bool			needs_release_claim;
-	bool use_sec_session;
 	ClaimIdParser claim_id;
-	classy_counted_ptr<DCMsgCallback> claim_requester;
+	classy_counted_ptr<DCMsgCallback> claim_requester{nullptr};
 
 		// if we created a dynamic hole in the DAEMON auth level
 		// to support flocking, this will be set to the id of the
 		// punched hole
-	std::string*	auth_hole_id;
-
-	bool m_startd_sends_alives;
-	bool m_claim_pslot;
-
-	int keep_while_idle; // number of seconds to hold onto an idle claim
-	int idle_timer_deadline; // if the above is nonzero, abstime to hold claim
+	std::string*	auth_hole_id{nullptr};
 
 		// Set the mrec status to the given value (also updates
 		// entered_current_status)
 	void	setStatus( int stat );
 
+	std::string m_description;
 	void makeDescription();
 	char const *description() const {
 		return m_description.c_str();
 	}
 
-	const std::string & getPool() const {return m_pool;}
-
-	PROC_ID m_now_job;
-
-	std::string m_pool; // negotiator hostname if flocking; else empty
 };
 
-class UserIdentity {
-	public:
-			// The default constructor is not recommended as it
-			// has no real identity.  It only exists so
-			// we can put UserIdentities in various templates.
-		UserIdentity() : m_username(""), m_osname(""), m_auxid("") { }
-		UserIdentity(const char * user, const char * osname, const char * aux)
-			: m_username(user), m_osname(osname), m_auxid(aux) { }
-		UserIdentity(const UserIdentity & src) {
-			m_username = src.m_username;
-			m_osname = src.m_osname;
-			m_auxid = src.m_auxid;			
-		}
-		UserIdentity(JobQueueJob& job_ad);
-		const UserIdentity & operator=(const UserIdentity & src) {
-			m_username = src.m_username;
-			m_osname = src.m_osname;
-			m_auxid = src.m_auxid;
-			return *this;
-		}
-		bool operator==(const UserIdentity & rhs) {
-			return m_username == rhs.m_username && 
-				m_auxid == rhs.m_auxid;
-		}
-		const std::string& username() const { return m_username; }
-		const std::string& osname() const { return m_osname; }
-		const std::string& auxid() const { return m_auxid; }
+struct GridUserIdentity {
+	GridUserIdentity() = default;
+	GridUserIdentity(JobQueueJob& job_ad);
 
-			// For use in HashTables
-		static size_t HashFcn(const UserIdentity & index);
+	auto operator<=>(const GridUserIdentity&) const = default;
 	
-	private:
-		std::string m_username;
-		std::string m_osname;
-		std::string m_auxid;
+	const JobQueueUserRec* m_ownerinfo{nullptr};
+	std::string m_auxid;
 };
 
 
@@ -477,6 +404,7 @@ private:
 };
 
 class JobSets; // forward reference - declared in jobsets.h
+class OCU; // forward reference - declared in qmgmt.h
 
 class Scheduler : public Service
 {
@@ -533,6 +461,7 @@ class Scheduler : public Service
     friend  void    add_shadow_birthdate(int cluster, int proc, bool is_reconnect);
 	void			display_shadow_recs();
 	int				actOnJobs(int, Stream *);
+	int				getChildConctactInfo(int /*cmd*/, Stream * s);
 	void            enqueueActOnJobMyself( PROC_ID job_id, JobAction action, bool log );
 	int             actOnJobMyselfHandler( ServiceData* data );
 	int				updateGSICred(int, Stream* s);
@@ -557,10 +486,18 @@ class Scheduler : public Service
 	// match managing
 	int 			publish( ClassAd *ad );
 	void			OptimizeMachineAdForMatchmaking(ClassAd *ad);
-    match_rec*      AddMrec(char const*, char const*, PROC_ID*, const ClassAd*, char const*, char const*, match_rec **pre_existing=NULL);
+	match_rec*		AddMrec(
+		const char * claim_id,
+		const char * addr,
+		const JOB_ID_KEY& jid,
+		const ClassAd* matchAd,
+		const char * user,
+		const char * pool,
+		const ClassAd* extraAttrs = nullptr, // extra attrs sent by negotiator
+		match_rec **pre_existing = nullptr);
 	// support for START_VANILLA_UNIVERSE
 	ExprTree *      flattenVanillaStartExpr(JobQueueJob * job, const OwnerInfo* powni);
-	bool            jobCanUseMatch(JobQueueJob * job, ClassAd * slot_ad, const std::string &pool, const char *&because); // returns true when START_VANILLA allows this job to run on this match
+	bool            jobCanUseMatch(JobQueueJob * job, ClassAd * slot_ad, const char * pool, const char *&because); // returns true when START_VANILLA allows this job to run on this match
 	bool            jobCanNegotiate(JobQueueJob * job, const char *&because); // returns true when START_VANILLA allows this job to negotiate
 	bool            vanillaStartExprIsConst(VanillaMatchAd &vad, bool &bval);
 	bool            evalVanillaStartExpr(VanillaMatchAd &vad);
@@ -578,32 +515,32 @@ class Scheduler : public Service
 	shadow_rec*		FindSrecByProcID(PROC_ID);
 	void			RemoveShadowRecFromMrec(shadow_rec*);
 	void            sendSignalToShadow(pid_t pid,int sig,PROC_ID proc);
-	int				AlreadyMatched(PROC_ID*);
-	int				AlreadyMatched(JobQueueJob * job, int universe);
 	void			ExpediteStartJobs() const;
 	void			StartJobs( int timerID = -1 );
 	void			StartJob(match_rec *rec);
-	void			sendAlives( int timerID = -1 );
+	void			checkClaimLeases( int timerID = -1 );
 	void			RecomputeAliveInterval(int cluster, int proc);
 	void			StartJobHandler( int timerID = -1 );
 	void			addRunnableJob( shadow_rec* );
 	void			spawnShadow( shadow_rec* );
 	void			spawnLocalStarter( shadow_rec* );
 	bool			claimLocalStartd();
-	bool			isStillRunnable( int cluster, int proc, int &status ); 
-	WriteUserLog*	InitializeUserLog( PROC_ID job_id );
-	bool			WriteSubmitToUserLog( JobQueueJob* job, bool do_fsync, const char * warning );
-	bool			WriteAbortToUserLog( PROC_ID job_id );
-	bool			WriteHoldToUserLog( PROC_ID job_id );
-	bool			WriteReleaseToUserLog( PROC_ID job_id );
-	bool			WriteExecuteToUserLog( PROC_ID job_id, const char* sinful = NULL );
-	bool			WriteEvictToUserLog( PROC_ID job_id, bool checkpointed = false );
-	bool			WriteTerminateToUserLog( PROC_ID job_id, int status );
-	bool			WriteRequeueToUserLog( PROC_ID job_id, int status, const char * reason );
-	bool			WriteAttrChangeToUserLog( const char* job_id_str, const char* attr, const char* attr_value, const char* old_value);
-	bool			WriteClusterSubmitToUserLog( JobQueueCluster* cluster, bool do_fsync );
-	bool			WriteClusterRemoveToUserLog( JobQueueCluster* cluster, bool do_fsync );
-	bool			WriteFactoryPauseToUserLog( JobQueueCluster* cluster, int hold_code, const char * reason, bool do_fsync=false ); // write pause or resume event.
+	bool			isStillRunnable( int cluster, int proc, int &status );
+
+	WriteUserLog*	InitializeUserLog(const JobQueueJob* ad);
+	bool			WriteSubmitToUserLog(const JobQueueJob* job, bool do_fsync, const char * warning );
+	bool			WriteAbortToUserLog(const JobQueueJob* job);
+	bool			WriteHoldToUserLog(const JobQueueJob* job);
+	bool			WriteReleaseToUserLog(const JobQueueJob* job);
+	bool			WriteExecuteToUserLog(const JobQueueJob* job, const char* sinful = NULL);
+	bool			WriteEvictToUserLog(const JobQueueJob* job, bool checkpointed = false);
+	bool			WriteTerminateToUserLog(const JobQueueJob* job, int status);
+	bool			WriteRequeueToUserLog(const JobQueueJob* job, int status, const char * reason);
+	bool			WriteAttrChangeToUserLog(const JobQueueJob* job, const char* attr, const char* attr_value, const char* old_value);
+	bool			WriteClusterSubmitToUserLog(const JobQueueCluster* cluster, bool do_fsync );
+	bool			WriteClusterRemoveToUserLog(const JobQueueCluster* cluster, bool do_fsync );
+	bool			WriteFactoryPauseToUserLog(const JobQueueCluster* cluster, int hold_code, const char * reason, bool do_fsync=false ); // write pause or resume event.
+
 	int				receive_startd_alive(int cmd, Stream *s) const;
 	void			InsertMachineAttrs( int cluster, int proc, ClassAd *machine, bool do_rotation );
 		// Public startd socket management functions
@@ -663,6 +600,7 @@ class Scheduler : public Service
 	bool			getAllowLateMaterialize() const { return AllowLateMaterialize; }
 	bool			getNonDurableLateMaterialize() const { return NonDurableLateMaterialize; }
 	const ClassAd & getUserRecDefaultsAd() const { return m_userRecDefaultsAd; }
+	const ClassAd & getProjectRecDefaultsAd() const { return m_projectRecDefaultsAd; }
 	const ClassAd * getExtendedSubmitCommands() const { return &m_extendedSubmitCommands; }
 	const std::string & getExtendedSubmitHelpFile() const { return m_extendedSubmitHelpFile; }
 	bool			getEnableJobQueueTimestamps() const { return EnableJobQueueTimestamps; }
@@ -672,7 +610,7 @@ class Scheduler : public Service
 	int				getMaxJobsPerOwner() const { return MaxJobsPerOwner; }
 	int				getMaxJobsPerSubmission() const { return MaxJobsPerSubmission; }
 
-		// Used by the UserIdentity class and some others
+		// Used by the GridUserIdentity class and some others
 	const ExprTree*	getGridParsedSelectionExpr() const 
 					{ return m_parsed_gridman_selection_expr; };
 	const char*		getGridUnparsedSelectionExpr() const
@@ -684,7 +622,7 @@ class Scheduler : public Service
 	void			delete_shadow_rec(int);
 	void			delete_shadow_rec(shadow_rec *rec);
 	shadow_rec*     add_shadow_rec( int pid, PROC_ID*, int univ, match_rec*,
-									int fd );
+									int fd, const char* secret );
 	shadow_rec*		add_shadow_rec(shadow_rec*);
 	void			add_shadow_rec_pid(shadow_rec*);
 	void			HadException( match_rec* );
@@ -722,33 +660,57 @@ class Scheduler : public Service
 	// live counters for running/held/idle jobs
 	LiveJobCounters liveJobCounts; // job counts that are always up-to-date with the committed job state
 
+	// fsync tracking by user
+	std::map<std::string, stats_entry_probe<double>> FsyncRuntimes;
+	
+
 	// the significant attributes that the schedd belives are absolutely required.
 	// This is NOT the effective set of sig attrs we get after we talk to negotiators
 	// it is the basic set needed for correct operation of the Schedd: Requirements,Rank,
 	classad::References MinimalSigAttrs;
 
-#ifdef USE_JOB_QUEUE_USERREC
 	int		nextUnusedUserRecId();
-	JobQueueUserRec * jobqueue_newUserRec(int userrec_id);
+	JobQueueUserRec * jobqueue_newUserRec(int userrec_id, const char * mytype);
 	void jobqueue_deleteUserRec(JobQueueUserRec * uad);
-	void mapPendingOwners();
+	void jobqueue_deleteProject(JobQueueProjectRec * pjad);
+	void mapPendingOwners(); // pending owners can be either userrec or projectrec
 	// these are used during startup to handle the case where jobs have Owner/User attributes but
 	// there is no persistnt JobQueueUserRec in the job_queue
 	const std::map<int, OwnerInfo*> & queryPendingOwners() { return pendingOwners; }
 	void clearPendingOwners();
 	bool HasPersistentOwnerInfo() const { return EnablePersistentOwnerInfo; }
-#endif
+	bool HasPersistentProjectInfo() const { return EnablePersistentProjectInfo; }
 	void deleteZombieOwners(); // delete all zombies (called on shutdown)
 	void purgeZombieOwners();  // delete unreferenced zombies (called in count_jobs)
-	const OwnerInfo * insert_owner_const(const char*);
+	const OwnerInfo * insert_owner_const(const char*, CondorError* errstack=nullptr);
 	const OwnerInfo * lookup_owner_const(const char*);
-	OwnerInfo * incrementRecentlyAdded(OwnerInfo * ownerinfo, const char * owner);
+	// make sure that the ownerInfo has a valid OsUser, assiging a generic one if needed.
+	const char *    solidify_os_user(const OwnerInfo *, CondorError* errstack=nullptr);
+	// make sure that a job object has a submitter record pointer
+	const SubmitterData * get_submitter(JobQueueJob * job) {
+		SubmitterData * subdat = nullptr;
+		if (job) { get_submitter_and_owner(job, subdat); }
+		return subdat;
+	}
+	void incrementRecentlyAdded(OwnerInfo * ownerinfo);
+	JobQueueProjectRec * find_projectinfo(const char * project_name);
+	JobQueueProjectRec * get_projectinfo(JobQueueJob * job);
+	// find a project record or insert a pending project record
+	JobQueueProjectRec * insert_projectinfo(const char * project_name);
+
+	void configGenericOsUsers();
+
+	bool m_useGenericOsUsers{false};
+	std::set<std::string> m_openGenericOsUsers;
+	std::set<std::string> m_claimedGenericOsUsers;
 
 	std::set<LocalJobRec> LocalJobsPrioQueue;
 
 	// Class to manage sets of Job 
 	JobSets *jobSets;
 
+	std::map<GridUserIdentity, GridJobCounts> GridJobOwners;
+
 	bool ExportJobs(ClassAd & result, std::set<int> & clusters, const char *output_dir, const OwnerInfo *user, const char * new_spool_dir="##");
 	bool ImportExportedJobResults(ClassAd & result, const char * import_dir, const OwnerInfo *user);
 	bool UnexportJobs(ClassAd & result, std::set<int> & clusters, const OwnerInfo *user);
@@ -768,9 +730,15 @@ class Scheduler : public Service
 		}
 		return &m_local_starter_ad;
 	}
-private:
 
-	bool JobCanFlock(classad::ClassAd &job_ad, const std::string &pool);
+    // This is a clumsy hack, but since FindRunnableJob() can be called
+    // from places other than the scheduler object, necessary.
+    ClassAd * getScheddAd() { return m_adSchedd; }
+
+	bool JobCanFlock(classad::ClassAd &job_ad, const char *pool);
+
+	OCU *getOCU(int ocu_id); 
+private:
 
 	// Setup a new security session for a remote negotiator.
 	// Returns a capability that can be included in an ad sent to the collector.
@@ -803,6 +771,7 @@ private:
 	ClassAd*			m_adSchedd;
 	ClassAd*        	m_adBase;
 	ClassAd             m_userRecDefaultsAd;
+	ClassAd             m_projectRecDefaultsAd;
 	ClassAd             m_extendedSubmitCommands;
 	std::string         m_extendedSubmitHelpFile;
 	ClassAd             m_local_starter_ad;
@@ -821,6 +790,7 @@ private:
 	Timeslice       PeriodicExprInterval;
 	int             periodicid;
 	int				QueueCleanInterval;
+	int				WriteHistRecordInterval{0};
 	int             RequestClaimTimeout;
 	int				JobStartDelay;
 	int				JobStartCount;
@@ -831,6 +801,7 @@ private:
 	int				MaxJobsRunning;
 	bool			AllowLateMaterialize;
 	bool			EnablePersistentOwnerInfo;
+	bool			EnablePersistentProjectInfo;
 	bool			NonDurableLateMaterialize;	// for testing, use non-durable transactions when materializing new jobs
 	bool			EnableJobQueueTimestamps;	// for testing
 	int				MaxMaterializedJobsPerCluster;
@@ -869,8 +840,8 @@ private:
 	OwnerInfoMap    OwnersInfo;    // map of job counters by owner, used to enforce MAX_*_PER_OWNER limits
 	std::map<int, OwnerInfo*> pendingOwners; // OwnerInfo records that have been created but not yet committed
 	std::vector<OwnerInfo*> zombieOwners; // OwnerInfo records that have been removed from the job_queue, but not yet deleted
+	ProjectInfoMap  ProjectInfo;   // map of JobQueueProjectRec ads by project name
 
-	HashTable<UserIdentity, GridJobCounts> GridJobOwners;
 	time_t			NegotiationRequestTime;
 	int				ExitWhenDone;  // Flag set for graceful shutdown
 	std::queue<shadow_rec*> RunnableJobQueue;
@@ -910,12 +881,6 @@ private:
 		// variables to implement SCHEDD_VANILLA_START expression
 	ConstraintHolder vanilla_start_expr;
 
-		// Get the associated GridJobCounts object for a given
-		// user identity.  If necessary, will create a new one for you.
-		// You can read or write the values, but don't go
-		// deleting the pointer!
-	GridJobCounts * GetGridJobCounts(UserIdentity user_identity);
-
 		// (un)parsed expressions from condor_config GRIDMANAGER_SELECTION_EXPR
 	ExprTree* m_parsed_gridman_selection_expr;
 	char* m_unparsed_gridman_selection_expr;
@@ -947,7 +912,7 @@ private:
 
 	// utility functions
 	void		sumAllSubmitterData(SubmitterData &all);
-	void		updateSubmitterAd(SubmitterData &submitterData, ClassAd &pAd, DCCollector *collector,  int flock_level, time_t time_now);
+	void		updateSubmitterAd(SubmitterData &subData, ClassAd &pAd, DCCollector *collector,  int flock_level, time_t time_now);
 	int			count_jobs();
 	bool		fill_submitter_ad(ClassAd & pAd, const SubmitterData & Owner, const std::string &pool_name, int flock_level);
 	int			make_ad_list(ClassAdList & ads, ClassAd * pQueryAd=NULL);
@@ -957,21 +922,28 @@ private:
 	int			command_query_job_aggregates(ClassAd & query, Stream* stream);
 	int			command_query_user_ads(int, Stream* stream);
 	int			command_act_on_user_ads(int, Stream* stream);
+	int			command_act_on_ocus(int, Stream* stream);
+    ClassAd     act_on_ocu_create(const ClassAd &request);
+    ClassAd     act_on_ocu_remove(const ClassAd &request);
+	std::vector<ClassAd>     act_on_ocu_query(const ClassAd &request);
 	void   			check_claim_request_timeouts( void );
 	OwnerInfo     * find_ownerinfo(const char*);
-	OwnerInfo     * insert_ownerinfo(const char*);
+	OwnerInfo     * insert_ownerinfo(const char*, CondorError* errstack=nullptr);
 	SubmitterData * insert_submitter(const char*);
 	SubmitterData * find_submitter(const char*);
 	OwnerInfo * get_submitter_and_owner(JobQueueJob * job, SubmitterData * & submitterinfo);
 	OwnerInfo * get_ownerinfo(JobQueueJob * job);
 	int			act_on_user(int cmd, const std::string & username, const ClassAd& cmdAd,
-					TransactionWatcher & txn, CondorError & errstack, struct UpdateUserAttributesInfo & info);
+					TransactionWatcher & txn, CondorError & errstack, struct UpdateUserRecAttributesInfo & info);
+	int			act_on_project(int cmd, const std::string & username, const ClassAd& cmdAd,
+					TransactionWatcher & txn, CondorError & errstack, struct UpdateUserRecAttributesInfo & info);
 	void		remove_unused_owners();
+	bool		any_userrec_refs(JobQueueUserRec * urec); // returns true if any schedd data structures are holding this given ptr
 	void			child_exit(int, int);
 	// AFAICT, reapers should be be registered void to begin with.
 	int				child_exit_from_reaper(int a, int b) { child_exit(a, b); return 0; }
 	void			scheduler_univ_job_exit(int pid, int status, shadow_rec * srec);
-	void			scheduler_univ_job_leave_queue(PROC_ID job_id, int status, ClassAd *ad);
+	void			scheduler_univ_job_leave_queue(JobQueueJob *job, int status);
 	void			clean_shadow_recs();
 	void			preempt( int n, bool force_sched_jobs = false );
 	void			attempt_shutdown( int timerID = -1 );
@@ -1004,7 +976,7 @@ private:
 		// to query the CronTab object to ask it what the next
 		// runtime is for job is
 		//
-	HashTable<PROC_ID, CronTab*> *cronTabs;	
+	std::map<PROC_ID, CronTab*> cronTabs;
 		//
 		// A list of cluster_ids that may have jobs that require
 		// CronTab execution scheduling
@@ -1042,11 +1014,11 @@ private:
 
 	static void		token_request_callback(bool success, void *miscdata);
 
-	HashTable <std::string, match_rec *> *matches;
-	HashTable <PROC_ID, match_rec *> *matchesByJobID;
-	HashTable <int, shadow_rec *> *shadowsByPid;
-	HashTable <PROC_ID, shadow_rec *> *shadowsByProcID;
-	HashTable <int, std::vector<PROC_ID> *> *spoolJobFileWorkers;
+	std::map<std::string, match_rec *> matches;
+	std::map<PROC_ID, match_rec *> matchesByJobID;
+	std::map<int, shadow_rec *> shadowsByPid;
+	std::map<PROC_ID, shadow_rec *> shadowsByProcID;
+	std::map<int, std::vector<PROC_ID> *> spoolJobFileWorkers;
 	int				numMatches;
 	int				numShadows;
 	std::vector<DCCollector> FlockCollectors;
@@ -1063,7 +1035,7 @@ private:
 		// leaseAliveInterval is the minimum interval we need to send
 		// keepalives based upon ATTR_JOB_LEASE_DURATION...
 	int				leaseAliveInterval;  
-	int				aliveid;	// timer id for sending keepalives to startd
+	int				aliveid;	// timer id for checking claim leases
 	int				MaxExceptions;	 // Max shadow excep. before we relinquish
 
 		// get connection info for creating sec session to a running job
@@ -1092,6 +1064,8 @@ private:
 		// Command handler for collector token requests.
 	int handle_collector_token_request(int, Stream *s);
 
+	void maybeWriteDaemonHistory(ClassAd* ad);
+
 		// A bit that says wether or not we've sent email to the admin
 		// about a shadow not starting.
 	int sent_shadow_failure_email;
@@ -1108,13 +1082,6 @@ private:
 	std::map<std::string, ClassAd *> m_unclaimedLocalStartds;
 	std::map<std::string, ClassAd *> m_claimedLocalStartds;
 
-    int m_userlog_file_cache_max;
-    time_t m_userlog_file_cache_clear_last;
-    int m_userlog_file_cache_clear_interval;
-    WriteUserLog::log_file_cache_map_t m_userlog_file_cache;
-    void userlog_file_cache_clear(bool force = false);
-    void userlog_file_cache_erase(const int& cluster, const int& proc);
-
 	// State for the history helper queue.
 	// object to manage history queries in flight
 	HistoryHelperQueue HistoryQue;
@@ -1127,6 +1094,7 @@ private:
 	MapFile m_protected_url_map;
 
 	friend class DedicatedScheduler;
+	friend class JobQueueUserRec;
 };
 
 
@@ -1136,7 +1104,6 @@ struct JOB_ID_KEY;
 extern void set_job_status(int cluster, int proc, int status);
 extern bool claimStartd( match_rec* mrec );
 extern bool claimStartdConnected( Sock *sock, match_rec* mrec, ClassAd *job_ad);
-extern bool sendAlive( match_rec* mrec );
 extern void fixReasonAttrs( PROC_ID job_id, JobAction action );
 extern bool moveStrAttr( PROC_ID job_id, const char* old_attr,  
 						 const char* new_attr, bool verbose );
@@ -1148,15 +1115,15 @@ extern void incrementJobAdAttr(int cluster, int proc, const char* attrName, cons
 extern bool holdJob( int cluster, int proc, const char* reason = NULL, 
 					 int reason_code=0, int reason_subcode=0,
 					 bool use_transaction = false, 
-					 bool email_user = false, bool email_admin = false,
+					 bool email_user = false,
 					 bool system_hold = true,
 					 bool write_to_user_log = true);
 extern bool releaseJob( int cluster, int proc, const char* reason = NULL, 
 					 bool use_transaction = false, 
-					 bool email_user = false, bool email_admin = false,
+					 bool email_user = false,
 					 bool write_to_user_log = true);
 extern bool setJobFactoryPauseAndLog(JobQueueCluster * cluster, int pause_mode, int hold_code, const std::string& reason);
-
+extern bool locate_and_advertise_local_credd(bool force);
 
 /** Hook to call whenever we're going to give a job to a "job
 	handler", be that a shadow, starter (local univ), or gridmanager.
