@@ -15,17 +15,21 @@
 
 import time
 import logging
-import htcondor
+import htcondor2 as htcondor
+import classad2 as classad
 import traceback
 
 from adstash.ad_sources.generic import GenericAdSource
-from adstash.convert import to_json, unique_doc_id
+from adstash.convert import to_json, unique_doc_id, REQUIRED_ATTRS
 
 
 class StartdHistorySource(GenericAdSource):
 
 
-    def fetch_ads(self, startd_ad, max_ads=10000):
+    def fetch_ads(self, startd_ad, max_ads=10000, projection=set()):
+        if projection:  # If user has defined a projection, make sure it contains required attrs
+            projection = projection | REQUIRED_ATTRS
+
         history_kwargs = {}
         if max_ads > 0:
             history_kwargs["match"] = max_ads
@@ -35,10 +39,10 @@ class StartdHistorySource(GenericAdSource):
             logging.warning(f"No checkpoint found for startd {startd_ad['Machine']}, getting all ads available.")
         else:
             since_expr = f"""(GlobalJobId == "{ckpt["GlobalJobId"]}") && (EnteredCurrentStatus == {ckpt["EnteredCurrentStatus"]})"""
-            history_kwargs["since"] = since_expr
+            history_kwargs["since"] = classad.ExprTree(since_expr)
             logging.warning(f"Getting ads from {startd_ad['Machine']} since {since_expr}.")
         startd = htcondor.Startd(startd_ad)
-        return startd.history(requirements=True, projection=[], **history_kwargs)
+        return startd.history(constraint=True, projection=list(projection), **history_kwargs)
 
 
     def process_ads(self, interface, ads, startd_ad, metadata={}, chunk_size=0, **kwargs):
@@ -50,7 +54,7 @@ class StartdHistorySource(GenericAdSource):
             try:
                 dict_ad = to_json(ad, return_dict=True)
             except Exception as e:
-                message = f"Failure when converting document in {startd_ad['name']} history: {str(e)}"
+                message = f"Failure when converting document in {startd_ad['Machine']} history: {str(e)}"
                 exc = traceback.format_exc()
                 message += f"\n{exc}"
                 logging.warning(message)
@@ -61,23 +65,23 @@ class StartdHistorySource(GenericAdSource):
             # Here, we assume that the interface is responsible for de-duping ads
             # and only update the checkpoint after the full history queue is pushed
             # through by returning the new checkpoint at the end.
-            
+
             if startd_checkpoint is None:  # set checkpoint based on first parseable ad
                 startd_checkpoint = {"GlobalJobId": ad["GlobalJobId"], "EnteredCurrentStatus": ad["EnteredCurrentStatus"]}
             chunk.append((unique_doc_id(dict_ad), dict_ad,))
 
             if (chunk_size > 0) and (len(chunk) >= chunk_size):
-                logging.debug(f"Posting {len(chunk)} ads from {startd_ad['Name']}.")
+                logging.debug(f"Posting {len(chunk)} ads from {startd_ad['Machine']}.")
                 result = interface.post_ads(chunk, metadata=metadata, **kwargs)
                 ads_posted += result["success"]
                 yield None  # don't update checkpoint yet, per note above
                 chunk = []
 
         if len(chunk) > 0:
-            logging.debug(f"Posting {len(chunk)} ads from {startd_ad['Name']}.")
+            logging.debug(f"Posting {len(chunk)} ads from {startd_ad['Machine']}.")
             result = interface.post_ads(chunk, metadata=metadata, **kwargs)
             ads_posted += result["success"]
-        
+
         endtime = time.time()
         logging.warning(f"Startd {startd_ad['Machine']} history: response count: {ads_posted}; upload time: {(endtime-starttime)/60:.2f} min")
         yield startd_checkpoint  # finally update checkpoint
