@@ -15,7 +15,7 @@ from hashlib import sha1
 from base64 import b64encode
 
 import requests
-import classad
+import classad2 as classad
 
 TOKEN_DIR_ENV_NAME = '_CONDOR_CREDS'
 TOKEN_FILE_EXT = '.use'
