@@ -33,9 +33,6 @@
 
 extern BaseShadow *Shadow;
 
-
-namespace {
-
 std::string getCredDir()
 {
 	auto job_ad = Shadow->getJobAd();
@@ -59,9 +56,6 @@ std::string getCredDir()
 	return cred_dir;
 }
 
-}
-
-
 ShadowHookMgr::ShadowHookMgr()
 	: JobHookClientMgr()
 {}
@@ -76,11 +70,12 @@ ShadowHookMgr::~ShadowHookMgr()
 	}
 	{
 		TemporaryPrivSentry sentry(PRIV_ROOT);
-		StatInfo si(cred_dir.c_str());
-		if (si.Error() != SINoFile) {
+		struct stat si{};
+		if (stat(cred_dir.c_str(), &si) == 0) {
 			Directory cred_dirp(cred_dir.c_str());
 			cred_dirp.Remove_Entire_Directory();
 		}
+		rmdir(cred_dir.c_str());
 	}
 }
 
@@ -148,7 +143,7 @@ ShadowHookMgr::tryHookPrepareJob()
 		dprintf(D_ERROR, "ERROR in ShadowHookMgr::tryHookPrepareJob: %s\n",
 			err_msg.c_str());
 		BaseShadow::log_except("Job hook execution failed");
-		Shadow->shutDown(JOB_NOT_STARTED, "Shadow prepare hook failed");
+		Shadow->shutDown(JOB_NOT_STARTED, "Shadow prepare hook failed", CONDOR_HOLD_CODE::HookShadowPrepareJobFailure);
 	}
 
 	dprintf(D_ALWAYS, "%s (%s) invoked.\n", hook_name, m_hook_prepare_job.c_str());
@@ -172,8 +167,8 @@ HookShadowPrepareJobClient::hookExited(int exit_status) {
 	}
 	{
 		TemporaryPrivSentry sentry(PRIV_ROOT);
-		StatInfo si(cred_dir.c_str());
-		if (si.Error() != SINoFile) {
+		struct stat si{};
+		if (stat(cred_dir.c_str(), &si) == 0) {
 			Directory cred_dirp(cred_dir.c_str());
 			cred_dirp.Remove_Entire_Directory();
 		}
@@ -221,8 +216,9 @@ HookShadowPrepareJobClient::hookExited(int exit_status) {
 		if (exit_status < 300) {
 			Shadow->holdJobAndExit(log_msg.c_str(), CONDOR_HOLD_CODE::HookShadowPrepareJobFailure, exit_status);
 		} else {
+			// Abort launch, but don't put job on hold
 			BaseShadow::log_except(log_msg.c_str());
-			Shadow->shutDown(JOB_NOT_STARTED, "Shadow prepare hook failed");
+			Shadow->shutDown(JOB_NOT_STARTED, "Shadow prepare hook failed", CONDOR_HOLD_CODE::HookShadowPrepareJobFailure, -1000);
 		}
 		return;
 	}
