@@ -106,11 +106,11 @@ class RemoteResource : public Service {
 			an ACTIVATE_CLAIM command on it.  The ClaimId, starternum
 			and Job ClassAd are pushed, and the executing host's 
 			full machine name and (hopefully) an OK are returned.
-			@param starterVersion The version number of the starter
-                   wanted. The default is 2.
+			@param refuse_code   returns the CONDOR_HOLD_CODE that indicates the reason for refusal
+			@param refuse_reason returns a string giving more detail about the refuse reason
 			@return true on success, false on failure
 		 */ 
-	bool activateClaim( int starterVersion = 2 );
+	bool activateClaim(int & refuse_code, std::string & refuse_reason);
 
 		/** Tell the remote starter to kill itself.
 			@param graceful Should we do a graceful or fast shutdown?
@@ -261,15 +261,17 @@ class RemoteResource : public Service {
 	void setMachineName( const char *machineName );
 
 		/// The number of bytes sent to this resource.
-	float bytesSent() const;
-		
+	uint64_t bytesSent() const;
+
 		/// The number of bytes received from this resource.
-	float bytesReceived() const;
+	uint64_t bytesReceived() const;
 
 	void getFileTransferStatus(FileTransferStatus &upload_status,FileTransferStatus &download_status) const;
 
 	FileTransfer filetrans;
+	FileTransfer::FileTransferInfo upload_transfer_info;
 	FileTransferStatus m_upload_xfer_status;
+	FileTransfer::FileTransferInfo download_transfer_info;
 	FileTransferStatus m_download_xfer_status;
 	ClassAd m_upload_file_stats;
 	ClassAd m_download_file_stats;
@@ -280,6 +282,9 @@ class RemoteResource : public Service {
 	virtual void updateFromStarter( ClassAd* update_ad );
 	virtual void incrementJobCompletionCount();
 
+	virtual void processResultAd( ClassAd * resultAd );
+	virtual void processInvocationAd( ClassAd * invocationAd );
+
 	int64_t getImageSize( int64_t & memory_usage_out, int64_t & rss, int64_t & pss  ) const { 
 		memory_usage_out = memory_usage_mb;
 		rss = remote_rusage.ru_maxrss;
@@ -341,7 +346,7 @@ class RemoteResource : public Service {
 		/** Return true if we received a job_exit syscall from the
 			starter, false if not.
 		*/
-	bool gotJobExit() const { return m_got_job_exit; };
+	bool gotJobDone() const { return m_got_job_done; };
 
 		/** If the job on this resource exited with a signal, return
 			the signal.  If not, return -1. */
@@ -368,7 +373,7 @@ class RemoteResource : public Service {
 	virtual void attemptReconnect( int timerID = -1 );
 
 		/// If this resource has a lease, how much time until it expires?
-	virtual int remainingLeaseDuration( void );
+	virtual time_t remainingLeaseDuration( void );
 
 		/** Check if the X509 has been updated, if so upload it to the shadow
 
@@ -409,10 +414,11 @@ class RemoteResource : public Service {
 	// return true if job should be allowed to write to attribute
 	bool allowRemoteWriteAttributeAccess( const std::string & name );
 
-    void recordActivationExitExecutionTime( time_t when );
+	void recordActivationExitExecutionTime( time_t when );
 
 	void setWaitOnKillFailure(bool wait) { m_wait_on_kill_failure = wait; };
 
+	std::string starter_version;
  protected:
 
 		/** The jobAd for this resource.  Why is this here and not
@@ -436,6 +442,7 @@ class RemoteResource : public Service {
 	bool m_want_remote_updates;
 	bool m_want_streaming_io;
 	bool m_want_delayed;
+	bool m_use_delayed_attr{true};
 	std::vector<std::string> m_delayed_update_prefix;
 	classad::References m_unsettable_attrs;
 
@@ -539,7 +546,7 @@ private:
 
 	bool already_killed_graceful;
 	bool already_killed_fast;
-	bool m_got_job_exit;
+	bool m_got_job_done; // set by the mis-named job_exit syscall, it means the starter is about to exit
 
 	bool m_wait_on_kill_failure;
 
@@ -551,8 +558,13 @@ private:
 	void attemptShutdownTimeout( int timerID = -1 );
 	void attemptShutdown();
 	int transferStatusUpdateCallback(FileTransfer *transobject);
+
+	bool doneInitFileTransfer {false};
 };
 
+// Refactored out of initFileTransfer() so we have a chance of checking
+// the input files we'll actually be sending.
+void modifyFileTransferObject( FileTransfer & filetrans, ClassAd * jobAd );
 
 #endif
 
