@@ -32,14 +32,30 @@
 #include "spool_version.h"
 #include "file_transfer.h"
 #include "condor_holdcodes.h"
+#include "job_ad_instance_recording.h"
 
 BaseShadow *Shadow = NULL;
 
+class InvokeTheGlobalDestructor {
+	public:
+		InvokeTheGlobalDestructor(BaseShadow * & bsp) : ptr(bsp) { }
+		~InvokeTheGlobalDestructor() {
+			if( ptr != NULL ) {
+				dprintf( D_ZKM, "Calling ~BaseShadow()...\n" );
+				delete ptr;
+			}
+		}
+	private:
+		BaseShadow * & ptr;
+};
+InvokeTheGlobalDestructor itgd(Shadow);
+
 // settings we're given on the command-line
 static const char* schedd_addr = NULL;
 const char* public_schedd_addr = NULL;
 static const char* job_ad_file = NULL;
 static bool is_reconnect = false;
+bool use_guidance_in_job_ad = false;
 static int cluster = -1;
 static int proc = -1;
 static const char * xfer_queue_contact_info = NULL;
@@ -109,6 +125,11 @@ parseArgs( int argc, char *argv[] )
 			continue;
 		}
 
+		if( strcmp(opt, "--use-guidance-in-job-ad") == 0 ) {
+			use_guidance_in_job_ad = true;
+			continue;
+		}
+
 		if (strncmp(opt, "--schedd", 8) == 0) {
 			char *ptr = strchr(opt, '<');
 			if (ptr && is_valid_sinful(ptr)) {
@@ -172,8 +193,9 @@ readJobAd( void )
 		if (fp == NULL) {
 			fp = safe_fopen_wrapper_follow( job_ad_file, "r" );
 			if( ! fp ) {
-				EXCEPT( "Failed to open ClassAd file (%s): %s (errno %d)",
-						job_ad_file, strerror(errno), errno );
+				dprintf(D_ERROR, "Failed to open ClassAd file (%s): %s (errno %d)\n",
+				        job_ad_file, strerror(errno), errno);
+				return nullptr;
 			}
 		}
 	}
@@ -195,7 +217,9 @@ readJobAd( void )
 			break;
 		}
         if( ! ad->Insert(line) ) {
-			EXCEPT( "Failed to insert \"%s\" into ClassAd!", line.c_str() );
+			dprintf(D_ERROR, "Failed to insert \"%s\" into ClassAd!\n", line.c_str());
+			delete ad;
+			return nullptr;
         }
     }
 
@@ -209,8 +233,10 @@ readJobAd( void )
 	}
 
 	if( ! read_something ) {
-		EXCEPT( "reading ClassAd from (%s): file is empty",
-				is_stdin ? "STDIN" : job_ad_file );
+		dprintf(D_ERROR, "reading ClassAd from (%s): file is empty",
+		        is_stdin ? "STDIN" : job_ad_file);
+		delete ad;
+		return nullptr;
 	}
 	if( IsDebugVerbose(D_JOB) ) {
 		dPrintAd( D_JOB, *ad );
@@ -293,6 +319,32 @@ void startShadow( ClassAd *ad )
 
 	initShadow( ad );
 
+	// Process configuration for writing epoch history start ClassAd (i.e. historical SPAWN ad)
+	classad::References filter;
+	bool do_filter = true;
+	std::string spawn_ad_filter;
+
+	param(spawn_ad_filter, "SPAWN_JOB_ATTRS");
+
+	if (istring_view(spawn_ad_filter.c_str()) == "all") {
+		// Configured to write the full job ad to epoch history at start time
+		do_filter = false;
+	} else {
+		// Base attributes needed for 'SPAWN' ad in epoch history
+		filter.insert(ATTR_CLUSTER_ID);
+		filter.insert(ATTR_PROC_ID);
+		filter.insert(ATTR_NUM_SHADOW_STARTS);
+		filter.insert(ATTR_OWNER);
+		filter.insert(ATTR_SHADOW_BIRTHDATE);
+		filter.insert(ATTR_RUN_INSTANCE_ID);
+		filter.insert(ATTR_EPOCH_AD_TYPE);
+		// Configured attributes for 'SPAWN' ad in epoch history
+		for (auto &attr : StringTokenIterator(spawn_ad_filter)) { filter.insert(attr); }
+	}
+
+	// Generate Spawn ClassAd to write to epoch history
+	writeAdProjectionToEpoch(ad, (do_filter ? &filter : nullptr), "SPAWN");
+
 	bool wantClaiming = false;
 	ad->LookupBool(ATTR_CLAIM_STARTD, wantClaiming);
 
@@ -466,7 +518,6 @@ printClassAd( void )
 	printf( "%s = True\n", ATTR_IS_DAEMON_CORE );
 	printf( "%s = True\n", ATTR_HAS_FILE_TRANSFER );
 	printf( "%s = True\n", ATTR_HAS_PER_FILE_ENCRYPTION );
-	printf( "%s = True\n", ATTR_HAS_MPI );
 	printf( "%s = True\n", ATTR_HAS_JAVA );
 	printf( "%s = True\n", ATTR_HAS_RECONNECT );
 	printf( "%s = True\n", ATTR_HAS_JOB_AD_FROM_FILE );
@@ -524,6 +575,13 @@ recycleShadow(int previous_job_exit_reason)
 		new_job_ad = readJobAd();
 	}
 
+	// Make sure to trigger the cfLock destructor even if we're not
+	// re-using this shadow.
+	delete Shadow;
+	Shadow = NULL;
+	is_reconnect = false;
+	BaseShadow::myshadow_ptr = NULL;
+
 	if( !new_job_ad ) {
 		dprintf(D_FULLDEBUG,"No new job found to run under this shadow.\n");
 		return false;
@@ -533,11 +591,6 @@ recycleShadow(int previous_job_exit_reason)
 	new_job_ad->LookupInteger(ATTR_PROC_ID,proc);
 	dprintf(D_ALWAYS,"Switching to new job %d.%d\n",cluster,proc);
 
-	delete Shadow;
-	Shadow = NULL;
-	is_reconnect = false;
-	BaseShadow::myshadow_ptr = NULL;
-
 	startShadow( new_job_ad );
 	return true;
 }
