@@ -35,48 +35,31 @@
 #include <stdint.h>
 #include <string>
 #include <stack>
-using std::string;
+
+// fractional disk is the old way of doing disk provisioning.
+// just in case, we leave this as an ifdef for a bit so we can revert
+// that part of the code without reverting the code that moves the disk
+// partition table into the MachAttribute class and the bug fixes for the DISK param
+// #define PROVISION_FRACTIONAL_DISK 1
+
+enum BuildSlotFailureMode { 
+	AllOrNothing, // do nothing if you can't do everything
+	BestEffort,   // build the slots that fit, then stop
+	Except,       // Legacy behavior, kill the process if you can't comply
+};
+
+// codes for use with CpuAttributes::set_broken
+#define BROKEN_CODE_NO_RES       1   // not enough resources to build the slot
+#define BROKEN_CODE_BIND_FAIL    2   // could not bind enough non-fungible resources
+#define BROKEN_CODE_NO_EXEC_DIR  3   // could not set an Execute dir
+#define BROKEN_CODE_UNCLEAN      4   // could not clean up after job
+#define BROKEN_CODE_UNCLEAN_LV   5   // could not clean up Logical Volume after job
+#define BROKEN_CODE_HUNG_PID     6   // could not delete all job processes after job
+#define BROKEN_CODE_HUNG_CGROUP  7   // could not clean up the cgroup (probably a hung pid)
 
 class VolumeManager;
-
 class Resource;
-
-typedef int amask_t;
-
-const amask_t A_PUBLIC	= 1;
-// no longer used: A_PRIVATE = 2
-const amask_t A_STATIC	= 4;
-const amask_t A_TIMEOUT	= 8;
-const amask_t A_UPDATE	= 16;
-const amask_t A_SHARED	= 32;
-const amask_t A_SUMMED	= 64;
-const amask_t A_EVALUATED = 128; 
-const amask_t A_SHARED_SLOT = 256;
-/*
-  NOTE: We don't want A_EVALUATED in A_ALL, since it's a special bit
-  that only applies to the Resource class, and it shouldn't be set
-  unless we explicity ask for it.
-  Same thing for A_SHARED_SLOT...
-*/
-const amask_t A_ALL	= (A_UPDATE | A_TIMEOUT | A_STATIC | A_SHARED | A_SUMMED);
-/*
-  HOWEVER: We do want to include A_EVALUATED and A_SHARED_SLOT in a
-  single bitmask since there are multiple places in the startd where
-  we really want *everything*, and it's lame to have to keep changing
-  all those call sites whenever we add another special-case bit.
-*/
-//const amask_t A_ALL_PUB	= (A_PUBLIC | A_ALL | A_EVALUATED | A_SHARED_SLOT);
-//const amask_t A_ALL_PUB	= (A_PUBLIC | A_ALL | A_EVALUATED);
-
-#define IS_PUBLIC(mask)		((mask) & A_PUBLIC)
-#define IS_STATIC(mask)		((mask) & A_STATIC)
-#define IS_TIMEOUT(mask)	((mask) & A_TIMEOUT)
-#define IS_UPDATE(mask)		((mask) & A_UPDATE)
-#define IS_SHARED(mask)		((mask) & A_SHARED)
-#define IS_SUMMED(mask)		((mask) & A_SUMMED)
-#define IS_EVALUATED(mask)	((mask) & A_EVALUATED)
-#define IS_SHARED_SLOT(mask) ((mask) & A_SHARED_SLOT)
-#define IS_ALL(mask)		((mask) & A_ALL)
+class CpuAttributes;
 
 // we cast share to int before comparison, because it might be a float
 // and we don't want the compiler to complain
@@ -182,12 +165,16 @@ typedef struct _AttribValue {
 } AttribValue;
 
 //
-class FullSlotId
+class NFROwner
 {
-public: 
-	int id;   // static or partitionable id
-	int dyn_id; // dynamic id
-	FullSlotId(int i, int j=0) : id(i), dyn_id(j) {};
+public:
+	unsigned short int id{0};     // static or partitionable id slot id
+	unsigned short int dyn_id{0}; // dynamic id, dslot_id or broken id
+	unsigned short int group{0};  // if non-zero the resource is shared by a group of slots
+	unsigned short int active:1{0}; // non-zero if resource is owned by an active slot (set only when dyn_id is > 0)
+	NFROwner(int i, int j=0) : id(i), dyn_id(j) {};
+
+	bool empty() { return id == 0 && dyn_id == 0; }
 };
 
 class NonFungibleRes
@@ -195,17 +182,18 @@ class NonFungibleRes
 public:
 	NonFungibleRes(const std::string & _id) : id(_id), owner(0), bkowner(0) {}
 	std::string id;
-	FullSlotId  owner;
-	FullSlotId  bkowner;
+	NFROwner  owner;
+	NFROwner  bkowner;
 
 	// report if resource is "owned" by the given slot
 	// if the given slot matches the primary owner, then return 1
 	// if the given slot matches the backfill owner and the primary owner is not set return 2
+	// if owned by both backfill and an active primary, return 3
 	int is_owned(int slot_id, int sub_id) const {
 		if (owner.id == slot_id && owner.dyn_id == sub_id) return 1;
 		if (bkowner.id == slot_id && bkowner.dyn_id == sub_id) {
-			// TODO: handle the case where the primary owner is a claimed static slot?
-			if (owner.dyn_id > 0) return 3;
+			// if the NFR is backfill owned by the given slot, and *also* by any active slot
+			if (owner.dyn_id > 0 || owner.active) return 3;
 			return 2;
 		}
 		return 0;
@@ -224,12 +212,12 @@ class MachAttributes
 {
 public:
 	// quantity (double) of resource for each resource tag
-	typedef std::map<string, double, classad::CaseIgnLTStr> slotres_map_t;
+	typedef std::map<std::string, double, classad::CaseIgnLTStr> slotres_map_t;
 
 	// these are used for non-fungible ids to track resource IDs that cannot be assigned
 	// for each resource tag, which resource IDs are offline. note that this is a SET rather than a VECTOR
 	typedef std::set<std::string> slotres_offline_ids_t;
-	typedef std::map<string, slotres_offline_ids_t, classad::CaseIgnLTStr> slotres_offline_ids_map_t;
+	typedef std::map<std::string, slotres_offline_ids_t, classad::CaseIgnLTStr> slotres_offline_ids_map_t;
 
 	// some slots may have constraints on which non-fungible resources they can use, this is handled
 	// by having an optional property classad for each unique non-fungible id, and a optional constraint
@@ -239,7 +227,7 @@ public:
 	typedef std::map<std::string, NonFungibleType, classad::CaseIgnLTStr> slotres_nft_map_t;
 	// these are used as lists of non-fungible ids for various purposes
 	typedef std::vector<std::string> slotres_assigned_ids_t;
-	typedef std::map<string, slotres_assigned_ids_t, classad::CaseIgnLTStr> slotres_devIds_map_t;
+	typedef std::map<std::string, slotres_assigned_ids_t, classad::CaseIgnLTStr> slotres_devIds_map_t;
 
 	MachAttributes();
 	~MachAttributes();
@@ -249,7 +237,7 @@ public:
                                // creating data structure needed in compute and publish
     void init_machine_resources();
 
-	void publish_static(ClassAd*);     // things that can only change on reconfig
+	void publish_static(ClassAd*, const CpuAttributes * childAttrs);     // things that can only change on reconfig
 	void publish_common_dynamic(ClassAd*, bool global=false); // things that can change at runtime
 	void publish_slot_dynamic(ClassAd*, int slotid, int slotsubid, bool backfill, const std::string & res_conflict); // things that can change at runtime
 	void compute_config();      // what compute(A_STATIC | A_SHARED) used to do
@@ -263,7 +251,8 @@ public:
 		}
 	}
 
-	bool has_nft_conflicts(int slotid, int slotsubid);
+	void set_nft_activity(std::map<int,bool> &active_slotids_map);
+	bool has_nft_conflicts(int slotid, int slotsubid) const;
 
 		// Initiate benchmark computations benchmarks on the given resource
 	void start_benchmarks( Resource*, int &count );	
@@ -284,32 +273,90 @@ public:
 	double			num_cpus()	const { return m_num_cpus; };
 	double			num_real_cpus()	const { return m_num_real_cpus; };
 	int				phys_mem()	const { return m_phys_mem; };
-	long long		virt_mem()	const { return m_virt_mem; };
+#ifdef PROVISION_FRACTIONAL_DISK
 	long long		total_disk() const { return m_total_disk; }
+	long long		virt_mem()	const { return m_virt_mem; };
 	bool			always_recompute_disk() const { return m_always_recompute_disk; }
-	double		load()			const { return m_load; };
-	double		condor_load()	const { return m_condor_load; };
-	time_t		keyboard_idle() const { return m_idle; };
-	time_t		console_idle()	const { return m_console_idle; };
+	bool			is_fractional_disk() const { return true; }
+#else
+	struct _disk_volume_res {
+		long long free_disk{0};         // free disk space on volume
+		long long detected_disk{0};     // total disk space on volume
+		long long non_condor_disk{0};   // in-use disk space on volume at startup
+		long long provisioned_disk{0};  // total disk provisioned to all slots from this volume
+	};
+	const std::map<std::string, _disk_volume_res> & disk_volumes() const { return m_disk_volumes; }
+	bool has_disk_volume(const char * volume_id) const { return m_disk_volumes.count(volume_id); }
+	void add_disk_volume(const char * volume_id,
+		long long free_disk,
+		long long detected_disk,
+		long long non_condor_disk)
+	{
+		auto & volume = m_disk_volumes[volume_id];
+		volume.free_disk = free_disk;
+		volume.detected_disk = detected_disk;
+		volume.non_condor_disk = non_condor_disk;
+	}
+	void update_provisioned_disk_for_volume(const char * volume_id, long long provisioned_disk) {
+		auto it = m_disk_volumes.find(volume_id);
+		if (it != m_disk_volumes.end()) {
+			it->second.provisioned_disk = std::max(it->second.provisioned_disk, provisioned_disk);
+		}
+	}
+	long long		free_disk(const char * volume_id) const {
+		auto it = m_disk_volumes.find(volume_id);
+		if (it != m_disk_volumes.end()) return it->second.free_disk;
+		return 0;
+	}
+	long long		detected_disk(const char * volume_id) const {
+		auto it = m_disk_volumes.find(volume_id);
+		if (it != m_disk_volumes.end()) {
+			return it->second.detected_disk;
+		}
+		return 0;
+	}
+	long long		provisioned_disk(const char * volume_id) const {
+		auto it = m_disk_volumes.find(volume_id);
+		if (it != m_disk_volumes.end()) return it->second.provisioned_disk;
+		return 0;
+	}
+	long long	num_swap()	const { return m_num_swap; };
+	bool		is_fractional_disk() const { return false; }
+	bool		recompute_disk_free() const { return m_recompute_disk_free; } // old Desktop policy where Disk is free space
+#endif
+	bool		total_disk_from_config() const { return m_total_disk_from_config; } // Disk specified in config
+	bool		using_volume_manager() const { return m_using_volume_manager; }
+	//void		set_using_volume_manager(bool using_it) { m_using_volume_manager = using_it; }
+	double		machine_load()			const { return m_load; };
+	double		machine_condor_load()	const { return m_condor_load; };
+	time_t		machine_keyboard_idle() const { return m_idle; };
+	time_t		machine_console_idle()	const { return m_console_idle; };
 	const slotres_map_t& machres() const { return m_machres_map; }
 	const slotres_nft_map_t& machres_devIds() const { return m_machres_nft_map; }
 	const ClassAd& machres_attrs() const { return m_machres_attr; }
-	const char * AllocateDevId(const std::string & tag, const char* request, int assign_to, int assign_to_sub, bool backfill);
-	bool         ReleaseDynamicDevId(const std::string & tag, const char * id, int was_assign_to, int was_assign_to_sub);
+	const char * AllocateDevId(const std::string & tag, const char* request, int assign_to, int assign_to_sub, bool backfill, int assign_from_sub);
+	bool         ReleaseDynamicDevId(const std::string & tag, const char * id, int was_assign_to, int was_assign_to_sub, int new_sub=0);
 	bool         DevIdMatches(const NonFungibleType & nft, int ixid, ConstraintHolder & require);
 	const char * DumpDevIds(std::string & buf, const char * tag = NULL, const char * sep = "\n");
 	void         ReconfigOfflineDevIds();
 	int          RefreshDevIds(const std::string & tag, slotres_assigned_ids_t & slot_res_devids, int assign_to, int assign_to_sub);
+	int          ReportBrokenDevIds(const std::string & tag, slotres_assigned_ids_t & devids, int broken_sub_id);
 	bool         ComputeDevProps(ClassAd & ad, const std::string & tag, const slotres_assigned_ids_t & ids);
 	//bool ReAssignDevId(const std::string & tag, const char * id, void * was_assigned_to, void * assign_to);
 
+	// return WithinResourceLimits, these also calculates it on the first call
+	const char * withinLimitsExpression(); // regular WithinResourceLimits
+	const char * consumptionLimitsExpression(); // consumption policy variant
+
+	void 		set_volume_manager(VolumeManager* volman);
+	VolumeManager*	get_volume_manager() const { return m_volume_manager_ref; }
+
 private:
 
 		// Dynamic info
 	double			m_load;
 	double			m_condor_load;
 	double			m_owner_load;
-	long long		m_virt_mem;
 	time_t			m_idle;
 	time_t			m_console_idle;
 	int				m_mips;
@@ -330,8 +377,18 @@ private:
 	double			m_num_cpus;
 	double			m_num_real_cpus;
 	int				m_phys_mem;
+	bool			m_no_job_networking_aware{false}; // include expressions for NO_JOB_NETWORKING
+#ifdef PROVISION_FRACTIONAL_DISK
 	bool			m_always_recompute_disk; // set from STARTD_RECOMPUTE_DISK_FREE knob and DISK knob
+	long long		m_virt_mem;
 	long long		m_total_disk; // the value of total_disk if m_recompute_disk is false
+#else
+	long long		m_num_swap{0};
+	std::map<std::string, _disk_volume_res> m_disk_volumes;
+	bool			m_recompute_disk_free{false}; // set from STARTD_RECOMPUTE_DISK_FREE knob and DISK knob
+#endif
+	bool			m_total_disk_from_config{false}; // set from STARTD_RECOMPUTE_DISK_FREE knob and DISK knob
+	bool			m_using_volume_manager{false};
 	slotres_map_t   m_machres_map;
 	slotres_nft_map_t m_machres_nft_map;
 	slotres_devIds_map_t m_machres_offline_devIds_map; // startup list of offline ids
@@ -346,6 +403,9 @@ private:
 	double init_machine_resource_from_script(const char * tag, const char * script_cmd);
 	ClassAd         m_machres_attr;
 
+	std::string		m_within_limits_expr_str;       // Expression for WithinResourceLimits
+	std::string		m_consumption_limits_expr_str;  // Expression for consumption policy WithinResourceLimits
+
 	char*			m_arch;
 	char*			m_opsys;
 	int 			m_opsysver;
@@ -376,14 +436,18 @@ private:
 	std::vector<std::string> m_user_specified;
 	int             m_user_settings_init;  // set to true when init_user_settings has been called at least once.
 
-	std::string		m_named_chroot;
 #if defined ( WIN32 )
 	int				m_got_windows_version_info;
 	OSVERSIONINFOEX	m_window_version_info;
     char*           m_dot_Net_Versions;
 #endif
+	VolumeManager* m_volume_manager_ref{nullptr}; // Note: We do not own this pointer... only reference
+};
 
-};	
+#ifdef PROVISION_FRACTIONAL_DISK
+#else
+void initExecutePartitionTable(MachAttributes * m_attr, VolumeManager * volman);
+#endif
 
 class ResBag;
 
@@ -403,8 +467,13 @@ public:
 
 	struct _slot_request {
 		double num_cpus=0;
+	#ifdef PROVISION_FRACTIONAL_DISK
 		double virt_mem_fraction=0;
 		double disk_fraction=0;
+	#else
+		long long num_swap=0; // in KB
+		long long num_disk=0; // in KB
+	#endif
 		int num_phys_mem=0;
 		bool allow_fractional_cpus=false;
 		MachAttributes::slotres_map_t slotres;
@@ -412,28 +481,34 @@ public:
 		const char * dump(std::string & buf) const;
 	};
 
-	CpuAttributes( unsigned int slot_type, double num_cpus,
-				   int num_phys_mem, double virt_mem_fraction,
-				   double disk_fraction,
-				   const slotres_map_t& slotres_map,
-				   const slotres_constraint_map_t& slotres_req_map,
-				   const std::string &execute_dir, const std::string &execute_partition_id );
-
 	// init a slot_request from config strings
-	static bool buildSlotRequest(_slot_request & request, MachAttributes *m_attr, const std::string& list, unsigned int type_id, bool except);
+	static bool buildSlotRequest(
+		_slot_request & request,
+		MachAttributes *m_attr,
+		const std::string& list,
+		unsigned int type_id,
+		const char * partition_id,
+		BuildSlotFailureMode failmode);
 
 	// construct from a _slot_request
 	CpuAttributes(unsigned int slot_type,
 		const struct _slot_request & req,
+		const MachAttributes & m_attr,
 		const std::string &execute_dir, const std::string &execute_partition_id )
 		: rip(nullptr)
 		, c_condor_load(-1.0)
 		, c_owner_load(-1.0)
 		, c_idle(-1)
 		, c_console_idle(-1)
+	#ifdef PROVISION_FRACTIONAL_DISK
 		, c_virt_mem(0)
 		, c_disk(0)
-		, c_total_disk(0)
+		, c_total_disk(m_attr.total_disk())
+	#else
+		, c_swap(req.num_swap)
+		, c_disk(req.num_disk)
+		, c_total_disk(m_attr.detected_disk(execute_partition_id.c_str()))
+	#endif
 		, c_phys_mem(req.num_phys_mem)
 		, c_slot_mem(req.num_phys_mem)
 		, c_allow_fractional_cpus(req.allow_fractional_cpus)
@@ -442,24 +517,40 @@ public:
 		, c_slottot_map(req.slotres)
 		, c_slotres_constraint_map(req.slotres_constr)
 		, c_num_slot_cpus(req.num_cpus)
+	#ifdef PROVISION_FRACTIONAL_DISK
 		, c_virt_mem_fraction(req.virt_mem_fraction)
 		, c_disk_fraction(req.disk_fraction)
 		, c_slot_disk(0)
+	#else
+		, c_slot_swap(req.num_swap)
+		, c_slot_disk(req.num_disk)
+	#endif
 		, c_execute_dir(execute_dir)
 		, c_execute_partition_id(execute_partition_id)
 		, c_type_id(slot_type)
+		, c_broken_code(0)
+		, c_broken_reason()
 	{
 	}
 
 	void attach( Resource* );	// Attach to the given Resource
-	bool bind_DevIds(MachAttributes* map, int slot_id, int slot_sub_id, bool backfill_slot, bool abort_on_fail);   // bind non-fungable resource ids to a slot
-	void unbind_DevIds(MachAttributes* map, int slot_id, int slot_sub_id); // release non-fungable resource ids
-	void reconfig_DevIds(MachAttributes* map, int slot_id, int slot_sub_id); // check for offline changes for non-fungible resource ids
+	// bind non-fungable resource ids to a slot
+	bool bind_DevIds(MachAttributes* map, int slot_id, int slot_sub_id, bool backfill_slot, bool abort_on_fail, int donor_sub_id);
+	// release non-fungable resource ids back to parent
+	void unbind_DevIds(MachAttributes* map, int slot_id, int slot_sub_id, int new_sub_id);
+	// check for offline changes for non-fungible resource ids
+	void reconfig_DevIds(MachAttributes* map, int slot_id, int slot_sub_id);
+	// Add broken non-fungible resource ids to this objects mapping
+	void claim_broken_DevIds(MachAttributes* map, int broken_sub_id);
 
-	void publish_static(ClassAd*, const ResBag * inuse) const;  // Publish desired info to given CA
+	void publish_static(ClassAd*, const ResBag * inuse, const ResBag * broken) const;  // Publish desired info to given CA
 	void publish_dynamic(ClassAd*) const;  // Publish desired info to given CA
+#ifdef PROVISION_FRACTIONAL_DISK
 	void compute_virt_mem_share(double virt_mem);
 	void compute_disk();
+#else
+	void recompute_disk(bool init);
+#endif
 	void set_condor_load(double load) { c_condor_load = load; }
 
 		// Load average methods
@@ -483,19 +574,28 @@ public:
 	double num_cpus() const { return c_num_cpus; }
 	bool allow_fractional_cpus(bool allow) { bool old = c_allow_fractional_cpus; c_allow_fractional_cpus = allow; return old; }
 	long long get_disk() const { return c_disk; }
+#ifdef PROVISION_FRACTIONAL_DISK
 	double get_disk_fraction() const { return c_disk_fraction; }
+#endif
+	char const *executeDir() const { return c_execute_dir.c_str(); }
+	char const *executePartitionID() const { return c_execute_partition_id.c_str(); }
 	long long get_total_disk() const { return c_total_disk; }
-	char const *executeDir() { return c_execute_dir.c_str(); }
-	char const *executePartitionID() { return c_execute_partition_id.c_str(); }
-    const slotres_map_t& get_slotres_map() { return c_slotres_map; }
-    const slotres_devIds_map_t & get_slotres_ids_map() { return c_slotres_ids_map; }
-
 	void init_total_disk(const CpuAttributes* r_attr) {
 		if (r_attr && (r_attr->c_execute_partition_id == c_execute_partition_id)) {
 			c_total_disk = r_attr->c_total_disk;
 		}
 	}
+	void init_total_disk(long long total) { c_total_disk = total; }
+#ifdef PROVISION_FRACTIONAL_DISK
 	bool set_total_disk(long long total, bool refresh, VolumeManager * volman);
+#endif
+
+	const slotres_map_t& get_slotres_map() { return c_slotres_map; }
+	const slotres_devIds_map_t & get_slotres_ids_map() { return c_slotres_ids_map; }
+
+	void set_broken(int code, std::string_view reason) { c_broken_code = code; c_broken_reason = reason; }
+	unsigned int is_broken(std::string * reason=nullptr) const { if (reason) *reason = c_broken_reason; return c_broken_code; }
+	const char * broken_reason() { return c_broken_reason.c_str(); }
 
 	CpuAttributes& operator+=( CpuAttributes& rhs);
 	CpuAttributes& operator-=( CpuAttributes& rhs);
@@ -508,7 +608,11 @@ private:
 	double			c_owner_load;
 	time_t			c_idle;
 	time_t			c_console_idle;
+#ifdef PROVISION_FRACTIONAL_DISK
 	long long c_virt_mem;
+#else
+	long long c_swap;
+#endif
 	long long c_disk;
 		// total_disk here means total disk space on the partition containing
 		// this slot's execute directory.  Since each slot may have an
@@ -533,14 +637,24 @@ private:
 
 		// These hold the fractions of shared, dynamic resources
 		// that are allocated to this CPU.
+#ifdef PROVISION_FRACTIONAL_DISK
 	double			c_virt_mem_fraction;
 
 	double			c_disk_fraction; // share of execute dir partition
 	double			c_slot_disk; // share of execute dir partition
+#else
+	long long		c_slot_swap;
+	long long		c_slot_disk;
+#endif
 	std::string     c_execute_dir;
 	std::string     c_execute_partition_id;  // unique id for partition
 
 	unsigned int	c_type_id;		// The slot type of this resource
+
+		// resource brokenness, set when the resource could not be provisioned
+		// or when one of the provisioned resources is not working
+	unsigned int    c_broken_code;
+	std::string     c_broken_reason;
 };	
 
 // a loose bag of resource quantities, for doing resource math
@@ -559,34 +673,29 @@ public:
 	ResBag& operator+=(const CpuAttributes& rhs);
 	ResBag& operator-=(const CpuAttributes& rhs);
 
+	ResBag& operator+=(const ResBag& rhs);
+	ResBag& operator-=(const ResBag& rhs);
+
+	bool empty() const {return (cpus<=0) && !disk && !mem && resmap.empty();}
 	void reset();
-	bool underrun(std::string * names);
-	bool excess(std::string * names);
+	bool underrun(std::string * names) const;
+	bool excess(std::string * names) const;
 	void clear_underrun(); // reset only the underrun values
 	void clear_excess();   // reset only the excess values
 	const char * dump(std::string & buf) const;
 	void Publish(ClassAd& ad, const char * prefix) const;
+	const MachAttributes::slotres_map_t & nfrmap() const { return resmap; }
+	void convert_to_request(CpuAttributes::_slot_request& req) const;
 
 protected:
-	double     cpus = 0;
-	long long  disk = 0;
-	int        mem = 0;
-	int        slots = 0;
+	double     cpus{0};
+	long long  disk{0};
+	int        mem{0};
+	int        slots{0};
 	MachAttributes::slotres_map_t resmap;
 	friend class CpuAttributes;
 };
 
-class AvailDiskPartition
-{
- public:
-	AvailDiskPartition() {
-		m_disk_fraction = 1.0;
-		m_auto_count = 0;
-	}
-	double m_disk_fraction; // share of this partition that is not taken yet
-	int m_auto_count; // number of slots using "auto" share of this partition
-};
-
 // Available machine-wide attributes
 class AvailAttributes
 {
@@ -595,26 +704,46 @@ public:
 
 	AvailAttributes( MachAttributes* map );
 
+#ifdef PROVISION_FRACTIONAL_DISK
+	void init_partition(const char * id, long long) { m_partitions[id].m_disk_fraction = 1.0; }
+#else
+	void init_partition(const char * id, long long disk) { m_partitions[id].m_disk = disk; }
+#endif
+
 	bool decrement( CpuAttributes* cap );
 	bool computeRemainder(slotres_map_t & remain_cap, slotres_map_t & remain_cnt);
 	bool computeAutoShares( CpuAttributes* cap, slotres_map_t & remain_cap, slotres_map_t & remain_cnt);
 	void cat_totals(std::string & buf, const char * execute_partition_id);
+	// reduce request as necessary to force a fit, and report the reductions in the unfit string
+	void trim_request_to_fit( CpuAttributes::_slot_request & req, const char * execute_partition_id, std::string & unfit );
+	void update_provisioned_disk( MachAttributes * map );
 
 private:
 	int				a_num_cpus;
 	int             a_num_cpus_auto_count; // number of slots specifying "auto"
 	int				a_phys_mem;
 	int             a_phys_mem_auto_count; // number of slots specifying "auto"
+#ifdef PROVISION_FRACTIONAL_DISK
 	float			a_virt_mem_fraction;
 	int             a_virt_mem_auto_count; // number of slots specifying "auto"
+#else
+	long long		a_num_swap{0};
+	int             a_swap_auto_count{0}; // number of slots specifying "auto"
+#endif
 
     slotres_map_t a_slotres_map;
     slotres_map_t a_autocnt_map;
 
-		// number of slots using "auto" for disk share in each partition
-	HashTable<std::string,AvailDiskPartition> m_execute_partitions;
 
-	AvailDiskPartition &GetAvailDiskPartition(std::string const &execute_partition_id);
+	struct _avail_disk_partition {
+	#ifdef PROVISION_FRACTIONAL_DISK
+		double m_disk_fraction{1.0}; // share of this partition that is not taken yet
+	#else
+		long long m_disk{0}; // disk KB not taken yet
+	#endif
+		int m_auto_count{0}; // number of slots using "auto" share of this partition
+	};
+	std::map<std::string, _avail_disk_partition> m_partitions;
 };
 
 #endif /* _RES_ATTRIBUTES_H */
