@@ -32,7 +32,9 @@
 #include <set>
 
 #define USE_STARTD_LATCHES 1
-#define DO_BULK_COLLECTOR_UPDATES 1
+
+enum class ResourceLockType;
+class BrokenItem;
 
 class SlotType
 {
@@ -92,41 +94,44 @@ struct AttrLatchLTStr {
 class Resource : public Service
 {
 public:
-	Resource( CpuAttributes*, int id, Resource* _parent = NULL, bool take_parent_claim = false);
+	Resource( CpuAttributes*, int id, Resource* _donor = nullptr, bool take_donor_claim = false);
 	~Resource();
 
 		// override param by slot_type
-	char * param(const char * name);
-	const char * param(std::string& out, const char * name);
-	const char * param(std::string& out, const char * name, const char * def);
+	char * param(const char * name) const;
+	const char * param(std::string& out, const char * name) const;
+	const char * param(std::string& out, const char * name, const char * def) const;
 
 		// Public methods that can be called from command handlers
-	int		retire_claim( bool reversible=false );	// Gracefully finish job and release claim
-	int		release_claim( void );	// Send softkill to starter; release claim
-	int		kill_claim( void );		// Quickly kill starter and release claim
+	int		retire_claim(bool reversible, const std::string& reason, int code, int subcode);	// Gracefully finish job and release claim
+	int		release_claim(const std::string& reason, int code, int subcode);	// Send softkill to starter; release claim
+	int		kill_claim(const std::string& reason, int code, int subcode);		// Quickly kill starter and release claim
 	int		got_alive( void );		// You got a keep alive command
 	int 	periodic_checkpoint( void );	// Do a periodic checkpoint
 	int 	suspend_claim(); // suspend the claim
 	int 	continue_claim(); // continue the claim
 
-		// Multi-shadow wants to run more processes.  Send a SIGHUP to
-		// the starter
-	int		request_new_proc( void );
-
 		// Gracefully kill starter but keep claim	
 	int		deactivate_claim( void );	
 
 		// Quickly kill starter but keep claim
 	int		deactivate_claim_forcibly( void );
 
+		// Job is done, wait for starter to exit, keep the claim
+	int		deactivate_claim_job_done( Stream*, bool claim_closing );
+
 		// Tell the starter to put the job on hold
 	void hold_job(bool soft);
 
+	void setVacateReason(const std::string reason, int code, int subcode);
+
+#if 0
 		// True if no more jobs will be accepted on the current claim.
 	bool curClaimIsClosing();
+#endif
 
 		// True if this slot is draining
-	bool isDraining();
+	bool isDraining() const;
 
 		// Remove the given claim from this Resource
 	void	removeClaim( Claim* );
@@ -135,18 +140,18 @@ public:
 
 		// Shutdown methods that deal w/ opportunistic *and* COD claims
 		// reversible: if true, claim may unretire
-	void	shutdownAllClaims( bool graceful, bool reversible=false );
+	void	shutdownAllClaims(bool graceful, bool reversible, const std::string& reason, int code, int subcode);
 
 	void	dropAdInLogFile( void );
 
 	void	setBadputCausedByDraining() { if (r_cur) r_cur->setBadputCausedByDraining(); }
-        bool	getBadputCausedByDraining() { return r_cur->getBadputCausedByDraining();}
+	bool	getBadputCausedByDraining() { return r_cur->getBadputCausedByDraining();}
 
-        void	setBadputCausedByPreemption() { if( r_cur ) r_cur->setBadputCausedByPreemption();}
-        bool	getBadputCausedByPreemption() { return r_cur->getBadputCausedByPreemption();}
+	void	setBadputCausedByPreemption() { if( r_cur ) r_cur->setBadputCausedByPreemption();}
+	bool	getBadputCausedByPreemption() { return r_cur->getBadputCausedByPreemption();}
 
         // Enable/Disable claims for hibernation
-    void    disable ();
+    void    disable(const std::string& reason, int code, int subcode);
     void    enable ();
 
     // Resource state methods
@@ -170,7 +175,8 @@ public:
 	bool	hasOppClaim( void );
 	bool	hasAnyClaim( void );
 	bool	isDeactivating( void )	{return r_cur->isDeactivating();};
-	bool	isSuspendedForCOD( void ) {return r_suspended_for_cod;};
+	const char * isDeactivatingReason( void )	{return r_cur->isDeactivatingReason();};
+	bool	isSuspendedForCOD( void ) const {return r_suspended_for_cod;};
 	void	hackLoadForCOD( void );
 
 	void	suspendForCOD( void );
@@ -261,7 +267,7 @@ public:
 		// Return a pointer to the Claim object with the given GlobalJobId
 	Claim*	findClaimByGlobalJobId( const char* id );
 
-	bool	claimIsActive( void ); 
+	bool	claimIsActive( void ) const;
 
 	Claim*	newCODClaim( int lease_duration );
 
@@ -275,6 +281,11 @@ public:
 		// Called when the starter of one of our claims exits
 	void	starterExited( Claim* cur_claim );
 
+		// save context for broken slots, this will take ownership of the job classad
+	const BrokenItem & set_broken_context(const Client* client, std::unique_ptr<ClassAd> & job, ResourceLockType lock);
+	void	remove_broken_context();
+	void	restore_broken_resources(const ResBag& broken, const int sub_id);
+
 		// Since the preempting state is so weird, and when we want to
 		// leave it, we need to decide where we want to go, and we
 		// have to do lots of funky twiddling with our claim objects,
@@ -307,13 +318,13 @@ public:
 	void	publish_slot_config_overrides(ClassAd * cad);
 
 	void	publish_static_resources(ClassAd * cad, const ResBag & inUse) {
-		if (r_backfill_slot && can_create_dslot()) {
+		if (r_backfill_slot && !m_parent) {
 			// deduct in-use resources from the backfill p-slot
 			// if there is more than one backfill p-slot, inuse resources will be overcounted.
 			// TODO: spread out deductions across multiple backfill p-slots and/or static slots?
-			r_attr->publish_static(cad, &inUse);
+			r_attr->publish_static(cad, &inUse, r_lost_child_res);
 		} else {
-			r_attr->publish_static(cad, nullptr);
+			r_attr->publish_static(cad, nullptr, r_lost_child_res);
 		}
 	}
 
@@ -334,12 +345,8 @@ public:
 	} WhyFor;
 	void	update_needed( WhyFor why );// Schedule to update the central manager.
 	void	update_walk_for_timer() { update_needed(wf_doUpdate); } // for use with Walk where arguments are not permitted
-#ifdef DO_BULK_COLLECTOR_UPDATES
 	void	get_update_ads(ClassAd & public_ad, ClassAd & private_ad);
 	unsigned int update_is_needed() { return r_update_is_for; }
-#else
-	void	do_update( int timerID = -1 );			// Actually update the CM
-#endif
 	void    process_update_ad(ClassAd & ad, int snapshot=0); // change the update ad before we send it 
     int     update_with_ack( void );    // Actually update the CM and wait for an ACK, used when hibernating.
 	void	final_update( void );		// Send a final update to the CM
@@ -350,16 +357,16 @@ public:
 	int	    benchmarks_finished( void );
 
  		// Helper functions to evaluate resource expressions
-	int     wants_hold( void );         // Default's to FALSE on undefined
-	int		wants_vacate( void );		// EXCEPT's on undefined
-	int		wants_suspend( void );		// EXCEPT's on undefined
-	int		wants_pckpt( void );		// Defaults to FALSE on undefined
-	int		eval_kill( void );			// EXCEPT's on undefined
-	int		eval_preempt( void );		// EXCEPT's on undefined
-	int		eval_suspend( void );		// EXCEPT's on undefined
-	int		eval_continue( void );		// EXCEPT's on undefined
-	int		eval_is_owner( void );		// EXCEPT's on undefined
-	int		eval_start( void );			// returns -1 on undefined
+	int     wants_hold( void ) const;   // Default's to FALSE on undefined
+	int		wants_vacate( void ) const; // EXCEPT's on undefined
+	int		wants_suspend( void ) const;// EXCEPT's on undefined
+	int		wants_pckpt( void ) const;  // Defaults to FALSE on undefined
+	int		eval_kill( void ) const;    // EXCEPT's on undefined
+	int		eval_preempt( void ) const; // EXCEPT's on undefined
+	int		eval_suspend( void ) const; // EXCEPT's on undefined
+	int		eval_continue( void ) const;// EXCEPT's on undefined
+	int		eval_is_owner( void ) const;// EXCEPT's on undefined
+	int		eval_start( void ) const;   // returns -1 on undefined
 #ifdef USE_STARTD_LATCHES  // more generic mechanism for CpuBusy
 #else
 	int		eval_cpu_busy( void );		// returns FALSE on undefined
@@ -369,15 +376,15 @@ public:
 	const char * analyze_match(std::string & buf, ClassAd* request_ad, bool slot_requirements, bool job_requirements);
 
 #if HAVE_BACKFILL
-	int		eval_start_backfill( void ); 
-	int		eval_evict_backfill( void ); 
+	int		eval_start_backfill( void ) const;
+	int		eval_evict_backfill( void ) const;
 	bool	start_backfill( void );
 	bool	softkill_backfill( void );
 	bool	hardkill_backfill( void );
 #endif /* HAVE_BACKFILL */
 
 #if HAVE_JOB_HOOKS
-	bool	isCurrentlyFetching( void ) { return m_currently_fetching; }
+	bool	isCurrentlyFetching( void ) const { return m_currently_fetching; }
 	void	tryFetchWork( int timerID = -1 );
 	void	createOrUpdateFetchClaim( ClassAd* job_ad, double rank = 0 );
 	bool	spawnFetchedWork( void );
@@ -395,15 +402,33 @@ public:
 	bool	evaluateHibernate( std::string &state ) const;
 #endif /* HAVE_HIBERNATION */
 
-	int     evalMaxVacateTime();
-	bool    claimWorklifeExpired();
-	bool    retirementExpired();
-	int     evalRetirementRemaining();
-	int		mayUnretire( void );
-	bool    inRetirement( void );
-	int		hasPreemptingClaim( void );
+	time_t  evalMaxVacateTime() const;
+	bool    claimWorklifeExpired() const;
+	bool    retirementExpired() const;
+	time_t  evalRetirementRemaining() const;
+	bool	mayUnretire( void ) const;
+	bool    inRetirement( void ) const;
+	bool	hasPreemptingClaim( void ) const;
 	int     preemptWasTrue( void ) const; //PREEMPT was true in current claim
-	void    preemptIsTrue();              //records that PREEMPT was true
+	void    setPreemptIsTrue();           //records that PREEMPT evaluated to True
+	const ExprTree * getDrainingExpr();
+
+	// methods that manipulate the Requirements attributes via the Reqexp struct
+	void 	publish_requirements(ClassAd*);
+	bool	reqexp_restore(); // Restore the original requirements
+	void	reqexp_unavail(const ExprTree * start_expr = nullptr);
+	void	reqexp_config();
+	int get_reqexp_state() { return r_reqexp.rstate; }
+	const char * get_reqexp_state_string() {
+		return r_reqexp.rstate==NORMAL_REQ ? "NORMAL"
+			: (r_reqexp.rstate==COD_REQ ? "COD" 
+			: (r_reqexp.rstate==UNAVAIL_REQ ? "UNAVAIL" : "?"));
+	}
+
+
+private:
+	void	reqexp_set_state(reqexp_state rst);
+public:
 
 		// Data members
 	ResState*		r_state;	// Startd state object, contains state and activity
@@ -425,15 +450,18 @@ public:
 	bool            r_backfill_slot;
 	bool            r_suspended_by_command;	// true when the claim was suspended by a SUSPEND_CLAIM command
 	bool            r_no_collector_updates; // true for HIDDEN slots
+	bool            r_acceptedWhileDraining;// true when the job was started while draining
+	bool            r_do_not_delete{false}; // true when slot is broken and should not be deleted
 
 	CODMgr*			r_cod_mgr;	// Object to manage COD claims
-	Reqexp*			r_reqexp;   // Object for the requirements expression
 	CpuAttributes*	r_attr;		// Attributes of this resource
-	LoadQueue*		r_load_queue;  // Holds 1 minute avg % cpu usage
+	ResBag*			r_lost_child_res{nullptr}; // quantities of resources that were assigned to child slots and then lost or broken
+	LoadQueue		r_load_queue;  // Holds 1 minute avg % cpu usage
 	char*			r_name;		// Name of this resource
+	char*			r_id_str;	// CPU id of this resource (string form)
 	int				r_id;		// CPU id of this resource (int form)
 	int				r_sub_id;	// Sub id of this resource (int form)
-	char*			r_id_str;	// CPU id of this resource (string form)
+	unsigned int		r_broken_id{0}; // Reference ID for broken item slot is associated with
 
 	unsigned int type_id( void ) { return r_attr->type_id(); };
 
@@ -448,22 +476,25 @@ public:
 		STANDARD_SLOT,
 		PARTITIONABLE_SLOT,
 		DYNAMIC_SLOT,
-		BROKEN_SLOT
+		BROKEN_SLOT,   // temporary state between failure to create and delete of slot
 	};
 
 	void	set_feature( ResourceFeature feature ) { m_resource_feature = feature; }
 	ResourceFeature	get_feature( void ) { return m_resource_feature; }
 
+	bool is_static_slot() const { return m_resource_feature == STANDARD_SLOT; }
 	bool is_partitionable_slot() const { return m_resource_feature == PARTITIONABLE_SLOT; }
 	bool is_dynamic_slot() const { return m_resource_feature == DYNAMIC_SLOT; }
 	bool is_broken_slot() const { return m_resource_feature == BROKEN_SLOT; }
-	bool can_create_dslot() const { return is_partitionable_slot(); } // for now, only p-slots are splitable
+	bool can_create_dslot() const { return is_partitionable_slot() || (is_dynamic_slot() && !r_claims.empty()); }
 
-	void set_parent( Resource* rip );
+	void set_parent(Resource* rip);
 	Resource* get_parent() { return m_parent; }
 	void clear_parent(); // disconnect from parent (called on deletion of dynamic slots)
 
-	std::string makeChildClaimIds();
+	Claim* is_split_claim_id(const char * id); // matches one of the slot splitting claim ids in r_claims;
+
+	std::string renderDslotClaimIdsList();
 	void add_dynamic_child(Resource *rip) { m_children.insert(rip); }
 	void remove_dynamic_child(Resource *rip) {m_children.erase(rip); }
 
@@ -474,67 +505,65 @@ public:
 	bool fix_require_tag_expr(const ExprTree * expr, ClassAd * request_ad, std::string & require);
 
 	void set_res_conflict(const std::string & conflict) { m_res_conflict = conflict; }
-	bool has_nft_conflicts(MachAttributes* ma) { return ma->has_nft_conflicts(r_id, r_sub_id); }
+	bool has_nft_conflicts(MachAttributes* ma) const {
+		return ma->has_nft_conflicts(r_id, r_sub_id);
+	}
 
-	bool wasAcceptedWhileDraining() const { return m_acceptedWhileDraining; }
-	void setAcceptedWhileDraining() { m_acceptedWhileDraining = isDraining(); }
+	bool wasAcceptedWhileDraining() const { return r_acceptedWhileDraining; }
+	void setAcceptedWhileDraining() { r_acceptedWhileDraining = isDraining(); }
 private:
+	Reqexp          r_reqexp;   // Object for the requirments expression
 	ResourceFeature m_resource_feature;
-
-	Resource*	m_parent;
+	Resource*       m_parent;
 
 	// Only partitionable slots have children
 	std::set<Resource *, ResourceLess> m_children;
 	// only non-partitionable backfill slots have resource conflicts
 	std::string m_res_conflict;
 
+	std::string m_orig_assigned_gpus; // AssignedGpus at birth, to help debug HTCONDOR-3072
+
 	IdDispenser* m_id_dispenser;
 
-#ifdef DO_BULK_COLLECTOR_UPDATES
 	// bulk updates use a single timer in the ResMgr for updates
 	// and a timestamp of the oldest requested time for the update
 	// along with a bitmask of the reason for the update
 	time_t r_update_is_due = 0;		// 0 for update is not due, otherwise the oldest time it was requested
 	unsigned int r_update_is_for = 0; // mask of WhyFor bits giving reason for update
-#else
-	int			update_tid;	// DaemonCore timer id for update delay
-#endif
 
 #ifdef USE_STARTD_LATCHES  // more generic mechanism for CpuBusy
 #else
 	int		r_cpu_busy;
 	time_t	r_cpu_busy_start_time; // time when cpu changed from busy to non-busy or visa versa
 #endif
-	time_t	r_last_compute_condor_load;
-	bool	r_suspended_for_cod;
-	bool	r_hack_load_for_cod;
-	int		r_cod_load_hack_tid;
+	time_t	r_last_compute_condor_load {0}; 
+	bool	r_suspended_for_cod = false;
+	bool	r_hack_load_for_cod = false;
+	int		r_cod_load_hack_tid = false;
 	void	beginCODLoadHack( void );
-	double	r_pre_cod_total_load;
-	double	r_pre_cod_condor_load;
+	double	r_pre_cod_total_load = 0.0;
+	double	r_pre_cod_condor_load = 0.0;
 	void 	startTimerToEndCODLoadHack();
 	void	endCODLoadHack( int timerID = -1 );
-	int		eval_expr( const char* expr_name, bool fatal, bool check_vanilla );
+	int		eval_expr( const char* expr_name, bool fatal, bool check_vanilla ) const;
 
 	std::string m_execute_dir;
 	std::string m_execute_partition_id;
 
 #if HAVE_JOB_HOOKS
-	time_t	m_last_fetch_work_spawned;
-	time_t	m_last_fetch_work_completed;
+	time_t	m_last_fetch_work_spawned = 0;
+	time_t	m_last_fetch_work_completed = 0;
 	bool	m_currently_fetching;
 	int		m_next_fetch_work_delay;
 	int		m_next_fetch_work_tid;
 	int		evalNextFetchWorkDelay( void );
 	void	createFetchClaim( ClassAd* job_ad, double rank = 0 );
-	void	resetFetchWorkTimer( int next_fetch = 0 );
+	void	resetFetchWorkTimer( time_t next_fetch = 0 );
 	char*	m_hook_keyword;
 	bool	m_hook_keyword_initialized;
 #endif /* HAVE_JOB_HOOKS */
 
 	std::list<int> m_affinity_mask;
-
-	bool	m_acceptedWhileDraining;
 };
 
 
