@@ -77,7 +77,6 @@ int command_release_claim(int, Stream* );
    seperately. 
 */
 int command_vacate_all(int, Stream* );
-int command_pckpt_all(int, Stream* );
 int command_x_event(int, Stream* );
 int	command_give_state(int, Stream* );
 int	command_give_totals_classad( int, Stream* );
@@ -103,10 +102,10 @@ int command_classad_handler(int, Stream* );
   some of the commands. 
 */
 // Negotiator sent match information to us
-int match_info( Resource*, char* );
+int match_info( Resource*, const char* );
 
 // Schedd Agent requests a claim
-int request_claim( Resource*, Claim *, char*, Stream* ); 
+int request_claim( Resource*, Claim *, const char*, Stream* ); 
 
 // Accept claim from schedd agent
 bool accept_request_claim(Claim* claim,
@@ -116,7 +115,7 @@ bool accept_request_claim(Claim* claim,
 	std::vector<Resource*> * dslots = nullptr);
 
 // Activate a claim with a given starter
-int activate_claim( Resource*, Stream* ); 
+int activate_claim( Resource* rip, Stream* stream, ClassAd * request_ad, bool send_failure_ad );
 
 // Start draining jobs
 int command_drain_jobs(int dc_cmd, Stream* s );
