@@ -25,6 +25,30 @@
 #include "condor_daemon_core.h"
 #include "slot_builder.h"
 #include "../condor_sysapi/sysapi.h"
+#include "VolumeManager.h"
+
+bool string_to_BuildSlotFailureMode(const char * str, enum BuildSlotFailureMode & failmode)
+{
+	if (MATCH == strcasecmp("CLEAR", str)) { failmode = BuildSlotFailureMode::AllOrNothing; }
+	else if (MATCH == strcasecmp("CONTINUE", str)) { failmode = BuildSlotFailureMode::BestEffort; }
+	else if (MATCH == strcasecmp("ABORT", str)) { failmode = BuildSlotFailureMode::Except; }
+	else { return false; }
+	return true;
+}
+
+const char * BuildSlotFailureMode_to_string(BuildSlotFailureMode failmode)
+{
+	switch (failmode) {
+	case BuildSlotFailureMode::AllOrNothing: return "CLEAR";
+	case BuildSlotFailureMode::BestEffort: return "CONTINUE";
+	case BuildSlotFailureMode::Except: return "ABORT";
+	}
+	return nullptr;
+}
+
+// ugly, but we can't include util.h without some refactoring, so...
+bool	check_execute_dir_perms(char const *exec_path, bool abort_on_error);
+extern bool execute_dir_checks_out; // set in startd_main for EXECUTE
 
 static void
 _checkInvalidParam( const char* name, bool except ) {
@@ -40,52 +64,141 @@ _checkInvalidParam( const char* name, bool except ) {
 		free(tmp);
 	}
 }
-void GetConfigExecuteDir( int slot_id, std::string &execute_dir, std::string &partition_id )
+
+// param for the execute dir and check to see if it exists, returns empty string on success
+// and error string on failure
+std::string paramExecuteDir( int slot_id, std::string &execute_dir, std::string &param_name )
 {
-	std::string execute_param;
-	char *execute_value = NULL;
-	formatstr(execute_param,"SLOT%d_EXECUTE",slot_id);
-	execute_value = param( execute_param.c_str() );
-	if( !execute_value ) {
-		execute_value = param( "EXECUTE" );
+	std::string ret;
+
+	formatstr(param_name,"SLOT%d_EXECUTE",slot_id);
+	const char * execute_param = param_name.c_str();
+	auto_free_ptr execute_value(param(execute_param));
+	if ( ! execute_value) {
+		param_name = "EXECUTE";
+		execute_value.set(param("EXECUTE"));
 	}
-	if( !execute_value ) {
-		EXCEPT("EXECUTE (or %s) is not defined in the configuration.",
-			   execute_param.c_str());
+	if ( ! execute_value) {
+		formatstr(ret, "EXECUTE (or %s) is not defined in the configuration.", param_name.c_str());
+		return ret;
 	}
 
 #if defined(WIN32)
-	int i;
-		// switch delimiter char in exec path on WIN32
-	for (i=0; execute_value[i]; i++) {
-		if (execute_value[i] == '/') {
-			execute_value[i] = '\\';
-		}
+	// switch delimiter char in exec path on WIN32
+	for (char * ptr = execute_value.ptr(); *ptr; ++ptr) {
+		if (*ptr == '/') { *ptr = '\\'; }
 	}
 #endif
 
-	execute_dir = execute_value;
-	dprintf(D_FULLDEBUG, "Got execute_dir = %s\n",execute_dir.c_str());
-	free( execute_value );
+	execute_dir = execute_value.ptr();
+	return ret;
+}
 
-		// Get a unique identifier for the partition containing the execute dir
-	char *partition_value = NULL;
-	bool partition_rc = sysapi_partition_id( execute_dir.c_str(), &partition_value );
-	if( !partition_rc ) {
+std::string GetExecuteDirPartitionId(const std::string &execute_dir, std::string &partition_id, const char * execute_param)
+{
+	std::string ret;
+	dprintf(D_FULLDEBUG, "Got execute_dir = %s\n",execute_dir.c_str());
+
+	// Get a unique identifier for the partition containing the execute dir
+	auto_free_ptr partition_value;
+	char * ptr = nullptr;
+	bool partition_rc = sysapi_partition_id( execute_dir.c_str(), &ptr );
+	partition_value.set(ptr);
+
+	if( !partition_rc) {
 		struct stat statbuf;
 		if( stat(execute_dir.c_str(), &statbuf)!=0 ) {
 			int stat_errno = errno;
-			EXCEPT("Error accessing execute directory %s specified in the configuration setting %s: (errno=%d) %s",
-				   execute_dir.c_str(), execute_param.c_str(), stat_errno, strerror(stat_errno) );
+			formatstr(ret, "Error accessing execute directory %s specified in the configuration setting %s: (errno=%d) %s",
+				execute_dir.c_str(), execute_param, stat_errno, strerror(stat_errno) );
+			return ret;
 		}
-		EXCEPT("Failed to get partition id for %s=%s",
-			   execute_param.c_str(), execute_dir.c_str());
+		formatstr(ret, "Failed to get partition id for %s=%s", execute_param, execute_dir.c_str());
+		return ret;
 	}
+
+	if ( ! check_execute_dir_perms(execute_dir.c_str(), false)) {
+		formatstr(ret, "Execute directory %s is invalid", execute_dir.c_str());
+	}
+
 	ASSERT( partition_value );
-	partition_id = partition_value;
-	free(partition_value);
+	partition_id = partition_value.ptr();
+	// remove trailing windows path separator if the partition id has one
+	// The classad parser has difficulty with a trailing windows path separator.
+	if ( ! partition_id.empty() && partition_id.back() == '\\') {
+		partition_id.pop_back();
+	}
+	return ret;
 }
 
+#ifdef PROVISION_FRACTIONAL_DISK
+#else
+// called by ResMgr::init_resources during startup to build the initial list of volumes for EXECUTE
+// this iterates the configured slot type definitions in the same way that buildCpuAttrs does
+// to insure it ends up with the same number of slots.
+// TODO: use a class per slot type to hold the initial slot config so this is less fragile
+bool initExecutePartitionTable (
+	MachAttributes *m_attr,
+	int max_types,
+	int num_res,
+	int *type_num_array,
+	bool * /*backfill_types*/,
+	BuildSlotFailureMode failmode,
+	std::vector<bool>& fail_types)
+{
+	// This code only handles the case when we do not have a volume manager
+	ASSERT( ! m_attr->using_volume_manager());
+
+	// And when config has DISK, we behave as if the execute volume does not matter
+	// the "<from-config>" volume is created by the MachAttributes constructor.
+	if (m_attr->total_disk_from_config()) {
+		ASSERT(m_attr->has_disk_volume("<from-config>"));
+		return true;
+	}
+
+	std::map<std::string, std::string> exec_dir_to_partition_id;
+
+	int num = 0;
+	for (int type_id = 0; type_id < max_types; ++type_id) {
+		if (num >= num_res) break;
+
+		for (int j=0; j < type_num_array[type_id]; j++) {
+			if (num >= num_res) break;
+
+			std::string execute_dir, param_name;
+			std::string edir_errstr = paramExecuteDir(num+1, execute_dir, param_name);
+			if (edir_errstr.empty()) {
+				auto & partition_id = exec_dir_to_partition_id[execute_dir];
+				if (partition_id.empty()) {
+					edir_errstr = GetExecuteDirPartitionId(execute_dir, partition_id, param_name.c_str());
+					if (partition_id.empty()) { partition_id = "<none>"; }
+				}
+			}
+			if ( ! edir_errstr.empty()) {
+				if (failmode == BuildSlotFailureMode::Except) {
+					EXCEPT("%s",edir_errstr.c_str());
+				}
+				dprintf(D_ERROR, "%s\n", edir_errstr.c_str());
+
+				if (failmode == BuildSlotFailureMode::AllOrNothing) {
+					fail_types[type_id] = true;
+					return false;
+				}
+			}
+			else {
+				long long total_disk = 0;
+				long long free_disk = sysapi_total_disk_space(execute_dir.c_str(), &total_disk);
+				auto partition_id = exec_dir_to_partition_id[execute_dir].c_str();
+				m_attr->add_disk_volume(partition_id, free_disk, total_disk, total_disk - free_disk);
+			}
+			num++;
+		}
+	}
+
+	return true;
+}
+#endif
+
 // called by ResMgr::init_resources during startup to build the initial slots
 // 
 CpuAttributes** buildCpuAttrs(
@@ -95,7 +208,8 @@ CpuAttributes** buildCpuAttrs(
 	int total,
 	int* type_num_array,
 	bool *backfill_types,
-	bool except )
+	BuildSlotFailureMode failmode,
+	std::vector<bool> & fail_types)
 {
 	int num;
 	CpuAttributes* cap;
@@ -106,45 +220,130 @@ CpuAttributes** buildCpuAttrs(
 	AvailAttributes avail( m_attr );
 	AvailAttributes bkavail( m_attr );
 
+	// for the non LVM case, we need a map of execute dir to partition id
+	std::map<std::string, std::string> exec_dir_to_partition_id;
+
+	int d_except = (failmode == BuildSlotFailureMode::Except) ? D_EXCEPT : 0;
+
 	cap_array = new CpuAttributes* [total];
-	if( ! cap_array ) {
-		EXCEPT( "Out of memory!" );
-	}
+	ASSERT(cap_array);
 
 	num = 0;
-	for (int i=0; i<max_types; i++) {
-		if( type_num_array[i] ) {
-			for (int j=0; j<type_num_array[i]; j++) {
-				cap = buildSlot( m_attr, num+1, type_strings[i], i, except );
-				bool fits = false;
-				if (backfill_types[i]) {
-					fits = bkavail.decrement(cap);
-				} else {
-					fits = avail.decrement(cap);
+	for (int type_id = 0; type_id < max_types; ++type_id) {
+		if (num >= total) break;
+
+		for (int j=0; j < type_num_array[type_id]; j++) {
+			if (num >= total) break;
+
+			std::string execute_dir, execute_param_name;
+			std::string edir_errstr = paramExecuteDir(num+1, execute_dir, execute_param_name);
+			if (edir_errstr.empty()) {
+				// the first time we see an exec dir we have to fill in the partition id value
+				// this ends up populating the exec_dir_to_partition_id map
+				auto & partition_id = exec_dir_to_partition_id[execute_dir];
+				if (partition_id.empty()) {
+					if (m_attr->using_volume_manager()) {
+						partition_id = "<none>";
+						auto volman = m_attr->get_volume_manager();
+						if (volman && volman->is_enabled()) {
+							partition_id = volman->GetBackingDevice();
+						}
+					} else {
+						edir_errstr = GetExecuteDirPartitionId(execute_dir, partition_id, execute_param_name.c_str());
+						if (partition_id.empty()) { partition_id = "<none>"; }
+						if (m_attr->total_disk_from_config()) { partition_id = "<from-config>"; }
+					}
 				}
-				if( fits && num < total ) {
+			}
+			if ( ! edir_errstr.empty()) {
+				if (failmode == BuildSlotFailureMode::Except) {
+					EXCEPT("%s",edir_errstr.c_str());
+				}
+				dprintf(D_ERROR, "%s\n", edir_errstr.c_str());
+
+				if (failmode == BuildSlotFailureMode::AllOrNothing) {
+					for (int ii=0; ii<num; ++ii) { delete cap_array[ii]; }
+					delete [] cap_array;
+					fail_types[type_id] = true;
+					return NULL;
+				}
+			}
+			auto partition_id = exec_dir_to_partition_id[execute_dir].c_str();
+
+			CpuAttributes::_slot_request request;
+			if ( ! CpuAttributes::buildSlotRequest(request, m_attr, type_strings[type_id], type_id, partition_id, failmode)) {
+				// the failure has already been logged.
+				fail_types[type_id] = true;
+				if (failmode == BuildSlotFailureMode::AllOrNothing) {
+					// Gracefully cleanup and abort
+					for(int ii=0; ii<num; ii++ ) { delete cap_array[ii]; }
+					delete [] cap_array;
+					return NULL;
+				}
+				continue; // mode is not except or all-or-nothing so keep going.
+
+			}
+			cap = new CpuAttributes(type_id, request, *m_attr, execute_dir, partition_id);
+			bool fits = false;
+			if (backfill_types[type_id]) {
+				fits = bkavail.decrement(cap);
+			} else {
+				fits = avail.decrement(cap);
+			}
+			if (fits) {
+				if ( ! edir_errstr.empty()) {
+					logbuf = edir_errstr;
+					cap->set_broken(BROKEN_CODE_NO_EXEC_DIR, logbuf);
+				}
+				cap_array[num] = cap;
+				num++;
+			} else {
+				// We ran out of system resources.
+				logbuf =    "\tRequesting: ";
+				cap->cat_totals(logbuf);
+				logbuf += "\n\tAvailable:  ";
+				avail.cat_totals(logbuf, cap->executePartitionID());
+				dprintf( D_ERROR | d_except,
+							"ERROR: Can't allocate %s slot of type %d\n%s\n",
+							num_string(j+1), type_id, logbuf.c_str() );
+				if (failmode == BuildSlotFailureMode::Except) {
+					EXCEPT( "Ran out of system resources" );
+				} else if (failmode == BuildSlotFailureMode::AllOrNothing) {
+					delete cap;	// This isn't in our array yet.
+					// Gracefully cleanup and abort
+					for (int ii=0; ii<num; ++ii) { delete cap_array[ii]; }
+					delete [] cap_array;
+					fail_types[type_id] = true;
+					return NULL;
+				} else if (j==0) {
+					// for first slot of this type, make a partial slot
+					delete cap;
+					logbuf.clear();
+					logbuf = "Not enough ";
+					if (backfill_types[type_id]) {
+						logbuf += "backfill ";
+						bkavail.trim_request_to_fit(request, partition_id, logbuf);
+						cap = new CpuAttributes(type_id, request, *m_attr, execute_dir, partition_id);
+						fits = bkavail.decrement(cap);
+					} else {
+						avail.trim_request_to_fit(request, partition_id, logbuf);
+						cap = new CpuAttributes(type_id, request, *m_attr, execute_dir, partition_id);
+						fits = avail.decrement(cap);
+					}
+
+					if ( ! fits) {
+						dprintf(D_ERROR, "ERROR: Can't allocate reduced slot of type %d\n", type_id);
+						fail_types[type_id] = true;
+						continue;
+					}
+
+					trim(logbuf);
+					cap->set_broken(BROKEN_CODE_NO_RES, logbuf);
 					cap_array[num] = cap;
 					num++;
 				} else {
-					// We ran out of system resources.
-					logbuf =    "\tRequesting: ";
-					cap->cat_totals(logbuf);
-					logbuf += "\n\tAvailable:  ";
-					avail.cat_totals(logbuf, cap->executePartitionID());
-					dprintf( D_ERROR | (except ? D_EXCEPT : 0),
-							 "ERROR: Can't allocate %s slot of type %d\n%s\n",
-							 num_string(j+1), i, logbuf.c_str() );
-					delete cap;	// This isn't in our array yet.
-					if( except ) {
-						EXCEPT( "Ran out of system resources" );
-					} else {
-							// Gracefully cleanup and abort
-						for(int ii=0; ii<num; ii++ ) {
-							delete cap_array[ii];
-						}
-						delete [] cap_array;
-						return NULL;
-					}
+					delete cap; // not the first slot of this type, just don't make it
+					fail_types[type_id] = true;
 				}
 			}
 		}
@@ -196,16 +395,20 @@ CpuAttributes** buildCpuAttrs(
 					"ERROR: Can't allocate slot id %d (slot type %d) during auto allocation of resources\n%s\n",
 					i+1, cap->type_id(), logbuf.c_str() );
 
-			delete cap;	// This isn't in our array yet.
-			if( except ) {
+			if (failmode == BuildSlotFailureMode::Except) {
 				EXCEPT( "Ran out of system resources in auto allocation" );
-			} else {
+			} else if (failmode == BuildSlotFailureMode::AllOrNothing) {
 					// Gracefully cleanup and abort
 				for (int j=0;  j<num;  j++) {
 					delete cap_array[j];
 				}
+				fail_types[cap->type_id()] = true;
 				delete [] cap_array;
 				return NULL;
+			} else {
+				// this may overwrite an existing set_broken() value, but that's ok for now.
+				cap->set_broken(BROKEN_CODE_NO_RES, logbuf);
+				continue;
 			}
 		}
 
@@ -216,46 +419,62 @@ CpuAttributes** buildCpuAttrs(
 		}
 	}
 
+	// update the disk partition tables in the MachAttributes instance
+	// to record how much disk total disk we provisioned.
+	avail.update_provisioned_disk(m_attr);
+	bkavail.update_provisioned_disk(m_attr);
+
 	for (int i=0; i<num; i++) {
 		bool backfill = backfill_types[cap_array[i]->type_id()];
-		cap_array[i]->bind_DevIds(m_attr, i+1, 0, backfill, true);
+		if ( ! cap_array[i]->bind_DevIds(m_attr, i+1, 0, backfill, failmode == BuildSlotFailureMode::Except, 0)) {
+			// if we get here, bind_DevIds should have marked the slot as broken
+			ASSERT(cap_array[i]->is_broken());
+		}
 	}
+
+	if (num == 0) {
+		delete [] cap_array;
+		cap_array = nullptr;
+	} else {
+		for ( ; num < total; ++num) { cap_array[num] = nullptr; }
+	}
+
 	return cap_array;
 }
 
-void initTypes( int max_types, std::vector<std::string>& type_strings, int except )
+void initTypes( int max_types, std::vector<std::string>& type_strings, BuildSlotFailureMode failmode, std::vector<bool>& fail_types )
 {
 	int i;
 	std::string buf;
 
-	_checkInvalidParam("SLOT_TYPE_0", except);
+	_checkInvalidParam("SLOT_TYPE_0", failmode == BuildSlotFailureMode::Except);
 		// CRUFT
-	_checkInvalidParam("VIRTUAL_MACHINE_TYPE_0", except);
+	_checkInvalidParam("VIRTUAL_MACHINE_TYPE_0", failmode == BuildSlotFailureMode::Except);
 
 		// Type 0 is the special type for evenly divided slots.
 	type_strings[0] = "auto";
+	fail_types[0] = false;
 
 	for( i=1; i < max_types; i++ ) {
 		type_strings[i].clear();
+		fail_types[i] = false;
 		formatstr(buf, "SLOT_TYPE_%d", i);
 		param(type_strings[i], buf.c_str());
 	}
 }
 
 
-int countTypes( int max_types, int num_cpus, int** array_ptr, bool** bkfill_ptr, bool except )
+int countTypes( int max_types, int num_cpus, int** array_ptr, bool** bkfill_ptr, BuildSlotFailureMode failmode )
 {
 	int i, num=0, num_set=0;
 	std::string param_name;
 	int* my_type_nums = new int[max_types];
 	bool* my_bkfill_bools = new bool[max_types];
 
-	if( ! array_ptr ) {
-		EXCEPT( "ResMgr:countTypes() called with NULL array_ptr!" );
-	}
+	ASSERT(array_ptr);
 
 		// Type 0 is special, user's shouldn't define it.
-	_checkInvalidParam("NUM_SLOTS_TYPE_0", except);
+	_checkInvalidParam("NUM_SLOTS_TYPE_0", failmode == BuildSlotFailureMode::Except);
 
 	for( i=1; i<max_types; i++ ) {
 		formatstr(param_name, "NUM_SLOTS_TYPE_%d", i);
@@ -297,8 +516,9 @@ typedef enum { SPECIAL_SHARE_NONE=0, SPECIAL_SHARE_AUTO=1, SPECIAL_SHARE_MINIMAL
    or it's a regular value, like "64"
    auto and fraction/percent are returned as negative values
 */
-static double parse_share_value(const char* str, int type, bool except, special_share_t & special)
+static double parse_share_value(const char* str, int type, BuildSlotFailureMode failmode, special_share_t & special, int & scale)
 {
+	scale = 0;
 	if( strcasecmp(str,"auto") == 0 || strcasecmp(str,"automatic") == 0 ) {
 		special = SPECIAL_SHARE_AUTO;
 		return AUTO_SHARE;
@@ -322,7 +542,7 @@ static double parse_share_value(const char* str, int type, bool except, special_
 		if (denom <= 0) {
 			dprintf( D_ALWAYS, "Can't parse attribute \"%s\" in description of slot type %d\n",
 					 str, type );
-			if( except ) {
+			if (failmode == BuildSlotFailureMode::Except) {
 				DC_Exit( 4 );
 			} else {
 				return 0;
@@ -331,16 +551,27 @@ static double parse_share_value(const char* str, int type, bool except, special_
 		return -(atof(str) / denom);
 	}
 
-		// This must just be an absolute value.  Return it as a negative
-		// value, so the caller knows it's not a fraction.
-	return atof(str);
+	// This must just be an absolute value.  Return it as a positive
+	// value, so the caller knows it's not a fraction.
+	char * endp = nullptr;
+	double value = strtod(str, &endp);
+
+	// check for a scale suffix. and return a scale factor if one was supplied
+	while (isspace(*endp)) ++endp;
+	if (*endp == 'k' || *endp == 'K') scale = 1;
+	else if (*endp == 'm' || *endp == 'M') scale = 2;
+	else if (*endp == 'g' || *endp == 'G') scale = 3;
+	else if (*endp == 't' || *endp == 'T') scale = 4;
+	else if (*endp == 'p' || *endp == 'P') scale = 5;
+
+	return value;
 }
 
 
 /*
   convert share value into a quantity if possible, positive values are
-  absolute quantities, they are truncated to an int, but otherwise pass
-  through this function.  Negative values should be between -1.0 and 0.0
+  absolute quantities, the pass through this function but may be scaled.
+  Negative values should be between -1.0 and 0.0
   and are interpreted as a proportion of the total.
 
   a CPU.  However, we never want to advertise less than 1.
@@ -348,10 +579,16 @@ static double parse_share_value(const char* str, int type, bool except, special_
   Positive values are absolute values.
 */
 template <class t>
-static t compute_local_resource(t num_res, double share, t min_res = 0)
+static t compute_local_resource(t num_res, double share, t min_res = 0, int scale=0, int native_scale=0)
 {
 	if (IS_AUTO_SHARE(share)) return AUTO_RES;
 	t res = (share < 0) ? t(-share * num_res) : t(share);
+	if (share > 0) {
+		while (scale > native_scale) {
+			res *= 1024;
+			--scale;
+		}
+	}
 	return MAX(res, min_res);
 }
 
@@ -363,19 +600,30 @@ const int UNSET_SHARE = -9998;
 	MachAttributes *m_attr,
 	const std::string& list,
 	unsigned int type_id,
-	bool except)
+	const char * partition_id,
+	BuildSlotFailureMode failmode)
 {
 	typedef CpuAttributes::slotres_map_t slotres_map_t;
 	special_share_t default_share_special = SPECIAL_SHARE_AUTO;
 	double default_share = AUTO_SHARE;
 
+	const double minimum_cpus = 1.0;
+	const int minimum_mem = 1;           // 1 MB
+	const long long minimum_swap = 0;
+	const long long minimum_disk = 1024; // 1 MB in KB
+
 	if ( list.empty()) {
 		// give everything the default share and return
 
-		request.num_cpus = compute_local_resource(m_attr->num_cpus(), default_share, 1.0);
-		request.num_phys_mem = compute_local_resource(m_attr->phys_mem(), default_share, 1);
+		request.num_cpus = compute_local_resource(m_attr->num_cpus(), default_share, minimum_cpus);
+		request.num_phys_mem = compute_local_resource(m_attr->phys_mem(), default_share, minimum_mem);
+	#ifdef PROVISION_FRACTIONAL_DISK
 		request.virt_mem_fraction = AUTO_SHARE;
 		request.disk_fraction = AUTO_SHARE;
+	#else
+		request.num_swap = compute_local_resource(m_attr->num_swap(), default_share, minimum_swap);
+		request.num_disk = compute_local_resource(m_attr->free_disk(partition_id), default_share, minimum_disk);
+	#endif
 
 		for (slotres_map_t::const_iterator j(m_attr->machres().begin());  j != m_attr->machres().end();  ++j) {
 			request.slotres[j->first] = default_share;
@@ -386,8 +634,13 @@ const int UNSET_SHARE = -9998;
 
 	double cpus = UNSET_SHARE;
 	int ram = UNSET_SHARE; // this is in MB so an int is enough
+#ifdef PROVISION_FRACTIONAL_DISK
 	double disk_fraction = UNSET_SHARE;
 	double swap_fraction = UNSET_SHARE;
+#else
+	long long swap = UNSET_SHARE; // in KB
+	long long disk = UNSET_SHARE; // in KB
+#endif
 
 	// For this parsing code, deal with the following example
 	// string list:
@@ -400,8 +653,12 @@ const int UNSET_SHARE = -9998;
 		request.slotres[j->first] = UNSET_SHARE;
 	}
 
-	// In a multi-line value, '\n' is the only delimiter
-	const char* delim = strchr(list.c_str(), '\n') ? "\n" : ", \t\r\n";
+	const char* delim = ", \t\r\n"; // default delim is whitespace or ,
+	if (list.find('\n') != std::string::npos) {
+		delim = "\n"; // In a multi-line value, '\n' is the only delimiter
+	} else if (list.find(',') != std::string::npos) {
+		delim = ",\r\n"; // if commas are used, then only commas and newlines are the delimiter
+	}
 
 	for (const auto& attr_expr: StringTokenIterator(list, delim)) {
 		std::string::size_type eqpos = attr_expr.find('=');
@@ -410,7 +667,8 @@ const int UNSET_SHARE = -9998;
 			// percentage or fraction for all attributes.
 			// For example "1/4" or "25%".  So, we can just parse
 			// it as a percentage and use that for everything.
-			default_share = parse_share_value(attr_expr.c_str(), type_id, except, default_share_special);
+			int scale;
+			default_share = parse_share_value(attr_expr.c_str(), type_id, failmode, default_share_special, scale);
 			if (default_share_special == SPECIAL_SHARE_NONE && (default_share < -1 || default_share > 0)) {
 				::dprintf( D_ALWAYS, "ERROR: Bad description of slot type %d: "
 					"\"%s\" is invalid.\n"
@@ -419,7 +677,7 @@ const int UNSET_SHARE = -9998;
 					"\tor list all attributes (like \"c=1, r=25%%, s=25%%, d=25%%\").\n"
 					"\tSee the manual for details.\n",
 					type_id, attr_expr.c_str());
-				if( except ) {
+				if (failmode == BuildSlotFailureMode::Except) {
 					DC_Exit( 4 );
 				} else {
 					return false;
@@ -430,7 +688,7 @@ const int UNSET_SHARE = -9998;
 		}
 
 		// If we're still here, this is part of a string that
-		// lists out seperate attributes and the share for each one.
+		// lists out separate attributes and the share for each one.
 
 		// Get the value for this attribute.  It'll either be a
 		// percentage, or it'll be a distinct value (in which
@@ -440,7 +698,7 @@ const int UNSET_SHARE = -9998;
 		if (val.empty()) {
 			::dprintf(D_ALWAYS, "Can't parse attribute \"%s\" in description of slot type %d\n",
 				attr_expr.c_str(), type_id);
-			if( except ) {
+			if (failmode == BuildSlotFailureMode::Except) {
 				DC_Exit( 4 );
 			} else {
 				return false;
@@ -453,8 +711,9 @@ const int UNSET_SHARE = -9998;
 			val = val.substr(0, colonpos);
 			trim(val);
 		}
+		int scale = 0; // set to 1 or more if there was a K suffix, etc.
 		special_share_t share_special = SPECIAL_SHARE_NONE;
-		double share = parse_share_value(val.c_str(), type_id, except, share_special);
+		double share = parse_share_value(val.c_str(), type_id, failmode, share_special, scale);
 
 		// Figure out what attribute we're dealing with.
 		std::string attr = attr_expr.substr(0, eqpos);
@@ -477,17 +736,18 @@ const int UNSET_SHARE = -9998;
 		int cattr = int(attr.length());
 		if (MATCH == strncasecmp(attr.c_str(), "cpus", cattr))
 		{
-			double lower_bound = (request.allow_fractional_cpus) ? 0.0 : 1.0;
+			double lower_bound = (request.allow_fractional_cpus) ? 0.0 : minimum_cpus;
 			cpus = compute_local_resource(m_attr->num_cpus(), share, lower_bound);
 		} else if (
 			MATCH == strncasecmp(attr.c_str(), "ram", cattr) ||
 			MATCH == strncasecmp(attr.c_str(), "memory", cattr))
 		{
-			ram = compute_local_resource(m_attr->phys_mem(), share, 1);
+			ram = compute_local_resource(m_attr->phys_mem(), share, minimum_mem, scale, 2);
 		} else if (
 			MATCH == strncasecmp(attr.c_str(), "swap", cattr) ||
 			MATCH == strncasecmp(attr.c_str(), "virtualmemory", cattr))
 		{
+		#ifdef PROVISION_FRACTIONAL_DISK
 			// for now, we can only tolerate swap expressed a 'auto' or a proportion
 			// because we don't know how much swap is available until later.
 			if (share <= 0 || IS_AUTO_SHARE(share)) {
@@ -496,15 +756,19 @@ const int UNSET_SHARE = -9998;
 				::dprintf( D_ALWAYS,
 					"You must specify a percent or fraction for swap in slot type %d\n",
 					type_id );
-				if( except ) {
+				if (failmode == BuildSlotFailureMode::Except) {
 					DC_Exit( 4 );
 				} else {
 					return false;
 				}
 			}
+		#else
+			swap = compute_local_resource(m_attr->num_swap(), share, minimum_swap, scale, 1);
+		#endif
 		} else if (
 			MATCH == strncasecmp(attr.c_str(), "disk", cattr))
 		{
+		#ifdef PROVISION_FRACTIONAL_DISK
 			// for now, we can only tolerate disk expressed a 'auto' or a proportion
 			// because we don't know how much disk is available until later.
 			if (share <= 0 || IS_AUTO_SHARE(share)) {
@@ -513,12 +777,15 @@ const int UNSET_SHARE = -9998;
 				::dprintf( D_ALWAYS,
 					"You must specify a percent or fraction for disk in slot type %d\n",
 					type_id );
-				if( except ) {
+				if (failmode == BuildSlotFailureMode::Except) {
 					DC_Exit( 4 );
 				} else {
 					return false;
 				}
 			}
+		#else
+			disk = compute_local_resource(m_attr->free_disk(partition_id), share, minimum_disk, scale, 1); // 1 MB minimum
+		#endif
 
 		} else {
 
@@ -529,7 +796,7 @@ const int UNSET_SHARE = -9998;
 			// like - "if this machine has GPUs, I don't want this slot to get any"..
 			if (share > 0) {
 				::dprintf( D_ALWAYS, "Unknown attribute \"%s\" in slot type %d\n", attr.c_str(), type_id);
-				if( except ) {
+				if (failmode == BuildSlotFailureMode::Except) {
 					DC_Exit( 4 );
 				} else {
 					return false;
@@ -554,6 +821,7 @@ const int UNSET_SHARE = -9998;
 	}
 	request.num_phys_mem = ram;
 
+#ifdef PROVISION_FRACTIONAL_DISK
 	if (IS_UNSET_SHARE(swap_fraction)) {
 		swap_fraction = compute_local_resource(1.0, default_share);
 	}
@@ -563,6 +831,17 @@ const int UNSET_SHARE = -9998;
 		disk_fraction = compute_local_resource(1.0, default_share);
 	}
 	request.disk_fraction = disk_fraction;
+#else
+	if (IS_UNSET_SHARE(swap)) {
+		swap = compute_local_resource(m_attr->num_swap(), default_share);
+	}
+	request.num_swap = swap;
+
+	if (IS_UNSET_SHARE(disk)) {
+		disk = compute_local_resource(m_attr->free_disk(partition_id), default_share);
+	}
+	request.num_disk = disk;
+#endif
 
 	for (slotres_map_t::iterator j(request.slotres.begin());  j != request.slotres.end();  ++j) {
 		if (IS_UNSET_SHARE(j->second)) {
@@ -577,18 +856,3 @@ const int UNSET_SHARE = -9998;
 	return true;
 }
 
-
-CpuAttributes* buildSlot( MachAttributes *m_attr, int slot_id, const std::string& list, unsigned int type_id, bool except )
-{
-	CpuAttributes::_slot_request request;
-	if (CpuAttributes::buildSlotRequest(request, m_attr, list, type_id, except)) {
-
-		std::string execute_dir, partition_id;
-		GetConfigExecuteDir( slot_id, execute_dir, partition_id );
-
-		return new CpuAttributes(type_id, request, execute_dir, partition_id);
-	}
-
-	return NULL;
-}
-
