@@ -32,6 +32,7 @@
 #include "slot_builder.h"
 #include "history_queue.h"
 #include "../condor_sysapi/sysapi.h"
+#include "docker-api.h"
 
 #if defined(WANT_CONTRIB) && defined(WITH_MANAGEMENT)
 #include "StartdPlugin.h"
@@ -61,9 +62,17 @@ int	update_interval = 0;	// Interval to update CM
 //      and advertise STARTD_OLD_ADTYPE to older collectors
 int enable_single_startd_daemon_ad = 0;
 
+BuildSlotFailureMode slot_config_failmode = BuildSlotFailureMode::Except;
+
+// set by CONTINUE_TO_ADVERTISE_BROKEN_DSLOTS on startup
+bool continue_to_advertise_broken_dslots = false;
+
 // set by ENABLE_CLAIMABLE_PARTITIONABLE_SLOTS on startup
 bool enable_claimable_partitionable_slots = false;
 
+// set by NO_JOB_NETWORKING  on startup (indirectly from the Starter capabilities)
+bool want_job_networking_is_a_resource_request = false;
+
 // String Lists
 std::vector<std::string> startd_job_attrs;
 std::vector<std::string> startd_slot_attrs;
@@ -89,7 +98,11 @@ int		console_slots = 0;	// # of nodes in an SMP that care about
 int		keyboard_slots = 0;  //   console and keyboard activity
 int		disconnected_keyboard_boost;	// # of seconds before when we
 	// started up that we advertise as the last key press for
-	// resources that aren't connected to anything.  
+	// resources that aren't connected to anything.
+int     startup_keyboard_boost = 0; // # of seconds before we started up
+    // that we advertise as the last key press until we get the next key press
+char*   simulated_cpuload_expr = nullptr;
+	// expression to evaluate against sysapi_load_avg to get simulated load
 
 int		startd_noclaim_shutdown = 0;	
     // # of seconds we can go without being claimed before we "pull
@@ -100,6 +113,10 @@ int		docker_cached_image_size_interval = 0; // how often we ask docker for the s
 bool	use_unique_lv_names = true; // LVM LV names should never be re-used
 int		lv_name_uniqueness = 0;
 
+bool	system_want_exec_encryption = false; // Configured to encrypt all job execute directories
+bool	disable_exec_encryption = false; // Disable job execute directory encryption
+
+bool	aggressive_cleanup = false; // Ignore cleanup reminder backoff intervals
 
 char* Name = NULL;
 
@@ -125,6 +142,8 @@ int cleanup_reminder_timer_interval = 62; // default to doing at least some clea
 CleanupReminderMap cleanup_reminders;
 extern void register_cleanup_reminder_timer();
 
+StartdEventLog ep_eventlog;
+
 /*
  * Prototypes of static functions.
  */
@@ -186,7 +205,8 @@ main_init( int, char* argv[] )
 
 		// Record the time we started up for use in determining
 		// keyboard idle time on SMP machines, etc.
-	startd_startup = time( 0 );
+	
+	startd_startup = ep_eventlog.composeEvent(ULOG_EP_STARTUP,nullptr).GetEventclock();
 
 #ifdef WIN32
 	// get the Windows sysapi load average thread going early
@@ -194,6 +214,16 @@ main_init( int, char* argv[] )
 	sysapi_load_avg();
 #endif
 
+	// if an EP eventlog is configured, open it now
+	auto_free_ptr eventlog_path(param("STARTD_EVENTLOG"));
+	if (eventlog_path) {
+		int format = ULogEvent::formatOpt::ISO_DATE | ULogEvent::formatOpt::SUB_SECOND;
+		auto_free_ptr fmt_string(param("STARTD_EVENTLOG_FORMAT"));
+		if (fmt_string) { format = ULogEvent::parse_opts(fmt_string, format); }
+		int max_len = param_integer("STARTD_EVENTLOG_MAX", 1024*1024);
+		ep_eventlog.initialize(eventlog_path, max_len, format);
+	}
+
 		// Instantiate the Resource Manager object.
 	resmgr = new ResMgr;
 
@@ -201,16 +231,22 @@ main_init( int, char* argv[] )
 	Starter::config();
 
 	ClassAd tmp_classad;
-	std::string starter_ability_list;
 	Starter::publish(&tmp_classad);
-	tmp_classad.LookupString(ATTR_STARTER_ABILITY_LIST, starter_ability_list);
-	if( starter_ability_list.find(ATTR_HAS_VM) != std::string::npos ) {
+	bool hasVM = false;
+	if (tmp_classad.LookupBool(ATTR_HAS_VM, hasVM) && hasVM) {
 		// Now starter has codes for vm universe.
 		resmgr->m_vmuniverse_mgr.setStarterAbility(true);
 		// check whether vm universe is available through vmgahp server
 		resmgr->m_vmuniverse_mgr.checkVMUniverse( false );
 	}
 
+	// if Starter has job networking disabled, we need to treat WantJobNetworking as a resource request
+	// this causes a clause to be added to WithinResourceLimits
+	bool has_job_networking = true;
+	if (tmp_classad.LookupBool(ATTR_HAS_JOB_NETWORKING, has_job_networking) && ! has_job_networking) {
+		want_job_networking_is_a_resource_request = true;
+	}
+
 		// Read in global parameters from the config file.
 		// We do this after we instantiate the resmgr, so we can know
 		// what num_cpus is, but before init_resources(), so we can
@@ -233,11 +269,25 @@ main_init( int, char* argv[] )
 		// Instantiate Resource objects in the ResMgr
 	resmgr->init_resources();
 
+		// now that we have build initial slots, we can write our STARTUP event
+	auto & startupEvent = ep_eventlog.composeEvent(ULOG_EP_STARTUP,nullptr);
+	startupEvent.Ad().Assign("NumSlots", resmgr->numSlots());
+	ep_eventlog.flush();
+
 		// Do a little sanity checking and cleanup
 	std::vector<std::string> execute_dirs;
 	resmgr->FillExecuteDirsList( execute_dirs );
-	check_execute_dir_perms( execute_dirs );
-	cleanup_execute_dirs( execute_dirs );
+
+	bool abort_on_error = slot_config_failmode == BuildSlotFailureMode::Except;
+	for (const auto& exec_path: execute_dirs) {
+		if (check_execute_dir_perms(exec_path.c_str(), abort_on_error)) {
+			cleanup_execute_dirs(exec_path);
+		}
+	}
+
+	DockerAPI::pruneContainers();
+
+	DockerAPI::removeImagesInImageFile();
 
 		// Compute all attributes
 	resmgr->compute_static();
@@ -273,21 +323,23 @@ main_init( int, char* argv[] )
 		// make sense when we're in the claimed state.  So, we can
 		// handle them all with a common handler.  For all of them,
 		// you need DAEMON permission.
-	daemonCore->Register_Command( ALIVE, "ALIVE", 
+	daemonCore->Register_Command( ALIVE, "ALIVE",
 								  command_handler,
-								  "command_handler", DAEMON ); 
+								  "command_handler", DAEMON );
 	daemonCore->Register_Command( DEACTIVATE_CLAIM,
-								  "DEACTIVATE_CLAIM",  
+								  "DEACTIVATE_CLAIM",
 								  command_handler,
 								  "command_handler", DAEMON );
-	daemonCore->Register_Command( DEACTIVATE_CLAIM_FORCIBLY, 
-								  "DEACTIVATE_CLAIM_FORCIBLY", 
+	daemonCore->Register_Command( DEACTIVATE_CLAIM_FORCIBLY,
+								  "DEACTIVATE_CLAIM_FORCIBLY",
 								  command_handler,
 								  "command_handler", DAEMON );
-	daemonCore->Register_Command( PCKPT_FRGN_JOB, "PCKPT_FRGN_JOB", 
+	daemonCore->Register_Command( DEACTIVATE_CLAIM_JOB_DONE,
+								  "DEACTIVATE_CLAIM_JOB_DONE",
 								  command_handler,
 								  "command_handler", DAEMON );
-	daemonCore->Register_Command( REQ_NEW_PROC, "REQ_NEW_PROC", 
+	daemonCore->Register_Command( REACTIVATE_CLAIM_CHECK,
+								  "REACTIVATE_CLAIM_CHECK",
 								  command_handler,
 								  "command_handler", DAEMON );
 
@@ -304,6 +356,9 @@ main_init( int, char* argv[] )
 	daemonCore->Register_Command( QUERY_STARTD_ADS, "QUERY_STARTD_ADS",
 								  command_query_ads,
 								  "command_query_ads", READ );
+	daemonCore->Register_Command( QUERY_MULTIPLE_ADS, "QUERY_MULTIPLE_ADS",
+								  command_query_ads,
+								  "command_query_ads", READ );
 	if (history_queue_mgr) {
 		daemonCore->Register_CommandWithPayload(GET_HISTORY,
 			"GET_HISTORY",
@@ -332,12 +387,6 @@ main_init( int, char* argv[] )
 								  "X_EVENT_NOTIFICATION",
 								  command_x_event,
 								  "command_x_event", ALLOW ); 
-	daemonCore->Register_Command( PCKPT_ALL_JOBS, "PCKPT_ALL_JOBS", 
-								  command_pckpt_all,
-								  "command_pckpt_all", DAEMON );
-	daemonCore->Register_Command( PCKPT_JOB, "PCKPT_JOB", 
-								  command_name_handler,
-								  "command_name_handler", DAEMON );
 #if !defined(WIN32)
 	daemonCore->Register_Command( DELEGATE_GSI_CRED_STARTD, "DELEGATE_GSI_CRED_STARTD",
 	                              command_delegate_gsi_cred,
@@ -496,6 +545,7 @@ init_params( int first_time)
 		classad::FunctionCall::RegisterFunction( func_name, OtherSlotEval );
 
 		enable_claimable_partitionable_slots = param_boolean("ENABLE_CLAIMABLE_PARTITIONABLE_SLOTS", false);
+		continue_to_advertise_broken_dslots = param_boolean("CONTINUE_TO_ADVERTISE_BROKEN_DYNAMIC_SLOTS", false);
 	}
 
 	resmgr->init_config_classad();
@@ -504,8 +554,7 @@ init_params( int first_time)
 
 	update_interval = param_integer( "UPDATE_INTERVAL", 300, 1 );
 
-	// TODO: change this default to True or Auto
-	enable_single_startd_daemon_ad = 0;
+	enable_single_startd_daemon_ad = 2;
 	auto_free_ptr send_daemon_ad(param("ENABLE_STARTD_DAEMON_AD"));
 	if (send_daemon_ad) {
 		bool bval = false;
@@ -518,6 +567,23 @@ init_params( int first_time)
 	dprintf(D_STATUS, "ENABLE_STARTD_DAEMON_AD=%d (%s)\n", enable_single_startd_daemon_ad,
 		send_daemon_ad.ptr() ? send_daemon_ad.ptr() : "");
 
+	if (first_time) {
+		// Init the failure mode for setup, if we have no daemon ad there isn't any way to
+		// report most failures, so we should default to Except in that case.
+		// But if there is a daemon ad we should default to BestEffort.
+		slot_config_failmode = BuildSlotFailureMode::Except;
+		if (enable_single_startd_daemon_ad) {
+			slot_config_failmode = BuildSlotFailureMode::BestEffort;
+		}
+		auto_free_ptr bsfm(param("SLOT_CONFIG_FAILURE_MODE"));
+		if (bsfm) {
+			if ( ! string_to_BuildSlotFailureMode(bsfm, slot_config_failmode)) {
+				dprintf(D_ERROR, "Ignoring unknown value %s for BUILD_SLOT_FAILURE_MODE\n", bsfm.ptr());
+			}
+		}
+		dprintf(D_STATUS, "SLOT_CONFIG_FAILURE_MODE is %s\n", BuildSlotFailureMode_to_string(slot_config_failmode));
+	}
+
 	if( accountant_host ) {
 		free( accountant_host );
 	}
@@ -553,7 +619,11 @@ init_params( int first_time)
 
 	console_slots = param_integer( "SLOTS_CONNECTED_TO_CONSOLE", 0);
 	keyboard_slots = param_integer( "SLOTS_CONNECTED_TO_KEYBOARD", 0);
-	disconnected_keyboard_boost = param_integer( "DISCONNECTED_KEYBOARD_IDLE_BOOST", 1200 );
+	disconnected_keyboard_boost = param_integer( "DISCONNECTED_KEYBOARD_IDLE_BOOST", 20*60 );
+	startup_keyboard_boost = param_integer( "STARTUP_KEYBOARD_IDLE_BOOST", 0 );
+	if (startup_keyboard_boost < 0) startup_keyboard_boost = 0;
+	if (simulated_cpuload_expr) free(simulated_cpuload_expr);
+	simulated_cpuload_expr = param("SIMULATED_CPULOAD_EXPR");
 
 	startd_noclaim_shutdown = param_integer( "STARTD_NOCLAIM_SHUTDOWN", 0 );
 
@@ -562,7 +632,16 @@ init_params( int first_time)
 
 	// these are secret, should not be in the param table
 	use_unique_lv_names = param_boolean("LVM_USE_UNIQUE_LV_NAMES", true); // LVM LV names should never be re-used
-	lv_name_uniqueness  = param_integer("LVM_FIRST_LV_ID", 1); // In case we want to set the initial value
+	if (first_time) lv_name_uniqueness  = param_integer("LVM_FIRST_LV_ID", 1); // In case we want to set the initial value
+
+	// Disable all job execute directory or enable for all jobs
+	disable_exec_encryption = param_boolean("DISABLE_EXECUTE_DIRECTORY_ENCRYPTION", false);
+	if ( ! disable_exec_encryption) {
+		system_want_exec_encryption = param_boolean_crufty("ENCRYPT_EXECUTE_DIRECTORY", false);
+	}
+
+	// Skip cleanup reminder backoff and always attempt cleanup: Note for internal testing
+	aggressive_cleanup = param_boolean("AGGRESSIVE_CLEANUP_REMINDER", false);
 
 	// Older condors incorrectly saved the docker image cache file as root.  Fix it to condor
 	// for compatibility
@@ -649,44 +728,54 @@ void CleanupReminderTimerCallback()
 {
 	dprintf(D_FULLDEBUG, "In CleanupReminderTimerCallback() there are %d reminders\n", (int)cleanup_reminders.size());
 
-	for (auto jt = cleanup_reminders.begin(); jt != cleanup_reminders.end(); /* advance in the loop */) {
-		auto it = jt++; // so we can remove the current item if we manage to clean it up
-		it->second += 1; // record that we looked at this.
-		bool erase_it = false; // set this to true when we succeed (or don't need to try anymore)
+	// Set of broken item IDs to restore if successfully cleaned up issue thing (logical volume)
+	std::set<unsigned int> broken_item_ids;
 
-		const CleanupReminder & cr = it->first; // alias the CleanupReminder so that the code below is clearer
+	auto done = [&broken_item_ids](auto& pair) -> bool {
+		const CleanupReminder& cr = pair.first;
+		const int iteration = ++cleanup_reminders[cr];
 
-		bool retry_now = retry_on_this_iter(it->second, cr.cat);
-		dprintf(D_FULLDEBUG, "cleanup_reminder %s, iter %d, retry_now = %d\n", cr.name.c_str(), it->second, retry_now);
+		if ( ! aggressive_cleanup && ! retry_on_this_iter(iteration, cr.cat)) { return false; }
 
-		// if our exponential backoff says we should retry this time, attempt the cleanup.
-		if (retry_now) {
-			int err=0;
-			switch (cr.cat) {
+		dprintf(D_FULLDEBUG, "cleanup_reminder for %s iteration %d\n", cr.name.c_str(), iteration);
+
+		int err = 0;
+		bool success = false;
+
+		switch (cr.cat) {
 			case CleanupReminder::category::exec_dir:
-				if (retry_cleanup_execute_dir(cr.name, cr.opt, err)) {
-					dprintf(D_ALWAYS, "Retry of directory delete '%s' succeeded. removing it from the retry list\n", cr.name.c_str());
-					erase_it = true;
-				} else {
-					dprintf(D_ALWAYS, "Retry of directory delete '%s' failed with error %d. will try again later\n", cr.name.c_str(), err);
-				}
+				success = retry_cleanup_execute_dir(cr.name, cr.opt, err);
 				break;
 			case CleanupReminder::category::account:
-				if (retry_cleanup_user_account(cr.name, cr.opt, err)) {
-					dprintf(D_ALWAYS, "Retry of account cleanup for '%s' succeeded. removing it from the retry list\n", cr.name.c_str());
-					erase_it = true;
-				} else {
-					dprintf(D_ALWAYS, "Retry of account cleanup '%s' failed with error %d. will try again later\n", cr.name.c_str(), err);
+				success = retry_cleanup_user_account(cr.name, cr.opt, err);
+				break;
+			case CleanupReminder::category::logical_volume:
+				success = retry_cleanup_logical_volume(cr.name, cr.opt, err);
+				// If LV was removed and the CR had an associated broken item ID
+				// then add to set of broken item IDs to restore
+				if (success && cr.broken_id) {
+					broken_item_ids.insert(cr.broken_id);
 				}
 				break;
-			}
-
+			default:
+				EXCEPT("Unknown CleanupReminder Category: %d\n", cr.cat);
 		}
 
-		// if we successfully cleaned up, or cleanup is now moot, remove the item from the list.
-		if (erase_it) {
-			cleanup_reminders.erase(it);
+		if (success) {
+			dprintf(D_ALWAYS, "Retry to clean up %s '%s' successful.\n",
+			        cr.Type(), cr.name.c_str());
+		} else {
+			dprintf(D_ERROR, "Retry to clean up %s '%s' failed (%d). Will retry again later...\n",
+			        cr.Type(), cr.name.c_str(), err);
 		}
+
+		return success;
+	};
+
+	std::erase_if(cleanup_reminders, done);
+
+	if ( ! broken_item_ids.empty()) {
+		resmgr->RestoreBrokenResources(ResourceLockType::LV, broken_item_ids);
 	}
 
 	// if the collection of things to try and clean up is empty, turn off the timer
@@ -710,9 +799,7 @@ void register_cleanup_reminder_timer()
 								cleanup_reminder_timer_interval,
 								(TimerHandler)CleanupReminderTimerCallback,
 								"CleanupReminderTimerCallback");
-		if  (id < 0) {
-			EXCEPT( "Can't register DaemonCore timer for cleanup reminders" );
-		}
+		ASSERT(id >= 0);
 		cleanup_reminder_timer_id = id;
 	}
 }
@@ -780,6 +867,8 @@ startd_exit()
 	StartdPluginManager::Shutdown();
 #endif
 
+	ep_eventlog.composeEvent(ULOG_EP_SHUTDOWN,nullptr).Ad().Assign("ExitCode", 0);
+	ep_eventlog.flush();
 	dprintf( D_ALWAYS, "All resources are free, exiting.\n" );
 	DC_Exit(0);
 }
@@ -800,7 +889,7 @@ main_shutdown_fast()
 	}
 
 		// If the machine is free, we can just exit right away.
-	startd_check_free();
+	startd_exit_if_idle();
 
 		// Remember that we're in shutdown-mode so we will refuse
 		// various commands. 
@@ -811,11 +900,11 @@ main_shutdown_fast()
 								 "shutdown_reaper" );
 
 		// Quickly kill all the starters that are running
-	resmgr->killAllClaims();
+	resmgr->killAllClaims("Startd was shutdown", CONDOR_HOLD_CODE::StartdShutdown, 0);
 
 	daemonCore->Register_Timer( 0, 5, 
-								startd_check_free,
-								 "startd_check_free" );
+								startd_exit_if_idle,
+								 "startd_exit_if_idle" );
 }
 
 
@@ -835,7 +924,7 @@ main_shutdown_graceful()
 	}
 
 		// If the machine is free, we can just exit right away.
-	startd_check_free();
+	startd_exit_if_idle();
 
 		// Remember that we're in shutdown-mode so we will refuse
 		// various commands. 
@@ -846,11 +935,11 @@ main_shutdown_graceful()
 								 "shutdown_reaper" );
 
 		// Release all claims, active or not
-	resmgr->releaseAllClaims();
+	resmgr->releaseAllClaims("Startd was shutdown", CONDOR_HOLD_CODE::StartdShutdown, 0);
 
 	daemonCore->Register_Timer( 0, 5, 
-								startd_check_free,
-								 "startd_check_free" );
+								startd_exit_if_idle,
+								 "startd_exit_if_idle" );
 }
 
 
@@ -890,7 +979,7 @@ int
 shutdown_reaper(int pid, int status)
 {
 	reaper(pid,status);
-	startd_check_free();
+	startd_exit_if_idle();
 	return TRUE;
 }
 
@@ -902,11 +991,11 @@ do_cleanup(int,int,const char*)
 
 	if ( already_excepted == FALSE ) {
 		already_excepted = TRUE;
-			// If the machine is already free, we can exit right away.
-		startd_check_free();		
-			// Otherwise, quickly kill all the active starters.
+		// If the machine is already free, we can exit right away.
+		startd_exit_if_idle();
+		// Otherwise, quickly kill all the active starters.
 		const bool fast = true;
-		resmgr->vacate_all(fast);
+		resmgr->vacate_all(fast, "Startd EXCEPT", CONDOR_HOLD_CODE::StartdException, 0);
 		dprintf( D_ERROR | D_EXCEPT, "startd exiting because of fatal exception.\n" );
 	}
 
@@ -915,8 +1004,8 @@ do_cleanup(int,int,const char*)
 
 
 void
-startd_check_free(int /* tid */)
-{	
+startd_exit_if_idle(int /* tid */)
+{
 	if ( cron_job_mgr && ( ! cron_job_mgr->ShutdownOk() ) ) {
 		return;
 	}
@@ -930,7 +1019,7 @@ startd_check_free(int /* tid */)
 	//   RELEASE_CLAIM for those before shutting down.
 	//   Today, those messages would fail, as the schedd doesn't keep
 	//   track of claimed pslots. We expect this to change in the future.
-	if( ! resmgr->hasAnyClaim() ) {
+	if( ! resmgr->hasAnyActiveClaim(true) ) {
 		startd_exit();
 	}
 	return;
