@@ -27,21 +27,25 @@ class Stream;
 class Resource;
 
 // Our utilities 
-void	cleanup_execute_dir(int pid, const char *exec_path, const char * lv_name, bool remove_exec_path, bool abnormal_exit);
-void	cleanup_execute_dirs(const std::vector<std::string> &list);
-void	check_execute_dir_perms(const std::vector<std::string> &list);
+void	cleanup_execute_dir(int pid, const char *exec_path, const char * lv_name, bool remove_exec_path, bool abnormal_exit, bool lv_encrypted);
+void	cleanup_execute_dirs(const std::string &exec_path);
+bool	check_execute_dir_perms(const char* exec_path, bool abort_on_error);
 void	check_recovery_file( const char *sandbox_dir, bool abnormal_exit );
 
 bool	reply( Stream*, int );
-bool	refuse( Stream* );
+bool	refuse( Stream*, ClassAd* replyAd, bool retry_ok=false);
+bool	refuseX( Stream*, CONDOR_HOLD_CODE code, const char * reason, bool retry_ok=false);
+inline bool refuse(Stream* s, bool send_failure_ad, CONDOR_HOLD_CODE code, const char * reason) {
+	return refuseX(s, code, send_failure_ad ? reason : nullptr, false);
+}
 bool	caInsert( ClassAd* target, ClassAd* source, const char* attr,
 				  const char* prefix = NULL );
 bool	caRevertToParent(ClassAd* target, const char * attr);
 // delete in chained ad, and also in parent ad (ClassAd::Delete does not delete in parent)
 void	caDeleteThruParent(ClassAd* target, const char * attr, const char * prefix = NULL);
-bool	configInsert( ClassAd* ad, const char* attr, bool is_fatal );
+bool	configInsert( ClassAd* ad, const char* attr, bool is_fatal, const char *default_value = nullptr);
 bool	configInsert( ClassAd* ad, const char* param_name, 
-					  const char* attr, bool is_fatal );
+					  const char* attr, bool is_fatal, const char *default_value = nullptr );
 Resource* stream_to_rip( Stream* );
 
 VacateType getVacateType( ClassAd* ad );
@@ -54,10 +58,11 @@ VacateType getVacateType( ClassAd* ad );
 //
 class CleanupReminder {
 public:
-	enum category { exec_dir=0, account };
+	enum category { logical_volume=0, exec_dir, account };
 	std::string name; // name of resource to cleanup
 	category cat;  // category of resource, e.g execute dir, account name, etc.
 	int      opt; // options, meaning depends on category
+	unsigned int broken_id{0}; // Associated broken item to act upon if cleanup is successfull
 
 	// so that some of the reminders can be case sensitive, and some not.
 	// in the future, we may consider cat as well as OS in this function, so prepare for that now.
@@ -97,6 +102,15 @@ public:
 		}
 	}
 
+	const char* Type() const {
+		switch (cat) {
+			case category::exec_dir: { return "directory"; }
+			case category::account: { return "account"; }
+			case category::logical_volume: { return "logical volume"; }
+			default: { return "unknown"; }
+		}
+	}
+
 };
 
 // map the cleanup reminder to a number of iterations of the cleanup loop.
@@ -104,9 +118,9 @@ public:
 // on ever iteration.
 typedef std::map<CleanupReminder, int> CleanupReminderMap;
 
-void add_exec_dir_cleanup_reminder(const std::string & dir, int options);
-void add_account_cleanup_reminder(const std::string& name);
+void add_cleanup_reminder(const std::string& item, CleanupReminder::category cat, int opts=0);
 
+bool retry_cleanup_logical_volume(const std::string& lv_name, int options, int& err);
 bool retry_cleanup_execute_dir(const std::string & path, int options, int &err);
 bool retry_cleanup_user_account(const std::string & path, int options, int &err);
 
