@@ -28,14 +28,15 @@
 #include "condor_daemon_client.h"
 #include "condor_daemon_core.h"
 #include "classad_helpers.h"
-#include "ToE.h"
+#include "NTsenders.h"
+#include "shortfile.h"
 
 #ifdef HAVE_SCM_RIGHTS_PASSFD
 #include "shared_port_scm_rights.h"
 #include "fdpass.h"
 #endif
 
-extern Starter *Starter;
+extern class Starter *starter;
 
 static void buildExtraVolumes(std::list<std::string> &extras, ClassAd &machAd, ClassAd &jobAd);
 
@@ -96,11 +97,19 @@ DockerProc::~DockerProc() {
 	}
 }
 
-int DockerProc::StartJob() {
+static
+std::string credentials_dir() {
+	std::string dir {"/tmp/docker_config."};
+	dir += std::to_string(getpid());
+	return dir;
+}
+
+int 
+DockerProc::StartJob() {
 	std::string imageID;
 
 	// Not really ready for ssh-to-job'ing until we start the container
-	Starter->SetJobEnvironmentReady(false);
+	starter->SetJobEnvironmentReady(false);
 
 	if( ! JobAd->LookupString( ATTR_DOCKER_IMAGE, imageID ) ) {
 		if (! JobAd->LookupString(ATTR_CONTAINER_IMAGE, imageID)) {
@@ -113,36 +122,119 @@ int DockerProc::StartJob() {
 		imageID = imageID.substr(9);
 	}
 
+	imageName = imageID;
+
+	// The GlobalJobID is unsuitable by virtue its octothorpes.  This
+	// construction is informative, but could be made even less likely
+	// to collide if it had a timestamp.
+	formatstr( containerName, "HTCJob%d_%d_%s_PID%d",
+		starter->jic->jobCluster(),
+		starter->jic->jobProc(),
+		starter->getMySlotName().c_str(), // note: this can be "" for single slot machines.
+		getpid() );
+
+
+	// Grab credentials from shadow, store them in a file,
+	// as docker pull requires them in a file named "config.json"
+	// in a directory pointed at by DOCKER_CONFIG
+	bool use_creds = false;
+	JobAd->LookupBool(ATTR_DOCKER_SEND_CREDENTIALS, use_creds);
+	if (use_creds) {
+		
+		// Grab the creds from the shadow
+		// Now write out just the auths as condor, which runs docker container create
+		ClassAd query; // not used now
+		ClassAd creds; // creds from the shadow
+
+		int r = starter->jic->fetch_docker_creds(query, creds);
+
+		std::string creds_error;
+		creds.LookupString("HTCondorError", creds_error);
+
+		if ((r == 0) && (creds_error.empty())) {
+			creds_error = "Cannot connect to shadow to fetch docker creds";
+		}
+
+		if (!creds_error.empty()) {
+			dprintf(D_ALWAYS, "Error fetching docker credentials: %s\n", creds_error.c_str());
+			starter->jic->holdJob(creds_error.c_str(), CONDOR_HOLD_CODE::InvalidDockerImage, 0);
+			return FALSE;
+		}
+		std::string tmp_dir = credentials_dir();
+
+		// mkdir as user condor...
+		int ret = mkdir(tmp_dir.c_str(), 0700);
+		if (ret != 0) {
+			dprintf(D_ALWAYS, "Cannot make directory for docker creds: %s\n", strerror(errno));
+			return FALSE;
+		}
+
+		// and a file named "config.json" in that directory
+		std::string config_output_filename;
+		config_output_filename = tmp_dir + "/config.json";
+
+		std::string auths_str;
+		classad::ClassAdJsonUnParser cajup;
+		cajup.Unparse(auths_str, &creds);
+
+		bool good = htcondor::writeShortFile(config_output_filename, auths_str);
+		if (!good) {
+			// This is an error on the EP side, do not hold the job
+			dprintf(D_ALWAYS, "Cannot write docker creds: %s\n", strerror(errno));
+			return FALSE;
+		}
+	}
+	// Get all our cached docker images
+	std::vector<DockerAPI::ImageInfo> imageInfos = DockerAPI::getImageInfos();
+	std::string annotatedImageName = DockerAPI::toAnnotatedImageName(imageName, *JobAd);
+	if (annotatedImageName.empty()) {
+		dprintf(D_ALWAYS, "DockerProc::Start job, cannot convert %s to a annotated image name\n", imageName.c_str());
+		return FALSE;
+	}
+
+	// See if we've already got this image annotation
+	auto it = std::ranges::find(imageInfos, annotatedImageName, &DockerAPI::ImageInfo::imageName);
+	if (imageInfos.end() ==  it) {
+		return PullImage();
+	} else {
+		// And do the real work of creating the container and launching it
+		return LaunchContainer();
+	}
+}
+
+int 
+DockerProc::LaunchContainer() {
+
 	std::string command;
 	JobAd->LookupString(ATTR_JOB_CMD, command);
 	dprintf(D_FULLDEBUG, "%s: '%s'\n", ATTR_JOB_CMD, command.c_str());
 
-	std::string sandboxPath = Starter->jic->jobRemoteIWD();
+	std::string sandboxPath = starter->jic->jobRemoteIWD();
 
 #ifdef WIN32
 	#if 1
 	// TODO: make this configurable? and the same on Windows/Linux
 	// TODO: make it settable by the job so that we can support Windows containers on Windows?
 	std::string innerdir("/test/execute/");
-	Starter->SetInnerWorkingDir(innerdir.c_str());
+	starter->SetInnerWorkingDir(innerdir.c_str());
 	#else
-	const char * outerdir = Starter->GetWorkingDir(false);
+	const char * outerdir = starter->GetWorkingDir(false);
 	std::string innerdir(outerdir);
 	const char * tmp = strstr(outerdir, "\\execute\\");
 	if (tmp) {
 		innerdir = tmp;
 		std::replace(innerdir.begin(), innerdir.end(), '\\', '/');
-		Starter->SetInnerWorkingDir(innerdir.c_str());
+		starter->SetInnerWorkingDir(innerdir.c_str());
 	}
 	#endif
 #else
 	// TODO: make this work on Linux also
-	std::string innerdir = Starter->jic->jobRemoteIWD();
+	std::string innerdir = starter->jic->jobRemoteIWD();
 #endif
 
 	bool transferExecutable = true;
 	JobAd->LookupBool( ATTR_TRANSFER_EXECUTABLE, transferExecutable );
-	if( Starter->jic->usingFileTransfer() && transferExecutable ) {
+	if( starter->jic->usingFileTransfer() && transferExecutable ) {
 		std::string old_cmd = command;
 		formatstr(command, "./%s", condor_basename(old_cmd.c_str()));
 	}
@@ -165,71 +257,61 @@ int DockerProc::StartJob() {
 
 	Env job_env;
 	std::string env_errors;
-	if( !Starter->GetJobEnv(JobAd,&job_env, env_errors) ) {
+	if( !starter->GetJobEnv(JobAd,&job_env, env_errors) ) {
 		dprintf( D_ALWAYS, "Aborting DockerProc::StartJob: %s\n", env_errors.c_str());
 		return 0;
 	}
 
-	// The GlobalJobID is unsuitable by virtue its octothorpes.  This
-	// construction is informative, but could be made even less likely
-	// to collide if it had a timestamp.
-	formatstr( containerName, "HTCJob%d_%d_%s_PID%d",
-		Starter->jic->jobCluster(),
-		Starter->jic->jobProc(),
-		Starter->getMySlotName().c_str(), // note: this can be "" for single slot machines.
-		getpid() );
-
 	ClassAd recoveryAd;
 	recoveryAd.Assign("DockerContainerName", containerName);
-	Starter->WriteRecoveryFile(&recoveryAd);
+	starter->WriteRecoveryFile(&recoveryAd);
 
 	int childFDs[3] = { 0, 0, 0 };
 	{
 	TemporaryPrivSentry sentry(PRIV_USER);
-	std::string workingDir = Starter->GetWorkingDir(0);
+	std::string workingDir = starter->GetWorkingDir(0);
 	//std::string DockerOutputFile = workingDir + "/docker_stdout";
-	std::string DockerErrorFile  = workingDir + "/docker_stderror";
 
 	//childFDs[1] = open(DockerOutputFile.c_str(), O_WRONLY | O_CREAT | O_TRUNC, 0666);
-	childFDs[2] = open(DockerErrorFile.c_str(), O_WRONLY | O_CREAT | O_TRUNC, 0666);
+	childFDs[2] = open(DockerErrorFile().c_str(), O_WRONLY | O_CREAT | O_TRUNC, 0666);
 	}
 
 	  // Ulog the execute event
-	Starter->jic->notifyJobPreSpawn();
+	starter->jic->notifyJobPreSpawn();
 
 	CondorError err;
 	// DockerAPI::createContainer() returns a PID from daemonCore->Create_Process(), which
 	// makes it suitable for passing up into VanillaProc.  This combination
 	// will trigger the reaper(s) when the container terminates.
 	
-	ClassAd *machineAd = Starter->jic->machClassAd();
+	ClassAd *machineAd = starter->jic->machClassAd();
 
 	std::list<std::string> extras;
-	std::string scratchDir = Starter->GetWorkingDir(0);
+	std::string slotDir = starter->GetSlotDir();
 
-	// map the scratch dir inside the container
-	extras.push_back(scratchDir + ":" + scratchDir);
+	// map the slot dir inside the container
+	extras.push_back(slotDir + ":" + slotDir);
 
-	// if file xfer is off, also map the iwd
-	std::string iwd = Starter->jic->jobRemoteIWD();
-	if (iwd != scratchDir) {
-		extras.push_back(iwd + ":" + iwd);
-	}
+	// also map the iwd.  Might be under the slot
+	// dir, but that's ok
+	std::string iwd = starter->jic->jobRemoteIWD();
+	extras.push_back(iwd + ":" + iwd);
 
 	buildExtraVolumes(extras, *machineAd, *JobAd);
 
-	int *affinity_mask = makeCpuAffinityMask(Starter->getMySlotNumber());
+	int *affinity_mask = makeCpuAffinityMask(starter->getMySlotNumber());
 
 	// The following line is for condor_who to parse
 	dprintf( D_ALWAYS, "About to exec docker:%s\n", command.c_str());
 	int rv = DockerAPI::createContainer( *machineAd, *JobAd,
-		containerName, imageID,
-		command, args, job_env,
-		sandboxPath, innerdir,
-		extras, JobPid, childFDs,
-		shouldAskForServicePorts, err, affinity_mask);
+			containerName, imageName,
+			command, args, job_env,
+			sandboxPath, innerdir,
+			extras, credentials_dir(), JobPid, childFDs,
+			shouldAskForServicePorts, err, affinity_mask);
+
 	if( rv < 0 ) {
-		dprintf( D_ERROR, "DockerAPI::createContainer( %s, %s, ... ) failed with return value %d\n", imageID.c_str(), command.c_str(), rv );
+		dprintf( D_ERROR, "DockerAPI::createContainer( %s, %s, ... ) failed with return value %d\n", imageName.c_str(), command.c_str(), rv );
 		handleFTL(rv);
 		return FALSE;
 	}
@@ -244,6 +326,45 @@ int DockerProc::StartJob() {
 	return TRUE;
 }
 
+int
+DockerProc::PullImage() {
+	CondorError err;
+	int pullReaperId = daemonCore->Register_Reaper("PullReaper", (ReaperHandlercpp)&DockerProc::PullReaper,
+			"PullReaper", this);
+	int r = DockerAPI::pullImage(imageName, credentials_dir(), starter->GetWorkingDir(0), *JobAd, pullReaperId, err);
+	if (r == 0) {
+		return TRUE;
+	} else {
+		return FALSE;
+	}
+}
+
+int
+DockerProc::PullReaper(int pid, int status) {
+	dprintf(D_FULLDEBUG, "DockerProc::pullReaper fired for pid %d with status %d\n", pid, status);
+	if (status == 0) {
+		bool success = LaunchContainer();
+		if (!success) {
+			starter->jic->holdJob("Unable to Launch Container", CONDOR_HOLD_CODE::InvalidDockerImage, 0);
+		}
+		return 0;
+	} else {
+		std::string message;
+		formatstr(message, "Cannot pull image %s", imageName.c_str());
+		std::string pull_stderror;
+		{
+			TemporaryPrivSentry sentry(PRIV_ROOT);
+			htcondor::readShortFile(DockerErrorFile(), pull_stderror);
+		}
+		std::erase(pull_stderror, '\n');
+		message += ' ' + pull_stderror;
+		dprintf(D_ALWAYS, "%s\n", message.c_str());
+		starter->jic->holdJob(message.c_str(), CONDOR_HOLD_CODE::InvalidDockerImage, 0);
+		return 0;
+	}
+	return 0;
+}
+
 int execPid;
 ReliSock *ns;
 
@@ -264,6 +385,17 @@ bool DockerProc::JobReaper( int pid, int status ) {
 	dprintf( D_FULLDEBUG, "DockerProc::JobReaper() pid is %d status is %d wait_for_Create is %d\n", pid, status, waitForCreate);
 
 	if (waitForCreate) {
+		// Cleanup the credentials we no longer need
+		// Should be done as user Condor
+		std::string creds_dir = credentials_dir();
+		std::string creds_file = creds_dir + "/config.json";
+
+		{
+			TemporaryPrivSentry sentry(PRIV_CONDOR);
+			std::ignore = remove(creds_file.c_str());
+			std::ignore = rmdir(creds_dir.c_str());
+		}
+
 		// When we get here, the docker create container has exited
 
 		if (status != 0) {
@@ -275,8 +407,7 @@ bool DockerProc::JobReaper( int pid, int status ) {
 
 			{
 			TemporaryPrivSentry sentry(PRIV_USER);
-			std::string fileName = Starter->GetWorkingDir(0);
-			fileName += "/docker_stderror";
+			std::string fileName = DockerErrorFile();
 			int fd = open(fileName.c_str(), O_RDONLY, 0000);
 			if (fd >= 0) {
 				int r = read(fd, buf, 511);
@@ -292,14 +423,14 @@ bool DockerProc::JobReaper( int pid, int status ) {
 				}
 				close(fd);
 			} else {
-				dprintf(D_ALWAYS, "Cannot open docker_stderror\n");
+				dprintf(D_ALWAYS, "Cannot open %s\n", DockerErrorFile().c_str() );
 			}
 			}
 			message = buf;
-			Starter->jic->holdJob(message.c_str(), CONDOR_HOLD_CODE::InvalidDockerImage, 0);
+			starter->jic->holdJob(message.c_str(), CONDOR_HOLD_CODE::InvalidDockerImage, 0);
 			{
 			TemporaryPrivSentry sentry(PRIV_USER);
-			unlink("docker_stderror");
+			unlink(".docker_stderror");
 			}
 			return VanillaProc::JobReaper( pid, status );
 		}
@@ -308,6 +439,17 @@ bool DockerProc::JobReaper( int pid, int status ) {
 		waitForCreate = false;
 		dprintf(D_FULLDEBUG, "DockerProc::JobReaper docker create (pid %d) exited with status %d\n", pid, status);
 
+		// Now tag the image we just downloaded (or re-used) with a tag annotation. 
+		// This let's us know that this image is one of ours.  If the tag already
+		// exists, this is not an error, but the LastTagModified field is updated.
+		//
+		std::string annotatedImage = DockerAPI::toAnnotatedImageName(imageName, *JobAd);
+		if (!annotatedImage.empty()) {
+			DockerAPI::tag(imageName, annotatedImage);
+		} else {
+			dprintf(D_ALWAYS, "DockerProc:: cannot create annotated image from image names %s\n", imageName.c_str());
+		}
+
 		// It would be nice if we could find out what port Docker selected
 		// for search service (if any) before we actually started the job,
 		// but (understandably) Docker doesn't do that.
@@ -316,8 +458,8 @@ bool DockerProc::JobReaper( int pid, int status ) {
 		#ifdef COPY_INPUT_SANDBOX
 		// copy the input sandbox into the container
 		{
-			std::string workingDir = Starter->GetWorkingDir(0);
-			std::string innerPath = Starter->GetWorkingDir(true);
+			std::string workingDir = starter->GetWorkingDir(0);
+			std::string innerPath = starter->GetWorkingDir(true);
 			std::vector<std::string> opts{"-a"};
 
 			//TODO: figure out if we need to do this, or to switch to  PRIV_USER
@@ -334,7 +476,7 @@ bool DockerProc::JobReaper( int pid, int status ) {
 	#endif
 
 		// It seems like this should be done _after_ we call start Container().
-		Starter->SetJobEnvironmentReady(true);
+		starter->SetJobEnvironmentReady(true);
 
 
 		CondorError err;
@@ -369,6 +511,17 @@ bool DockerProc::JobReaper( int pid, int status ) {
 
 		{
 		TemporaryPrivSentry sentry(PRIV_ROOT);
+		std::string arch;
+		DockerAPI::getImageArch(imageName, arch);
+
+		if (!DockerAPI::imageArchIsCompatible(arch)) {
+			std::string message;
+			formatstr(message, "DockerProc::StartJob(): Image Architecture %s not compatible with this machine.", arch.c_str());
+			dprintf(D_ALWAYS, "%s\n", message.c_str());
+			starter->jic->holdJob(message.c_str(), CONDOR_HOLD_CODE::InvalidDockerImage, 0);
+			return false;
+		}
+
 		DockerAPI::startContainer( containerName, JobPid, childFDs, err );
 		}
 		condor_gettimestamp( job_start_time );
@@ -379,7 +532,7 @@ bool DockerProc::JobReaper( int pid, int status ) {
 				polling_interval, (TimerHandlercpp)&DockerProc::getStats, 
 					"DockerProc::getStats",this);
 
-		bool ssh_enabled = param_boolean("ENABLE_SSH_TO_JOB",true,true,Starter->jic->machClassAd(),JobAd);
+		bool ssh_enabled = param_boolean("ENABLE_SSH_TO_JOB",true,true,starter->jic->machClassAd(),JobAd);
 		if( ssh_enabled ) {
 			SetupDockerSsh();
 		}
@@ -456,6 +609,7 @@ bool DockerProc::JobReaper( int pid, int status ) {
 			} else {
 				dprintf(D_ERROR,"    Container %s remove failed -- does it even exist? If so, startd will remove on next boot.\n", containerName.c_str());
 			}
+			starter->SetVacateReason("Cannot inspect exited container", CONDOR_HOLD_CODE::ContainerError, 0);
 			EXCEPT("Cannot inspect exited container");
 		}
 
@@ -479,7 +633,7 @@ bool DockerProc::JobReaper( int pid, int status ) {
 		}
 
 		if (oomkilled.find("true") == 0) {
-			ClassAd *machineAd = Starter->jic->machClassAd();
+			ClassAd *machineAd = starter->jic->machClassAd();
 			int memory;
 			machineAd->LookupInteger(ATTR_MEMORY, memory);
 			std::string message;
@@ -487,15 +641,15 @@ bool DockerProc::JobReaper( int pid, int status ) {
 			dprintf(D_ALWAYS, "%s, going on hold\n", message.c_str());
 
 
-			Starter->jic->holdJob(message.c_str(), CONDOR_HOLD_CODE::JobOutOfResources, 0);
+			starter->jic->holdJob(message.c_str(), CONDOR_HOLD_CODE::JobOutOfResources, OUT_OF_RESOURCES_SUB_CODE::Memory);
 			DockerAPI::rm( containerName, error );
 
-			if ( Starter->Hold( ) ) {
-				Starter->allJobsDone();
+			if ( starter->Hold( ) ) {
+				starter->allJobsDone();
 				this->JobExit();
 			}
 
-			Starter->ShutdownFast();
+			starter->ShutdownFast();
 			return 0;
 		}
 
@@ -512,15 +666,15 @@ bool DockerProc::JobReaper( int pid, int status ) {
 			dprintf(D_ALWAYS, "%s, going on hold\n", message.c_str());
 
 
-			Starter->jic->holdJob(message.c_str(), CONDOR_HOLD_CODE::FailedToCreateProcess, 0);
+			starter->jic->holdJob(message.c_str(), CONDOR_HOLD_CODE::FailedToCreateProcess, 0);
 			DockerAPI::rm( containerName, error );
 
-			if ( Starter->Hold( ) ) {
-				Starter->allJobsDone();
+			if ( starter->Hold( ) ) {
+				starter->allJobsDone();
 				this->JobExit();
 			}
 
-			Starter->ShutdownFast();
+			starter->ShutdownFast();
 			return 0;
 		}
 
@@ -571,7 +725,7 @@ DockerProc::SetupDockerSsh() {
 	pipe_addr.sun_family = AF_UNIX;
 	unsigned pipe_addr_len;
 
-	std::string workingDir = Starter->GetWorkingDir(0);
+	std::string workingDir = starter->GetWorkingDir(0);
 	std::string pipeName = workingDir + "/.docker_sock";	
 
 	strncpy(pipe_addr.sun_path, pipeName.c_str(), sizeof(pipe_addr.sun_path)-1);
@@ -627,7 +781,7 @@ DockerProc::AcceptSSHClient(Stream *stream) {
 
 	Env env;
 	std::string env_errors;
-	if( !Starter->GetJobEnv(JobAd,&env, env_errors) ) {
+	if( !starter->GetJobEnv(JobAd,&env, env_errors) ) {
 		dprintf( D_ALWAYS, "Aborting DockerProc::exec: %s\n", env_errors.c_str());
 		return 0;
 	}
@@ -825,11 +979,18 @@ DockerProc::getStats( int /* timerID */ ) {
 
 			// Append serviceAd to the sandbox's copy of the job ad.
 			std::string jobAdFileName;
-			formatstr( jobAdFileName, "%s/.job.ad", Starter->GetWorkingDir(0) );
+			formatstr( jobAdFileName, "%s/.job.ad", starter->GetWorkingDir(0) );
 			{
 				TemporaryPrivSentry sentry(PRIV_ROOT);
 				// ... sigh ...
-				ToE::writeTag( & serviceAd, jobAdFileName );
+				FILE * jobAdFile = safe_fopen_wrapper_follow(jobAdFileName.c_str(), "a");
+				if(! jobAdFile) {
+					dprintf(D_ALWAYS, "Failed to write docker service ports to .job.ad file (%d): %s\n",
+					        errno, strerror(errno));
+				} else {
+					fPrintAd(jobAdFile, serviceAd);
+					fclose(jobAdFile);
+				}
 			}
 		}
 	}
@@ -882,7 +1043,11 @@ bool DockerProc::PublishUpdateAd( ClassAd * ad ) {
 
 
 // TODO: Implement.
-void DockerProc::PublishToEnv( Env * /* env */ ) {
+void DockerProc::PublishToEnv(Env *env) {
+	// Publish an vanilla universe job to the environment.
+	// this also takes care of pre/post scripts
+	VanillaProc::PublishToEnv(env);
+
 	dprintf( D_FULLDEBUG, "DockerProc::PublishToEnv()\n" );
 	return;
 }
@@ -927,7 +1092,7 @@ bool DockerProc::Version( std::string & version ) {
 
 	return foundVersion;
 }
-void 
+bool
 DockerProc::restartCheckpointedJob() {
 	TemporaryPrivSentry sentry(PRIV_ROOT);
 	CondorError error;
@@ -937,7 +1102,7 @@ DockerProc::restartCheckpointedJob() {
 		// Will fail later when we try to restart if it still exists.  If it doesn't :shrug: all good!
 	}
 
-	VanillaProc::restartCheckpointedJob();
+	return VanillaProc::restartCheckpointedJob();
 }
 
 // Generate a list of strings that are suitable arguments to
@@ -950,6 +1115,16 @@ static void buildExtraVolumes(std::list<std::string> &extras, ClassAd &machAd, C
 		param(scratchNames, "MOUNT_UNDER_SCRATCH");
 	} 
 
+	// Now add in scratch mounts requested by the job.
+	std::string job_mount_under_scratch;
+	jobAd.LookupString(ATTR_JOB_MOUNT_UNDER_SCRATCH, job_mount_under_scratch);
+	if (job_mount_under_scratch.length() > 0) {
+		if (scratchNames.length() > 0) {
+			scratchNames += ' ';
+		}
+		scratchNames += job_mount_under_scratch;
+	}
+
 #ifdef DOCKER_ALLOW_RUN_AS_ROOT
 		// If docker is allowing the user to be root, don't mount anything
 		// so that we can't create rootly files in shared places.
@@ -959,7 +1134,7 @@ static void buildExtraVolumes(std::list<std::string> &extras, ClassAd &machAd, C
 #endif
 
 	if (scratchNames.length() > 0) {
-		std::string workingDir = Starter->GetWorkingDir(0);
+		std::string workingDir = starter->GetWorkingDir(0);
 			// Foreach scratch name...
 		for (const auto &scratchName: StringTokenIterator(scratchNames)) {
 			std::string hostdirbuf;
@@ -994,7 +1169,7 @@ static void buildExtraVolumes(std::list<std::string> &extras, ClassAd &machAd, C
 			if (strchr(volumePath, ':') == 0) {
 				// Must have a colon.  If none, assume we meant
 				// source:source
-				char *volumePath2 = (char *)malloc(1 + 2 * strlen(volumePath));
+				char *volumePath2 = (char *)malloc(2 + 2 * strlen(volumePath));
 				strcpy(volumePath2, volumePath);
 				strcat(volumePath2, ":");
 				strcat(volumePath2, volumePath);
@@ -1029,4 +1204,5 @@ static void buildExtraVolumes(std::list<std::string> &extras, ClassAd &machAd, C
 			dprintf(D_ALWAYS, "WARNING: DOCKER_VOLUME_DIR_%s is missing in config file.\n", volumeName.c_str());
 		}
 	}
+	free(volumeNames);
 }
