@@ -3,14 +3,20 @@
 
 #include "vanilla_proc.h"
 
+#include "starter.h"
+extern Starter *starter;
+
 class DockerProc : public VanillaProc {
 	public:
 		DockerProc( ClassAd * jobAd );
 		virtual ~DockerProc();
 
 		virtual int StartJob();
+		virtual int LaunchContainer();
+		virtual int PullImage();
 		virtual bool JobReaper( int pid, int status );
 		virtual int ExecReaper( int pid, int status );
+		virtual int PullReaper( int pid, int status );
 		virtual bool JobExit();
 
 		virtual void Suspend();
@@ -38,13 +44,17 @@ class DockerProc : public VanillaProc {
 		static bool Detect();
 		static bool Version( std::string & version );
 
+		std::string DockerErrorFile() { 
+			return std::string(starter->GetWorkingDir(0)) + "/.docker_stderror";
+		}
 	protected:
-		virtual void restartCheckpointedJob();
+		virtual bool restartCheckpointedJob();
 	private:
 
 		ReliSock listener;
 		std::string containerID;
 		std::string containerName;
+		std::string imageName;
 		int updateTid;
 		uint64_t memUsage, max_memUsage, netIn, netOut, userCpu, sysCpu;
 		bool waitForCreate;
