@@ -32,7 +32,7 @@
 #include "subsystem_info.h"
 
 
-extern Starter *Starter;
+extern class Starter *starter;
 
 
 JobInfoCommunicator::JobInfoCommunicator()
@@ -84,6 +84,9 @@ JobInfoCommunicator::~JobInfoCommunicator()
 	if( mach_ad ) {
 		delete mach_ad;
 	}
+	if( machine_secrets_ad ) {
+		delete machine_secrets_ad;
+	}
 	if( u_log ) {
 		delete u_log;
 	}
@@ -306,7 +309,7 @@ JobInfoCommunicator::allJobsDone( void )
 		static ClassAd* job_exit_ad = NULL;
 		if (!job_exit_ad) {
 			job_exit_ad = new ClassAd(*job_ad);
-			Starter->publishJobExitAd(job_exit_ad);
+			starter->publishJobExitAd(job_exit_ad);
 		}
 		const char* exit_reason = getExitReasonString();
 		int rval = m_hook_mgr->tryHookJobExit(job_exit_ad, exit_reason);
@@ -365,7 +368,7 @@ JobInfoCommunicator::finishAllJobsDone( void )
 		// Record the fact the hook finished.
 	m_allJobsDone_finished = true;
 		// Tell the starter to try job cleanup again so it can move on.
-	Starter->allJobsDone();
+	starter->allJobsDone();
 }
 
 
@@ -404,7 +407,6 @@ JobInfoCommunicator::gotHold( void )
 	had_hold = true;
 }
 
-
 void
 JobInfoCommunicator::setOutputAdFile( const char* path )
 {
@@ -519,7 +521,7 @@ JobInfoCommunicator::initOutputAdFile( void )
 		if( job_output_ad_file[0] == '-' && job_output_ad_file[1] == '\0' ) {
 			job_output_ad_is_stdout = true;
 		} else if( ! fullpath(job_output_ad_file) ) {
-			std::string path = Starter->GetWorkingDir(0);
+			std::string path = starter->GetWorkingDir(0);
 			path += DIR_DELIM_CHAR;
 			path += job_output_ad_file;
 			free( job_output_ad_file );
@@ -530,7 +532,7 @@ JobInfoCommunicator::initOutputAdFile( void )
 	}
 	if( ! m_job_update_ad_file.empty() ) {
 		if( ! fullpath(m_job_update_ad_file.c_str()) ) {
-			std::string path = Starter->GetWorkingDir(0);
+			std::string path = starter->GetWorkingDir(0);
 			path += DIR_DELIM_CHAR;
 			path += m_job_update_ad_file;
 			m_job_update_ad_file = path;
@@ -704,15 +706,21 @@ JobInfoCommunicator::initUserPrivWindows( void )
 	bool init_priv_succeeded = true;
 	bool run_as_owner = allowRunAsOwner( false, false );
 
-	if( !name ) {	
-		if ( run_as_owner ) {
+	if (run_as_owner) {
+		char *run_jobs_as;
+		if (job_ad->LookupString(ATTR_OS_USER, &run_jobs_as)) {
+			std::string buf;
+			name = strdup(name_of_user(run_jobs_as, buf));
+			domain = strdup(domain_of_user(run_jobs_as, ""));
+			free(run_jobs_as);
+		} else {
 			job_ad->LookupString(ATTR_OWNER,&name);
 			job_ad->LookupString(ATTR_NT_DOMAIN,&domain);
 		}
 	}
 
 	if ( !name ) {
-		std::string slotName = Starter->getMySlotName();
+		std::string slotName = starter->getMySlotName();
 		upper_case(slotName);
 		slotName += "_USER";
 		char *run_jobs_as = param(slotName.c_str());
@@ -798,15 +806,26 @@ JobInfoCommunicator::initJobInfo( void )
 void
 JobInfoCommunicator::checkForStarterDebugging( void )
 {
-	if( ! job_ad ) {
-		EXCEPT( "checkForStarterDebugging() called with no job ad!" );
+	ASSERT(job_ad);
+
+		// for testing, see if there's an attribute that requests the starter return a broken exit code
+		// which we will honor, only if the knob is in a non-default state and the starter would
+		// otherwise exit with a success code.
+	int tmp = 0;
+	if (job_ad->LookupInteger("StarterShouldExitBroken", tmp) && tmp) {
+		if (param_boolean("TEST_JOB_CAN_BREAK_SLOT", false)) {
+			dprintf(D_STATUS, "TEST_JOB_CAN_BREAK_SLOT=true and job has StarterShouldExitBroken=%d. this slot gonna break!\n", tmp);
+			starter->JobRequestsBrokenExit(true);
+		} else {
+			dprintf(D_ALWAYS, "job has StarterShouldExitBroken=%d. This will be ignored because TEST_JOB_CAN_BREAK_SLOT is false\n", tmp);
+		}
 	}
 
 		// For debugging, see if there's a special attribute in the
 		// job ad that sends us into an infinite loop, waiting for
 		// someone to attach with a debugger
 	volatile int starter_should_wait = 0;
-	int tmp = 0; // Can't pass volatile int into LookupInteger
+	tmp = 0; // Can't pass volatile int into LookupInteger
 	job_ad->LookupInteger( ATTR_STARTER_WAIT_FOR_DEBUG, tmp );
 	starter_should_wait = tmp;
 	if( starter_should_wait ) {
@@ -867,37 +886,43 @@ JobInfoCommunicator::setupCompleted(int status, const struct UnreadyReason * pur
 	if (status != 0) {
 		ASSERT(purea); // the UnreadyReason is not optional when setup fails
 		m_allJobsDone_finished = true; // so allJobsDone returns trivial true and does not run the exit hook
-		Starter->jobEnvironmentCannotReady(status, *purea);
+		starter->jobEnvironmentCannotReady(status, *purea);
 		return;
 	}
 
+		// If we made it here, either we're not compiled for hook
+		// support, or we didn't spawn a hook.  Either way, we're
+		// done and should tell the starter we're ready.
+		// The starter will finish setup and queue a timer to actually start the job
+	starter->jobEnvironmentReady();
+}
+
+
+void
+JobInfoCommunicator::runPrepareJobHook() {
 #if HAVE_JOB_HOOKS
 	if (m_hook_mgr) {
 		int rval = m_hook_mgr->tryHookPrepareJob();
 		switch (rval) {
-		case -1:   // Error
-			Starter->RemoteShutdownFast(0);
+		case -1: // Error
+			starter->RemoteShutdownFast(0);
 			return;
 			break;
 
-		case 0:    // Hook not configured
-				// Nothing to do, break out and finish.
+		case 0:  // Hook not configured
+			     // Nothing to do, break out and finish.
 			break;
 
-		case 1:    // Spawned the hook.
-				// We need to bail now, and let the handler call
-				// jobEnvironmentReady() when the hook returns.
+		case 1:  // Spawned the hook.
+			     // We need to bail now, and let the handler call
+			     // Starter::prepareJobHookDone() when the hook returns.
 			return;
 			break;
 		}
 	}
 #endif /* HAVE_JOB_HOOKS */
 
-		// If we made it here, either we're not compiled for hook
-		// support, or we didn't spawn a hook.  Either way, we're
-		// done and should tell the starter we're ready.
-		// The starter will finish setup and queue a timer to actually start the job
-	Starter->jobEnvironmentReady();
+	starter->prepareJobHookDone();
 }
 
 
@@ -944,9 +969,7 @@ JobInfoCommunicator::startUpdateTimer( void )
 		Register_Timer(interval,
 	      (TimerHandlercpp)&JobInfoCommunicator::periodicJobUpdateTimerHandler,
 		  "JobInfoCommunicator::periodicJobUpdateTimerHandler", this);
-	if( m_periodic_job_update_tid < 0 ) {
-		EXCEPT( "Can't register DC Timer!" );
-	}
+	ASSERT(m_periodic_job_update_tid >= 0);
 }
 
 int
