@@ -32,7 +32,7 @@
 #include "basename.h"
 
 
-extern Starter *Starter;
+extern class Starter *starter;
 
 
 /* ScriptProc class implementation */
@@ -40,11 +40,8 @@ extern Starter *Starter;
 ScriptProc::ScriptProc( ClassAd* ad, const char* proc_name )
 {
     dprintf ( D_FULLDEBUG, "In ScriptProc::ScriptProc()\n" );
-	if( proc_name ) {
-		name = strdup( proc_name );
-	} else {
-		EXCEPT( "Can't instantiate a ScriptProc without a name!" );
-	}
+	ASSERT(proc_name);
+	name = strdup( proc_name );
 	JobAd = ad;
 	is_suspended = false;
 	UserProc::initialize();
@@ -87,7 +84,7 @@ ScriptProc::StartJob()
 		// path to the binary, and don't try to chmod it.
 	std::string exe_path = "";
 	if( tmp != NULL && !fullpath( tmp ) ) {
-		exe_path += Starter->GetWorkingDir(0);
+		exe_path += starter->GetWorkingDir(0);
 		exe_path += DIR_DELIM_CHAR;
 	}
 
@@ -97,7 +94,7 @@ ScriptProc::StartJob()
 	free( tmp ); 
 	tmp = NULL;
 
-	if( Starter->isGridshell() ) {
+	if( starter->isGridshell() ) {
 			// if we're a gridshell, chmod() the binary, since globus
 			// probably transfered it for us and left it with bad
 			// permissions...
@@ -192,7 +189,7 @@ ScriptProc::StartJob()
 	}
 
 		// Now, let the starter publish any env vars it wants to add
-	Starter->PublishToEnv( &job_env );
+	starter->PublishToEnv( &job_env );
 
 
 		// TODO: Deal with port regulation stuff?
@@ -218,7 +215,7 @@ ScriptProc::StartJob()
 		// // // // // // 
 
 		// TODO?
-		// Starter->jic->notifyJobPreSpawn( name );
+		// starter->jic->notifyJobPreSpawn( name );
 
 		// compute job's renice value by evaluating the machine's
 		// JOB_RENICE_INCREMENT in the context of the job ad...
@@ -248,26 +245,18 @@ ScriptProc::StartJob()
 		core_size = (size_t)core_size_truncated;
 	}
 
-	JobPid = daemonCore->Create_Process(exe_path.c_str(), 
-	                                    args,
-	                                    PRIV_USER_FINAL,
-	                                    1,
-	                                    FALSE,
-	                                    FALSE,
-	                                    &job_env,
-	                                    Starter->jic->jobIWD(),
-	                                    NULL,
-	                                    NULL,
-	                                    NULL,
-	                                    NULL,
-	                                    nice_inc,
-	                                    NULL,
-	                                    DCJOBOPT_NO_ENV_INHERIT,
-	                                    core_size_ptr );
+    std::string create_process_err_msg;
+	OptionalCreateProcessArgs cpArgs(create_process_err_msg);
+	JobPid = daemonCore->CreateProcessNew( exe_path, args,
+		 cpArgs.priv(PRIV_USER_FINAL)
+		.wantCommandPort(FALSE).wantUDPCommandPort(FALSE)
+		.env(&job_env).cwd(starter->jic->jobIWD()).coreHardLimit(core_size_ptr)
+		.niceInc(nice_inc).jobOptMask(DCJOBOPT_NO_ENV_INHERIT)
+	);
 
 	//NOTE: Create_Process() saves the errno for us if it is an
 	//"interesting" error.
-	char const *create_process_error = NULL;
+	char const *create_process_error = nullptr;
 	int create_process_errno = errno;
 	if( JobPid == FALSE && errno ) {
 		create_process_error = strerror( errno );
@@ -286,7 +275,7 @@ ScriptProc::StartJob()
 			}
 			err_msg += ": ";
 			err_msg += create_process_error;
-			Starter->jic->notifyStarterError( err_msg.c_str(), true, CONDOR_HOLD_CODE::FailedToCreateProcess, create_process_errno );
+			starter->jic->notifyStarterError( err_msg.c_str(), true, CONDOR_HOLD_CODE::FailedToCreateProcess, create_process_errno );
 		}
 
 		EXCEPT( "Create_Process(%s,%s, ...) failed",
