@@ -25,7 +25,7 @@
 #include "starter.h"
 #include "directory.h"
 
-extern Starter *Starter;
+extern class Starter *starter;
 
 SSHDProc::SSHDProc(ClassAd* job_ad, const std::string &sess, bool delete_ad) : VanillaProc(job_ad), session_dir(sess)
 {
@@ -78,10 +78,10 @@ SSHDProc::JobReaper(int pid, int status)
 	JobAd->LookupBool("WantDocker", isDocker);
 
 	if (interactive && isDocker) {
-		dprintf(D_ALWAYS, "Ssh exitting from interactive docker job, shutting down\n");
-		Starter->RemoteRemove(0);
+		dprintf(D_ALWAYS, "Ssh exiting from interactive docker job, shutting down\n");
+		starter->RemoteRemove(0);
 		VanillaProc::JobReaper(pid, status);
-		Starter->ShutdownFast();
+		starter->ShutdownFast();
 		return true;
 	}
 
