@@ -23,8 +23,17 @@
 #include "condor_daemon_core.h"
 #include "user_proc.h"
 #include "job_info_communicator.h"
+#include "execute_dir_monitor.h"
 #include "exit.h"
 
+#if defined(LINUX) || defined(DARWIN)
+    // We don't test on BSD, so don't claim the hardlink code works there.
+    #define CFT_VERSION 2
+#else
+    #define CFT_VERSION 0
+#endif
+
+
 #if defined(LINUX)
 #include "../condor_startd.V6/VolumeManager.h"
 #endif
@@ -76,7 +85,7 @@ public:
 			several times if, say, an EXCEPT()ion happens while
 			shutting down, so be careful in the implementation.
 		*/
-	virtual void FinalCleanup();
+	virtual int FinalCleanup(int code);
 
 		/** Params for "EXECUTE" and other useful stuff 
 		 */
@@ -115,7 +124,7 @@ public:
 	virtual bool Remove( void );
 
 		/** Call Hold() on all elements in m_job_list */
-	int remoteHoldCommand( int cmd, Stream* s );
+	int remoteVacateCommand( int cmd, Stream* s );
 	virtual int RemoteHold(int);
 	virtual bool Hold( void );
 
@@ -144,6 +153,7 @@ public:
 		 * when it is the correct time to run the job
 		 */
 	virtual bool jobWaitUntilExecuteTime( void );
+	virtual bool skipJobImmediately( void );
 	
 		/**
 		 * Clean up any the timer that we have might
@@ -151,18 +161,25 @@ public:
 		 * there can only be one job on hold
 		 */
 	virtual bool removeDeferredJobs( void );
-		
+
 		/** Called by the JobInfoCommunicator whenever the job
 			execution environment is ready so we can actually spawn
 			the job.
 		*/
 	virtual int jobEnvironmentReady( void );
-	
+
 	virtual int jobEnvironmentCannotReady(int status, const struct UnreadyReason & urea);
 
+
+	static void requestGuidanceJobEnvironmentReady( Starter * s );
+	static void requestGuidanceJobEnvironmentUnready( Starter * s );
+
+	static void requestGuidanceSetupJobEnvironment( Starter * s, const ClassAd & context );
+	static void requestGuidanceCommandJobSetup( Starter * s, const ClassAd & context, std::function<void(void)> continue_conversation );
+
 		/**
-		 * 
-		 * 
+		 *
+		 *
 		 **/
 	virtual void SpawnPreScript( int timerID = -1 );
 
@@ -215,6 +232,10 @@ public:
 		}
 		return WorkingDir.c_str();
 	}
+
+	const char *GetSlotDir() const {
+		return SlotDir.c_str();
+	}
 		/* Should the temporary directory under Execute be expected to
 		 * exist?
 		 */
@@ -232,6 +253,9 @@ public:
 		}
 	}
 
+	// Get job working directory disk usage: return bytes used & num dirs + files
+	DiskUsage GetDiskUsage(bool exiting=false) const;
+
 		/** Publish all attributes we care about for our job
 			controller into the given ClassAd.  Walk through all our
 			UserProcs and have them publish.
@@ -262,7 +286,7 @@ public:
 
 		/** Open a file in the 'manifest' directory.
 		 */
-	FILE * OpenManifestFile(const char * filename);
+	FILE * OpenManifestFile(const char * filename, bool add_to_output = false);
 
 		/** Set up the complete environment for the job.  This includes
 			STARTER_JOB_ENVIRONMENT, the job ClassAd, and PublishToEnv()
@@ -291,8 +315,9 @@ public:
 	int numberOfJobs( void ) { return m_job_list.size(); };
 
 	bool isGridshell( void ) const {return is_gridshell;};
-#ifdef WIN32
+
 	bool hasEncryptedWorkingDir(void) { return has_encrypted_working_dir; }
+#ifdef WIN32
 	bool loadUserRegistry(const ClassAd * jobAd);
 #endif
 	const char* origCwd( void ) {return (const char*) orig_cwd;};
@@ -322,22 +347,45 @@ public:
 
 	int GetShutdownExitCode() const { return m_shutdown_exit_code; };
 	void SetShutdownExitCode( int code ) { m_shutdown_exit_code = code; };
-
-#ifdef HAVE_DATA_REUSE_DIR
-	htcondor::DataReuseDirectory * getDataReuseDirectory() const {return m_reuse_dir.get();}
-#else
-	htcondor::DataReuseDirectory * getDataReuseDirectory() const {return nullptr;}
-#endif
+	bool HasJobRequestedBrokenExit() const { return job_requests_broken_exit; }
+	void JobRequestsBrokenExit(bool b) { job_requests_broken_exit = b; }
 
 	void SetJobEnvironmentReady(const bool isReady) {m_job_environment_is_ready = isReady;}
 
 	virtual void RecordJobExitStatus(int status);
 
 	void setTmpDir(const std::string &dir) { this->tmpdir = dir;}
+
+	void SetVacateReason(const std::string& msg, int code, int subcode);
+	void jicNotifyStarterError( bool critical );
+
+	void ExceptHandler(const char* errmsg);
+
+		/*
+		  @param result Buffer in which to store claim id string from job.
+		  Returns false if no claim id could be found.
+		 */
+	bool getJobClaimId(std::string &result) const;
+
+	void prepareJobHookDone();
+
 protected:
 	std::vector<UserProc *> m_job_list;
 	std::vector<UserProc *> m_reaped_job_list;
 
+	// Code shared by the requestGuidance...() functions.
+	static bool handleJobEnvironmentCommand(
+		Starter * s,
+		const ClassAd & guidance,
+		std::function<void(void)> continue_conversation
+	);
+
+	static bool handleJobSetupCommand(
+		Starter * s,
+		const ClassAd & guidance,
+		std::function<void(const ClassAd & context)> continue_conversation
+	);
+
 	// JobEnvironmentCannotReady sets these to pass along the setup failure info that
 	// we want to report *after* we finish transfer of FailureFiles
 	int            m_setupStatus = 0; // 0 is success, non-zero indicates failure of job setup
@@ -351,6 +399,7 @@ protected:
 
 	bool recorded_job_exit_status{false};
 	int job_exit_status;
+
 private:
 
 		// // // // // // // //
@@ -358,7 +407,8 @@ private:
 		// // // // // // // //
 
 		/// Remove the execute/dir_<pid> directory
-	virtual bool removeTempExecuteDir( void );
+		/// Argument exit_code: override Starter exit code with value
+	virtual bool removeTempExecuteDir(int& exit_code, const char * move_to);
 
 		/**
 		   Iterate through a UserProc list and have each UserProc
@@ -381,12 +431,6 @@ private:
 		 */
 	void getJobOwnerFQUOrDummy(std::string &result) const;
 
-		/*
-		  @param result Buffer in which to store claim id string from job.
-		  Returns false if no claim id could be found.
-		 */
-	bool getJobClaimId(std::string &result) const;
-
 
 	bool WriteAdFiles() const;
 
@@ -394,6 +438,9 @@ private:
 		// Private Data Members
 		// // // // // // // //
 
+	// Monitor for job working dir for disk usage
+	ExecDirMonitor* dirMonitor;
+
 	int jobUniverse;
 
 		// The base EXECUTE directory for this slot
@@ -401,23 +448,28 @@ private:
 		// The temporary directory created under Execute for this job.
 		// If file transfer is used, this will also be the IWD of the job.
 	std::string WorkingDir;
+	// Same as WorkingDir if nested scratch is not enabled, otherwise immediate subdir of EXECUTE
+	std::string SlotDir;
+	std::string JobHomeDir; // Owned by user and writeable thereby
 	std::string InnerWorkingDir; // if non-empty, this is the jobs view if the working dir
+	std::string tmpdir; // The string to set the tmp env vars to
 	char *orig_cwd;
 	std::string m_recoveryFile;
+	std::string m_move_working_dir_on_exit; // if non-empty move/rename the WorkingDir to this instead of deleting it
 	bool is_gridshell;
+	bool job_requests_broken_exit{false};
 	bool m_workingDirExists;
-#ifdef WIN32
 	bool has_encrypted_working_dir;
-#endif
-#ifdef LINUX
-	std::unique_ptr<VolumeManager::Handle> m_lv_handle;
-#endif // LINUX
 
 	int ShuttingDown;
 	int starter_stdin_fd;
 	int starter_stdout_fd;
 	int starter_stderr_fd;
-	
+
+	std::string m_vacateReason;
+	int m_vacateCode{0};
+	int m_vacateSubcode{0};
+
 		//
 		// When set to true, that means the Starter was asked to
 		// suspend all jobs. This is used when jobs are started up
@@ -435,13 +487,14 @@ private:
 		// When HTCondor manages dedicated disk space, this tracks
 		// the maximum permitted disk usage and the polling timer
 		//
-#if defined(LINUX)
+#ifdef LINUX
 	void CheckLVUsage( int timerID = -1 );
+	std::unique_ptr<VolumeManager::Handle> m_lv_handle;
 	int64_t m_lvm_lv_size_kb{0};
 	time_t m_lvm_last_space_issue{-1};
 	int m_lvm_poll_tid{-1};
 	bool m_lvm_held_job{false};
-#endif
+#endif /* LINUX */
 
 	UserProc* pre_script;
 	UserProc* post_script;
@@ -460,14 +513,6 @@ private:
 		// When doing a ShutdownFast or ShutdownGraceful, what should the
 		// starter's exit code be?
 	int m_shutdown_exit_code;
-
-#ifdef HAVE_DATA_REUSE_DIR
-	// Manage the data reuse directory.
-	std::unique_ptr<htcondor::DataReuseDirectory> m_reuse_dir;
-#endif
-
-	// The string to set the tmp env vars to
-	std::string tmpdir;
 };
 
 #define SANDBOX_STARTER_LOG_FILENAME ".starter.log"
