@@ -39,17 +39,19 @@
 #include "docker_proc.h"
 #include "condor_getcwd.h"
 #include "singularity.h"
-
+#include "which.h"
+#include "../condor_startd.V6/VolumeManager.h"
 
 extern "C" int exception_cleanup(int,int,const char*);	/* Our function called by EXCEPT */
 JobInfoCommunicator* parseArgs( int argc, char* argv [] );
 
 static Starter StarterObj;
-Starter *Starter = &StarterObj;
+Starter *starter = &StarterObj;
 
 extern int Foreground;	// from daemoncore
 static bool is_gridshell = false;
 
+
 // this appears at the bottom of this file:
 int display_dprintf_header(char **buf,int *bufpos,int *buflen);
 static char* dprintf_header = NULL;
@@ -92,12 +94,23 @@ printClassAd( void )
 	}
 	printf( "%s = True\n", ATTR_HAS_PER_FILE_ENCRYPTION );
 	printf( "%s = True\n", ATTR_HAS_RECONNECT );
-	printf( "%s = True\n", ATTR_HAS_MPI );
 	printf( "%s = True\n", ATTR_HAS_TDP );
 	printf( "%s = True\n", ATTR_HAS_JOB_DEFERRAL );
-    printf( "%s = True\n", ATTR_HAS_TRANSFER_INPUT_REMAPS );
-    printf( "%s = True\n", ATTR_HAS_SELF_CHECKPOINT_TRANSFERS );
+	printf( "%s = True\n", ATTR_HAS_TRANSFER_INPUT_REMAPS );
+	printf( "%s = True\n", ATTR_HAS_SELF_CHECKPOINT_TRANSFERS );
+	printf( "%s = True\n", ATTR_HAS_OS_USER );
 
+	// Include HasCommonFilesTransfer has an integer (version number)
+	// that therefore always also evaluates to boolean true.
+	if( CFT_VERSION != 0 ) {
+		printf( "%s = %d\n", ATTR_HAS_COMMON_FILES_TRANSFER, CFT_VERSION );
+	}
+
+	if (param_boolean("NO_JOB_NETWORKING", false)) {
+		printf("%s = False\n", ATTR_HAS_JOB_NETWORKING);
+	} else {
+		printf("%s = True\n", ATTR_HAS_JOB_NETWORKING);
+	}
 		/*
 		  Attributes describing what kinds of Job Info Communicators
 		  this starter has.  This is mostly for COD, but someday might
@@ -144,6 +157,12 @@ printClassAd( void )
 		//  done by vmuniverse manager in startd.
 		// ATTR_HAS_VM may be overwritten by vmuniverse manager in startd
 		printf( "%s = True\n",ATTR_HAS_VM );
+
+		// VM universe does not work with LVM using mount namespaces to hide mounts
+		if (VolumeManager::DetectLVM() && VolumeManager::GetHideMount() == LVM_ALWAYS_HIDE_MOUNT) {
+			printf("%s = False\n", ATTR_HAS_VM);
+			printf("%s = \"LVM configured to hide scratch dir via LVM_HIDE_MOUNT, which prohibits VM universe jobs\"\n", ATTR_VM_OFFLINE_REASON);
+		}
 	}
 
 	// Docker "universe."
@@ -158,8 +177,15 @@ printClassAd( void )
 			if (dockerVersion.find("20.10.4,") != std::string::npos) {
 				dprintf(D_ALWAYS, "Docker Version 20.10.4 detected.  This version cannot work with HTCondor.  Please upgrade docker to get Docker universe support\n");
 				printf( "%s = False\n", ATTR_HAS_DOCKER );
+				printf("%s = \"Unsupported Docker Version (20.10.4) detected\"\n", ATTR_DOCKER_OFFLINE_REASON);
 			}
 		}
+
+		// Docker universe does not work with LVM using mount namespaces to hide mounts
+		if (VolumeManager::DetectLVM() && VolumeManager::GetHideMount() == LVM_ALWAYS_HIDE_MOUNT) {
+			printf("%s = False\n", ATTR_HAS_DOCKER);
+			printf("%s = \"LVM configured to hide scratch dir via LVM_HIDE_MOUNT, which prohibits docker jobs\"\n", ATTR_DOCKER_OFFLINE_REASON);
+		}
 	}
 
 	// Singularity support
@@ -167,6 +193,7 @@ printClassAd( void )
 
 		bool can_run_sandbox = false;
 		bool can_use_pidnamespaces = true;
+
 		if (htcondor::Singularity::canRunSandbox(can_use_pidnamespaces))  {
 			can_run_sandbox = true;
 		}
@@ -175,6 +202,21 @@ printClassAd( void )
 			can_run_sif = true;
 		}
 
+		htcondor::Singularity::IsSetuid isSetuid =
+			htcondor::Singularity::usesUserNamespaces();
+
+		switch (isSetuid) {
+			case htcondor::Singularity::SingSetuid:
+				printf("SingularityUserNamespaces = False\n");
+				break;
+			case htcondor::Singularity::SingUserNamespaces:
+				printf("SingularityUserNamespaces = True\n");
+				break;
+			case htcondor::Singularity::SingSetuidUnknown:
+				// Whereof one cannot speak, thereof one must be silent
+				break;
+		};
+
 		// To consider Singularity operational, we needed it to pass
 		// running something... either sandbox or sif...
 		if (can_run_sandbox || can_run_sif) {
@@ -212,14 +254,13 @@ printClassAd( void )
 		printf("%s = \"%s\"\n", ATTR_SINGULARITY_VERSION, htcondor::Singularity::version());
 	}
 
-	// Detect ability to encrypt execute directory
-#ifdef LINUX
-	if ( FilesystemRemap::EncryptedMappingDetect() ) {
-		printf( "%s = True\n", ATTR_HAS_ENCRYPT_EXECUTE_DIRECTORY );
-	}
-#endif
 #ifdef WIN32
 	printf( "%s = True\n", ATTR_HAS_ENCRYPT_EXECUTE_DIRECTORY );
+#else
+	std::string cryptsetup = which("cryptsetup");
+	if (VolumeManager::DetectLVM() && !cryptsetup.empty()) {
+		printf( "%s = True\n", ATTR_HAS_ENCRYPT_EXECUTE_DIRECTORY );
+	}
 #endif
 
 	// Advertise which file transfer plugins are supported
@@ -232,7 +273,7 @@ printClassAd( void )
 
 	// even if we have no transfer plugin methods, we may want to
 	// advertise plugin extra attributes
-	if ( ! ft.getPluginQueryAds().empty()) {
+	if ( ! ft.getPlugins().empty()) {
 		// publish plugin extra attributes, GetSupportedMethods will collect plugin ads
 		// when it queries the plugins for their '-classad' 
 		// We will incorporate into the starter classad any attributes from the plugins
@@ -247,7 +288,8 @@ printClassAd( void )
 	   #endif
 		std::string listval;
 
-		for (auto & ad : ft.getPluginQueryAds()) {
+		for (auto & plugin : ft.getPlugins()) {
+			const ClassAd & ad = plugin.ad;
 			classad::References has_attrs;
 			if (ad.LookupString("FailedMethods", listval)) {
 				for (auto & meth : StringTokenIterator(listval)) { failed_methods.insert(meth); }
@@ -467,7 +509,7 @@ main_init(int argc, char *argv[])
 		usage();
 	}
 
-	if( !Starter->Init(jic, orig_cwd, is_gridshell, starter_stdin_fd,
+	if( !starter->Init(jic, orig_cwd, is_gridshell, starter_stdin_fd,
 					   starter_stdout_fd, starter_stderr_fd) ) {
 		dprintf(D_ALWAYS, "Unable to start job.\n");
 		DC_Exit(1);
@@ -926,17 +968,17 @@ parseArgs( int argc, char* argv [] )
 void
 main_config()
 {
-	Starter->Config();
+	starter->Config();
 }
 
 
 void
 main_shutdown_fast()
 {
-	if ( Starter->RemoteShutdownFast(0) ) {
+	if ( starter->RemoteShutdownFast(0) ) {
 		// ShutdownFast says it is already finished, because there are
 		// no jobs to shutdown.  No need to stick around.
-		Starter->StarterExit(Starter->GetShutdownExitCode());
+		starter->StarterExit(starter->GetShutdownExitCode());
 	}
 }
 
@@ -944,10 +986,10 @@ main_shutdown_fast()
 void
 main_shutdown_graceful()
 {
-	if ( Starter->RemoteShutdownGraceful(0) ) {
+	if ( starter->RemoteShutdownGraceful(0) ) {
 		// ShutdownGraceful says it is already finished, because
 		// there are no jobs to shutdown.  No need to stick around.
-		Starter->StarterExit(Starter->GetShutdownExitCode());
+		starter->StarterExit(starter->GetShutdownExitCode());
 	}
 }
 
@@ -955,9 +997,7 @@ extern "C"
 int exception_cleanup(int,int,const char*errmsg)
 {
 	_EXCEPT_Cleanup = NULL;
-	Starter->jic->notifyStarterError(errmsg,true,0,0);
-	Starter->RemoteShutdownFast(0);
-	Starter->FinalCleanup();
+	starter->ExceptHandler(errmsg);
 	return 0;
 }
 
