@@ -21,6 +21,7 @@
 #include "condor_common.h"
 #include "condor_classad.h"
 #include "condor_debug.h"
+#include "condor_daemon_client.h"
 #include "condor_daemon_core.h"
 #include "condor_attributes.h"
 #include "vanilla_proc.h"
@@ -49,7 +50,7 @@
 #include <sys/eventfd.h>
 #endif
 
-extern Starter *Starter;
+extern class Starter *starter;
 
 void StarterStatistics::Clear() {
    this->InitTime = time(NULL);
@@ -182,7 +183,7 @@ static bool cgroup_controller_is_writeable(const std::string &controller, std::s
 		TemporaryPrivSentry sentry(PRIV_ROOT); // Test with all our powers
 
 		if (access(test_path.c_str(), R_OK | W_OK) == 0) {
-			dprintf(D_ALWAYS, "    Cgroup %s/%s is useable\n", controller.c_str(), relative_cgroup.c_str());
+			dprintf(D_ALWAYS, "    Cgroup %s/%s is usable\n", controller.c_str(), relative_cgroup.c_str());
 			return true;
 		}
 	}
@@ -217,45 +218,6 @@ static bool cgroup_v2_is_writeable(const std::string &relative_cgroup) {
 		cgroup_controller_is_writeable("", relative_cgroup);
 }
 
-static std::string current_parent_cgroup() {
-	TemporaryPrivSentry sentry(PRIV_ROOT);
-	std::string cgroup;
-
-	int fd = open("/proc/self/cgroup", O_RDONLY);
-	if (fd < 0) {
-		dprintf(D_ALWAYS, "Cannot open /proc/self/cgroup: %s\n", strerror(errno));
-		return cgroup; // empty cgroup is invalid
-	}
-
-	char buf[1024];
-	int r = read(fd, buf, sizeof(buf)-1);
-	if (r < 0) {
-		dprintf(D_ALWAYS, "Cannot read /proc/self/cgroup: %s\n", strerror(errno));
-		close(fd);
-		return cgroup;
-	}
-	buf[r] = '\0';
-	cgroup = buf;
-	close(fd);
-
-	if (cgroup.starts_with("0::")) {
-		cgroup = cgroup.substr(3,cgroup.size() - 4); // 4 because of newline at end
-	} else {
-		dprintf(D_ALWAYS, "Unknown prefix for /proc/self/cgroup: %s\n", cgroup.c_str());
-		cgroup = "";
-	}
-
-	size_t lastSlash = cgroup.rfind('/');
-	if (lastSlash == std::string::npos) {
-		// This can happen if you are at the root of a cgroup namespace.  Not sure how to handle
-		dprintf(D_ALWAYS, "Cgroup %s has no internal directory to chdir .. to...\n", cgroup.c_str());
-		cgroup = "";
-	} else {
-		cgroup.erase(lastSlash); // Remove trailing slash
-	}
-	return cgroup;
-}
-
 static bool hasCgroupV2() {
 	struct stat statbuf{};
 	// Should be readable by everyone
@@ -293,7 +255,7 @@ VanillaProc::StartJob()
 	//
 	fi.max_snapshot_interval = param_integer("PID_SNAPSHOT_INTERVAL", 15);
 
-	m_dedicated_account = Starter->jic->getExecuteAccountIsDedicated();
+	m_dedicated_account = starter->jic->getExecuteAccountIsDedicated();
 	if( ThisProcRunsAlongsideMainProc() ) {
 			// If we track a secondary proc's family tree (such as
 			// sshd) using the same dedicated account as the job's
@@ -356,15 +318,9 @@ VanillaProc::StartJob()
 	// For v2, let's put the job into the current cgroup hierarchy
 	// Because of the "no process in interior cgroups" rule, this means
 	// we create a new child of our parent. (a sibling, if you will).
-	if (hasCgroupV2()) {
-		std::string current = current_parent_cgroup();
-		cgroup_base = current + '/' + cgroup_base;
 
-		// remove leading / from cgroup_name. cgroupv2 code hates that
-		if (cgroup_base.starts_with('/')) {
-			cgroup_base = cgroup_base.substr(1, cgroup_base.size() - 1);
-		}
-		replace_str(cgroup_base, "//", "/");
+	if (hasCgroupV2()) {
+		cgroup_base = ProcFamilyDirectCgroupV2::make_full_cgroup_name(cgroup_base);
 	}
 
 	if (create_cgroup && cgroup_is_writeable(cgroup_base)) {
@@ -373,14 +329,14 @@ VanillaProc::StartJob()
 		std::string execute_str;
 		param(execute_str, "EXECUTE", "EXECUTE_UNKNOWN");
 			// Note: Starter is a global variable from os_proc.cpp
-		Starter->jic->machClassAd()->LookupString(ATTR_NAME, starter_name);
+		starter->jic->machClassAd()->LookupString(ATTR_NAME, starter_name);
 		if (starter_name.size() == 0) {
 			starter_name = std::to_string(getpid());
 		}
 		formatstr(cgroup_uniq, "%s_%s", execute_str.c_str(), starter_name.c_str());
 		const char dir_delim[2] = {DIR_DELIM_CHAR, '\0'};
 		replace_str(cgroup_uniq, dir_delim, "_");
-		formatstr(cgroup_str, "%s%ccondor%s", cgroup_base.c_str(), DIR_DELIM_CHAR,
+		formatstr(cgroup_str, "%s%c%s", cgroup_base.c_str(), DIR_DELIM_CHAR,
 			cgroup_uniq.c_str());
 		cgroup_str += this->CgroupSuffix();
 		
@@ -389,9 +345,15 @@ VanillaProc::StartJob()
 		fi.cgroup = cgroup;
 
 		int numCores = 1;
-		if (!Starter->jic->machClassAd()->LookupInteger(ATTR_CPUS, numCores)) {
-			dprintf(D_ALWAYS, "Invalid value of Cpus in machine ClassAd; assuming 1 for cgroup limit purposes.\n");
+		if (!starter->jic->machClassAd()->LookupInteger(ATTR_CPUS, numCores)) {
+			dprintf(D_ALWAYS, "Invalid value of Cpus in machine ClassAd.\n");
+			if (param_boolean("LOCAL_UNIVERSE_CGROUP_ENFORCEMENT", false)) {
+				if (!starter->jic->jobClassAd()->LookupInteger(ATTR_REQUEST_CPUS, numCores)) {
+					dprintf(D_ALWAYS, "   Job does not have RequestCpus either, falling back to 1 cpu\n");
+				}
+			}
 		}
+
 		fi.cgroup_cpu_shares = 100 * numCores;
 
 		if (param_boolean("STARTER_HIDE_GPU_DEVICES", true)) {
@@ -399,16 +361,23 @@ VanillaProc::StartJob()
 			std::string available_gpus;
 			const char *gpu_expr = "join(\",\",evalInEachContext(strcat(\"GPU-\",DeviceUuid),AvailableGPUs))";
 			classad::Value v;
-			Starter->jic->machClassAd()->EvaluateExpr(gpu_expr, v);
+			starter->jic->machClassAd()->EvaluateExpr(gpu_expr, v);
 			v.IsStringValue(available_gpus);
 
 			// will remain empty if not set, meaning hide all
 			fi.cgroup_hide_devices = nvidia_env_var_to_exclude_list(available_gpus);
 		}
 		int64_t memory = 0;
-		if (!Starter->jic->machClassAd()->LookupInteger(ATTR_MEMORY, memory)) {
-			dprintf(D_ALWAYS, "Invalid value of memory in machine ClassAd; not setting memory limits\n");
-			memory = 0; // just to be sure
+		if (!starter->jic->machClassAd()->LookupInteger(ATTR_MEMORY, memory)) {
+			dprintf(D_ALWAYS, "Invalid value of memory in machine ClassAd.\n");
+			if (param_boolean("LOCAL_UNIVERSE_CGROUP_ENFORCEMENT", false)) {
+				if (!starter->jic->jobClassAd()->LookupInteger(ATTR_REQUEST_MEMORY, memory)) {
+					dprintf(D_ALWAYS, "   Job does not have RequestMemory either, falling back to no memory limit\n");
+					memory = 0; // just to be sure
+				} else {
+					dprintf(D_ALWAYS, "   Using RequestMemory from job at of %ld Mb\n", memory);
+				}
+			}
 		}
 		fi.cgroup_memory_limit = 0; // meaning no limit
 
@@ -429,7 +398,7 @@ VanillaProc::StartJob()
 					false, // check_ranges
 					0,     // min_value
 					std::numeric_limits<long long>::max(), // max_value
-					Starter->jic->machClassAd(), // my ad
+					starter->jic->machClassAd(), // my ad
 					JobAd)) {
 			fi.cgroup_memory_limit_low = (uint64_t) low_value * 1024 * 1024;
 		}
@@ -448,7 +417,7 @@ VanillaProc::StartJob()
 					check_ranges,
 					min_value,
 					max_value,
-					Starter->jic->machClassAd(),
+					starter->jic->machClassAd(),
 					JobAd)) {
 				fi.cgroup_memory_limit = (uint64_t) hard_value * 1024 * 1024;
 			} else {
@@ -457,7 +426,7 @@ VanillaProc::StartJob()
 		}
 
 		// if DISABLE_SWAP_FOR_JOB is true, set swap limit to memory (meaning no swap) 
-		bool disable_swap = param_boolean("DISABLE_SWAP_FOR_JOB", false);
+		bool disable_swap = param_boolean("DISABLE_SWAP_FOR_JOB", true);
 		if (disable_swap && fi.cgroup_memory_limit > 0) {
 			fi.cgroup_memory_and_swap_limit = fi.cgroup_memory_limit;
 		}
@@ -467,94 +436,7 @@ VanillaProc::StartJob()
 
 #endif
 
-// The chroot stuff really only works on linux
 #ifdef LINUX
-	{
-        // Have Condor manage a chroot
-       std::string requested_chroot_name;
-       JobAd->LookupString("RequestedChroot", requested_chroot_name);
-       const char * allowed_root_dirs = param("NAMED_CHROOT");
-       if (requested_chroot_name.size()) {
-               dprintf(D_FULLDEBUG, "Checking for chroot: %s\n", requested_chroot_name.c_str());
-               bool acceptable_chroot = false;
-               std::string requested_chroot;
-               for (const auto& next_chroot: StringTokenIterator(allowed_root_dirs)) {
-                       StringTokenIterator chroot_spec(next_chroot, "=");
-                       const char* tok;
-                       tok = chroot_spec.next();
-                       if (tok == NULL) {
-                               dprintf(D_ALWAYS, "Invalid named chroot: %s\n", next_chroot.c_str());
-                               continue;
-                       }
-                       std::string chroot_name = tok;
-                       tok = chroot_spec.next();
-                       if (tok == NULL) {
-                               dprintf(D_ALWAYS, "Invalid named chroot: %s\n", next_chroot.c_str());
-                               continue;
-                       }
-                       std::string next_dir = tok;
-                       dprintf(D_FULLDEBUG, "Considering directory %s for chroot %s.\n", next_dir.c_str(), next_chroot.c_str());
-                       if (IsDirectory(next_dir.c_str()) && requested_chroot_name == chroot_name) {
-                               acceptable_chroot = true;
-                               requested_chroot = next_dir;
-                       }
-               }
-               // TODO: path to chroot MUST be all root-owned, or we have a nice security exploit.
-               // Is this the responsibility of Condor to check, or the sysadmin who set it up?
-               if (!acceptable_chroot) {
-                       return FALSE;
-               }
-               dprintf(D_FULLDEBUG, "Will attempt to set the chroot to %s.\n", requested_chroot.c_str());
-
-               std::stringstream ss;
-               std::stringstream ss2;
-               ss2 << Starter->GetExecuteDir() << DIR_DELIM_CHAR << "dir_" << getpid();
-               std::string execute_dir = ss2.str();
-               ss << requested_chroot << DIR_DELIM_CHAR << ss2.str();
-               std::string full_dir_str = ss.str();
-               if (is_trivial_rootdir(requested_chroot)) {
-                   dprintf(D_FULLDEBUG, "Requested a trivial chroot %s; this is a no-op.\n", requested_chroot.c_str());
-               } else if (IsDirectory(execute_dir.c_str())) {
-                       {
-                           TemporaryPrivSentry sentry(PRIV_ROOT);
-                           if( mkdir(full_dir_str.c_str(), S_IRWXU) < 0 ) {
-                               dprintf( D_ERROR,
-                                   "Failed to create sandbox directory in chroot (%s): %s\n",
-                                   full_dir_str.c_str(),
-                                   strerror(errno) );
-                               return FALSE;
-                           }
-                           if (chown(full_dir_str.c_str(),
-                                     get_user_uid(),
-                                     get_user_gid()) == -1)
-                           {
-                               EXCEPT("chown error on %s: %s",
-                                      full_dir_str.c_str(),
-                                      strerror(errno));
-                           }
-                       }
-                       if (!fs_remap) {
-                           fs_remap.reset(new FilesystemRemap());
-                       }
-                       dprintf(D_FULLDEBUG, "Adding mapping: %s -> %s.\n", execute_dir.c_str(), full_dir_str.c_str());
-                       if (fs_remap->AddMapping(execute_dir, full_dir_str)) {
-                               // FilesystemRemap object prints out an error message for us.
-                               return FALSE;
-                       }
-                       dprintf(D_FULLDEBUG, "Adding mapping %s -> %s.\n", requested_chroot.c_str(), "/");
-                       std::string root_str("/");
-                       if (fs_remap->AddMapping(requested_chroot, root_str)) {
-                               return FALSE;
-                       }
-               } else {
-                       dprintf(D_ALWAYS, "Unable to do chroot because working dir %s does not exist.\n", execute_dir.c_str());
-               }
-       } else {
-               dprintf(D_FULLDEBUG, "Value of RequestedChroot is unset.\n");
-       }
-	}
-// End of chroot 
-
 	// On Linux kernel 2.4.19 and later, we can give each job its
 	// own FS mounts.
 	std::string mount_under_scratch;
@@ -592,7 +474,7 @@ VanillaProc::StartJob()
 
 	// mount_under_scratch only works with rootly powers
 	if (! mount_under_scratch.empty() && can_switch_ids() && has_sysadmin_cap() && (job_universe != CONDOR_UNIVERSE_LOCAL)) {
-		const char* working_dir = Starter->GetWorkingDir(0);
+		const char* working_dir = starter->GetWorkingDir(0);
 
 		if (IsDirectory(working_dir)) {
 			if (!fs_remap) {
@@ -629,7 +511,7 @@ VanillaProc::StartJob()
 					dprintf(D_ALWAYS, "Unable to add mapping %s -> %s because %s doesn't exist.\n", working_dir, next_dir.c_str(), next_dir.c_str());
 				}
 			}
-		Starter->setTmpDir("/tmp");
+		starter->setTmpDir("/tmp");
 		} else {
 			dprintf(D_ALWAYS, "Unable to perform mappings because %s doesn't exist.\n", working_dir);
 			return FALSE;
@@ -643,12 +525,9 @@ VanillaProc::StartJob()
 	static bool previously_setup_for_pid_namespace = false;
 
 	if ( (previously_setup_for_pid_namespace || param_boolean("USE_PID_NAMESPACES", false))
-			&& !htcondor::Singularity::job_enabled(*Starter->jic->machClassAd(), *JobAd) ) 
+			&& !htcondor::Singularity::job_enabled(*starter->jic->machClassAd(), *JobAd)
+			&& can_switch_ids() )
 	{
-		if (!can_switch_ids()) {
-			EXCEPT("USE_PID_NAMESPACES enabled, but can't perform this "
-				"call in Linux unless running as root.");
-		}
 		fi.want_pid_namespace = this->SupportsPIDNamespace();
 		if (fi.want_pid_namespace) {
 			if (!fs_remap) {
@@ -672,7 +551,7 @@ VanillaProc::StartJob()
 			std::string arg_errors;
 			std::string filename;
 
-			filename = Starter->GetWorkingDir(0);
+			filename = starter->GetWorkingDir(0);
 			filename += "/.condor_pid_ns_status";
 
 			if (!env.MergeFrom(JobAd,  env_errors)) {
@@ -686,7 +565,7 @@ VanillaProc::StartJob()
 				return 0;
 			}
 
-			Starter->jic->removeFromOutputFiles(condor_basename(filename.c_str()));
+			starter->jic->removeFromOutputFiles(condor_basename(filename.c_str()));
 			
 			// Now, set the job's CMD to the wrapper, and shift
 			// over the arguments by one
@@ -696,7 +575,7 @@ VanillaProc::StartJob()
 
 			JobAd->LookupString(ATTR_JOB_CMD, cmd);
 
-			this->canonicalizeJobPath(cmd, Starter->jic->jobRemoteIWD());
+			this->canonicalizeJobPath(cmd, starter->jic->jobRemoteIWD());
 
 			// Must set this *after* calling canonicalizeJobPath!
 			this->m_pid_ns_status_filename = filename;
@@ -723,6 +602,10 @@ VanillaProc::StartJob()
 		}
 	}
 	dprintf(D_FULLDEBUG, "PID namespace option: %s\n", fi.want_pid_namespace ? "true" : "false");
+
+	if (param_boolean("NO_JOB_NETWORKING", false)) {
+		fi.want_net_namespace = true;
+	}
 #endif
 
 
@@ -740,6 +623,11 @@ VanillaProc::StartJob()
 		setupOOMScore(0,0);
 	}
 
+	if (cgroup) {
+		int interval = param_integer("CGROUP_POLLING_INTERVAL", 5);
+		procFamilyTimerId = daemonCore->Register_Timer( 0, interval,
+				(TimerHandlercpp)&VanillaProc::pollFamilyUsage, "cgroup usage poller", this );
+	}
 #endif
 
 	return retval;
@@ -754,11 +642,8 @@ VanillaProc::PublishUpdateAd( ClassAd* ad )
 	static unsigned int max_pss = 0;
 #endif
 
-	ProcFamilyUsage current_usage;
-	if( m_proc_exited ) {
-		current_usage = m_final_usage;
-	} else {
-		if (daemonCore->Get_Family_Usage(JobPid, current_usage) == FALSE) {
+	if (!m_proc_exited) {
+		if (daemonCore->Get_Family_Usage(JobPid, m_current_usage) == FALSE) {
 			dprintf(D_ALWAYS, "error getting family usage in "
 					"VanillaProc::PublishUpdateAd() for pid %d\n", JobPid);
 			return false;
@@ -766,7 +651,7 @@ VanillaProc::PublishUpdateAd( ClassAd* ad )
 	}
 
 	ProcFamilyUsage reported_usage = m_checkpoint_usage;
-	reported_usage += current_usage;
+	reported_usage += m_current_usage;
 	ProcFamilyUsage * usage = & reported_usage;
 
         // prepare for updating "generic_stats" stats, call Tick() to update current time
@@ -880,47 +765,57 @@ VanillaProc::notifySuccessfulPeriodicCheckpoint( int checkpointNumber ) {
 	//
 	ClassAd updateAd;
 
-	Starter->publishUpdateAd( & updateAd );
+	starter->publishUpdateAd( & updateAd );
 	updateAd.Assign( ATTR_JOB_CHECKPOINT_NUMBER, checkpointNumber );
 
 	// UserProc::PublishUpdateAd() truncates, so we will too.
-	int lastCheckpointTime = job_exit_time.tv_sec;
+	time_t lastCheckpointTime = job_exit_time.tv_sec;
 	updateAd.Assign( ATTR_JOB_LAST_CHECKPOINT_TIME, lastCheckpointTime );
 
 	// UserProc::PublishUpdateAd() truncates, so we will too.
 	int newlyCommittedTime = (int)timersub_double(job_exit_time, job_start_time);
 	updateAd.Assign( ATTR_JOB_NEWLY_COMMITTED_TIME, newlyCommittedTime );
 
-	Starter->jic->periodicJobUpdate( & updateAd );
+	starter->jic->periodicJobUpdate( & updateAd );
 
 	// Let's not try to be subtle and confusing (by reacting to the above
 	// update in the shadow instead of to a specific event).
 	ClassAd eventAd;
+	int ignored = -1;
 	eventAd.InsertAttr( "EventType", "SuccessfulCheckpoint" );
 	eventAd.InsertAttr( ATTR_JOB_CHECKPOINT_NUMBER, checkpointNumber );
-	Starter->jic->notifyGenericEvent( eventAd );
+	starter->jic->notifyGenericEvent( eventAd, ignored );
 }
 
 void
 VanillaProc::notifyFailedPeriodicCheckpoint( int checkpointNumber ) {
     ClassAd ad;
+    int ignored = -1;
     ad.InsertAttr( "EventType", "FailedCheckpoint" );
     ad.InsertAttr( ATTR_JOB_CHECKPOINT_NUMBER, checkpointNumber );
-    Starter->jic->notifyGenericEvent( ad );
+    starter->jic->notifyGenericEvent( ad, ignored );
 }
 
 void VanillaProc::recordFinalUsage() {
-	if( daemonCore->Get_Family_Usage(JobPid, m_final_usage) == FALSE ) {
+	if( daemonCore->Get_Family_Usage(JobPid, m_current_usage) == FALSE ) {
 		dprintf( D_ALWAYS, "error getting family usage for pid %d in "
 			"VanillaProc::JobReaper()\n", JobPid );
 	}
 }
+ 
+void VanillaProc::pollFamilyUsage(int /*timerid*/) {
+	if (JobPid > 0) {
+		if( daemonCore && daemonCore->Get_Family_Usage(JobPid, m_current_usage) == FALSE ) {
+			dprintf( D_ALWAYS, "error polling family usage\n");
+		}
+	}
+}
 
 void VanillaProc::killFamilyIfWarranted() {
 	// Kill_Family() will (incorrectly?) kill the SSH-to-job daemon
 	// if we're using dedicated accounts or cgroups, so don't unless we know
 	// we're the only job.
-	if (Starter->numberOfJobs() == 1 ) {
+	if (starter->numberOfJobs() == 1 ) {
 		dprintf( D_PROCFAMILY, "About to call Kill_Family()\n" );
 		daemonCore->Kill_Family(JobPid);
 	} else {
@@ -933,7 +828,7 @@ void VanillaProc::killFamilyIfWarranted() {
 	}
 }
 
-void VanillaProc::restartCheckpointedJob() {
+bool VanillaProc::restartCheckpointedJob() {
 
 	// daemoncore unregisters the family
 	// after it calls the reaper, if it was registered.
@@ -962,7 +857,7 @@ void VanillaProc::restartCheckpointedJob() {
 	// Because not all upload attempts fail without writing any data, we
 	// need to clean up after failed attempts, which implies numbering them.
 	++checkpointNumber;
-	if( Starter->jic->uploadCheckpointFiles(checkpointNumber) ) {
+	if( starter->jic->uploadCheckpointFiles(checkpointNumber) ) {
 			notifySuccessfulPeriodicCheckpoint(checkpointNumber);
 	} else {
 			// We assume this is a transient failure and will try
@@ -971,6 +866,47 @@ void VanillaProc::restartCheckpointedJob() {
 			notifyFailedPeriodicCheckpoint(checkpointNumber);
 	}
 
+	//
+	// New semantic: restarting a checkpointed job is now considered
+	// equivalent to reactivating a claim, so ask the startd if a claim
+	// reactivation would succeed.
+	//
+	if( param_boolean( "CHECK_REACTIVATE_AFTER_CHECKPOINT", false ) )
+	{
+		DCStartd startd((const char *)nullptr);
+		if(! startd.locate()) {
+			dprintf( D_ERROR, "Unable to locate startd while attempting to determine if the checkpointed job should restart: %s\n", startd.error() );
+
+	        // Fall back to the semantics from before this check was added.
+	        goto restart_job;
+		}
+		std::string claimID;
+		if(! starter->getJobClaimId(claimID)) {
+			dprintf( D_ERROR, "Unable to get my job's claim ID.\n" );
+
+	        // Fall back to the semantics from before this check was added.
+	        goto restart_job;
+		}
+		startd.setClaimId(claimID);
+
+		bool claim_is_closing = false;
+		bool OK = startd.reactivateClaimCheck(claim_is_closing);
+		if(! OK) {
+			dprintf( D_ERROR, "Attempt to check if this checkpointed job should restart failed: %s\n", startd.error() );
+
+	        // Fall back to the semantics from before this check was added.
+	        goto restart_job;
+		}
+		if( claim_is_closing ) {
+			dprintf( D_ALWAYS, "This checkpointed job should NOT restart.\n" );
+
+			// We didn't restart the job (and didn't want to).
+			return false;
+		}
+    }
+
+    restart_job:;
+
 	// While it's arguably sensible to kill the process family
 	// before we restart the job, that would mean that checkpointing
 	// would behave differently during ssh-to-job, which seems bad.
@@ -978,6 +914,9 @@ void VanillaProc::restartCheckpointedJob() {
 
 	m_proc_exited = false;
 	StartJob();
+
+	// We started the job.
+	return true;
 }
 
 
@@ -1001,12 +940,18 @@ VanillaProc::outOfMemoryEvent() {
 	// memory they used.
 	ClassAd updateAd;
 	PublishUpdateAd( &updateAd );
-	Starter->jic->periodicJobUpdate( &updateAd );
+	starter->jic->periodicJobUpdate( &updateAd );
 	int64_t usageKB = 0;
 	// This is the peak
 	updateAd.LookupInteger(ATTR_IMAGE_SIZE, usageKB);
 	int64_t usageMB = usageKB / 1024;
 
+	// But sometimes the job dies before we can poll for memory on systems 
+	// that don't have a memory.peak, and we get 0 MB. Assume job hit
+	// memory limit, and report that number, not the confusing 0 bytes used.
+	if (usageMB == 0) {
+		usageMB = m_memory_limit / (1024 * 1024);
+	}
 	//
 	//  Cgroup memory limits are limits, not reservations.
 	//  For many reasons, a job could be below the memory limit,
@@ -1024,21 +969,25 @@ VanillaProc::outOfMemoryEvent() {
 	// Why not 100%?  We have seen cases where our last cgroup poll was a bit 
 	// lower than the limit when the OOM killer fired.
 	// So have some slop, just in case.
-	if (usageMB < (0.9 * (m_memory_limit / (1024 * 1024)))) {
-		// But sometimes the job dies before we can poll for memory on systems 
-		// that don't have a memory.peak, and we get 0 MB. Assume job should go
-		// on hold in that case 
-		if (usageMB > 0) {
+
+	// Now that we are polling cgroup, and not getting peaks from cgroup
+	// a quickly-growing job can have a last-reported memory significantly
+	// lower than the limit.  In this case we want to always hold the job
+	// and report and out-of-memory condition
+	bool should_hold = param_boolean("STARTER_ALWAYS_HOLD_ON_OOM", true);
+
+	if (!should_hold) {
+		if (usageMB < (0.9 * (m_memory_limit / (1024 * 1024)))) {
 			dprintf(D_ALWAYS, "Evicting job because system is out of memory, even though the job is below requested memory: Usage is %lld Mb limit is %lld\n", (long long)usageMB, (long long)m_memory_limit);
-			Starter->jic->notifyStarterError("Worker node is out of memory", true, 0, 0);
-			Starter->jic->allJobsGone(); // and exit to clean up more memory
+			starter->jic->notifyStarterError("Worker node is out of memory", true, 0, 0);
+			starter->jic->allJobsGone(); // and exit to clean up more memory
 			return 0;
 		}
 	}
 
 	std::string ss;
 	if (m_memory_limit >= 0) {
-		formatstr(ss, "Job has gone over cgroup memory limit of %lld megabytes. Peak usage: %lld megabytes.  Consider resubmitting with a higher request_memory.", 
+		formatstr(ss, "Job has gone over cgroup memory limit of %lld megabytes. Last measured usage: %lld megabytes.  Consider resubmitting with a higher request_memory.", 
 				(long long)(m_memory_limit / (1024 * 1024)), (long long)usageMB);
 	} else {
 		ss = "Job has encountered an out-of-memory event.";
@@ -1050,7 +999,7 @@ VanillaProc::outOfMemoryEvent() {
 	dprintf( D_ALWAYS, "Job was held due to OOM event: %s\n", ss.c_str());
 
 	// This ulogs the hold event and KILLS the shadow
-	Starter->jic->holdJob(ss.c_str(), CONDOR_HOLD_CODE::JobOutOfResources, 0);
+	starter->jic->holdJob(ss.c_str(), CONDOR_HOLD_CODE::JobOutOfResources, OUT_OF_RESOURCES_SUB_CODE::Memory);
 
 	return 0;
 }
@@ -1060,9 +1009,13 @@ VanillaProc::JobReaper(int pid, int status)
 {
 	dprintf(D_FULLDEBUG,"Inside VanillaProc::JobReaper()\n");
 
+	if (procFamilyTimerId > 0) {
+		daemonCore->Cancel_Timer(procFamilyTimerId);
+		procFamilyTimerId = -1;
+	}
 	// If cgroup v2 is enabled, we'll get this high bit set in exit_status
 #ifdef LINUX
-	if (status & DC_STATUS_OOM_KILLED) {
+	if (!isSoftKilling && (status & DC_STATUS_OOM_KILLED)) {
 		// Will put the job on hold
 		this->outOfMemoryEvent();
 		status &= ~DC_STATUS_OOM_KILLED;
@@ -1100,9 +1053,45 @@ VanillaProc::JobReaper(int pid, int status)
 				return true;
 			}
 
-			restartCheckpointedJob();
-			isCheckpointing = false;
-			return false;
+			if( restartCheckpointedJob() ) {
+				isCheckpointing = false;
+				return false;
+			} else {
+				// We need to prevent (final) output transfer from happening
+				// as well as ensure that the job is requeued.  The latter
+				// should happen automatically as a result of the former,
+				// but doesn't.
+				starter->jic->setOutputTransfer(true);
+				starter->SetVacateReason(
+					"Rescheduling self-checkpoint job after checkpoint upload because reactivating the claim would have failed.",
+					CONDOR_HOLD_CODE::SuccessfulCheckpoint, 0
+				);
+				starter->jicNotifyStarterError( true );
+				requested_exit = true;
+
+				// At this point, the shadow will ask the startd to deactivate
+				// the claim, but the startd will block waiting for the starter
+				// to exit, so we'll just exit first.  (Arguably, we should
+				// schedule a zero-second timer here to exit so that we can
+				// exit through the event loop.)
+				starter->StarterExit( JOB_SHOULD_REQUEUE );
+
+				// This job is done, but we want to avoid reporting a job
+				// exit here, because it will be misinterpreted as a job
+				// termination, and we need the job rescheduled.
+				return false;
+
+
+
+				// FIXME: but doesn't.
+
+
+
+				starter->jic->setOutputTransfer(true);
+
+				// This job is done.
+				return true;
+			}
 		} else {
 			// The job exited without taking a checkpoint.  If we don't do
 			// anything, it will be reported as if the error code or signal
@@ -1132,8 +1121,8 @@ VanillaProc::JobReaper(int pid, int status)
 				checkpointExitBySignal ? checkpointExitSignal : checkpointExitCode,
 				WIFSIGNALED( exit_status ) ? "on signal" : "with exit code",
 				WIFSIGNALED( exit_status ) ? WTERMSIG( exit_status ) : WEXITSTATUS( exit_status ) );
-			Starter->jic->holdJob( holdMessage.c_str(), CONDOR_HOLD_CODE::FailedToCheckpoint, exit_status );
-			Starter->Hold();
+			starter->jic->holdJob( holdMessage.c_str(), CONDOR_HOLD_CODE::FailedToCheckpoint, exit_status );
+			starter->Hold();
 			return true;
 		}
 	} else if( wantsFileTransferOnCheckpointExit && exit_status == successfulCheckpointStatus ) {
@@ -1143,8 +1132,33 @@ VanillaProc::JobReaper(int pid, int status)
 			notifySuccessfulEvictionCheckpoint();
 			return true;
 		} else {
-			restartCheckpointedJob();
-			return false;
+			if( restartCheckpointedJob() ) {
+				return false;
+			} else {
+				// We need to prevent (final) output transfer from happening
+				// as well as ensure that the job is requeued.  The latter
+				// should happen automatically as a result of the former,
+				// but doesn't.
+				starter->jic->setOutputTransfer(true);
+				starter->SetVacateReason(
+					"Rescheduling self-checkpoint job after checkpoint upload because reactivating the claim would have failed.",
+					CONDOR_HOLD_CODE::SuccessfulCheckpoint, 0
+				);
+				starter->jicNotifyStarterError( true );
+				requested_exit = true;
+
+				// At this point, the shadow will ask the startd to deactivate
+				// the claim, but the startd will block waiting for the starter
+				// to exit, so we'll just exit first.  (Arguably, we should
+				// schedule a zero-second timer here to exit so that we can
+				// exit through the event loop.)
+				starter->StarterExit( JOB_SHOULD_REQUEUE );
+
+				// This job is done, but we want to avoid reporting a job
+				// exit here, because it will be misinterpreted as a job
+				// termination, and we need the job rescheduled.
+				return false;
+			}
 		}
 	} else {
 		// If the parent job process died, clean up all of the job's processes.
