@@ -23,6 +23,8 @@
 
 #include <string>
 
+#include "condor_sockaddr.h"
+
 /* For debugging */
 #if defined(LINUX)
 typedef struct {
@@ -46,6 +48,10 @@ int sysapi_phys_memory_raw_no_param(void);
 long long sysapi_reserve_for_fs();
 long long sysapi_disk_space_raw(const char *filename);
 long long sysapi_disk_space(const char *filename);
+/* how to get total and free disk blocks from a full pathname, answers in KB */
+long long sysapi_total_disk_space_raw(const char *filename, long long * total);
+long long sysapi_total_disk_space(const char *filename, long long * total);
+
 
 /* get ncpus without making any calls to config system, this uses cached values */
 void sysapi_ncpus_raw(int * num_cpus, int * num_hyperthreads);
@@ -79,7 +85,7 @@ void sysapi_internal_reconfig(void);
 
 /* if this is called, then the sysapi knows that a last_x_event has happend
 	and records the time it happened, this is quite useful in idle_time.C */
-void sysapi_last_xevent(void);
+void sysapi_last_xevent(int delta=0);
 
 /* return the one minute load average on a machine */
 float sysapi_load_avg_raw(void);
@@ -138,10 +144,6 @@ void sysapi_get_windows_info( void );
 /* set appropriate resource limits on each platform */
 void sysapi_set_resource_limits( int stack_size );
 
-/* determine a canonical kernel version */
-const char* sysapi_kernel_version_raw( void );
-const char* sysapi_kernel_version( void );
-
 /* determine the instruction set extensions on x86 machines */
 /* Would like to just use a classad here, but were in a 
  * classad-free layer */
@@ -172,32 +174,17 @@ int sysapi_partition_id(char const *path,char **result);
 #include <string>
 #include <vector>
 
-class NetworkDeviceInfo {
-public:
-	NetworkDeviceInfo(char const *the_name,char const *the_ip, bool the_up):
-		m_name(the_name),
-		m_ip(the_ip),
-		m_up(the_up)
-	{
-	}
-
-	char const *name() const { return m_name.c_str(); }
-	char const *IP() const { return m_ip.c_str(); }
-	bool is_up() const { return m_up; }
-
-private:
-	std::string m_name;
-	std::string m_ip;
-	bool m_up;
+struct NetworkDeviceInfo {
+	std::string name;
+	std::string name2;
+	condor_sockaddr addr;
+	bool is_up;
 };
 
 bool sysapi_get_network_device_info(std::vector<NetworkDeviceInfo> &devices, bool want_ipv4, bool want_ipv6);
 
 void sysapi_clear_network_device_info_cache();
 
-/* determine if a linux version is version X or newer */
-bool sysapi_is_linux_version_atleast(const char *version_to_check);
-
 #ifdef LINUX
 /* enum to represent the type of capability set mask we want to return*/
 enum LinuxCapsMaskType {
