@@ -31,7 +31,7 @@ import pytest
 from _pytest.nodes import Item
 from _pytest.runner import CallInfo
 
-import htcondor
+import htcondor2 as htcondor
 
 from ornithology import Condor, CONFIG_IDS
 
@@ -74,8 +74,9 @@ def get_base_test_dir(config):
     return base_dir
 
 
+# I need this, don't remove it.
 @pytest.hookimpl(tryfirst=True)
-def pytest_report_header(config, startdir):
+def pytest_report_header(config):
     return [
         "",
         "Base per-test directory: {}".format(get_base_test_dir(config)),
