@@ -3,7 +3,8 @@
 # Custom Ornithology transfer plugin that transfers files to /dev/null
 # This allows the mimicry of transfer without actually transferring
 
-import classad
+import classad2 as classad
+
 import json
 import os
 import posixpath
@@ -126,7 +127,7 @@ class DevNullPlugin:
         # Get transfer statistics
         transfer_stats = {
             'TransferSuccess': True,
-            'TransferProtocol': 'xferstats',
+            'TransferProtocol': 'null',
             'TransferType': 'upload',
             'TransferFileName': local_file_path,
             'TransferFileBytes': stats[1],
@@ -158,7 +159,7 @@ class DevNullPlugin:
         # Get transfer statistics
         transfer_stats = {
             'TransferSuccess': True,
-            'TransferProtocol': 'xferstats',
+            'TransferProtocol': 'null',
             'TransferType': 'upload',
             'TransferFileName': local_file_path,
             'TransferFileBytes': stats[1],
