@@ -2,6 +2,7 @@
 
 #testreq: personal
 """<<CONDOR_TESTREQ_CONFIG
+	NUM_CPUS = 8
 	# make sure that file transfer plugins are enabled (might be disabled by default)
 	ENABLE_URL_TRANSFERS = true
 """
@@ -10,7 +11,7 @@
 import time
 import logging
 
-from htcondor import (
+from htcondor2 import (
     JobEventType,
 )
 
@@ -491,9 +492,9 @@ def sleep_job_description(path_to_sleep):
 @action
 def path_to_sleep_transfer_plugin(test_dir):
     script="""
-    #!/usr/bin/env python3
+    #!/usr/bin/python3
 
-    import classad
+    import classad2 as classad
     import sys
     import time
 
