@@ -14,8 +14,8 @@ from ornithology import (
     Condor,
 )
 
-import htcondor
-from htcondor import (
+import htcondor2 as htcondor
+from htcondor2 import (
     JobEventType,
     FileTransferEventType,
 )
@@ -75,7 +75,7 @@ def plugin_python_file(test_dir):
         from urllib.parse import urlparse
         from pathlib import Path
 
-        import classad
+        import classad2 as classad
 
         DEFAULT_TIMEOUT = 30
         PLUGIN_VERSION = '1.0.0'
@@ -952,6 +952,19 @@ def fail_job(the_condor, fail_job_handle):
                 break
             time.sleep(1)
 
+    #
+    # At this point, we would like to see the job get rescheduled and
+    # (successfully) run again.  Sadly, this (still) means calling
+    # condor_reschedule.
+    #
+    the_condor.run_command(['condor_reschedule'])
+
+    assert fail_job_handle.wait(
+        timeout=300,
+        condition=ClusterState.all_complete,
+        fail_condition=ClusterState.any_held,
+    )
+
     return fail_job_handle
 
 
@@ -1169,11 +1182,18 @@ class TestCheckpointDestination:
         constraint = f'ClusterID == {the_removed_job.clusterid} && ProcID == 0'
 
         # Make sure the job has left the queue.
-        result = schedd.query(
-            constraint=constraint,
-            projection=["CheckpointDestination", "GlobalJobID"],
-        )
-        assert(len(result) == 0)
+        # Note that we've called condor_rm above, so it may still be in the 'X'
+        # state for a bit, so poll a few times to make sure it has left the queue.
+        for i in range(10):
+            result = schedd.query(
+                constraint=constraint,
+                projection=["CheckpointDestination", "GlobalJobID"],
+            )
+            if (len(result) == 0):
+                # Success!
+                break
+            time.sleep(5)
+        assert(i < 9)
 
         # Get the CheckpointDestination and GlobalJobID from the history file.
         result = schedd.history(
@@ -1267,3 +1287,5 @@ class TestCheckpointDestination:
             assert "Invalid MANIFEST file, aborting" in remote_error["ErrorMsg"]
         elif "corrupt_file" in fail_job_name:
             assert "did not have expected checksum" in remote_error["ErrorMsg"]
+
+        # We asserted in the fixture that the job eventually succeeded.
