@@ -14,8 +14,8 @@ from ornithology import (
     ClusterState,
 )
 
-import htcondor
-import classad
+import htcondor2 as htcondor
+import classad2 as classad
 
 import logging
 logger = logging.getLogger(__name__)
@@ -203,7 +203,7 @@ def path_to_the_job_script(test_dir, the_condor):
 
     # Evict ourselves once just before the end.
     rv = subprocess.run(
-        ["condor_q", "-jobads", ".job.ad", "-af", "NumJobStarts"],
+        ["condor_q", "-jobads", os.environ['_CONDOR_JOB_AD'], "-af", "NumJobStarts"],
         stdout=subprocess.PIPE,
         stderr=subprocess.PIPE,
         timeout=20
@@ -280,7 +280,7 @@ def the_condor(test_dir, the_config, the_config_name):
             'JOB_EPOCH_HISTORY':                '$(LOG)/EpochLog',
             'SCHEDD_INTERVAL':                  '2',
             'SHADOW.FILE_TRANSFER_STATS_LOG':   '$(LOG)/shadow_transfer_history',
-            'SHADOW_DEBUG':                     'D_CATEGORY D_SUBSECOND D_STATUS D_TEST',
+            'SHADOW_DEBUG':                     'D_CATEGORY D_SUB_SECOND D_STATUS D_TEST',
             ** the_config['config'],
         }
     ) as the_condor:
@@ -653,7 +653,7 @@ def path_to_the_plugin(test_dir):
         from urllib.parse import urlparse
         from pathlib import Path
 
-        import classad
+        import classad2 as classad
 
         DEFAULT_TIMEOUT = 30
         PLUGIN_VERSION = '1.0.0'
