@@ -32,8 +32,9 @@ from ornithology import *
 from time import time
 import os
 import sys
-import htcondor
-import htcondor2
+import htcondor2 as htcondor
+import re
+
 
 #Custom class to help build job ads for history files
 class HistAdsViaCluster():
@@ -41,7 +42,7 @@ class HistAdsViaCluster():
         self.owner = ownr
         self.clusterId = clust
         #Procs = number of job procs for this cluster
-        self.numProcs = procs
+        self.procs = procs
         self.QDate = qdate
         #cust_attrs = dictionary of key value pairs to add/change attrs for this cluster
         self.attrs = cust_attrs
@@ -71,8 +72,8 @@ STD_HEADER = ["ID","OWNER","SUBMITTED","RUN_TIME","ST","COMPLETED","CMD"];
 #Any test command that uses the flags -file, -userlog, or -search should put those last
 #because the test structure checks for those flags and adds the appropriate file to the end of the command
 TEST_CASES = {
-    "base_history"      : TestReqs("condor_history", ["1.0","2.0","2.1","2.2","3.0","4.0","4.1","5.0","5.1","5.2","5.3","5.4"], STD_HEADER),
-    "startd_history"    : TestReqs("condor_history -startd", ["1.0","2.0","2.1","2.2","3.0","4.0","4.1","5.0","5.1","5.2","5.3","5.4"], STD_HEADER),
+    "base_history"      : TestReqs("condor_history", ["1.0","2.0","2.1","2.2","3.0","4.0","4.1","5.0","5.1","5.2","5.3","5.4","123456789.1","123456789.55555"], STD_HEADER),
+    "startd_history"    : TestReqs("condor_history -startd", ["1.0","2.0","2.1","2.2","3.0","4.0","4.1","5.0","5.1","5.2","5.3","5.4","123456789.1","123456789.55555"], STD_HEADER),
     "search_cluster"    : TestReqs("condor_history 2", ["2.0","2.1","2.2"], STD_HEADER),
     "search_clust.proc" : TestReqs("condor_history 5.1", ["5.1"], STD_HEADER),
     "search_owner"      : TestReqs("condor_history unclegreg",["4.0","4.1"], STD_HEADER),
@@ -90,8 +91,8 @@ TEST_CASES = {
     "af_headers"        : TestReqs("condor_history -af:jh Owner QDate DAGManJobId CMD -limit 1", ["5.4"], ["ID","Owner","QDate","DAGManJobId","CMD"]),
     "no_match_af_const" : TestReqs("condor_history -af:j Beers -const Beers>10 -file"),
     # Remote history checks
-    "remote_schedd"     : TestReqs("condor_history -name TEST_SCHEDD@", ["1.0","2.0","2.1","2.2","3.0","4.0","4.1","5.0","5.1","5.2","5.3","5.4"], STD_HEADER),
-    "remote_startd"     : TestReqs("condor_history -name TEST_STARTD@ -startd", ["1.0","2.0","2.1","2.2","3.0","4.0","4.1","5.0","5.1","5.2","5.3","5.4"], STD_HEADER),
+    "remote_schedd"     : TestReqs("condor_history -name TEST_SCHEDD@", ["1.0","2.0","2.1","2.2","3.0","4.0","4.1","5.0","5.1","5.2","5.3","5.4","123456789.1","123456789.55555"], STD_HEADER),
+    "remote_startd"     : TestReqs("condor_history -name TEST_STARTD@ -startd", ["1.0","2.0","2.1","2.2","3.0","4.0","4.1","5.0","5.1","5.2","5.3","5.4","123456789.1","123456789.55555"], STD_HEADER),
     "remote_cluster"    : TestReqs("condor_history 5 -name TEST_SCHEDD@", ["5.0","5.1","5.2","5.3","5.4"], STD_HEADER),
     "remote_clust.proc" : TestReqs("condor_history 5.3 -name TEST_SCHEDD@", ["5.3"], STD_HEADER),
     "remote_match"      : TestReqs("condor_history 5 -name TEST_SCHEDD@ -limit 2", ["5.4","5.3"], STD_HEADER),
@@ -112,6 +113,16 @@ TEST_CASES = {
     "epoch_transfer_type":TestReqs("condor_history -epochs -type transfer", ["2.0","4.0","4.1","6.0","6.1","10.0"], STD_HEADER),
     "epoch_all_type"    : TestReqs("condor_history -epochs -type ALL", ["1.0","2.0","3.0","4.0","4.1","5.0","6.0","6.1","7.0","8.0","9.0","10.0"], STD_HEADER),
     "epoch_unknown_type": TestReqs("condor_history -epochs -type UNKNOWN", header=STD_HEADER),
+    # Test -transfer[-history] flag > Note: This tests the flags autosetup not the actual contents of the Ads since the
+    #                                       tests 'transfer' ads are synthesized from a job ad not actual transfer ads
+    "transfer_flag_all"     : TestReqs("condor_history -transfer -af:jh Owner", ["2.0","4.0","4.1","6.0","6.1","10.0"], ["ID", "Owner"]),
+    "transfer_flag_in"      : TestReqs("condor_history -transfer:i -af:jh Owner", ["2.0","6.0","6.1"], ["ID", "Owner"]),
+    "transfer_flag_out"     : TestReqs("condor_history -transfer-history:o -af:jh Owner", ["4.0","4.1"], ["ID", "Owner"]),
+    "transfer_flag_ckpt"    : TestReqs("condor_history -transfer:c -af:jh Owner", ["10.0"], ["ID", "Owner"]),
+    "transfer_flag_multi_1" : TestReqs("condor_history -transfer-history:IOC -af:jh Owner", ["2.0","4.0","4.1","6.0","6.1","10.0"], ["ID", "Owner"]),
+    "transfer_flag_multi_2" : TestReqs("condor_history -transfer:io -af:jh Owner", ["2.0","4.0","4.1","6.0","6.1"], ["ID", "Owner"]),
+    "transfer_flag_multi_3" : TestReqs("condor_history -transfer:oC -af:jh Owner", ["4.0","4.1","10.0"], ["ID", "Owner"]),
+    "transfer_flag_multi_4" : TestReqs("condor_history -transfer:Ic -af:jh Owner", ["2.0","6.0","6.1","10.0"], ["ID", "Owner"]),
     # Special case requires reconfig
     # Test remote query works when history=  and schedd.history=file
     "remote_schedd.hist": TestReqs("condor_history 4 -name TEST_SCHEDD@", ["4.0","4.1"], STD_HEADER),
@@ -126,6 +137,24 @@ TEST_ERROR_CASES = {
     "fail_multi_source"      : { "cmd":"condor_history -startd -epoch",
                                  "err":"can not be used in association with",
                                  "config":{}},
+    "fail_extract_no_const"  : { "cmd":"condor_history -extract foo",
+                                 "err":"-extract requires a constraint for which ClassAds to copy",
+                                 "config":{}},
+    "fail_xfer_no_fmt"       : {
+        "cmd": "condor_history -transfer-history",
+        "err": "-transfer-history does not have a default print table. Please use -long, -json, -xml, or a custom print format.",
+        "config": {},
+    },
+    "fail_xfer_unknown_colon_opt" : {
+        "cmd": "condor_history -transfer:q",
+        "err": "Unknown -transfer-history extra attribute 'q'",
+        "config": {},
+    },
+    "fail_unknown_daemon_history" : {
+        "cmd": "condor_history -daemon:foo",
+        "err": "No daemon history to read: FOO_DAEMON_HISTORY undefined",
+        "config": {},
+    }
 }
 #===============================================================================================
 # Reconfig condor to how we want
@@ -148,6 +177,20 @@ def reconfig(condor, test_name, test_dir, config={}):
             f.write(f"{knob}={value}\n")
     #Reconfig
     p = condor.run_command(reconfig_cmd)
+
+
+#-----------------------------------------------------------------------------------------------
+def iter_procs(procs):
+    if isinstance(procs, list):
+        for p in procs:
+            yield p
+    elif isinstance(procs, int):
+        for p in range(procs):
+            yield p
+    else:
+        raise TypeError(f"History Creation Expects Procs to be a count (int) or explicit list (list)")
+
+
 #-----------------------------------------------------------------------------------------------
 #Write cluster(s) job history to specified file
 def writeAdToHistory(filename, clusters, ad, is_schedd=True):
@@ -178,7 +221,7 @@ def writeAdToHistory(filename, clusters, ad, is_schedd=True):
                 ad[key] = value
                 changed_attrs.append(key)
         #For each desired proc in cluster
-        for i in range(clust.numProcs):
+        for i in iter_procs(clust.procs):
             #Update clusterid, owner, and proc id
             ad["ClusterId"] = clust.clusterId
             ad["Owner"] = f"\"{clust.owner}\""
@@ -264,6 +307,7 @@ def writeHistoryFile(default_condor, test_dir, jobAd):
         HistAdsViaCluster("cole",1,1,t),
         HistAdsViaCluster("tj",2,3,t+10),
         HistAdsViaCluster("tj",3,1,t+15,{"CustAttr":True}),
+        HistAdsViaCluster("andrew", 123456789, [55555, 1], t),
         HistAdsViaCluster("unclegreg",4,2,t+20),
         HistAdsViaCluster("cole",5,5,t+25),
     ]
@@ -493,8 +537,9 @@ def runTypeFilterPyBindings(default_condor):
         v1_schedd = htcondor.Schedd()
         test_info["V1-Output/INPUT"]["ads"] = v1_schedd.jobEpochHistory(None, proj, ad_type="output,input")
         test_info["V1-Normal"]["ads"] = v1_schedd.jobEpochHistory(None, proj)
+
         # Test V2 py bindings
-        v2_schedd = htcondor2.Schedd()
+        v2_schedd = htcondor.Schedd()
         test_info["V2-INPUT/CHECKPOINT"]["ads"] = v2_schedd.jobEpochHistory(None, proj, ad_type="input,checkpoint")
         test_info["V2-STARTER/OUTPUT"]["ads"] = v2_schedd.jobEpochHistory(None, proj, ad_type=["starter","output"])
         test_info["V2-Normal"]["ads"] = v2_schedd.jobEpochHistory(None, proj)
@@ -524,6 +569,41 @@ def runErrorCmds(default_condor, test_dir):
 @action(params={name: name for name in TEST_ERROR_CASES})
 def runFailed(request, runErrorCmds):
     return (request.param,runErrorCmds[request.param][0], runErrorCmds[request.param][1])
+
+#===============================================================================================
+@action(params={"forwards" : "-forwards", "backwards" : "-backwards"})
+def runEmptyBanner(default_condor, request):
+    EMPTY_BANNER_HISTORY = "empty.banner.hist"
+    with open(EMPTY_BANNER_HISTORY, "w") as f:
+        f.write(
+"""
+Foo=True
+Bar=2
+***
+Foo=False
+Bar=1
+***
+"""
+        )
+    p = default_condor.run_command(["condor_history",request.param,"-file",EMPTY_BANNER_HISTORY,"-af","Bar","Foo"])
+    return p
+
+#===============================================================================================
+@action
+def getDaemonHistory(default_condor):
+    # NOTE: This (by default) gets Schedd records from daemon history
+    p = default_condor.run_command(["condor_history", "-daemon"])
+    with default_condor.use_config():
+        hist = htcondor.param.get("SCHEDD_DAEMON_HISTORY")
+    return (hist, p)
+
+#===============================================================================================
+@action
+def pyBindGetScheddHist(default_condor):
+    with default_condor.use_config():
+        ads = htcondor.Schedd().daemonHistory(match=1, projection=["RecordWriteDate"])
+    return ads
+
 #===============================================================================================
 class TestCondorHistory:
 
@@ -628,3 +708,51 @@ class TestCondorHistory:
                 print(f"\nERROR: {runFailed[0]} failed to produce error message '{runFailed[1]}'")
         assert error_found == True
 
+    def test_empty_banner(self, runEmptyBanner):
+        assert runEmptyBanner.stderr == ""
+        for line in runEmptyBanner.stdout.split("\n"):
+            assert line == "2 true" or line == "1 false"
+
+    def test_daemon_history(self, getDaemonHistory):
+        path, cmd = getDaemonHistory
+
+        # Check daemon history file exists
+        assert path is not None
+        assert os.path.exists(path)
+
+        # Assert no errors written
+        assert cmd.stderr == ""
+
+        # Schedd Ad Sample: ' 5/13 09:14     67.11%         832       1903          67       145      13   632.10       4      1      13.05'
+        check = re.compile(r"\s*(\d+)/(\d+) *(\d+):(\d+)\s*(\d+).(\d+)%\s*(\d+)\s*(\d+)\s*(\d+)\s*(\d+)\s*(\d+)\s*(\d+).(\d+)\s*(\d+)\s*(\d+)\s*(\d+).(\d+)")
+        line_no = 0
+
+        # Check all stdout lines for matching header and regex pattern
+        for line in cmd.stdout.split("\n"):
+            if line_no == 0:
+                headings = line.strip().split()
+                assert headings == [
+                    "TIMESTAMP",
+                    "DUTY_CYCLE",
+                    "RunningJobs",
+                    "IdleJobs",
+                    "HeldJobs",
+                    "Download",
+                    "Waiting",   # download statistic
+                    "WaitingMB", # download statistic
+                    "Upload",
+                    "Waiting",   # upload statistic
+                    "WaitingMB"  # upload statistic
+                ]
+            else:
+                assert check.match(line)
+
+            line_no += 1
+
+    def test_py_schedd_hist(self, pyBindGetScheddHist):
+        assert pyBindGetScheddHist is not None
+        assert len(pyBindGetScheddHist) == 1
+
+        record = pyBindGetScheddHist[0]
+        # Make sure DaemonCore wrote write date attribute to record
+        assert "RecordWriteDate" in record
