@@ -2,7 +2,7 @@
 
 import os
 import logging
-import htcondor
+import htcondor2 as htcondor
 
 from ornithology import (
     action,
