@@ -67,9 +67,22 @@ def sif_file_fixture():
 def condor(test_dir):
     # Bind all the hosts's bin and lib dirs into the container, so can run /bin/cat and similar commands
     # without building a large container with lots of shared libraries
-    with Condor(test_dir / "condor", config={"SINGULARITY_BIND_EXPR": "\"/bin:/bin /usr:/usr /lib:/lib /lib64:/lib64\""}) as condor:
+    with Condor(test_dir / "condor", config={
+        "SINGULARITY_BIND_EXPR": "\"/bin:/bin /usr:/usr /lib:/lib /lib64:/lib64\"",
+        "SINGULARITY": "/usr/bin/singularity"
+        }) as condor:
         yield condor
 
+# Setup a personal condor with SINGULARITY_TARGET_DIR set
+@standup
+def condor_with_td(test_dir):
+    with Condor(test_dir / "condor_td", config={
+        "SINGULARITY_BIND_EXPR": "\"/bin:/bin /usr:/usr /lib:/lib /lib64:/lib64\"",
+        "SINGULARITY":           "/usr/bin/singularity",
+        "SINGULARITY_TARGET_DIR": "/td"}) as condor_with_td:
+        yield condor_with_td
+
+# The actual job will cat a file
 # The actual job will cat a file
 @action
 def test_job_hash():
@@ -102,6 +115,41 @@ def completed_test_job(condor, test_job_hash):
 
     return ctj.query()[0]
 
+@action
+def test_job_hash_with_xfer():
+    return {
+            "universe": "container",
+            "container_image": "empty.sif",
+            "executable": "/bin/cp",
+            "arguments": "an_input_file an_output_file",
+            "transfer_input_files": "an_input_file",
+            "should_transfer_files": "yes",
+            "when_to_transfer_output": "on_exit",
+            "transfer_executable": "False",
+            "output": "output",
+            "error": "error",
+            "log": "log_td",
+            "leave_in_queue": "True"
+            }
+# Test that file xfer works with SINGULARITY_TARGET_DIR
+@action
+def completed_test_job_with_xfer(condor_with_td, test_job_hash_with_xfer):
+
+    with open("an_input_file", "w") as f:
+        f.write("This is the input file\n")
+
+    ctj_td = condor_with_td.submit(
+        {**test_job_hash_with_xfer}, count=1
+    )
+
+    assert ctj_td.wait(
+        condition=ClusterState.all_terminal,
+        timeout=60,
+        verbose=True,
+        fail_condition=ClusterState.any_held,
+    )
+    return ctj_td.query()[0]
+
 # For the test to work, we need a singularity/apptainer which can work with
 # SIF files, which is any version of apptainer, or singularity >= 3
 def SingularityIsWorthy():
@@ -143,9 +191,11 @@ def UserNamespacesFunctional():
         print("unshare command failed, test cannot work, skipping test\n")
         return False
 
+@pytest.mark.skipif(not SingularityIsWorthy(), reason="No worthy Singularity/Apptainer found")
+@pytest.mark.skipif(not UserNamespacesFunctional(), reason="User namespaces not working -- some limit hit?")
+@pytest.mark.skipif(not SingularityIsWorking(), reason="Singularity doesn't seem to be working")
 class TestContainerUni:
-    @pytest.mark.skipif(not SingularityIsWorthy(), reason="No worthy Singularity/Apptainer found")
-    @pytest.mark.skipif(not UserNamespacesFunctional(), reason="User namespaces not working -- some limit hit?")
-    @pytest.mark.skipif(not SingularityIsWorking(), reason="Singularity doesn't seem to be working")
     def test_container_uni(self, sif_file_fixture, completed_test_job):
             assert completed_test_job['ExitCode'] == 0
+    def test_container_uni_with_xfer(self, completed_test_job_with_xfer):
+            assert completed_test_job_with_xfer['ExitCode'] == 0
