@@ -65,14 +65,6 @@ daemon_t dt = DT_MASTER;
 bool mixedcase = false;
 bool diagnostic = false;
 
-// The pure-tools (PureCoverage, Purify, etc) spit out a bunch of
-// stuff to stderr, which is where we normally put our error
-// messages.  To enable config_val.test to produce easily readable
-// output, even with pure-tools, we just write everything to stdout.  
-#if defined( PURE_DEBUG ) 
-#	define stderr stdout
-#endif
-
 enum PrintType {CONDOR_OWNER, CONDOR_TILDE, CONDOR_NONE};
 enum ModeType {CONDOR_QUERY, CONDOR_SET, CONDOR_UNSET,
 			   CONDOR_RUNTIME_SET, CONDOR_RUNTIME_UNSET};
@@ -1664,9 +1656,6 @@ int GetRemoteParamStats(Daemon* target, ClassAd & ad)
 			fprintf(stderr, "Can't read stats ad from %s\n", name);
 		}
 	}
-	// CRUFT: Remove this Remove() once no recent versions of Condor
-	//   automatically add CurrentTime to all ads.
-	ad.Remove("CurrentTime");
 	return 0;
 }
 
@@ -2277,174 +2266,6 @@ void PrintMetaParam(const char * name, bool expand, bool verbose)
 	}
 }
 
-#if 0 // code not yet ready for prime time.
-
-void DumpRemoteParams(Daemon* target, const char * tmp)
-{
-	std::vector<std::string> names;
-
-	// returns count of matches, negative values are errors
-	int iret = GetRemoteParamNamesMatching(target, tmp, names);
-	if (iret == -2) {
-		fprintf(stderr, "-dump is not supported by the remote version of HTCondor\n");
-		my_exit(1);
-	}
-	if (iret <= 0)
-		return;
-
-	std::string name_used, raw_value, file_and_line, def_value, usage_report;
-	bool raw_supported = false;
-
-	char * value = NULL;
-	for (int ii = 0; ii < (int)names.size(); ++ii) {
-		if (value) free(value);
-		char * value = GetRemoteParamRaw(target, names[ii].c_str(), raw_supported, raw_value, file_and_line, def_value, usage_report);
-		if (show_by_usage && ! usage_report.IsEmpty()) {
-			if (show_by_usage_unused && usage_report != "0")
-				continue;
-			if ( ! show_by_usage_unused && usage_report == "0")
-				continue;
-		}
-		if ( ! value && ! raw_supported)
-			continue;
-
-		name_used = names[ii];
-		if (expand_dumped_variables || ! raw_supported) {
-			printf("%s = %s\n", name_used.c_str(), value ? value : "");
-		} else {
-			printf("%s = %s\n", name_used.c_str(), RemoteRawValuePart(raw_value));
-		}
-		if ( ! raw_supported && (verbose || ! expand_dumped_variables)) {
-			printf(" # remote HTCondor version does not support -verbose\n");
-		}
-		if (verbose) {
-			if ( ! file_and_line.empty()) {
-				printf(" # at: %s\n", file_and_line.c_str());
-			}
-			if (expand_dumped_variables) {
-				printf(" # raw: %s\n", raw_value.c_str());
-			} else {
-				printf(" # expanded: %s\n", value);
-			}
-			if ( ! def_value.empty()) {
-				printf(" # default: %s\n", def_value.c_str());
-			}
-			if (dash_usage && ! usage_report.empty()) {
-				printf(" # use_count: %s\n", usage_report.c_str());
-			}
-		}
-	}
-	if (value) free(value);
-}
-
-
-void PrintParam(const char * tmp)
-{
-	std::string name_used, raw_value, file_and_line, def_value, usage_report;
-	bool raw_supported = false;
-	//fprintf(stderr, "param = %s\n", tmp);
-	if( target ) {
-		if (dump_stats) {
-			ClassAd stats;
-			int iret = GetRemoteParamStats(target, stats);
-			if (iret == -2) {
-				fprintf(stderr, "-stats is not supported by the remote version of HTCondor\n");
-				my_exit(1);
-			}
-			fPrintAd (stdout, stats);
-			fprintf (stdout, "\n");
-			if ( ! dump_all_variables) continue;
-		}
-		//fprintf(stderr, "dump = %d\n", dump_all_variables);
-		if (dump_all_variables) {
-			?? DumpRemoteParams();
-		} else if (dash_raw || verbose) {
-			name_used = tmp;
-			upper_case(name_used);
-			value = GetRemoteParamRaw(target, tmp, raw_supported, raw_value, file_and_line, def_value, usage_report);
-			if ( ! verbose && ! raw_value.empty()) {
-				free(value);
-				value = strdup(RemoteRawValuePart(raw_value));
-			}
-		} else {
-			name_used = tmp;
-			upper_case(name_used);
-			value = GetRemoteParam(target, tmp);
-		}
-        if (value && evaluate_daemon_vars) {
-            char* ev = EvaluatedValue(value, target->daemonAd());
-            if (ev != NULL) {
-                free(value);
-                value = ev;
-            } else {
-                fprintf(stderr, "Warning: Failed to evaluate '%s', returning it as config value\n", value);
-            }
-        }
-	} else {
-		const char * def_val;
-		const MACRO_META * pmet = NULL;
-		const char * val = param_get_info(tmp, subsys, local_name,
-								name_used, &def_val, &pmet);
-								//use_count, ref_count,
-								//file_and_line, line_number);
-		if ( ! name_used.empty()) {
-			raw_supported = true;
-			if (dash_raw) {
-				value = strdup(val ? val : "");
-			} else {
-				value = param(name_used.c_str());
-				raw_value = name_used;
-				//raw_value.upper_case();
-				raw_value += " = ";
-				raw_value += val ? val : "";
-			}
-			param_get_location(pmet, file_and_line);
-			if (pmet->ref_count) {
-				formatstr(usage_report, "%d / %d", pmet->use_count, pmet->ref_count);
-			} else {
-				formatstr(usage_report, "%d", pmet->use_count);
-			}
-		} else {
-			name_used = tmp;
-			upper_case(name_used);
-			value = NULL;
-		}
-	}
-	if( value == NULL ) {
-		fprintf(stderr, "Not defined: %s\n", name_used.c_str());
-		my_exit( 1 );
-	} else {
-		if (verbose) {
-			printf("%s = %s\n", name_used.c_str(), value);
-		} else {
-			printf("%s\n", value);
-		}
-		free( value );
-		if ( ! raw_supported && (dash_raw || verbose)) {
-			printf(" # the remote HTCondor version does not support -raw or -verbose");
-		}
-		if (verbose) {
-			if ( ! file_and_line.empty()) {
-				printf(" # at: %s\n", file_and_line.c_str());
-			}
-			if ( ! raw_value.empty()) {
-				printf(" # raw: %s\n", raw_value.c_str());
-			}
-			if ( ! def_value.empty()) {
-				printf(" # default: %s\n", def_value.c_str());
-			}
-			if (dump_both_only_type > 0) {
-				print_as_type(tmp, dump_both_only_type);
-			}
-			if (dash_usage && ! usage_report.empty()) {
-				printf(" # use_count: %s\n", usage_report.c_str());
-			}
-			printf("\n");
-		}
-	}
-}
-#endif
-
 static int do_write_config(const char* pathname, WRITE_CONFIG_OPTIONS opts)
 {
 	FILE * fh = NULL;
