@@ -32,7 +32,7 @@ class System:
         self.default_queue = default_queue
 
         assert isinstance(batch_system, str)
-        self.batch_system =  batch_system
+        self.batch_system = batch_system
 
         assert isinstance(executable, str)
         self.executable = executable
@@ -151,6 +151,52 @@ class Bridges2System(System):
         return queue
 
 
+class AWSEC2System(System):
+    def __init__(self, *,
+        pretty_name: str,
+        host_name: str,
+        default_queue: str,
+        batch_system: str,
+        executable: str,
+        other_scripts: list,
+        allocation_reqd: bool = False,
+        queues: dict,
+    ):
+        assert isinstance(pretty_name, str)
+
+        assert isinstance(pretty_name, str)
+        self.pretty_name = pretty_name
+
+        # assert isinstance(host_name, str)
+        self.host_name = host_name
+
+        # assert isinstance(default_queue, str)
+        self.default_queue = default_queue
+
+        assert isinstance(batch_system, str)
+        self.batch_system = batch_system
+
+        # assert isinstance(executable, str)
+        self.executable = executable
+
+        # assert isinstance(other_scripts, list)
+        self.other_scripts = other_scripts
+
+        assert isinstance(allocation_reqd, bool)
+        self.allocation_required = allocation_reqd
+
+        assert isinstance(queues, dict)
+        self.queues = queues
+
+
+    def validate_system_specific_constraints(self, queue_name, cpus, mem_mb):
+        return queue_name
+
+    def get_constraints(self, queue_name, gpus, gpu_type):
+        # Let's not to any validation for this quite yet.
+        return self.queues.get("<instance-type>")
+
+
 class PerlmutterSystem(System):
     def __init__(self, **kwargs):
         super().__init__(**kwargs)
@@ -220,8 +266,8 @@ SYSTEM_TABLE = {
         "host_name":        "perlmutter-p1.nersc.gov",
         "default_queue":    "regular",
         "batch_system":     "SLURM",
-        "executable":       "hpc.sh",
-        "other_scripts":    ["hpc.pilot", "hpc.multi-pilot"],
+        "executable":       "spark.sh",
+        "other_scripts":    ["spark.pilot", "spark.multi-pilot"],
         "allocation_reqd":  True,  # Only for GPUs, oddly.
 
         # Actually "QoS" limits.  See get_constraints().
@@ -257,8 +303,8 @@ SYSTEM_TABLE = {
         "host_name":        "login.delta.ncsa.illinois.edu",
         "default_queue":    "cpu",
         "batch_system":     "SLURM",
-        "executable":       "hpc.sh",
-        "other_scripts":    ["hpc.pilot", "hpc.multi-pilot"],
+        "executable":       "spark.sh",
+        "other_scripts":    ["spark.pilot", "spark.multi-pilot"],
         "allocation_reqd":  True,
 
         "queues": {
@@ -316,59 +362,51 @@ SYSTEM_TABLE = {
                 "gpus_per_node":        4,
                 "max_nodes_per_job":    4, # "TBD"
             },
-# Disabled until HTCONDOR-2377 is released (and the well-known URL for
-# binaries is updated to match).
-#            "gpuMI100x8": {
-#                "max_duration":         48 * 60 * 60,
-#                "allocation_type":      "cores_or_ram",
-#                "cores_per_node":       128,
-#                "ram_per_node":         2048 * 1024,
-#                "gpus_per_node":        8,
-#                "max_nodes_per_job":    4, # "TBD"
-#            },
+            "gpuMI100x8": {
+                "max_duration":         48 * 60 * 60,
+                "allocation_type":      "cores_or_ram",
+                "cores_per_node":       128,
+                "ram_per_node":         2048 * 1024,
+                "gpus_per_node":        8,
+                "max_nodes_per_job":    4, # "TBD"
+            },
         },
     }
     ),
 
-    "stampede2": System( **{
-        "pretty_name":      "Stampede 2",
-        "host_name":        "stampede2.tacc.utexas.edu",
-        "default_queue":    "normal",
+    "spark": System( **{
+        "pretty_name":      "Spark",
+        "host_name":        "spark-login.chtc.wisc.edu",
+        "default_queue":    "shared",
         "batch_system":     "SLURM",
-        "executable":       "hpc.sh",
-        "other_scripts":    ["hpc.pilot", "hpc.multi-pilot"],
+        "executable":       "spark.sh",
+        "other_scripts":    ["spark.pilot", "spark.multi-pilot"],
         "allocation_reqd":  False,
 
         "queues": {
-            "normal": {
-                "max_nodes_per_job":    256,
-                "max_duration":         48 * 60 * 60,
-                "allocation_type":      "node",
-                "cores_per_node":       68,
-                "ram_per_node":         96 * 1024,
-
-                "max_jobs_in_queue":    50,
+            "shared": {
+                "max_duration":         7 * 24 * 60 * 60,
+                "allocation_type":      "cores_or_ram",
+                "cores_per_node":       64,             # or 128
+                "ram_per_node":         512 * 1024,
+                "max_nodes_per_job":    2,              # "max 320 cores/job"
             },
-            "development": {
-                "max_nodes_per_job":    16,
-                "max_duration":         2 * 60 * 60,
-                "allocation_type":      "node",
-                "cores_per_node":       68,
-                "ram_per_node":         96 * 1024,
-
-                "max_jobs_in_queue":    1,
+            "int": {
+                "max_duration":         4 * 60 * 60,
+                "allocation_type":      "cores_or_ram",
+                "cores_per_node":       16,             # "max 16 cores/job"
+                "ram_per_node":         512 * 1024,     # "max 64 per job"
+                "max_nodes_per_job":    1,
             },
-            "skx-normal": {
-                "max_nodes_per_job":    128,
-                "max_duration":         48 * 60 * 60,
-                "allocation_type":      "node",
-                "cores_per_node":       48,
-                "ram_per_node":         192 * 1024,
-
-                "max_jobs_in_queue":    20,
+            "pre": {
+                "max_duration":         24 * 60 * 60,
+                "allocation_type":      "cores_or_ram",
+                "cores_per_node":       64,             # or 128
+                "ram_per_node":         512 * 1024,
+                "max_nodes_per_job":    2,              # "max 320 cores/job"
             },
         },
-    },
+    }
     ),
 
     "expanse": System( **{
@@ -376,8 +414,8 @@ SYSTEM_TABLE = {
         "host_name":        "login.expanse.sdsc.edu",
         "default_queue":    "compute",
         "batch_system":     "SLURM",
-        "executable":       "hpc.sh",
-        "other_scripts":    ["hpc.pilot", "hpc.multi-pilot"],
+        "executable":       "spark.sh",
+        "other_scripts":    ["spark.pilot", "spark.multi-pilot"],
         "allocation_reqd":  True,
 
         "queues": {
@@ -395,12 +433,13 @@ SYSTEM_TABLE = {
                 "max_duration":         48 * 60 * 60,
                 "allocation_type":      "node",
                 "cores_per_node":       40,
-                "ram_per_node":         384 * 1024,
+                "ram_per_node":         367 * 1024,
                 "gpus_per_node":        4,
 
                 "max_jobs_in_queue":    8,
 
                 "gpu_flag_type":        "job",
+                "memory_required":      True,
             },
             "shared": {
                 "max_nodes_per_job":    1,
@@ -416,8 +455,8 @@ SYSTEM_TABLE = {
                 "max_duration":         48 * 60 * 60,
                 "allocation_type":      "cores_or_ram",
                 "cores_per_node":       40,
-                "ram_per_node":         384 * 1024,
-                "gpus_per_node":        4,
+                "ram_per_node":         367 * 1024,
+                "gpus_per_node":        3,
 
                 "max_jobs_in_queue":    24,
 
@@ -432,8 +471,8 @@ SYSTEM_TABLE = {
         "host_name":        "anvil.rcac.purdue.edu",
         "default_queue":    "wholenode",
         "batch_system":     "SLURM",
-        "executable":       "hpc.sh",
-        "other_scripts":    ["hpc.pilot", "hpc.multi-pilot"],
+        "executable":       "spark.sh",
+        "other_scripts":    ["spark.pilot", "spark.multi-pilot"],
         "allocation_reqd":  False,
 
         "queues": {
@@ -471,7 +510,7 @@ SYSTEM_TABLE = {
                 "max_duration":         48 * 60 * 60,
                 "max_jobs_in_queue":    None,
 
-                "allocation_type":      "node",
+                "allocation_type":      "cores_or_ram",
 
                 "cores_per_node":       128,
                 "ram_per_node":         512000,
@@ -483,7 +522,7 @@ SYSTEM_TABLE = {
                 "max_duration":         30 * 60,
                 "max_jobs_in_queue":    1,
 
-                "allocation_type":      "node",
+                "allocation_type":      "cores_or_ram",
 
                 "cores_per_node":       128,
                 "ram_per_node":         512000,
@@ -498,8 +537,8 @@ SYSTEM_TABLE = {
         "pretty_name":      "Bridges-2",
         "host_name":        "bridges2.psc.edu",
         "default_queue":    "RM",
-        "executable":       "hpc.sh",
-        "other_scripts":    ["hpc.pilot", "hpc.multi-pilot"],
+        "executable":       "spark.sh",
+        "other_scripts":    ["spark.pilot", "spark.multi-pilot"],
         "batch_system":     "SLURM",
         "allocation_reqd":  False,
 
@@ -596,6 +635,26 @@ SYSTEM_TABLE = {
         },
     },
     ),
+
+
+    "aws-ec2": AWSEC2System( **{
+        "pretty_name":      "AWS EC2",
+        "host_name":        None,
+        "default_queue":    "m4.large",
+        "batch_system":     "aws-ec2",
+        "executable":       None,
+        "other_scripts":    None,
+        "allocation_reqd":  False,
+
+        "queues": {
+            "<instance-type>": {
+                # All instance types are whole-node allocated.
+                "allocation_type":      "node",
+                "max_nodes_per_job":    None,
+            },
+        },
+    },
+    ),
 }
 
 
@@ -639,8 +698,8 @@ def validate_constraints( *,
             raise ValueError(error_string)
 
         # Don't allow the user to request more than max nodes.
-        mnpj = queue['max_nodes_per_job']
-        if nodes > mnpj:
+        mnpj = queue.get('max_nodes_per_job')
+        if mnpj is not None and nodes > mnpj:
             error_string = f"The '{queue_name}' queue is limited to {mnpj} nodes per job.  Use --nodes to set."
             raise ValueError(error_string)
 
@@ -683,7 +742,8 @@ def validate_constraints( *,
             raise ValueError(error_string)
 
     # (E) You must not specify a lifetime longer than the queue's duration.
-    if queue['max_duration'] < lifetime_in_seconds:
+    md = queue.get('max_duration')
+    if md is not None and md < lifetime_in_seconds:
         error_string = f"The '{queue_name}' queue has a maximum duration of {queue['max_duration']} seconds, which is less than the requested lifetime ({lifetime_in_seconds} seconds).  Use --lifetime to set."
         raise ValueError(error_string)
 
@@ -715,7 +775,10 @@ def validate_constraints( *,
             if queue['allocation_type'] == 'node':
                 gpus = gpus_per_node
 
-        if gpus <= 0:
+        # `gpus` is a local, so we can't just set it to a sane default
+        # in an else clause above.  We can't set a default in the argument
+        # parser because `None` means "all" for the whole-node queues.
+        if gpus is None or gpus <= 0:
             error_string = f"The '{queue_name}' queue is a GPU queue.  You must specify a number of GPUs (--gpus)."
             raise ValueError(error_string)
 
@@ -759,4 +822,8 @@ def validate_constraints( *,
                 assert nodes is not None and nodes >= 1, f"Internal error during validation: node count ({nodes}) should have been >= 1 by now."
                 gpus = gpus * nodes
 
-    return gpus, queue_name
+    if queue.get('memory_required') is True:
+        if mem_mb is None:
+            mem_mb = queue['ram_per_node']
+
+    return gpus, queue_name, mem_mb
