@@ -64,11 +64,12 @@ enum {
    DetailOrder     = 0x8000,
    DetailRequested = 0x10000,
    DetailCeiling   = 0x20000,
+   DetailFloor     = 0x40000,
    DetailPrios     = DetailPriority | DetailFactor | DetailRealPrio,
    DetailUsage     = DetailResUsed | DetailWtResUsed,
    DetailQuota2    = DetailEffQuota | DetailCfgQuota,
    DetailQuotas    = DetailEffQuota | DetailCfgQuota | DetailTreeQuota | DetailSurplus | DetailRequested,
-   DetailMost      = DetailCfgQuota | DetailSurplus | DetailPriority | DetailFactor | DetailUsage | DetailUseDeltaT | DetailRequested | DetailCeiling,
+   DetailMost      = DetailCfgQuota | DetailSurplus | DetailPriority | DetailFactor | DetailUsage | DetailUseDeltaT | DetailRequested | DetailCeiling | DetailFloor,
    DetailAll       = DetailMost | DetailQuotas | DetailPrios | DetailUseTime1 | DetailUseTime2,
    DetailDefault   = DetailMost // show this if none of the flags controlling details is set.
 };
@@ -89,8 +90,8 @@ struct LineRec {
   double AccUsage;
   double Requested;
   double Factor;
-  int BeginUsage;
-  int LastUsage;
+  time_t BeginUsage;
+  time_t LastUsage;
   std::string AcctGroup;
   bool IsAcctGroup;
   int   HasDetail;      // one or more of Detailxxx flags indicating the that data exists.
@@ -98,6 +99,7 @@ struct LineRec {
   double ConfigQuota;
   double SubtreeQuota;
   double Ceiling;
+  double Floor;
   double SortKey;
   int   Surplus;       // 0 is no, 1 = regroup (by prio), 2 = accept_surplus (by quota)
   int   index;
@@ -107,7 +109,7 @@ struct LineRec {
 
 //-----------------------------------------------------------------
 
-static int CalcTime(int,int,int);
+static time_t CalcTime(int,int,int);
 static void usage(const char* name);
 static void ProcessInfo(ClassAd* ad,std::vector<ClassAd> &accountingAds, bool GroupRollup,bool HierFlag);
 static int CountElem(ClassAd* ad);
@@ -591,21 +593,16 @@ main(int argc, const char* argv[])
   if (!negotiator.locate(Daemon::LOCATE_FOR_ADMIN)) {
 	  fprintf(stderr, "%s: Can't locate negotiator in %s\n", 
               argv[0], pool_name ? pool_name : "local pool");
-	  exit(1);
+	  // If we are getting userprio info from the collector, don't exit,
+	  // as we don't need to talk to the negotiator. If we do, we'll
+	  // exit with an approriate error later.
   }
 
-	// Only 8.5.3+ negotiators send their accounting data to the collector
-  const char *ver = negotiator.version();
-  if (ver) {
-	CondorVersionInfo vsi(ver);
-	if (!forceFromCollector && vsi.built_since_version(8,5,3)) {
-		fromCollector = true;
-	}
-	negotiatorCanDoDirect = vsi.built_since_version(9,1,1);
-  } else {
-	// no version means that the negotiator is local, so it must be our version... right?
-	negotiatorCanDoDirect = true;
+  if (!forceFromCollector) {
+	  fromCollector = true;
   }
+  negotiatorCanDoDirect = true; // Assumes a post version 9.1.1 negotiator, which supports direct query.
+ 
   // knob to disable negotiator modular direct query, in case this causes problems (the results *are* a bit different)
   if ( ! param_boolean("USERPRIO_USE_NEGOTIATOR_MODULAR_QUERY", false)) {
 	  negotiatorCanDoDirect = false;
@@ -702,7 +699,7 @@ main(int argc, const char* argv[])
 
   }
 
-  else if (SetFloor || SetCeiling) { // set ceiling
+  else if (SetFloor || SetCeiling) { // set floor/ceiling
 
 
 	int argIndex;
@@ -970,7 +967,7 @@ main(int argc, const char* argv[])
   else if (UserPrioFile) {
 
     const char * filename = argv[UserPrioFile+1];
-    FILE* file = safe_fopen_wrapper_follow(filename, "r");
+    FILE* file = safe_fopen_wrapper_follow(filename, "rb");
     if (file == NULL) {
       fprintf(stderr, "Can't open file of userprio ads: %s\n", filename);
       exit(1);
@@ -1223,10 +1220,16 @@ static void CollectInfo(int numElem, ClassAd* ad, std::vector<ClassAd> &accounti
   char  attrLastUsage[64];
   std::string attrAcctGroup;
   std::string attrIsAcctGroup;
+  std::string attrFloor;
   std::string attrCeiling;
   char  name[128], policy[32];
-  double priority = 0, Factor = 0, AccUsage = -1, ceiling = -1;
-  int   resUsed = 0, BeginUsage = 0;
+  double priority = 0;
+  double Factor = 0;
+  double AccUsage = -1;
+  double ceiling = -1;
+  double floor = -1;
+  int   resUsed = 0;
+  time_t BeginUsage = 0;
   int   LastUsage = 0;
   double wtResUsed, requested = 0;
   std::string AcctGroup;
@@ -1245,6 +1248,7 @@ static void CollectInfo(int numElem, ClassAd* ad, std::vector<ClassAd> &accounti
     LR[i-1].LastUsage=MinLastUsageTime;
     LR[i-1].Requested=0.0;
     LR[i-1].Ceiling=-1;
+    LR[i-1].Floor=-1;
 
 	char strI[32];
 
@@ -1269,6 +1273,7 @@ static void CollectInfo(int numElem, ClassAd* ad, std::vector<ClassAd> &accounti
     formatstr(attrAcctGroup, "AccountingGroup%s", strI);
     formatstr(attrIsAcctGroup, "IsAccountingGroup%s", strI);
     formatstr(attrCeiling, "Ceiling%s", strI);
+    formatstr(attrFloor, "Floor%s", strI);
 
     if( !ad->LookupString	( attrName, name, COUNTOF(name) ) 		|| 
 		!ad->LookupFloat	( attrPrio, priority ) )
@@ -1283,6 +1288,7 @@ static void CollectInfo(int numElem, ClassAd* ad, std::vector<ClassAd> &accounti
        }
 	if( ad->LookupFloat( attrAccUsage, AccUsage ) ) LR[i-1].HasDetail |= DetailUsage;
 	if( ad->LookupFloat( attrCeiling, ceiling ) ) LR[i-1].HasDetail |= DetailCeiling;
+	if( ad->LookupFloat( attrFloor, floor ) ) LR[i-1].HasDetail |= DetailFloor;
 	if( ad->LookupFloat( attrRequested, requested ) ) LR[i-1].HasDetail |= DetailRequested;
 	if( ad->LookupInteger( attrBeginUsage, BeginUsage ) ) LR[i-1].HasDetail |= DetailUseTime1;
 	if( ad->LookupInteger( attrLastUsage, LastUsage ) ) LR[i-1].HasDetail |= DetailUseTime2 | DetailUseDeltaT;
@@ -1382,6 +1388,7 @@ static void CollectInfo(int numElem, ClassAd* ad, std::vector<ClassAd> &accounti
     LR[i-1].LastUsage=LastUsage;
     LR[i-1].AccUsage=AccUsage;
     LR[i-1].Ceiling=ceiling;
+    LR[i-1].Floor=floor;
     LR[i-1].AcctGroup=AcctGroup;
     LR[i-1].IsAcctGroup=IsAcctGroup;
     LR[i-1].EffectiveQuota = effective_quota;
@@ -1509,7 +1516,7 @@ static char * CopyAndPadToWidth(char * pszDest, const char * pszSrc, int cch, in
    return pszDest;
 }
 
-static char * FormatDateTime(char * pszDest, int cchDest, int dtOne, const char * pszTimeZero)
+static char * FormatDateTime(char * pszDest, int cchDest, time_t dtOne, const char * pszTimeZero)
 {
    if (pszTimeZero && dtOne <= 0)
       CopyAndPadToWidth(pszDest, pszTimeZero, cchDest, ' ', PAD_LEFT);
@@ -1518,7 +1525,7 @@ static char * FormatDateTime(char * pszDest, int cchDest, int dtOne, const char
    return pszDest;
 }
 
-static char * FormatDeltaTime(char * pszDest, int cchDest, int tmDelta, const char * pszDeltaZero)
+static char * FormatDeltaTime(char * pszDest, int cchDest, time_t tmDelta, const char * pszDeltaZero)
 {
    if (pszDeltaZero && tmDelta <= 0) {
       CopyAndPadToWidth(pszDest, pszDeltaZero, cchDest, ' ', PAD_LEFT);
@@ -1578,7 +1585,8 @@ static const struct {
    { DetailUseTime2,  16, "Last\0Usage Time" },
    { DetailUseDeltaT, 10, "Time Since\0Last Usage" },
    { DetailRequested, 10, "Weighted\0Requested" },
-   { DetailCeiling,    9, "Submitter\0Ceiling" }
+   { DetailFloor  ,    9, "Submitter\0Floor" },
+   { DetailCeiling,    9, "Submitter\0Ceiling" },
 };
 const int MAX_NAME_COLUMN_WIDTH = 99;
 
@@ -1783,6 +1791,13 @@ static void PrintInfo(int tmLast, LineRec* LR, int NumElem, bool HierFlag)
 					CopyAndPadToWidth(Line+ix, NULL, aCols[ii].width+1, ' ');
 				}
                break;
+            case DetailFloor: 
+				if (LR[j].Floor > 0.0) {
+					FormatFloat(Line+ix, aCols[ii].width, 2, LR[j].Floor);
+				} else {
+					CopyAndPadToWidth(Line+ix, NULL, aCols[ii].width+1, ' ');
+				}
+               break;
             case item_NA:
                CopyAndPadToWidth(Line+ix, "n/a", aCols[ii].width+1, ' ', PAD_LEFT);
                break;
@@ -1899,6 +1914,10 @@ static void usage(const char* name) {
      "\t-hierarchical\t\tDisplay users under their groups\n"
      "\t-grouporder\t\tDisplay groups first, then users\n"
      "\t-grouprollup\t\tGroup value are the sum of user values\n"
+     "\t-groupid\t\tDisplay group ID numbers\n"
+     "\t-order\t\t\tDisplay group order\n"
+     "\t-sortkey\t\tDisplay group sort key\n"
+     "\t-surplus\t\tDisplay usage surplus\n"
      "\t-collector\t\tForce the query to come from the collector\n"
      "\t-negotiator\t\tForce the query to come from the negotiator\n"
      "\t-long\t\t\tVerbose output (entire classads)\n"
@@ -1965,7 +1984,7 @@ static void PrintResList(ClassAd* ad)
 
 //-----------------------------------------------------------------
 
-int CalcTime(int month, int day, int year) {
+time_t CalcTime(int month, int day, int year) {
   struct tm time_str;
   if (year<50) year +=100; // If I ask for 1 1 00, I want 1 1 2000, not 1 1 1900
   if (year>1900) year-=1900;
