@@ -32,6 +32,7 @@ enum {
 	FormatOptionNoTruncate = 0x04,
 	FormatOptionAutoWidth = 0x08,
 	FormatOptionLeftAlign = 0x10,
+	FormatOptionMultiLine = 0x40,
 	FormatOptionAlwaysCall = 0x80,
 	FormatOptionHideMe    = 0x100,
 	FormatOptionFitToData = 0x200,	  // for use by the adjust_formats callback
@@ -135,6 +136,7 @@ class AttrListPrintMask
 	~AttrListPrintMask ();
 
 	void SetAutoSep(const char* rpre, const char * cpre, const char * cpost, const char * rpost);
+	bool IsLinePerColumn() { return (col_suffix && strchr(col_suffix, '\n')) || (col_prefix && strchr(col_prefix, '\n')); }
 	void SetOverallWidth(int wid);
 
 	// register a format and an attribute
@@ -169,6 +171,8 @@ class AttrListPrintMask
 	void set_heading(const char * heading);
 	bool has_headings() { return !headings.empty(); }
 	void clear_headings() { headings.clear(); }
+	void pop_back_heading() { headings.pop_back(); } // support hacky heading re-writing in condor_status
+	const char * store(std::string_view str) { return stringpool.insert(str); }
 	const char * store(const char * psz) { return stringpool.insert(psz); } // store a string in the local string pool.
 	// iterate formatter and attribs, calling pfn and allowing fmt to be changed until pfn returns < 0
 	int adjust_formats(int (*pfn)(void*pv, int index, Formatter * fmt, const char * attr), void* pv);
@@ -316,9 +320,9 @@ protected:
 
 // simple line at a time file (or stdin) reader
 class SimpleFileInputStream : public SimpleInputStream {
-	FILE * file;
-	bool   auto_close_file; // file is owned by this class, close it in destructor
-	int    lines_read;
+	FILE * file{nullptr};
+	bool   auto_close_file{false}; // file is owned by this class, close it in destructor
+	int    lines_read{0};
 	SimpleFileInputStream() {}; // don't allow default construction
 public:
 	// Create a simple input stream for reading lines from an open file
@@ -331,10 +335,10 @@ public:
 
 // Simple line at a time string literal reader 
 class StringLiteralInputStream : public SimpleInputStream {
-	const char * lit; // points to a string literal, so it's not freed by this class
+	const char * lit{nullptr}; // points to a string literal, so it's not freed by this class
 	std::string line; // temp for the current line to return.
-	size_t ix_eol;    // end of current line in lit
-	int    lines_read;
+	size_t ix_eol{0};    // end of current line in lit
+	int    lines_read{0};
 	StringLiteralInputStream() {}; // don't allow default construction
 public:
 	StringLiteralInputStream(const char* psz) : lit(psz), ix_eol(0), lines_read(0) {}
@@ -370,14 +374,15 @@ public:
 // it is used by SetAttrListPrintMaskFromStream and by PrintPrintMask
 typedef struct PrintMaskMakeSettings {
 	std::string select_from;           // out: adtype name from SELECT
-	printmask_headerfooter_t headfoot; // out, header and footer flags set in SELECT or SUMMARY
-	printmask_aggregation_t aggregate; // out: aggregation mode in SELECT
+	printmask_headerfooter_t headfoot{STD_HEADFOOT};      // out, header and footer flags set in SELECT or SUMMARY
+	printmask_aggregation_t aggregate{PR_NO_AGGREGATION}; // out: aggregation mode in SELECT
 	std::string where_expression;      // out: classad expression from WHERE
 	classad::References attrs;        // out: ClassAd attributes referenced in mask or group_by outputs
 	classad::References scopes;       // out: scopes for ClassAd attributes referenced in mask or group_by outputs (i.e. target or job)
 	classad::References sumattrs;     // out: ClassAd attributes referenced in summary mask
+	bool generate_printf_from_width{true}; // if WIDTH given but not PRINTF or PRINTAS, assume a %Nv or %N.Nv printf
+	bool fixed_width_implies_truncate{true}; // if WIDTH specified and not AUTO or FIT assume TRUNCATE
 
-	PrintMaskMakeSettings() : headfoot(STD_HEADFOOT), aggregate(PR_NO_AGGREGATION) {}
 	void reset() {
 		select_from.clear();
 		where_expression.clear();
