@@ -69,7 +69,7 @@ int cleanStringForUseAsAttr(std::string &str, char chReplace/*=0*/, bool compact
          replace_str(str, " ", "");
       else {
          std::string tmp; tmp += chReplace; tmp += chReplace;
-         replace_str(str, tmp.c_str(), tmp.c_str()+1);
+         replace_str(str, tmp, tmp.c_str()+1);
       }
    }
    trim(str);
@@ -83,10 +83,15 @@ int cleanStringForUseAsAttr(std::string &str, char chReplace/*=0*/, bool compact
   specifies a signal we want to use.  The signal could either be
   stored as an int, or as a string.  if it's a string, we want to
   translate it into the appropriate signal number for this platform. 
+  This is relevant only on unix. On Windows, we always return SIGTERM,
+  which DaemonCore will map to WM_CLOSE.
 */
 static int
 findSignal( ClassAd* ad, const char* attr_name )
 {
+#if defined(WIN32)
+	return SIGTERM;
+#else
 	if( ! ad ) {
 		return -1;
 	}
@@ -100,6 +105,7 @@ findSignal( ClassAd* ad, const char* attr_name )
 	} else {
 		return -1;
 	}
+#endif
 }
 
 
@@ -255,7 +261,7 @@ ClassAd *CreateJobAd( const char *owner, int universe, const char *cmd )
 	job_ad->Assign( ATTR_JOB_UNIVERSE, universe );
 	job_ad->Assign( ATTR_JOB_CMD, cmd );
 
-	job_ad->Assign( ATTR_Q_DATE, (int)time(NULL) );
+	job_ad->Assign( ATTR_Q_DATE, time(nullptr));
 
 	job_ad->Assign( ATTR_JOB_REMOTE_WALL_CLOCK, 0.0 );
 	job_ad->Assign( ATTR_JOB_REMOTE_USER_CPU, 0.0 );
@@ -286,9 +292,6 @@ ClassAd *CreateJobAd( const char *owner, int universe, const char *cmd )
 	job_ad->Assign( ATTR_ENTERED_CURRENT_STATUS, time(nullptr) );
 
 	job_ad->Assign( ATTR_JOB_PRIO, 0 );
-#ifdef NO_DEPRECATE_NICE_USER
-	job_ad->Assign( ATTR_NICE_USER, false );
-#endif
 
 	job_ad->Assign( ATTR_JOB_NOTIFICATION, NOTIFY_NEVER );
 
@@ -384,29 +387,3 @@ const char *print_attrs(std::string &out, bool append, const classad::References
 	}
 	return out.c_str();
 }
-
-// copy attrs into stringlist, returns true if list was modified
-// if append is false, list is cleared first.
-// if check_exist is true, items are only added if the are not already in the list. comparison is case-insensitive.
-bool initStringListFromAttrs(StringList & list, bool append, const classad::References & attrs, bool check_exist /*=false*/)
-{
-	bool modified = false;
-	if ( ! append) {
-		if ( ! list.isEmpty()) {
-			modified = true;
-			list.clearAll();
-		}
-		check_exist = false; // no need to do this if we cleared the list
-	}
-	for (classad::References::const_iterator it = attrs.begin(); it != attrs.end(); ++it) {
-		if (check_exist && list.contains_anycase(it->c_str())) {
-			continue;
-		}
-		list.append(it->c_str());
-		modified = true;
-	}
-	return modified;
-}
-
-
-
