@@ -352,27 +352,11 @@ bool getClassAdEx( Stream *sock, classad::ClassAd& ad, int options)
 		dprintf(D_FULLDEBUG, "getClassAd FAILED to get MyType\n" );
 		return false;
 	}
-#if 0 // we fetch but ignore MyType and TargetType
-	if (strptr && strptr[0]) {
-		if (YourString(strptr) != "(unknown type)" && !ad.InsertAttr("MyType",strptr)) {
-			dprintf(D_FULLDEBUG, "getClassAd FAILED to insert MyType=\"%s\"\n", strptr );
-			return false;
-		}
-	}
-#endif
 
 	if (!sock->get_string_ptr(strptr, cb)) {
 		dprintf(D_FULLDEBUG, "getClassAd FAILED to get TargetType\n" );
 		return false;
 	}
-#if 0 // we fetch but ignore MyType and TargetType
-	if (strptr && strptr[0]) {
-		if (YourString(strptr) != "(unknown type)" && !ad.InsertAttr("TargetType",strptr)) {
-			dprintf(D_FULLDEBUG, "getClassAd FAILED to insert TargetType=\"%s\"\n", strptr );
-			return false;
-		}
-	}
-#endif
 
 	return true;
 }
@@ -512,8 +496,6 @@ int putClassAd (Stream *sock, const classad::ClassAd& ad, int options, const cla
 
 	bool expand_whitelist = ! (options & PUT_CLASSAD_NO_EXPAND_WHITELIST);
 	if (whitelist && expand_whitelist) {
-		// Jaime made changes to the core classad lib that make this unneeded...
-		//ad.InsertAttr("MY","SELF");
 		for (classad::References::const_iterator attr = whitelist->begin(); attr != whitelist->end(); ++attr) {
 			ExprTree * tree = ad.Lookup(*attr);
 			if (tree) {
@@ -523,9 +505,6 @@ int putClassAd (Stream *sock, const classad::ClassAd& ad, int options, const cla
 				}
 			}
 		}
-		//ad.Delete("MY");
-		//classad::References::iterator my = expanded_whitelist.find("MY");
-		//if (my != expanded_whitelist.end()) { expanded_whitelist.erase(my); }
 		whitelist = &expanded_whitelist;
 	}
 
@@ -596,7 +575,6 @@ int _putClassAd( Stream *sock, const classad::ClassAd& ad, int options,
 
 	classad::ClassAdUnParser	unp;
 	std::string					buf;
-	buf.reserve(65536);
 	bool send_server_time = false;
 
 	unp.SetOldClassAd( true, true );
@@ -788,7 +766,6 @@ int _putClassAd( Stream *sock, const classad::ClassAd& ad, int options, const cl
 	}
 
 	std::string buf;
-	buf.reserve(65536);
 	bool crypto_is_noop =  sock->prepare_crypto_for_secret_is_noop();
 	for (classad::References::const_iterator attr = whitelist.begin(); attr != whitelist.end(); ++attr) {
 
