@@ -35,10 +35,10 @@ int ParseClassAdRvalExpr(const char*s, classad::ExprTree*&tree)
 {
 	classad::ClassAdParser parser;
 	parser.SetOldClassAd( true );
-	if ( parser.ParseExpression( s, tree, true ) ) {
+	tree = parser.ParseExpression(s, true);
+	if ( tree ) {
 		return 0;
 	} else {
-		tree = NULL;
 		return 1;
 	}
 }
@@ -241,6 +241,28 @@ bool ExprTreeIsLiteralString(classad::ExprTree * expr, std::string & sval)
 	return val.IsStringValue(sval);
 }
 
+bool ExprTreeIsArray(classad::ExprTree * expr, size_t & num_elms)
+{
+	if ( ! expr) return false;
+
+	classad::ExprTree::NodeKind kind = expr->GetKind();
+	if (kind == classad::ExprTree::EXPR_ENVELOPE) {
+		expr = ((classad::CachedExprEnvelope*)expr)->get();
+		if ( ! expr) return false;
+		kind = expr->GetKind();
+	}
+
+	if (kind == classad::ExprTree::EXPR_LIST_NODE) {
+		std::vector<classad::ExprTree*> exprs;
+		((const classad::ExprList*)expr)->GetComponents( exprs );
+		num_elms = exprs.size();
+		return true;
+	}
+
+	return false;
+}
+
+
 bool ExprTreeIsAttrRef(classad::ExprTree * expr, std::string & attr, bool * is_absolute /*=NULL*/)
 {
 	if ( ! expr) return false;
@@ -563,6 +585,57 @@ int GetAttrRefsOfScope(classad::ExprTree * expr, classad::References &attrs, con
 	 return walk_attr_refs(expr, AccumAttrsOfScopes, &tmp);
 }
 
+class AttrsScopesOrUnscoped {
+public:
+	AttrsScopesOrUnscoped() : attrs(nullptr) {}
+	classad::References *attrs;
+	classad::References scopes; // scopes we care about
+};
+
+int AccumAttrsOfScopeOrUnscoped(void *pv, const std::string &attr, const std::string &scope, bool /*absolute*/)
+{
+	AttrsScopesOrUnscoped &ctx = *(AttrsScopesOrUnscoped *)pv;
+	if (scope.empty() || ctx.scopes.find(scope) != ctx.scopes.end()) {
+		ctx.attrs->insert(attr);
+	}
+	return 1;
+}
+
+class AttrsTwoScopesOrUnscoped {
+public:
+	AttrsTwoScopesOrUnscoped(classad::References &a1, const std::string &s1,
+	                        classad::References &a2, const std::string &s2)
+	    : scope1_attrs(a1), scope2_attrs(a2), scope1(s1), scope2(s2) {}
+
+	classad::References &scope1_attrs;
+	classad::References &scope2_attrs;
+	std::string scope1;
+	std::string scope2;
+};
+
+int AccumAttrsOfScopesOrUnscoped(void *pv, const std::string &attr, const std::string &scope, bool /*absolute*/)
+{
+	AttrsTwoScopesOrUnscoped &ctx = *(AttrsTwoScopesOrUnscoped *)pv;
+	if (scope.empty()) {
+		ctx.scope1_attrs.insert(attr);
+		ctx.scope2_attrs.insert(attr);
+	} else if (scope == ctx.scope1) {
+		ctx.scope1_attrs.insert(attr);
+	} else if (scope == ctx.scope2) {
+		ctx.scope2_attrs.insert(attr);
+	}
+	return 1;
+}
+
+// add attribute references to two scope sets in a single walk. Unscoped references go to both sets.
+int GetAttrRefsOfScopesOrUnscoped(classad::ExprTree *expr,
+    classad::References &scope1_attrs, const std::string &scope1,
+    classad::References &scope2_attrs, const std::string &scope2)
+{
+	AttrsTwoScopesOrUnscoped ctx(scope1_attrs, scope1, scope2_attrs, scope2);
+	return walk_attr_refs(expr, AccumAttrsOfScopesOrUnscoped, &ctx);
+}
+
 
 // edit the given expr changing attribute references as the mapping indicates
 int RewriteAttrRefs(classad::ExprTree * tree, const NOCASE_STRING_MAP & mapping)
@@ -655,10 +728,14 @@ int RewriteAttrRefs(classad::ExprTree * tree, const NOCASE_STRING_MAP & mapping)
 		}
 		break;
 
-		case classad::ExprTree::EXPR_ENVELOPE:
+		case classad::ExprTree::EXPR_ENVELOPE: {
+			classad::ExprTree *inner = SkipExprEnvelope(tree);
+			if (inner) { iret += RewriteAttrRefs(inner, mapping); }
+		}
+		break;
+
 		default:
-			// unknown or unallowed node.
-			ASSERT(0);
+			// unknown or unhandled node kind; leave untouched.
 		break;
 	}
 	return iret;
@@ -734,7 +811,8 @@ bool ClassAdsAreSame( ClassAd *ad1, ClassAd * ad2, classad::References *ignored_
 // This will also copy Available<res> attributes and any attributes referenced therein
 void CopyMachineResources(ClassAd &destAd, const ClassAd & srcAd, bool include_res_list)
 {
-	std::string resnames, attr;
+	std::string resnames;
+	std::string attr;
 	if(srcAd.LookupString( ATTR_MACHINE_RESOURCES, resnames)) {
 		if (include_res_list) { destAd.Assign(ATTR_MACHINE_RESOURCES, resnames); }
 	} else {
@@ -761,7 +839,7 @@ void CopyMachineResources(ClassAd &destAd, const ClassAd & srcAd, bool include_r
 
 			classad::References refs;
 			srcAd.GetInternalReferences(tree, refs, true);
-			for (auto it : refs) {
+			for (const auto& it : refs) {
 				ExprTree * expr = srcAd.Lookup(it);
 				if (expr) { 
 					expr = SkipExprEnvelope(expr);
@@ -780,7 +858,7 @@ void CopySelectAttrs(ClassAd &destAd, const ClassAd &srcAd, const std::string &a
 	classad::References refs;
 	StringTokenIterator listAttrs(attrs);
 	// Create set of attribute references to copy
-	for (auto attr : listAttrs) {
+	for (const auto& attr : listAttrs) {
 		ExprTree *tree = srcAd.Lookup(attr);
 		if (tree) {
 			refs.insert(attr);
@@ -788,7 +866,7 @@ void CopySelectAttrs(ClassAd &destAd, const ClassAd &srcAd, const std::string &a
 		}
 	}
 	// Copy found references
-	for (auto it : refs) {
+	for (const auto &it : refs) {
 		ExprTree *expr = srcAd.Lookup(it);
 		if (expr) {
 			// Only copy if given overwrite or if not found in destAd
@@ -1008,9 +1086,31 @@ ClassAdFileParseType::ParseType parseAdsFileFormat(const char * arg, ClassAdFile
 	YourString fmt(arg);
 	if (fmt == "long") { parse_type = ClassAdFileParseType::Parse_long; }
 	else if (fmt == "json") { parse_type = ClassAdFileParseType::Parse_json; }
+	else if (fmt == "jsonl") { parse_type = ClassAdFileParseType::Parse_json_lines; }
 	else if (fmt == "xml") { parse_type = ClassAdFileParseType::Parse_xml; }
 	else if (fmt == "new") { parse_type = ClassAdFileParseType::Parse_new; }
+	else if (fmt == "newl") { parse_type = ClassAdFileParseType::Parse_new_l; }
 	else if (fmt == "auto") { parse_type = ClassAdFileParseType::Parse_auto; }
 	return parse_type;
 }
 
+int 
+SlotRankSortFunc(ClassAd *ad1,ClassAd *ad2,void *data)
+{
+	ClassAd *rank_ad = (ClassAd *)data;
+
+	double rank1 = 0.0;
+	double rank2 = 0.0;
+	EvalFloat(ATTR_RANK,rank_ad,ad1,rank1);
+	EvalFloat(ATTR_RANK,rank_ad,ad2,rank2);
+
+	// Must provide a strict weak ordering, so we can't just
+	// return rank1 > rank2
+
+	if (fabs(rank1 - rank2) < DBL_EPSILON) {
+		return ad1 > ad2;
+	}
+	return rank1 > rank2;
+}
+
+
