@@ -57,7 +57,7 @@ AdTypeToString( AdTypes type )
 		DATABASE_ADTYPE,	//DATABASE_AD
 		TT_ADTYPE,			//TT_AD
 		GRID_ADTYPE, 		//GRID_AD
-		XFER_SERVICE_ADTYPE,//XFER_SERVICE_AD	/* No longer used */
+		PLACEMENTD_ADTYPE,	//PLACEMENTD_AD
 		LEASE_MANAGER_ADTYPE,//LEASE_MANAGER_AD	/* No longer used */
 		DEFRAG_ADTYPE,		//DEFRAG_AD
 		ACCOUNTING_ADTYPE,	//ACCOUNTING_AD = 23
