@@ -13,18 +13,22 @@ condor_netaddr::condor_netaddr(const condor_sockaddr& base,
 
 void condor_netaddr::set_mask() {
 	if (base_.is_ipv4()) {
-		uint32_t mask;
-		mask = htonl(~(0xffffffff >> maskbit_));
+		uint32_t mask = 0xffffffff;
+		if (maskbit_ < 32) {
+			mask = htonl(~(0xffffffff >> maskbit_));
+		}
 		mask_ = condor_sockaddr( *(struct in_addr*)&mask );
 	} else {
 		uint32_t mask[4] = { };
 		int curmaskbit = maskbit_;
-		for (size_t i = 0; i < sizeof(mask); i++) {
-			if (curmaskbit <= 0) { break; }
+		for (auto &word: mask) {
+			if (curmaskbit <= 0) { 
+				break;
+			}
 			if (curmaskbit >= 32) {
-				mask[i] = 0xffffffff;
+				word = 0xffffffff;
 			} else {
-				mask[i] = htonl(~(0xffffffff >> curmaskbit));
+				word = htonl(~(0xffffffff >> curmaskbit));
 			}
 			curmaskbit -= 32;
 		}
