@@ -53,7 +53,7 @@ static const int command_int_from_adtype_table[NUM_AD_TYPES] = {
 	QUERY_GENERIC_ADS,	//DATABASE_AD
 	QUERY_GENERIC_ADS,	//TT_AD
 	QUERY_GRID_ADS,		//GRID_AD
-	0,//XFER_SERVICE_AD  /* No longer used */
+	QUERY_GENERIC_ADS,	//PLACEMENTD_AD
 	0,//LEASE_MANAGER_AD /* No longer used */
 	QUERY_GENERIC_ADS,	//DEFRAG_AD
 	QUERY_ACCOUNTING_ADS,//ACCOUNTING_AD = 23
@@ -325,6 +325,13 @@ CondorQuery (AdTypes qType)
 		command = QUERY_GENERIC_ADS;
 		break;
 
+	  case PLACEMENTD_AD:
+		query.setNumStringCats (0);
+		query.setNumIntegerCats(0);
+		query.setNumFloatCats  (0);
+		command = QUERY_GENERIC_ADS;
+		break;
+
 	  case GENERIC_AD:
 		query.setNumStringCats (0);
 		query.setNumIntegerCats(0);
@@ -491,6 +498,7 @@ CondorQuery::setLocationLookup(const std::string &location, bool want_one_result
 	if (queryType == SCHEDD_AD)
 	{
 		attrs.push_back(ATTR_SCHEDD_IP_ADDR);
+		attrs.push_back(ATTR_CREDD_IP_ADDR);
 	}
 
 	setDesiredAttrs(attrs);
@@ -503,44 +511,54 @@ CondorQuery::setLocationLookup(const std::string &location, bool want_one_result
 QueryResult CondorQuery::
 processAds (bool (*callback)(void*, ClassAd *), void* pv, const char * poolName, CondorError* errstack /*= NULL*/)
 {
-	Sock*    sock; 
-	QueryResult result;
-	ClassAd  queryAd(extraAttrs);
-
 	if ( !poolName ) {
 		return Q_NO_COLLECTOR_HOST;
 	}
 
+	Daemon collector(DT_COLLECTOR, poolName, nullptr);
+	return processAds(callback, pv, collector, errstack);
+}
+
+QueryResult CondorQuery::
+processAds (bool (*callback)(void*, ClassAd *), void* pv, Daemon& collector, CondorError* errstack /*= NULL*/)
+{
+	Sock*    sock;
+	QueryResult result;
+	ClassAd  queryAd(extraAttrs);
+
 	// contact collector
-	Daemon my_collector( DT_COLLECTOR, poolName, NULL );
-	if( !my_collector.locate() ) {
+	if( !collector.locate() ) {
 			// We were passed a bogus poolName, abort gracefully
 		return Q_NO_COLLECTOR_HOST;
 	}
 
-
 	// make the query ad
 	result = getQueryAd (queryAd);
 	if (result != Q_OK) return result;
 
 	if (IsDebugLevel(D_HOSTNAME)) {
 		dprintf( D_HOSTNAME, "Querying collector %s (%s) with classad:\n", 
-				 my_collector.addr(), my_collector.fullHostname() );
+				 collector.addr(), collector.fullHostname() );
 		dPrintAd( D_HOSTNAME, queryAd );
 		dprintf( D_HOSTNAME, " --- End of Query ClassAd ---\n" );
 	}
 
+	bool orig_auth = collector.getForceAuthentication();
+	collector.setForceAuthentication(requireAuth);
 
 	int mytimeout = param_integer ("QUERY_TIMEOUT",60); 
-	if (!(sock = my_collector.startCommand(command, Stream::reli_sock, mytimeout, errstack)) ||
+	if (!(sock = collector.startCommand(command, Stream::reli_sock, mytimeout, errstack)) ||
 	    !putClassAd (sock, queryAd) || !sock->end_of_message()) {
 
+		collector.setForceAuthentication(orig_auth);
 		if (sock) {
 			delete sock;
 		}
 		return Q_COMMUNICATION_ERROR;
 	}
 
+	collector.setForceAuthentication(orig_auth);
+
 	// get result
 	sock->decode ();
 	int more = 1;
@@ -583,11 +601,34 @@ fetchAds (ClassAdList &adList, const char *poolName, CondorError* errstack)
 	return processAds(fetchAds_callback, &adList, poolName, errstack);
 }
 
+QueryResult CondorQuery::
+fetchAds (ClassAdList &adList, Daemon& collector, CondorError* errstack)
+{
+	return processAds(fetchAds_callback, &adList, collector, errstack);
+}
+
 void CondorQuery::
 setGenericQueryType(const char* genericType) {
 	if(genericQueryType) {
 		free(genericQueryType);
 	}
+
+	// canonicalize known generic adtype names
+	// collectors from 23.7 until HTCONDOR-2797 have
+	// case sensitive TargetType for generic lookups
+	static const char * const generic_adtypes[] = {
+		CREDD_ADTYPE,    //CREDD_AD
+		DATABASE_ADTYPE, //DATABASE_AD
+		DEFRAG_ADTYPE,   //DEFRAG_AD
+		TT_ADTYPE,       //TT_AD
+	};
+	for (auto gname : generic_adtypes) {
+		if (MATCH == strcasecmp(gname, genericType)) {
+			genericQueryType = strdup(gname);
+			return;
+		}
+	}
+
 	genericQueryType = strdup(genericType);
 }
 
@@ -698,9 +739,13 @@ getQueryAd (ClassAd &queryAd)
 		queryAd.Assign(ATTR_TARGET_TYPE, STORAGE_ADTYPE);
 		break;
 
-	  case CREDD_AD:
-		queryAd.Assign(ATTR_TARGET_TYPE, CREDD_ADTYPE);
-		break;
+	  case CREDD_AD: 
+		queryAd.Assign(ATTR_TARGET_TYPE, CREDD_ADTYPE); 
+		break; 
+
+	  case PLACEMENTD_AD: 
+		queryAd.Assign(ATTR_TARGET_TYPE, PLACEMENTD_ADTYPE); 
+		break; 
 
 	  case GENERIC_AD:
 		if ( genericQueryType ) {
@@ -796,7 +841,7 @@ CondorQuery::setDesiredAttrs(char const * const *attrs)
 {
 	std::string val;
 	::join_args(attrs,val);
-	setDesiredAttrs(val.c_str());
+	setDesiredAttrs(val);
 }
 
 void
@@ -808,14 +853,14 @@ CondorQuery::setDesiredAttrs(const classad::References &attrs)
 		if ( ! str.empty()) str += " ";
 		str += *it;
 	}
-	setDesiredAttrs(str.c_str());
+	setDesiredAttrs(str);
 }
 
 void
 CondorQuery::setDesiredAttrs(const std::vector<std::string> &attrs)
 {
 	std::string str = join(attrs, " ");
-	setDesiredAttrs(str.c_str());
+	setDesiredAttrs(str);
 }
 
 void
