@@ -1,6 +1,6 @@
 /***************************************************************
  *
- * Copyright (C) 1990-2018, Condor Team, Computer Sciences Department,
+ * Copyright (C) 1990-2025, Condor Team, Computer Sciences Department,
  * University of Wisconsin-Madison, WI.
  * 
  * Licensed under the Apache License, Version 2.0 (the "License"); you
@@ -32,6 +32,7 @@
 #include "tokener.h"
 #include "which.h"
 #include "directory.h"
+#include "dag_parser.h"
 
 namespace shallow = DagmanShallowOptions;
 namespace deep = DagmanDeepOptions;
@@ -51,26 +52,61 @@ ImportFilter(const std::string &var, const std::string &val) {
 	return Env::IsSafeEnvV2Value( val.c_str() );
 }
 
+void
+DagmanUtils::print_msg(const char* fmt, ...) const {
+	va_list args;
+	va_start(args, fmt);
+
+	switch (msgStream) {
+		case DEBUG_MSG_STREAM::STANDARD:
+			vfprintf(stdout, fmt, args);
+			break;
+		case DEBUG_MSG_STREAM::DEBUG_LOG:
+			_condor_dprintf_va(D_STATUS, 0, fmt, args);
+			break;
+		default:
+			EXCEPT("Unknown message stream %d specified.", (int)msgStream);
+	};
+
+	va_end(args);
+}
+
+void
+DagmanUtils::print_error(const char* fmt, ...) const {
+	va_list args;
+	va_start(args, fmt);
+
+	switch (msgStream) {
+		case DEBUG_MSG_STREAM::STANDARD:
+			vfprintf(stderr, fmt, args);
+			break;
+		case DEBUG_MSG_STREAM::DEBUG_LOG:
+			_condor_dprintf_va(D_ERROR, 0, fmt, args);
+			break;
+		default:
+			EXCEPT("Unknown message stream %d specified.", (int)msgStream);
+	};
+
+	va_end(args);
+}
+
 bool
 DagmanUtils::writeSubmitFile(DagmanOptions &options, str_list &dagFileAttrLines) const
 {
 	std::string submitFile = options[shallow::str::SubFile];
 	FILE *pSubFile = safe_fopen_wrapper_follow(submitFile.c_str(), "w");
 	if ( ! pSubFile) {
-		fprintf(stderr, "ERROR: unable to create submit file %s\n",
-		        submitFile.c_str() );
+		print_error("ERROR: Unable to create submit file %s\n", submitFile.c_str());
 		return false;
 	}
 
 	std::string executable;
 	if (options[shallow::b::RunValgrind]) {
-		std::string valgrindPath = which(valgrind_exe);
-		if (valgrindPath.empty()) {
-			fprintf(stderr, "ERROR: can't find %s in PATH, aborting.\n", valgrind_exe);
+		executable = which(valgrind_exe);
+		if (executable.empty()) {
+			print_error("ERROR: can't find %s in PATH, aborting.\n", valgrind_exe);
 			fclose(pSubFile);
 			return false;
-		} else {
-			executable = valgrindPath;
 		}
 	} else {
 		executable = options[deep::str::DagmanPath];
@@ -80,7 +116,7 @@ DagmanUtils::writeSubmitFile(DagmanOptions &options, str_list &dagFileAttrLines)
 	*	Update DAGMAN_MANAGER_JOB_APPEND_GETENV Macro documentation if base
 	*	getEnv value changes. -Cole Bollig 2023-02-21
 	*/
-	std::string getEnv = "CONDOR_CONFIG,_CONDOR_*,PATH,PYTHONPATH,PERL*,PEGASUS_*,TZ,HOME,USER,LANG,LC_ALL";
+	std::string getEnv = "CONDOR_CONFIG,_CONDOR_*,PATH,PYTHONPATH,PERL*,PEGASUS_*,TZ,HOME,USER,LANG,LC_ALL,ASAN_OPTIONS,LSAN_OPTIONS";
 	auto_free_ptr conf_getenvVars = param("DAGMAN_MANAGER_JOB_APPEND_GETENV");
 	if (conf_getenvVars && strcasecmp(conf_getenvVars.ptr(),"true") == MATCH) {
 		getEnv = "true";
@@ -96,6 +132,8 @@ DagmanUtils::writeSubmitFile(DagmanOptions &options, str_list &dagFileAttrLines)
 		if (conf_getenvVars) { getEnv += ","; getEnv += conf_getenvVars.ptr(); }
 	}
 
+	bool disable_port = param_boolean("DAGMAN_DISABLE_PORT", false);
+
 	fprintf(pSubFile, "# Filename: %s\n", submitFile.c_str());
 
 	fprintf(pSubFile, "# Generated by condor_submit_dag ");
@@ -124,6 +162,9 @@ DagmanUtils::writeSubmitFile(DagmanOptions &options, str_list &dagFileAttrLines)
 	fprintf(pSubFile, "My.%s = \"%s =?= $(cluster)\"\n",
 	        ATTR_OTHER_JOB_REMOVE_REQUIREMENTS, ATTR_DAGMAN_JOB_ID );
 
+	// Set ClassAd Attribute to inform Schedd that DAGMan wants a port set up
+	if ( ! disable_port) { fprintf(pSubFile, "My." ATTR_IS_DAEMON_CORE " = True\n"); }
+
 		// ensure DAGMan is automatically requeued by the schedd if it
 		// exits abnormally or is killed (e.g., during a reboot)
 	const char *defaultRemoveExpr = "(ExitSignal =?= 11 || (ExitCode =!= UNDEFINED && ExitCode >=0 && ExitCode <= 2))";
@@ -148,16 +189,23 @@ DagmanUtils::writeSubmitFile(DagmanOptions &options, str_list &dagFileAttrLines)
 	ArgList args;
 
 	if (options[shallow::b::RunValgrind]) {
+#ifdef DARWIN
+		args.AppendArg("--atExit");
+		args.AppendArg("--");
+#else
 		args.AppendArg("--tool=memcheck");
 		args.AppendArg("--leak-check=yes");
 		args.AppendArg("--show-reachable=yes");
+#endif
 		args.AppendArg(options[deep::str::DagmanPath].c_str());
 	}
 
 	//======DaemonCore Commands======
 	// -p 0 causes DAGMan to run w/o a command socket (see gittrac #4987).
-	args.AppendArg("-p");
-	args.AppendArg("0");
+	if (disable_port) { // Disable DAGMan port
+		args.AppendArg("-p");
+		args.AppendArg("0");
+	}
 	args.AppendArg("-f");
 	args.AppendArg("-l");
 	args.AppendArg(".");
@@ -175,22 +223,22 @@ DagmanUtils::writeSubmitFile(DagmanOptions &options, str_list &dagFileAttrLines)
 		args.AppendArg(dagFile);
 	}
 
-	if (options[shallow::i::MaxIdle] != 0) {
+	if (options[shallow::i::MaxIdle] >= 0) {
 		args.AppendArg("-MaxIdle");
 		args.AppendArg(std::to_string(options[shallow::i::MaxIdle]));
 	}
 
-	if (options[shallow::i::MaxJobs] != 0) {
+	if (options[shallow::i::MaxJobs] >= 0) {
 		args.AppendArg("-MaxJobs");
 		args.AppendArg(std::to_string(options[shallow::i::MaxJobs]));
 	}
 
-	if (options[shallow::i::MaxPre] != 0) {
+	if (options[shallow::i::MaxPre] >= 0) {
 		args.AppendArg("-MaxPre");
 		args.AppendArg(std::to_string(options[shallow::i::MaxPre]));
 	}
 
-	if (options[shallow::i::MaxPost] != 0) {
+	if (options[shallow::i::MaxPost] >= 0) {
 		args.AppendArg("-MaxPost");
 		args.AppendArg(std::to_string(options[shallow::i::MaxPost]));
 	}
@@ -224,11 +272,16 @@ DagmanUtils::writeSubmitFile(DagmanOptions &options, str_list &dagFileAttrLines)
 		args.AppendArg(options[shallow::str::SaveFile]);
 	}
 
+	if ( ! options[shallow::str::RescueFile].empty()) {
+		args.AppendArg("-RescueFile");
+		args.AppendArg(options[shallow::str::RescueFile]);
+	}
+
 	options.addDeepArgs(args);
 
 	std::string arg_str, args_error;
 	if ( ! args.GetArgsStringV1WackedOrV2Quoted(arg_str, args_error)) {
-		fprintf(stderr, "Failed to insert arguments: %s", args_error.c_str());
+		print_error("ERROR: Failed to insert arguments: %s", args_error.c_str());
 		fclose(pSubFile);
 		return false;
 	}
@@ -248,13 +301,17 @@ DagmanUtils::writeSubmitFile(DagmanOptions &options, str_list &dagFileAttrLines)
 		std::string err_msg;
 		env.MergeFromV1RawOrV2Quoted(kv_pairs.c_str(), err_msg);
 		if ( ! err_msg.empty()) {
-			fprintf(stderr,"Error: Failed to add %s to DAGMan manager jobs environment because %s\n",
-			        kv_pairs.c_str(), err_msg.c_str());
+			print_error("ERROR: Failed to add %s to DAGMan manager jobs environment because %s\n",
+			            kv_pairs.c_str(), err_msg.c_str());
 			fclose(pSubFile);
 			return false;
 		}
 	}
 
+#ifdef DARWIN
+	if (options[shallow::b::RunValgrind]) { env.SetEnv("MallocStackLogging=1"); }
+#endif
+
 	env.SetEnv("_CONDOR_DAGMAN_LOG", options[shallow::str::DebugLog].c_str());
 	env.SetEnv("_CONDOR_MAX_DAGMAN_LOG=0");
 	if ( ! options[shallow::str::ScheddDaemonAdFile].empty()) {
@@ -267,8 +324,8 @@ DagmanUtils::writeSubmitFile(DagmanOptions &options, str_list &dagFileAttrLines)
 	}
 	if ( ! options[shallow::str::ConfigFile].empty()) {
 		if (access(options[shallow::str::ConfigFile].c_str(), F_OK) != 0) {
-			fprintf(stderr, "ERROR: unable to read config file %s (error %d, %s)\n",
-			        options[shallow::str::ConfigFile].c_str(), errno, strerror(errno));
+			print_error("ERROR: Unable to read config file %s (%d): %s\n",
+			            options[shallow::str::ConfigFile].c_str(), errno, strerror(errno));
 			fclose(pSubFile);
 			return false;
 		}
@@ -289,8 +346,8 @@ DagmanUtils::writeSubmitFile(DagmanOptions &options, str_list &dagFileAttrLines)
 	if ( ! options[shallow::str::AppendFile].empty()) {
 		FILE *aFile = safe_fopen_wrapper_follow(options[shallow::str::AppendFile].c_str(), "r");
 		if ( ! aFile) {
-			fprintf(stderr, "ERROR: unable to read submit append file (%s)\n",
-			        options[shallow::str::AppendFile].c_str());
+			print_error("ERROR: Unable to read submit append file %s\n",
+			            options[shallow::str::AppendFile].c_str());
 			fclose(pSubFile);
 			return false;
 		}
@@ -340,8 +397,8 @@ DagmanUtils::runSubmitDag(const DagmanOptions &options, const char *dagFile,
 	std::string errMsg;
 	if (directory) {
 		if ( ! tmpDir.Cd2TmpDir(directory, errMsg)) {
-			fprintf(stderr, "Error (%s) changing to node directory\n",
-			        errMsg.c_str() );
+			print_error("ERROR: Failed to change to node directory: %s\n",
+			            errMsg.c_str());
 			return 1;
 		}
 	}
@@ -372,21 +429,20 @@ DagmanUtils::runSubmitDag(const DagmanOptions &options, const char *dagFile,
 
 	std::string cmdLine;
 	args.GetArgsStringForDisplay(cmdLine);
-	dprintf(D_ALWAYS, "Recursive submit command: <%s>\n",
-	        cmdLine.c_str() );
+	print_msg("Recursive submit command: <%s>\n", cmdLine.c_str());
 
 		// Now actually run the command.
 	int retval = my_system(args);
 	if (retval != 0) {
-		dprintf(D_ALWAYS, "ERROR: condor_submit_dag -no_submit failed on DAG file %s.\n",
-		        dagFile);
+		print_error("ERROR: condor_submit_dag -no_submit failed on DAG file %s.\n",
+		            dagFile);
 		result = 1;
 	}
 
 		// Change back to the directory we started from.
 	if ( ! tmpDir.Cd2MainDir(errMsg)) {
-		dprintf(D_ALWAYS, "Error (%s) changing back to original directory\n",
-		        errMsg.c_str() );
+		print_error("ERROR: Failed to change back to original directory: %s\n",
+		            errMsg.c_str());
 	}
 
 	return result;
@@ -399,7 +455,7 @@ DagmanUtils::runSubmitDag(const DagmanOptions &options, const char *dagFile,
 	@return true on success and false on failure
 */
 bool
-DagmanUtils::setUpOptions(DagmanOptions &options, str_list &dagFileAttrLines)
+DagmanUtils::setUpOptions(DagmanOptions &options, str_list &dagFileAttrLines, std::string* err)
 {
 	std::string primaryDag = options.primaryDag();
 	options[shallow::str::LibOut] = primaryDag + ".lib.out";
@@ -415,45 +471,24 @@ DagmanUtils::setUpOptions(DagmanOptions &options, str_list &dagFileAttrLines)
 	options[shallow::str::SchedLog] = primaryDag + ".dagman.log";
 	options[shallow::str::SubFile] = primaryDag + DAG_SUBMIT_FILE_SUFFIX;
 
-	std::string rescueDagBase;
-
-		// If we're running each DAG in its own directory, write any rescue
-		// DAG to the current directory, to avoid confusion (since the
-		// rescue DAG must be run from the current directory).
-	if (options[deep::b::UseDagDir]) {
-		if ( ! condor_getcwd(rescueDagBase)) {
-			fprintf(stderr, "ERROR: unable to get cwd: %d, %s\n",
-			        errno, strerror(errno));
-			return false;
-		}
-		rescueDagBase += DIR_DELIM_STRING;
-		rescueDagBase += condor_basename(primaryDag.c_str());
-	} else {
-		rescueDagBase = primaryDag;
-	}
-
-		// If we're running multiple DAGs, put "_multi" in the rescue
-		// DAG name to indicate that the rescue DAG is for *all* of
-		// the DAGs we're running.
-	if (options.isMultiDag()) {
-		rescueDagBase += "_multi";
-	}
-	options[shallow::str::RescueFile] = rescueDagBase + ".rescue";
-
 	options[shallow::str::LockFile] = primaryDag + ".lock";
 
 	if (options[deep::str::DagmanPath].empty()) {
 		options[deep::str::DagmanPath] = which(dagman_exe);
 	}
 
+	std::string msg;
+
 	if (options[deep::str::DagmanPath].empty()) {
-		fprintf(stderr, "ERROR: can't find %s in PATH, aborting.\n",
-		        dagman_exe );
+		formatstr(msg, "Failed to locate %s executable in PATH", dagman_exe);
+		print_error("ERROR: %s\n", msg.c_str());
+		if (err) { *err = msg; }
 		return false;
 	}
-	std::string msg;
+
 	if ( ! processDagCommands(options, dagFileAttrLines, msg)) {
-		fprintf(stderr, "ERROR: %s\n", msg.c_str());
+		print_error("ERROR: %s\n", msg.c_str());
+		if (err) { *err = msg; }
 		return false;
 	}
 
@@ -473,11 +508,12 @@ bool
 DagmanUtils::processDagCommands(DagmanOptions &options, str_list &attrLines, std::string &errMsg)
 {
 	bool result = true;
+	bool use_old_parser = param_boolean("DAGMAN_USE_OLD_FILE_PARSER", false);
 	// Note: destructor will change back to original directory.
 	TmpDir dagDir;
 	std::set<std::string> configFiles;
 
-	for (auto & dagFile : options.dagFiles()) {
+	for (const auto & dagFile : options.dagFiles()) {
 		std::string newDagFile;
 		// Switch to DAG Dir if needed
 		if (options[deep::b::UseDagDir]) {
@@ -488,92 +524,151 @@ DagmanUtils::processDagCommands(DagmanOptions &options, str_list &attrLines, std
 			}
 			newDagFile = condor_basename(dagFile.c_str());
 		} else {
-			newDagFile = dagFile.c_str();
+			newDagFile = dagFile;
 		}
 
-		// Note: destructor will close file.
-		MultiLogFiles::FileReader reader;
-		errMsg = reader.Open( newDagFile );
-		if ( ! errMsg.empty()) {
-			return false;
-		}
+		if (use_old_parser) {
+			// Note: destructor will close file.
+			MultiLogFiles::FileReader reader;
+			errMsg = reader.Open( newDagFile );
+			if ( ! errMsg.empty()) {
+				return false;
+			}
 
-		//Read DAG file
-		std::string logicalLine;
-		while (reader.NextLogicalLine(logicalLine)) {
-			if ( ! logicalLine.empty()) {
-				StringTokenIterator tokens(logicalLine, " \t\r");
-				const char* cmd = tokens.first();
-				if ( ! cmd) { continue; }
+			//Read DAG file
+			std::string logicalLine;
+			while (reader.NextLogicalLine(logicalLine)) {
+				if ( ! logicalLine.empty()) {
+					StringTokenIterator tokens(logicalLine, " \t\r");
+					const char* cmd = tokens.first();
+					if ( ! cmd) { continue; }
 
-				// Parse CONFIG command
-				if (strcasecmp(cmd, "CONFIG") == MATCH) {
-					const char* newFile = tokens.remain();
-					while (newFile && isspace(*newFile) && *newFile != '\0') { newFile++; }
-					if ( ! newFile || *newFile == '\0') {
-						AppendError(errMsg, "Improperly-formatted file: value missing after keyword CONFIG");
-						result = false;
-					} else {
-						std::string conf(newFile), tmpErr;
-						if (MakePathAbsolute(conf, tmpErr)) {
-							configFiles.insert(conf);
-						} else {
-							AppendError(errMsg, tmpErr);
-							result = false;
-						}
-					}
-
-				// Parse SET_JOB_ATTR command
-				} else if (strcasecmp(cmd, "SET_JOB_ATTR") == MATCH) {
-					const char* attr = tokens.remain();
-					while (attr && isspace(*attr) && *attr != '\0') { attr++; }
-					if (!attr || *attr == '\0') {
-						AppendError(errMsg, "Improperly-formatted file: value missing after keyword SET_JOB_ATTR");
-						result = false;
-					} else {
-						attrLines.emplace_back(attr);
-					}
-
-				// Parse ENV command
-				} else if (strcasecmp(cmd, "ENV") == MATCH) {
-					const char* type = tokens.next();
-					// Parse GET option
-					if (strcasecmp(type, "GET") == MATCH) {
-						const char* remain = tokens.remain();
-						while (remain && isspace(*remain) && *remain != '\0') { remain++; }
-						if (!remain || *remain == '\0') {
-							AppendError(errMsg, "Improperly-formatted file: environment variables missing after ENV GET");
+					// Parse CONFIG command
+					if (strcasecmp(cmd, "CONFIG") == MATCH) {
+						const char* newFile = tokens.remain();
+						while (newFile && isspace(*newFile) && *newFile != '\0') { newFile++; }
+						if ( ! newFile || *newFile == '\0') {
+							AppendError(errMsg, "Improperly-formatted file: value missing after keyword CONFIG");
 							result = false;
 						} else {
-							StringTokenIterator vars(remain);
-							std::string delimVars;
-							for (const auto& var : vars) {
-								if ( ! delimVars.empty()) { delimVars += ","; }
-								delimVars += var;
+							std::string conf(newFile), tmpErr;
+							if (MakePathAbsolute(conf, tmpErr)) {
+								configFiles.insert(conf);
+							} else {
+								AppendError(errMsg, tmpErr);
+								result = false;
 							}
-							options.extend("GetFromEnv", delimVars);
 						}
-					// Parse SET option
-					} else if (strcasecmp(type, "SET") == MATCH) {
-						const char* info = tokens.remain();
-						while (info && isspace(*info) && *info != '\0') { info++; }
-						if (!info || *info == '\0') {
-							AppendError(errMsg, "Improperly-formatted file: environment variables missing after ENV SET");
+
+					// Parse SET_JOB_ATTR command
+					} else if (strcasecmp(cmd, "SET_JOB_ATTR") == MATCH) {
+						const char* attr = tokens.remain();
+						while (attr && isspace(*attr) && *attr != '\0') { attr++; }
+						if (!attr || *attr == '\0') {
+							AppendError(errMsg, "Improperly-formatted file: value missing after keyword SET_JOB_ATTR");
 							result = false;
 						} else {
-							std::string kv_pairs = options.processOptionArg("AddToEnv", std::string(info));
-							options.extend("AddToEnv", kv_pairs);
+							attrLines.emplace_back(attr);
+						}
+
+					// Parse ENV command
+					} else if (strcasecmp(cmd, "ENV") == MATCH) {
+						const char* type = tokens.next();
+						// Parse GET option
+						if (strcasecmp(type, "GET") == MATCH) {
+							const char* remain = tokens.remain();
+							while (remain && isspace(*remain) && *remain != '\0') { remain++; }
+							if (!remain || *remain == '\0') {
+								AppendError(errMsg, "Improperly-formatted file: environment variables missing after ENV GET");
+								result = false;
+							} else {
+								StringTokenIterator vars(remain);
+								std::string delimVars;
+								for (const auto& var : vars) {
+									if ( ! delimVars.empty()) { delimVars += ","; }
+									delimVars += var;
+								}
+								options.extend("GetFromEnv", delimVars);
+							}
+						// Parse SET option
+						} else if (strcasecmp(type, "SET") == MATCH) {
+							const char* info = tokens.remain();
+							while (info && isspace(*info) && *info != '\0') { info++; }
+							if (!info || *info == '\0') {
+								AppendError(errMsg, "Improperly-formatted file: environment variables missing after ENV SET");
+								result = false;
+							} else {
+								std::string kv_pairs = options.processOptionArg("AddToEnv", std::string(info));
+								options.extend("AddToEnv", kv_pairs);
+							}
+						// Else error
+						} else {
+							AppendError(errMsg, "Improperly-formatted file: sub-command (SET or GET) missing after keyword ENV");
+							result = false;
 						}
-					// Else error
-					} else {
-						AppendError(errMsg, "Improperly-formatted file: sub-command (SET or GET) missing after keyword ENV");
-						result = false;
 					}
 				}
 			}
-		}
 
-		reader.Close();
+			reader.Close();
+		} else {
+			static const std::set<DAG::CMD> filter_commands {
+				DAG::CMD::CONFIG,
+				DAG::CMD::SET_JOB_ATTR,
+				DAG::CMD::ENV,
+			};
+
+			DagParser parser(newDagFile);
+			if (parser.failed()) {
+				errMsg = parser.error();
+				return false;
+			}
+
+			parser.SearchFor(filter_commands);
+
+			for (const auto cmd : parser) {
+				if ( ! cmd) { continue; }
+
+				// TODO: Handle reject here so user knows at submit time not once DAGMan notices
+				switch (cmd->GetCommand()) {
+					case DAG::CMD::CONFIG:
+						{
+							std::string conf = DAG::DERIVE_CMD<FileCommand>(cmd)->GetFile();
+							std::string error;
+							if (MakePathAbsolute(conf, error)) {
+								configFiles.insert(conf);
+							} else {
+								AppendError(errMsg, error);
+								result = false;
+							}
+						}
+						break;
+					case DAG::CMD::SET_JOB_ATTR:
+						attrLines.emplace_back(DAG::DERIVE_CMD<SetAttrCommand>(cmd)->GetAttrLine());
+						break;
+					case DAG::CMD::ENV:
+						{
+							const EnvCommand* env = DAG::DERIVE_CMD<EnvCommand>(cmd);
+							if (env->IsSet()) {
+								std::string kv_pairs = options.processOptionArg("AddToEnv", env->GetEnvVariables());
+								options.extend("AddToEnv", kv_pairs);
+							} else {
+								options.extend("GetFromEnv", env->GetEnvVariables());
+							}
+						}
+						break;
+					default:
+						print_msg("WARNING: DAGMan Utils does not know how to process %s command...\n",
+						        DAG::GET_KEYWORD_STRING(cmd->GetCommand()));
+						break;
+				}
+			}
+
+			if (parser.failed()) {
+				AppendError(errMsg, parser.error());
+				result = false;
+			}
+		}
 
 		// Switch back to original directory
 		std::string tmpErrMsg;
@@ -654,8 +749,8 @@ std::tuple<std::string, bool> DagmanUtils::ResolveSaveFile(const std::string& pr
 			Directory dir(saveDir.c_str());
 			if ( ! dir.IsDirectory()) {
 				if (mkdir(saveDir.c_str(),0755) < 0 && errno != EEXIST) {
-					dprintf(D_ALWAYS, "Error: Failed to create save file dir (%s): Errno %d (%s)\n",
-					        saveDir.c_str(), errno, strerror(errno));
+					print_error("ERROR: Failed to create save file dir %s (%d): (%s)\n",
+					            saveDir.c_str(), errno, strerror(errno));
 					return {"", false};
 				}
 			}
@@ -688,21 +783,26 @@ DagmanUtils::FindLastRescueDagNum(const std::string &primaryDagFile, bool multiD
 					// that for now because the fact that this code
 					// is used in both condor_dagman and condor_submit_dag
 					// makes that harder to implement. wenger 2011-01-28
-				dprintf(D_ALWAYS, "Warning: found rescue DAG number %d, but not rescue DAG number %d\n",
-				        test, test - 1);
+				print_msg("Warning: Found rescue DAG number %d, but not rescue DAG number %d\n",
+				          test, test - 1);
 			}
 			lastRescue = test;
 		}
 	}
 	
 	if (lastRescue >= maxRescueDagNum) {
-		dprintf(D_ALWAYS, "Warning: FindLastRescueDagNum() hit maximum rescue DAG number: %d\n",
-		        maxRescueDagNum);
+		print_msg("Warning: Hit maximum rescue DAG number: %d\n", maxRescueDagNum);
 	}
 
 	return lastRescue;
 }
 
+static std::string RescueDagBase(const std::string& base, bool multi) {
+	std::string ret(base);
+	if (multi) { ret += "_multi"; }
+	return ret + ".rescue";
+}
+
 /** Creates a rescue DAG name, given a primary DAG name and rescue
 	DAG number
 	@param primaryDagFile The primary DAG file name
@@ -715,11 +815,7 @@ DagmanUtils::RescueDagName(const std::string& primaryDagFile, bool multiDags, in
 {
 	ASSERT( rescueDagNum >= 1 );
 
-	std::string fileName(primaryDagFile);
-	if ( multiDags ) {
-		fileName += "_multi";
-	}
-	fileName += ".rescue";
+	std::string fileName = RescueDagBase(primaryDagFile, multiDags);
 	formatstr_cat(fileName, "%.3d", rescueDagNum);
 
 	return fileName;
@@ -758,6 +854,15 @@ DagmanUtils::RenameRescueDagsAfter(const std::string& primaryDagFile, bool multi
 	}
 }
 
+int DagmanUtils::ExtractRescueNum(const std::string& file, const std::string& primaryDagFile, bool multiDags) {
+	std::string rescue = RescueDagBase(primaryDagFile, multiDags);
+	if (starts_with(file, rescue) && file.length() > rescue.length()) {
+		return std::atoi(file.substr(rescue.length()).c_str());
+	}
+
+	return 0;
+}
+
 /** Attempts to unlink the given file, and prints an appropriate error
 	message if this fails (but doesn't return an error, so only call
 	this if a failure of the unlink is okay).
@@ -766,14 +871,13 @@ DagmanUtils::RenameRescueDagsAfter(const std::string& primaryDagFile, bool multi
 void
 DagmanUtils::tolerant_unlink(const std::string &pathname)
 {
+	if ( ! fileExists(pathname)) { return; }
+
 	if (unlink(pathname.c_str()) != 0) {
-		if (errno == ENOENT) {
-			dprintf(D_SYSCALLS, "Warning: failure (%d (%s)) attempting to unlink file %s\n",
-			        errno, strerror(errno), pathname.c_str());
-		} else {
-			dprintf(D_ALWAYS, "Error (%d (%s)) attempting to unlink file %s\n",
-			        errno, strerror(errno), pathname.c_str());
-		}
+		int status = errno;
+		const char* err = strerror(errno);
+		print_error("ERROR: Failed to unlink file %s (%d): %s\n",
+		            pathname.c_str(), status, err);
 	}
 }
 
@@ -782,8 +886,7 @@ bool
 DagmanUtils::fileExists(const std::string &strFile)
 {
 	int fd = safe_open_wrapper_follow(strFile.c_str(), O_RDONLY);
-	if (fd == -1)
-		return false;
+	if (fd == -1) { return false; }
 	close(fd);
 	return true;
 }
@@ -797,8 +900,8 @@ DagmanUtils::ensureOutputFilesExist(const DagmanOptions &options)
 	if (options[deep::i::DoRescueFrom] > 0) {
 		std::string rescueDagName = RescueDagName(options.primaryDag(), options.isMultiDag(), options[deep::i::DoRescueFrom]);
 		if ( ! fileExists(rescueDagName)) {
-			fprintf(stderr, "-dorescuefrom %d specified, but rescue DAG file %s does not exist!\n",
-			                options[deep::i::DoRescueFrom], rescueDagName.c_str());
+			print_error("ERROR: -dorescuefrom %d specified, but rescue DAG file %s does not exist!\n",
+			            options[deep::i::DoRescueFrom], rescueDagName.c_str());
 			return false;
 		}
 	}
@@ -821,7 +924,7 @@ DagmanUtils::ensureOutputFilesExist(const DagmanOptions &options)
 	if (options[deep::i::AutoRescue]) {
 		int rescueDagNum = FindLastRescueDagNum(options.primaryDag(), options.isMultiDag(), maxRescueDagNum);
 		if (rescueDagNum > 0) {
-			printf("Running rescue DAG %d\n", rescueDagNum);
+			print_msg("Running rescue DAG %d\n", rescueDagNum);
 			autoRunningRescue = true;
 		}
 	}
@@ -829,54 +932,64 @@ DagmanUtils::ensureOutputFilesExist(const DagmanOptions &options)
 	bool bHadError = false;
 		// If not running a rescue DAG, check for existing files
 		// generated by condor_submit_dag...
-	if ( ! autoRunningRescue && options[deep::i::DoRescueFrom] < 1 &&
+	if ( ! autoRunningRescue && options[deep::i::DoRescueFrom] < 1 && options[shallow::str::RescueFile].empty() &&
 		 ! options[deep::b::UpdateSubmit] && options[shallow::str::SaveFile].empty()) {
 			if (fileExists(options[shallow::str::SubFile])) {
-				fprintf(stderr, "ERROR: \"%s\" already exists.\n",
-				        options[shallow::str::SubFile].c_str());
+				print_error("ERROR: \"%s\" already exists.\n",
+				            options[shallow::str::SubFile].c_str());
 				bHadError = true;
 			}
 			if (fileExists(options[shallow::str::LibOut])) {
-				fprintf(stderr, "ERROR: \"%s\" already exists.\n",
-				        options[shallow::str::LibOut].c_str());
+				print_error("ERROR: \"%s\" already exists.\n",
+				            options[shallow::str::LibOut].c_str());
 				bHadError = true;
 			}
 			if (fileExists(options[shallow::str::LibErr])) {
-				fprintf(stderr, "ERROR: \"%s\" already exists.\n",
-				        options[shallow::str::LibErr].c_str());
+				print_error("ERROR: \"%s\" already exists.\n",
+				            options[shallow::str::LibErr].c_str());
 				bHadError = true;
 			}
 			if (fileExists(options[shallow::str::SchedLog])) {
-				fprintf(stderr, "ERROR: \"%s\" already exists.\n",
-				        options[shallow::str::SchedLog].c_str());
+				print_error("ERROR: \"%s\" already exists.\n",
+				            options[shallow::str::SchedLog].c_str());
 				bHadError = true;
 			}
 	}
 
-		// This is checking for the existance of an "old-style" rescue
-		// DAG file.
-	if ( ! options[deep::i::AutoRescue] && options[deep::i::DoRescueFrom] < 1 &&
-		 fileExists(options[shallow::str::RescueFile])) {
-			fprintf(stderr, "ERROR: \"%s\" already exists.\n",
-			        options[shallow::str::RescueFile].c_str());
-			fprintf(stderr, "\tYou may want to resubmit your DAG using that file, instead of \"%s\"\n",
-			        options.primaryDag().c_str());
-			fprintf(stderr, "\tLook at the HTCondor manual for details about DAG rescue files.\n");
-			fprintf(stderr, "\tPlease investigate and either remove \"%s\",\n",
-			        options[shallow::str::RescueFile].c_str());
-			fprintf(stderr, "\tor use it as the input to condor_submit_dag.\n");
+	std::string oldRescueDag = RescueDagBase(options.primaryDag(), options.isMultiDag());
+	if (options[deep::b::UseDagDir]) {
+		// Use absolute path for DAGs running in their own directory to avoid confusion
+		std::string path;
+		if ( ! condor_getcwd(path)) {
+			print_error("ERROR: Unable to get cwd (%d): %s\n", errno, strerror(errno));
+			return false;
+		}
+
+		std::string fullPath;
+		dircat(path.c_str(), oldRescueDag.c_str(), fullPath);
+		oldRescueDag = fullPath;
+	}
+
+	// This is checking for the existance of an "old-style" rescue DAG file.
+	if ( ! options[deep::i::AutoRescue] && options[deep::i::DoRescueFrom] < 1 && fileExists(oldRescueDag)) {
+			print_error("ERROR: \"%s\" already exists.\n", oldRescueDag.c_str());
+			print_error("\tYou may want to resubmit your DAG using that file, instead of \"%s\"\n",
+			            options.primaryDag().c_str());
+			print_error("\tLook at the HTCondor manual for details about DAG rescue files.\n");
+			print_error("\tPlease investigate and either remove \"%s\",\n",
+			            oldRescueDag.c_str());
+			print_error("\tor use it as the input to condor_submit_dag.\n");
 			bHadError = true;
 	}
 
 	if (bHadError) {
-		fprintf(stderr, "\nSome file(s) needed by %s already exist. Either:\n- Rename them\n",
-		        dagman_exe );
+		print_error("\nSome file(s) needed by %s already exist. Either:\n- Rename them\n", dagman_exe);
 		if( ! usingPythonBindings) {
-			fprintf(stderr, "- Use the \"-f\" option to force them to be overwritten\n");
-			fprintf(stderr, "\tor\n- Use the \"-update_submit\" option to update the submit file and continue.\n" );
+			print_error("- Use the \"-f\" option to force them to be overwritten\n");
+			print_error("\tor\n- Use the \"-update_submit\" option to update the submit file and continue.\n" );
 		}
 		else {
-			fprintf(stderr, "\tor\n- Set the { \"force\" : True } option to force them to be overwritten.\n");
+			print_error("\tor\n- Set the { \"force\" : True } option to force them to be overwritten.\n");
 		}
 		return false;
 	}
@@ -889,13 +1002,13 @@ int
 DagmanUtils::popen (ArgList &args) {
 	std::string cmd; // for debug output
 	args.GetArgsStringForDisplay(cmd);
-	dprintf(D_ALWAYS, "Running: %s\n", cmd.c_str());
+	print_msg("Running: %s\n", cmd.c_str());
 
 	FILE *fp = my_popen( args, "r", MY_POPEN_OPT_WANT_STDERR );
 
 	int r = 0;
 	if (fp == nullptr || (r = my_pclose(fp) & 0xff) != 0) {
-		dprintf(D_ERROR, "Warning: failure: %s\n", cmd.c_str());
+		print_error("ERROR: Failed to execute %s\n", cmd.c_str());
 		if(fp != nullptr) {
 			dprintf(D_ALWAYS, "\t(my_pclose() returned %d (errno %d, %s))\n",
 			        r, errno, strerror(errno));
@@ -915,7 +1028,7 @@ DagmanUtils::create_lock_file(const char *lockFileName, bool abortDuplicates) {
 
 	FILE *fp = safe_fopen_wrapper_follow(lockFileName, "w");
 	if (fp == nullptr) {
-		dprintf(D_ALWAYS, "ERROR: could not open lock file %s for writing.\n", lockFileName);
+		print_error("ERROR: Failed to open lock file %s for writing.\n", lockFileName);
 		result = -1;
 	}
 
@@ -925,7 +1038,7 @@ DagmanUtils::create_lock_file(const char *lockFileName, bool abortDuplicates) {
 		int status;
 		int precision_range = 1;
 		if (ProcAPI::createProcessId(daemonCore->getpid(), procId, status, &precision_range) != PROCAPI_SUCCESS) {
-			dprintf(D_ALWAYS, "ERROR: ProcAPI::createProcessId() failed; %d\n", status);
+			print_error("ERROR: Failed to create process ID (%d)\n", status);
 			result = -1;
 		}
 	}
@@ -933,7 +1046,7 @@ DagmanUtils::create_lock_file(const char *lockFileName, bool abortDuplicates) {
 	// Write out the ProcessId object.
 	if (result == 0 && abortDuplicates) {
 		if (procId->write(fp) != ProcessId::SUCCESS) {
-			dprintf(D_ALWAYS, "ERROR: ProcessId::write() failed\n");
+			print_error("ERROR: Failed to write process ID information to %s\n", lockFileName);
 			result = -1;
 		}
 	}
@@ -942,13 +1055,12 @@ DagmanUtils::create_lock_file(const char *lockFileName, bool abortDuplicates) {
 	if (result == 0 && abortDuplicates) {
 		int status;
 		if (ProcAPI::confirmProcessId(*procId, status) != PROCAPI_SUCCESS) {
-			dprintf(D_ERROR, "Warning: ProcAPI::confirmProcessId() failed; %d\n",
-			        status);
+			print_error("Warning: Failed to confirm process ID (%d)\n", status);
 		} else if ( ! procId->isConfirmed()) {
-			dprintf(D_ERROR, "Warning: ProcessId not confirmed unique\n");
+			print_msg("Warning: Ignoring error that ProcessId not confirmed unique\n");
 		} else if (procId->writeConfirmationOnly(fp) != ProcessId::SUCCESS) {
-			dprintf(D_ERROR, "ERROR: ProcessId::writeConfirmationOnly() failed\n");
-			        result = -1;
+			print_error("ERROR: Failed to confirm writing of process ID information\n");
+			result = -1;
 		}
 	}
 
@@ -956,7 +1068,7 @@ DagmanUtils::create_lock_file(const char *lockFileName, bool abortDuplicates) {
 
 	if (fp != nullptr) {
 		if (fclose(fp) != 0) {
-			dprintf(D_ALWAYS, "ERROR: closing lock file failed with errno %d (%s)\n",
+			print_error("ERROR: closing lock file failed with (%d): %s\n",
 			        errno, strerror(errno));
 		}
 	}
@@ -971,8 +1083,8 @@ DagmanUtils::check_lock_file(const char *lockFileName) {
 
 	FILE *fp = safe_fopen_wrapper_follow(lockFileName, "r");
 	if (fp == nullptr) {
-		dprintf(D_ALWAYS, "ERROR: could not open lock file %s for reading.\n",
-		        lockFileName);
+		print_error("ERROR: Failed to open lock file %s for reading.\n",
+		            lockFileName);
 		result = -1;
 	}
 
@@ -981,8 +1093,8 @@ DagmanUtils::check_lock_file(const char *lockFileName) {
 		int status;
 		procId = new ProcessId(fp, status);
 		if (status != ProcessId::SUCCESS) {
-			dprintf(D_ALWAYS, "ERROR: unable to create ProcessId object from lock file %s\n",
-			        lockFileName);
+			print_error("ERROR: Failed to create process Id object from lock file %s\n",
+			            lockFileName);
 			result = -1;
 		}
 	}
@@ -990,22 +1102,22 @@ DagmanUtils::check_lock_file(const char *lockFileName) {
 	if (result != -1) {
 		int status;
 		if (ProcAPI::isAlive(*procId, status) != PROCAPI_SUCCESS) {
-			dprintf(D_ALWAYS, "ERROR: failed to determine whether DAGMan that wrote lock file is alive\n");
+			print_error("ERROR: Failed to determine whether DAGMan that wrote lock file is alive.\n");
 			result = -1;
 		} else if (status == PROCAPI_ALIVE) {
-			dprintf(D_ALWAYS, "Duplicate DAGMan PID %d is alive; this DAGMan should abort.\n",
-			        procId->getPid());
+			print_error("ERROR: Duplicate DAGMan PID %d is alive; this DAGMan should abort.\n",
+			            procId->getPid());
 			result = 1;
 
 		} else if (status == PROCAPI_DEAD) {
-			dprintf(D_ALWAYS, "Duplicate DAGMan PID %d is no longer alive; this DAGMan should continue.\n",
-			        procId->getPid());
+			print_msg("Duplicate DAGMan PID %d is no longer alive; this DAGMan should continue.\n",
+			          procId->getPid());
 			result = 0;
 
 		} else if (status == PROCAPI_UNCERTAIN) {
-			dprintf(D_ALWAYS, "Duplicate DAGMan PID %d *may* be alive; this DAGMan is continuing, "
-			                  "but this will cause problems if the duplicate DAGMan is alive.\n",
-			                  procId->getPid());
+			print_msg("Duplicate DAGMan PID %d *may* be alive; this DAGMan is continuing, "
+			          "but this will cause problems if the duplicate DAGMan is alive.\n",
+			          procId->getPid());
 			result = 0;
 
 		} else {
@@ -1017,8 +1129,8 @@ DagmanUtils::check_lock_file(const char *lockFileName) {
 
 	if (fp != nullptr) {
 		if (fclose(fp) != 0) {
-			dprintf(D_ALWAYS, "ERROR: closing lock file failed with errno %d (%s)\n",
-			        errno, strerror(errno));
+			print_error("ERROR: Failed to close lock file failed (%d): %s\n",
+			            errno, strerror(errno));
 		}
 	}
 
@@ -1072,7 +1184,7 @@ void DagmanOptions::addDeepArgs(ArgList& args, bool inWriteSubmit) const {
 	args.AppendArg("-AutoRescue");
 	args.AppendArg(std::to_string(self[i::AutoRescue]));
 
-	if (inWriteSubmit || self[i::DoRescueFrom]) {
+	if (self[i::DoRescueFrom]) {
 		args.AppendArg("-DoRescueFrom");
 		args.AppendArg(std::to_string(self[i::DoRescueFrom]));
 	}
@@ -1342,18 +1454,62 @@ std::string DagmanOptions::processOptionArg(const std::string& opt, std::string
 	return arg;
 }
 
+bool DagmanOptions::checkMutualExclusion(const std::string& opt, std::string& err, const std::string& flag) {
+	// Check for mutual exclusion
+	auto it = dagOptsMutualExclusions.find(opt);
+	if (it != dagOptsMutualExclusions.end()) {
+		std::vector<std::string> intersection;
+		std::ranges::set_intersection(it->second, setOptions, std::back_inserter(intersection));
+		if ( ! intersection.empty()) {
+			std::string conflicts, desc;
+			if (flag.empty()) {
+				// Just join option keywords
+				conflicts = join(intersection, ",");
+				desc = "option";
+			} else {
+				// Join option flags
+				conflicts = std::accumulate(dagOptionsInfoMap.begin(), dagOptionsInfoMap.end(), std::string{},
+							[intersection](std::string curr, const auto& pair) {
+								const auto& [opt_flag, info] = pair;
+								if (std::ranges::find(intersection, std::get<0>(info)) != intersection.end()) {
+									if (curr.size()) { curr += ","; }
+									curr += opt_flag;
+								}
+								return curr;
+							});
+				desc = "flag";
+			}
+			if (intersection.size() > 1) { desc += "s"; }
+			formatstr(err, "%s can not be used with the following %s: %s",
+			          flag.empty() ? opt.c_str() : flag.c_str(), desc.c_str(), conflicts.c_str());
+			return false;
+		}
+	}
+
+	setOptions.insert(opt);
+	return true;
+}
+
 bool DagmanOptions::AutoParse(const std::string &flag, size_t &iArg, const size_t argc, const char * const argv[], std::string &err, DagmanOptions* duplicate) {
 	SetDagOpt ret = SetDagOpt::KEY_DNE;
 	// Get information about flag
 	std::string fullFlag = DagmanGetFullFlag(flag);
 	const auto& [opt, meta, _, __] = DagmanGetFlagInfo(flag);
+
 	// No option means invalid flag
 	if (opt.empty()) {
 		formatstr(err,"Error: Unknown flag '%s' provided", flag.c_str());
 		return false;
+	}
+
+	// Check for mutual exclusion of flags/options (note independent of contradictory bool options)
+	if ( ! checkMutualExclusion(opt, err, fullFlag)) {
+		err = "Error: " + err;
+		return false;
+	}
 
 	// Handle bool options
-	} else if (IsOptionTypeBool(opt)) {
+	if (IsOptionTypeBool(opt)) {
 		// Check if opposite flags for the same option were specified
 		if (boolFlagCheck.contains(opt)) {
 			std::string usedFlag = boolFlagCheck[opt];
