@@ -1,6 +1,6 @@
 /***************************************************************
  *
- * Copyright (C) 1990-2018, Condor Team, Computer Sciences Department,
+ * Copyright (C) 1990-2025, Condor Team, Computer Sciences Department,
  * University of Wisconsin-Madison, WI.
  * 
  * Licensed under the Apache License, Version 2.0 (the "License"); you
@@ -34,8 +34,15 @@
 #define dagman_exe "condor_dagman.exe"
 #define valgrind_exe "valgrind.exe"
 #else
+
 #define dagman_exe "condor_dagman"
+
+#ifdef LINUX
 #define valgrind_exe "valgrind"
+#else
+#define valgrind_exe "leaks"
+#endif
+
 #endif
 
 const int UTIL_MAX_LINE_LENGTH = 1024;
@@ -194,15 +201,16 @@ static const std::map<std::string, DagOptionInfo, KeyNoCaseCmp> dagOptionsInfoMa
 	/* {Flag, {option, metavar, description, display sources}}, */
 	{"-AllowVersionMismatch", {"AllowVersionMismatch", "True", "Allow version difference between *.condor.sub file and condor_dagman", DAG_OPT_DISP_ALL}},
 	{"-AlwaysRunPost", {"PostRun", "True", "Run POST script if PRE script fails", DAG_OPT_DISP_ALL}},
+	{"-a",             {"AppendLines", "<command>", "See -Append", 0}}, // Single letter flag to make -a equal to -append
 	{"-Append",        {"AppendLines", "<command>", "Append submit description command to *.condor.sub file", DAG_OPT_DISP_CSD}},
 	{"-AutoRescue",    {"AutoRescue", "<0|1>", "Control automatically running new rescue DAG (0=False | 1=True)", DAG_OPT_DISP_ALL}}, // Note non-zero = True
 	{"-Batch-Name",    {"BatchName", "<name>", "Set DAG batch name", DAG_OPT_DISP_ALL}},
 	{"-Config",        {"ConfigFile", "<filename>", "Specify DAGMan configuration file", DAG_OPT_DISP_CSD|DAG_OPT_DISP_PY_BIND}},
-	{"-CsdVersion",    {"CsdVersion", "<Version String>", "Current condor_submit_dag version string", 0}}, // Not user controlled and manually displayed in condor_dagman
-	{"-Dag",           {"DagFiles", "<NAME.dag>", "DAG file for DAGMan to execute", 0}}, // Only applies to DAGMan Main and is manually entered
-	{"-DAGMan",        {"DagmanPath", "<path>", "Full path to alternate condor_dagman executable", DAG_OPT_DISP_ALL}},
+	{"-CsdVersion",    {"CsdVersion", "<Version String>", "Current condor_submit_dag version string", DAG_OPT_DISP_DAGMAN}},
+	{"-Dag",           {"DagFiles", "<NAME.dag>", "DAG file for DAGMan to execute", DAG_OPT_DISP_DAGMAN}},
+	{"-DAGMan",        {"DagmanPath", "<path>", "Full path to alternate condor_dagman executable", DAG_OPT_DISP_CSD|DAG_OPT_DISP_PY_BIND}},
 	{"-Debug",         {"DebugLevel", "<level>", "Set DAGMan debug logs verbosity", DAG_OPT_DISP_ALL}},
-	{"-do_recurse",    {"Recurse", "True", "Recursively generate Sub-DAG *.condor.sub files", DAG_OPT_DISP_DAGMAN|DAG_OPT_DISP_CSD}},
+	{"-do_recurse",    {"Recurse", "True", "Recursively generate Sub-DAG *.condor.sub files", DAG_OPT_DISP_CSD}},
 	{"-dont_suppress_notification", {"SuppressNotification", "False", "Suppress email notifications for DAGMan and all its submitted jobs", DAG_OPT_DISP_ALL}},
 	{"-DontAlwaysRunPost", {"PostRun", "False", "Don't run POST script if PRE script fails", DAG_OPT_DISP_ALL}},
 	{"-DoRecovery",    {"DoRecovery", "True", "Allow DAG submitted jobs to send email notifications", DAG_OPT_DISP_ALL}},
@@ -211,24 +219,26 @@ static const std::map<std::string, DagOptionInfo, KeyNoCaseCmp> dagOptionsInfoMa
 	{"-DryRun",        {"DryRun", "True", "Dry run condor_dagman execution of DAG", DAG_OPT_DISP_DAGMAN}},
 	{"-DumpRescue",    {"DumpRescueDag", "True", "DAGMan dump rescue DAG and exit", DAG_OPT_DISP_ALL}},
 	{"-f",             {"Force", "True", "See -Force", 0}}, // Single letter flag to make -f equal to -Force
-	{"-Force",         {"Force", "True", "Overwrite used DAG file if they exist", DAG_OPT_DISP_ALL}},
-	{"-import_env",    {"ImportEnv", "True", "Import current environment into *.condor.sub file", DAG_OPT_DISP_ALL}},
-	{"-include_env",   {"GetFromEnv", "<variables>", "Comma separated list of environment variables to *.condor.sub file getenv filter", DAG_OPT_DISP_ALL}},
-	{"-insert_env",    {"AddToEnv", "<key=value>", "Delimited key=value pairs to explicitly set in the *.condor.sub file environment", DAG_OPT_DISP_ALL}},
+	{"-Force",         {"Force", "True", "Overwrite used DAG file if they exist", DAG_OPT_DISP_CSD|DAG_OPT_DISP_PY_BIND}},
+	{"-import_env",    {"ImportEnv", "True", "Import current environment into *.condor.sub file", DAG_OPT_DISP_CSD|DAG_OPT_DISP_PY_BIND}},
+	{"-include_env",   {"GetFromEnv", "<variables>", "Comma separated list of environment variables to *.condor.sub file getenv filter", DAG_OPT_DISP_CSD|DAG_OPT_DISP_PY_BIND}},
+	{"-insert_env",    {"AddToEnv", "<key=value>", "Delimited key=value pairs to explicitly set in the *.condor.sub file environment", DAG_OPT_DISP_CSD|DAG_OPT_DISP_PY_BIND}},
 	{"-insert_sub_file", {"AppendFile", "<filename>", "Append specified submit file to *.condor.sub file", DAG_OPT_DISP_CSD|DAG_OPT_DISP_PY_BIND}},
 	{"-load_save",     {"SaveFile", "<filename>", "Run DAG from provided save point file", DAG_OPT_DISP_ALL}},
-	{"-Lockfile",      {"LockFile", "<NAME.dag.lock>", "DAGMan lock filename", 0}}, // Only applies to DAGMan Main and is manually displayed
+	{"-Lockfile",      {"LockFile", "<filename>", "DAGMan lock filename", DAG_OPT_DISP_ALL}},
 	{"-MaxIdle",       {"MaxIdle", "<N>", "Maximum number of Idle nodes allowed", DAG_OPT_DISP_ALL}},
 	{"-MaxHold",       {"MaxHold", "<N>", "Maximum number of HOLD scripts to run at once", DAG_OPT_DISP_ALL}},
 	{"-MaxJobs",       {"MaxJobs", "<N>", "Maximum number of jobs submitted at once", DAG_OPT_DISP_ALL}},
 	{"-MaxPost",       {"MaxPost", "<N>", "Maximum number of POST scripts to run at once", DAG_OPT_DISP_ALL}},
 	{"-MaxPre",        {"MaxPre", "<N>", "Maximum number of PRE scripts to run at once", DAG_OPT_DISP_ALL}},
-	{"-no_recurse",    {"Recurse", "False", "Don't recursively generate Sub-DAG *.condor.sub files (Default)", DAG_OPT_DISP_DAGMAN|DAG_OPT_DISP_CSD}},
+	{"-no_recurse",    {"Recurse", "False", "Don't recursively generate Sub-DAG *.condor.sub files (Default)", DAG_OPT_DISP_CSD}},
 	{"-no_submit",     {"DoSubmit", "False", "DAG is not submitted to HTCondor automatically", DAG_OPT_DISP_CSD}},
-	{"-Notification",  {"Notification", "<option>", "Set HTCondor email notification level for DAG", DAG_OPT_DISP_ALL}},
-	{"-outfile_dir",   {"OutfileDir", "<path>", "Directory path to write *.dagman.out file", DAG_OPT_DISP_ALL}},
+	{"-Notification",  {"Notification", "<option>", "Set HTCondor email notification level for DAG", DAG_OPT_DISP_CSD|DAG_OPT_DISP_PY_BIND}},
+	{"-outfile_dir",   {"OutfileDir", "<path>", "Directory path to write *.dagman.out file", DAG_OPT_DISP_CSD|DAG_OPT_DISP_PY_BIND}},
 	{"-Priority",      {"Priority", "<priority>", "Default priority for all jobs submitted by DAGMan", DAG_OPT_DISP_ALL}},
-	{"-Remote",        {"RemoteSchedd", "<schedd name>", "Name of remote schedd to submit DAGMan", DAG_OPT_DISP_CSD}}, // Note: -r works for this
+	{"-r",             {"RemoteSchedd", "<schedd name>", "See -Remote", 0}}, // Single letter flag to make -r equeal -remote
+	{"-Remote",        {"RemoteSchedd", "<schedd name>", "Name of remote schedd to submit DAGMan", DAG_OPT_DISP_CSD}},
+	{"-RescueFile",    {"RescueFile", "<filename>", "Run DAG from specified rescue file", DAG_OPT_DISP_ALL}},
 	{"-schedd-address-file", {"ScheddAddressFile", "<path>", "Submit DAG to Schedd provided by address file", DAG_OPT_DISP_CSD|DAG_OPT_DISP_PY_BIND}},
 	{"-schedd-daemon-ad-file", {"ScheddDaemonAdFile", "<path>", "Submit DAG to Schedd provided by ad file", DAG_OPT_DISP_CSD|DAG_OPT_DISP_PY_BIND}},
 	{"-suppress_notification", {"SuppressNotification", "True", "Suppress email notifications for DAGMan and all its submitted jobs", DAG_OPT_DISP_ALL}},
@@ -241,6 +251,14 @@ static const std::map<std::string, DagOptionInfo, KeyNoCaseCmp> dagOptionsInfoMa
 	//{"-WaitForDebug",  {"WaitForDebug", "True", "Pause condor_dagman execution until debugger is attached", DAG_OPT_DISP_DAGMAN}}, // Note: Handled in condor_dagman
 };
 
+// Key -> Set of keys it does not work with
+static const std::map<std::string, std::set<std::string, KeyNoCaseCmp>, KeyNoCaseCmp> dagOptsMutualExclusions {
+	{"DoRecovery", {"SaveFile"}},
+	{"DoRescueFrom", {"SaveFile", "RescueFile"}},
+	{"RescueFile", {"SaveFile", "DoRescueFrom"}},
+	{"SaveFile", {"DoRecovery", "DoRescueFrom", "RescueFile"}},
+};
+
 enum class DagOptionSrc {
 	DAGMAN_MAIN = 0,
 	CONDOR_SUBMIT_DAG,
@@ -287,10 +305,10 @@ public:
 			shallow.stringOpts[str::AppendFile] = appendFile;
 			shallow.boolOpts[b::DoSubmit] = true;
 			shallow.boolOpts[b::CopyToSpool] = param_boolean( "DAGMAN_COPY_TO_SPOOL", false );
-			shallow.intOpts[i::MaxIdle] = MAX_IDLE_DEFAULT;
-			shallow.intOpts[i::MaxJobs] = 0;
-			shallow.intOpts[i::MaxPre] = 0;
-			shallow.intOpts[i::MaxPost] = 0;
+			shallow.intOpts[i::MaxIdle] = -1;
+			shallow.intOpts[i::MaxJobs] = -1;
+			shallow.intOpts[i::MaxPre] = -1;
+			shallow.intOpts[i::MaxPost] = -1;
 			shallow.intOpts[i::DebugLevel] = DEBUG_UNSET;
 			shallow.intOpts[i::Priority] = 0;
 		} //End Shallow Option Initialization
@@ -355,29 +373,46 @@ public:
 	CLI_BOOL_FLAG & operator[]( DSO::b opt ) { return deep.boolOpts[opt._to_integral()]; }
 	int & operator[]( DSO::i opt ) { return deep.intOpts[opt._to_integral()]; }
 
+	// Check for option mutual exclusion (note if flag is provided then return conflicts as flags)
+	bool checkMutualExclusion(const std::string& opt, std::string& err, const std::string& flag = "");
+
 private:
 	//Shallow options used only by this DAG
 	DagOptionData<SSO> shallow;
 	//Deep options passed down to subdags
 	DagOptionData<DSO> deep;
-	bool is_MultiDag{false};
 	// Map of used bool options to prevent contradictary flags
 	// Used in AutoParse()
 	std::map<std::string, std::string> boolFlagCheck;
+	// Set of options used to check for mutual exclusion
+	std::set<std::string, KeyNoCaseCmp> setOptions;
+	// Multiple DAG files specified
+	bool is_MultiDag{false};
+};
+
+enum class DEBUG_MSG_STREAM {
+	STANDARD = 0, // STDOUT & STDERR
+	DEBUG_LOG,
 };
 
 class DagmanUtils {
 
 public:
 
-	bool usingPythonBindings = false;
+	DagmanUtils(DEBUG_MSG_STREAM stream) : msgStream(stream) {}
+	DagmanUtils() = default;
+
+	// Print normal message to designated stream
+	void print_msg(const char* fmt, ...) const CHECK_PRINTF_FORMAT(2,3);
+	// Print error message to designated stream
+	void print_error(const char* fmt, ...) const CHECK_PRINTF_FORMAT(2,3);
 
 	bool writeSubmitFile(DagmanOptions &options, str_list &dagFileAttrLines) const;
 
 	int runSubmitDag(const DagmanOptions &options, const char *dagFile,
 	                 const char *directory, int priority, bool isRetry);
 
-	bool setUpOptions(DagmanOptions &options, str_list &dagFileAttrLines);
+	bool setUpOptions(DagmanOptions &options, str_list &dagFileAttrLines, std::string* err=nullptr);
 
 	bool processDagCommands(DagmanOptions &options, str_list &attrLines, std::string &errMsg);
 
@@ -395,6 +430,9 @@ public:
 
 	void RenameRescueDagsAfter(const std::string &primaryDagFile, bool multiDags, int rescueDagNum, int maxRescueDagNum);
 
+	// Return rescue DAG number from file or 0 for not matching the rescue dag format
+	int ExtractRescueNum(const std::string& file, const std::string& primaryDagFile, bool multiDags);
+
 	static inline std::string HaltFileName(const std::string &primaryDagFile) { return primaryDagFile + ".halt"; }
 
 	void tolerant_unlink(const std::string &pathname);
@@ -441,6 +479,9 @@ public:
 	DagOptionInfo GetFlagInfo(const std::string& flag);
 	std::string GetFullFlag(const std::string& flag);
 
+	DEBUG_MSG_STREAM msgStream{DEBUG_MSG_STREAM::STANDARD};
+	bool usingPythonBindings{false};
+
 };
 
 
