@@ -48,6 +48,7 @@ class DockerAPI {
 						const std::string & outside_directory,
 						const std::string & inside_directory,
 						const std::list<std::string> extraVolumes,
+						const std::string credentials_dir,
 						int & pid,
 						int * childFDs,
 						bool & shouldAskForPorts,
@@ -59,6 +60,13 @@ class DockerAPI {
 						int *childFDs,
 						CondorError &error);
 
+		static int pullImage(const std::string &image_name, 
+						const std::string &container_name,
+						const std::string &diag_dir,
+						const ClassAd &jobAd,
+						int reaperId,
+						CondorError &error);
+
 		static int execInContainer( const std::string &containerName,
 					    const std::string &command,
 					    const ArgList &arguments,
@@ -133,17 +141,6 @@ class DockerAPI {
 
 		static int stats(const std::string &container, uint64_t &memUsage, uint64_t &netIn, uint64_t &netOut, uint64_t &userCpu, uint64_t &sysCpu);
 
-		/**
-		 * Obtains the docker-inspect values State.Running and State.ExitCode.
-		 *
-		 * @param container		The Docker GUID, or the name passed to run().
-		 * @param isRunning		On success, will be set to State.Running.  Otherwise, unchanged.
-		 * @param exitCode		On success, will be set to State.ExitCode.  Otherwise, unchanged.
-		 * @param error			....
-		 * @return				0 on success, negative otherwise.
-		 */
-		static int getStatus( const std::string & container, bool isRunning, int & result, CondorError & err );
-
 		static int getServicePorts( const std::string & container,
 			const ClassAd & jobAd, ClassAd & serviceAd );
 
@@ -180,11 +177,30 @@ class DockerAPI {
 		 * @return				0 on success, negative otherwise.
 		 */
 		static int inspect( const std::string & container, ClassAd * inspectionAd, CondorError & err );
+		static int tag(const std::string &src, const std::string &dst);
 
 		/* Return the number of bytes that images put into the cache by this HTCondor
 		 * are currently using
 		 */
 		static int64_t imageCacheUsed();
+
+		static bool imageArchIsCompatible(const std::string &);
+		/* Return the cpu architecture of a cached image, or empty string */
+		static int getImageArch(const std::string &image_name, std::string &arch);
+
+		static int removeImagesInImageFile();
+
+		struct ImageInfo {
+			std::string imageName;
+			std::string sha256;
+			std::string lastTagTime;
+			size_t      size_in_bytes;
+		};
+
+		static std::vector<ImageInfo> getImageInfos();
+
+		static std::string toAnnotatedImageName(const std::string &rawImageName, const ClassAd &job);
+		static std::string fromAnnotatedImageName(const std::string &annotatedName);
 };
 
 #endif /* _CONDOR_DOCKER_API_H */
