@@ -69,8 +69,6 @@
 # include <time.h>
 #endif
 
-#include <sstream>
-
 // call when you want to insure that dprintfs are thread safe on Linux regardless of
 // wether daemon core threads are enabled. thread safety cannot be disabled once enabled
 #ifdef WIN32
@@ -132,7 +130,7 @@ extern int _condor_dprintf_works;
 int DebugShouldLockToAppend = 0;
 static int DebugIsLocked = 0;
 
-static int DebugLockDelay = 0; /* seconds spent waiting for lock */
+static time_t DebugLockDelay = 0; /* seconds spent waiting for lock */
 static time_t DebugLockDelayPeriodStarted = 0;
 
 /*
@@ -380,10 +378,10 @@ const char* _format_global_header(int cat_and_flags, int hdr_flags, DebugHeaderI
 				#ifdef D_SUB_SECOND_IS_MICROSECONDS
 				rc = sprintf_realloc( &buf, &bufpos, &buflen, "%d.%06d ", (int)clock_now, (int)info.tv.tv_usec );
 				#else
-				int seconds = (int)clock_now;
+				time_t seconds = clock_now;
 				int micros = info.tv.tv_usec + 500;
 				if( micros >= 1'000'000 ) { micros = 0; seconds += 1; }
-				rc = sprintf_realloc( &buf, &bufpos, &buflen, "%d.%03d ", seconds, micros / 1000 );
+				rc = sprintf_realloc( &buf, &bufpos, &buflen, "%lld.%03d ", (long long)seconds, micros / 1000 );
 				#endif
 			} else {
 				rc = sprintf_realloc( &buf, &bufpos, &buflen, "%lld ", (long long)clock_now );
@@ -1177,7 +1175,7 @@ debug_open_lock(void)
 			}	
 		}
 
-		start_time = time(NULL);
+		start_time = time(nullptr);
 		if( DebugLockDelayPeriodStarted == 0 ) {
 			DebugLockDelayPeriodStarted = start_time;
 		}
@@ -1200,7 +1198,7 @@ debug_open_lock(void)
 			/* Update DebugLockDelay.  Ignore delays that are less than
 			 * two seconds because the resolution is only 1s.
 			 */
-		end_time = time(NULL);
+		end_time = time(nullptr);
 		if( end_time-start_time > 1 ) {
 			DebugLockDelay += end_time-start_time;
 		}
@@ -1408,11 +1406,11 @@ debug_close_file(struct DebugFileInfo* it)
 	if( debug_file_ptr ) {
 		if (debug_file_ptr) {
 			int close_result = fclose_wrapper( debug_file_ptr, FCLOSE_RETRY_MAX );
+			(*it).debugFP = nullptr;
 			if (close_result < 0) {
 				DebugUnlockBroken = 1;
 				_condor_dprintf_exit(errno, "Can't fclose debug log file\n");
 			}
-			(*it).debugFP = NULL;
 		}
 	}
 }
@@ -1432,11 +1430,11 @@ debug_close_all_files()
 		if(!debug_file_ptr)
 			continue;
 		int close_result = fclose_wrapper( debug_file_ptr, FCLOSE_RETRY_MAX );
+		(*it).debugFP = nullptr;
 		if (close_result < 0) {
 			DebugUnlockBroken = 1;
 			_condor_dprintf_exit(errno, "Can't fclose debug log file\n");
 		}
-		(*it).debugFP = NULL;
 	}
 }
 
@@ -1871,6 +1869,7 @@ bool dprintf_retry_errno( int value )
 #endif
 }
 
+#ifdef PSILORD_WHAT_WERE_YOU_THINKING
 /* This function calls fclose(), soaking up EINTRs up to maxRetries times.
    The motivation for this function is Gnats PR 937 (DAGMan crashes if
    straced).  Psilord investigated this and found that, because LIGO
@@ -1905,6 +1904,39 @@ fclose_wrapper( FILE *stream, int maxRetries )
 
 	return result;
 }
+#else
+
+// Recent manual pages now tell use to NEVER retry close, or fclose
+// even on error.  From the close man page:
+
+//  Dealing with error returns from close()
+//       Retrying the close() after a failure return is the wrong thing  to  do,
+//       since this may cause a reused file descriptor from another thread to be
+//       closed.  This can occur because the Linux kernel  always  releases  the
+//       file descriptor early in the close operation, freeing it for reuse; the
+//       steps that may return an error, such as flushing data to the filesystem
+//       or device, occur only later in the close operation.
+//
+//The EINTR error is a somewhat special case.  Regarding the EINTR error,
+//       POSIX.1-2008 says:
+//
+//              If close() is interrupted by a signal that is to be  caught,  it
+//              shall  return -1 with errno set to EINTR and the state of fildes
+//              is unspecified.
+//
+//       This permits the behavior that occurs on Linux and many other implemen‐
+//       tations,  where,  as with other errors that may be reported by close(),
+//       the file descriptor is guaranteed to be closed.
+
+// SO, let's never retry close or fclose, even on EINTR.  We have seen Retrying
+// the fclose to cause glibc to assert with a double free error, which hides
+// our ability to report dprintf problem to our parent.
+int
+fclose_wrapper( FILE *stream, int /*maxRetries*/ )
+{
+	return fclose(stream);
+}
+#endif
 
 void
 _condor_save_dprintf_line( int flags, const char* fmt, ... )
@@ -2520,6 +2552,12 @@ int dprintf_open_logs_in_directory(const char * dir, bool fTruncate /*=false*/)
 	if ( ! DebugLogs) return 0;
 	int opened = 0;
 	auto_free_ptr realdir(realpath(dir, nullptr));
+	if (!realdir) {
+		// Some valid situations (starter log in the job sandbox) will hit
+		// this failure repeatedly. Don't be noisy about it.
+		//dprintf(D_ERROR, "Failed to find real pathname of %s\n", dir);
+		return 0;
+	}
 	const char* flags = fTruncate ? "wN" : "aN";
 	for (auto & it : *DebugLogs) {
 		// TODO: do a better job with directory matching here?
