@@ -30,9 +30,7 @@
 #include "metric_units.h"
 #include "condor_arglist.h"
 #include "condor_holdcodes.h"
-
-
-extern "C" char* d_format_time(double);   // this should be in a .h
+#include "format_time.h"
 
 
 // "private" helper for when we already know the cluster/proc
@@ -112,14 +110,14 @@ construct_custom_attributes( std::string &attributes, ClassAd* job_ad )
 }
 
 
-char*
+std::string
 email_check_domain( const char* addr, ClassAd* job_ad )
 {
 	std::string full_addr = addr;
 
 	if( full_addr.find('@') != std::string::npos ) {
 			// Already has a domain, we're done
-		return strdup( addr );
+		return addr;
 	}
 
 		// No host name specified; add a domain.
@@ -141,7 +139,7 @@ email_check_domain( const char* addr, ClassAd* job_ad )
 	if( ! domain ) {
 			// we're screwed, we can't append a domain, just return
 			// the username again...
-		return strdup( addr );
+		return addr;
 	}
 	
 	full_addr += '@';
@@ -151,7 +149,7 @@ email_check_domain( const char* addr, ClassAd* job_ad )
 		// we've got to free() it now so we don't leak memory.
 	free( domain );
 
-	return strdup( full_addr.c_str() );
+	return full_addr;
 }
 
 
@@ -192,34 +190,35 @@ Email::sendRemove( ClassAd* ad, const char* reason)
 	sendAction( ad, reason, "removed", -1);
 }
 
+void
+Email::sendStart( ClassAd* ad, const char* reason)
+{
+	int notification = NOTIFY_NEVER;
+	ad->LookupInteger( ATTR_JOB_NOTIFICATION, notification );
+	if (notification != NOTIFY_START) {
+		// If the job ad does not request a START notification, we
+		// don't send one.
+		return;
+	}
+
+	std::string full_subject = build_subject_line(ad, reason);
+	fp = email_user_open_id( ad, cluster, proc, full_subject.c_str() );
+	writeJobId( ad );
+
+	std::string remote_host;
+	ad->LookupString( ATTR_REMOTE_HOST, remote_host );
+	fprintf( fp, "\nhas started on %s\n\n", remote_host.c_str() );
+	fprintf( fp, "%s", reason );
+
+	send();
+}
+
 void
 Email::sendRelease( ClassAd* ad, const char* reason)
 {
 	sendAction( ad, reason, "released from hold", -1);
 }
 
-void
-Email::sendHoldAdmin( ClassAd* ad, const char* reason)
-{
-	email_admin = true;
-	sendAction( ad, reason, "put on hold", JOB_SHOULD_HOLD);
-}
-
-void
-Email::sendRemoveAdmin( ClassAd* ad, const char* reason )
-{
-	email_admin = true;
-	sendAction( ad, reason, "removed", -1);
-}
-
-void
-Email::sendReleaseAdmin( ClassAd* ad, const char* reason )
-{
-	email_admin = true;
-	sendAction( ad, reason, "released from hold", -1);
-}
-
-
 void
 Email::sendAction( ClassAd* ad, const char* reason,
 						const char* action, int exit_code)
@@ -252,6 +251,20 @@ Email::sendError( ClassAd* ad, const char* err_summary,
 */
 
 
+std::string 
+Email::build_subject_line( ClassAd *ad, const char *subject ) {
+	std::string full_subject;
+	ad->LookupInteger( ATTR_CLUSTER_ID, cluster );
+	ad->LookupInteger( ATTR_PROC_ID, proc );
+
+	formatstr(full_subject, "Condor Job %d.%d", cluster, proc);
+	if( subject ) {
+		full_subject += " ";
+		full_subject += subject;
+	}
+	return full_subject;
+}
+
 FILE*
 Email::open_stream( ClassAd* ad, int exit_reason, const char* subject )
 {
@@ -260,15 +273,7 @@ Email::open_stream( ClassAd* ad, int exit_reason, const char* subject )
 		return NULL;
 	}
 
-	ad->LookupInteger( ATTR_CLUSTER_ID, cluster );
-	ad->LookupInteger( ATTR_PROC_ID, proc );
-
-	std::string full_subject;
-	formatstr(full_subject, "Condor Job %d.%d", cluster, proc);
-	if( subject ) {
-		full_subject += " ";
-		full_subject += subject;
-	}
+	std::string full_subject = build_subject_line(ad, subject);
 	if(email_admin) {
 		fp = email_admin_open( full_subject.c_str() );
 	} else {
@@ -354,7 +359,7 @@ Email::writeExit( ClassAd* ad, int exit_reason )
 	int image_size = 0;
 	ad->LookupInteger( ATTR_IMAGE_SIZE, image_size );
 	
-	int shadow_bday = 0;
+	time_t shadow_bday = 0;
 	ad->LookupInteger( ATTR_SHADOW_BIRTHDATE, shadow_bday );
 	
 	double previous_runs = 0;
@@ -386,12 +391,12 @@ Email::writeExit( ClassAd* ad, int exit_reason )
 	fprintf(fp, "\n\nSubmitted at:        %s", ctime(&arch_time));
 	
 	if( exit_reason == JOB_EXITED || exit_reason == JOB_COREDUMPED ) {
-		double real_time = now - q_date;
+		time_t real_time = now - q_date;
 		arch_time = now;
 		fprintf(fp, "Completed at:        %s", ctime(&arch_time));
 		
 		fprintf(fp, "Real Time:           %s\n",
-				d_format_time(real_time));
+				format_time(real_time));
 	}	
 
 	fprintf( fp, "\n" );
@@ -400,21 +405,21 @@ Email::writeExit( ClassAd* ad, int exit_reason )
 	
 	double rutime = remote_user_cpu;
 	double rstime = remote_sys_cpu;
-	double trtime = rutime + rstime;
-	double wall_time = 0;
+	time_t trtime = (long long)(rutime + rstime);
+	time_t wall_time = 0;
 	fprintf(fp, "Statistics from last run:\n");
 	if(shadow_bday != 0) {	// Handle cases where this wasn't set (grid)
 		wall_time = now - shadow_bday;
 	}
-	fprintf(fp, "Allocation/Run time:     %s\n",d_format_time(wall_time) );
-	fprintf(fp, "Remote User CPU Time:    %s\n", d_format_time(rutime) );
-	fprintf(fp, "Remote System CPU Time:  %s\n", d_format_time(rstime) );
-	fprintf(fp, "Total Remote CPU Time:   %s\n\n", d_format_time(trtime));
+	fprintf(fp, "Allocation/Run time:     %s\n",format_time(wall_time) );
+	fprintf(fp, "Remote User CPU Time:    %s\n", format_time((long long)rutime) );
+	fprintf(fp, "Remote System CPU Time:  %s\n", format_time((long long)rstime) );
+	fprintf(fp, "Total Remote CPU Time:   %s\n\n", format_time(trtime));
 	
 	double total_wall_time = previous_runs + wall_time;
 	fprintf(fp, "Statistics totaled from all runs:\n");
 	fprintf(fp, "Allocation/Run time:     %s\n",
-			d_format_time(total_wall_time) );
+			format_time(total_wall_time));
 
 		// TODO: deal w/ bytes directly from the classad
 	return true;
@@ -562,6 +567,9 @@ Email::shouldSend( ClassAd* ad, int exit_reason, bool is_error )
 			}
 			break;
 
+		case NOTIFY_START:
+			// Always handled earlier.
+			break;
 		default:
 			ad->LookupInteger( ATTR_CLUSTER_ID, ad_cluster );
 			ad->LookupInteger( ATTR_PROC_ID, ad_proc );
