@@ -132,6 +132,41 @@ void stats_entry_recent_histogram<T>::PublishDebug(ClassAd & ad, const char * pa
    ad.Assign(pattr, str);
 }
 
+// specialization of PublishDebug to dump the internal data storage
+// of stats_entry_daily. 
+//
+template <class T>
+void stats_entry_daily<T>::PublishDebug(ClassAd & ad, const char * pattr, int flags) const {
+    std::string str;
+
+    str += std::to_string(this->value);
+    formatstr_cat(str, " {h:%d c:%d m:%d a:%d}",
+        this->hours.ixHead, this->hours.cItems, this->hours.cMax, this->hours.cAlloc);
+    if (this->hours.pbuf) {
+        for (int ix = 0; ix < this->hours.cAlloc; ++ix) {
+            str += !ix ? "[" : (ix == this->hours.cMax ? "|" : ",");
+            str += std::to_string(this->hours.pbuf[ix]);
+        }
+        str += "]";
+    }
+    formatstr_cat(str, " {h:%d c:%d m:%d a:%d}",
+        this->days.ixHead, this->days.cItems, this->days.cMax, this->days.cAlloc);
+    if (this->days.pbuf) {
+        for (int ix = 0; ix < this->days.cAlloc; ++ix) {
+            str += !ix ? "[" : (ix == this->days.cMax ? "|" : ",");
+            str += std::to_string(this->days.pbuf[ix]);
+        }
+        str += "]";
+    }
+
+    std::string attr(pattr);
+    if (flags & this->PubDecorateAttr)
+        attr += "Debug";
+
+    ad.Assign(pattr, str);
+}
+
+
 
 // Determine if enough time has passed to Advance the recent buffers,
 // returns an advance count. 
@@ -174,7 +209,7 @@ int generic_stats_Tick(
       // and update the prev update time.
       if (delta >= RecentQuantum)
          {
-         cTicks = delta / RecentQuantum;
+         cTicks = (int) (delta / RecentQuantum);
          RecentTickTime = now - (delta % RecentQuantum);
          }
 
@@ -558,25 +593,25 @@ template <> void stats_entry_recent<Probe>::Unpublish(ClassAd& ad, const char *
    std::string attr;
    ad.Delete(pattr);
    formatstr(attr, "Recent%s", pattr);
-   ad.Delete(attr.c_str());
+   ad.Delete(attr);
 
    formatstr(attr, "Recent%sCount", pattr);
-   ad.Delete(attr.c_str());
+   ad.Delete(attr);
    ad.Delete(attr.c_str()+6);
    formatstr(attr, "Recent%sSum", pattr);
-   ad.Delete(attr.c_str());
+   ad.Delete(attr);
    ad.Delete(attr.c_str()+6);
    formatstr(attr, "Recent%sAvg", pattr);
-   ad.Delete(attr.c_str());
+   ad.Delete(attr);
    ad.Delete(attr.c_str()+6);
    formatstr(attr, "Recent%sMin", pattr);
-   ad.Delete(attr.c_str());
+   ad.Delete(attr);
    ad.Delete(attr.c_str()+6);
    formatstr(attr, "Recent%sMax", pattr);
-   ad.Delete(attr.c_str());
+   ad.Delete(attr);
    ad.Delete(attr.c_str()+6);
    formatstr(attr, "Recent%sStd", pattr);
-   ad.Delete(attr.c_str());
+   ad.Delete(attr);
    ad.Delete(attr.c_str()+6);
 }
 
@@ -1038,7 +1073,7 @@ int StatisticsPool::RemoveProbesByAddress(void* first, void *last)
    auto pool_itr = pool.begin();
    while (pool_itr != pool.end()) {
       auto probe = pool_itr->first;
-	  auto item2 = pool_itr->second;
+	  const auto &item2 = pool_itr->second;
       if (probe < first || probe > last) {
          pool_itr++;
 	  } else {
@@ -1281,7 +1316,7 @@ void StatisticsPool::Unpublish(ClassAd & ad, const char * prefix) const
          (probe->*(item.Unpublish))(ad, attr.c_str());
          }
       else
-         ad.Delete(attr.c_str());
+         ad.Delete(attr);
       }
 }
 
@@ -1415,7 +1450,7 @@ void stats_entry_sum_ema_rate<T>::Unpublish(ClassAd & ad, const char * pattr) co
 		else {
 			formatstr(attr_name,"%sPerSecond_%s",pattr,config.horizon_name.c_str());
 		}
-		ad.Delete(attr_name.c_str());
+		ad.Delete(attr_name);
 	}
 }
 
@@ -1451,7 +1486,7 @@ void stats_entry_ema<T>::Unpublish(ClassAd & ad, const char * pattr) const {
 		stats_ema_config::horizon_config &config = this->ema_config->horizons[i];
 		std::string attr_name;
 		formatstr(attr_name,"%s_%s",pattr,config.horizon_name.c_str());
-		ad.Delete(attr_name.c_str());
+		ad.Delete(attr_name);
 	}
 }
 
@@ -1538,6 +1573,9 @@ template class stats_entry_sum_ema_rate<uint64_t>;
 template class stats_entry_ema<int>;
 template class stats_entry_ema<double>;
 template class stats_entry_probe<double>;
+template class stats_entry_daily<int>;
+template class stats_entry_daily<long long>;
+template class stats_entry_daily<double>;
 
 //
 // This is how you use the generic_stats functions.
