@@ -1,7 +1,6 @@
 #include "condor_common.h"
 #include <set>
 #include "condor_sockaddr.h"
-#include "condor_netdb.h"
 #include "condor_config.h"
 #include "condor_sockfunc.h"
 #include "ipv6_hostname.h"
@@ -15,6 +14,184 @@ static std::string local_hostname;
 static std::string local_fqdn;
 static bool hostname_initialized = false;
 
+uint32_t ipv6_get_scope_id() {
+	static bool scope_id_inited = false;
+	static uint32_t scope_id = 0;
+
+	if (!scope_id_inited) {
+		std::string network_interface;
+		condor_sockaddr ipv4;
+		condor_sockaddr ipv6;
+		condor_sockaddr ipbest;
+		if (param(network_interface, "NETWORK_INTERFACE") &&
+		    network_interface_to_sockaddr("NETWORK_INTERFACE",
+		                                  network_interface.c_str(),
+		                                  ipv4, ipv6, ipbest) &&
+		    ipv6.is_valid() && ipv6.is_link_local())
+		{
+			scope_id = ipv6.to_sin6().sin6_scope_id;
+		}
+		else if (network_interface_to_sockaddr("Ipv6LinkLocal", "fe80:*",
+		                                       ipv4, ipv6, ipbest) &&
+		         ipv6.is_valid() && ipv6.is_link_local())
+		{
+			scope_id = ipv6.to_sin6().sin6_scope_id;
+		}
+		scope_id_inited = true;
+	}
+
+	return scope_id;
+}
+
+/* SPECIAL NAMES:
+ *
+ * If NO_DNS is configured we do some special name/ip handling. IP
+ * addresses given to gethostbyaddr, XXX.XXX.XXX.XXX, will return
+ * XXX-XXX-XXX-XXX.DEFAULT_DOMAIN, and an attempt will be made to
+ * parse gethostbyname names from XXX-XXX-XXX-XXX.DEFAULT_DOMAIN into
+ * XXX.XXX.XXX.XXX.
+ */
+/* WARNING: None of these functions properly set h_error, or errno */
+int
+condor_gethostname(char *name, size_t namelen) {
+
+	if (param_boolean("NO_DNS", false)) {
+
+		char tmp[MAXHOSTNAMELEN];
+		char *param_buf;
+
+			// First, we try NETWORK_INTERFACE
+		if ( (param_buf = param( "NETWORK_INTERFACE" )) ) {
+
+			dprintf( D_HOSTNAME, "NO_DNS: Using NETWORK_INTERFACE='%s' "
+					 "to determine hostname\n", param_buf );
+
+			condor_sockaddr ipv4;
+			condor_sockaddr ipv6;
+			condor_sockaddr ipbest;
+			if ( !network_interface_to_sockaddr("NETWORK_INTERFACE", param_buf, ipv4, ipv6, ipbest) ) {
+				dprintf(D_HOSTNAME, "NO_DNS: network_interface_to_sockaddr() failed\n");
+				free( param_buf );
+				return -1;
+			}
+			free( param_buf );
+
+			std::string hostname = convert_ipaddr_to_fake_hostname(ipbest);
+			if (hostname.length() >= namelen) {
+				return -1;
+			}
+			strcpy(name, hostname.c_str());
+			return 0;
+		}
+
+			// Second, we try COLLECTOR_HOST
+
+			// To get the hostname with NODNS we are going to jump
+			// through some hoops. We will try to make a UDP
+			// connection to the COLLECTOR_HOST. This will result in not
+			// only letting us call getsockname to find an IP for this
+			// machine, but it will select the proper IP that we can
+			// use to contact the COLLECTOR_HOST
+		if ( (param_buf = param( "COLLECTOR_HOST" )) ) {
+
+			int s;
+			char collector_host[MAXHOSTNAMELEN];
+			char *idx;
+			condor_sockaddr collector_addr;
+			condor_sockaddr addr;
+			std::vector<condor_sockaddr> collector_addrs;
+
+			dprintf( D_HOSTNAME, "NO_DNS: Using COLLECTOR_HOST='%s' "
+					 "to determine hostname\n", param_buf );
+
+				// Get only the name portion of the COLLECTOR_HOST
+			if ((idx = index(param_buf, ':'))) {
+				*idx = '\0';
+			}
+			snprintf( collector_host, MAXHOSTNAMELEN, "%s", param_buf );
+			free( param_buf );
+
+				// Now that we have the name we need an IP
+
+			collector_addrs = resolve_hostname(collector_host);
+			if (collector_addrs.empty()) {
+				dprintf(D_HOSTNAME,
+						"NO_DNS: Failed to get IP address of collector "
+						"host '%s'\n", collector_host);
+				return -1;
+			}
+
+			collector_addr = collector_addrs.front();
+			collector_addr.set_port(1980);
+
+				// We are doing UDP, the benefit is connect will not send
+				// any network traffic on a UDP socket
+			if (-1 == (s = socket(collector_addr.get_aftype(), 
+								  SOCK_DGRAM, 0))) {
+				dprintf(D_HOSTNAME,
+						"NO_DNS: Failed to create socket, errno=%d (%s)\n",
+						errno, strerror(errno));
+				return -1;
+			}
+
+			if (condor_connect(s, collector_addr)) {
+				close(s);
+				dprintf(D_HOSTNAME,
+						"NO_DNS: Failed to bind socket, errno=%d (%s)\n",
+						errno, strerror(errno));
+				return -1;
+			}
+
+			if (condor_getsockname(s, addr)) {
+				close(s);
+				dprintf(D_HOSTNAME,
+						"NO_DNS: Failed to get socket name, errno=%d (%s)\n",
+						errno, strerror(errno));
+				return -1;
+			}
+
+			close(s);
+			std::string hostname = convert_ipaddr_to_fake_hostname(addr);
+			if (hostname.length() >= namelen) {
+				return -1;
+			}
+			strcpy(name, hostname.c_str());
+			return 0;
+		}
+
+			// Last, we try gethostname()
+		if ( gethostname( tmp, MAXHOSTNAMELEN ) == 0 ) {
+
+			dprintf( D_HOSTNAME, "NO_DNS: Using gethostname()='%s' "
+					 "to determine hostname\n", tmp );
+
+			std::vector<condor_sockaddr> addrs;
+			std::string my_hostname(tmp);
+			addrs = resolve_hostname_raw(my_hostname);
+			if (addrs.empty()) {
+				dprintf(D_HOSTNAME,
+						"NO_DNS: resolve_hostname_raw() failed, errno=%d"
+						" (%s)\n", errno, strerror(errno));
+				return -1;
+			}
+
+			std::string hostname = convert_ipaddr_to_fake_hostname(addrs.front());
+			if (hostname.length() >= namelen) {
+				return -1;
+			}
+			strcpy(name, hostname.c_str());
+			return 0;
+		}
+
+		dprintf(D_HOSTNAME,
+				"Failed in determining hostname for this machine\n");
+		return -1;
+
+	} else {
+		return gethostname(name, namelen);
+	}
+}
+
 static bool nodns_enabled()
 {
 	return param_boolean("NO_DNS", false);
@@ -59,20 +236,13 @@ bool init_local_hostname_impl()
 	}
 
 	if( ! local_ipaddr_initialized ) {
-		std::string ipv4, ipv6, ipbest;
-		if( network_interface_to_ip("NETWORK_INTERFACE", network_interface.c_str(), ipv4, ipv6, ipbest)) {
-			ASSERT(local_ipaddr.from_ip_string(ipbest));
-			// If this fails, network_interface_to_ip returns something invalid.
+		if( network_interface_to_sockaddr("NETWORK_INTERFACE", network_interface.c_str(), local_ipv4addr, local_ipv6addr, local_ipaddr)) {
+			ASSERT(local_ipaddr.is_valid());
+			// If this fails, network_interface_to_sockaddr returns something invalid.
 			local_ipaddr_initialized = true;
 		} else {
 			dprintf(D_ALWAYS, "Unable to identify IP address from interfaces.  None match NETWORK_INTERFACE=%s. Problems are likely.\n", network_interface.c_str());
 		}
-		if((!ipv4.empty()) && local_ipv4addr.from_ip_string(ipv4)) {
-			ASSERT(local_ipv4addr.is_ipv4());
-		}
-		if((!ipv6.empty()) && local_ipv6addr.from_ip_string(ipv6)) {
-			ASSERT(local_ipv6addr.is_ipv6());
-		}
 	}
 
 	if (nodns_enabled()) {
@@ -86,12 +256,12 @@ bool init_local_hostname_impl()
 			}
 		}
 	} else if (!local_hostname_initialized) {
-		addrinfo_iterator ai;
+		addrinfo* info = nullptr;
 		const int MAX_TRIES = 20;
 		const int SLEEP_DUR = 3;
 		bool gai_success = false;
 		for(int try_count = 1; true; try_count++) {
-			int ret = ipv6_getaddrinfo(test_hostname.c_str(), NULL, ai);
+			int ret = ipv6_getaddrinfo(test_hostname.c_str(), NULL, info);
 			if(ret == 0) { gai_success = true; break; }
 			if(ret != EAI_AGAIN ) {
 				dprintf(D_ALWAYS, "init_local_hostname_impl: ipv6_getaddrinfo() could not look up '%s': %s (%d).  Error is not recoverable; giving up.  Problems are likely.\n", test_hostname.c_str(), gai_strerror(ret), ret );
@@ -108,10 +278,10 @@ bool init_local_hostname_impl()
 		}
 
 		if(gai_success) {
-			addrinfo* info = ai.next();
 			if (info->ai_canonname) {
 				local_hostname = info->ai_canonname;
 			}
+			freeaddrinfo(info);
 		}
 
 	}
@@ -176,31 +346,22 @@ std::string get_fqdn_from_hostname(const std::string& hostname) {
 	std::string ret;
 
 	if (!nodns_enabled()) {
-		addrinfo_iterator ai;
-		int res  = ipv6_getaddrinfo(hostname.c_str(), NULL, ai);
+		addrinfo* info = nullptr;
+		int res  = ipv6_getaddrinfo(hostname.c_str(), NULL, info);
 		if (res) {
 			dprintf(D_HOSTNAME, "ipv6_getaddrinfo() could not look up %s: %s (%d)\n", 
 				hostname.c_str(), gai_strerror(res), res);
 			return ret;
 		}
 
-		addrinfo *info = ai.next();
-		if (info != NULL && info->ai_canonname != NULL) {
+		if (info && info->ai_canonname != nullptr) {
 			if (strchr(info->ai_canonname, '.')) {
-				return info->ai_canonname;
-			}
-		}
-
-		hostent* h = gethostbyname(hostname.c_str());
-		if (h && h->h_name && strchr(h->h_name, '.')) {
-			return h->h_name;
-		}
-		if (h && h->h_aliases && *h->h_aliases) {
-			for (char** alias = h->h_aliases; *alias; ++alias) {
-				if (strchr(*alias, '.'))
-					return *alias;
+				ret = info->ai_canonname;
+				freeaddrinfo(info);
+				return ret;
 			}
 		}
+		freeaddrinfo(info);
 	}
 
 	std::string default_domain;
@@ -217,75 +378,25 @@ int get_fqdn_and_ip_from_hostname(const std::string & hostname,
 		std::string & fqdn, condor_sockaddr& addr) {
 
 	std::string ret;
-	condor_sockaddr ret_addr;
-	bool found_ip = false;
+	std::vector<condor_sockaddr> addr_list;
 
-	// if the hostname contains dot, hostname is assumed to be full hostname
-	if( hostname.find('.') != std::string::npos ) {
-		ret = hostname;
-	}
+	addr_list = resolve_hostname(hostname, &ret);
 
-	if (nodns_enabled()) {
-		// if nodns is enabled, convert hostname to ip address directly
-		ret_addr = convert_fake_hostname_to_ipaddr(hostname);
-
-		// note that convert_fake_hostname_to_ipaddr() could fail; if so,
-		// leave found_ip = false and fall through to the block below
-		// where we try to use the resolver.
-		if (ret_addr != condor_sockaddr::null) {
-			found_ip = true;
+	// if FQDN is still unresolved, try to use given hostname
+	if (ret.empty()) {
+		std::string default_domain;
+		// if the hostname contains dot, hostname is assumed to be full hostname
+		// else try DEFAULT_DOMAIN_NAME
+		if (hostname.find('.') != std::string::npos) {
+			ret = hostname;
+		} else if (param(default_domain, "DEFAULT_DOMAIN_NAME")) {
+			ret = hostname + "." + default_domain;
 		}
 	}
 
-	if (!found_ip) {
-		// we look through getaddrinfo and gethostbyname
-		// to further seek fully-qualified domain name and corresponding
-		// ip address
-		addrinfo_iterator ai;
-		int res  = ipv6_getaddrinfo(hostname.c_str(), NULL, ai);
-		if (res) {
-			dprintf(D_HOSTNAME, "ipv6_getaddrinfo() could not look up %s: %s (%d)\n", hostname.c_str(),
-				gai_strerror(res), res);
-			return 0;
-		}
-
-		addrinfo *info = ai.next();
-		if (info != NULL && info->ai_canonname != NULL) {
-			fqdn = info->ai_canonname;
-			addr = condor_sockaddr(info->ai_addr);
-			return 1;
-		}
-
-		hostent* h = gethostbyname(hostname.c_str());
-		if (h && h->h_name && strchr(h->h_name, '.')) {
-			fqdn = h->h_name;
-			addr = condor_sockaddr((sockaddr*)h->h_addr);
-			return 1;
-		}
-		if (h && h->h_aliases && *h->h_aliases) {
-			for (char** alias = h->h_aliases; *alias; ++alias) {
-				if (strchr(*alias, '.')) {
-					fqdn = *alias;
-					addr = condor_sockaddr((sockaddr*)h->h_addr);
-					return 1;
-				}
-			}
-		}
-	}
-
-	std::string default_domain;
-
-	// if FQDN is still unresolved, try DEFAULT_DOMAIN_NAME
-	if (ret.length() == 0 && param(default_domain, "DEFAULT_DOMAIN_NAME")) {
-		ret = hostname;
-		if (ret[ret.length() - 1] != '.')
-			ret += ".";
-		ret += default_domain;
-	}
-
-	if (ret.length() > 0 && found_ip) {
+	if (!ret.empty() && !addr_list.empty()) {
 		fqdn = ret;
-		addr = ret_addr;
+		addr = addr_list[0];
 		return 1;
 	}
 	return 0;
@@ -335,7 +446,7 @@ bool verify_name_has_ip(std::string name, condor_sockaddr addr){
 		std::string ips_str; ips_str.reserve(addrs.size()*40);
 		for(unsigned int i = 0; i < addrs.size(); i++) {
 			ips_str += "\n\t";
-			ips_str += addrs[i].to_ip_string().c_str();
+			ips_str += addrs[i].to_ip_string();
 		}
 		dprintf(D_SECURITY|D_VERBOSE, "IPVERIFY: checking %s against %s addrs are:%s\n",
 				name.c_str(), addr.to_ip_string().c_str(), ips_str.c_str());
@@ -436,27 +547,21 @@ std::string get_full_hostname(const condor_sockaddr& addr)
 	return ret;
 }
 
-std::vector<condor_sockaddr> resolve_hostname(const char* hostname)
-{
-	std::string host(hostname);
-	return resolve_hostname(host);
-}
-
-
-std::vector<condor_sockaddr> resolve_hostname(const std::string& hostname)
+std::vector<condor_sockaddr> resolve_hostname(const std::string& hostname, std::string* canonical)
 {
 	std::vector<condor_sockaddr> ret;
 	if (nodns_enabled()) {
 		condor_sockaddr addr = convert_fake_hostname_to_ipaddr(hostname);
-		if (addr == condor_sockaddr::null)
-			return ret;
-		ret.push_back(addr);
+		if (addr != condor_sockaddr::null) {
+			ret.push_back(addr);
+			if (canonical) *canonical = hostname;
+		}
 		return ret;
 	}
-	return resolve_hostname_raw(hostname);
+	return resolve_hostname_raw(hostname, canonical);
 }
 
-std::vector<condor_sockaddr> resolve_hostname_raw(const std::string& hostname) {
+std::vector<condor_sockaddr> resolve_hostname_raw(const std::string& hostname, std::string* canonical) {
 	//
 	// For now, we can just document that you need the Punycoded DSN name.
 	// If somebody complains about that, we can revisit this.  (If we
@@ -474,24 +579,45 @@ std::vector<condor_sockaddr> resolve_hostname_raw(const std::string& hostname) {
 		return ret;
 	}
 
-	addrinfo_iterator ai;
-	int res  = ipv6_getaddrinfo(hostname.c_str(), NULL, ai);
+	addrinfo* info = nullptr;
+	int res  = ipv6_getaddrinfo(hostname.c_str(), NULL, info);
 	if (res) {
 		dprintf(D_HOSTNAME, "ipv6_getaddrinfo() could not look up %s: %s (%d)\n", 
 			hostname.c_str(), gai_strerror(res), res);
 		return ret;
 	}
 
-	// To eliminate duplicate address, here we use std::set.
-	// we also want to maintain the order given by getaddrinfo.
-	std::set<condor_sockaddr> s;
-	while (addrinfo* info = ai.next()) {
-		condor_sockaddr addr(info->ai_addr);
-		if (s.find(addr) == s.end()) {
-			ret.push_back(addr);
-			s.insert(addr);
-		}
+	if (canonical && info->ai_canonname) {
+		*canonical = info->ai_canonname;
 	}
+
+	addrinfo* next = info;
+	do {
+		// Ignore non-IP address families
+		if (next->ai_family == AF_INET || next->ai_family == AF_INET6) {
+			ret.emplace_back(next->ai_addr);
+		}
+	} while ( (next = next->ai_next) );
+
+	// Sort IPv6 link-local addresses to the end of the list.
+	// Optionally sort IPv4 addresses to the top of the list.
+	bool prefer_ipv4 = false;
+	bool have_preference = param_boolean("IGNORE_DNS_PROTOCOL_PREFERENCE", true);
+	if (have_preference) {
+		prefer_ipv4 = param_boolean("PREFER_OUTBOUND_IPV4", true);
+	}
+	auto ip_sort = [=](const condor_sockaddr& a, const condor_sockaddr& b) {
+		if ((a.is_ipv4() || !a.is_link_local()) && b.is_ipv6() && b.is_link_local()) {
+			return true;
+		}
+		if (have_preference && a.is_ipv4() != b.is_ipv4()) {
+			return prefer_ipv4 == a.is_ipv4();
+		}
+		return false;
+	};
+	std::sort(ret.begin(), ret.end(), ip_sort);
+
+	freeaddrinfo(info);
 	return ret;
 }
 
@@ -532,7 +658,7 @@ condor_sockaddr convert_fake_hostname_to_ipaddr(const std::string& fullname)
 	if (param(default_domain, "DEFAULT_DOMAIN_NAME")) {
 		std::string dotted_domain = ".";
 		dotted_domain += default_domain;
-		size_t pos = fullname.find(dotted_domain.c_str());
+		size_t pos = fullname.find(dotted_domain);
 		if (pos != std::string::npos) {
 			truncated = true;
 			hostname = fullname.substr(0, pos);
