@@ -18,9 +18,9 @@
  ***************************************************************/
 
 
-#define _POSIX_SOURCE
 #include "condor_common.h"
 #include "condor_debug.h"
+#include "condor_config.h"
 #include "condor_arglist.h"
 #include "my_popen.h"
 #include "sig_install.h"
@@ -276,7 +276,7 @@ my_popenv(const char *const args[], const char *mode, int options)
 }
 
 int
-my_pclose_ex(FILE *fp, unsigned int timeout, bool kill_after_timeout)
+my_pclose_ex(FILE *fp, time_t timeout, bool kill_after_timeout)
 {
 	HANDLE hChildProcess;
 	DWORD result;
@@ -289,9 +289,13 @@ my_pclose_ex(FILE *fp, unsigned int timeout, bool kill_after_timeout)
 		return MYPCLOSE_EX_NO_SUCH_FP;
 	}
 
+	if (timeout != 0xFFFFFFFF) {
+		timeout *= 1000;
+	}
+
 	result = WaitForSingleObject(hChildProcess, timeout);
 	if (result != WAIT_OBJECT_0) {
-		dprintf(D_FULLDEBUG, "my_pclose: Child process has not exited after %u seconds\n", timeout);
+		dprintf(D_FULLDEBUG, "my_pclose: Child process has not exited after %u milliseconds\n", timeout);
 		if (kill_after_timeout) {
 			TerminateProcess(hChildProcess, -9);
 			CloseHandle(hChildProcess);
@@ -725,7 +729,7 @@ my_pclose( FILE *fp, unsigned int timeout, bool kill_after_timeout ) {
 }
 
 int
-my_pclose_ex(FILE *fp, unsigned int timeout, bool kill_after_timeout)
+my_pclose_ex(FILE *fp, time_t timeout, bool kill_after_timeout)
 {
 	int			status;
 	pid_t			pid;
@@ -1030,7 +1034,7 @@ bool MyPopenTimer::close_program(time_t wait_for_term)
 	// fp will always be NULL when we get here because
 	// read_until_eof called my_pclose at eof()
 	if (fp) {
-		status = my_pclose_ex(fp, (unsigned int)wait_for_term, true);
+		status = my_pclose_ex(fp, wait_for_term, true);
 		run_time = (int)(time(NULL) - begin_time);
 		fp = NULL;
 	}
@@ -1041,11 +1045,16 @@ bool MyPopenTimer::close_program(time_t wait_for_term)
 // timeout is measured relative to the time that start_program was called.
 // returns true if program runs to completion and output is captured
 // false if error or timeout while reading the output.
-const char*  MyPopenTimer::wait_for_output(time_t timeout)
-{
+const char*  MyPopenTimer::wait_for_output(time_t timeout) {
+	return wait_for_output(timeout, [](void) -> void { return; });
+}
+
+const char*  MyPopenTimer::wait_for_output(
+	time_t timeout, std::function<void()> callback
+) {
 	if (error && (error != ETIMEDOUT)) // if error was not a previous timeout, assume we cannot continue
 		return NULL;
-	if (read_until_eof(timeout) != 0)
+	if (read_until_eof(timeout, &callback) != 0)
 		return NULL;
 	return src.data() ? src.data() : "";
 }
@@ -1104,7 +1113,7 @@ static void store_buffers (
 
 
 // returns error, 0 on success
-int MyPopenTimer::read_until_eof(time_t timeout)
+int MyPopenTimer::read_until_eof(time_t timeout, std::function<void()> * callback)
 {
 	if ( ! fp)
 		return error;
@@ -1123,6 +1132,7 @@ int MyPopenTimer::read_until_eof(time_t timeout)
 	int ix = 0;
 	const int cbBuf = 0x2000;
 	char * buffer = (char*)calloc(1, cbBuf);
+	time_t keepalive_interval = param_integer("MY_POPEN_KEEPALIVE_INTERVAL", 60);
 	for(;;) {
 		bool wait_for_hotness = false;
 		int cb = (int)fread(buffer+ix, 1, cbBuf-ix, fp);
@@ -1170,13 +1180,15 @@ int MyPopenTimer::read_until_eof(time_t timeout)
 				//PRAGMA_REMIND("use PeekNamedPipe to check for pipe hotness...")
 				sleep(2);
 			#else
-				int wait_time = timeout - (int)elapsed_time;
-				int rv = poll(&fdt, 1, wait_time*1000);
+				time_t wait_time = timeout - elapsed_time;
+				if( wait_time > keepalive_interval ) {
+					wait_time = keepalive_interval;
+				}
+				int rv = poll(&fdt, 1, int(wait_time*1000));
 				if(rv == 0) {
-					error = ETIMEDOUT;
-					break;
-				} else if (rv > 0) {
-					// new data can be read.
+					if( callback != nullptr ) {
+						(* callback)();
+					}
 				}
 			#endif
 		}
