@@ -30,6 +30,7 @@
 #include "ipv6_hostname.h"
 #include "internet.h"
 #include <string>
+#include <iterator>
 
 /*
 *	This is for tools print tables to have a singular globaly shared
@@ -81,7 +82,7 @@ const char * extractStringsFromList ( const classad::Value & value, Formatter &,
 	classad::ExprList::const_iterator i = list->begin();
 	for( ; i != list->end(); ++i ) {
 		std::string item;
-		if ((*i)->GetKind() != classad::ExprTree::LITERAL_NODE) continue;
+		if (dynamic_cast<classad::Literal *>(*i) == nullptr) continue;
 		classad::Value val;
 		reinterpret_cast<classad::Literal*>(*i)->GetValue(val);
 		if (val.IsStringValue(item)) {
@@ -107,7 +108,7 @@ const char * extractUniqueStrings ( const classad::Value & value, Formatter &, s
 		// for lists, unparse each item into the uniqueness set.
 		for(classad::ExprList::const_iterator it = list->begin() ; it != list->end(); ++it ) {
 			std::string item;
-			if ((*it)->GetKind() != classad::ExprTree::LITERAL_NODE) {
+			if (dynamic_cast<classad::Literal *>(*it) == nullptr) {
 				unparser.Unparse( item, *it );
 			} else {
 				classad::Value val;
@@ -182,11 +183,17 @@ const char * format_version (const char * condorver, Formatter & fmt)
 		else p++;
 	}
 	while (*p == ' ') ++p;
-	while (*p && *p != ' ') ++p; // skip Feb
-	while (*p == ' ') ++p;
-	while (*p && *p != ' ') ++p; // skip 12
-	while (*p == ' ') ++p;
-	while (*p && *p != ' ') ++p; // skip 2016
+	// check for date in YYYY-MM-DD format
+	if (strchr(p, '-') == p+4 && strchr(p+5, '-') == p+7) {
+		while (*p && *p != ' ') ++p; // skip YYYY-MM-DD
+	} else {
+		// old date format Mon DD YYYY
+		while (*p && *p != ' ') ++p; // skip Feb
+		while (*p == ' ') ++p;
+		while (*p && *p != ' ') ++p; // skip 12
+		while (*p == ' ') ++p;
+		while (*p && *p != ' ') ++p; // skip 2016
+	}
 	while (*p == ' ') ++p;
 	// *p now points to a BuildId:, or "PRE-RELEASE"
 	if (*p == 'B') {
@@ -294,15 +301,6 @@ bool render_owner (std::string & out, ClassAd *ad, Formatter & /*fmt*/)
 	if ( ! ad->LookupString(ATTR_OWNER, out))
 		return false;
 
-#ifdef NO_DEPRECATE_NICE_USER
-	int niceUser;
-	if (ad->LookupInteger( ATTR_NICE_USER, niceUser) && niceUser ) {
-		char tmp[sizeof(NiceUserName)+2];
-		strcpy(tmp, NiceUserName);
-		strcat(tmp, ".");
-		out.insert(0, tmp);
-	}
-#endif
 	return true;
 }
 
@@ -694,7 +692,9 @@ bool render_goodput (double & goodput_time, ClassAd *ad, Formatter & /*fmt*/)
 	if ( ! ad->LookupInteger(ATTR_JOB_STATUS, job_status))
 		return false;
 
-	int ckpt_time = 0, shadow_bday = 0, last_ckpt = 0;
+	time_t ckpt_time = 0;
+	time_t shadow_bday = 0;
+	time_t last_ckpt = 0;
 	double wall_clock = 0.0;
 	ad->LookupInteger( ATTR_JOB_COMMITTED_TIME, ckpt_time );
 	ad->LookupInteger( ATTR_SHADOW_BIRTHDATE, shadow_bday );
@@ -719,7 +719,9 @@ bool render_mbps (double & mbps, ClassAd *ad, Formatter & /*fmt*/)
 		return false;
 
 	double wall_clock=0.0, bytes_recvd=0.0, total_mbits;
-	int shadow_bday = 0, last_ckpt = 0, job_status = IDLE;
+	time_t shadow_bday = 0;
+	time_t last_ckpt = 0;
+	int job_status = IDLE;
 	ad->LookupFloat( ATTR_JOB_REMOTE_WALL_CLOCK, wall_clock );
 	ad->LookupInteger( ATTR_SHADOW_BIRTHDATE, shadow_bday );
 	ad->LookupInteger( ATTR_LAST_CKPT_TIME, last_ckpt );
@@ -774,6 +776,24 @@ bool render_activity_code (std::string & act, ClassAd *al, Formatter &)
 	return ok;
 }
 
+// If we are idle and an OCU holder, report "OCU", else report activity
+bool render_activity_or_ocu (std::string & act, ClassAd *al, Formatter &)
+{
+	std::string activity;
+	al->LookupString(ATTR_ACTIVITY, activity);
+	if (activity.empty()) {
+		act = "??";
+		return true;
+	}
+
+	bool is_ocu = false;
+	al->LookupBool(ATTR_OCU, is_ocu);
+	if (is_ocu && activity == "Idle") {
+		act = "OCU";
+	}
+
+	return true;
+}
 
 
 bool render_activity_time (long long & atime, ClassAd *al, Formatter &)
@@ -818,6 +838,21 @@ bool render_version (std::string & str, ClassAd*, Formatter & fmt)
 	return false;
 }
 
+bool render_member_count (classad::Value & value, ClassAd*, Formatter &)
+{
+	const char * list = nullptr;
+	classad::ExprList* ary = nullptr;
+	if (value.IsStringValue(list) && list) {
+		StringTokenIterator sti(list);
+		value.SetIntegerValue(std::distance(sti.begin(), sti.end()));
+		return true;
+	} else if (value.IsListValue(ary) && ary) {
+		value.SetIntegerValue(ary->size());
+		return true;
+	}
+	return false;
+}
+
 bool render_due_date (long long & dt, ClassAd *al, Formatter &)
 {
 	long long now;
@@ -875,6 +910,7 @@ bool render_unique_strings ( classad::Value & value, ClassAd*, Formatter & fmt )
 //          !!! ENTRIES IN THIS TABLE MUST BE SORTED BY THE FIRST FIELD !!!
 const CustomFormatFnTableItem GlobalPrintFormats[] = {
 	{ "ACTIVITY_CODE",   ATTR_ACTIVITY, 0, render_activity_code, ATTR_STATE "\0" },
+	{ "ACTIVITY_OR_OCU", ATTR_ACTIVITY, 0, render_activity_or_ocu, ATTR_STATE "\0" },
 	{ "ACTIVITY_TIME",   ATTR_ENTERED_CURRENT_ACTIVITY, "%T", render_activity_time, ATTR_LAST_HEARD_FROM "\0" ATTR_MY_CURRENT_TIME "\0"  },
 	{ "BATCH_NAME",      ATTR_JOB_CMD, 0, render_batch_name, ATTR_JOB_BATCH_NAME "\0" ATTR_JOB_CMD "\0" ATTR_DAGMAN_JOB_ID "\0" ATTR_DAG_NODE_NAME "\0" },
 	{ "BUFFER_IO_MISC",  ATTR_JOB_UNIVERSE, 0, render_buffer_io_misc, ATTR_FILE_SEEK_COUNT "\0" ATTR_BUFFER_SIZE "\0" ATTR_BUFFER_BLOCK_SIZE "\0" ATTR_TRANSFERRING_INPUT "\0" ATTR_TRANSFERRING_OUTPUT "\0" ATTR_TRANSFER_QUEUED "\0" },
@@ -904,6 +940,7 @@ const CustomFormatFnTableItem GlobalPrintFormats[] = {
 	{ "JOB_STATUS_RAW",  ATTR_JOB_STATUS, 0, format_job_status_raw, NULL },
 	{ "JOB_UNIVERSE",    ATTR_JOB_UNIVERSE, 0, format_job_universe, NULL },
 	{ "LOAD_AVG",        NULL, 0, format_load_avg, NULL },
+	{ "MEMBER_COUNT",    NULL, "%d", render_member_count, NULL },
 	{ "MEMORY_USAGE",    ATTR_IMAGE_SIZE, "%.1f", render_memory_usage, ATTR_MEMORY_USAGE "\0" },
 	{ "OWNER",           ATTR_OWNER, 0, render_owner, ATTR_NICE_USER_deprecated "\0" },
 	{ "PLATFORM",        ATTR_OPSYS, 0, render_platform, ATTR_ARCH "\0" ATTR_OPSYS_AND_VER "\0" ATTR_OPSYS_SHORT_NAME "\0" },
