@@ -12,6 +12,12 @@ readShortFile( const std::string & fileName, std::string & contents );
 bool
 writeShortFile( const std::string & fileName, const std::string & contents );
 
+bool
+writeShortFile(
+    const std::string & fileName,
+    unsigned char * bytes, size_t count
+);
+
 bool
 appendShortFile( const std::string & fileName, const std::string & contents );
 
