@@ -17,13 +17,14 @@
  *
  ***************************************************************/
 
-#include "condor_common.h" 
+#include "condor_common.h"
 #include "condor_snutils.h"
 #include "strcasestr.h"
 #include "condor_debug.h"
 #include "condor_random_num.h"
 #include <limits>
 #include <regex>
+#include <set>
 
 #include "stl_string_utils.h"
 
@@ -413,7 +414,7 @@ const char * is_attr_in_attr_list(const char * attr, const char * list)
 	return NULL;
 }
 
-std::vector<std::string> split(const std::string& str, const char* delim, bool trim)
+std::vector<std::string> split(const std::string& str, const char* delim, STI_TrimBehavior trim)
 {
 	std::vector<std::string> list;
 	for (const auto& item: StringTokenIterator(str, delim, trim)) {
@@ -422,7 +423,7 @@ std::vector<std::string> split(const std::string& str, const char* delim, bool t
 	return list;
 }
 
-std::vector<std::string> split(const char* str, const char* delim, bool trim)
+std::vector<std::string> split(const char* str, const char* delim, STI_TrimBehavior trim)
 {
 	std::vector<std::string> list;
 	for (const auto& item: StringTokenIterator(str, delim, trim)) {
@@ -431,12 +432,15 @@ std::vector<std::string> split(const char* str, const char* delim, bool trim)
 	return list;
 }
 
-std::string join(const std::vector<std::string> &list, const char* delim)
+
+template<class T>
+std::string join( const T & list, const char * delim )
 {
 	std::string str;
 	if (!list.empty()) {
-		str = list.front();
-		for (auto it = (list.begin() + 1); it != list.end(); it++) {
+	    auto it = list.begin();
+		str = * it++;
+		for( ; it != list.end(); ++it ) {
 			str += delim;
 			str += *it;
 		}
@@ -444,6 +448,12 @@ std::string join(const std::vector<std::string> &list, const char* delim)
 	return str;
 }
 
+// Library code requires explicit template instantiation, I guess.
+template std::string join<std::vector<const char *>>(const std::vector<const char *> &list, const char* delim);
+template std::string join<std::vector<std::string>>(const std::vector<std::string> &list, const char* delim);
+template std::string join<std::vector<std::string_view>>(const std::vector<std::string_view> &list, const char* delim);
+template std::string join<std::set<std::string>>(const std::set<std::string> & list, const char* delim);
+
 bool contains(const std::vector<std::string> &list, const char* str)
 {
 	if (!str) {
@@ -776,13 +786,13 @@ int StringTokenIterator::next_token(int & length)
 	size_t ixEnd = ix;
 
 	// skip leading separators and whitespace (if trimming)
-	while (ix < len && str[ix] && (strchr(delims, str[ix]) || (m_trim && isspace(str[ix])))) ++ix;
+	while (ix < len && str[ix] && (strchr(delims, str[ix]) || (m_trim == STI_TRIM && isspace(str[ix])))) ++ix;
 	ixNext = ix;
 	ixEnd = ix;
 
 	// scan for next delimiter or \0
 	while (ix < len && str[ix] && !strchr(delims, str[ix])) {
-		if (!m_trim || !isspace(str[ix])) {
+		if (m_trim != STI_TRIM || !isspace(str[ix])) {
 			ixEnd = ix;
 		}
 		++ix;
@@ -806,7 +816,7 @@ const std::string * StringTokenIterator::next_string()
 	int len;
 	int start = next_token(len);
 	if (start < 0) return NULL;
-	current.assign(str, start, len);
+	current.assign(str + start, len);
 	return &current;
 }
 
