@@ -224,6 +224,9 @@ static void insert_testing_macros(const char * local, const char * subsys)
 		{"fileUp1Rel", "../file"},
 		{"fileOver1Rel", "../peer/file.dat"},
 		{"fileCurRel2", "./uno/dos.tres"},
+		{"fileMultiDot", "/dir/b.o.x"},
+		{"fileDoubleDot", "/dir/boo..ox"},
+		{"fileTarGz", "stuff.tar.gz"},
 		{"wfileAbs", "c:\\one\\two\\three.for"},
 		{"wfileAbsDeep", "c:\\six\\five\\four\\three\\two\\one\\file.ext"},
 		{"wUNCfileAbs", "\\\\server\\share\\one\\two\\file.ext"},
@@ -264,19 +267,7 @@ static void insert_testing_macros(const char * local, const char * subsys)
 
 static void clear_macro_set(MACRO_SET & set)
 {
-	if (set.table) {
-		memset(set.table, 0, sizeof(set.table[0]) * set.allocation_size);
-	}
-	if (set.metat) {
-		memset(set.metat, 0, sizeof(set.metat[0]) * set.allocation_size);
-	}
-	set.size = 0;
-	set.sorted = 0;
-	set.apool.clear();
-	// set.sources.clear();
-	if (set.defaults && set.defaults->metat) {
-		memset(set.defaults->metat, 0, sizeof(set.defaults->metat[0]) * set.defaults->size);
-	}
+	set.clear();
 }
 
 static void dump_macro_set(MACRO_SET & set, std::string & str, const char * prefix) {
@@ -762,6 +753,20 @@ void testing_$F_expand(bool verbose)
 	REQUIRE( expand("$Fdnxf(fileCurRel2)") == "uno/dos.tres" );
 	REQUIRE( expand("$Fpf(fileCurRel2)") == "/home/testing/uno/" );
 
+	REQUIRE( expand("$Fnx(fileDoubleDot)") == "boo..ox" );
+	REQUIRE( expand("$Fn(fileDoubleDot)") == "boo." );
+	REQUIRE( expand("$BASENAME(fileDoubleDot, ..ox)") == "boo" );
+	REQUIRE( expand("$Fn(fileTarGz)") == "stuff.tar" );
+	REQUIRE( expand("$BASENAME(fileTarGz)") == "stuff.tar.gz" );
+	REQUIRE( expand("$BASENAME(fileTarGz,.gz)") == "stuff.tar" );
+	REQUIRE( expand("$BASENAME(fileTarGz, .TAR.GZ)") == "stuff" );
+	REQUIRE( expand("$BASENAME(fileTarGz,.tar)") == "stuff.tar.gz" );
+	REQUIRE( expand("$BASENAME(fileMultiDot,.x)") == "b.o" );
+	REQUIRE( expand("$BASENAME(fileMultiDot,b.o.x)") == "" );
+	REQUIRE( expand("$BASENAME(fileLong, all good men)") == "Now is the time for all good men." );
+	REQUIRE( expand("$BASENAME(fileLong,.)") == "Now is the time for all good men" );
+	REQUIRE( expand("$BASENAME(fileLong, all good men.)") == "Now is the time for " );
+
 	REQUIRE( expand("$Ff(wfileRel)") == "/home/testing/ein\\zwei\\drei.fir" );
 	REQUIRE( expand("$Ffw(wfileRel)") == "\\home\\testing\\ein\\zwei\\drei.fir" );
 	REQUIRE( expand("$Ffu(wfileRel)") == "/home/testing/ein/zwei/drei.fir" );
