@@ -111,7 +111,7 @@ void EmitExpression(unsigned int mode, const char *attr, ExprTree* attr_expr)
 /* is this classad oldstyle, newstyle, or even a job ad? */
 int JadKind(ClassAd *suspect)
 {
-	int cdate;
+	time_t cdate;
 
 	/* determine if I have a user job ad with the new user policy expressions
 		enabled. */
@@ -263,10 +263,10 @@ UserPolicy::AnalyzePolicy(ClassAd & ad, int mode, int state)
 	}
 
 	if (state == RUNNING || state == SUSPENDED) {
-		int birthday;
+		time_t birthday;
 
 		/* Should I perform a hold based on the "running" time of the job? */
-		int allowedJobDuration;
+		time_t allowedJobDuration;
 		if( ad.LookupInteger( ATTR_JOB_ALLOWED_JOB_DURATION, allowedJobDuration ) ) {
 			// Arguably, we should be calling BaseUserPolicy::getJobBirthday()
 			// here, but we don't have access to that here.  This will probably
@@ -274,7 +274,7 @@ UserPolicy::AnalyzePolicy(ClassAd & ad, int mode, int state)
 			// uses ATTR_JOB_START_DATE to determine duration, but using
 			// ATTR_SHADOW_BIRTHDATE was the assignment and is simpler.
 			if( ad.LookupInteger( ATTR_SHADOW_BIRTHDATE, birthday ) ) {
-				if( time(NULL) - birthday >= allowedJobDuration ) {
+				if( time(nullptr) - birthday >= allowedJobDuration ) {
 					m_fire_expr = ATTR_JOB_ALLOWED_JOB_DURATION;
 					m_fire_source = FS_JobDuration;
 					formatstr(m_fire_reason, "The job exceeded allowed job duration of %s", format_time_short(allowedJobDuration));
@@ -288,7 +288,7 @@ UserPolicy::AnalyzePolicy(ClassAd & ad, int mode, int state)
 		 * ATTR_SHADOW_BIRTHDATE, then it's left over from a previous
 		 * execution attempt.
 		 */
-		int allowedExecuteDuration, beganExecuting;
+		time_t allowedExecuteDuration, beganExecuting;
 		if (ad.LookupInteger(ATTR_JOB_ALLOWED_EXECUTE_DURATION, allowedExecuteDuration) &&
 			ad.LookupInteger(ATTR_JOB_CURRENT_START_EXECUTING_DATE, beganExecuting) &&
 			ad.LookupInteger(ATTR_SHADOW_BIRTHDATE, birthday) &&
@@ -296,7 +296,7 @@ UserPolicy::AnalyzePolicy(ClassAd & ad, int mode, int state)
 
 			// We use TransferOutFinished because the shadow only sets
 			// ATTR_JOB_CURRENT_FINISH_TRANSFER_OUTPUT_DATE at job exit.
-			int TransferOutFinished;
+			time_t TransferOutFinished;
 			bool tof = ad.LookupInteger("TransferOutFinished", TransferOutFinished);
 			bool checkpointed = tof && (TransferOutFinished > beganExecuting);
 			if (checkpointed) {
@@ -355,7 +355,15 @@ UserPolicy::AnalyzePolicy(ClassAd & ad, int mode, int state)
 		/* Ignore jobs held at user request */
 		int hold_code = 0;
 		ad.LookupInteger(ATTR_HOLD_REASON_CODE, hold_code);
-		if(hold_code != CONDOR_HOLD_CODE::UserRequest) {
+
+		int num_holds = 0;
+		int sys_max_releases = param_integer("SYSTEM_MAX_RELEASES", -1);
+		if (sys_max_releases == -1) {
+			sys_max_releases = std::numeric_limits<int>::max();
+		}
+		ad.LookupInteger(ATTR_NUM_HOLDS, num_holds);
+
+		if ((num_holds < sys_max_releases) && (hold_code != CONDOR_HOLD_CODE::UserRequest)) {
 			if(AnalyzeSinglePeriodicPolicy(ad, ATTR_PERIODIC_RELEASE_CHECK, POLICY_SYSTEM_PERIODIC_RELEASE, RELEASE_FROM_HOLD, retval)) {
 				return retval;
 			}
@@ -601,7 +609,7 @@ bool UserPolicy::FiringReason(std::string &reason,int &reason_code,int &reason_s
 
 		case FS_JobAttribute:
 			expr_src = "job attribute";
-			exprString = m_fire_unparsed_expr.c_str();
+			exprString = m_fire_unparsed_expr;
 			if (m_fire_expr_val == -1) {
 				reason_code = CONDOR_HOLD_CODE::JobPolicyUndefined;
 			} else {
@@ -625,7 +633,7 @@ bool UserPolicy::FiringReason(std::string &reason,int &reason_code,int &reason_s
 
 		case FS_SystemMacro:
 			expr_src = "system macro";
-			exprString = m_fire_unparsed_expr.c_str();
+			exprString = m_fire_unparsed_expr;
 			if( m_fire_expr_val == -1 ) {
 				reason_code = CONDOR_HOLD_CODE::SystemPolicyUndefined;
 			}
