@@ -33,7 +33,7 @@ class WaitForUserLog {
 
 		void releaseResources();
 		bool isInitialized() { return reader.isInitialized() && trigger.isInitialized(); }
-		ULogEventOutcome readEvent( ULogEvent * & event, int timeout = -1, bool following = true );
+		ULogEventOutcome readEvent( ULogEvent * & event, time_t timeout = -1, bool following = true );
 		const std::string & getFilename() const { return filename; }
 
 		size_t getOffset() const;
