@@ -29,12 +29,12 @@
 #include "condor_uid.h"
 #include "condor_config.h"
 #include "utc_time.h"
-#include "stat_wrapper.h"
 #include "file_lock.h"
 #include "user_log_header.h"
 #include "condor_fsync.h"
 #include "condor_attributes.h"
 #include "CondorError.h"
+#include "directory.h"
 
 #include <string>
 #include <algorithm>
@@ -43,8 +43,6 @@
 // Set to non-zero to enable fine-grained rotation debugging / timing
 #define ROTATION_TRACE	0
 
-static const char SynchDelimiter[] = "...\n";
-
 // Simple class to normalize use of 64 bit ints
 class UserLogInt64_t
 {
@@ -73,7 +71,9 @@ class UserLogFilesize_t : public UserLogInt64_t
 };
 
 
-static int should_use_keyring_sessions() {
+// Note this implementation is a bit different
+// than the one in uids.cpp
+static int should_use_keyring_sessions_for_log() {
 #ifdef LINUX
 	static int UseKeyringSessions = FALSE;
 	static int DidParamForKeyringSessions = FALSE;
@@ -130,7 +130,6 @@ bool getPathToUserLog(const classad::ClassAd *job_ad, std::string &result,
 // ***************************
 WriteUserLog::WriteUserLog()
 {
-	log_file_cache = NULL;
 	Reset( );
 }
 
@@ -139,9 +138,6 @@ WriteUserLog::~WriteUserLog()
 {
 	FreeGlobalResources( true );
 	FreeLocalResources( );
-	if ( m_init_user_ids ) {
-		uninit_user_ids();
-	}
 }
 
 
@@ -150,7 +146,7 @@ WriteUserLog::~WriteUserLog()
 // ***********************************
 
 bool
-WriteUserLog::initialize(const ClassAd &job_ad, bool init_user)
+WriteUserLog::initialize(const ClassAd &job_ad)
 {
 	int cluster = -1;
 	int proc = -1;
@@ -159,24 +155,13 @@ WriteUserLog::initialize(const ClassAd &job_ad, bool init_user)
 
 	m_global_disable = false;
 
-	if ( init_user ) {
-		std::string owner;
-		std::string domain;
-
-		job_ad.LookupString(ATTR_OWNER, owner);
-		job_ad.LookupString(ATTR_NT_DOMAIN, domain);
-
-		uninit_user_ids();
-		if ( ! init_user_ids(owner.c_str(), domain.c_str()) ) {
-			if ( ! domain.empty()) { owner += "@"; owner += domain; }
-			dprintf(D_ALWAYS,
-				"WriteUserLog::initialize: init_user_ids(%s) failed!\n", owner.c_str());
-			return false;
-		}
-		m_init_user_ids = true;
-	}
 	m_set_user_priv = true;
 
+	if (can_switch_ids() && !user_ids_are_inited()) {
+		dprintf(D_ERROR, "WriteUserLog: user ids not initialized\n");
+		return false;
+	}
+
 	TemporaryPrivSentry temp_priv;
 
 	// switch to user priv
@@ -228,18 +213,6 @@ WriteUserLog::initialize( const std::vector<const char *>& file, int c, int p, i
 		for(std::vector<const char*>::const_iterator it = file.begin();
 				it != file.end(); ++it) {
 
-			if (log_file_cache != NULL) {
-				dprintf(D_FULLDEBUG, "WriteUserLog::initialize: looking up log file %s in cache\n", *it);
-				log_file_cache_map_t::iterator f(log_file_cache->find(*it));
-				if (f != log_file_cache->end()) {
-					dprintf(D_FULLDEBUG, "WriteUserLog::initialize: found log file %s in cache, re-using\n", *it);
-					logs.push_back(f->second);
-					logs.back()->refset.insert(std::make_pair(c,p));
-					first = false;
-					continue;
-				}
-			}
-
 			log_file* log = new log_file(*it);
 			if (first) {
 				// The first entry in the vector is the user's userlog, which we rarely want to fsync
@@ -260,7 +233,7 @@ WriteUserLog::initialize( const std::vector<const char *>& file, int c, int p, i
 					log->path.c_str());
 				logs.push_back(log);
 
-				// setting the flag m_init_user_ids will cause the logging code in doWriteEvent()
+				// setting the flag m_set_user_priv will cause the logging code in doWriteEvent()
 				// to switch to PRIV_USER every time it does a write (as opposed to PRIV_CONDOR).
 				// even though the file is already open, this is necessary because AFS needs access
 				// to the user token on every write(), whereas other filesystems typically only need
@@ -274,22 +247,14 @@ WriteUserLog::initialize( const std::vector<const char *>& file, int c, int p, i
 				// is called in the destructor of the log_file object, we need to set a flag inside
 				// that object as well. -zmiller
 				//
-				if(should_use_keyring_sessions()) {
+				if(should_use_keyring_sessions_for_log()) {
 					dprintf(D_FULLDEBUG, "WriteUserLog::initialize: current priv is %i\n", get_priv_state());
 					if(get_priv_state() == PRIV_USER || get_priv_state() == PRIV_USER_FINAL) {
 						dprintf(D_FULLDEBUG, "WriteUserLog::initialize: opened %s in priv state %i\n", log->path.c_str(), get_priv_state());
-						// TODO Shouldn't set m_init_user_ids here
-						m_init_user_ids = true;
 						m_set_user_priv = true;
 						log->set_user_priv_flag(true);
 					}
 				}
-
-				if (log_file_cache != NULL) {
-					dprintf(D_FULLDEBUG, "WriteUserLog::initialize: caching log file %s\n", *it);
-					(*log_file_cache)[*it] = log;
-					log->refset.insert(std::make_pair(c,p));
-				}
 			}
 		}
 	}
@@ -373,7 +338,7 @@ WriteUserLog::Configure( bool force )
 	if ( NULL == m_global_path ) {
 		return true;
 	}
-	m_global_stat = new StatWrapper( m_global_path );
+	m_global_stat = {};
 	m_global_state = new WriteUserLogState( );
 
 
@@ -442,7 +407,6 @@ WriteUserLog::Reset( void )
 {
 	m_initialized = false;
 	m_configured = false;
-	m_init_user_ids = false;
 	m_set_user_priv = false;
 
 	m_cluster = -1;
@@ -453,7 +417,6 @@ WriteUserLog::Reset( void )
 
     freeLogs();
    	logs.clear();
-    log_file_cache = NULL;
 
 	m_enable_locking = true;
 	m_skip_fsync_this_event = false;
@@ -461,7 +424,7 @@ WriteUserLog::Reset( void )
 	m_global_path = NULL;
 	m_global_fd = -1;
 	m_global_lock = NULL;
-	m_global_stat = NULL;
+	m_global_stat = {};
 	m_global_state = NULL;
 
 	m_rotation_lock = NULL;
@@ -508,10 +471,6 @@ WriteUserLog::FreeGlobalResources( bool final )
 		free( m_global_id_base );
 		m_global_id_base = NULL;
 	}
-	if (m_global_stat != NULL) {
-		delete m_global_stat;
-		m_global_stat = NULL;
-	}
 	if (m_global_state != NULL) {
 		delete m_global_state;
 		m_global_state = NULL;
@@ -603,8 +562,6 @@ WriteUserLog::log_file::~log_file()
 }
 
 void WriteUserLog::freeLogs() {
-    // we do this only if local log files aren't being cached
-    if (log_file_cache != NULL) return;
     for (std::vector<log_file*>::iterator j(logs.begin());  j != logs.end();  ++j) {
         delete *j;
     }
@@ -718,6 +675,31 @@ WriteUserLog::openFile(
 	mode_t mode = 0664;
 	fd = safe_open_wrapper_follow( file, flags, mode );
 	if( fd < 0 ) {
+		// See if there's a missing intermediate directory
+		if ( errno == ENOENT ) {
+			std::string dir = file;
+			// Remove the filename from the path
+			size_t pos = dir.find_last_of( "/\\" );
+			if ( pos != std::string::npos ) {
+				dir.erase( pos );
+			}
+
+			if (!mkdir_and_parents_if_needed(dir.c_str(), 0755, PRIV_USER)) {
+				dprintf( D_ALWAYS,
+						 "WriteUserLog::initialize: "
+							 "mkdir_and_parents_if_needed(\"%s\") failed - errno %d (%s)\n",
+						 dir.c_str(),
+						 errno,
+						 strerror(errno) );
+				return false;
+			}
+			// Try to open the file again
+			fd = safe_open_wrapper_follow( file, flags, mode );
+		}
+	}
+
+	// check again 
+	if (fd < 0 ) {
 		dprintf( D_ALWAYS,
 					"WriteUserLog::initialize: "
 						"safe_open_wrapper(\"%s\") failed - errno %d (%s)\n",
@@ -783,9 +765,9 @@ WriteUserLog::openGlobalLog( bool reopen, const UserLogHeader &header )
 		return false;
 	}
 
-	StatWrapper		statinfo;
-	if (  ( !(statinfo.Stat(m_global_path))    )  &&
-		  ( 0 == statinfo.GetBuf()->st_size )  )  {
+	struct stat statinfo;
+	if (  !(stat(m_global_path, &statinfo))  &&
+		  ( 0 == statinfo.st_size )  )  {
 
 		// Generate a header event
 		WriteUserLogHeader writer( header );
@@ -827,7 +809,7 @@ WriteUserLog::openGlobalLog( bool reopen, const UserLogHeader &header )
 					 "WriteUserLog Failed to update global stat after header write\n" );
 		}
 		else {
-			m_global_state->Update( *m_global_stat );
+			m_global_state->Update( m_global_stat );
 		}
 	}
 
@@ -881,11 +863,11 @@ WriteUserLog::checkGlobalLogRotation( void )
 	ReadUserLogHeader	header_reader;
 
 	// New file?  Another process rotated it
-	if ( m_global_state->isNewFile(*m_global_stat) ) {
+	if ( m_global_state->isNewFile(m_global_stat) ) {
 		globalLogRotated( header_reader );
 		return true;
 	}
-	m_global_state->Update( *m_global_stat );
+	m_global_state->Update( m_global_stat );
 
 	// Less than the size limit -- nothing to do
 	if ( !m_global_state->isOverSize(m_global_max_filesize) ) {
@@ -910,12 +892,12 @@ WriteUserLog::checkGlobalLogRotation( void )
 	}
 
 	// New file?  Another process rotated it
-	if ( m_global_state->isNewFile(*m_global_stat) ) {
+	if ( m_global_state->isNewFile(m_global_stat) ) {
 		m_rotation_lock->release( );
 		globalLogRotated( header_reader );
 		return true;
 	}
-	m_global_state->Update( *m_global_stat );
+	m_global_state->Update( m_global_stat );
 
 	// Less than the size limit -- nothing to do
 	// Note: This should never be true, but checking just in case
@@ -928,12 +910,12 @@ WriteUserLog::checkGlobalLogRotation( void )
 	// Now, we have the rotation lock *and* the file is over the limit
 	// Let's get down to the business of rotating it
 	filesize_t	current_filesize = 0;
-	StatWrapper	sbuf;
-	if ( sbuf.Stat( m_global_fd ) ) {
+	struct stat sbuf;
+	if ( fstat(m_global_fd, &sbuf) ) {
 		dprintf( D_ALWAYS, "WriteUserLog Failed to stat file handle\n" );
 	}
 	else {
-		current_filesize = sbuf.GetBuf()->st_size;
+		current_filesize = sbuf.st_size;
 	}
 
 
@@ -945,10 +927,11 @@ WriteUserLog::checkGlobalLogRotation( void )
 
 #if ROTATION_TRACE
 	{
-		StatWrapper	swrap( m_global_path );
+		struct stat sinfo = {};
+		stat(m_global_path, &sinfo);
 		double start_time = condor_gettimestamp_double();
 		dprintf( D_FULLDEBUG, "Rotating inode #%ld @ %.6f (stat @ %.6f)\n",
-				 (long)swrap.GetBuf()->st_ino, start_time,
+				 (long)sinfo.st_ino, start_time,
 				 stat_time );
 		m_global_lock->display();
 	}
@@ -1099,10 +1082,7 @@ WriteUserLog::checkGlobalLogRotation( void )
 bool
 WriteUserLog::updateGlobalStat( void )
 {
-	if ( (NULL == m_global_stat) || (m_global_stat->Stat()) ) {
-		return false;
-	}
-	if ( m_global_stat->IsBufValid() == false ) {
+	if ( (NULL == m_global_path) || stat(m_global_path, &m_global_stat) ) {
 		return false;
 	}
 	return true;
@@ -1111,7 +1091,7 @@ WriteUserLog::updateGlobalStat( void )
 bool
 WriteUserLog::getGlobalLogSize( unsigned long &size, bool use_fd )
 {
-	StatWrapper	stat;
+	struct stat statinfo;
 	if ( m_global_close && m_global_fd < 0 ) {
 		use_fd = false;
 	}
@@ -1119,16 +1099,16 @@ WriteUserLog::getGlobalLogSize( unsigned long &size, bool use_fd )
 		if ( m_global_fd < 0 ) {
 			return false;
 		}
-		if ( stat.Stat(m_global_fd) ) {
+		if ( fstat(m_global_fd, &statinfo) ) {
 			return false;
 		}
 	}
 	else {
-		if ( stat.Stat(m_global_path) ) {
+		if ( stat(m_global_path, &statinfo) ) {
 			return false;
 		}
 	}
-	size = (unsigned long) stat.GetBuf()->st_size;
+	size = (unsigned long) statinfo.st_size;
 	return true;
 }
 
@@ -1146,7 +1126,7 @@ WriteUserLog::globalLogRotated( ReadUserLogHeader &reader )
 			m_global_state->Clear( );
 		}
 		else {
-			m_global_state->Update( *m_global_stat );
+			m_global_state->Update( m_global_stat );
 		}
 	}
 	return true;
@@ -1168,8 +1148,8 @@ WriteUserLog::doRotation( const char *path, int &fd,
 			std::string old1( path );
 			formatstr_cat(old1, ".%d", i-1 );
 
-			StatWrapper	s( old1 );
-			if ( 0 == s.GetRc() ) {
+			struct stat s;
+			if ( 0 == stat(old1.c_str(), &s) ) {
 				std::string old2( path );
 				formatstr_cat(old2, ".%d", i );
 				if (rename( old1.c_str(), old2.c_str() )) {
