@@ -160,7 +160,7 @@ void Defrag::config()
 	}
 	else {
 		time_t now = time(NULL);
-		int first_time = 0;
+		time_t first_time = 0;
 		if( m_last_poll != 0 && now-m_last_poll < m_polling_interval && m_last_poll <= now ) {
 			first_time = m_polling_interval - (now-m_last_poll);
 		}
@@ -300,18 +300,6 @@ void Defrag::stop()
 	}
 }
 
-static int StartdSortFunc(ClassAd *ad1,ClassAd *ad2,void *data)
-{
-	ClassAd *rank_ad = (ClassAd *)data;
-
-	float rank1 = 0;
-	float rank2 = 0;
-	EvalFloat(ATTR_RANK,rank_ad,ad1,rank1);
-	EvalFloat(ATTR_RANK,rank_ad,ad2,rank2);
-
-	return rank1 > rank2;
-}
-
 void Defrag::validateExpr(char const *constraint,char const *constraint_source)
 {
 	ExprTree *requirements = NULL;
@@ -336,7 +324,7 @@ bool Defrag::queryMachines(char const *constraint,char const *constraint_source,
 		// if no projection supplied, just get the Name attribute
 		startdQuery.setDesiredAttrs("Name");
 	}
-	startdQuery.addExtraAttribute(ATTR_SEND_PRIVATE_ATTRIBUTES, "true");
+	startdQuery.requestPrivateAttrs();
 
 	CollectorList* collects = daemonCore->getCollectorList();
 	ASSERT( collects );
@@ -399,17 +387,17 @@ Defrag::queryDrainingCost()
 	ClassAd *startd_ad;
 	while( (startd_ad=startdAds.Next()) ) {
 		int unclaimed = 0;
-		int badput = 0;
-		int start_time = 0;
+		time_t badput = 0;
+		time_t start_time = 0;
 		int cpus = 0;
-		int last_heard_from = 0;
+		time_t last_heard_from = 0;
 		startd_ad->LookupInteger(ATTR_TOTAL_MACHINE_DRAINING_UNCLAIMED_TIME,unclaimed);
 		startd_ad->LookupInteger(ATTR_TOTAL_MACHINE_DRAINING_BADPUT,badput);
 		startd_ad->LookupInteger(ATTR_DAEMON_START_TIME,start_time);
 		startd_ad->LookupInteger(ATTR_LAST_HEARD_FROM,last_heard_from);
 		startd_ad->LookupInteger(ATTR_TOTAL_CPUS,cpus);
 
-		int age = last_heard_from - start_time;
+		time_t age = last_heard_from - start_time;
 		if( last_heard_from == 0 || start_time == 0 || age <= 0 ) {
 			continue;
 		}
@@ -547,9 +535,9 @@ void Defrag::slotNameToDaemonName(std::string const &name,std::string &machine)
 // n is a number per period.  If we are partly through
 // the interval, make n be in proportion to how much
 // is left.
-static int prorate(int n,int period_elapsed,int period,int granularity, double* real_answer)
+static int prorate(int n,time_t period_elapsed,time_t period,time_t granularity, double* real_answer)
 {
-	int time_remaining = period-period_elapsed;
+	time_t time_remaining = period-period_elapsed;
 	double frac = ((double)time_remaining)/period;
 
 		// Add in maximum time in this interval that could have been
@@ -592,7 +580,7 @@ void Defrag::poll_cancel(MachineSet &cancelled_machines)
 	}
 
 	startdAds.Shuffle();
-	startdAds.Sort(StartdSortFunc,&m_rank_ad);
+	startdAds.Sort(SlotRankSortFunc,&m_rank_ad);
 
 	startdAds.Open();
 
@@ -825,7 +813,7 @@ void Defrag::poll( int /* timerID */ )
 	}
 
 	startdAds.Shuffle();
-	startdAds.Sort(StartdSortFunc,&m_rank_ad);
+	startdAds.Sort(SlotRankSortFunc,&m_rank_ad);
 
 	startdAds.Open();
 	int num_drained = 0;
@@ -892,37 +880,37 @@ Defrag::drain_this(const ClassAd &startd_ad)
 
 	DCStartd startd( &startd_ad );
 
-	int graceful_completion = 0;
+	time_t graceful_completion = 0;
 	startd_ad.LookupInteger(ATTR_EXPECTED_MACHINE_GRACEFUL_DRAINING_COMPLETION,graceful_completion);
-	int quick_completion = 0;
+	time_t quick_completion = 0;
 	startd_ad.LookupInteger(ATTR_EXPECTED_MACHINE_QUICK_DRAINING_COMPLETION,quick_completion);
-	int graceful_badput = 0;
+	time_t graceful_badput = 0;
 	startd_ad.LookupInteger(ATTR_EXPECTED_MACHINE_GRACEFUL_DRAINING_BADPUT,graceful_badput);
-	int quick_badput = 0;
+	time_t quick_badput = 0;
 	startd_ad.LookupInteger(ATTR_EXPECTED_MACHINE_QUICK_DRAINING_BADPUT,quick_badput);
 
 	time_t now = time(NULL);
 	std::string draining_check_expr;
 	double badput_growth_tolerance = 1.25; // for now, this is hard-coded
-	int negligible_badput = 1200;
-	int negligible_deadline_slippage = 1200;
+	time_t negligible_badput = 1200;
+	time_t negligible_deadline_slippage = 1200;
 	if( m_draining_schedule <= DRAIN_GRACEFUL ) {
-		dprintf(D_ALWAYS,"Expected draining completion time is %ds; expected draining badput is %d cpu-seconds\n",
-				(int)(graceful_completion-now),graceful_badput);
-		formatstr(draining_check_expr,"%s <= %d && %s <= %d",
+		dprintf(D_ALWAYS,"Expected draining completion time is %llds; expected draining badput is %lld cpu-seconds\n",
+				(long long)(graceful_completion-now),(long long) graceful_badput);
+		formatstr(draining_check_expr,"%s <= %lld && %s <= %lld",
 				ATTR_EXPECTED_MACHINE_GRACEFUL_DRAINING_COMPLETION,
-				graceful_completion + negligible_deadline_slippage,
+				(long long) graceful_completion + negligible_deadline_slippage,
 				ATTR_EXPECTED_MACHINE_GRACEFUL_DRAINING_BADPUT,
-				(int)(badput_growth_tolerance*graceful_badput) + negligible_badput);
+				(long long)(badput_growth_tolerance*graceful_badput) + negligible_badput);
 	}
 	else { // DRAIN_FAST and DRAIN_QUICK are effectively equivalent here
-		dprintf(D_ALWAYS,"Expected draining completion time is %ds; expected draining badput is %d cpu-seconds\n",
-				(int)(quick_completion-now),quick_badput);
-		formatstr(draining_check_expr,"%s <= %d && %s <= %d",
+		dprintf(D_ALWAYS,"Expected draining completion time is %llds; expected draining badput is %lld cpu-seconds\n",
+				(long long)(quick_completion-now),(long long)quick_badput);
+		formatstr(draining_check_expr,"%s <= %lld && %s <= %lld",
 				ATTR_EXPECTED_MACHINE_QUICK_DRAINING_COMPLETION,
-				quick_completion + negligible_deadline_slippage,
+				(long long) quick_completion + negligible_deadline_slippage,
 				ATTR_EXPECTED_MACHINE_QUICK_DRAINING_BADPUT,
-				(int)(badput_growth_tolerance*quick_badput) + negligible_badput);
+				(long long) (badput_growth_tolerance*quick_badput) + negligible_badput);
 	}
 
 	std::string drain_reason;
@@ -981,7 +969,7 @@ Defrag::publish(ClassAd *ad)
 void
 Defrag::updateCollector( int /* timerID */ ) {
 	publish(&m_public_ad);
-	daemonCore->sendUpdates(UPDATE_AD_GENERIC, &m_public_ad);
+	daemonCore->sendUpdates(UPDATE_AD_GENERIC, &m_public_ad, nullptr, true);
 }
 
 void
