@@ -22,6 +22,7 @@
 #include "condor_config.h"
 #include "amazongahp_common.h"
 #include "amazonCommands.h"
+#include <vector>
 
 static std::string amazon_proxy_host;
 static int amazon_proxy_port;
@@ -392,7 +393,7 @@ create_success_result( int req_id, std::vector<std::string> *result_list)
 		index_count = result_list->size();
 	}
 
-	const char *tmp_result[index_count + 1];
+	std::vector<const char *> tmp_result(index_count + 1);
 
 	tmp_result[0] = AMAZON_COMMAND_SUCCESS_OUTPUT;
 
@@ -404,7 +405,7 @@ create_success_result( int req_id, std::vector<std::string> *result_list)
 		}
 	}
 
-	return create_output_string (req_id, tmp_result, i);
+	return create_output_string (req_id, tmp_result.data(), i);
 }
 
 std::string
