@@ -1,3 +1,4 @@
+// The caller owns the returned pointer.
 ExprTree * convert_python_to_classad_exprtree(PyObject * py_v);
 
 static PyObject *
@@ -35,7 +36,7 @@ _classad_init_from_string( PyObject *, PyObject * args ) {
     classad::ClassAd * result = parser.ParseClassAd(from_string);
     if( result == NULL ) {
         // This was ClassAdParseError in version 1.
-        PyErr_SetString( PyExc_SyntaxError, "Unable to parse string into a ClassAd." );
+        PyErr_SetString( PyExc_ClassAdException, "Unable to parse string into a ClassAd." );
         return NULL;
     }
 
@@ -203,7 +204,7 @@ convert_classad_value_to_python(classad::Value & v) {
                     if(! (*i)->Evaluate( v )) {
                         Py_DecRef(list);
                         // This was ClassAdEvaluationError in version 1.
-                        PyErr_SetString( PyExc_RuntimeError, "Failed to evaluate convertible expression" );
+                        PyErr_SetString( PyExc_ClassAdException, "Failed to evaluate convertible expression" );
                         return NULL;
                     }
 
@@ -232,7 +233,7 @@ convert_classad_value_to_python(classad::Value & v) {
 
         default:
             // This was ClassAdEnumError in version 1.
-            PyErr_SetString( PyExc_RuntimeError, "Unknown ClassAd value type" );
+            PyErr_SetString( PyExc_ClassAdException, "Unknown ClassAd value type" );
             return NULL;
     }
 }
@@ -262,7 +263,7 @@ _classad_get_item( PyObject *, PyObject * args ) {
         classad::Value v;
         if(! expr->Evaluate( v )) {
             // This was ClassAdEvaluationError in version 1.
-            PyErr_SetString( PyExc_RuntimeError, "Failed to evaluate convertible expression" );
+            PyErr_SetString( PyExc_ClassAdException, "Failed to evaluate convertible expression" );
             return NULL;
         }
 
@@ -296,6 +297,7 @@ _classad_del_item( PyObject *, PyObject * args ) {
 }
 
 
+// The caller owns the returned pointer.
 ExprTree *
 convert_python_to_classad_exprtree(PyObject * py_v) {
     if( py_v == Py_None ) {
@@ -329,21 +331,19 @@ convert_python_to_classad_exprtree(PyObject * py_v) {
         PyObject * py_v_int = PyNumber_Long(py_v);
         if( py_v_int == NULL ) {
             // This was ClassAdInternalError in version 1.
-            PyErr_SetString( PyExc_RuntimeError, "Unknown ClassAd value type." );
+            PyErr_SetString( PyExc_ClassAdException, "Unknown ClassAd value type." );
             return NULL;
         }
         long lv = PyLong_AsLong(py_v_int);
         Py_DecRef(py_v_int);
 
         if( lv == 1<<0 ) {
-            v.SetErrorValue();
-            return classad::Literal::MakeLiteral(v);
+            return classad::Literal::MakeError();
         } else if( lv == 1<<1 ) {
-            v.SetUndefinedValue();
-            return classad::Literal::MakeLiteral(v);
+            return classad::Literal::MakeUndefined();
         } else {
             // This was ClassAdInternalError in version 1.
-            PyErr_SetString( PyExc_RuntimeError, "Unknown ClassAd value type." );
+            PyErr_SetString( PyExc_ClassAdException, "Unknown ClassAd value type." );
             return NULL;
         }
     }
@@ -378,8 +378,7 @@ convert_python_to_classad_exprtree(PyObject * py_v) {
     }
 
     if( PyBool_Check(py_v) ) {
-        v.SetBooleanValue( py_v == Py_True );
-        return classad::Literal::MakeLiteral(v);
+        return classad::Literal::MakeBool(py_v == Py_True);
     }
 
     if( PyUnicode_Check(py_v) ) {
@@ -391,10 +390,10 @@ convert_python_to_classad_exprtree(PyObject * py_v) {
             // PyBytes_AsStringAndSize() has already set an exception for us.
             return NULL;
         }
-        v.SetStringValue( buffer, size );
+        classad::ExprTree* tree = classad::Literal::MakeString(buffer, size);
 
         Py_DecRef(py_bytes);
-        return classad::Literal::MakeLiteral(v);
+        return tree;
     }
 
     if( PyBytes_Check(py_v) ) {
@@ -404,20 +403,17 @@ convert_python_to_classad_exprtree(PyObject * py_v) {
             // PyBytes_AsStringAndSize() has already set an exception for us.
             return NULL;
         }
-        v.SetStringValue( buffer, size );
-        return classad::Literal::MakeLiteral(v);
+        return classad::Literal::MakeString(buffer, size);
     }
 
     if( PyLong_Check(py_v) ) {
         // This was ...AsLong() in version 1, but we'll assume nobody
         // was depending on that truncation.
-        v.SetIntegerValue(PyLong_AsLongLong(py_v));
-        return classad::Literal::MakeLiteral(v);
+        return classad::Literal::MakeInteger(PyLong_AsLongLong(py_v));
     }
 
     if( PyFloat_Check(py_v) ) {
-        v.SetRealValue(PyFloat_AsDouble(py_v));
-        return classad::Literal::MakeLiteral(v);
+        return classad::Literal::MakeReal(PyFloat_AsDouble(py_v));
     }
 
     // PyDict_Check() is not part of the stable ABI.
@@ -441,6 +437,7 @@ convert_python_to_classad_exprtree(PyObject * py_v) {
                 PyObject * value = NULL;
                 if(! PyArg_ParseTuple( item, "zO", &key, &value)) {
                     // PyArg_ParseTuple() has already set an exception for us.
+                    delete c;
                     return NULL;
                 }
 
@@ -451,6 +448,7 @@ convert_python_to_classad_exprtree(PyObject * py_v) {
 
             return c;
         }
+        delete c;
     }
 
     PyObject * iter = PyObject_GetIter(py_v);
@@ -470,7 +468,7 @@ convert_python_to_classad_exprtree(PyObject * py_v) {
     }
 
     // This was ClassAdValueError in version 1.
-    PyErr_SetString( PyExc_RuntimeError, "Unable to convert Python object to a ClassAd expression." );
+    PyErr_SetString( PyExc_ClassAdException, "Unable to convert Python object to a ClassAd expression." );
     return NULL;
 }
 
@@ -488,11 +486,14 @@ _classad_set_item( PyObject *, PyObject * args ) {
     }
 
     auto * classAd = (ClassAd *)handle->t;
+
     ExprTree * v = convert_python_to_classad_exprtree(value);
+    if( v == NULL ) {
+        // convert_python_to_classad_exprtree() has already set an exception for us.
+        return NULL;
+    }
     if(! classAd->Insert(key, v)) {
-        if(! PyErr_Occurred()) {
-            PyErr_SetString(PyExc_AttributeError, key);
-        }
+        PyErr_SetString(PyExc_ClassAdException, "Insert(key, v) failed");
         return NULL;
     }
 
@@ -572,7 +573,6 @@ _classad_parse_next( PyObject *, PyObject * args ) {
         return NULL;
     }
 
-
     // The 'auto' type from the ClassAdFileParseType and the 'auto' type
     // from version 1 both have weird, undocumented semantics, and they
     // disagree.  The 'auto' from version 1 only distinguished between
@@ -587,12 +587,9 @@ _classad_parse_next( PyObject *, PyObject * args ) {
         pType = isOldAd(from_string) ? ClassAdFileParseType::Parse_long : ClassAdFileParseType::Parse_new;
     }
 
+    std::string_view instr(from_string);
+    size_t from_string_length = instr.size();
 
-    size_t from_string_length = strlen(from_string);
-
-    // On Mac, fmemopen() returns NULL if the buffer is 0 bytes long.  Since
-    // there's clearly not another ad in a 0-byte buffer, we're done iterating.
-    //
     // This looks silly -- we could certainly check if the returned offset
     // was the whole buffer -- but doing it this way simplifies the generator.
     if( from_string_length == 0 ) {
@@ -600,33 +597,19 @@ _classad_parse_next( PyObject *, PyObject * args ) {
         return Py_BuildValue("Oi", Py_None, 0);
     }
 
-
-#if defined(WINDOWS)
-    FILE * file = NULL;
-    // The Windows C API does not appear to have a race-free way to create
-    // temporary files that aren't in the root directory, which is insane.
-    auto e = tmpfile_s(& file);
-
-    if( e != 0 || file == NULL ) {
-        PyErr_SetString(PyExc_RuntimeError, "Unable to open temporary file.");
-        return NULL;
+    // remove leading whitespace to avoid confusing the auto parser
+    // or getting back an empty ad without reading the whole buffer
+    // but keep track of how many prefix chars we skipped, since
+    // we need to return the number of chars we parsed.
+    long offset = 0;
+    while ( ! instr.empty() && isspace(instr.front())) {
+        instr.remove_prefix(1);
+        ++offset;
     }
 
-    fprintf( file, "%s\0", from_string );
-    rewind( file );
-#else
-    FILE * file = fmemopen( const_cast<char *>(from_string), from_string_length, "r" );
-
-    if( file == NULL ) {
-        // This was a ClassAdParseError in version 1.
-        PyErr_SetString(PyExc_ValueError, "Unable to parse input stream into a ClassAd.");
-        return NULL;
-    }
-#endif /* WINDOWS */
-
     CondorClassAdFileIterator ccafi;
-    if(! ccafi.begin( file, false, pType )) {
-        fclose( file );
+    CompatStringViewLexerSource lexsrc(instr,0);
+    if(! ccafi.begin(&lexsrc, false, pType)) {
 
         // This was a ClassAdParseError in version 1.
         PyErr_SetString(PyExc_ValueError, "Unable to parse input stream into a ClassAd.");
@@ -635,8 +618,7 @@ _classad_parse_next( PyObject *, PyObject * args ) {
 
     ClassAd * result = new ClassAd();
     int numAttrs = ccafi.next( * result );
-    long offset = ftell( file );
-    fclose( file );
+    offset += lexsrc.GetCurrentLocation(); // calculate end offset relative to from_string
 
     if( numAttrs <= 0 ) {
         if( offset == (long)from_string_length ) {
@@ -651,7 +633,7 @@ _classad_parse_next( PyObject *, PyObject * args ) {
 
 
     auto py_class_ad = py_new_classad2_classad(result);
-    return Py_BuildValue("Ol", py_class_ad, offset);
+    return Py_BuildValue("Nl", py_class_ad, offset);
 }
 
 
@@ -679,7 +661,8 @@ _classad_parse_next_fd( PyObject *, PyObject * args ) {
     // We _must_ use unbuffered reads; otherwise the underyling FD's
     // position will be wrong when it returns to Python.
     if(setvbuf( file, NULL, _IONBF, 0 ) != 0) {
-        PyErr_SetString(PyExc_RuntimeError, "setvbuf() failed");
+        PyErr_SetString(PyExc_ClassAdException, "setvbuf() failed");
+		fclose(file);
         return NULL;
     }
 
@@ -720,9 +703,7 @@ _classad_quote( PyObject *, PyObject * args ) {
     }
 
 
-    classad::Value v;
-    v.SetStringValue(from_string);
-    classad::ExprTree * expr = classad::Literal::MakeLiteral(v);
+    classad::ExprTree * expr = classad::Literal::MakeString(from_string);
     classad::ClassAdUnParser sink;
 
     std::string result;
@@ -746,15 +727,10 @@ _classad_unquote( PyObject *, PyObject * args ) {
 
 
     classad::ClassAdParser parser;
-    classad::ExprTree * expr = NULL;
-    if(! parser.ParseExpression(from_string, expr, true)) {
-        // This was a ClassAdParseError in version 1.
-        PyErr_SetString(PyExc_ValueError, "Invalid string to unquote");
-        return NULL;
-    }
+    classad::ExprTree * expr = parser.ParseExpression(from_string, true);
     if( expr == NULL ) {
         // This was a ClassAdParseError in version 1.
-        PyErr_SetString(PyExc_ValueError, "String does not parse to a ClassAd string literal");
+        PyErr_SetString(PyExc_ValueError, "Invalid string to unquote");
         return NULL;
     }
     if( dynamic_cast<classad::Literal *>(expr) == nullptr) {
