@@ -9,7 +9,9 @@ _collector_init( PyObject *, PyObject * args ) {
 	PyObject * self = NULL;
 	PyObject_Handle * handle = NULL;
 	const char * pool = NULL;
-	if(! PyArg_ParseTuple( args, "OOz", & self, (PyObject **)& handle, & pool )) {
+	const char * token = nullptr;
+	CollectorList * collector_list = nullptr;
+	if(! PyArg_ParseTuple( args, "OOzz", & self, (PyObject **)& handle, & pool, & token )) {
 		// PyArg_ParseTuple() has already set an exception for us.
 		return NULL;
 	}
@@ -17,14 +19,16 @@ _collector_init( PyObject *, PyObject * args ) {
 	handle->f = [](void *& v) { dprintf( D_PERF_TRACE, "[unconstructed CollectorList]\n" ); if(v != NULL){ dprintf(D_ALWAYS, "Error!  Unconstructed Collector has non-NULL handle %p\n", v); } };
 
 	if( pool == NULL || strlen(pool) == 0 ) {
-		handle->t = (void *) CollectorList::create();
+		collector_list = CollectorList::create();
+		handle->t = (void *) collector_list;
 
 		if( PyObject_SetAttrString( self, "default", Py_True ) != 0 ) {
 			// PyObject_setAttrString() has already set an exception for us.
 			return NULL;
 		}
 	} else {
-		handle->t = (void *) CollectorList::create(pool);
+		collector_list = CollectorList::create(pool);
+		handle->t = (void *) collector_list;
 
 		if( PyObject_SetAttrString( self, "default", Py_False ) != 0 ) {
 			// PyObject_setAttrString() has already set an exception for us.
@@ -32,6 +36,11 @@ _collector_init( PyObject *, PyObject * args ) {
 		}
 	}
 
+	if (token) {
+		for (auto& collector: collector_list->getList()) {
+			collector->setPreferredToken(token);
+		}
+	}
 
 	handle->f = [](void *& v) { dprintf( D_PERF_TRACE, "[CollectorList]\n" ); delete (CollectorList *)v; v = NULL; };
 	Py_RETURN_NONE;
@@ -118,27 +127,27 @@ _collector_query( PyObject *, PyObject * args ) {
 		switch (result) {
 			case Q_COMMUNICATION_ERROR:
 				// This was HTCondorIOError in version 1.
-				PyErr_SetString( PyExc_RuntimeError, "Failed communication with collector." );
+				PyErr_SetString( PyExc_HTCondorException, "Failed communication with collector." );
 				return NULL;
 
 			case Q_INVALID_QUERY:
 				// This was HTCondorIOError in version 1.
-				PyErr_SetString( PyExc_RuntimeError, "Invalid query." );
+				PyErr_SetString( PyExc_HTCondorException, "Invalid query." );
 				return NULL;
 
 			case Q_NO_COLLECTOR_HOST:
 				// This was HTCondorLocateError in version 1.
-				PyErr_SetString( PyExc_RuntimeError, "Unable to determine collector host." );
+				PyErr_SetString( PyExc_HTCondorException, "Unable to determine collector host." );
 				return NULL;
 
-			// In version 1, we believe these errors were impossible.
+			// In version 1, we believed these errors were impossible.
 			case Q_PARSE_ERROR:
 			case Q_MEMORY_ERROR:
 			case Q_INVALID_CATEGORY:
 
 			default:
 				// This was HTCondorInternalError in version 1.
-				PyErr_SetString( PyExc_RuntimeError, "Unknown error from collector query." );
+				PyErr_SetString( PyExc_HTCondorException, "Unknown error from collector query." );
 				return NULL;
 		}
 	}
@@ -161,7 +170,7 @@ _collector_locate_local( PyObject *, PyObject * args ) {
 	if( local.locate() ) {
 		ClassAd * locateOnlyAd = local.locationAd();
 		if( locateOnlyAd == NULL ) {
-			PyErr_SetString( PyExc_RuntimeError, "Found local daemon but failed to acquire its ad." );
+			PyErr_SetString( PyExc_HTCondorException, "Found local daemon but failed to acquire its ad." );
 			return NULL;
 		}
 
@@ -170,7 +179,7 @@ _collector_locate_local( PyObject *, PyObject * args ) {
 		return pyClassAd;
 	} else {
 		// This was HTCondorLocateError in version 1.
-		PyErr_SetString( PyExc_RuntimeError, "Unable to locate local daemon." );
+		PyErr_SetString( PyExc_HTCondorException, "Unable to locate local daemon." );
 		return NULL;
 	}
 }
@@ -217,7 +226,7 @@ _collector_advertise( PyObject *, PyObject * args ) {
     for( auto & collector : collectorList->getList() ) {
 		if(! collector->locate()) {
 			// This was HTCondorLocateError in version 1.
-			PyErr_SetString( PyExc_RuntimeError, "Unable to locate collector." );
+			PyErr_SetString( PyExc_HTCondorException, "Unable to locate collector." );
 			return NULL;
 		}
 
@@ -260,7 +269,7 @@ _collector_advertise( PyObject *, PyObject * args ) {
 
 			if( result != 2 ) {
 				// This was HTCondorIOError in version 1.
-				PyErr_SetString( PyExc_IOError, "Failed to advertise to collector." );
+				PyErr_SetString( PyExc_HTCondorException, "Failed to advertise to collector." );
 				return NULL;
 			}
 		}
