@@ -23,7 +23,7 @@ _negotiator_command(PyObject *, PyObject * args) {
     Sock * s = start_negotiator_command(command, addr);
     if( s == NULL ) {
         // This was HTCondorIOError in version 1.
-        PyErr_SetString( PyExc_IOError, "Unable to connect to the negotiator" );
+        PyErr_SetString( PyExc_HTCondorException, "Unable to connect to the negotiator" );
         return NULL;
     }
 
@@ -35,7 +35,7 @@ _negotiator_command(PyObject *, PyObject * args) {
 
     if( rv ) {
         // This was HTCondorIOError in version 1.
-        PyErr_SetString( PyExc_IOError, "Failed to send command to negotiator" );
+        PyErr_SetString( PyExc_HTCondorException, "Failed to send command to negotiator" );
         return NULL;
     }
 
@@ -57,7 +57,7 @@ _negotiator_command_return(PyObject *, PyObject * args) {
     Sock * s = start_negotiator_command(command, addr);
     if( s == NULL ) {
         // This was HTCondorIOError in version 1.
-        PyErr_SetString( PyExc_IOError, "Unable to connect to the negotiator" );
+        PyErr_SetString( PyExc_HTCondorException, "Unable to connect to the negotiator" );
         return NULL;
     }
 
@@ -73,7 +73,7 @@ _negotiator_command_return(PyObject *, PyObject * args) {
 
     if( rv ) {
         // This was HTCondorIOError in version 1.
-        PyErr_SetString( PyExc_IOError, "Failed to send command to negotiator" );
+        PyErr_SetString( PyExc_HTCondorException, "Failed to send command to negotiator" );
         return NULL;
     }
 
@@ -96,7 +96,7 @@ _negotiator_command_user(PyObject *, PyObject * args) {
     Sock * s = start_negotiator_command(command, addr);
     if( s == NULL ) {
         // This was HTCondorIOError in version 1.
-        PyErr_SetString( PyExc_IOError, "Unable to connect to the negotiator" );
+        PyErr_SetString( PyExc_HTCondorException, "Unable to connect to the negotiator" );
         return NULL;
     }
 
@@ -108,7 +108,7 @@ _negotiator_command_user(PyObject *, PyObject * args) {
 
     if( rv ) {
         // This was HTCondorIOError in version 1.
-        PyErr_SetString( PyExc_IOError, "Failed to send command to negotiator" );
+        PyErr_SetString( PyExc_HTCondorException, "Failed to send command to negotiator" );
         return NULL;
     }
 
@@ -131,7 +131,7 @@ _negotiator_command_user_return(PyObject *, PyObject * args) {
     Sock * s = start_negotiator_command(command, addr);
     if( s == NULL ) {
         // This was HTCondorIOError in version 1.
-        PyErr_SetString( PyExc_IOError, "Unable to connect to the negotiator" );
+        PyErr_SetString( PyExc_HTCondorException, "Unable to connect to the negotiator" );
         return NULL;
     }
 
@@ -147,7 +147,7 @@ _negotiator_command_user_return(PyObject *, PyObject * args) {
 
     if( rv ) {
         // This was HTCondorIOError in version 1.
-        PyErr_SetString( PyExc_IOError, "Failed to send command to negotiator" );
+        PyErr_SetString( PyExc_HTCondorException, "Failed to send command to negotiator" );
         return NULL;
     }
 
@@ -178,7 +178,7 @@ _negotiator_command_user_value(PyObject *, PyObject * args) {
     Sock * s = start_negotiator_command(command, addr);
     if( s == NULL ) {
         // This was HTCondorIOError in version 1.
-        PyErr_SetString( PyExc_IOError, "Unable to connect to the negotiator" );
+        PyErr_SetString( PyExc_HTCondorException, "Unable to connect to the negotiator" );
         return NULL;
     }
 
@@ -196,7 +196,7 @@ _negotiator_command_user_value(PyObject *, PyObject * args) {
 
     if( rv ) {
         // This was HTCondorIOError in version 1.
-        PyErr_SetString( PyExc_IOError, "Failed to send command to negotiator" );
+        PyErr_SetString( PyExc_HTCondorException, "Failed to send command to negotiator" );
         return NULL;
     }
 
